Package: vcdExtra
Type: Package
Title: 'vcd' Extensions and Additions
Version: 0.9.8
Date: 2026-08-21
Language: en-US
Authors@R: c(
            person(given = "Michael", family = "Friendly", 
                   role=c("aut", "cre"), 
                   email="friendly@yorku.ca",
                   comment = c(ORCID = "0000-0002-3237-0941")),
            person(given = "David", family = "Meyer", 
                   role="ctb",
                   email = "david.meyer@wu.ac.at"),
            person(given = "Achim", family = "Zeileis", 
                   role="ctb",
                   email = "Achim.Zeileis@uibk.ac.at",
                   comment = c(ORCID = "0000-0003-0918-3766")),
            person(given = "Duncan", 
                   family = "Murdoch",
                   email = "murdoch.duncan@gmail.com",
                   role="ctb"),
            person(given = "Heather", family = "Turner", 
                   role="ctb",
                   email = "h.turner.1@warwick.ac.uk",
                   comment = c(ORCID = "0000-0002-1256-3375")),
            person(given = "David", family = "Firth", 
                   role="ctb",
                   email = "d.firth@warwick.ac.uk"),
            person(given = "Daniel", family = "Sabanes Bove",
                   role="ctb",
                   email = "daniel@rconis.com",
                   comment = c(ORCID = "0000-0002-0176-9239")),
            person(given = "Matt", family = "Kumar", role="ctb"),
            person(given = "Shuguang", family = "Sun", role="ctb"),
            person(given = "Gavin", family = "Klorfine", 
                   role = "aut",
                   email = "gklorfin@yorku.ca",
                   comment = c(ORCID = "0009-0003-0395-4512"))
            )
Maintainer: Michael Friendly <friendly@yorku.ca>
Depends: R (>= 4.1.0), vcd, gnm (>= 1.0-3)
Suggests: gmodels, Fahrmeir, effects, VGAM, plyr, lmtest, nnet,
        Sleuth2, car, lattice, stats4, AER, coin, Hmisc, rmarkdown,
        seriation, testthat (>= 3.0.0), tibble, glue, here, purrr,
        readxl, stringr, tidyr (>= 1.3.0), factoextra, rgl
Imports: MASS, grDevices, grid, stats, utils, dplyr, ca, igraph,
        colorspace, gt, scales, methods, knitr, htmlwidgets, webshot2,
        forcats, rlang, ggplot2 (>= 3.4.0)
Description: Provides additional data sets, methods and documentation to complement the 'vcd' package for Visualizing Categorical Data
    and the 'gnm' package for Generalized Nonlinear Models.
	In particular, 'vcdExtra' extends mosaic, assoc and sieve plots from 'vcd' to handle 'glm()' and 'gnm()' models and
	adds a 3D version in 'mosaic3d'.  Additionally, methods are provided for comparing and visualizing lists of
	'glm' and 'loglm' objects. This package is now a support package for the book Friendly, M. and Meyer, D.
  (2016, ISBN:978-1-4987-2583-5) 'Discrete Data Analysis with R: Visualization and Modeling Techniques for
  Categorical and Count Data'. Recent work adds colorized tables of frequencies to highlight patterns of association,
  association graphs to visualize conditional independence and a variety of new or improved statistical tests for
  categorical data analysis.
License: GPL (>= 2)
URL: https://friendly.github.io/vcdExtra/,
        https://github.com/friendly/vcdExtra
BugReports: https://github.com/friendly/vcdExtra/issues
VignetteBuilder: knitr, rmarkdown
LazyLoad: yes
LazyData: yes
Encoding: UTF-8
Config/testthat/edition: 3
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-08-21 17:54:41 UTC; friendly
Author: Michael Friendly [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3237-0941>),
  David Meyer [ctb],
  Achim Zeileis [ctb] (ORCID: <https://orcid.org/0000-0003-0918-3766>),
  Duncan Murdoch [ctb],
  Heather Turner [ctb] (ORCID: <https://orcid.org/0000-0002-1256-3375>),
  David Firth [ctb],
  Daniel Sabanes Bove [ctb] (ORCID:
    <https://orcid.org/0000-0002-0176-9239>),
  Matt Kumar [ctb],
  Shuguang Sun [ctb],
  Gavin Klorfine [aut] (ORCID: <https://orcid.org/0009-0003-0395-4512>)
Repository: CRAN
Date/Publication: 2026-08-21 22:00:03 UTC

# Generated by roxygen2: do not edit by hand

S3method(CMHtest,default)
S3method(CMHtest,formula)
S3method(LRstats,default)
S3method(LRstats,glmlist)
S3method(LRstats,loglmlist)
S3method(Summarise,default)
S3method(Summarise,glmlist)
S3method(Summarise,loglmlist)
S3method(assoc,glm)
S3method(assoc_graph,glm)
S3method(assoc_graph,list)
S3method(assoc_graph,loglm)
S3method(coef,glmlist)
S3method(color_table,data.frame)
S3method(color_table,default)
S3method(color_table,ftable)
S3method(color_table,matrix)
S3method(color_table,structable)
S3method(color_table,table)
S3method(drop1,loglm)
S3method(logLik,loglm)
S3method(logist_plot,data.frame)
S3method(logist_plot,default)
S3method(logist_plot,formula)
S3method(modFit,glm)
S3method(modFit,loglm)
S3method(mosaic,glm)
S3method(mosaic,glmlist)
S3method(mosaic,loglmlist)
S3method(mosaic3d,default)
S3method(mosaic3d,loglm)
S3method(pairs,table)
S3method(plot,HLtest)
S3method(plot,assoc_graph)
S3method(print,CMHtest)
S3method(print,GKgamma)
S3method(print,HLtest)
S3method(print,Kappa)
S3method(print,assoc_graph)
S3method(print,breslow_day_test)
S3method(print,woolf_test)
S3method(rootogram,HLtest)
S3method(sieve,glm)
S3method(split3d,list)
S3method(split3d,shape3d)
S3method(summary,HLtest)
S3method(update,xtabs)
export(CMHtest)
export(Crossings)
export(GKgamma)
export(HLtest)
export(Kway)
export(LRanova)
export(LRstats)
export(Summarise)
export(as_array)
export(as_caseform)
export(as_freqform)
export(as_matrix)
export(as_table)
export(assoc_graph)
export(assoc_strength)
export(blogits)
export(breslow_day_test)
export(center3d)
export(collapse.table)
export(collapse_levels)
export(color_table)
export(conditional)
export(cutfac)
export(datasets)
export(dlogseries)
export(drop1.loglm)

export(expand.dft)
export(expand.table)
export(get_model)
export(get_models)
export(glmlist)
export(joint)
export(knit_include)
export(labeling_points)
export(logist_density)
export(logist_hist)
export(logist_plot)
export(logist_point)
export(loglin2formula)
export(loglin2string)
export(loglmlist)
export(markov)
export(mcaplot)
export(modFit)
export(mosaic3d)
export(mutual)
export(pairs_diagonal_mosaic)
export(plogseries)
export(qlogseries)
export(range3d)
export(rlogseries)
export(saturated)
export(seq_loglm)
export(seq_mosaic)
export(split3d)
export(update.xtabs)
export(woolf_test)
export(zero.test)
importFrom(MASS,loglm)
importFrom(ca,
  cacoord,
  multilines
)
importFrom(dplyr,
  all_of,
  as_tibble,
  everything,
  is.tbl,
  rename,
  summarise
)
importFrom(forcats,fct_collapse)
importFrom(gnm,meanResiduals)
importFrom(grDevices,
  col2rgb,
  hsv,
  rgb
)
importFrom(graphics,
  abline,
  plot,
  points,
  text
)
importFrom(grid,
  gpar,
  grid.layout,
  grid.newpage,
  grid.points,
  popViewport,
  pushViewport,
  seekViewport,
  unit,
  upViewport,
  viewport
)
importFrom(htmlwidgets,saveWidget)
importFrom(knitr,
  include_graphics,
  is_html_output
)
importFrom(methods,is)
importFrom(rlang,
  .data,
  set_names
)
importFrom(stats,
  as.formula,
  chisq.test,
  deviance,
  family,
  fitted,
  formula,
  glm,
  logLik,
  loglin,
  mantelhaen.test,
  model.frame,
  na.pass,
  nobs,
  pchisq,
  poisson,
  qnorm,
  quantile,
  reformulate,
  residuals,
  rstandard,
  runif,
  setNames,
  terms,
  update,
  update.formula,
  weighted.mean,
  xtabs
)
importFrom(utils,
  data,
  menu,
  type.convert
)
importFrom(vcd,
  assoc,
  assocstats,
  is.structable,
  labeling_border,
  labeling_values,
  mosaic,
  rootogram,
  shading_hcl,
  sieve
)
importFrom(webshot2,webshot)

[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/vcdExtra)](https://cran.r-project.org/package=vcdExtra)
[![](http://cranlogs.r-pkg.org/badges/grand-total/vcdExtra)](https://cran.r-project.org/package=vcdExtra)
[![Rdoc](http://www.rdocumentation.org/badges/version/vcdExtra)](https://www.rdocumentation.org/packages/vcdExtra)


# vcdExtra <img src="vcdExtra-logo.png" align="right" height="200px" />
## Extensions and additions to vcd: Visualizing Categorical Data 

Version 0.7-5

This package provides additional data sets, documentation, and
a few functions designed to extend the [vcd](https://CRAN.R-project.org/package=vcd) package for *Visualizing Categorical Data*
and the [gnm](https://CRAN.R-project.org/package=gnm) package for *Generalized Nonlinear Models*. 
In particular, vcdExtra extends mosaic, assoc and sieve plots from vcd to handle `glm()` and `gnm()` models and
adds a 3D version in `mosaic3d()`.

`vcdExtra` is now a support package for the book [*Discrete Data Analysis with R*](http://ddar.datavis.ca) by Michael Friendly and David Meyer.

### Details

The main purpose of this package is to serve as a sandbox for
introducing extensions of
mosaic plots and related graphical methods
that apply to loglinear models fitted using
`glm()` and related, generalized nonlinear models fitted
with `gnm()` in the gnm package.
A related purpose is to fill in some holes in the analysis of
categorical data in R, not provided in base R, vcd, 
or other commonly used packages.

* The method `mosaic.glm()` 
extends the `mosaic.loglm()` method in the vcd
package to this wider class of models.  This method also works for
the generalized nonlinear models fit with the gnm package,
including models for square tables and models with multiplicative associations.

* `mosaic3d()`
introduces a 3D generalization of mosaic displays using the
[rgl](https://CRAN.R-project.org/package=rgl) package.

* A new class, `glmlist`, is introduced for working with
collections of glm objects, e.g., `Kway()` for fitting
all K-way models from a basic marginal model, and `LRstats()`
for brief statistical summaries of goodnes-of-fit for a collection of
models.

* For square tables with ordered factors, `Crossings()` supplements the 
specification of terms in model formulas using
`Symm()`,
`Diag()`, 
`Topo(),` etc. in the [gnm](https://CRAN.R-project.org/package=gnm) package.

* In addition, there are many new data sets, a tutorial vignette,
_Working with categorical data with R and the vcd package_, `vignette("vcd-tutorial", package = "vcdExtra")`,
and a few useful utility functions for manipulating categorical data sets and working with models for
categorical data. 

## Installation

Get the released version from CRAN:

     install.packages("vcdExtra")

The development version can be installed to your R library directly from this repo via:

     if (!require(devtools)) install.packages("devtools")
     library(devtools)
     install_github("friendly/vcdExtra")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tut04-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")


# Load packages
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
library(seriation)

data(HairEyeColor)
data(PreSex)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
#if(!file.exists("fig")) dir.create("fig")

## -----------------------------------------------------------------------------
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
mosaic(art, gp = shading_max, 
            split_vertical = TRUE, 
            main="Arthritis: [Treatment] [Improved]")

## ----art1---------------------------------------------------------------------
summary(art)

## -----------------------------------------------------------------------------
mosaic(art, gp = shading_Friendly, 
            split_vertical = TRUE, 
            main="Arthritis: gp = shading_Friendly")

## ----glass--------------------------------------------------------------------
data(Glass, package="vcdExtra")
(glass.tab <- xtabs(Freq ~ father + son, data=Glass))

## ----glass-mosaic1------------------------------------------------------------
largs <- list(set_varnames=list(father="Father's Occupation", 
                                son="Son's Occupation"),
              abbreviate=10)
gargs <- list(interpolate=c(1,2,4,8))

mosaic(glass.tab, 
  shade=TRUE, 
  labeling_args=largs, 
  gp_args=gargs,
  main="Alphabetic order", 
  legend=FALSE, 
  rot_labels=c(20,90,0,70))

## ----glass-order--------------------------------------------------------------
# reorder by status
ord <- c(2, 1, 4, 3, 5) 
row.names(glass.tab)[ord]

## ----glass-mosaic2------------------------------------------------------------
mosaic(glass.tab[ord, ord], 
  shade=TRUE, 
  labeling_args=largs,  
  gp_args=gargs,
  main="Effect order", 
  legend=FALSE, 
  rot_labels=c(20,90,0,70))

## ----glass-ord----------------------------------------------------------------
Glass.ord <- Glass
Glass.ord$father <- ordered(Glass.ord$father, levels=levels(Glass$father)[ord])
Glass.ord$son    <- ordered(Glass.ord$son,    levels=levels(Glass$son)[ord])
str(Glass.ord)

## ----diag---------------------------------------------------------------------
rowfac <- gl(4, 4, 16)
colfac <- gl(4, 1, 16)
diag4by4 <- Diag(rowfac, colfac)
matrix(Diag(rowfac, colfac, binary = FALSE), 4, 4)

## ----symm---------------------------------------------------------------------
symm4by4 <- Symm(rowfac, colfac)
matrix(symm4by4, 4, 4)

## ----glass-models-------------------------------------------------------------
library(gnm)
glass.indep <- glm(Freq ~ father + son, 
                   data = Glass.ord, family=poisson)
glass.quasi <- glm(Freq ~ father + son + Diag(father, son),  
                   data = Glass.ord, family=poisson)
glass.symm  <- glm(Freq ~ Symm(father, son),  
                   data = Glass.ord, family=poisson)
glass.qsymm <- glm(Freq ~ father + son + Symm(father, son),  
                   data = Glass.ord, family=poisson)

## ----glass-quasi--------------------------------------------------------------
mosaic(glass.quasi, 
  residuals_type="rstandard", 
  shade=TRUE, 
  labeling_args=largs,  
  gp_args=gargs,
  main="Quasi-Independence",
  legend=FALSE, 
  rot_labels=c(20,90,0,70)
  )

## ----glass-anova--------------------------------------------------------------
# model comparisons: for *nested* models
anova(glass.indep, glass.quasi, glass.qsymm, test="Chisq")

## ----glass-lrstats------------------------------------------------------------
models <- glmlist(glass.indep, glass.quasi, glass.symm, glass.qsymm)
LRstats(models)

## ----glass-qsymm--------------------------------------------------------------
mosaic(glass.qsymm, 
  residuals_type="rstandard", 
  shade=TRUE, 
  labeling_args=largs,  
  gp_args=gargs,
  main = paste("Quasi-Symmetry", modFit(glass.qsymm)),
  legend=FALSE, 
  rot_labels=c(20,90,0,70)
  )

## ----housetasks---------------------------------------------------------------
data("HouseTasks", package = "vcdExtra")
HouseTasks

## ----housetasks-mos1----------------------------------------------------------
require(vcd)
mosaic(HouseTasks, shade = TRUE,
       labeling = labeling_border(rot_labels = c(45,0, 0, 0), 
                                  offset_label =c(.5,5,0, 0),
                                  varnames = c(FALSE, TRUE),
                                  just_labels=c("center","right"),
                                  tl_varnames = FALSE),
       legend = FALSE)


## ----housetasks-ca------------------------------------------------------------
require(ca)
HT.ca <- ca(HouseTasks)
summary(HT.ca, rows=FALSE, columns=FALSE)

## ----housetasks-ca-plot-------------------------------------------------------
plot(HT.ca, lines = TRUE)

## ----housetasks-seriation-----------------------------------------------------
require(seriation)
order <- seriate(HouseTasks, method = "CA")
# the permuted row and column labels
rownames(HouseTasks)[order[[1]]]
colnames(HouseTasks)[order[[2]]]

## ----housetasks-mos2----------------------------------------------------------
# do the permutation
HT_perm <- permute(HouseTasks, order, margin=1)

mosaic(HT_perm, shade = TRUE,
       labeling = labeling_border(rot_labels = c(45,0, 0, 0), 
                                  offset_label =c(.5,5,0, 0),
                                  varnames = c(FALSE, TRUE),
                                  just_labels=c("center","right"),
                                  tl_varnames = FALSE),
       legend = FALSE)

## housing.R    Visualize models from example(housing, package="MASS")

# These examples fit a variety of models to the data(housing), giving a 4-way
# frequency table of 1681 individuals from the Copenhagen Housing Conditions
# Survey, classified by their Type of rental dwelling, perceived Influence on
# management of the property, and degree of Contact with other residents. The
# response variable here is Satisfaction of householders with their present
# housing circumstances.


library(vcdExtra)
library(MASS)
data(housing, package="MASS")


oldop <-options(contrasts = c("contr.treatment", "contr.poly"))

##########################
#  Poisson models for Freq, equivalent to loglinear models
##########################
# Baseline model, with Satisfaction as a response
house.glm0 <- glm(Freq ~ Infl*Type*Cont + Sat, family = poisson,
                  data = housing)
modFit(house.glm0)

# labeling_args for mosaic()
largs <- list(set_varnames = c(
  Infl="Influence on management", 
	Cont="Contact among residents", 
  Type="Type of dwelling", 
  Sat="Satisfaction"),
	abbreviate=c(Type=3))

mosaic(house.glm0, labeling_args=largs, main='Baseline model: [ITC][Sat]')

# reorder variables in the mosaic, putting Sat last
mosaic(house.glm0, ~ Type+Infl+Cont+Sat, labeling_args=largs, main='Baseline model: [ITC][Sat]')

# what terms need to be added? Consider main effects, interactions of Sat with each other
MASS::addterm(house.glm0, ~. + Sat:(Infl+Type+Cont), test = "Chisq")

# add all two way terms with Satisfaction
house.glm1 <- update(house.glm0, . ~ . + Sat*(Infl+Type+Cont))

# did it get better?
anova(house.glm0, house.glm1, test="Chisq")
# plot it
mosaic(house.glm1, labeling_args=largs, main='Model [IS][TS][CS]', gp=shading_Friendly)

# Same model, fit by iterative proportional scaling
(house.loglm <- MASS::loglm(Freq ~ Infl*Type*Cont + Sat*(Infl+Type+Cont), data = housing))

# Can we drop any terms?
MASS::dropterm(house.glm1, test = "Chisq")

# Need to add any terms?
MASS::addterm(house.glm1, ~. + Sat:(Infl+Type+Cont)^2, test  =  "Chisq")

# add an interaction
house.glm2 <- update(house.glm1,
                     . ~ . + Sat:Infl:Type)

LRstats(house.glm0, house.glm1, house.glm2)

##########################
# Effect plots, for glm1 model
##########################
library(effects)
house.eff <-allEffects(house.glm1)

# show the interactions of Infl, Cont and Type with Sat
plot(house.eff, 'Infl:Sat', x.var='Sat', xlab="Satisfaction")

plot(house.eff, 'Infl:Sat', x.var='Infl', xlab="Influence")


# same plot in one panel, no std errors shown
plot(house.eff, 'Infl:Sat', x.var='Sat', xlab="Satisfaction", multiline=TRUE)

plot(house.eff, 'Cont:Sat', x.var='Sat', xlab="Satisfaction")

plot(house.eff, 'Type:Sat', x.var='Sat', xlab="Satisfaction")


##########################
# multinomial model
##########################
library(nnet)
# multinomial model, similar in spirit to house.glm1
(house.mult<- multinom(Sat ~ Infl + Type + Cont, weights = Freq,
                       data = housing))
# Do we need a more complex model?
house.mult2 <- multinom(Sat ~ Infl*Type*Cont, weights = Freq,
                        data = housing)
anova(house.mult, house.mult2)


# effect plots for multinomial model
house.effm <- allEffects(house.mult)

plot(house.effm, 'Infl', xlab='Influence on management', style="stacked",
	main="Multinomial: Infl effect plot")

plot(house.effm, 'Cont', xlab='Contact among residents', style="stacked",
	main="Multinomial: Cont effect plot")

plot(house.effm, 'Type', xlab='Type of dwelling', style="stacked",
	main="Multinomial: Type effect plot")


##########################
# proportional odds model
##########################

(house.plr <- polr(Sat ~ Infl + Type + Cont,
                   data = housing, weights = Freq))

# Test proportional odds assumption by likelihood ratio test
# NB: multinom() objects do not have a df.residual component, so we have
#     to use the difference in edf to get df for the test
pchisq(deviance(house.plr) - deviance(house.mult), 
		df = house.mult$edf -house.plr$edf, lower.tail = FALSE)

# try more complex models
house.plr2 <- stepAIC(house.plr, ~.^2)
house.plr2$anova

house.effp <- allEffects(house.plr)

plot(house.effp, 'Infl', xlab='Influence on management', style="stacked",
	main="Proportional odds: Infl effect plot")
plot(house.effp, 'Cont', xlab='Contact among residents', style="stacked",
	main="Proportional odds: Cont effect plot")
plot(house.effp, 'Type', xlab='Type of dwelling', style="stacked",
	main="Proportional odds: Type effect plot")

options(oldop)

# Yaish data: Unidiff model for 3-way table
library(gnm)
library(vcd)

data(yaish)

# Ignore orig==7 & dest == 7 with very few cases
yaish <- yaish[,1:6,1:6]
## Fit mutual independence model.
long.labels <- list(set_varnames =
	c(orig="Origin status", dest="Destination status", educ="Education"))

mosaic(~orig + dest + educ, data=yaish, gp=shading_Friendly,
	labeling_args=long.labels)

## Fit conditional independence model
mosaic(~orig + dest | educ, data=yaish, gp=shading_Friendly,
	labeling_args=long.labels)

## Fit the "UNIDIFF" mobility model across education levels
##
unidiff <- gnm(Freq ~ educ*orig + educ*dest +
                     Mult(Exp(educ), orig:dest), family = poisson,
                     data = yaish, subset = (dest != 7 & orig != 7))

structable(round(residuals(unidiff), digits=2))

# can use mosaic.loglm, passing residuals
mosaic(yaish[, 1:6, 1:6], residuals=residuals(unidiff),
       gp=shading_Friendly, labeling_args=long.labels)

# what about mosaic.gnm?
mosaic(unidiff, gp=shading_Friendly,
	labeling_args=long.labels)

### mosaic3d-demo: proof-of-concept for exploring 3D mosaic plots


##
## split a 3D object along a given dimension, dim, into copies whose
## extent along that dimension are given by the proportions in vector p
## (rescaled to proportions if they are not already so).
##
## The objects are slightly separated along that dimension, allowing
## a total inter-object space = space
##

split3d <- function(obj, p, dim, space=.10) {
  range <-range3d(obj)
  min <- range[1,]
  p <- p/sum(p)                 # assure proportions
  uspace <- space/(length(p)-1) # unit space between objects
  scales <- p * (1-space)
  shifts <- c(0, cumsum(p)[-length(p)])*diff(range[,dim])
  result <- list()
  for (i in seq_along(p)) {
     xscale <- yscale <- zscale <- 1
     xshift <- yshift <- zshift <- 0

     if (dim == 1) {
       xscale <- scales[i]
       xshift <- shifts[i] + min[1]*(1-xscale) + (uspace * (i-1))
     } else if (dim == 2) {
       yscale <- scales[i]
       yshift <- shifts[i] + min[2]*(1-yscale) + (uspace * (i-1))
     } else if (dim == 3) {
       zscale <- scales[i]
       zshift <- shifts[i] + min[3]*(1-zscale) + (uspace * (i-1))
     }

     result[[i]] <- translate3d(scale3d(obj, xscale, yscale, zscale),
                                xshift, yshift, zshift)

   }
   result
}


range3d <- function(obj) {
	if (!"vb" %in% names(obj)) stop("Not a mesh3d or shape3d object")
  x <- with(obj, range(vb[1,]/vb[4,]))
  y <- with(obj, range(vb[2,]/vb[4,]))
  z <- with(obj, range(vb[3,]/vb[4,]))
  result <- cbind(x,y,z)
  rownames(result)<- c('min', 'max')
  result
}

label3d <- function(objlist, dim, text, offset=.1, adj=c(0.5, 1), ...) {
	ranges <- lapply(objlist, range3d)
	loc <- t(sapply(ranges, colMeans))   # positions of labels on dimension dim
	min <- t(sapply(ranges, function(x) x[1,]))  # other dimensions at min values
	xyz <- min - offset
	xyz[,dim] <- loc[,dim]
	texts3d(xyz, texts=text, adj=adj, ...)
	
}


library(rgl)
# level 1
open3d()

# use transparent colors for the side walls of mosaic cubes
crgb <- col2rgb(c("red", "gray90", "blue"))/255
clr <-rbind(crgb, alpha=0.5)
col <- rgb(clr[1,], clr[2,], clr[3,], clr[4,])
#col <- c("#FF000080", "#E5E5E580", "#0000FF80")

sl0 <- cube <- cube3d(alpha=0.3)
sl1 <- split3d(sl0, c(.2, .3, .5), 1)
shapelist3d(sl1, col=col)
label3d(sl1, 1, c("A1", "A2", "A3"))

# level 2
open3d()
sl2 <- list()
for (i in seq_along(sl1)) {
	p <- runif(1, .2, .8)
	sl2 <- c(sl2, split3d(sl1[[i]], c(p, 1-p), 2, space=.1))
	}
shapelist3d(sl2, col=col)
label3d(sl1, 1, c("A1", "A2", "A3"))
label3d(sl2[1:2], 2, c("B1", "B2"))

# level 3
open3d()
sl3 <- list()
for (i in seq_along(sl2)) {
	p <- runif(1, .2, .8)
	sl3 <- c(sl3, split3d(sl2[[i]], c(p, 1-p), 3, space=.05))
	}
shapelist3d(sl3, col=col)
label3d(sl1, 1, c("A1", "A2", "A3"))
label3d(sl2[1:2], 2, c("B1", "B2"))
#label3d(sl3[1:2], 3, c("C1", "C2"))
label3d(sl3[1:2], 3, c("C1", "C2"), adj=rev(c(0.5, 1)))

# wong3-1  Political views, support for women to work and national welfare spending

library(vcdExtra)
# Data from Wong, R. (2010), Association Models, Los Angeles: Sage, Number 07-164
#      Table 3.1, from the General Social Survey, 2006.
# Questions:
#   polviews:  Think of self as liberal or conservative
#   fefam:     Better for men to work and women to tend home
#   natfare:   National welfare spending: too little, about right, too much

# Table 3.1
Freq<-c( 9, 5, 5, 1,    1, 6, 5, 1,    2, 2, 2, 1,
        17,13, 7, 4,   13,22, 9, 1,    7,13, 6, 2,
         8,14, 6, 0,   10,29,10, 0,    5,14, 6, 2,
        20,38,24, 8,   23,72,34,10,   17,67,36,12,
         4,21,12, 4,    7,30, 9, 1,    9,19,14, 2,
         2, 9, 8, 3,    1,16,19, 2,   11,28,28,11,
         0, 1, 5, 0,    2, 3, 3, 2,    2, 7, 6, 6)

polviews<-gl(7,4*3)
fefam<-gl(4,1,length=7*4*3)
natfare<-gl(3,4,length=7*4*3)

long.vnames <- list(set_varnames = c(polviews="Political views", fefam="Females should tend home",
                                     natfare="National welfare spending"))
long.lnames <- list(polviews = c("Lib++", "2", "3", "Moderate", "5", "6", "Cons++"),
                    fefam = c("Dis+", "Dis", "Agr", "Agr+"),
                    natfare = c("--", "OK", "++")
                    )


############################################
Wong31 <- data.frame(Freq, polviews, fefam, natfare)

Wong31.xtab <- xtabs(Freq ~ polviews+fefam+natfare, data=Wong31)
dimnames(Wong31.xtab) <- long.lnames

# Quick look at all pairwise associations
pairs(Wong31.xtab, gp=shading_Friendly, diag_panel=pairs_diagonal_mosaic)

############################################
# Model 1 - Independence Model
Wong31.O<-gnm(Freq~polviews+fefam+natfare,family=poisson, data=Wong31)
summary(Wong31.O)

mosaic(Wong31.O, main=paste("Independence model",modFit(Wong31.O)),
	labeling_args=long.vnames, set_labels=long.lnames,
	split_vertical=c(TRUE, FALSE, FALSE),
	labeling=labeling_residuals, suppress=2, gp=shading_Friendly)


############################################
# NB: add1 doesn't work with gnm() objects.  Re-fit using glm()
Wong31.O<-glm(Freq~polviews+fefam+natfare,family=poisson, data=Wong31)

# consider all two-way terms
add1(Wong31.O, ~.+(polviews + fefam + natfare)^2, test="Chisq")

# same result with MASS::addterm
addterm(Wong31.O, ~.+(polviews + fefam + natfare)^2, test="Chisq")

# or, start with saturated model and drop terms
Wong31.sat<-glm(Freq~polviews*fefam*natfare, family=poisson, data=Wong31)
drop1(Wong31.sat, test="Chisq")


############################################
# Model 2 - Full Two-way Interaction

Wong31.twoway <- update(Wong31.O, ~ .^2)
summary(Wong31.twoway)

mosaic(Wong31.twoway, main=paste("All two-way model", modFit(Wong31.twoway)),
	labeling_args=long.vnames, set_labels=long.lnames,
	split_vertical=c(TRUE, FALSE, FALSE),
	labeling=labeling_residuals, suppress=1, gp=shading_Friendly)


############################################
# Model 3 - Conditional Independence on polviews
Wong31.cond1 <- glm(Freq~polviews * (fefam + natfare), family=poisson)
summary(Wong31.cond1)

mosaic(Wong31.cond1, main=paste("Cond1: ~P * (F+N)", modFit(Wong31.cond1)),
	labeling_args=long.vnames, set_labels=long.lnames,
	split_vertical=c(TRUE, FALSE, FALSE),
	labeling=labeling_residuals, suppress=1, gp=shading_Friendly)

############################################
# Model 4 - Conditional Independence on fefam
Wong31.cond2 <- glm(Freq~polviews*fefam + fefam*natfare,family=poisson)
summary(Wong31.cond2)

mosaic(Wong31.cond2, main=paste("Cond2: ~F * (P+N)", modFit(Wong31.cond2)),
	labeling_args=long.vnames, set_labels=long.lnames,
	split_vertical=c(TRUE, FALSE, FALSE),
	labeling=labeling_residuals, suppress=1, gp=shading_Friendly)

############################################
# Model 5 - Conditional Independence on natfare
Wong31.cond3<-glm(Freq~fefam*natfare+polviews*natfare,family=poisson)
summary(Wong31.cond3)

mosaic(Wong31.cond3, main=paste("Cond2: ~N * (F+N)", modFit(Wong31.cond3)),
	labeling_args=long.vnames, set_labels=long.lnames,
	split_vertical=c(TRUE, FALSE, FALSE),
	labeling=labeling_residuals, suppress=1, gp=shading_Friendly)

anova(Wong31.O,  Wong31.cond3, Wong31.cond2, Wong31.cond1,Wong31.twoway, Wong31.sat)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tut03-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(HairEyeColor)
data(PreSex)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
#if(!file.exists("fig")) dir.create("fig")


## ----loglm-hec1---------------------------------------------------------------
library(MASS)
## Independence model of hair and eye color and sex.  
hec.1 <- loglm(~Hair+Eye+Sex, data=HairEyeColor)
hec.1

## ----loglm-hec2---------------------------------------------------------------
## Conditional independence
hec.2 <- loglm(~(Hair + Eye) * Sex, data=HairEyeColor)
hec.2

## ----loglm-hec3---------------------------------------------------------------
## Joint independence model.  
hec.3 <- loglm(~Hair*Eye + Sex, data=HairEyeColor)
hec.3

## ----loglm-anova--------------------------------------------------------------
anova(hec.1, hec.2, hec.3)

## ----mental1------------------------------------------------------------------
data(Mental, package = "vcdExtra")
str(Mental)
xtabs(Freq ~ mental + ses, data=Mental)   # display the frequency table

## ----mental2------------------------------------------------------------------
indep <- glm(Freq ~ mental + ses, family = poisson, data = Mental)  # independence model

## ----mental3------------------------------------------------------------------
# Use integer scores for rows/cols 
Cscore <- as.numeric(Mental$ses)
Rscore <- as.numeric(Mental$mental)	

## ----mental4------------------------------------------------------------------
# column effects model (ses)
coleff <- glm(Freq ~ mental + ses + Rscore:ses, family = poisson, data = Mental)

# row effects model (mental)
roweff <- glm(Freq ~ mental + ses + mental:Cscore, family = poisson, data = Mental)

# linear x linear association
linlin <- glm(Freq ~ mental + ses + Rscore:Cscore, family = poisson, data = Mental)

## ----mental4a-----------------------------------------------------------------
# compare models using AIC, BIC, etc
vcdExtra::LRstats(glmlist(indep, roweff, coleff, linlin))

## ----mental5------------------------------------------------------------------
anova(indep, linlin, coleff, test="Chisq")	
anova(indep, linlin, roweff, test="Chisq")	

## ----mental6------------------------------------------------------------------
CMHtest(xtabs(Freq~ses+mental, data=Mental))

## ----mental7------------------------------------------------------------------
RC1 <- gnm(Freq ~ mental + ses + Mult(mental,ses), data=Mental, 
             family=poisson, verbose=FALSE)
RC2 <- gnm(Freq ~ mental+ses + instances(Mult(mental,ses),2), data=Mental, 
             family=poisson, verbose=FALSE)
anova(indep, RC1, RC2, test="Chisq")

# VisualAcuity data: Quasi- and Symmetry models

library(vcdExtra)
library(gnm)
women <- subset(VisualAcuity, gender=="female", select=-gender)

indep <- glm(Freq ~ right + left,  data = women, family=poisson)
mosaic(indep, residuals_type="rstandard", gp=shading_Friendly,
       main="Vision data: Independence (women)"  )

quasi.indep <- glm(Freq ~ right + left + Diag(right, left),
       data = women, family = poisson)
mosaic(quasi.indep, residuals_type="rstandard", gp=shading_Friendly,
       main="Quasi-Independence (women)"  )

symmetry <- glm(Freq ~ Symm(right, left),
       data = women, family = poisson)
# BUG FIXED
mosaic(symmetry, residuals_type="rstandard", gp=shading_Friendly,
		main="Symmetry model (women)")

quasi.symm <- glm(Freq ~ right + left + Symm(right, left),
       data = women, family = poisson)
mosaic(quasi.symm, residuals_type="rstandard", gp=shading_Friendly,
       main="Quasi-Symmetry model (women)")

# model comparisons: for *nested* models
anova(indep, quasi.indep, quasi.symm, test="Chisq")
anova(symmetry, quasi.symm, test="Chisq")

# model summaries, with AIC and BIC
models <- glmlist(indep, quasi.indep, symmetry, quasi.symm)
LRstats(models)

# UCBAdmissions data: Conditional independence via loglm and glm
library(vcd)
data("UCBAdmissions")

structable(Dept ~ Admit+Gender,UCBAdmissions)

## conditional independence in UCB admissions data
mod.1 <- loglm(~ Dept * (Gender + Admit), data=UCBAdmissions)
mod.1

# this is correct, except the Pearson residuals dont show that
# all the lack of fit is concentrated in Dept A
mosaic(mod.1, gp=shading_Friendly, labeling=labeling_residuals)


library(vcdExtra)

# using glm()
berkeley <- as.data.frame(UCBAdmissions)
mod.3 <- glm(Freq ~ Dept * (Gender+Admit), data=berkeley, family="poisson")
summary(mod.3)

# (BUG FIXED )the large residuals are all in Dept A
mosaic(mod.3, 
       formula = ~Admit + Gender + Dept,
       residuals_type="rstandard", 
       labeling=labeling_residuals)

# wong2-3  Political views and support for women to work

library(vcdExtra)
# Data from Wong, R. (2010), Association Models, Los Angeles: Sage, Number 07-164
#      Table 2.3A, from the General Social Survey, 1998-2000.
# Questions:
#   polviews:  Think of self as liberal or conservative
#   fefam:     Better for men to work and women to tend home

Freq<-c(39, 50, 18,  4,
       140,178, 85, 23,
       108,195, 97, 23,
       238,598,363,111,
        78,250,150, 55,
        50,200,208, 74,
         8, 29, 46, 21)
polviews<- gl(7,4)
fefam <- gl(4,1,length=28)

# create better labels for levels in mosaic()
## but, this screws up parameter names: use set_labels instead
#polviews <- factor(polviews, labels=c("Lib++", "2", "3", "Moderate", "5", "6", "Cons++"))
#fefam <- factor(fefam, labels=c("Dis+", "Dis", "Agr", "Agr+"))
#
long.vnames <- list(set_varnames = c(polviews="Political views", fefam="Females should tend home"))
long.lnames <- list(polviews = c("Lib++", "2", "3", "Moderate", "5", "6", "Cons++"),
                    fefam = c("Dis+", "Dis", "Agr", "Agr+"))


# numeric versions for U, R, C, RC models
Rscore<-as.numeric(polviews)
Cscore<-as.numeric(fefam)

# make a data frame
Wong23 <- data.frame(Freq, polviews, fefam, Rscore, Cscore)

####################################
# do a correspondence analysis first, to see the row/category relations
Wong23.xtab <- xtabs(Freq ~ polviews+fefam, data=Wong23)
dimnames(Wong23.xtab) <- long.lnames
library(ca)

plot(ca(Wong23.xtab))
title(main="Political views and support for women to work",
      xlab="Dim 1 (90.8%)", ylab="Dim 2 (8.5%)")


####################################


## OK now, gives warning
Wong23.O <- gnm(Freq~polviews+fefam, family=poisson, data=Wong23)

# OK, with formula
mosaic(Wong23.O, main=paste("Independence model", modFit(Wong23.O)),
	formula=~polviews+fefam,
	labeling_args=long.vnames, set_labels=long.lnames)

####################################
# Uniform association model

Wong23.U<-gnm(Freq~polviews+fefam+Rscore:Cscore,family=poisson,tolerance = 1e-12, data=Wong23)
anova(Wong23.U)
# OK, w/o formula
mosaic(Wong23.U, main=paste("Uniform association", modFit(Wong23.U)),
	formula=~polviews+fefam,
	labeling_args=long.vnames, set_labels=long.lnames)

# display standardized residuals
mosaic(Wong23.U, formula=~polviews+fefam, main=paste("Uniform association", modFit(Wong23.U)),
	labeling_args=long.vnames, set_labels=long.lnames, residuals_type="rstandard",
	labeling=labeling_residuals, suppress=1)

# compare with gp=shading_Friendly)
mosaic(Wong23.U, formula=~polviews+fefam, main=paste("Uniform association", modFit(Wong23.U)),
	labeling_args=long.vnames, set_labels=long.lnames, residuals_type="rstandard",
	labeling=labeling_residuals, suppress=1, gp=shading_Friendly)

####################################

# Model B - R: Row Effects
Wong23.R<-gnm(Freq~polviews+fefam+Cscore:polviews,family=poisson, data=Wong23)
anova(Wong23.R)
mosaic(Wong23.R, formula=~polviews+fefam, main="Row effects model",
	labeling_args=long.vnames, set_labels=long.lnames, residuals_type="rstandard",
	labeling=labeling_residuals, suppress=1, gp=shading_Friendly)

#####################################
# Model C - C: Column Effect
Wong23.C<-gnm(Freq~polviews+fefam+Rscore:fefam,family=poisson, data=Wong23)
anova(Wong23.C)
mosaic(Wong23.C, formula=~polviews+fefam, main="Column effects model",
	labeling_args=long.vnames, set_labels=long.lnames, residuals_type="rstandard",
	labeling=labeling_residuals, suppress=1, gp=shading_Friendly)

#####################################
# Model D - R+C: Row and Column Effect
oldopt <- options(contrasts = c(factor="contr.treatment", ordered="contr.treatment"))
Wong23.RplusC<-gnm(Freq~polviews+fefam+Rscore:Cscore+Cscore:polviews+Rscore:fefam,
         constrain=c(17,20),constrainTo=c(0,0),
         family=poisson,tolerance = 1e-12)
anova(Wong23.RplusC)
mosaic(Wong23.RplusC, formula=~polviews+fefam, main="Column effects model",
	labeling_args=long.vnames, set_labels=long.lnames, residuals_type="rstandard",
	labeling=labeling_residuals, suppress=1, gp=shading_Friendly)

options(oldopt)

#####################################
# Model E - RC: RC(1) model
Wong23.RC1<-gnm(Freq~polviews+fefam+Mult(1,polviews,fefam),
         family=poisson,tolerance = 1e-12)

mosaic(Wong23.RC1, formula=~polviews+fefam, main="RC(1) model",
	labeling_args=long.vnames, set_labels=long.lnames, residuals_type="rstandard",
	labeling=labeling_residuals, suppress=1, gp=shading_Friendly)

#####################################
# LRstats the collection of models
models <- list(Indep=Wong23.O, Uniform=Wong23.U, RowEff=Wong23.R, ColEff=Wong23.C, RplusC=Wong23.RplusC, RC1=Wong23.RC1)
res <- lapply(models, residuals)
boxplot(as.data.frame(res), main="Residuals from various models")

aic <- t(as.data.frame(lapply(models, extractAIC)))
colnames(aic) <- c("df", "AIC")
aic
# sort by df
aic <- aic[order(aic[,1]),]
plot(aic, type = "b", main="AIC plot")
text(aic, labels=rownames(aic), pos=c(4,1,3,1,3,1))

######################################
# compare models;  they are not nested, so only some Chisq tests make sense
anova(Wong23.O, Wong23.U, Wong23.R, Wong23.C, Wong23.RplusC, Wong23.RC1)

anova(Wong23.O, Wong23.U, Wong23.R, Wong23.RplusC, test="Chisq")
anova(Wong23.O, Wong23.U, Wong23.C, Wong23.RplusC, test="Chisq")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/mobility-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# colorize text
colorize <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}

## ----hauser-data--------------------------------------------------------------
data("Hauser79", package="vcdExtra")
str(Hauser79)
(Hauser_tab <- xtabs(Freq ~ Father + Son, data=Hauser79))

## ----load---------------------------------------------------------------------
library(vcdExtra)
library(gnm)
library(dplyr)

## ----mosaicplot---------------------------------------------------------------
plot(Hauser_tab, shade=TRUE)

## ----mosaic1------------------------------------------------------------------
labels <- list(set_varnames = c(Father="Father's occupation", 
                                Son="Son's occupation"))

mosaic(Freq ~ Father + Son, data=Hauser79,
       labeling_args = labels,
       shade=TRUE,
       legend = FALSE)

## ----indep--------------------------------------------------------------------
hauser.indep <- glm(Freq ~ Father + Son, 
  data=Hauser79, 
  family=poisson)

# the same mosaic, using the fitted model
mosaic(hauser.indep, formula = ~ Father + Son, 
       labeling_args = labels,
       legend = FALSE,
       main="Independence model")

## ----Diag---------------------------------------------------------------------
# with symbols
with(Hauser79, Diag(Father, Son)) |> matrix(nrow=5)

## ----quasi--------------------------------------------------------------------
hauser.quasi <-  update(hauser.indep, 
                        ~ . + Diag(Father, Son))

mosaic(hauser.quasi, ~ Father+Son, 
       labeling_args = labels,
       legend = FALSE,
       main="Quasi-independence model")


## ----symm---------------------------------------------------------------------
with(Hauser79, Symm(Father, Son)) |> matrix(nrow=5)

## ----qsymm--------------------------------------------------------------------
hauser.qsymm <-  update(hauser.indep, 
                        ~ . + Diag(Father,Son) + Symm(Father,Son))

## ----anova1-------------------------------------------------------------------
anova(hauser.indep, hauser.quasi, hauser.qsymm, test="Chisq")

LRstats(hauser.indep, hauser.quasi, hauser.qsymm)

## ----qsymm-mosaic-------------------------------------------------------------
mosaic(hauser.qsymm, ~ Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Quasi-symmetry model")

## ----topo-levels--------------------------------------------------------------
# Levels for Hauser 5-level model
levels <- matrix(c(
      2,  4,  5,  5,  5,
      3,  4,  5,  5,  5,
      5,  5,  5,  5,  5,
      5,  5,  5,  4,  4,
      5,  5,  5,  4,  1), 
      nrow = 5, ncol = 5, 
      byrow=TRUE)

## ----topo-mosaic--------------------------------------------------------------
hauser.topo <- update(hauser.indep, 
                      ~ . + Topo(Father, Son, spec=levels))

mosaic(hauser.topo, ~Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Topological model")

## -----------------------------------------------------------------------------
LRstats(hauser.indep, hauser.quasi, hauser.qsymm, hauser.topo, sortby = "AIC")


## ----scores-------------------------------------------------------------------
Sscore <- as.numeric(Hauser79$Son)
Fscore <- as.numeric(Hauser79$Father)

Hauser79 |> cbind(Fscore, Fscore) |> head()

## ----hauser-UAdiag------------------------------------------------------------
hauser.UAdiag <- update(hauser.indep,
                        . ~ . + Fscore : Sscore + Diag(Father, Son))

LRstats(hauser.UAdiag)

## -----------------------------------------------------------------------------
coef(hauser.UAdiag)[["Fscore:Sscore"]]

## ----UAdiag-mosaic------------------------------------------------------------
mosaic(hauser.UAdiag, ~ Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Uniform association + Diag()")

## Models for Yamaguchi (1987) data on social mobility in US, UK and Japan, following Xie (1992)
## These models reproduce the results in Table 1, appplied to the off-diagonal cells

library(gnm)
library(vcdExtra)

data(Yamaguchi87)
# create table form
Yama.tab <- xtabs(Freq ~ Father + Son + Country, data=Yamaguchi87)

# define labeling_args for convenient reuse in 3-way displays
largs <- list(rot_labels=c(right=0), offset_varnames = c(right = 0.6), offset_labels = c(right = 0.2),
		set_varnames = c(Father="Father's status", Son="Son's status") 
)

# no association between F and S given country ('perfect mobility')
# asserts same associations for all countries

yamaNull <- gnm(Freq ~ (Father + Son) * Country, data=Yamaguchi87, family=poisson)
LRstats(yamaNull)
mosaic(yamaNull, ~Country + Son + Father, condvars="Country", 
		labeling_args=largs,
		main="[FC][SC] Null [FS] association (perfect mobility)")

## same, with data in xtabs form
#yamaNull <- gnm(Freq ~ (Father + Son) * Country, data=Yama.tab, family=poisson)
#LRstats(yamaNull)
#mosaic(yamaNull, ~Country + Son + Father, condvars="Country", 
#	labeling_args=largs,
#	main="[FC][SC] Null [FS] association (perfect mobility)")

# ignore diagonal cells, overall
#yamaDiag0 <- gnm(Freq ~ (Father + Son) * Country + Diag(Father, Son), data=Yama.tab, family=poisson)
#LRstats(yamaDiag0)
# same, using update()
yamaDiag0 <- update(yamaNull, ~ . + Diag(Father, Son))
LRstats(yamaDiag0)

# ignore diagonal cells in each Country [Model NA in Xie(1992), Table 1]
yamaDiag <- update(yamaNull, ~ . + Diag(Father, Son):Country)
LRstats(yamaDiag)
mosaic(yamaDiag, ~Country + Son + Father, condvars="Country", 
		labeling_args=largs, gp=shading_Friendly,
		main="[FC][SC] Quasi perfect mobility, +Diag(F,S)")


# fit models using integer scores for rows/cols 
Rscore <- as.numeric(Yamaguchi87$Father)
Cscore <- as.numeric(Yamaguchi87$Son)

# cross-nationally homogeneous row effect associations (Xie, model R_o)
yamaRo <- update(yamaDiag, ~ . + Father:Cscore)
LRstats(yamaRo)
mosaic(yamaRo, ~Country + Son + Father, condvars="Country", 
		labeling_args=largs, gp=shading_Friendly,
		main="Model Ro: homogeneous row effects, +Father:j ")

# cross-nationally log multiplicative row effect associations (Xie, model R_x)
yamaRx <- update(yamaDiag, ~ . + Mult(Father:Cscore, Exp(Country)))
LRstats(yamaRx)

# cross-nationally homogeneous col effect associations (Xie, model C_o)
yamaCo <- update(yamaDiag, ~ . + Rscore:Son)
LRstats(yamaCo)
mosaic(yamaCo, ~Country + Son + Father, condvars="Country", 
		labeling_args=largs, gp=shading_Friendly,
		main="Model Co: homogeneous col effects, +i:Son")


# cross-nationally log multiplicative col effect associations (Xie, model C_x)
yamaCx <- update(yamaDiag, ~ . + Mult(Rscore:Son, Exp(Country)))
LRstats(yamaCx)

# cross-nationally homogeneous row + col effect associations I (Xie, model (R+C)_o)
yamaRpCo <- update(yamaDiag, ~ . + Father:Cscore + Rscore:Son)
LRstats(yamaRpCo)
mosaic(yamaRpCo, ~Country + Son + Father, condvars="Country", 
		labeling_args=largs, gp=shading_Friendly,
		main="Model (R+C)o: homogeneous, F:j + i:S")

# cross-nationally log multiplicative row + col effect associations I (Xie, model (R+C)_x)
yamaRpCx <- update(yamaDiag, ~ . + Mult(Father:Cscore + Rscore:Son, Exp(Country)))
LRstats(yamaRpCx)
mosaic(yamaRpCx, ~Country + Son + Father, condvars="Country", 
		labeling_args=largs, gp=shading_Friendly,
		main="Model (R+C)x: log multiplicative (Fj + iS) : Country")

# cross-nationally homogeneous row and col effect associations II (Xie, model RC_o)
yamaRCo <- update(yamaDiag, ~ . + Mult(Father,Son))
LRstats(yamaRCo)
mosaic(yamaRCo, ~Country + Son + Father, condvars="Country", 
		labeling_args=largs, gp=shading_Friendly,
		main="Model RCo: homogeneous RC(1)")

# cross-nationally log multiplicative row and col effect associations II (Xie, model RC_x)
yamaRCx <- update(yamaDiag, ~ . + Mult(Father,Son, Exp(Country)))
LRstats(yamaRCx)
mosaic(yamaRCx, ~Country + Son + Father, condvars="Country", 
		labeling_args=largs, gp=shading_Friendly,
		main="Model RCx: log multiplicative RC(1) : Country")

# cross-nationally homogeneous full two-way RxC association (Xie, model FI_o)
yamaFIo <- update(yamaDiag, ~ . + Father:Son)
LRstats(yamaFIo)

# cross-nationally log multiplicative full two-way RxC association (Xie, model FI_x)
yamaFIx <- update(yamaDiag, ~ . + Mult(Father:Son, Exp(Country)))
LRstats(yamaFIx)

# compare models
models <- glmlist(yamaNull, yamaDiag, yamaRo, yamaRx, yamaCo, yamaCx, yamaRpCo, yamaRpCx, yamaRCo, yamaRCx, yamaFIo, yamaFIx)
LRstats(models)


# extract models sumaries, consider as factorial of RC model by layer model

BIC <- matrix(LRstats(models)$BIC[-(1:2)], 5, 2, byrow=TRUE)
dimnames(BIC) <- list("Father-Son model" = c("row eff", "col eff", "row+col", "RC(1)", "R:C"),
		"Country model" = c("homogeneous", "log multiplicative"))
BIC

matplot(BIC, type='b', xlab="Father-Son model", xaxt='n', pch=15:16, cex=1.5, cex.lab=1.5,
		main="Yamaguchi-Xie models: R:C model by Layer model Summary")
axis(side=1, at=1:nrow(BIC), labels=rownames(BIC), cex.axis=1.2)
text(5, BIC[5,], colnames(BIC), pos=2, col=1:2, cex=1.2)
text(5, max(BIC[5,])+10, "Country model", pos=2, cex=1.3)

AIC <- matrix(LRstats(models)$AIC[-(1:2)], 5, 2, byrow=TRUE)
dimnames(AIC) <- list("Father-Son model" = c("row eff", "col eff", "row+col", "RC(1)", "R:C"),
		"Country model" = c("homogeneous", "log multiplicative"))
AIC

matplot(AIC, type='b', xlab="Father-Son model", xaxt='n', pch=15:16, cex=1.5, cex.lab=1.5,
		main="Yamaguchi-Xie models: R:C model by Layer model Summary")
axis(side=1, at=1:nrow(AIC), labels=rownames(AIC), cex.axis=1.2)
text(5, AIC[5,], colnames(AIC), pos=2, col=1:2, cex=1.2)
text(5, max(AIC[5,])+10, "Country model", pos=2, cex=1.3)

## Mental health data: mosaics for glm() and gnm() models
library(gnm)
library(vcdExtra)
data(Mental)

# display the frequency table
(Mental.tab <- xtabs(Freq ~ mental+ses, data=Mental))

# fit independence model
# Residual deviance: 47.418 on 15 degrees of freedom
indep <- glm(Freq ~ mental+ses,
                family = poisson, data = Mental)
deviance(indep)


long.labels <- list(set_varnames = c(mental="Mental Health Status", ses="Parent SES"))
mosaic(indep,residuals_type="rstandard", labeling_args = long.labels, labeling=labeling_residuals,
       main="Mental health data: Independence")
# as a sieve diagram
mosaic(indep, labeling_args = long.labels, panel=sieve, gp=shading_Friendly,
       main="Mental health data: Independence")
 
# fit linear x linear (uniform) association.  Use integer scores for rows/cols 
Cscore <- as.numeric(Mental$ses)
Rscore <- as.numeric(Mental$mental)

# column effects model (ses)
coleff <- glm(Freq ~ mental + ses + Rscore:ses,
                family = poisson, data = Mental)
mosaic(coleff,residuals_type="rstandard", 
 labeling_args = long.labels, labeling=labeling_residuals, suppress=1, gp=shading_Friendly,
 main="Mental health data: Col effects (ses)")

# row effects model (mental)
roweff <- glm(Freq ~ mental + ses + mental:Cscore,
                family = poisson, data = Mental)
mosaic(roweff,residuals_type="rstandard", 
 labeling_args = long.labels, labeling=labeling_residuals, suppress=1, gp=shading_Friendly,
 main="Mental health data: Row effects (mental)")
               
linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
                family = poisson, data = Mental)

# compare models
anova(indep, roweff, coleff, linlin)
AIC(indep, roweff, coleff, linlin)
            
mosaic(linlin,residuals_type="rstandard", 
 labeling_args = long.labels, labeling=labeling_residuals, suppress=1, gp=shading_Friendly,
 main="Mental health data: Linear x Linear")


##  Goodman Row-Column association model fits well (deviance 3.57, df 8)
Mental$mental <- C(Mental$mental, treatment)
Mental$ses <- C(Mental$ses, treatment)
RC1model <- gnm(Freq ~ mental + ses + Mult(mental, ses),
                family = poisson, data = Mental)

mosaic(RC1model,residuals_type="rstandard", 
 labeling_args = long.labels, labeling=labeling_residuals, suppress=1, gp=shading_Friendly,
 main="Mental health data: RC1 model")

# Occupational status data from Goodman (1979) and Duncan (1979)
# Fit a variety of models.  Compare mosaic using expected= to mosaic.glm
# Occ status (1:8)-- professional, managerial, upper non-man, lower non-man,
#        ... unskilled

library(gnm)
library(vcdExtra)
data(occupationalStatus, package="datasets")
str(occupationalStatus)
occupationalStatus

# graphics::mosaicplot is the default plot method for a table
plot(occupationalStatus, shade=TRUE)

# define long labels for use in mosaics
long.labels <- list(set_varnames = c(origin="origin: Son's status", 
                                     destination="destination: Father's status"))

mosaic(occupationalStatus, shade=TRUE, 
       main="Occupational status: Independence model", 
       labeling_args = long.labels, 
       legend=FALSE)

# the standard model of independence
indep <- glm(Freq ~ origin + destination, 
             family = poisson, 
             data=occupationalStatus)

# the same mosaic, using the fitted model
mosaic(indep, 
       main="Independence model",
       labeling_args = long.labels, 
       legend=FALSE, 
       gp=shading_Friendly)


# fit the model of quasi-independence, ignoring the diagonal cells
quasi <-  gnm(Freq ~ origin + destination + Diag(origin,destination), 
                           family=poisson, data=occupationalStatus)
#str(quasi$data)
anova(quasi, test="Chisq")

## BUGLET (vcd): the diagonal cells should be considered 0 here--- outlined in black
mosaic(occupationalStatus, 
       expected=fitted(quasi), 
       main="Quasi-independence model", 
       labeling_args = long.labels, 
       legend=FALSE, gp=shading_Friendly)


## using mosaic.gnm
mosaic(quasi, main="Quasi-independence model",
  labeling_args = long.labels, 
  legend=FALSE, gp=shading_Friendly, 
  labeling=labeling_residuals)


# symmetry model
symmetry <- glm(Freq ~ Symm(origin, destination), 
                family=poisson, 
                data=occupationalStatus)
# mosaic(occupationalStatus, expected=fitted(symmetry), main="Symmetry model",
# 	gp=shading_Friendly, labeling=labeling_residuals, labeling_args = long.labels )

# using mosaic.glm --- OK
mosaic(symmetry, 
       main="Symmetry model", 
       gp=shading_Friendly, 
       labeling=labeling_residuals, 
       labeling_args = long.labels )

quasi.symm <- glm(Freq ~ origin + destination + Symm(origin, destination), 
                  family=poisson, 
                  data=occupationalStatus)
anova(quasi.symm)
mosaic(occupationalStatus, 
       expected=fitted(quasi.symm), 
       main="Quasi-symmetry model")

# model comparisons
anova(independence, quasi, quasi.symm, test="Chisq")

# compare symmetry to quasi summetry
anova(symmetry, quasi.symm, test="Chisq")

# association models
# uniform association, aka linear x linear association
Rscore <- as.vector(row(occupationalStatus))
Cscore <- as.vector(col(occupationalStatus))

uniform <- gnm(Freq ~ origin + destination + Rscore:Cscore, 
               family=poisson, 
               data=occupationalStatus)

mosaic(uniform, 
       main="Uniform association model", 
       labeling_args = long.labels, 
       legend=FALSE,
       gp=shading_Friendly, 
       labeling=labeling_residuals )

RChomog <- gnm(Freq ~ origin + destination + Diag(origin, destination) + 
                 MultHomog(origin, destination), 
               family=poisson, 
               data=occupationalStatus)

mosaic(RChomog, 
       main="RC homogeneous model", 
       labeling_args = long.labels, 
       legend=FALSE,
       gp=shading_Friendly, 
       labeling=labeling_residuals)

# RC1 - heterogeneous association
RC1 <- gnm(Freq ~ origin + destination + Diag(origin, destination) + 
             Mult(origin, destination),	
           family=poisson, 
           data=occupationalStatus)

mosaic(RC1, main="RC heterogeneous model", 
       labeling_args = long.labels, 
       legend=FALSE,
       gp=shading_Friendly, 
       labeling=labeling_residuals)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tut01-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# Old Sweave options
# \SweaveOpts{engine=R,eps=TRUE,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{engine=R,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{prefix.string=fig/vcd-tut,eps=FALSE}
# \SweaveOpts{keep.source=TRUE}


# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(HairEyeColor)
data(PreSex)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
#if(!file.exists("fig")) dir.create("fig")


## ----case-form----------------------------------------------------------------
names(Arthritis)      # show the variables

str(Arthritis)        # show the structure

head(Arthritis,5)     # first 5 observations, same as Arthritis[1:5,] 

## ----frequency-form-----------------------------------------------------------
# Agresti (2002), table 3.11, p. 106
GSS <- data.frame(
  expand.grid(sex = c("female", "male"), 
              party = c("dem", "indep", "rep")),
  count = c(279,165,73,47,225,191))

GSS
names(GSS)
str(GSS)

sum(GSS$count)

## ----table-form1--------------------------------------------------------------
str(HairEyeColor)                      # show the structure

sum(HairEyeColor)                      # number of cases

sapply(dimnames(HairEyeColor), length) # table dimension sizes

## ----table-form2--------------------------------------------------------------
# A 4 x 4 table  Agresti (2002, Table 2.8, p. 57) Job Satisfaction
JobSat <- matrix(c( 1, 2, 1, 0, 
                    3, 3, 6, 1, 
                   10,10,14, 9, 
                    6, 7,12,11), 4, 4)

dimnames(JobSat) = list(
  income = c("< 15k", "15-25k", "25-40k", "> 40k"),
  satisfaction = c("VeryD", "LittleD", "ModerateS", "VeryS")
  )

JobSat

## ----table-form3--------------------------------------------------------------
JobSat <- as.table(JobSat)
str(JobSat)

## ----relevel, eval=FALSE------------------------------------------------------
# dimnames(JobSat)$income <- c(7.5,20,32.5,60)
# dimnames(JobSat)$satisfaction <- 1:4

## ----reorder1-----------------------------------------------------------------
HairEyeColor <- HairEyeColor[, c(1,3,4,2), ]
str(HairEyeColor)

## ----reorder2, echo=TRUE, eval=FALSE------------------------------------------
# Arthritis <- read.csv("arthritis.txt",header=TRUE)
# Arthritis$Improved <- ordered(Arthritis$Improved,
#                               levels=c("None", "Some", "Marked")
#                               )

## -----------------------------------------------------------------------------
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
mosaic(art, gp = shading_max, split_vertical = TRUE, main="Arthritis: [Treatment] [Improved]")

## ----reorder3-----------------------------------------------------------------
UCB <- aperm(UCBAdmissions, c(2, 1, 3))
dimnames(UCB)[[2]] <- c("Yes", "No")
names(dimnames(UCB)) <- c("Sex", "Admit?", "Department")

# display as a flattened table
stats::ftable(UCB)

## ----structable---------------------------------------------------------------
structable(HairEyeColor)                   # show the table: default

structable(Hair+Sex ~ Eye, HairEyeColor)   # specify col ~ row variables

## ----structable1,eval=FALSE---------------------------------------------------
# HSE < - structable(Hair+Sex ~ Eye, HairEyeColor)   # save structable object
# mosaic(HSE)                                        # plot it

## ----table-setup--------------------------------------------------------------
 n=500
 A <- factor(sample(c("a1","a2"), n, rep=TRUE))
 B <- factor(sample(c("b1","b2"), n, rep=TRUE))
 C <- factor(sample(c("c1","c2"), n, rep=TRUE))
 mydata <- data.frame(A,B,C)

## ----table-ex1----------------------------------------------------------------
# 2-Way Frequency Table
attach(mydata)
mytable <- table(A,B)   # A will be rows, B will be columns
mytable                 # print table

margin.table(mytable, 1) # A frequencies (summed over B)
margin.table(mytable, 2) # B frequencies (summed over A)

prop.table(mytable)    # cell percentages
prop.table(mytable, 1) # row percentages
prop.table(mytable, 2) # column percentages

## ----table-ex2----------------------------------------------------------------
# 3-Way Frequency Table
mytable <- table(A, B, C)
ftable(mytable)

## ----xtabs-ex1----------------------------------------------------------------
# 3-Way Frequency Table
mytable <- xtabs(~A+B+C, data=mydata)

ftable(mytable)    # print table

summary(mytable)   # chi-square test of indepedence

## ----xtabs-ex2----------------------------------------------------------------
(GSStab <- xtabs(count ~ sex + party, data=GSS))

summary(GSStab)

## ----dayton1------------------------------------------------------------------
data("DaytonSurvey", package="vcdExtra")
str(DaytonSurvey)
head(DaytonSurvey)

## ----dayton2------------------------------------------------------------------
# data in frequency form
# collapse over sex and race
Dayton.ACM.df <- aggregate(Freq ~ cigarette+alcohol+marijuana, 
                           data=DaytonSurvey, 
                           FUN=sum)
Dayton.ACM.df

## ----dayton3------------------------------------------------------------------
# in table form
Dayton.tab <- xtabs(Freq ~ cigarette+alcohol+marijuana+sex+race, 
                    data=DaytonSurvey)
structable(cigarette+alcohol+marijuana ~ sex+race, 
           data=Dayton.tab)

## ----dayton4------------------------------------------------------------------
# collapse over sex and race
Dayton.ACM.tab <- apply(Dayton.tab, MARGIN=1:3, FUN=sum)
Dayton.ACM.tab <- margin.table(Dayton.tab, 1:3)   # same result

structable(cigarette+alcohol ~ marijuana, data=Dayton.ACM.tab)

## ----dayton5------------------------------------------------------------------
library(plyr)
Dayton.ACM.df <- plyr::ddply(DaytonSurvey, 
                             .(cigarette, alcohol, marijuana), 
                             plyr::summarise, Freq=sum(Freq))

Dayton.ACM.df

## ----collapse1----------------------------------------------------------------
# create some sample data in frequency form
sex <- c("Male", "Female")
age <- c("10-19", "20-29",  "30-39", "40-49", "50-59", "60-69")
education <- c("low", 'med', 'high')
data <- expand.grid(sex=sex, age=age, education=education)
counts <- rpois(36, 100)   # random Possion cell frequencies
data <- cbind(data, counts)

# make it into a 3-way table
t1 <- xtabs(counts ~ sex + age + education, data=data)
structable(t1)

## ----collapse2----------------------------------------------------------------
# collapse age to 3 levels, education to 2 levels
t2 <- collapse.table(t1, 
         age=c("10-29", "10-29",  "30-49", "30-49", "50-69", "50-69"),
         education=c("<high", "<high", "high"))
structable(t2)

## ----titanicSAVE, include=FALSE, echo=FALSE-----------------------------------
# Save unmodified Titanicp data for collapse_levels() example
saveDat <- Titanicp

## ----titanicp1----------------------------------------------------------------
table(Titanicp$sibsp, Titanicp$parch)

## ----titanicp2----------------------------------------------------------------
library(dplyr)
Titanicp <- Titanicp |>
  mutate(sibspF = case_match(sibsp,
                            0 ~ "0",
                            1 ~ "1",
                            2:max(sibsp) ~ "2+")) |>
  mutate(sibspF = ordered(sibspF)) |>
  mutate(parchF = case_match(parch,
                             0 ~ "0",
                             1 ~ "1",
                             2:max(parch) ~ "2+")) |>
  mutate(parchF = ordered(parchF)) 

table(Titanicp$sibspF, Titanicp$parchF)

## ----titanicLOAD, include=FALSE, echo=FALSE-----------------------------------
# Load unmodified Titanicp data for collapse_levels() example
Titanicp <- saveDat

## ----collapse_level-ex1-------------------------------------------------------
str(Titanicp) # The original dataset

Titanicp <- within(Titanicp, {
  parchF <- factor(parch)
  sibspF <- factor(sibsp)
})

Titanicp <- collapse_levels(
  Titanicp,
  parchF = list(
    "2+" = c("2", "3", "4", "5", "6", "9")
  ),
  sibspF = list(
    "2+" = c("2", "3", "4", "5", "8")
  )
)

table(Titanicp$sibspF, Titanicp$parchF)

## ----convert-ex1--------------------------------------------------------------
as.data.frame(GSStab)

## ----convert-ex2--------------------------------------------------------------
Art.tab <- with(Arthritis, table(Treatment, Sex, Improved))
str(Art.tab)

ftable(Art.tab)

## ----convert-ex3--------------------------------------------------------------
Art.df <- expand.dft(Art.tab)
str(Art.df)

## ----tidyconvert-ex1----------------------------------------------------------
# tidy = TRUE (default) returns a tibble
GSS.ff <- as_freqform(GSStab, tidy = FALSE)
GSS.ff

## ----tidyconvert-ex2----------------------------------------------------------
as_table(GSS.ff, freq = "Freq") # When present, freq column must be specified

## ----tidyconvert-ex3----------------------------------------------------------
tidy_Art.tab <- as_table(Arthritis, dims = c("Treatment", "Sex", "Improved"))
str(tidy_Art.tab)

ftable(tidy_Art.tab)

## ----tidyconvert-ex4----------------------------------------------------------
tidy_Art.df <- as_caseform(tidy_Art.tab, tidy = FALSE)
str(tidy_Art.df)

## ----tv1----------------------------------------------------------------------
tv.data<-read.table(system.file("extdata","tv.dat", package="vcdExtra"))
head(tv.data,5)

## ----tv2,eval=FALSE-----------------------------------------------------------
# tv.data<-read.table("C:/R/data/tv.dat")

## ----tv3----------------------------------------------------------------------
TV <- array(tv.data[,5], dim=c(5,11,5,3))                                        
dimnames(TV) <- list(c("Monday","Tuesday","Wednesday","Thursday","Friday"), 
                     c("8:00","8:15","8:30","8:45","9:00","9:15","9:30",         
                       "9:45","10:00","10:15","10:30"),                            
                     c("ABC","CBS","NBC","Fox","Other"), 
                     c("Off","Switch","Persist"))

names(dimnames(TV))<-c("Day", "Time", "Network", "State")

## ----tv3a,eval=FALSE----------------------------------------------------------
# TV <- xtabs(V5 ~ ., data=tv.data)
# dimnames(TV) <- list(Day = c("Monday","Tuesday","Wednesday","Thursday","Friday"),
#                      Time = c("8:00","8:15","8:30","8:45","9:00","9:15","9:30",
#                               "9:45","10:00","10:15","10:30"),
#                      Network = c("ABC","CBS","NBC","Fox","Other"),
#                      State = c("Off","Switch","Persist"))
# 
# # table dimensions
# dim(TV)

## ----tv4----------------------------------------------------------------------
TV2 <- TV[,,1:3,]      # keep only ABC, CBS, NBC
TV2 <- TV2[,,,3]       # keep only Persist -- now a 3 way table
structable(TV2)

## ----tv5----------------------------------------------------------------------
TV.df <- as.data.frame.table(TV2)
levels(TV.df$Time) <- c(rep("8:00", 2),
                        rep("8:30", 2),
                        rep("9:00", 2), 
                        rep("9:30", 2), 
                        rep("10:00",2),
                            "10:30"
                        )

TV3 <- xtabs(Freq ~ Day + Time + Network, TV.df)

structable(Day ~ Time+Network, TV3)

## ----tv-mosaic1, fig.height=6, fig.width=7------------------------------------
mosaic(TV3, shade = TRUE,
       labeling = labeling_border(rot_labels = c(0, 0, 0, 90)))

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  dev = "png",
  comment = "##"
)

library(vcdExtra)
library(dplyr)
library(tidyr)

## ----overoll_loadselect-------------------------------------------------------
data("starwars", package = "dplyr")

star_case <- starwars |>
  dplyr::select(c("hair_color", "skin_color", "eye_color")) |> 
  tidyr::drop_na()

str(star_case)

## ----overcoll_hairunique------------------------------------------------------
unique(star_case$hair_color)

## ----overcoll_ex1-------------------------------------------------------------
collapsed.star_case <- collapse_levels(
  star_case,             # The dataset
  hair_color = list(     # Assign the variable to be collapsed to a list
    
    # Format the list as NewLevel = c("old1", "old2", ..., "oldn")
    Blonde = c("blond", "blonde"), 
    Brown = c("brown", "brown, grey"),
    Auburn = c("auburn, white", "auburn, grey", "auburn")
  )
)
str(collapsed.star_case)
unique(collapsed.star_case$hair_color)

## ----overcoll_skinunique------------------------------------------------------
unique(star_case$skin_color)

## ----overcoll_ex2-------------------------------------------------------------
collapsed.star_case <- collapse_levels(
  collapsed.star_case,
  skin_color = list(
    Shades = c(
      "fair", "white", "light", "dark", "grey", "grey, red", 
      "grey, blue", "white, blue", "grey, green, yellow", "fair, green, yellow"
    ), 
    Rainbows = c(
      "green", "pale", "metal", "brown mottle", "brown", "mottled green", 
      "orange", "blue, grey", "red", "blue", "yellow", "tan", "silver, red",
      "green, grey", "red, blue, white", "brown, white"
    )
  )
)
str(collapsed.star_case)
unique(collapsed.star_case$skin_color)

## ----overcoll_eyeunique-------------------------------------------------------
unique(star_case$eye_color)

## ----overcoll_ex3-------------------------------------------------------------
collapsed.star_case <- collapse_levels(
  collapsed.star_case,
  eye_color = list(
    Normal = c("blue", "brown", "blue-gray", "hazel", "dark"), 
    Abnormal = c(
      "yellow", "red", "orange", "black", "pink", "red, blue", "gold", 
      "green, yellow", "white"
    )
  )
)
str(collapsed.star_case)
unique(collapsed.star_case$eye_color)

## ----overcoll_ex4-------------------------------------------------------------
collapsed.star_case <- collapse_levels(
  collapsed.star_case,
  hair_color = list(    # First variable
    Dark = c("Brown", "black", "Auburn"),
    Light = c("Blonde", "white", "grey")
  ),
  skin_color = list(    # Second variable
    Other = c("none", "unknown")
  )
)
unique(collapsed.star_case$hair_color)
unique(collapsed.star_case$skin_color)
str(collapsed.star_case)

## ----overconv-ex1-------------------------------------------------------------
star_freqform <- as_freqform(collapsed.star_case)

str(star_freqform)

## ----overconv-ex2-------------------------------------------------------------
as_freqform(collapsed.star_case, tidy = FALSE) |> str()

## ----overconv-ex3-------------------------------------------------------------
star_tab <- as_table(star_freqform, freq = "Freq")

str(star_tab)

## ----overconv-ex4-------------------------------------------------------------
star_array <- as_array(star_tab)

class(star_array)
str(star_array)

## ----overconv-ex5-------------------------------------------------------------
star_mat <- as_matrix(star_array, dims = c("hair_color", "eye_color"))

class(star_mat)
str(star_mat)

## ----overconv-ex6-------------------------------------------------------------
as_freqform(star_tab, dims = c("hair_color", "eye_color")) |> str()

## ----propconv-ex1-------------------------------------------------------------
star_mat # To view the original

as_table(star_mat, prop = TRUE)

## ----propconv-ex2-------------------------------------------------------------
as_table(star_mat, prop = "hair_color")

## ----propconv-ex3-------------------------------------------------------------
as_table(star_mat, prop = c("hair_color", "eye_color"))

## ----tt-ex1-------------------------------------------------------------------
home_star <- starwars |>
  dplyr::select(c("hair_color", "skin_color", "eye_color", "homeworld")) |> 
  tidyr::drop_na()

# Sort unique levels of homeworld
lvls <- home_star$homeworld |> unique() |> sort()
lvls

# Collapse variable levels
collapsed.home_star <- collapse_levels(
  home_star,
  homeworld = list(
    abc = lvls[1:(length(lvls)/2)],
    xyz = lvls[(length(lvls)/2 + 1):length(lvls)]
  ),
  eye_color = list(
    Normal = c("blue", "brown", "blue-gray", "hazel", "dark"), 
    Abnormal = c(
      "yellow", "red", "orange", "black", "pink", "red, blue", "gold", 
      "green, yellow", "white"
    )
  )
)
# Convert to table of dimensions 'homeworld' and 'eye_color'
tab.home_star <- as_table(collapsed.home_star, dims = c("homeworld", "eye_color"))

# Plot as mosaic display
mosaic(tab.home_star, shading = TRUE, gp = shading_Friendly)

# Convert table into matrix of proportions. Note argument 'dims' was not supplied
# as we already know that there are exactly 2 dimensions.
as_matrix(tab.home_star, prop = TRUE)

## ----ttpipeline, eval=FALSE---------------------------------------------------
# dataset |>                             # Gather the data
#   select(...) |> drop_na() |> ... |>   # Clean the data
#   collapse_levels(...) |>              # Collapse levels as necessary
#   as_form(...)   # Convert forms, select dimensions, take proportions

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/datasets-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

## ----load---------------------------------------------------------------------
library(dplyr)
library(tidyr)
library(readxl)

## ----read-datasets------------------------------------------------------------
dsets_tagged <- read_excel(here::here("inst", "extdata", "vcdExtra-datasets.xlsx"), 
                           sheet="vcdExtra-datasets")

dsets_tagged <- dsets_tagged |>
  dplyr::select(-Title, -dim) |>
  dplyr::rename(dataset = Item)

head(dsets_tagged)

## ----split-tags---------------------------------------------------------------
dset_split <- dsets_tagged |>
  tidyr::separate_longer_delim(tags, delim = ";") |>
  dplyr::mutate(tag = stringr::str_trim(tags)) |>
  dplyr::select(-tags)

#' ## collapse the rows for the same tag
tag_dset <- dset_split |>
  arrange(tag) |>
  dplyr::group_by(tag) |>
  dplyr::summarise(datasets = paste(dataset, collapse = "; ")) |> ungroup()

# get a list of the unique tags
unique(tag_dset$tag)

## ----read-tags----------------------------------------------------------------
tags <- read_excel(here::here("inst", "extdata", "vcdExtra-datasets.xlsx"), 
                   sheet="tags")
head(tags)

## ----join-tags----------------------------------------------------------------
tag_dset <- tag_dset |>
  dplyr::left_join(tags, by = "tag") |>
  dplyr::relocate(topic, .after = tag)

tag_dset |>
  dplyr::select(-tag) |>
  head()

## ----add-links----------------------------------------------------------------
add_links <- function(dsets, 
                      style = c("reference", "help", "rdrr.io"),
                      sep = "; ") {

  style <- match.arg(style)
  names <- stringr::str_split_1(dsets, sep)

  names <- dplyr::case_when(
    style == "help"      ~ glue::glue("[{names}](help({names}))"),
    style == "reference" ~ glue::glue("[{names}](../reference/{names}.html)"),
    style == "rdrr.io"   ~ glue::glue("[{names}](https://rdrr.io/cran/vcdExtra/man/{names}.html)")
  )  
  glue::glue_collapse(names, sep = sep)
}


## ----kable--------------------------------------------------------------------
tag_dset |>
  dplyr::select(-tag) |>
  dplyr::mutate(datasets = purrr::map(datasets, add_links)) |>
  knitr::kable()

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tut05-",
  fig.align = "center",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
#if(!file.exists("fig")) dir.create("fig")


## ----spine1-------------------------------------------------------------------
(spine(Improved ~ Age, data = Arthritis, breaks = 3))
(spine(Improved ~ Age, data = Arthritis, breaks = "Scott"))

## -----------------------------------------------------------------------------
cdplot(Improved ~ Age, data = Arthritis)

## -----------------------------------------------------------------------------
cdplot(Improved ~ Age, data = Arthritis)
with(Arthritis, rug(jitter(Age), col="white", quiet=TRUE))

## ----donner1------------------------------------------------------------------
data(Donner, package="vcdExtra")
str(Donner)

## ----donner2a, fig=FALSE, eval=FALSE------------------------------------------
# # separate linear fits on age for M/F
# ggplot(Donner, aes(age, survived, color = sex)) +
#   geom_point(position = position_jitter(height = 0.02, width = 0)) +
#   stat_smooth(method = "glm",
#               method.args = list(family = binomial),
#               formula = y ~ x,
#               alpha = 0.2, size=2, aes(fill = sex))

## ----donner2b, fig=FALSE, eval=FALSE------------------------------------------
# # separate quadratics
# ggplot(Donner, aes(age, survived, color = sex)) +
#   geom_point(position = position_jitter(height = 0.02, width = 0)) +
#   stat_smooth(method = "glm",
#               method.args = list(family = binomial),
#               formula = y ~ poly(x,2),
#               alpha = 0.2, size=2, aes(fill = sex))

## -----------------------------------------------------------------------------
# separate linear fits on age for M/F
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ x,
              alpha = 0.2, size=2, aes(fill = sex))

# separate quadratics
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ poly(x,2),
              alpha = 0.2, size=2, aes(fill = sex))

## mosaic3d-hec	2D and 3D visualizations of HairEyeColor data

library(vcdExtra)

# two-way mosaic displays
HairEye <- margin.table(HairEyeColor, c(1,2))
HairEye

mosaic(HairEye, shade=TRUE)

# three-way mosaic displays
structable(HairEyeColor)

# mutual independence model
mosaic(HairEyeColor, shade=TRUE)
# joint independence of Hair*Eye with Sex
mosaic(HairEyeColor, expected = ~(Hair*Eye)+Sex)

# observed frequencies, mutual independence
mosaic3d(HairEyeColor)

# expected frequencies under mutual independence
mosaic3d(HairEyeColor, type="expected")

# expected frequencies under joint independence
mosaic3d(HairEyeColor, type="expected", expected =~(Hair*Eye)+Sex)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/tut04-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")


# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(HairEyeColor)
data(PreSex)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
if(!file.exists("fig")) dir.create("fig")


## -----------------------------------------------------------------------------
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
mosaic(art, gp = shading_max, 
            split_vertical = TRUE, 
            main="Arthritis: [Treatment] [Improved]")

## ---- art1--------------------------------------------------------------------
summary(art)

## -----------------------------------------------------------------------------
mosaic(art, gp = shading_Friendly, 
            split_vertical = TRUE, 
            main="Arthritis: gp = shading_Friendly")

## ----glass--------------------------------------------------------------------
data(Glass)
(glass.tab <- xtabs(Freq ~ father + son, data=Glass))

## ----glass-mosaic1------------------------------------------------------------
largs <- list(set_varnames=list(father="Father's Occupation", 
                                son="Son's Occupation"),
              abbreviate=10)
gargs <- list(interpolate=c(1,2,4,8))

mosaic(glass.tab, 
  shade=TRUE, 
  labeling_args=largs, 
  gp_args=gargs,
  main="Alphabetic order", 
  legend=FALSE, 
  rot_labels=c(20,90,0,70))

## ----glass-order--------------------------------------------------------------
# reorder by status
ord <- c(2, 1, 4, 3, 5) 
row.names(glass.tab)[ord]

## ----glass-mosaic2------------------------------------------------------------
mosaic(glass.tab[ord, ord], 
  shade=TRUE, 
  labeling_args=largs,  
  gp_args=gargs,
  main="Effect order", 
  legend=FALSE, 
  rot_labels=c(20,90,0,70))

## ----glass-ord----------------------------------------------------------------
Glass.ord <- Glass
Glass.ord$father <- ordered(Glass.ord$father, levels=levels(Glass$father)[ord])
Glass.ord$son    <- ordered(Glass.ord$son,    levels=levels(Glass$son)[ord])
str(Glass.ord)

## ----diag---------------------------------------------------------------------
rowfac <- gl(4, 4, 16)
colfac <- gl(4, 1, 16)
diag4by4 <- Diag(rowfac, colfac)
matrix(Diag(rowfac, colfac, binary = FALSE), 4, 4)

## ----symm---------------------------------------------------------------------
symm4by4 <- Symm(rowfac, colfac)
matrix(symm4by4, 4, 4)

## ----glass-models-------------------------------------------------------------
library(gnm)
glass.indep <- glm(Freq ~ father + son, 
                   data = Glass.ord, family=poisson)
glass.quasi <- glm(Freq ~ father + son + Diag(father, son),  
                   data = Glass.ord, family=poisson)
glass.symm  <- glm(Freq ~ Symm(father, son),  
                   data = Glass.ord, family=poisson)
glass.qsymm <- glm(Freq ~ father + son + Symm(father, son),  
                   data = Glass.ord, family=poisson)

## ----glass-quasi--------------------------------------------------------------
mosaic(glass.quasi, 
  residuals_type="rstandard", 
  shade=TRUE, 
  labeling_args=largs,  
  gp_args=gargs,
  main="Quasi-Independence",
  legend=FALSE, 
  rot_labels=c(20,90,0,70)
  )

## ----glass-anova--------------------------------------------------------------
# model comparisons: for *nested* models
anova(glass.indep, glass.quasi, glass.qsymm, test="Chisq")

## ----glass-lrstats------------------------------------------------------------
models <- glmlist(glass.indep, glass.quasi, glass.symm, glass.qsymm)
LRstats(models)

## ----glass-qsymm--------------------------------------------------------------
mosaic(glass.qsymm, 
  residuals_type="rstandard", 
  shade=TRUE, 
  labeling_args=largs,  
  gp_args=gargs,
  main = paste("Quasi-Symmetry", modFit(glass.qsymm)),
  legend=FALSE, 
  rot_labels=c(20,90,0,70)
  )

## ----housetasks---------------------------------------------------------------
data("HouseTasks", package = "vcdExtra")
HouseTasks

## ----housetasks-mos1----------------------------------------------------------
require(vcd)
mosaic(HouseTasks, shade = TRUE,
       labeling = labeling_border(rot_labels = c(45,0, 0, 0), 
                                  offset_label =c(.5,5,0, 0),
                                  varnames = c(FALSE, TRUE),
                                  just_labels=c("center","right"),
                                  tl_varnames = FALSE),
       legend = FALSE)


## ----housetasks-ca------------------------------------------------------------
require(ca)
HT.ca <- ca(HouseTasks)
summary(HT.ca, rows=FALSE, columns=FALSE)

## ----housetasks-ca-plot-------------------------------------------------------
plot(HT.ca, lines = TRUE)

## ----housetasks-seriation-----------------------------------------------------
require(seriation)
order <- seriate(HouseTasks, method = "CA")
# the permuted row and column labels
rownames(HouseTasks)[order[[1]]]
colnames(HouseTasks)[order[[2]]]

## ----housetasks-mos2----------------------------------------------------------
# do the permutation
HT_perm <- permute(HouseTasks, order, margin=1)

mosaic(HT_perm, shade = TRUE,
       labeling = labeling_border(rot_labels = c(45,0, 0, 0), 
                                  offset_label =c(.5,5,0, 0),
                                  varnames = c(FALSE, TRUE),
                                  just_labels=c("center","right"),
                                  tl_varnames = FALSE),
       legend = FALSE)

# fixed buglet when deviance() returns a null
# fixed bug: residual df calculated incorrectly
#  but this now depends on objects having a df.residual component
#  TRUE for lm, glm, polr, negbin objects

# made generic, adding a glmlist method

Summarise <- function(object, ...) {
	UseMethod("Summarise")
}

Summarise.glmlist <- function(object, ..., saturated = NULL, sortby=NULL)
{
    ns <- sapply(object, function(x) length(x$residuals))
    if (any(ns != ns[1L])) 
        stop("models were not all fitted to the same size of dataset")
    nmodels <- length(object)
    if (nmodels == 1) 
        return(Summarise.default(object[[1L]], saturated=saturated))
    
    rval <- lapply(object, Summarise.default, saturated=saturated)
    rval <- do.call(rbind, rval)    
		if (!is.null(sortby)) {
			rval <- rval[order(rval[,sortby], decreasing=TRUE),]
			}
		rval
}

# could just do Summarise.loglmlist <- Summarise.glmlist
Summarise.loglmlist <- function(object, ..., saturated = NULL, sortby=NULL)
{
	ns <- sapply(object, function(x) length(x$residuals))
	if (any(ns != ns[1L])) 
		stop("models were not all fitted to the same size of dataset")
	nmodels <- length(object)
	if (nmodels == 1) 
		return(Summarise.default(object[[1L]], saturated=saturated))
	
	rval <- lapply(object, Summarise.default, saturated=saturated)
	rval <- do.call(rbind, rval)    
	if (!is.null(sortby)) {
		rval <- rval[order(rval[,sortby], decreasing=TRUE),]
	}
	rval
}

Summarise.default <- function(object, ..., saturated = NULL, sortby=NULL)
{
  ## interface methods for logLik() and nobs()
  ## - use S4 methods if loaded
  ## - use residuals() if nobs() is not available
  logLik0 <- if("stats4" %in% loadedNamespaces()) stats4::logLik else logLik
  nobs0   <- function(x, ...) {
    nobs1 <- if("stats4" %in% loadedNamespaces()) stats4::nobs else nobs
    nobs2 <- function(x, ...) NROW(residuals(x, ...))
    rval <- try(nobs1(x, ...), silent = TRUE)
    if(inherits(rval, "try-error") | is.null(rval)) rval <- nobs2(x, ...)
    return(rval)
  }
  dof <- function(x) {
  	if (inherits(x, "loglm")) {
  		rval <- x$df 
  		} else {
  		rval <- try(x$df.residual, silent=TRUE)
  		}
  	if (inherits(rval, "try-error") || is.null(rval)) stop(paste("Can't determine residual df for a", class(x), "object"))
  	rval
  	}

  ## collect all objects
  objects <- list(object, ...)
  nmodels <- length(objects)
  
  ## check sample sizes
  ns <- sapply(objects, nobs0)
  if(any(ns != ns[1L])) stop("models were not all fitted to the same size of dataset")

  ## extract log-likelihood and df (number of parameters)
  ll <- lapply(objects, logLik0)
  par <- as.numeric(sapply(ll, function(x) attr(x, "df")))
	df <- as.numeric(sapply(objects, function(x) dof(x)))
  ll <- sapply(ll, as.numeric)
  
  ## compute saturated reference value (use 0 if deviance is not available)
  if(is.null(saturated)) {
    dev <- try(sapply(objects, deviance), silent = TRUE)
    if(inherits(dev, "try-error") || any(sapply(dev, is.null))) {
      saturated <- 0
    } else {
      saturated <- ll + dev/2
    }
  }

  ## setup ANOVA-style matrix
  rval <- matrix(rep(NA, 5 * nmodels), ncol = 5)
  colnames(rval) <- c("AIC", "BIC", "LR Chisq", "Df", "Pr(>Chisq)")
  rownames(rval) <- as.character(sapply(match.call(), deparse)[-1L])[1:nmodels]
  rval[,1] <- -2 * ll + 2 * par
  rval[,2] <- -2 * ll + log(ns) * par
  rval[,3] <- -2 * (ll - saturated)
  rval[,4] <- df
  rval[,5] <- pchisq(rval[,3], df, lower.tail = FALSE)

	if (!is.null(sortby)) {
		rval <- rval[order(rval[,sortby], decreasing=TRUE),]
	}

  ## return
  structure(as.data.frame(rval), heading = "Likelihood summary table:", class = c("anova", "data.frame"))
}

# Calculate Goodman-Kruskal Gamma
# Original from: Laura Thompson,
# https://home.comcast.net/~lthompson221/Splusdiscrete2.pdf



#' Calculate Goodman-Kruskal Gamma for ordered tables
#'
#' The Goodman-Kruskal \eqn{\gamma}{gamma} statistic is a measure of
#' association for ordinal factors in a two-way table proposed by Goodman and
#' Kruskal (1954).
#'
#'
#' @aliases GKgamma print.GKgamma
#' @param x A two-way frequency table, in matrix or table form.  The rows and columns are considered to be ordinal factors
#' @param level Confidence level for a significance test of \eqn{\gamma \ne =}{gamma !=0}
#' @return Returns an object of class `"GKgamma"` with 6 components, as follows
#' \describe{
#'   \item{gamma}{The gamma statistic}
#'   \item{C}{Total number of concordant pairs in the table}
#'   \item{D}{Total number of discordant pairs in the table}
#'   \item{sigma}{Standard error of gamma }
#'   \item{CIlevel}{Confidence level}
#'   \item{CI}{Confidence interval}
#'   }
#'
#' @author Michael Friendly; original version by Laura Thompson
#' @seealso \code{\link[vcd]{assocstats}}, \link[vcd]{Kappa}
#' @family association tests
#' @references Agresti, A. *Categorical Data Analysis*. John Wiley & Sons,
#' 2002, pp. 57--59.
#'
#' Goodman, L. A., & Kruskal, W. H. (1954). Measures of association for cross
#' classifications. *Journal of the American Statistical Association*, 49,
#' 732-764.
#'
#' Goodman, L. A., & Kruskal, W. H. (1963). Measures of association for cross
#' classifications III: Approximate sampling theory. *Journal of the
#' American Statistical Association*, 58, 310-364.
#' @keywords htest category
#' @examples
#'
#' data(JobSat)
#' GKgamma(JobSat)
#'
#' @export
GKgamma<-function(x, level=0.95)
{
    # x is a matrix of counts.  You can use output of crosstabs or xtabs in R.
    # Confidence interval calculation and output from Greg Rodd

    # Check for using S-PLUS and output is from crosstabs (needs >= S-PLUS 6.0)
    if(is.null(version$language) && inherits(x, "crosstabs")) { oldClass(x)<-NULL; attr(x, "marginals")<-NULL}

	## TODO: add tests for matrix or table

    n <- nrow(x)
    m <- ncol(x)
    pi.c<-pi.d<-matrix(0, nrow=n, ncol=m)

    row.x<-row(x)
    col.x<-col(x)

    for(i in 1:(n)){
        for(j in 1:(m)){
            pi.c[i, j]<-sum(x[row.x<i & col.x<j]) + sum(x[row.x>i & col.x>j])
            pi.d[i, j]<-sum(x[row.x<i & col.x>j]) + sum(x[row.x>i & col.x<j])
        }
    }

    C <- sum(pi.c*x)/2
    D <- sum(pi.d*x)/2

    psi<-2*(D*pi.c-C*pi.d)/(C+D)^2
    sigma2<-sum(x*psi^2)-sum(x*psi)^2

    gamma <- (C - D)/(C + D)
    pr2 <- 1 - (1 - level)/2
    CI <- qnorm(pr2) * sqrt(sigma2) * c(-1, 1) + gamma

    result <- list(gamma = gamma, C = C, D = D, sigma = sqrt(sigma2),
        CIlevel = level,
        CI = c(max(CI[1], -1), min(CI[2], 1))
        )
    class(result) <- "GKgamma"
    result
}

#' @rdname GKgamma
#' @param digits number to digits to use in the print method
#' @param ... other arguments (unused), for conformity with the print generic
#' @export
print.GKgamma <-
function (x, digits = 3, ...)
{
	cat("gamma        :", round(x$gamma, digits = digits),
			"\n")
	cat("std. error   :", round(x$sigma, digits = digits),
			"\n")
	cat("CI           :", round(x$CI, digits = digits),
			"\n")
	invisible(x)
}

#' Labeling Function for Mosaic Displays Using Points
#'
#' This labeling function for use with \code{\link[vcd]{strucplot}} displays,
#' such as \code{\link[vcd]{mosaic}}, draws
#' random points within each cell of a mosaic plot, where the number of points
#' represents observed or expected frequencies. This creates a "dot-density"
#' visualization that provides a direct visual representation of cell frequencies.
#'
#' @param labels Logical vector or scalar indicating whether labels should be
#'        drawn for the table dimensions via \code{\link[vcd]{labeling_border}}.
#'        Defaults to \code{TRUE}.
#' @param varnames Logical vector or scalar indicating whether variable names
#'        should be drawn. Defaults to \code{labels}.
#' @param value_type Character string specifying whether to display
#'        \code{"observed"} or \code{"expected"} frequencies as points.
#' @param scale Numeric scaling factor. The number of points drawn equals
#'        \code{round(frequency / scale)}. Use larger values for tables with
#'        large counts.
#' @param pch Point character (plotting symbol). Default is 19 (filled circle).
#' @param size Point size as a \code{\link[grid]{unit}} object.
#'        Default is \code{unit(0.5, "char")}.
#' @param gp_points A \code{\link[grid]{gpar}} object controlling point appearance
#'        (color, alpha, etc.), for example: \code{gp_points = gpar(col = "red")}.
#' @param margin Margin inside cells as a \code{\link[grid]{unit}} object.
#'        Points are drawn within this inset area. Default is \code{unit(0.05, "npc")}.
#' @param seed Optional integer seed for reproducible point placement across several similar plots.
#'        If \code{NULL} (default), no seed is set.
#' @param jitter Numeric jitter amount (0-1) for point placement.
#'        Default is 1 (full random). Values < 1 create more regular patterns.
#' @param clip Logical indicating whether to clip points at cell boundaries.
#'        Default is \code{FALSE}.
#' @param ... Additional arguments passed to \code{\link[vcd]{labeling_border}}
#'        for axis labels.
#'
#' @details
#' This function follows the "grapcon_generator" pattern used by vcd labeling
#' functions. It returns a function that can be passed to the \code{labeling}
#' argument of \code{\link[vcd]{strucplot}}, \code{\link[vcd]{mosaic}}, or
#' related functions as the argument \code{labeling = labeling_poionts(...)}.
#'
#' The visualization is inspired by the conceptual model described in
#' Friendly (1995), where cell frequencies are represented as physical
#' counts of objects. Under independence, point density would be uniform
#' across cells (adjusted for marginals), so departures from independence
#' become visible as density variations.
#'
#' This approach is related to sieve diagrams (\code{\link[vcd]{sieve}}),
#' which also use density to represent association.
#'
#' @return A function of class \code{"grapcon_generator"} suitable for use
#'   as the \code{labeling} argument in \code{\link[vcd]{strucplot}} and
#'   related functions like \code{\link[vcd]{mosaic}}.
#'
#' @references
#' Friendly, M. (1995). Conceptual and Visual Models for Categorical Data.
#' \emph{The American Statistician}, \strong{49}, 153-160.
#' \doi{10.1080/00031305.1995.10476131}
#'
#' Friendly, M. (1997). Conceptual Models for Visualizing Contingency Table Data.
#' In M. Greenacre & J. Blasius (Eds.), \emph{Visualization of Categorical Data} (pp. 17–35).
#' Academic Press. \url{https://www.datavis.ca/papers/koln/kolnpapr.pdf}
#'
#' Meyer, D., Zeileis, A., and Hornik, K. (2006).
#' The Strucplot Framework: Visualizing Multi-way Contingency Tables with vcd.
#' \emph{Journal of Statistical Software}, \strong{17}(3), 1-48.
#' \doi{10.18637/jss.v017.i03}
#'
#' @seealso
#' \code{\link[vcd]{labeling_cells}}, \code{\link[vcd]{labeling_border}},
#' \code{\link[vcd]{mosaic}}, \code{\link[vcd]{sieve}}
#'
#' @examples
#' library(vcd)
#'
#' # 2-way table of Hair and Eye color
#' HairEye <- margin.table(HairEyeColor, 2:1)
#'
#' # Basic usage - observed frequencies as points
#' mosaic(HairEye,
#'        labeling = labeling_points(scale = 1))
#'
#' # Show expected frequencies instead of observed
#' mosaic(HairEye,
#'        labeling = labeling_points(
#'                      value_type = "expected",
#'                      scale = 2,
#'                      seed = 42)
#'       )
#'
#' # Combine with residual shading
#' mosaic(HairEye,
#'        shade = TRUE, legend = FALSE,
#'        labeling = labeling_points(scale = 2, seed = 42))
#'
#' # Make tiles show expected frequencies, show points for observed frequencies
#' # Reproduces: Fig 6 in Friendly (1995)
#' mosaic(HairEye,
#'        type = "expected",
#'        shade = TRUE, legend = FALSE,
#'        labeling = labeling_points(scale = 2, seed = 42))
#'
#' @importFrom grid seekViewport pushViewport popViewport upViewport viewport
#' @importFrom grid grid.points unit gpar
#' @importFrom stats runif
#' @importFrom vcd labeling_border
#' @export
labeling_points <- function(
    labels = TRUE,
    varnames = labels,
    value_type = c("observed", "expected"),
    scale = 1,
    pch = 19,
    size = unit(0.5, "char"),
    gp_points = gpar(col = "black", alpha = 0.7),
    margin = unit(0.05, "npc"),
    seed = NULL,
    jitter = 1,
    clip = FALSE,
    ...
) {
  value_type <- match.arg(value_type)

  # Ensure margin is a unit

if (!inherits(margin, "unit")) {
    margin <- unit(margin, "npc")
  }

  # Ensure size is a unit
  if (!inherits(size, "unit")) {
    size <- unit(size, "char")
  }

  # Return the labeling function
  function(d, split_vertical, condvars, prefix = "") {

    # Draw border labels first (variable names and factor levels)
    labeling_border(labels = labels, varnames = varnames, ...)(
      d, split_vertical, condvars, prefix
    )

    # Get dimnames
    if (is.table(d) || inherits(d, "structable")) {
      d <- dimnames(d)
    }
    dn <- names(d)
    ld <- length(d)

    # Get the frequency values from the parent frame
    # (strucplot stores observed in 'x' and expected in 'expected')
    lookup <- if (value_type == "observed") "x" else "expected"
    if (!exists(lookup, envir = parent.frame())) {
      stop(paste("Could not find", sQuote(value_type), "values in calling environment."))
    }
    values <- get(lookup, envir = parent.frame())

    # Set seed for reproducibility if provided
    if (!is.null(seed)) set.seed(seed)

    # Recursive function to traverse all cells
    # (adapted from labeling_cells in vcd)
    draw_points <- function(vind = 1, labs = c()) {
      n <- d[[vind]]

      for (labind in seq_along(n)) {
        lab <- c(labs, n[labind])
        names(lab) <- names(d)[1:vind]

        # Build viewport name for this cell
        mlab <- paste(prefix, "cell:",
                      paste(dn[1:vind], lab, sep = "=", collapse = ","),
                      sep = "")

        if (vind < ld) {
          # Not at leaf level yet - recurse deeper
          draw_points(vind + 1, lab)
        } else {
          # At leaf level - draw points in this cell
          seekViewport(mlab)

          # Push a viewport with margins and optional clipping
          pushViewport(viewport(
            width = unit(1, "npc") - 2 * margin,
            height = unit(1, "npc") - 2 * margin,
            clip = if (clip) "on" else "off"
          ))

          # Get the count for this cell
          count <- do.call("[", c(list(values), as.list(lab)))

          # Calculate number of points
          n_points <- round(count / scale)

          if (n_points > 0) {
            # Generate random positions
            if (jitter >= 1) {
              # Full random placement
              x_pos <- runif(n_points)
              y_pos <- runif(n_points)
            } else {
              # Jittered grid for more even distribution
              n_side <- ceiling(sqrt(n_points))
              grid_x <- rep(seq(0.5/n_side, 1 - 0.5/n_side, length.out = n_side), n_side)
              grid_y <- rep(seq(0.5/n_side, 1 - 0.5/n_side, length.out = n_side), each = n_side)
              # Take only n_points from the grid
              idx <- seq_len(min(n_points, length(grid_x)))
              x_pos <- grid_x[idx] + runif(length(idx), -jitter/(2*n_side), jitter/(2*n_side))
              y_pos <- grid_y[idx] + runif(length(idx), -jitter/(2*n_side), jitter/(2*n_side))
              # Clamp to [0, 1]
              x_pos <- pmax(0, pmin(1, x_pos))
              y_pos <- pmax(0, pmin(1, y_pos))
            }

            # Draw the points
            grid.points(
              x = x_pos,
              y = y_pos,
              pch = pch,
              size = size,
              gp = gp_points,
              default.units = "npc"
            )
          }

          popViewport()
        }
      }
    }

    # Start the recursive traversal
    draw_points()

    # Return to base viewport
    seekViewport(paste(prefix, "base", sep = ""))
    upViewport(1)
  }
}

class(labeling_points) <- "grapcon_generator"

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/tut05-",
  fig.align = "center",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# Old Sweave options
# \SweaveOpts{engine=R,eps=TRUE,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{engine=R,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{prefix.string=fig/vcd-tut,eps=FALSE}
# \SweaveOpts{keep.source=TRUE}


# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
if(!file.exists("fig")) dir.create("fig")


## ---- spine1------------------------------------------------------------------
(spine(Improved ~ Age, data = Arthritis, breaks = 3))
(spine(Improved ~ Age, data = Arthritis, breaks = "Scott"))

## -----------------------------------------------------------------------------
cdplot(Improved ~ Age, data = Arthritis)

## -----------------------------------------------------------------------------
cdplot(Improved ~ Age, data = Arthritis)
with(Arthritis, rug(jitter(Age), col="white", quiet=TRUE))

## ---- donner1-----------------------------------------------------------------
data(Donner, package="vcdExtra")
str(Donner)

## ---- donner2a, fig=FALSE, eval=FALSE-----------------------------------------
#  # separate linear fits on age for M/F
#  ggplot(Donner, aes(age, survived, color = sex)) +
#    geom_point(position = position_jitter(height = 0.02, width = 0)) +
#    stat_smooth(method = "glm",
#                method.args = list(family = binomial),
#                formula = y ~ x,
#                alpha = 0.2, size=2, aes(fill = sex))

## ---- donner2b, fig=FALSE, eval=FALSE-----------------------------------------
#  # separate quadratics
#  ggplot(Donner, aes(age, survived, color = sex)) +
#    geom_point(position = position_jitter(height = 0.02, width = 0)) +
#    stat_smooth(method = "glm",
#                method.args = list(family = binomial),
#                formula = y ~ poly(x,2),
#                alpha = 0.2, size=2, aes(fill = sex))

## -----------------------------------------------------------------------------
# separate linear fits on age for M/F
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ x,
              alpha = 0.2, size=2, aes(fill = sex))

# separate quadratics
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ poly(x,2),
              alpha = 0.2, size=2, aes(fill = sex))

# Cochran-Mantel-Haenszel tests for ordinal factors in contingency tables

# The code below follows Stokes, Davis & Koch, (2000).
#   "Categorical Data Analysis using the SAS System", 2nd Ed.,
#   pp 74--75, 92--101, 124--129.

# Ref: Landis, R. J., Heyman, E. R., and Koch, G. G. (1978),
#		Average Partial Association in Three-way Contingency Tables:
#		A Review and Discussion of Alternative Tests,
#		International Statistical Review, 46, 237-254.

# See: https://onlinecourses.science.psu.edu/stat504/book/export/html/90
# http://support.sas.com/documentation/cdl/en/statug/63033/HTML/default/viewer.htm#statug_freq_a0000000648.htm

# DONE: this should be the main function, handling 2-way & higher-way tables
#  With strata, use apply() or recursion over strata
# DONE: With strata, calculate overall CMH tests controlling for strata
# FIXED: rmeans and cmeans tests were labeled incorrectly

#' Generalized Cochran-Mantel-Haenszel Tests
#'
#' Provides generalized Cochran-Mantel-Haenszel tests of association of two
#' possibly ordered factors, optionally stratified other factor(s).  With
#' strata, `CMHtest` calculates these tests for each level of the
#' stratifying variables and also provides overall tests controlling for the
#' strata. See **Details** for descriptions of what these entail and enhancements to this function.
#'
#' @details
#' For ordinal factors, more powerful tests than the test for general
#' association (independence) are obtained by assigning scores to the row and
#' column categories.
#'
#' The standard \eqn{\chi^2} tests for association in a two-way table treat
#' both table factors as nominal (unordered) categories. When one or both
#' factors of a two-way table are quantitative or ordinal, more powerful tests
#' of association may be obtained by taking ordinality into account using row
#' and or column scores to test for linear trends or differences in row or
#' column means.
#'
#' The CMH analysis for a two-way table produces **generalized**
#' Cochran-Mantel-Haenszel statistics (Landis etal., 1978):
#'
#' - The CMH **correlation** statistic (`"cor"`), treating both factors as
#'   ordered. For a given statum, with equally spaced row and column scores,
#'   this CMH statistic reduces to \eqn{(n-1) r^2}, where \eqn{r} is the
#'   Pearson correlation between X and Y. With `"midrank"` scores, this CMH
#'   statistic is analogous to \eqn{(n-1) r_S^2}, using the Spearman rank
#'   correlation.
#' - The **ANOVA** (row mean scores and column mean scores) statistics,
#'   treat the columns and rows respectively as ordinal, and are sensitive to
#'   mean shifts over columns or rows. These are transforms of the \eqn{F}
#'   statistics from one-way ANOVAs with equally spaced scores and to
#'   Kruskal-Wallis tests with `"midrank"` scores.
#' - The CMH **general** association statistic treat both factors as
#'   unordered, and give a test closely related to the Pearson \eqn{\chi^2}
#'   test. When there is more than one stratum, the overall general CMH
#'   statistic gives a stratum-adjusted Pearson \eqn{\chi^2}, equivalent to
#'   what is calculated by \code{\link[stats]{mantelhaen.test}}.
#'
#' ## Strata
#'
#' For a 3+ way table, one table of CMH tests is produced for each combination
#' of the factors identified as `strata`. If `overall=TRUE`, an
#' additional table is calculated for the same two primary variables,
#' controlling for (pooling over) the `strata` variables.
#'
#' These overall tests implicitly assume no interactions between the primary
#' variables and the strata and they will have low power in the presence of
#' interactions.
#'
#' Note that strata combinations with insufficient data (less than 2
#' observations) are automatically omitted from the analysis.
#'
#' ## 2x2 layout
#'
#' `print.CMHtest(layout = "2x2")` is an attempt to provide an alternative view of these
#' four statistics that may be more useful/understandable than the traditional flat 4-row
#' table: it reorganizes them into a 2x2 table crossing how the row and column variables
#' are each treated (general/nominal vs. ordered/scored).
#'
#' The corner cell of this display (`general - rmeans - cmeans + cor`, a "diff of diffs")
#' is algebraically consistent but its distribution as chi-square(df) is
#' **experimental/unverified** -- these are four different quadratic-form statistics, not
#' a nested sequence of LR tests, so the naive inclusion-exclusion combination is not
#' guaranteed to be non-negative in general (see `dev/CMH-2x2.md` for a worked
#' counter-example and a loglinear `LRstats()`-based alternative).
#'
#' @aliases CMHtest CMHtest.formula CMHtest.default Cochran Mantel Haenszel test print.CMHtest
#' @param x A 2+ way contingency table in array form, or a class `"table"`
#'          object with optional category labels specified in the dimnames(x) attribute.
#' @param formula a formula specifying the variables used to create a
#'           contingency table from `data`.  This should be a one-sided formula when
#'           `data` is in array form, and a two-sided formula with a response
#'           `Freq` if `data` is a data frame with a cell frequency variable.
#'            For convenience, conditioning formulas can be specified indicating strata.
#' @param data either a data frame, or an object of class `"table"` or `"ftable"`.
#' @param subset an optional vector specifying a subset of observations to be used.
#' @param na.action a function which indicates what should happen when the data contain `NA`s.
#'           Ignored if `data` is a contingency table.
#' @param strata For a 3- or higher-way table, the names or numbers of the
#'           factors to be treated as strata.  By default, the first 2 factors are
#'           treated as the main table variables, and all others considered stratifying factors.
#' @param rscores Row scores.  Either a set of numbers (typically integers,
#'            `1:R`) or the string `"midrank"` for standardized midrank scores,
#' or `NULL` to exclude tests that depend on row scores.
#' @param cscores Column scores. Same as for row scores.
#' @param types Types of CMH tests to compute: Any one or more of `c("cor", "cmeans", "rmeans", "general")`, or
#'           `"ALL"` for all of these.
#' @param overall logical. Whether to calculate overall tests, controlling for the stratifying factors.
#' @param details logical.  Whether to include computational details in the result
#' @param \dots Other arguments passed to default method.
#' @param digits Digits to print.
#' @param layout For `print.CMHtest()`, one of `"table"` (default) for the
#'           traditional flat 4-row printout, or `"2x2"` to reorganize the four CMH
#'           statistics into a 2x2 table crossing how the row and column variables are
#'           each treated (general/nominal vs. ordered/scored). `layout = "2x2"`
#'           requires all four of `cor`, `rmeans`, `cmeans`, `general` to be present in
#'           `x`; if `types` or `rscores`/`cscores = NULL` excluded any of them, this
#'           falls back to `layout = "table"` with a warning. See **Details**.
#' @param stars For `print.CMHtest()` with `layout = "2x2"`, logical: annotate
#'           each cell with significance stars (`'***'`/`'**'`/`'*'` for p < .001/.01/.05)
#'           based on its own (Chisq, Df). Default `FALSE`, matching `layout = "table"`'s
#'           plain display.
#' @param scale For `print.CMHtest()` with `layout = "2x2"`, logical: show
#'           `Chisq/Df` instead of `Chisq (Df)` in each cell -- normalizes for the very
#'           different degrees of freedom across cells. Default `FALSE`.
#'
#' @return An object of class `"CMHtest"` , a list with the following 4 components:
#'
#' \item{table}{A matrix containing the test statistics, with columns `Chisq`, `Df` and `Prob` }
#' \item{names}{The names of the table row and column variables}
#' \item{rscore}{Row scores}
#' \item{cscore}{Column scores}
#'
#' If `details==TRUE`, additional components are included.
#'
#' If there are strata, the result is a list of `"CMHtest"` objects. If
#' `overall=TRUE` another component, labeled `ALL` is appended to the
#' list.
#'
#' @author Michael Friendly
#'
#' @seealso \code{\link[coin]{cmh_test}} provides the CMH test of general
#' association; \code{\link[coin]{lbl_test}} provides the CMH correlation test
#' of linear by linear association.
#'
#' \code{\link[stats]{mantelhaen.test}} provides the overall general
#' Cochran-Mantel-Haenszel chi-squared test of the null that two nominal
#' variables are conditionally independent in each stratum, assuming that there
#' is no three-way interaction
#' @family association tests
#'
#' @references
#' Stokes, M. E. & Davis, C. S.  & Koch, G., (2000).
#' *Categorical Data Analysis using the SAS System*, 2nd Ed., Cary, NC:
#' SAS Institute, pp 74-75, 92-101, 124-129.  Details of the computation are
#' given at: <http://support.sas.com/documentation/cdl/en/statug/63033/HTML/default/viewer.htm#statug_freq_a0000000648.htm>
#'
#' Cochran, W. G. (1954), Some Methods for Strengthening the Common
#' \eqn{\chi^2} Tests, *Biometrics*, 10, 417-451.
#'
#' Landis, R. J., Heyman, E. R., and Koch, G. G. (1978).  Average Partial
#' Association in Three-way Contingency Tables: A Review and Discussion of
#' Alternative Tests, *International Statistical Review*, **46**,
#' 237-254.
#'
#' Mantel, N. (1963), Chi-square Tests with One Degree of Freedom: Extensions
#' of the Mantel-Haenszel Procedure," *Journal of the American Statistical
#' Association*, 58, 690-700.
#' @keywords htest
#' @export
#' @importFrom stats setNames
#' @examples
#'
#' data(JobSat, package="vcdExtra")
#' CMHtest(JobSat)
#' CMHtest(JobSat, rscores="midrank", cscores="midrank")
#'
#' # formula interface
#' CMHtest(~ ., data=JobSat)
#'
#' # A 3-way table (both factors ordinal)
#' data(MSPatients, package="vcd")
#' CMHtest(MSPatients)
#'
#'
#' # also calculate overall tests, controlling for Patient
#' CMHtest(MSPatients, overall = TRUE)
#' # compare with mantelhaen.test
#' mantelhaen.test(MSPatients)
#'
#' # formula interface
#' CMHtest(~ ., data = MSPatients, overall = TRUE)
#'
#' # using a frequency data.frame
#' CMHtest(xtabs(Freq~ses + mental, data = Mental))
#' # or, more simply
#' CMHtest(Freq~ses + mental, data = Mental)
#'
#' # conditioning formulae
#' CMHtest(Freq~right + left | gender, data = VisualAcuity)
#'
#' CMHtest(Freq ~ attitude + memory | education + age, data = Punishment)
#'
#' # 2x2 layout, reorganizing the four CMH statistics by how the row/column
#' # variables are treated (general/nominal vs. ordered/scored)
#' cmh_mental <- CMHtest(Freq ~ ses + mental, data = Mental)
#' print(cmh_mental, layout = "2x2")
#' print(cmh_mental, layout = "2x2", stars = TRUE)
#' print(cmh_mental, layout = "2x2", scale = TRUE)
#'
#'
#' # Stokes etal, Table 5.1, p 92: two unordered factors
#' parties <- matrix(
#' 	c(221, 160, 360, 140,
#' 	  200, 291, 160, 311,
#' 	  208, 106, 316, 97),
#' 	nrow=3, ncol=4,
#' 	byrow=TRUE)
#' dimnames(parties) <- list(party=c("Dem", "Indep", "Rep"),
#'              neighborhood=c("Bayside", "Highland", "Longview", "Sheffield"))
#' CMHtest(parties, rscores=NULL, cscores=NULL)
#'
#' # compare with Pearson chisquare
#' chisq.test(parties)
#'
CMHtest <- function(x, ...) {
  UseMethod("CMHtest")
}

#' @rdname CMHtest
#' @export
CMHtest.formula <-
  function(formula, data = NULL, subset = NULL, na.action = NULL, ...) {
    m <- match.call(expand.dots = FALSE)
    edata <- eval(m$data, parent.frame())

    fstr <- strsplit(paste(deparse(formula), collapse = ""), "~")
    vars <- strsplit(strsplit(gsub(" ", "", fstr[[1]][2]), "\\|")[[1]], "\\+")
    varnames <- vars[[1]]

    condnames <- if (length(vars) > 1) vars[[2]] else NULL

    dep <- gsub(" ", "", fstr[[1]][1])
    if (!dep %in% c("", "Freq")) {
      if (all(varnames == ".")) {
        varnames <- if (is.data.frame(data)) {
          colnames(data)
        } else {
          names(dimnames(as.table(data)))
        }
        varnames <- varnames[-which(varnames %in% dep)]
      }

      varnames <- c(varnames, dep)
    }

    if (
      inherits(edata, "ftable") ||
        inherits(edata, "table") ||
        length(dim(edata)) > 2
    ) {
      condind <- NULL
      dat <- as.table(data)
      if (all(varnames != ".")) {
        ind <- match(varnames, names(dimnames(dat)))
        if (any(is.na(ind))) {
          stop(paste(
            "Can't find",
            paste(varnames[is.na(ind)], collapse = " / "),
            "in",
            deparse(substitute(data))
          ))
        }

        if (!is.null(condnames)) {
          condind <- match(condnames, names(dimnames(dat)))
          if (any(is.na(condind))) {
            stop(paste(
              "Can't find",
              paste(condnames[is.na(condind)], collapse = " / "),
              "in",
              deparse(substitute(data))
            ))
          }
          ind <- c(condind, ind)
        }
        dat <- margin.table(dat, ind)
      }
      CMHtest.default(
        dat,
        strata = if (is.null(condind)) {
          NULL
        } else {
          match(condnames, names(dimnames(dat)))
        },
        ...
      )
    } else {
      m <- m[c(
        1,
        match(c("formula", "data", "subset", "na.action"), names(m), 0)
      )]
      m[[1]] <- as.name("xtabs")
      m$formula <-
        formula(paste(
          if ("Freq" %in% colnames(data)) "Freq",
          "~",
          paste(c(varnames, condnames), collapse = "+")
        ))
      tab <- eval(m, parent.frame())
      CMHtest.default(tab, ...)
    }
  }

#' @rdname CMHtest
#' @export
CMHtest.default <- function(
  x,
  strata = NULL,
  rscores = 1:R,
  cscores = 1:C,
  types = c("cor", "rmeans", "cmeans", "general"),
  overall = FALSE,
  details = overall,
  ...
) {
  snames <- function(x, strata) {
    sn <- dimnames(x)[strata]
    dn <- names(sn)
    apply(expand.grid(sn), 1, function(x) {
      paste(dn, x, sep = ":", collapse = "|")
    })
  }

  ## check dimensions
  L <- length(d <- dim(x))
  if (any(d < 2L)) {
    stop("All table dimensions must be 2 or greater")
  }
  if (L > 2L & is.null(strata)) {
    strata <- 3L:L
  }
  if (is.character(strata)) {
    strata <- which(names(dimnames(x)) == strata)
  }
  if (L - length(strata) != 2L) {
    stop("All but 2 dimensions must be specified as strata.")
  }

  ## rearrange table to put primary dimensions first
  x <- aperm(x, c(setdiff(1:L, strata), strata))

  d <- dim(x)
  R <- d[1]
  C <- d[2]

  # handle strata
  if (!is.null(strata)) {
    sn <- snames(x, strata)
    res <- c(apply(
      x,
      strata,
      CMHtest2,
      rscores = rscores,
      cscores = cscores,
      types = types,
      details = details,
      ...
    ))
    # Drop results for strata with insufficient data (NULL result)
    strata_gives_null <- sapply(res, is.null)
    res <- res[!strata_gives_null]
    sn <- sn[!strata_gives_null]
    # DONE: fix names if there are 2+ strata
    names(res) <- sn
    for (i in seq_along(res)) {
      res[[i]]$stratum <- sn[i]
    }
    # DONE: Calculate generalized CMH, controlling for strata
    if (overall) {
      if (!details) {
        warning("Overall CMH tests not calculated because details=FALSE")
      } else {
        resall <- CMHtest3(res, types = types)
        res$ALL <- resall
      }
    }
    return(res)
  } else {
    CMHtest2(
      x,
      rscores = rscores,
      cscores = cscores,
      types = types,
      details = details,
      ...
    )
  }
}

# handle two-way case, for a given stratum
#  DONE:  now allow rscores/cscores == 'midrank' for midrank scores
#  DONE:  allow rscores/cscores=NULL for unordered factors, where ordinal
#     scores don't make sense
#  DONE: modified to return all A matrices as a list
#  DONE: cmh() moved outside

CMHtest2 <- function(
  x,
  stratum = NULL,
  rscores = 1:R,
  cscores = 1:C,
  types = c("cor", "rmeans", "cmeans", "general"),
  details = FALSE,
  ...
) {
  # left kronecker product
  lkronecker <- function(x, y, make.dimnames = TRUE, ...) {
    kronecker(y, x, make.dimnames = make.dimnames, ...)
  }

  # midrank scores (modified ridits) based on row/column totals
  midrank <- function(n) {
    cs <- cumsum(n)
    (2 * cs - n + 1) / (2 * (cs[length(cs)] + 1))
  }

  L <- length(d <- dim(x))
  R <- d[1]
  C <- d[2]

  if (is.character(rscores) && rscores == "midrank") {
    rscores <- midrank(rowSums(x))
  }
  if (is.character(cscores) && cscores == "midrank") {
    cscores <- midrank(colSums(x))
  }

  nt <- sum(x)

  # If there are not at least 2 observations in this stratum,
  # the below computations won't work, therefore return early NULL here.
  if (nt <= 1) {
    return(NULL)
  }

  pr <- rowSums(x) / nt
  pc <- colSums(x) / nt

  m <- as.vector(nt * outer(pr, pc)) # expected values under independence
  n <- as.vector(x) # cell frequencies

  V1 <- (diag(pr) - pr %*% t(pr))
  V2 <- (diag(pc) - pc %*% t(pc))
  V <- (nt^2 / (nt - 1)) * lkronecker(V1, V2, make.dimnames = TRUE)

  if (length(types) == 1 && types == "ALL") {
    types <- c("general", "rmeans", "cmeans", "cor")
  }
  types <- match.arg(types, several.ok = TRUE)
  # handle is.null(rscores) etc here
  if (is.null(rscores)) {
    types <- setdiff(types, c("cmeans", "cor"))
  }
  if (is.null(cscores)) {
    types <- setdiff(types, c("rmeans", "cor"))
  }

  table <- NULL
  Amats <- list()
  for (type in types) {
    if ("cor" == type) {
      A <- lkronecker(t(rscores), t(cscores))
      df <- 1
      table <- rbind(table, cmh(n, m, A, V, df))
      Amats$cor <- A
    } else if ("rmeans" == type) {
      A <- lkronecker(cbind(diag(R - 1), rep(0, R - 1)), t(cscores))
      df <- R - 1
      table <- rbind(table, cmh(n, m, A, V, df))
      Amats$rmeans <- A
    } else if ("cmeans" == type) {
      A <- lkronecker(t(rscores), cbind(diag(C - 1), rep(0, C - 1)))
      df <- C - 1
      table <- rbind(table, cmh(n, m, A, V, df))
      Amats$cmeans <- A
    } else if ("general" == type) {
      A <- lkronecker(
        cbind(diag(R - 1), rep(0, R - 1)),
        cbind(diag(C - 1), rep(0, C - 1))
      )
      df <- (R - 1) * (C - 1)
      table <- rbind(table, cmh(n, m, A, V, df))
      Amats$general <- A
    }
  }

  colnames(table) <- c("Chisq", "Df", "Prob")
  rownames(table) <- types
  xnames <- names(dimnames(x))
  result <- list(
    table = table,
    names = xnames,
    rscores = rscores,
    cscores = cscores,
    stratum = stratum
  )
  if (details) {
    result <- c(result, list(A = Amats, V = V, n = n, m = m))
  }
  class(result) <- "CMHtest"
  result
}

# do overall test, from a computed CMHtest list
CMHtest3 <- function(object, types = c("cor", "rmeans", "cmeans", "general")) {
  nstrat <- length(object) # number of strata

  # extract components, each a list of nstrat terms
  n.list <- lapply(object, function(s) s$n)
  m.list <- lapply(object, function(s) s$m)
  V.list <- lapply(object, function(s) s$V)
  A.list <- lapply(object, function(s) s$A)
  nt <- sapply(lapply(object, function(s) s$n), sum)

  tab1 <- object[[1]]$table
  Df <- stats::setNames(
    tab1[, "Df"],
    rownames(tab1)
  )

  if (length(types) == 1 && types == "ALL") {
    types <- c("general", "rmeans", "cmeans", "cor")
  }
  types <- match.arg(types, several.ok = TRUE)

  table <- list()
  for (type in types) {
    AVA <- 0
    Anm <- 0
    for (k in 1:nstrat) {
      A <- A.list[[k]][[type]]
      V <- V.list[[k]]
      n <- n.list[[k]]
      m <- m.list[[k]]
      AVA <- AVA + A %*% V %*% t(A)
      Anm <- Anm + A %*% (n - m)
    }
    Q <- t(Anm) %*% MASS::ginv(AVA) %*% Anm
    df <- Df[type]
    pvalue <- pchisq(Q, df, lower.tail = FALSE)
    table <- rbind(table, c(Q, df, pvalue))
  }
  rownames(table) <- types
  colnames(table) <- c("Chisq", "Df", "Prob")
  xnames <- object[[1]]$names
  result = list(table = table, names = xnames, stratum = "ALL")
  class(result) <- "CMHtest"
  result
}


# basic CMH calculation
cmh <- function(n, m, A, V, df) {
  AVA <- A %*% V %*% t(A)
  Q <- t(n - m) %*% t(A) %*% MASS::ginv(AVA) %*% A %*% (n - m)
  pvalue <- pchisq(Q, df, lower.tail = FALSE)
  c(Q, df, pvalue)
}

# DONE: incorporate stratum name in the heading
# TODO: handle the printing of pvalues better
# TODO: determine score types (integer, midrank) for heading

#' @rdname CMHtest
#' @export
print.CMHtest <- function(x, digits = max(getOption("digits") - 2, 3),
                           layout = c("table", "2x2"), stars = FALSE,
                           scale = FALSE, ...) {
  layout <- match.arg(layout)
  stars <- isTRUE(stars)
  scale <- isTRUE(scale)

  heading <- "Cochran-Mantel-Haenszel Statistics"
  if (!is.null(x$names)) {
    heading <- paste(heading, "for", paste(x$names, collapse = " by "))
  }
  if (!is.null(x$stratum)) {
    heading <- paste(
      heading,
      ifelse(
        x$stratum == "ALL",
        "\n\tOverall tests, controlling for all strata",
        paste("\n\tin stratum", x$stratum)
      )
    )
  }

  tbl <- x$table
  types <- rownames(tbl)

  if (layout == "2x2") {
    required <- c("cor", "rmeans", "cmeans", "general")
    if (!all(required %in% types)) {
      warning("layout = '2x2' needs all of ", paste(required, collapse = ", "),
              "; falling back to layout = 'table'. Missing: ",
              paste(setdiff(required, types), collapse = ", "))
      layout <- "table"
    }
  }

  if (layout == "table") {
    labels <- list(
      cor = "Nonzero correlation",
      rmeans = "Row mean scores differ",
      cmeans = "Col mean scores differ",
      general = "General association"
    )
    labels <- unlist(labels[types])
    df <- data.frame(
      "AltHypothesis" = as.character(labels),
      tbl,
      stringsAsFactors = FALSE
    )
    cat(heading, "\n\n")
    print(df, digits = digits, ...)
    cat("\n")
    return(invisible(x))
  }

  # ---- layout == "2x2" -------------------------------------------------------

  # NB: for multi-stratum results, CMHtest3()'s `table` (built above) is
  # constructed via rbind() starting from an empty list(), which produces a
  # "list matrix" -- each cell a length-1 list, not a plain double -- even
  # though it prints identically to a normal numeric matrix. The original
  # "table" layout above never noticed, since it just wraps the values in a
  # data.frame; arithmetic on the raw values (needed here) breaks on that
  # without unlisting first. Worth fixing at the source (CMHtest3() itself,
  # e.g. `table <- matrix(nrow = 0, ncol = 3)` instead of `table <- list()`),
  # but defending against it here too so this works with existing results.
  Chisq <- setNames(as.numeric(unlist(tbl[, "Chisq"])), types)
  Df    <- setNames(as.numeric(unlist(tbl[, "Df"])), types)

  stat  <- function(type) unname(Chisq[type])
  dfree <- function(type) unname(Df[type])

  pval <- function(q, d) {
    if (is.na(q) || is.na(d) || q < 0 || d <= 0) return(NA_real_)
    pchisq(q, d, lower.tail = FALSE)
  }
  sig_stars <- function(p) {
    if (is.na(p)) ""
    else if (p < .001) "***"
    else if (p < .01)  "**"
    else if (p < .05)  "*"
    else ""
  }
  cell <- function(q, d, flag_negative = FALSE) {
    note <- if (flag_negative && !is.na(q) && q < 0) " [invalid: negative]" else ""
    star <- if (stars) sig_stars(pval(q, d)) else ""
    # `digits` means significant digits, as in print(df, digits = digits) for the
    # "table" layout -- NOT decimal places. format() matches that semantics; a
    # sprintf("%.*f", digits, ...) here would print `digits` decimal places instead,
    # which for these X^2 values in the tens/hundreds is far more precision than
    # the "table" layout shows for the same numbers.
    valstr <- format(if (scale) q / d else q, digits = digits, nsmall = 0)
    if (scale) {
      sprintf("%s%s%s", valstr, star, note)
    } else {
      sprintf("%s (%d)%s%s", valstr, d, star, note)
    }
  }

  # core 2x2
  g  <- stat("general"); g_df  <- dfree("general")
  rm <- stat("rmeans");  rm_df <- dfree("rmeans")
  cm <- stat("cmeans");  cm_df <- dfree("cmeans")
  cr <- stat("cor");     cr_df <- dfree("cor")

  # margins: within a fixed column/row, general-treatment minus ordered-treatment
  colgen_diff <- g - cm;  colgen_diff_df <- g_df - cm_df   # col: general;  row gen - row ord
  colord_diff <- rm - cr; colord_diff_df <- rm_df - cr_df  # col: ordered;  row gen - row ord
  rowgen_diff <- g - rm;  rowgen_diff_df <- g_df - rm_df   # row: general;  col gen - col ord
  rowor_diff  <- cm - cr; rowor_diff_df  <- cm_df - cr_df  # row: ordered;  col gen - col ord

  # corner: diff of diffs -- EXPERIMENTAL, see dev/CMH-2x2.md. Not guaranteed >= 0:
  # these are four different quadratic forms, not a nested LR sequence, so the naive
  # inclusion-exclusion combination can go negative (observed with MSPatients).
  corner    <- g - rm - cm + cr
  corner_df <- g_df - rm_df - cm_df + cr_df

  out <- matrix(
    c(
      cell(g, g_df),                       cell(rm, rm_df),                     cell(rowgen_diff, rowgen_diff_df),
      cell(cm, cm_df),                     cell(cr, cr_df),                     cell(rowor_diff, rowor_diff_df),
      cell(colgen_diff, colgen_diff_df),   cell(colord_diff, colord_diff_df),   cell(corner, corner_df, flag_negative = TRUE)
    ),
    nrow = 3, byrow = TRUE,
    dimnames = list(
      c("row: general", "row: ordered", "diff (gen-ord)"),
      c("col: general", "col: ordered", "diff (gen-ord)")
    )
  )

  cat(heading, "\n")
  cat("\tRow/column display: general (nominal) vs. ordered (scored)\n")
  cat(if (scale) "\tCell values: X^2/df\n\n" else "\tCell values: X^2 (df)\n\n")
  print(as.data.frame(out), right = TRUE)
  if (stars) {
    cat("\nSignif. codes: '***' p<.001  '**' p<.01  '*' p<.05\n")
  }
  cat("\n")

  invisible(x)
}

# CMHtest(..., overall = TRUE) (or any strata call) returns a plain, unclassed
# list of per-stratum "CMHtest" objects (plus "ALL" when overall = TRUE) -- each
# element already carries class "CMHtest", so printing all of them with the same
# layout/stars/scale is just an lapply() over print.CMHtest(); the list itself
# has no class to dispatch print(x, layout = "2x2") on directly. Not exported --
# internal helper, since it's a workaround for the unclassed-list quirk above,
# not a designed public entry point.
print_CMHtest_list <- function(x, ...) {
  invisible(lapply(x, print.CMHtest, ...))
}

# CMHtest ----

test_that("CMHtest automatically filters out strata with single observations", {
  data <- data.frame(
    A = c("a", "a", "b", "b", "b", "c"),
    Y = c(1, 2, 3, 4, 5, 6),
    B = c("s1", "s1", "s2", "s2", "s2", "s3"),
    C = c(10, 20, 30, 40, 50, 60)
  )
  result <- expect_silent(CMHtest(
    C ~ A | B,
    data = data,
    overall = TRUE
  ))
  expect_type(result, "list")
  # B:s3 has been removed automatically.
  expect_identical(names(result), c("B:s1", "B:s2", "ALL"))
})

test_that("CMHtest works with multiple and sparse strata variables", {
  data <- data.frame(
    A = c("a", "a", "b", "b", "b", "c", "c", "c"),
    Y = c(1, 2, 3, 4, 5, 6, 7, 8),
    B1 = c("s1", "s1", "s2", "s2", "s2", "s1", "s1", "s1"),
    B2 = c("t1", "t1", "t1", "t1", "t1", "t2", "t2", "t2"),
    C = c(10, 20, 30, 40, 50, 60, 70, 80)
  )
  result <- expect_silent(CMHtest(
    C ~ A | B1 + B2,
    data = data,
    overall = TRUE
  ))
  expect_type(result, "list")
  # B1:s2 & B2:t2 combination has been removed automatically.
  expect_identical(
    names(result),
    c("B1:s1|B2:t1", "B1:s2|B2:t1", "B1:s1|B2:t2", "ALL")
  )
})

# integration tests vs. SAS results ----

# The test cases have been published first in the CAMIS project
# (https://psiaims.github.io/CAMIS/)
# The CAMIS test code has been written by
# Christina Fillmore, Lyn Taylor, Yannick Vandendijck and Logan Johnson.
# The work is published at https://github.com/PSIAIMS/CAMIS/blob/main/R/cmh.qmd
# under the Apache License 2.0.
# The code has been adapted here in order to fit into the testthat framework.

data <- read.csv(testthat::test_path("data", "adcibc.csv"))
data2 <- data %>%
  dplyr::filter(
    TRTPN != "54",
    AGEGR1 != ">80"
  )

sas_results <- tibble::tribble(
  ~Scenario , ~Test                 , ~Chisq , ~Df , ~Prob  ,
  1L        , "Correlation"         , 0.2166 ,   1 , 0.6417 ,
  1L        , "Row Means"           , 0.2166 ,   1 , 0.6417 ,
  1L        , "General Association" , 0.2166 ,   1 , 0.6417 ,

  2L        , "Correlation"         , 0.0009 ,   1 , 0.9765 ,
  2L        , "Row Means"           , 2.4820 ,   2 , 0.2891 , # corrected Df
  2L        , "General Association" , 2.4820 ,   2 , 0.2891 , # corrected Df

  3L        , "Correlation"         , 0.0028 ,   1 , 0.9579 ,
  3L        , "Row Means"           , 2.3861 ,   2 , 0.3033 ,
  3L        , "General Association" , 2.3861 ,   2 , 0.3033 ,

  4L        , "Correlation"         , 1.7487 ,   1 , 0.1860 ,
  4L        , "Row Means"           , 1.7487 ,   1 , 0.1860 ,
  4L        , "General Association" , 8.0534 ,   4 , 0.0896 ,

  5L        , "Correlation"         , 0.0854 ,   1 , 0.7701 ,
  5L        , "Row Means"           , 2.4763 ,   2 , 0.2899 ,
  5L        , "General Association" , 7.0339 ,   8 , 0.5330 ,

  6L        , "Correlation"         , 1.6621 ,   1 , 0.1973 ,
  6L        , "Row Means"           , 2.2980 ,   4 , 0.6811 ,
  6L        , "General Association" , 5.7305 ,   8 , 0.6774
)

convert_to_tibble <- function(cmh_test) {
  table <- cmh_test$ALL$table
  result <- tibble::as_tibble(table) %>%
    dplyr::mutate(
      dplyr::across(dplyr::everything(), unlist),
      Test = rownames(table),
      Test = dplyr::case_when(
        Test == "cor" ~ "Correlation",
        Test == "rmeans" ~ "Row Means",
        Test == "cmeans" ~ "Column Means",
        Test == "general" ~ "General Association"
      )
    )
  result[
    match(c("Correlation", "Row Means", "General Association"), result$Test),
    c("Test", "Chisq", "Df", "Prob")
  ]
}

get_sas_results <- function(scenario) {
  sas_results %>%
    dplyr::filter(Scenario == scenario) %>%
    dplyr::select(Test, Chisq, Df, Prob)
}

test_that("2x2x2 schema, rather balanced groups", {
  result <- convert_to_tibble(CMHtest(
    Freq ~ TRTP + SEX | AGEGR1,
    data = data2,
    overall = TRUE
  ))
  expected <- get_sas_results(1L)
  expect_equal(result, expected, tolerance = 1e-3)
})

test_that("3x2x3 schema, rather balanced groups", {
  result <- convert_to_tibble(vcdExtra::CMHtest(
    Freq ~ TRTP + SEX | AGEGR1,
    data = data,
    overall = TRUE
  ))
  expected <- get_sas_results(2L)
  expect_equal(result, expected, tolerance = 1e-3)
})

test_that("3x2x3 schema, one stratum with a single observation automatically omitted", {
  stopifnot(any(table(data$RACE) == 1))
  result <- convert_to_tibble(vcdExtra::CMHtest(
    Freq ~ TRTP + SEX | RACE,
    data = data,
    overall = TRUE
  ))
  expected <- get_sas_results(3L)
  expect_equal(result, expected, tolerance = 1e-3)
})

test_that("2x5x2 schema, ordinal variable", {
  result <- convert_to_tibble(vcdExtra::CMHtest(
    Freq ~ TRTP + AVAL | SEX,
    data = data2,
    overall = TRUE
  ))
  expected <- get_sas_results(4L)
  expect_equal(result, expected, tolerance = 1e-3)
})

test_that("3x5x17 schema, sparse groups, ordinal variable", {
  stopifnot(any(table(data$SITEID) == 1))
  result <- convert_to_tibble(vcdExtra::CMHtest(
    Freq ~ TRTP + AVAL | SITEID,
    data = data,
    overall = TRUE
  ))
  expected <- get_sas_results(5L)
  expect_equal(result, expected, tolerance = 1e-3)
})

test_that("5x3x3 schema, ordinal variables", {
  result <- convert_to_tibble(vcdExtra::CMHtest(
    Freq ~ AVAL + AGEGR1N | TRTP,
    data = data,
    overall = TRUE
  ))
  expected <- get_sas_results(6L)
  expect_equal(result, expected, tolerance = 1e-3)
})

test_that("CMHtest gives overall df and p-value with a single type", {
  ord_tab <- xtabs(~ TRTP + SEX + AGEGR1, data = data)
  result <- CMHtest(ord_tab, types = "rmeans", overall = TRUE)
  expect_true(is.finite(result$ALL$table["rmeans", "Df", drop = TRUE][[1]]))
  expect_true(is.finite(result$ALL$table["rmeans", "Prob", drop = TRUE][[1]]))
})

test_that("CMHtest gives overall df and p-value with multiple types", {
  ord_tab <- xtabs(~ TRTP + SEX + AGEGR1, data = data)
  result <- CMHtest(ord_tab, types = c("rmeans", "cmeans"), overall = TRUE)
  expect_true(all(is.finite(unlist(result$ALL$table[, "Df", drop = TRUE]))))
  expect_true(all(is.finite(unlist(result$ALL$table[, "Prob", drop = TRUE]))))
})

# glmlist - make a glmlist object containing a list of fitted glm objects with their names

# borrowing code from Hmisc::llist


#' Create a Model List Object
#'
#' `glmlist` creates a `glmlist` object containing a list of fitted
#' `glm` objects with their names. `loglmlist` does the same for
#' `loglm` objects.
#'
#' The intention is to provide object classes to facilitate model comparison,
#' extraction, summary and plotting of model components, etc., perhaps using
#' \code{\link[base]{lapply}} or similar.
#'
#' @details
#'
#'
#' There exists a \code{\link[stats]{anova.glm}} method for `glmlist`
#' objects.  Here, a `coef` method is also defined, collecting the
#' coefficients from all models in a single object of type determined by
#' `result`.
#'
#' The arguments to `glmlist` or `loglmlist` are of the form
#' `value` or `name=value`.
#'
#' Any objects which do not inherit the appropriate class `glm` or
#' `loglm` are excluded, with a warning.
#'
#' In the `coef` method, coefficients from the different models are
#' matched by name in the list of unique names across all models.
#'
#' @aliases glmlist loglmlist coef.glmlist
#' @param \dots One or more model objects, as appropriate to the function, optionally assigned names as in \code{\link{list}}.
#' @param object a `"glmlist"` object
#' @param result type of the result to be returned
#'
#' @return An object of class `glmlist` `loglmlist`, just like a `list`, except that each model is given a `name` attribute.
#'
#' @author Michael Friendly; `coef` method by John Fox
#'
#' @seealso The function \code{\link[Hmisc]{llist}} in package `Hmisc` is
#' similar, but perplexingly more general.
#'
#' The function \code{\link[stats]{anova.glm}} also handles `glmlist objects`
#'
#' \code{\link{LRstats}} gives LR statistics and tests for a `glmlist`
#' object.
#' @family glmlist functions
#' @family loglinear models
#'
#' @keywords utilities models
#' @examples
#'
#' data(Mental)
#' indep <- glm(Freq ~ mental+ses,
#'                 family = poisson, data = Mental)
#' Cscore <- as.numeric(Mental$ses)
#' Rscore <- as.numeric(Mental$mental)
#'
#' coleff <- glm(Freq ~ mental + ses + Rscore:ses,
#'                 family = poisson, data = Mental)
#' roweff <- glm(Freq ~ mental + ses + mental:Cscore,
#'                 family = poisson, data = Mental)
#' linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
#'                 family = poisson, data = Mental)
#'
#' # use object names
#' mods <- glmlist(indep, coleff, roweff, linlin)
#' names(mods)
#'
#' # assign new names
#' mods <- glmlist(Indep=indep, Col=coleff, Row=roweff, LinxLin=linlin)
#' names(mods)
#'
#' LRstats(mods)
#'
#' coef(mods, result='data.frame')
#'
#' #extract model components
#' unlist(lapply(mods, deviance))
#'
#' res <- lapply(mods, residuals)
#' boxplot(as.data.frame(res), main="Residuals from various models")
#'
#' @export
glmlist <- function(...) {
    args  <- list(...);
    lname <- names(args)
    name <- vname <- as.character(sys.call())[-1]
    for (i in 1:length(args)) {
        vname[i] <- if (length(lname) && lname[i] != "")
            lname[i]
        else name[i]
    }
    names(args) <- vname[1:length(args)]
    is.glm <- unlist(lapply(args, function(x) inherits(x, "glm")))
    if (!all(is.glm)) {
    	warning("Objects ", paste(vname[!is.glm], collapse=', '),
              " removed because they are not `glm` objects")
    	args <- args[is.glm]
    }
    class(args) <- "glmlist"
    return(args);
}

# loglmlist - do the same for loglm objects

#' @rdname glmlist
#' @export
loglmlist <- function(...) {
    args  <- list(...);
    lname <- names(args)
    name <- vname <- as.character(sys.call())[-1]
    for (i in 1:length(args)) {
        vname[i] <- if (length(lname) && lname[i] != "")
            lname[i]
        else name[i]
    }
    names(args) <- vname[1:length(args)]
    is.loglm <- unlist(lapply(args, function(x) inherits(x, "loglm")))
    if (!all(is.loglm)) {
    	warning("Objects ", paste(vname[!is.loglm], collapse=', '),
              " removed because they are not `loglm` objects")
    	args <- args[is.loglm]
    }
    class(args) <- "loglmlist"
    return(args);
}

# generic version: named list

nlist <- function(...) {
    args  <- list(...);
    lname <- names(args)
    name <- vname <- as.character(sys.call())[-1]
    for (i in 1:length(args)) {
        vname[i] <- if (length(lname) && lname[i] != "")
            lname[i]
        else name[i]
    }
    names(args) <- vname[1:length(args)]
    return(args);
}

# coeficient method for a glmlist (from John Fox, r-help, 10-28-2014)

#' @rdname glmlist
#' @export
coef.glmlist <- function(object, result=c("list", "matrix", "data.frame"),
		...){
	result <- match.arg(result)
	coefs <- lapply(object, coef)
	if (result == "list") return(coefs)
	coef.names <- unique(unlist(lapply(coefs, names)))
	n.mods <- length(object)
	coef.matrix <- matrix(NA, length(coef.names), n.mods)
	rownames(coef.matrix) <- coef.names
	colnames(coef.matrix) <- names(object)
	for (i in 1:n.mods){
		coef <- coef(object[[i]])
		coef.matrix[names(coef), i] <- coef
	}
	if (result == "matrix") return(coef.matrix)
	as.data.frame(coef.matrix)
}

#' Convert any form (case or table form) into frequency form.
#' 
#' A wrapper for \code{\link[base]{as.data.frame}} that is able to properly handle arrays.
#' Converts object (`obj`) in case or table form into frequency form. The 
#' column containing the frequencies (`freq`) must be supplied if `obj` is 
#' already in frequency form (and you are using this function to select
#' dimensions). Returns a tibble if `tidy` is set to `TRUE`.
#' 
#' @param obj 
#'  Object to be converted to frequency form.
#' @param freq 
#'  If `obj` is already in frequency form, this is the name of the frequency 
#'  column. If `obj` is in any other form, do not supply an argument (see "Details").
#' @param dims 
#'  A character vector of dimensions. If not specified, all variables apart from 
#'  `freq` will be used as dimensions.
#' @param prop 
#'  If set to `TRUE`, the resulting "frequency" column will contain proportions 
#'  (that sum to 1). May also be set to a character or numeric vector of 
#'  dimensions to be used as margins from which proportions will be computed. 
#'  The resulting "frequency" column is renamed to "Prop."
#' @param tidy 
#'  Returns a tibble if set to `TRUE`.
#' @return Object in frequency form.
#' 
#' @details
#' Converts `obj` to a table using \code{\link{as_table}} before converting to
#' frequency form
#' 
#' @author Gavin M. Klorfine
#' 
#' @seealso
#' \code{\link{as_table}}, \code{\link{as_caseform}}, \code{\link{as_array}},
#' \code{\link{as_matrix}}
#' 
#' @importFrom dplyr as_tibble rename all_of
#' 
#' @examples
#' library(vcdExtra)
#' 
#' data("HairEyeColor")
#' 
#' freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
#' tableForm <- as_table(HairEyeColor) # Generate table form data
#' arrayDat <- as_array(HairEyeColor) # Generate an array
#' caseForm <- as_caseform(HairEyeColor) # Generate case form data
#' 
#' # array -> frequency form
#' as_freqform(arrayDat) |> str()
#' 
#' # table -> frequency form
#' as_freqform(tableForm) |> str()
#' 
#' # case -> frequency form
#' as_freqform(caseForm) |> str()
#' 
#' # Selecting dimensions (optional)
#' as_freqform(freqForm, freq = "Freq", dims = c("Hair", "Eye")) |> str()
#' 
#' as_freqform(tableForm, dims = c("Hair", "Eye")) |> str()
#' 
#' #-----For proportions-----#
#' 
#' as_freqform(tableForm, prop = TRUE) |> head() # print only Sex == Male rows
#' 
#' # Marginalize proportions along "Sex" (i.e., male proportions sum to 1, female proportions sum to 1)
#' as_freqform(tableForm, prop = "Sex") |> head()
#' 
#' as_freqform(tableForm, prop = 3) |> head() # Same as above
#' 
#' # Marginalize proportions along multiple variables
#' as_freqform(tableForm, prop = c("Hair", "Sex")) |> head()
#' 
#' as_freqform(tableForm, prop = c(1, 3)) |> head() # Same as above
#' 
#' # Using dims and prop arguments in tandem
#' as_freqform(tableForm, dims = c("Hair", "Eye"), prop = TRUE)
#' 
#' @export

as_freqform <- function(obj, freq = NULL, dims = NULL, prop = NULL, tidy = TRUE){
  
  tab <- as.data.frame(as_table(obj, freq = freq, dims = dims, prop = prop))
  
  if (tidy)
    tab <- dplyr::as_tibble(tab)
  
  # Account for new column named "Freq" if freq was NULL
  if (is.null(freq))
    freq <- "Freq"
  
  if (!is.null(prop))
    tab <- tab |> dplyr::rename("Prop" = dplyr::all_of(freq))
  
  return(tab)
}

data("HairEyeColor")
data("Hoyt", package = "vcdExtra")

test_that("Equal sum produced via comparison with HairEyeColor data", {
  coll.sum <- collapse_levels(
    HairEyeColor,                 
    Hair = list(                  
      Dark = c("Black", "Brown"), 
      Light = c("Blond", "Red")  
    ),
    Eye = list(                   
      Common = c("Brown"),
      Uncommon = c("Blue", "Green", "Hazel")
    )
  ) |> sum()
  expect_equal(sum(HairEyeColor), coll.sum)
})

test_that("Equal sum produced via comparison with Hoyt data", {
  ff_Hoyt <- as_freqform(Hoyt)
  names(ff_Hoyt)[length(ff_Hoyt)] <- "n"
  
  ff_coll <- collapse_levels(
    ff_Hoyt,
    freq = "n",
    Occupation = list(
      High = c(1, 2),
      Middle = 3,
      Low = 4,
      VeryLow = c(5, 6, 7)
    )
  )
  expect_equal(sum(ff_coll$Freq), sum(Hoyt))
})

data("HairEyeColor")

arrayDat <- as_array(HairEyeColor) # Generate an array
freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
caseForm <- expand.dft(freqForm) # Generate case form data

test_that("Array form input returns a tibble", {
  ff <- as_freqform(arrayDat)
  expect_s3_class(ff, "tbl")
})

test_that("Table form input returns a tibble", {
  ff <- as_freqform(HairEyeColor)
  expect_s3_class(ff, "tbl")
})

test_that("Tibble frequency form input returns a tibble", {
  ff <- as_freqform(tidy_freqForm, freq = "n")
  expect_s3_class(ff, "tbl")
})

test_that("Frequency form input returns a tibble", {
  ff <- as_freqform(freqForm, freq = "Freq")
  expect_s3_class(ff, "tbl")
})

test_that("Case form input returns a tibble", {
  ff <- as_freqform(caseForm)
  expect_s3_class(ff, "tbl")
})

test_that("Tibble frequency form input returns unmodified", {
  ff <- as_freqform(tidy_freqForm, freq = "n")
  expect_equal(sum(ff$Freq), sum(tidy_freqForm$n))
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/datasets-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

## ----load---------------------------------------------------------------------
library(dplyr)
library(tidyr)
library(readxl)

## ----read-datasets------------------------------------------------------------
dsets_tagged <- read_excel(here::here("inst", "extdata", "vcdExtra-datasets.xlsx"), 
                           sheet="vcdExtra-datasets")

dsets_tagged <- dsets_tagged |>
  dplyr::select(-Title, -dim) |>
  dplyr::rename(dataset = Item)

head(dsets_tagged)

## ----split-tags---------------------------------------------------------------
dset_split <- dsets_tagged |>
  tidyr::separate_longer_delim(tags, delim = ";") |>
  dplyr::mutate(tag = stringr::str_trim(tags)) |>
  dplyr::select(-tags)

#' ## collapse the rows for the same tag
tag_dset <- dset_split |>
  arrange(tag) |>
  dplyr::group_by(tag) |>
  dplyr::summarise(datasets = paste(dataset, collapse = "; ")) |> ungroup()

# get a list of the unique tags
unique(tag_dset$tag)

## ----read-tags----------------------------------------------------------------
tags <- read_excel(here::here("inst", "extdata", "vcdExtra-datasets.xlsx"), 
                   sheet="tags")
head(tags)

## ----join-tags----------------------------------------------------------------
tag_dset <- tag_dset |>
  dplyr::left_join(tags, by = "tag") |>
  dplyr::relocate(topic, .after = tag)

tag_dset |>
  dplyr::select(-tag) |>
  head()

## ----add-links----------------------------------------------------------------
add_links <- function(dsets, 
                      style = c("reference", "help", "rdrr.io"),
                      sep = "; ") {

  style <- match.arg(style)
  names <- stringr::str_split_1(dsets, sep)

  names <- dplyr::case_when(
    style == "help"      ~ glue::glue("[{names}](help({names}))"),
    style == "reference" ~ glue::glue("[{names}](../reference/{names}.html)"),
    style == "rdrr.io"   ~ glue::glue("[{names}](https://rdrr.io/cran/vcdExtra/man/{names}.html)")
  )  
  glue::glue_collapse(names, sep = sep)
}


## ----kable--------------------------------------------------------------------
tag_dset |>
  dplyr::select(-tag) |>
  dplyr::mutate(datasets = purrr::map(datasets, add_links)) |>
  knitr::kable()

#' Convert frequency or table form into case form.
#' 
#' Converts object (`obj`) in frequency or table form into case form. The 
#' column containing the frequencies (`freq`) must be supplied if `obj` is in 
#' frequency form. Returns a tibble if `tidy` is set to `TRUE`.
#' 
#' @param obj 
#'  Object to be converted to case form.
#' @param freq 
#'  If `obj` is in frequency form, this is the name of the frequency column. If 
#'  `obj` is in any other form, do not supply an argument (see "Details").
#' @param dims 
#'  A character vector of dimensions. If not specified, all variables apart from 
#'  `freq` will be used as dimensions.
#' @param tidy 
#'  Returns a tibble if set to `TRUE`.
#' @return object in case form.
#' 
#' @details
#' A wrapper for \code{\link{expand.dft}} that is able to handle arrays.
#' 
#' If a frequency column is not supplied, this function defaults to "Freq"
#' just like \code{\link{expand.dft}}. Converts `obj` to a table using 
#' \code{\link{as_table}} before converting to case form.
#' 
#' @author Gavin M. Klorfine
#' 
#' @seealso
#' \code{\link{as_table}}, \code{\link{as_freqform}}, \code{\link{as_array}},
#' \code{\link{as_matrix}}, \code{\link{expand.dft}}
#' 
#' @importFrom dplyr as_tibble is.tbl
#' 
#' @examples
#' library(vcdExtra)
#' 
#' data("HairEyeColor")
#' 
#' freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
#' tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
#' tableForm <- as_table(HairEyeColor) # Generate table form data
#' arrayDat <- as_array(HairEyeColor) # Generate an array
#' 
#' # Frequency form -> case form
#' as_caseform(freqForm) |> str()
#' 
#' # Frequency form (tibble) -> case form
#' as_caseform(tidy_freqForm, freq = "n") |> str()
#' 
#' # Array -> case form
#' as_caseform(arrayDat) |> str()
#' 
#' # Optionally specify dims
#' as_caseform(tableForm, dims = c("Hair", "Eye")) |> str()
#'
#' 
#' @export

as_caseform <- function(obj, freq = "Freq", dims = NULL, tidy = TRUE){
  
  if ((dplyr::is.tbl(obj) || is.data.frame(obj)))
    freqs <- obj[[freq]]
  else
    freqs <- as.numeric(obj)
  
  if (any(freqs %% 1 != 0))
    stop("Frequency column contains decimal values.")
  if (any(freqs < 0))
    stop("Frequency column contains negative values.")
  
  tab <- as_table(obj, freq = freq, dims = dims)
  cf <- expand.dft(tab, freq = freq)
  
  if (tidy)
    cf <- dplyr::as_tibble(cf)
  
  return(cf)
}

data("HairEyeColor")

freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
tableForm <- as_table(HairEyeColor) # Generate table form data
arrayDat <- as_array(HairEyeColor) # Generate an array

test_that("Number of rows equal to number of entries", {
  expect_equal(
    sum(freqForm$Freq), 
    HairEyeColor |> as_caseform() |> nrow()
  )
})

test_that("Returns tibble by default", {
  expect_s3_class(as_caseform(freqForm), "tbl")
})

# ✔️DONE: Initial implementation, using `gt` pkg for the table
#
# ✔️DONE: When the cell is colored a dark shade, use white text for readability.
#         Uses colorspace::contrast_ratio() to pick optimal text color.
# ✔️DONE: The default palette for diverging uses darker shades from RdBu (#2166AC, #B2182B)
# ✔️DONE: Suppress Warning `In chisq.test(x) : Chi-squared approximation may be incorrect`
# ✔️DONE: Test cases in dev/test-color_table.R using vcd::Suicide, vcd::PreSex, vcdExtra::Abortion
# ✔️DONE: Fixed bug in format(round(...)) - now checks is.numeric() before applying round()
#         The error occurred because as.data.frame() on a matrix can include factor columns.
# ✔️DONE: Make the display of the Total row and column optional. -- handled by `margins`
#
# ✔️DONE: Row category labels (stub) now bold, matching column labels; Total row stub is italic.
#         Future enhancement: could extend `margins` to accept a list for custom styling.
#
# ✔️DONE: Add filename arg, which if not NULL saves the `gt` result as an image via gt::gtsave().
#         Supports .png, .pdf, .html, .rtf, .docx formats. Additional args passed via `...`.
#
# ✔️DONE: Refactored as S3 generic with methods for table, xtabs, ftable, structable, data.frame, matrix.
#         The .color_table_impl() internal function handles the core gt table building.
#
# ✔️DONE: Allow input argument x to be a dataset in frequency form (data.frame with Freq column).
#         New freq_col parameter allows specifying the frequency column name.
#
# ✔️DONE: For multi-way tables, use MASS::loglm() to fit complete independence model and compute
#         proper Pearson residuals. Print message with X^2, df, p-value when shade != "freq".
#         Uses cat() for output to ensure visibility in all R environments.
#
# ✔️DONE: Note legend for table shading interpretation implemented via `gt::tab_source_note()`.
#         `legend = TRUE` or `legend = "note"` adds note (default); `legend = FALSE` suppresses.
#         For `shade = "freq"`: "Shading based on values of observed frequencies".
#         For residuals: uses same text as printed to console (X^2, df, p-value).
#         Future: `legend = "graphic"` could show graphic legend like
#         `dev/color-tab-figs/shading-legend.png`.
#
# ✔️DONE: Add an argument `values = "frequency" | "residuals"` (possibly
#         abbreviated) to all the values displayed in the table to be the cell
#         frequencies (as now, and default), or to be the residuals for the `model` fit.
#
# ✔️DONE: When there are two (or more) variables for the row, these now appear as a nested
#         hierarchy similar to what is shown in the table for the Titanic data in
#         `dev/color-tab-figs/Titanic-residual-shading.png`. Row-variable combinations
#         that used to appear concatenated as "Black_Male" now become separate stub
#         columns, "Hair" and "Sex", with repeated values left blank. Uses gt's
#         multi-column stub support (`rowname_col` accepting a character vector of column
#         names, gt >= 0.11) -- see dev/color_table/multi-col-stubs.md and
#         dev/color_table/test-multi-col-stubs.R. Mirrors the has_multi_col_vars spanner
#         logic below, using the `row_vars` attribute (attached by the S3 methods but
#         previously unused).
#
# ✔️DONE: Resolved the apparent `formula` row/col inconsistency flagged earlier. The convention
#         is `col_vars ~ row_vars` (LHS = columns, RHS = rows) -- the opposite of what was
#         originally documented. Systematically compared `vcd::structable()` vs `stats::ftable()`
#         across every 2-vs-1 split of `HairEyeColor` and every 2-vs-2 and 3-vs-1 split of
#         `Titanic` (see dev/color_table/formula-row-col-comparison.R): when both are called the
#         same way `color_table()` calls them (`formula` first, `data =` named), they agree in
#         every case on `col_vars ~ row_vars`. The earlier-seen disagreement (in
#         dev/color_table/test-multi-col-stubs.R) came from an ad-hoc comparison call with `data`
#         positional before `formula` -- not a real inconsistency between the two functions.
#         Roxygen docs (`@param formula`, `@details`, `@examples`) updated to match.
#
# ✔️TODO: Column spanner headings: When two or more variables are in the columns, the output
#         is confusing and ugly. Examples:
#     color_table(PreSex,  formula = MaritalStatus + Gender ~ PremaritalSex + ExtramaritalSex)
#     color_table(PreSex,  formula = Gender + PremaritalSex + ExtramaritalSex ~  MaritalStatus)
#         This can be handled using column spanners: https://gt.rstudio.com/reference/tab_spanner.html
#


#' Smart Display of Frequency Tables with Colored Cell Backgrounds
#'
#' `color_table()` creates a formatted, semi-graphic "heatmap" table display of frequency data with cell backgrounds
#' colored according to observed frequencies or their residuals from a loglinear model. The goal is to provide a
#' "smart" tabular display of cross-classified frequency data, with some abilities to highlight possibly interesting
#' patterns or unusual cells.
#' This is an S3 generic function, providing methods for different input types: `"table"`, `"xtabs"`, `"matrix"`, `"ftable"`, `"structable"` or `"data.frame"`.
#'
#' @param x A `"table"`, `"xtabs"`, `"matrix"`, `"ftable"`, `"structable"`, or `"data.frame"` object
#' @param ... Additional arguments passed to methods
#'
#' @details
#' This function provides a heatmap-style representation of a frequency table,
#' where background coloring is used to visualize patterns and anomalies in the data.
#' When shading by _residuals_ (the default), cells with large positive residuals
#' (more observations than expected) are shaded red, while cells with large negative
#' residuals (fewer than expected) are shaded blue. This makes it easy to identify
#' cells that deviate substantially from what would be expected under a given model
#' (by default, the independence model).
#'
#' For multi-way tables (3 or more dimensions), residuals are computed from the
#' model of complete independence among all factors using \code{\link[MASS]{loglm}}.
#' But you can specify a model using the `model` or `expected` arguments, in a way similar to that provided
#' by `mosaic.glm()`.
#' A message is printed showing the chi-squared statistic, degrees of freedom,
#' and p-value for this test.
#'
#' **Multi-variable row and column labels**
#'
#' `formula` follows the `col_vars ~ row_vars` convention used by
#' [vcd::structable()] and [stats::ftable()] (confirmed to agree between the
#' two across many row/col splits of `HairEyeColor` and `Titanic` -- see
#' `dev/color_table/formula-row-col-comparison.R`). When `formula` specifies
#' more than one variable on either side, `color_table()` avoids concatenated
#' labels like `"Black_Male"`. Multiple column variables (on the left of `~`,
#' e.g. `formula = Class + Sex + Age ~ Survived`) are shown using
#' [gt::tab_spanner()] headings that group columns under their outer
#' variable(s). Multiple row variables (on the right of `~`, e.g.
#' `formula = Survived ~ Class + Sex`) are shown as separate stub columns, one
#' per row variable, using `gt`'s multi-column stub support (`rowname_col`
#' accepting a vector of column names), with repeated values left blank. The
#' two can be combined in the same table.
#'
#' **Contrast shading**
#'
#' For cells with dark background colors, black text can be difficult to read.
#' This function automatically selects white or black text for each cell based
#' on which provides better contrast against the background color. If the
#' \pkg{colorspace} package is available, \code{\link[colorspace]{contrast_ratio}}
#' is used to determine the optimal text color according to WCAG 2.1 guidelines.
#' Otherwise, a fallback based on relative luminance (ITU-R BT.709) is used.
#'
#' **Use in documents**
#'
#' In R Markdown (\code{.Rmd}) or Quarto (\code{.qmd}) documents, \pkg{gt} tables
#' render natively in \strong{HTML output} — simply return the \code{gt} object from
#' a chunk and knitr renders it automatically via \pkg{gt}'s built-in
#' \code{knit_print} method. No \code{filename} argument is needed.
#'
#' For \strong{PDF or Word output}, \pkg{gt} does not render natively. Use the
#' \code{filename} argument to save the table as a \code{.png} image, then include
#' it with \code{\link[knitr]{include_graphics}}:
#'
#' \preformatted{
#'     color_table(my_table, filename = "my_table.png")
#'     knitr::include_graphics("my_table.png")
#' }
#'
#' The \code{vwidth} and \code{vheight} arguments (passed via \code{...}) control
#' the image viewport size in pixels.  Supported save formats are
#' \code{.png}, \code{.pdf}, \code{.html}, \code{.rtf}, and \code{.docx}.
#'
#' For documents that target \strong{multiple output formats}, a small helper that
#' branches on \code{\link[knitr]{is_html_output}} avoids duplicating code:
#'
#' \preformatted{
#'     gt_obj <- color_table(my_table)
#'     if (knitr::is_html_output()) {
#'       gt_obj
#'     } else {
#'       gt::gtsave(gt_obj, "my_table.png")
#'       knitr::include_graphics("my_table.png")
#'     }
#' }
#'
#' If you need a caption or cross-reference label, use \code{gt::tab_caption()}
#' on the returned object:
#' \preformatted{
#'     color_table(my_table) |>
#'       gt::tab_caption("Table 1: Pattern of association in MyTable")
#' }
#'
#' @return A gt table object that can be further customized
#'
#' @examples
#' # Basic usage with 2-way table - shade by residuals from independence
#' data(HairEyeColor)
#' HEC <- margin.table(HairEyeColor, 1:2)  # 2-way: Hair x Eye
#' color_table(HEC)
#'
#' # Shade by frequencies instead (no message printed)
#' color_table(HEC, shade = "freq")
#'
#' # 3-way table - using a formula to specify layout (col_vars ~ row_vars):
#' # Eye is the single column variable, Hair and Sex are row variables shown
#' # as separate stub columns
#' color_table(HairEyeColor, formula = Eye ~ Hair + Sex)
#'
#' # Display residual values in cells instead of frequencies
#' color_table(HEC, values = "residuals")
#'
#' # Multiple column variables - shown using nested spanner headings instead
#' # of concatenated column labels
#' data(Titanic)
#' color_table(Titanic, formula = Class + Sex ~ Survived)
#'
#' # Multiple row AND column variables together: column spanners (Class, Sex)
#' # combined with row-variable stub columns (Age, Survived)
#' color_table(Titanic, formula = Class + Sex ~ Age + Survived, legend = TRUE)
#'
#' \dontrun{
#' # From a data.frame in frequency form (2-way)
#' hec_df <- as.data.frame(HEC)
#' color_table(hec_df)
#'
#' # Save table as an image file
#' color_table(HEC, filename = "hair_eye_table.png")
#' }
#'
#' @importFrom stats chisq.test residuals xtabs pchisq
#' @importFrom grDevices col2rgb
#' @importFrom dplyr all_of everything
#' @export
color_table <- function(x, ...) {
  UseMethod("color_table")
}

#' @describeIn color_table Method for table objects (including result of xtabs)
#' @param formula Formula specifying a `col_vars ~ row_vars` layout (for multi-way tables) to
#'        make them "flat" as defined for `vcd::structable()` and `stats::ftable()`.
#' @param values What values to display in cells: `"freq"` for observed frequencies (default),
#'   or `"residuals"` to display the residual values. When `values = "residuals"`, margins
#'   are suppressed since residuals don't have meaningful totals.
#' @param shade What values determine cell shading: `"residuals"` (default), `"freq"`,
#'        `"pearson"`, or `"deviance"`
#' @param model A fitted model (loglm or glm) to compute residuals from.
#'        If NULL and shade involves residuals, uses an independence model for all factors.
#' @param expected Expected frequencies (alternative to `model`), a data structure of the same shape
#'        as `x`
#' @param palette Color palette function or vector for background colors. Default depends on shade
#'        type. When `shade = "freq"` the default is `palette = c("white", "firebrick")`; otherwise
#'        `c("#B2182B", "white", , "#2166AC")` ranging from red to blue for negative and positive
#'         residuals. The background colors are computed by interpolation using
#'         `scales::col_numeric()`.
#'
#' @param legend Controls display of shading interpretation note:
#'        \code{TRUE} or \code{"note"} (default) adds a source note explaining the shading;
#'        \code{FALSE} (default) suppresses the note, but a message is printed in the console.
#' @param margins Logical, include row/column totals?
#' @param digits Number of decimal places for displayed values
#' @param title Optional table title
#' @param filename Optional filename to save the table as an image. If provided,
#'        the table is saved using \code{\link[gt]{gtsave}}. Supported formats include
#'        \code{.png}, \code{.pdf}, \code{.html}, \code{.rtf}, and \code{.docx}.
#'        The file format is determined by the file extension. Other arguments can be passed
#'        to \code{\link[gt]{gtsave}} via `...`.
#' @export
color_table.table <- function(x,
                               formula = NULL,
                               values = c("freq", "residuals"),
                               shade = c("residuals", "freq", "pearson", "deviance"),
                               model = NULL,
                               expected = NULL,
                               palette = NULL,
                               legend = FALSE,
                               margins = TRUE,
                               digits = 0,
                               title = NULL,
                               filename = NULL,
                               ...) {

  values <- match.arg(values)
  shade <- match.arg(shade)
  if (shade == "pearson") shade <- "residuals"

  # Store original table for proper residual computation
  x_orig <- x

  dims <- dim(x)
  ndim <- length(dims)

  # For multi-way tables, use structable to flatten
  if (ndim > 2) {
    if (requireNamespace("vcd", quietly = TRUE)) {
      if (!is.null(formula)) {
        st <- vcd::structable(formula, data = x)
      } else {
        # Default: first variable as rows, remaining as columns
        st <- vcd::structable(x)
      }

      # Extract variable structure BEFORE converting to matrix
      col_vars <- attr(st, "col.vars")
      row_vars <- attr(st, "row.vars")

      x_mat <- as.matrix(st)

      # Attach as attributes to the matrix for use in .color_table_impl
      attr(x_mat, "col_vars") <- col_vars
      attr(x_mat, "row_vars") <- row_vars
    } else {
      stop("Package 'vcd' is required for multi-way tables.")
    }
  } else {
    x_mat <- as.matrix(x)
  }

  .color_table_impl(x_mat,
                    x_orig = x_orig,
                    formula = formula,
                    values = values,
                    shade = shade,
                    model = model,
                    expected = expected,
                    palette = palette,
                    legend = legend,
                    margins = margins,
                    digits = digits,
                    title = title,
                    filename = filename,
                    ...)
}

#' @describeIn color_table Method for ftable objects
#' @export
color_table.ftable <- function(x,
                                values = c("freq", "residuals"),
                                shade = c("residuals", "freq", "pearson", "deviance"),
                                model = NULL,
                                expected = NULL,
                                palette = NULL,
                                legend = FALSE,
                                margins = TRUE,
                                digits = 0,
                                title = NULL,
                                filename = NULL,
                                ...) {

  values <- match.arg(values)
  shade <- match.arg(shade)
  if (shade == "pearson") shade <- "residuals"

  # Convert ftable to matrix
  x_mat <- as.matrix(x)

  # ftable loses the original structure, so we can only use 2D residuals
  .color_table_impl(x_mat,
                    x_orig = NULL,
                    formula = NULL,
                    values = values,
                    shade = shade,
                    model = model,
                    expected = expected,
                    palette = palette,
                    legend = legend,
                    margins = margins,
                    digits = digits,
                    title = title,
                    filename = filename,
                    ...)
}

#' @describeIn color_table Method for structable objects (vcd package)
#' @export
color_table.structable <- function(x,
                                    values = c("freq", "residuals"),
                                    shade = c("residuals", "freq", "pearson", "deviance"),
                                    model = NULL,
                                    expected = NULL,
                                    palette = NULL,
                                    legend = FALSE,
                                    margins = TRUE,
                                    digits = 0,
                                    title = NULL,
                                    filename = NULL,
                                    ...) {

  values <- match.arg(values)
  shade <- match.arg(shade)
  if (shade == "pearson") shade <- "residuals"


  # Extract variable structure BEFORE converting to matrix
  col_vars <- attr(x, "col.vars")
  row_vars <- attr(x, "row.vars")

  # Convert structable to matrix (flattened 2D form)
  x_mat <- as.matrix(x)

  # Attach as attributes for use in .color_table_impl
  attr(x_mat, "col_vars") <- col_vars
  attr(x_mat, "row_vars") <- row_vars

  # structable loses the original structure, so we can only use 2D residuals
  .color_table_impl(x_mat,
                    x_orig = NULL,
                    formula = NULL,
                    values = values,
                    shade = shade,
                    model = model,
                    expected = expected,
                    palette = palette,
                    legend = legend,
                    margins = margins,
                    digits = digits,
                    title = title,
                    filename = filename,
                    ...)
}

#' @describeIn color_table Method for data.frame in frequency form
#' @param freq_col Name of the frequency column. If NULL, looks for "Freq" or "count".
#' @export
color_table.data.frame <- function(x,
                                    formula = NULL,
                                    freq_col = NULL,
                                    values = c("freq", "residuals"),
                                    shade = c("residuals", "freq", "pearson", "deviance"),
                                    model = NULL,
                                    expected = NULL,
                                    palette = NULL,
                                    legend = FALSE,
                                    margins = TRUE,
                                    digits = 0,
                                    title = NULL,
                                    filename = NULL,
                                    ...) {

  values <- match.arg(values)
  shade <- match.arg(shade)
  if (shade == "pearson") shade <- "residuals"

  # Find the frequency column
  if (is.null(freq_col)) {
    if ("Freq" %in% names(x)) {
      freq_col <- "Freq"
    } else if ("count" %in% names(x)) {
      freq_col <- "count"
    } else {
      stop("Cannot find frequency column. Specify freq_col or use a column named 'Freq' or 'count'.")
    }
  }

  # Get factor columns (all columns except the frequency column)
  factor_cols <- setdiff(names(x), freq_col)

  if (length(factor_cols) < 2) {
    stop("Data frame must have at least 2 factor columns plus a frequency column.")
  }

  # Build xtabs formula
  if (is.null(formula)) {
    # Default: first factor as rows, second as columns
    xtabs_formula <- as.formula(paste(freq_col, "~", paste(factor_cols, collapse = " + ")))
  } else {
    # User provided formula - need to add freq_col to LHS
    xtabs_formula <- as.formula(paste(freq_col, "~", as.character(formula)[2]))
  }

  # Convert to table using xtabs
  x_table <- xtabs(xtabs_formula, data = x)

  # Now use the table method
  color_table.table(x_table,
                    formula = formula,
                    shade = shade,
                    model = model,
                    expected = expected,
                    palette = palette,
                    legend = legend,
                    margins = margins,
                    digits = digits,
                    values = values,
                    title = title,
                    filename = filename,
                    ...)
}

#' @describeIn color_table Method for matrix objects
#' @export
color_table.matrix <- function(x,
                                values = c("freq", "residuals"),
                                shade = c("residuals", "freq", "pearson", "deviance"),
                                model = NULL,
                                expected = NULL,
                                palette = NULL,
                                legend = FALSE,
                                margins = TRUE,
                                digits = 0,
                                title = NULL,
                                filename = NULL,
                                ...) {

  values <- match.arg(values)
  shade <- match.arg(shade)
  if (shade == "pearson") shade <- "residuals"

  .color_table_impl(x,
                    x_orig = NULL,
                    formula = NULL,
                    values = values,
                    shade = shade,
                    model = model,
                    expected = expected,
                    palette = palette,
                    legend = legend,
                    margins = margins,
                    digits = digits,
                    title = title,
                    filename = filename,
                    ...)
}

#' @describeIn color_table Default method
#' @export
color_table.default <- function(x, ...) {
  stop("color_table() does not know how to handle object of class ",
       paste(class(x), collapse = "/"),
       ". Supported classes: table, xtabs, ftable, structable, data.frame, matrix.")
}

# ###
# Internal implementation function
# ###

# Internal workhorse function for color_table
#
# This function takes a 2D matrix and builds the gt table with colored cells.
# All S3 methods convert their input to a matrix and call this function.
#
# @param x A 2D matrix with row and column names (the flattened display matrix)
# @param x_orig The original table (for multi-way tables, used to compute
#   complete independence residuals). NULL for 2D tables.
# @param formula The formula used for flattening (needed for residual mapping)
# @param shade Type of shading
# @param model Optional fitted model
# @param expected Optional expected frequencies
# @param palette Color palette
# @param legend Show legend?
# @param margins Show margins?
# @param values What to display in cells: "freq" or "residuals"
# @param digits Decimal places
# @param title Table title
# @param filename Optional file to save
# @param ... Additional arguments passed to gtsave
#
# @noRd
.color_table_impl <- function(x,
                               x_orig = NULL,
                               formula = NULL,
                               values = "freq",
                               shade = "residuals",
                               model = NULL,
                               expected = NULL,
                               palette = NULL,
                               legend = FALSE,
                               margins = TRUE,
                               digits = 0,
                               title = NULL,
                               filename = NULL,
                               ...) {


  # # DEBUG: Confirm function entry and parameters
  # cat("DEBUG: Entering .color_table_impl\n")
  # cat("DEBUG: shade =", shade, "\n")
  # cat("DEBUG: is.null(x_orig) =", is.null(x_orig), "\n")
  # if (!is.null(x_orig)) {
  #   cat("DEBUG: length(dim(x_orig)) =", length(dim(x_orig)), "\n")
  # }

  if (!requireNamespace("gt", quietly = TRUE)) {
    stop("Package 'gt' is required for color_table(). Please install it.")
  }

  # Ensure x is a matrix
  if (!is.matrix(x)) {
    x <- as.matrix(x)
  }

  dims <- dim(x)

  # Get dimension names
  dn <- dimnames(x)
  rnames <- dn[[1]]
  cnames <- dn[[2]]
  rvar <- names(dn)[1]
  cvar <- names(dn)[2]

  # Provide default names if missing
  if (is.null(rnames)) rnames <- paste0("R", seq_len(dims[1]))
  if (is.null(cnames)) cnames <- paste0("C", seq_len(dims[2]))
  if (is.null(rvar) || rvar == "") rvar <- "Row"
  if (is.null(cvar) || cvar == "") cvar <- "Col"

  # Ensure dimnames are set on x
  dimnames(x) <- list(rnames, cnames)
  names(dimnames(x)) <- c(rvar, cvar)

  # Validate values parameter
  if (values == "residuals" && shade == "freq") {
    stop("Cannot display residuals when shade = 'freq'. Use shade = 'residuals' or similar.")
  }

  # Initialize shading message for legend
  shading_message <- NULL

  # Compute shading values
  # cat("DEBUG: About to compute shading values\n")
  if (shade == "freq") {
    # cat("DEBUG: shade == 'freq', skipping residuals\n")
    shade_values <- x
    resid_mat <- NULL  # No residuals computed
    shading_message <- "Shading based on values of observed frequencies"
  } else {
    # cat("DEBUG: Computing residuals (shade != 'freq')\n")
    # Compute residuals
    if (!is.null(model)) {
      # Use provided model - extract and reshape residuals
      resid_arr <- residuals(model, type = "pearson")
      resid_mat <- matrix(as.vector(resid_arr), nrow = dims[1], ncol = dims[2])
      # Print model info
      if (inherits(model, "loglm")) {
        shading_message <- sprintf("Shading based on residuals from fitted model, X^2 = %.2f, df = %d, p = %.4g",
                    model$pearson, model$df, 1 - pchisq(model$pearson, model$df))
        cat(shading_message, "\n")
      }
    } else if (!is.null(expected)) {
      # Use provided expected values
      resid_mat <- (x - expected) / sqrt(expected)
      shading_message <- "Shading based on residuals from user-supplied expected frequencies"
      cat(shading_message, "\n")
    } else {
      # Fit independence model
      # For multi-way tables, use loglm for complete independence
      if (!is.null(x_orig) && length(dim(x_orig)) > 2) {
        # Multi-way table: fit complete independence model
        # cat("DEBUG: In multi-way table branch (>2 dimensions)\n")
        if (!requireNamespace("MASS", quietly = TRUE)) {
          stop("Package 'MASS' is required for residuals from multi-way tables.")
        }

        # Build formula for complete independence: ~ A + B + C + ...
        var_names <- names(dimnames(x_orig))
        indep_formula <- as.formula(paste("~", paste(var_names, collapse = " + ")))

        # Use do.call to avoid scoping issues with loglm's non-standard evaluation
        mod_indep <- do.call(MASS::loglm, list(formula = indep_formula, data = x_orig))

        # Get residuals from the model
        resid_arr <- residuals(mod_indep, type = "pearson")

        # Map residuals to the flattened matrix structure
        # The structable flattening preserves cell order, so we can reshape
        if (!is.null(formula)) {
          # Re-create the structable to get proper cell ordering
          resid_st <- vcd::structable(formula, data = resid_arr)
          resid_mat <- as.matrix(resid_st)
        } else {
          # No formula - use default structable flattening
          resid_st <- vcd::structable(resid_arr)
          resid_mat <- as.matrix(resid_st)
        }

        # Print model info
        shading_message <- sprintf("Shading based on residuals from model of complete independence, X^2 = %.2f, df = %d, p = %.4g",
                       mod_indep$pearson, mod_indep$df, 1 - pchisq(mod_indep$pearson, mod_indep$df))
        if (is.null(formula)) {
          shading_message <- paste0(shading_message, "\nUse formula for better control.")
        }
        cat(shading_message, "\n")

      } else {
        # 2-way table: use chisq.test (equivalent to independence)
        # cat("DEBUG: In 2-way table branch, about to run chisq.test\n")
        chi_result <- suppressWarnings(chisq.test(x))
        resid_mat <- chi_result$residuals
        # cat("DEBUG: chisq.test completed, X^2 =", unname(chi_result$statistic), "\n")

        # Print model info
        shading_message <- sprintf("Shading based on residuals from model of independence,\n X^2 = %.2f, df = %d, p = %.4g",
                    unname(chi_result$statistic),
                    unname(chi_result$parameter),
                    chi_result$p.value)
        cat(shading_message, "\n")
      }
    }

    if (shade == "deviance" && is.null(model)) {
      # Compute deviance residuals
      # For multi-way tables, need expected from loglm
      if (!is.null(x_orig) && length(dim(x_orig)) > 2) {
        var_names <- names(dimnames(x_orig))
        indep_formula <- as.formula(paste("~", paste(var_names, collapse = " + ")))
        mod_indep <- do.call(MASS::loglm, list(formula = indep_formula, data = x_orig))
        exp_arr <- fitted(mod_indep)
        if (!is.null(formula)) {
          exp_st <- vcd::structable(formula, data = exp_arr)
          exp_mat <- as.matrix(exp_st)
        } else {
          exp_st <- vcd::structable(exp_arr)
          exp_mat <- as.matrix(exp_st)
        }
      } else {
        chi_result <- suppressWarnings(chisq.test(x))
        exp_mat <- chi_result$expected
      }
      obs <- x
      resid_mat <- sign(obs - exp_mat) * sqrt(2 * ifelse(obs > 0, obs * log(obs / exp_mat), 0))
    }

    shade_values <- resid_mat
  }

  # Determine what values to display in cells
  if (values == "residuals") {
    display_mat <- resid_mat
    # Margins don't make sense for residuals
    show_margins <- FALSE
    # Use more decimal places for residuals if digits is 0
    if (digits == 0) digits <- 2
  } else {
    display_mat <- x
    show_margins <- margins
  }

  # Create data frame for gt
  x_mat <- matrix(as.vector(display_mat), nrow = nrow(display_mat), ncol = ncol(display_mat),
                  dimnames = list(rnames, cnames))
  df <- as.data.frame(x_mat, check.names = FALSE)

  # Add row totals if margins requested (only for frequencies)
  if (show_margins) {
    df$Total <- rowSums(x)  # Always use frequencies for totals
  }

  # Convert to character for display
  df_display <- df
  for (col in names(df_display)) {
    if (is.numeric(df_display[[col]])) {
      df_display[[col]] <- format(round(df_display[[col]], digits), nsmall = digits)
    }
  }

  # Check for multi-variable row structure (from structable attributes)
  row_vars <- attr(x, "row_vars")
  has_multi_row_vars <- !is.null(row_vars) && length(row_vars) > 1

  if (has_multi_row_vars) {
    # Multi-column stub: one column per row variable, using gt's support for
    # rowname_col = <character vector> (gt >= 0.11) instead of a single column
    # of concatenated labels like "Black_Male".
    row_var_names <- names(row_vars)
    n_row_vars <- length(row_var_names)

    # Split composite row names (e.g., "Black_Male" -> c("Black", "Male"))
    split_rnames <- strsplit(rownames(df), "_", fixed = TRUE)
    row_var_df <- as.data.frame(do.call(rbind, split_rnames), stringsAsFactors = FALSE)
    names(row_var_df) <- row_var_names

    df_display <- cbind(row_var_df, df_display)
  } else {
    # Add row names as first column
    df_display <- cbind(data.frame(row_var = rownames(df)), df_display)
    names(df_display)[1] <- rvar
  }

  # Add column totals row if margins requested (only for frequencies)
  if (show_margins) {
    total_label <- if (has_multi_row_vars) c("Total", rep("", n_row_vars - 1)) else "Total"
    col_totals <- c(total_label, as.character(colSums(x)),
                    as.character(sum(x)))
    df_display <- rbind(df_display, col_totals)
  }

  # Set up color palette
  if (is.null(palette)) {
    if (shade == "freq") {
      palette <- c("white", "firebrick")
    } else {
      palette <- c("#B2182B", "white", "#2166AC")
    }
  }

  # Determine domain for colors
  if (shade == "freq") {
    domain <- c(0, max(shade_values, na.rm = TRUE))
  } else {
    max_abs <- max(abs(shade_values), na.rm = TRUE)
    domain <- c(-max_abs, max_abs)
  }

  # Build gt table
  stub_cols <- if (has_multi_row_vars) row_var_names else rvar
  gt_tbl <- gt::gt(df_display, rowname_col = stub_cols)

  # Apply colors to data cells (not totals)
  n_row <- nrow(shade_values)
  n_col <- ncol(shade_values)

  # Color function for background
  color_fn <- scales::col_numeric(palette = palette, domain = domain)

  # Helper function to determine text color based on background
  # Uses colorspace::contrast_ratio if available, otherwise luminance-based fallback
  get_text_color <- function(bg_color) {
    if (requireNamespace("colorspace", quietly = TRUE)) {
      contrast_white <- colorspace::contrast_ratio(bg_color, "white")
      contrast_black <- colorspace::contrast_ratio(bg_color, "black")
      if (contrast_white > contrast_black) "white" else "black"
    } else {
      rgb_vals <- grDevices::col2rgb(bg_color) / 255
      luminance <- 0.2126 * rgb_vals[1] + 0.7152 * rgb_vals[2] + 0.0722 * rgb_vals[3]
      if (luminance < 0.5) "white" else "black"
    }
  }

  # Apply cell-by-cell coloring with appropriate text color
  for (i in 1:n_row) {
    for (j in 1:n_col) {
      col_name <- cnames[j]
      bg_color <- color_fn(shade_values[i, j])
      text_color <- get_text_color(bg_color)

      gt_tbl <- gt_tbl |>
        gt::tab_style(
          style = list(
            gt::cell_fill(color = bg_color),
            gt::cell_text(color = text_color)
          ),
          locations = gt::cells_body(columns = col_name, rows = i)
        )
    }
  }

  # Style the totals row and column
  if (show_margins) {
    gt_tbl <- gt_tbl |>
      gt::tab_style(
        style = gt::cell_fill(color = "#D1FFBD"),
        locations = gt::cells_body(columns = "Total")
      )

    gt_tbl <- gt_tbl |>
      gt::tab_style(
        style = gt::cell_fill(color = "#D1FFBD"),
        locations = gt::cells_body(rows = n_row + 1)
      ) |>
      gt::tab_style(
        style = gt::cell_fill(color = "#D1FFBD"),
        locations = gt::cells_stub(rows = n_row + 1)
      )
  }


  # Check for multi-variable column structure (from structable attributes)
  col_vars <- attr(x, "col_vars")
  has_multi_col_vars <- !is.null(col_vars) && length(col_vars) > 1

  if (has_multi_col_vars) {
    # Build hierarchical column spanners from col_vars structure
    col_var_names <- names(col_vars)
    n_col_vars <- length(col_var_names)

    # Exclude "Total" column from spanner processing if present
    data_cnames <- if (show_margins) setdiff(cnames, "Total") else cnames

    # Split composite column names (e.g., "Yes_Yes" -> c("Yes", "Yes"))
    split_cnames <- strsplit(data_cnames, "_", fixed = TRUE)

    # Build spanners from outermost (first col_var) to innermost (last col_var)
    # The LAST col_var becomes the column labels, so we create spanners for all but the last
    for (level in seq_len(n_col_vars - 1)) {
      # Get the value at this level for each column
      level_values <- sapply(split_cnames, `[`, level)

      # Group consecutive columns with the same value
      rle_result <- rle(level_values)

      end_pos <- cumsum(rle_result$lengths)
      start_pos <- c(1, end_pos[-length(end_pos)] + 1)

      for (i in seq_along(rle_result$values)) {
        group_cols <- data_cnames[start_pos[i]:end_pos[i]]
        spanner_label <- rle_result$values[i]
        # Use a unique id to avoid collisions when the same label appears
        # in multiple groups (e.g., "Yes" under both "Divorced" and "Married")
        spanner_id <- paste0("spanner_l", level, "_", i)

        gt_tbl <- gt_tbl |>
          gt::tab_spanner(
            label = spanner_label,
            id = spanner_id,
            columns = dplyr::all_of(group_cols),
            level = n_col_vars - level  # Higher level number = more outer
          )
      }
    }

    # Add variable name as the top-level spanner
    gt_tbl <- gt_tbl |>
      gt::tab_spanner(
        label = col_var_names[1],
        columns = dplyr::all_of(data_cnames),
        level = n_col_vars
      )

    # Rename column labels to show only the innermost variable value
    inner_labels <- sapply(split_cnames, function(x) x[length(x)])
    rename_list <- stats::setNames(inner_labels, data_cnames)
    gt_tbl <- gt_tbl |> gt::cols_label(.list = rename_list)

  } else {
    # Single column variable - use simple spanner with the variable name
    gt_tbl <- gt_tbl |>
      gt::tab_spanner(
        label = cvar,
        columns = dplyr::all_of(cnames)
      )
  }

  # Add title if provided
  if (!is.null(title)) {
    gt_tbl <- gt_tbl |> gt::tab_header(title = title)
  }

  # Add styling
  gt_tbl <- gt_tbl |>
    gt::tab_options(
      table.font.size = 14,
      column_labels.font.weight = "bold",
      row_group.font.weight = "bold"
    ) |>
    gt::cols_align(align = "center", columns = dplyr::everything())

  # Style row labels (stub) to match column label styling
  # Bold for category labels, italic for Total
  if (show_margins) {
    gt_tbl <- gt_tbl |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_stub(rows = seq_len(n_row))
      )

    gt_tbl <- gt_tbl |>
      gt::tab_style(
        style = gt::cell_text(style = "italic"),
        locations = gt::cells_body(columns = "Total")
      ) |>
      gt::tab_style(
        style = gt::cell_text(style = "italic"),
        locations = gt::cells_column_labels(columns = "Total")
      )

    gt_tbl <- gt_tbl |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold", style = "italic"),
        locations = gt::cells_stub(rows = n_row + 1)
      ) |>
      gt::tab_style(
        style = gt::cell_text(style = "italic"),
        locations = gt::cells_body(rows = n_row + 1)
      )
  } else {
    gt_tbl <- gt_tbl |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_stub()
      )
  }

  # Add source note legend if requested

  show_legend <- isTRUE(legend) || identical(legend, "note")
  if (show_legend && !is.null(shading_message)) {
    gt_tbl <- gt_tbl |> gt::tab_source_note(source_note = shading_message)
  }

  # Save to file if filename is provided
  if (!is.null(filename)) {
    gt::gtsave(gt_tbl, filename = filename, ...)
  }

  return(gt_tbl)
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/mobility-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# colorize text
colorize <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}

## ----hauser-data--------------------------------------------------------------
data("Hauser79", package="vcdExtra")
str(Hauser79)
(Hauser_tab <- xtabs(Freq ~ Father + Son, data=Hauser79))

## ----load---------------------------------------------------------------------
library(vcdExtra)
library(gnm)
library(dplyr)

## ----mosaicplot---------------------------------------------------------------
plot(Hauser_tab, shade=TRUE)

## ----mosaic1------------------------------------------------------------------
labels <- list(set_varnames = c(Father="Father's occupation", 
                                Son="Son's occupation"))

mosaic(Freq ~ Father + Son, data=Hauser79,
       labeling_args = labels,
       shade=TRUE,
       legend = FALSE)

## ----indep--------------------------------------------------------------------
hauser.indep <- glm(Freq ~ Father + Son, 
  data=Hauser79, 
  family=poisson)

# the same mosaic, using the fitted model
mosaic(hauser.indep, formula = ~ Father + Son, 
       labeling_args = labels,
       legend = FALSE,
       main="Independence model")

## ----Diag---------------------------------------------------------------------
# with symbols
with(Hauser79, Diag(Father, Son)) |> matrix(nrow=5)

## ----quasi--------------------------------------------------------------------
hauser.quasi <-  update(hauser.indep, 
                        ~ . + Diag(Father, Son))

mosaic(hauser.quasi, ~ Father+Son, 
       labeling_args = labels,
       legend = FALSE,
       main="Quasi-independence model")


## ----symm---------------------------------------------------------------------
with(Hauser79, Symm(Father, Son)) |> matrix(nrow=5)

## ----qsymm--------------------------------------------------------------------
hauser.qsymm <-  update(hauser.indep, 
                        ~ . + Diag(Father,Son) + Symm(Father,Son))

## ----anova1-------------------------------------------------------------------
anova(hauser.indep, hauser.quasi, hauser.qsymm, test="Chisq")

LRstats(hauser.indep, hauser.quasi, hauser.qsymm)

## ----qsymm-mosaic-------------------------------------------------------------
mosaic(hauser.qsymm, ~ Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Quasi-symmetry model")

## ----topo-levels--------------------------------------------------------------
# Levels for Hauser 5-level model
levels <- matrix(c(
      2,  4,  5,  5,  5,
      3,  4,  5,  5,  5,
      5,  5,  5,  5,  5,
      5,  5,  5,  4,  4,
      5,  5,  5,  4,  1), 
      nrow = 5, ncol = 5, 
      byrow=TRUE)

## ----topo-mosaic--------------------------------------------------------------
hauser.topo <- update(hauser.indep, 
                      ~ . + Topo(Father, Son, spec=levels))

mosaic(hauser.topo, ~Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Topological model")

## -----------------------------------------------------------------------------
LRstats(hauser.indep, hauser.quasi, hauser.qsymm, hauser.topo, sortby = "AIC")


## ----scores-------------------------------------------------------------------
Sscore <- as.numeric(Hauser79$Son)
Fscore <- as.numeric(Hauser79$Father)

Hauser79 |> cbind(Fscore, Fscore) |> head()

## ----hauser-UAdiag------------------------------------------------------------
hauser.UAdiag <- update(hauser.indep,
                        . ~ . + Fscore : Sscore + Diag(Father, Son))

LRstats(hauser.UAdiag)

## -----------------------------------------------------------------------------
coef(hauser.UAdiag)[["Fscore:Sscore"]]

## ----UAdiag-mosaic------------------------------------------------------------
mosaic(hauser.UAdiag, ~ Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Uniform association + Diag()")

# The test data is from the CDISC Pilot data set,
# which is publicly available on the PHUSE Test Data Factory repository.

library(dplyr)
url <- "https://github.com/phuse-org/TestDataFactory/raw/main/Updated/TDF_ADaM/adcibc.xpt"
adcbic <- haven::read_xpt(url) %>%
  dplyr::filter(
    EFFFL == "Y",
    ITTFL == "Y",
    AVISIT == "Week 8",
    ANL01FL == "Y"
  )
write.csv(
  adcbic,
  file = testthat::test_path("data", "adcibc.csv")
)

# Print method for Kappa: Add a column showing z values
## DONE: now set digits
## DONE: now include CI


#' Print Kappa
#'
#' This is a replacement for the `print.Kappa` method in `vcd`,
#' adding display of `z` values to the `vcd` version and optional
#' confidence intervals.
#'
#'
#' @param x A Kappa object
#' @param digits number of digits to print
#' @param CI Include confidence intervals in the display?
#' @param level confidence level
#' @param \dots Other arguments
#' @return Returns the Kappa object, invisibly.
#' @author Michael Friendly
#' @seealso \code{\link[vcd]{confint.Kappa}}
#' @keywords htest category
#' @export
#' @examples
#'
#' data("SexualFun")
#' Kappa(SexualFun)
#' print(Kappa(SexualFun), CI=TRUE)
#'
#' # stratified 3-way table
#' apply(MSPatients, 3, Kappa)
#'
print.Kappa <-
		function (x, digits=max(getOption("digits") - 3, 3), CI=FALSE, level=0.95, ...)
{
	tab <- rbind(x$Unweighted, x$Weighted)
	z <- tab[,1] / tab[,2]
	tab <- cbind(tab, z)
	if (CI) {
		q <-  qnorm((1 + level)/2)
		lower <- tab[,1] - q * tab[,2]
		upper <- tab[,1] + q * tab[,2]
		tab <- cbind(tab, lower, upper)
	}

	rownames(tab) <- names(x)[1:2]
	print(tab, digits=digits, ...)
	invisible(x)
}

# split a 3D object along dimension dim, according to the proportions or
# frequencies specified in vector p



#' Subdivide a 3D Object
#'
#' Subdivides a `shape3d` object or a list of `shape3d` objects into
#' objects of the same shape along a given dimension according to the
#' proportions or frequencies specified in vector(s).
#'
#' `split3d` is the basic workhorse used in \code{\link{mosaic3d}}, but
#' may be useful in other contexts.
#'
#' `range3d` and `center3d` are utility functions, also useful in
#' other contexts.
#'
#' The resulting list of `shape3d` objects is actually composed of
#' *copies* of the input object(s), scaled according to the proportions in
#' `p` and then translated to make their range along the splitting
#' dimension equal to that of the input object(s).
#'
#' @aliases split3d split3d.shape3d split3d.list range3d center3d
#' @param obj A `shape3d` object, or a list composed of them
#' @param \dots Other arguments for split3d methods
#' @param p For a single `shade3d` object, a vector of proportions (or a
#' vector of non-negative numbers which will be normed to proportions)
#' indicating the number of subdivisions and their scaling along dimension
#' `dim`. For a list of `shade3d` objects, a matrix whose columns
#' indicate the subdivisions of each object.
#' @param dim The dimension along which the object is to be subdivided. Either
#' an integer: 1, 2, or 3, or a character: "x", "y", or "z".
#' @param space The total space used to separate the copies of the object along
#' dimension `dim`. The unit inter-object space is therefore
#' `space/(length(p)-1)`.
#' @return `split3d` returns a list of `shape3d` objects.
#'
#' `range3d` returns a 2 x 3 matrix, whose first row contains the minima
#' on dimensions x, y, z, and whose second row contains the maxima.
#'
#' `center3d` returns a numeric vector containing the means of the minima
#' and maxima on dimensions x, y, z.
#' @author Duncan Murdoch, with refinements by Michael Friendly
#' @seealso \code{\link{mosaic3d}}
#'
#' \code{\link[rgl]{shapelist3d}} for the plotting of lists of `shape3d`
#' objects.
#' @concept 3D
#' @keywords dplot
#' @examples
#'
#' if (!rgl::rgl.useNULL() && interactive()){
#'   open3d()
#'   cube <- cube3d(alpha=0.4)
#'   sl1 <- split3d(cube, c(.2, .3, .5), 1)
#'   col <- c("#FF000080", "#E5E5E580", "#0000FF80")
#'   shapelist3d(sl1, col=col)
#'
#'   open3d()
#'   p <- matrix(c(.6, .4, .5, .5, .2, .8), nrow=2)
#'   sl2 <- split3d(sl1, p, 2)
#'   shapelist3d(sl2, col=col)
#'   }
#'
#' @export split3d
split3d <- function(obj, ...) {
	UseMethod("split3d")
}

#' @rdname split3d
#' @export
split3d.shape3d <- function(obj, p, dim, space=.10, ...) {
	range <-range3d(obj)
	min <- range[1,]
	p <- p/sum(p)                 # assure proportions
	uspace <- space/(length(p)-1) # unit space between objects
	scales <- p * (1-space)
	shifts <- c(0, cumsum(p)[-length(p)])*diff(range[,dim])
	result <- list()
	for (i in seq_along(p)) {
		xscale <- yscale <- zscale <- 1
		xshift <- yshift <- zshift <- 0

		if (dim == 1 || tolower(dim)=='x') {
			xscale <- scales[i]
			xshift <- shifts[i] + min[1]*(1-xscale) + (uspace * (i-1))
		} else if (dim == 2|| tolower(dim)=='y') {
			yscale <- scales[i]
			yshift <- shifts[i] + min[2]*(1-yscale) + (uspace * (i-1))
		} else if (dim == 3|| tolower(dim)=='y') {
			zscale <- scales[i]
			zshift <- shifts[i] + min[3]*(1-zscale) + (uspace * (i-1))
		}

		result[[i]] <- rgl::translate3d(rgl::scale3d(obj, xscale, yscale, zscale),
				xshift, yshift, zshift)

	}
	result
}

# split a list of 3D objects, according to the proportions specified in
# the columns of p.

#' @rdname split3d
#' @export
split3d.list <- function(obj, p, dim, space=.10, ...) {
	nl <- length(obj)
	if (!is.matrix(p) || ncol(p) != nl) stop(gettextf("p must be a matrix with %i columns", nl))
	sl <- list()
	for (i in seq_along(obj)) {
		sl <- c(sl, split3d(obj[[i]], p[,i], dim=dim, space=space))
	}
	sl
}

#range3d <- function(obj, ...) {
#	UseMethod("range3d")
#}

#' @rdname split3d
#' @export
range3d <- function(obj) {
	if (!"vb" %in% names(obj)) stop("Not a mesh3d or shape3d object")
  x <- with(obj, range(vb[1,]/vb[4,]))
  y <- with(obj, range(vb[2,]/vb[4,]))
  z <- with(obj, range(vb[3,]/vb[4,]))
  result <- cbind(x,y,z)
  rownames(result)<- c('min', 'max')
  result
}

#' @rdname split3d
#' @export
center3d <- function(obj) {
	range <-range3d(obj)
	colMeans(range)
}

# summarise a glm object or glmlist

summarise <- function(...) {
  .Deprecated("LRstats")
  LRstats(...)
}

# summarise <- function(object, ...) {
# 	UseMethod("summarise")
# }
# 
# stat.summarise <- function(deviance, df, onames, n) {
# 	p <- pchisq(deviance, df, lower.tail=FALSE)
# 	aic <- deviance - 2*df
# 	if (missing(n)) {
# 		result <- data.frame(aic, deviance, df, p)
# 		names(result) <- c("AIC", "LR Chisq", "Df", "Pr(>Chisq)")
# 	}
# 	else {
# 		bic <- deviance - log(n)*df
# 		result <- data.frame(aic, bic, deviance, df, p)
# 		names(result) <- c("AIC", "BIC", "LR Chisq", "Df", "Pr(>Chisq)")
# 	}
# 	
# 	rownames(result) <- onames
# 	attr(result, "heading") <- "Model Summary:"
# 	class(result) <- c("anova", "data.frame")
# 	result
# }
# 
# 
# summarise.glm <-function(object, ..., test=NULL){
# 	dotargs <- list(...)
# 	is.glm <- unlist(lapply(dotargs, function(x) inherits(x, "glm")))
# 	dotargs <- dotargs[is.glm]
# 	if (length(dotargs)) 
# 		return(summarise.glmlist(c(list(object), dotargs), test = test))
# 	
# 	oname <- as.character(sys.call())[2]
# 	result <- stat.summarise(object$deviance, object$df.residual, oname, sum(fitted(object)))
# 	result
# }
# 
# summarise.glmlist <-function(object, ..., test=NULL, sortby=NULL){
# 	nmodels <- length(object)
# 	if (nmodels == 1) 
# 		return(summarise.glm(object[[1]], test = test))
# 	if (is.null(names(object))) {
# 		oname <- as.character(sys.call())[-1]
# 		oname <- oname[1:length(object)]
# 	}
# 	else oname <- names(object)
# 	
# 	resdf <- as.numeric(lapply(object, function(x) x$df.residual))
# 	resdev <- as.numeric(lapply(object, function(x) x$deviance))
# 	n <- as.numeric(lapply(object, function(x) sum(fitted(x))))
# 	result <- stat.summarise(resdev, resdf, oname, n)
# 	if (!is.null(sortby)) {
# 		result <- result[order(result[,sortby], decreasing=TRUE),]
# 	}
# 	result
# }
# 
# 
# summarise.loglm <-function(object, ...){
# 	dotargs <- list(...)
# 	is.loglm <- unlist(lapply(dotargs, function(x) inherits(x, "loglm")))
# 	dotargs <- dotargs[is.loglm]
# 	if (length(dotargs)) 
# 		return(summarise.loglmlist(c(list(object), dotargs)))
# 	
# 	oname <- as.character(sys.call())[2]
# 	result <- stat.summarise(object$deviance, object$df, oname, sum(fitted(object)))
# 	result
# }
# 
# summarise.loglmlist <-function(object, ..., sortby=NULL){
# 	nmodels <- length(object)
# 	if (nmodels == 1) 
# 		return(summarise.loglm(object[[1]]))
# 	if (is.null(names(object))) {
# 		oname <- as.character(sys.call())[-1]
# 		oname <- oname[1:length(object)]
# 	}
# 	else oname <- names(object)
# 	
# 	resdf <- as.numeric(lapply(object, function(x) x$df))
# 	resdev <- as.numeric(lapply(object, function(x) x$deviance))
# 	n <- as.numeric(lapply(object, function(x) sum(fitted(x))))
# 	result <- stat.summarise(resdev, resdf, oname, n)
# 	if (!is.null(sortby)) {
# 		result <- result[order(result[,sortby], decreasing=TRUE),]
# 	}
# 	result
# }
# 

#####################################
## Produce a 3D mosaic plot using rgl
#####################################

# TODO: provide formula interface
# TODO: handle zero margins (causes display to be erased in shapelist3d)
# DONE: handle zero cells
# DONE: generalize the calculation of residuals
# DONE: allow display of type=c("observed", "expected")
# DONE: if ndim>3, provide for labels at max or min
# DONE: make object oriented and provide a loglm method

# mosaic3d: provide observed array of counts and either residuals, expected frequencies,
# or a loglin set of margins to fit



#'
#' 3D Mosaic Plots
#'
#' Produces a 3D mosaic plot for a contingency table (or a
#' \code{link[MASS]{loglm}} model) using the \code{\link[rgl]{rgl-package}}.
#'
#' Generalizing the 2D mosaic plot, this begins with a given 3D shape (a unit
#' cube), and successively sub-divides it along the X, Y, Z dimensions
#' according to the table margins, generating a nested set of 3D tiles. The
#' volume of the resulting tiles is therefore proportional to the frequency
#' represented in the table cells. Residuals from a given loglinear model are
#' then used to color or shade each of the tiles.
#'
#' This is a developing implementation.  The arguments and details are subject
#' to change.
#'
#' Friendly (1995), Friendly (2000, Sect. 4.5) and Theus and Lauer (1999) have
#' all used the idea of 3D mosaic displays to explain various aspects of
#' loglinear models (the iterative proportional fitting algorithm, the
#' structure of various models for 3-way and n-way tables, etc.), but no
#' implementation of 3D mosaics was previously available.
#'
#' For the default method, residuals, used to color and shade the 3D tiles, can
#' be passed explicitly, or, more typically, are computed as needed from
#' observed and expected frequencies. In this case, the expected frequencies
#' are optionally computed for a specified loglinear model given by the
#' `expected` argument. For the loglm method, residuals and observed
#' frequencies are calculated from the model object.
#'
#' @aliases mosaic3d mosaic3d.default mosaic3d.loglm
#'
#' @param x A \code{link[MASS]{loglm}} model object. Alternatively, a multidimensional `array` or `table`
#'          or\code{\link[vcd]{structable}} of frequencies in a contingency table.  In the present implementation, the dimensions
#'          are taken in sequential order. Use \code{link[base]{aperm}} or \code{\link[vcd]{structable}} to change this.
#' @param expected optionally, for contingency tables, an array of expected frequencies of the same dimension as `x`, or alternatively the
#'          corresponding loglinear model specification as used by\code{link[stats]{loglin}} or \code{link[MASS]{loglm}} (see
#'          \code{\link[vcd]{structable}} for details).
#' @param residuals optionally, an array of residuals of the same dimension as `x` (see details).
#' @param type a character string indicating whether the `"observed"` or the `"expected"` frequencies in the table should be
#'          visualized by the volume of the 3D tiles.
#' @param residuals_type a character string indicating the type of residuals to be computed when none are supplied.  If residuals is `NULL`,
#'          `residuals_type` must be one of `"pearson"` (default; giving components of Pearson's chi-squared), `"deviance"`
#'          (giving components of the likelihood ratio chi-squared), or `"FT"` for the Freeman-Tukey residuals.  The value of this
#'          argument can be abbreviated.
#' @param shape The initial 3D shape on which the mosaic is based.  Typically this is a call to an rgl function, and must produce a `shape3d` object.
#'          The default is a "unit cube" on (-1, +1), with transparency specified by `alpha`.
#' @param alpha Specifies the transparency of the 3D tiles used to compose the 3D mosaic.
#' @param spacing A number or vector giving the total amount of space used to separate the 3D tiles along each of the
#'          dimensions of the table. The values specified are re-cycled to the number of table dimensions.
#' @param split_dir A numeric vector composed of the integers `1:3` or a character vector composed of `c("x", "y", "z")`, where
#'          `split_dir[i]` specifies the axis along which the tiles should be split
#'          for dimension `i` of the table. The values specified are re-cycled to the number of table dimensions.
#' @param shading A function, taking an array or vector of residuals for the
#'          given model, returning a vector of colors.  At present, only the default `shading=shading_basic` is provided.
#'          This is roughly equivalent to the
#'          use of the `shade` argument in \code{\link[graphics]{mosaicplot}} or to
#'          the use of `gp=shading_Friendly` in \code{\link[vcd]{mosaic}}.
#' @param interpolate a vector of interpolation values for the `shading` function.
#' @param zero_size The radius of a small sphere used to mark zero cells in the display.
#' @param label_edge A character vector composed of `c("-", "+")` indicating whether the labels for a given table dimension
#'          are to be written at the minima (`"-"`) or maxima (`"+"`) of the *other*
#'          dimensions in the plot. The default is `rep( c('-', '+'), each=3, length=ndim)`, meaning that the first three
#'          table variables are labeled at the minima, and successive ones at the maxima.
#' @param labeling_args This argument is intended to be used to specify details of the rendering of labels for the table dimensions, but
#'          at present has no effect.
#' @param newpage logical indicating whether a new page should be created for the plot or not.
#' @param box logical indicating whether a bounding box should be drawn around the plot.
#' @param \dots Other arguments passed down to `mosaic.default` or 3D functions.
#'
#' @return Invisibly, the list of `shape3d` objects used to draw the 3D
#' mosaic, with names corresponding to the concatenation of the level labels,
#' separated by ":".
#'
#' @author Michael Friendly, with the help of Duncan Murdoch and Achim Zeileis
#'
#' @seealso
#' \code{\link[vcd]{strucplot}}, \code{\link[vcd]{mosaic}},
#' \code{\link[graphics]{mosaicplot}} for aspects of mosaic plots.
#'
#' \code{\link[stats]{loglin}}, \code{\link[MASS]{loglm}} for details on
#' fitting loglinear models
#'
#' \code{\link[rgl]{play3d}}, \code{\link[rgl]{movie3d}} for what you can do with a 3D mosaic plot
#' @family mosaic plots
#'
#' @references Friendly, M. (1995). Conceptual and Visual Models for
#' Categorical Data, *The American Statistician*, **49**, 153-160.
#'
#' Friendly, M. *Visualizing Categorical Data*, Cary NC: SAS Institute,
#' 2000. Web materials: <http://www.datavis.ca/books/vcd/>.
#'
#' Theus, M. & Lauer, S. R. W. (1999) Visualizing Loglinear Models.
#' *Journal of Computational and Graphical Statistics*, **8**, 396-412.
#' @concept 3D
#' @keywords hplot
#' @examples
#'
#' # 2 x 2 x 2
#' if (!rgl::rgl.useNULL() && interactive()){
#' mosaic3d(Bartlett, box=TRUE)
#' # compare with expected frequencies under model of mutual independence
#' mosaic3d(Bartlett, type="expected", box=TRUE)
#'
#' # 2 x 2 x 3
#' mosaic3d(Heart, box=TRUE)
#' }
#'
#' # Make a dynamic display
#' \dontrun{
#'   mosaic3d(Heart, box=TRUE, alpha = 0.3, interpolate = c(1, 2, 4))
#'   play3d(spin3d(axis = c(0, 1, 0), rpm = 10), duration = 5)
#' }
#'
#' \dontrun{
#' # 2 x 2 x 2 x 3
#' # illustrates a 4D table
#' mosaic3d(Detergent)
#'
#' # compare 2D and 3D mosaics
#' #demo("mosaic-hec")
#' }
#'
#'
#' @export
mosaic3d <- function(x, ...) {
	UseMethod("mosaic3d")
}

#' @rdname mosaic3d
#' @export
mosaic3d.loglm <-
function (x, type = c("observed", "expected"),
    residuals_type = c("pearson", "deviance"),
#    gp = shading_hcl, gp_args = list(),
    ...)
{
    residuals_type <- match.arg(tolower(residuals_type), c("pearson", "deviance"))
    if (is.null(x$fitted))
        x <- update(x, fitted = TRUE)
    expected <- fitted(x)
    residuals <- residuals(x, type = "pearson")
    observed <- residuals * sqrt(expected) + expected
    if (residuals_type == "deviance")
        residuals <- residuals(x, type = "deviance")
#    gp <- if (inherits(gp, "grapcon_generator"))
#        do.call("gp", c(list(observed, residuals, expected, x$df),
#            as.list(gp_args)))
#    else gp
    mosaic3d.default(observed, residuals = residuals, expected = expected,
        type = type, residuals_type = residuals_type,
#        gp = gp,
        ...)
}

#' @rdname mosaic3d
#' @importFrom vcd is.structable
#' @export
mosaic3d.default <- function(x, expected=NULL, residuals=NULL,
		type = c("observed", "expected"), residuals_type = NULL,
		shape=rgl::cube3d(alpha=alpha), alpha=0.5,
		spacing=0.1, split_dir=1:3,
		shading=shading_basic, interpolate=c(2,4), zero_size=.05,
		label_edge,
		labeling_args=list(), newpage=TRUE, box=FALSE, ...) {

  if (!requireNamespace("rgl")) stop("rgl is required")

  type <- match.arg(type)
  if (is.null(residuals)) {
      residuals_type <- if (is.null(residuals_type))
          "pearson"
      else match.arg(tolower(residuals_type), c("pearson",
          "deviance", "ft"))
  }

  ## convert structable object
  if (vcd::is.structable(x)) {
    x <- as.table(x)
  }

  ## table characteristics
  levels <- dim(x)
  ndim <- length(levels)
  dn <- dimnames(x)
  if (is.null(dn))
    dn <- dimnames(x) <- lapply(levels, seq)
  vnames <- names(dimnames(x))
  if (is.null(vnames))
    vnames <- names(dn) <- names(dimnames(x)) <- LETTERS[1:ndim]

  ## replace NAs by 0
  if (any(nas <- is.na(x))) x[nas] <- 0

  ## model fitting:
  ## calculate expected if needed
  if ((is.null(expected) && is.null(residuals)) ||
      !is.numeric(expected)) {
      if (inherits(expected, "formula")) {
          fm <- loglm(expected, x, fitted = TRUE)
          expected <- fitted(fm)
          df <- fm$df
      } else {
          if (is.null(expected))
              expected <-  as.list(1:ndim)
          fm <- loglin(x, expected, fit = TRUE, print = FALSE)
          expected <- fm$fit
          df <- fm$df
      }
  }

  ## compute residuals
  if (is.null(residuals))
    residuals <- switch(residuals_type,
                        pearson = (x - expected) / sqrt(ifelse(expected > 0, expected, 1)),
                        deviance = {
                          tmp <- 2 * (x * log(ifelse(x == 0, 1, x / ifelse(expected > 0, expected, 1))) - (x - expected))
                          tmp <- sqrt(pmax(tmp, 0))
                          ifelse(x > expected, tmp, -tmp)
                        },
                        ft = sqrt(x) + sqrt(x + 1) - sqrt(4 * expected + 1)
                        )
  ## replace NAs by 0
  if (any(nas <- is.na(residuals))) residuals[nas] <- 0

	# switch observed and expected if required
  observed <- if (type == "observed") x else expected
  expected <- if (type == "observed") expected else x


	# replicate arguments to number of dimensions
	spacing <- rep(spacing, length=ndim)
	split_dir <- rep(split_dir, length=ndim)
	if(missing(label_edge))
		label_edge <- rep( c('-', '+'), each=3, length=ndim)

	zeros <- observed <= .Machine$double.eps

	shapelist <- shape
	# sanity check
	if (!inherits(shapelist, "shape3d")) stop("shape must be a shape3d object")

	if (newpage) rgl::open3d()
	for (k in 1:ndim) {
		marg <- margin.table(observed, k:1)
		if (k==1) {
			shapelist <- split3d(shapelist, marg, split_dir[k], space=spacing[k])
			label3d(shapelist, split_dir[k], dn[[k]], vnames[k], edge=label_edge[k], ...)
		}
		else {
			marg <- matrix(marg, nrow=levels[k])
			shapelist <- split3d(shapelist, marg, split_dir[k], space=spacing[k])
			names(shapelist) <- apply(as.matrix(expand.grid(dn[1:k])), 1, paste, collapse=":")
			L <- length(shapelist)
			label_cells <- if (label_edge[k]=='-') 1:levels[k]
											else (L-levels[k]+1):L
			label3d(shapelist[label_cells], split_dir[k], dn[[k]], vnames[k], edge=label_edge[k], ...)
		}
	}

	# assign colors
	# TODO: allow alpha to control transparency of side walls
	col <- shading(residuals, interpolate=interpolate)

	# display, but exclude the zero cells
	rgl::shapelist3d(shapelist[!as.vector(zeros)], col=col[!as.vector(zeros)], ...)

	# plot markers for zero cells
	if (any(zeros)) {
		ctrs <- t(sapply(shapelist, center3d))
		rgl::spheres3d(ctrs[as.vector(zeros),], radius=zero_size)
	}

#	invisible(structable(observed))
	invisible(shapelist)

}


# basic shading_Friendly, adapting the simple code used in mosaicplot()

shading_basic <- function(residuals, interpolate=TRUE) {
	if (is.logical(interpolate))
		interpolate <- c(2, 4)
	else if (any(interpolate <= 0) || length(interpolate) > 5)
		stop("invalid 'interpolate' specification")
	shade <- sort(interpolate)
	breaks <- c(-Inf, -rev(shade), 0, shade, Inf)
	colors <- c(hsv(0, s = seq.int(1, to = 0, length.out = length(shade) +
									1)), hsv(4/6, s = seq.int(0, to = 1, length.out = length(shade) +
									1)))
	colors[as.numeric(cut(residuals, breaks))]
}


# provide labels for 3D objects below/above their extent along a given dimension
# FIXME: kludge for interline gap between level labels and variable name
# TODO: how to pass & extract labeling_args, e.g., labeling_args=list(at='min', fontsize=10)

label3d <- function(objlist, dim, text, varname, offset=.05, adj, edge="-", gap=.1, labeling_args, ...) {
	if(missing(adj)) {
		if (dim < 3) adj <- ifelse(edge == '-', c(0.5, 1), c(0.5, 0))
		else         adj <- ifelse(edge == '-', c(1, 0.5), c(0, 0.5))
	}
	ranges <- lapply(objlist, range3d)
	loc <- t(sapply(ranges, colMeans))   # positions of labels on dimension dim
	min <- t(sapply(ranges, function(x) x[1,]))  # other dimensions at min values
	max <- t(sapply(ranges, function(x) x[2,]))  # other dimensions at max values
	xyz <- if (edge == '-') (min - offset) else (max + offset)
	xyz[,dim] <- loc[,dim]
	if(!missing(varname)) {
		loclab <- colMeans(loc)               # NB: doesn't take space into acct
		xyzlab <- if (edge == '-')
			min[1,] - offset - gap
			else max[1,] + offset + gap
		xyzlab[dim] <- loclab[dim]
		xyz <- rbind(xyz, xyzlab)
		text <- c(text, varname)
	}
	result <- c(labels = rgl::texts3d(xyz, texts=text, adj=adj, ...))
	invisible(result)
}

# calculate bivariate logits and OR


#' Bivariate Logits and Log Odds Ratio
#' 
#' This function calculates the log odds and log odds ratio for two binary
#' responses classified by one or more stratifying variables.
#' 
#' It is useful for plotting the results of bivariate logistic regression
#' models, such as those fit using \code{\link[VGAM]{vglm}} in the \pkg{VGAM}.
#' 
#' For two binary variables with levels 0,1 the logits are calculated assuming
#' the columns in `Y` are given in the order 11, 10, 01, 00, so the logits
#' give the log odds of the 1 response compared to 0.  If this is not the case,
#' either use `rev=TRUE` or supply `Y[,4:1]` as the first argument.
#' 
#' @param Y A four-column matrix or data frame whose columns correspond to the
#' 2 x 2 combinations of two binary responses.
#' @param add Constant added to all cells to allow for zero frequencies.  The
#' default is 0.5 if `any(Y)==0` and 0 otherwise.
#' @param colnames Names for the columns of the results. The default is
#' `c("logit1", "logit2", "logOR")`.  If less than three names are
#' supplied, the remaining ones are filled in from the default.
#' @param row.vars A data frame or matrix giving the factor levels of one or
#' more factors corresponding to the rows of `Y`
#' @param rev A logical, indicating whether the order of the columns in
#' `Y` should be reversed.
#' @return A data frame with `nrow(Y)` rows and `3 + ncol(row.vars)`
#' columns
#' @author Michael Friendly
#' @seealso \code{\link[VGAM]{vglm}}
#' @references Friendly, M. and Meyer, D. (2016).  *Discrete Data Analysis
#' with R: Visualization and Modeling Techniques for Categorical and Count
#' Data*.  Boca Raton, FL: Chapman & Hall/CRC. <http://ddar.datavis.ca>.
#' @keywords manip
#' @examples
#' 
#' data(Toxaemia)
#' tox.tab <- xtabs(Freq~class + smoke + hyper + urea, Toxaemia)
#' 
#' # reshape to 4-column matrix
#' toxaemia <- t(matrix(aperm(tox.tab), 4, 15))
#' colnames(toxaemia) <- c("hu", "hU", "Hu", "HU")
#' rowlabs <- expand.grid(smoke=c("0", "1-19", "20+"), class=factor(1:5))
#' toxaemia <- cbind(toxaemia, rowlabs)
#' 
#' # logits for H and U
#' logitsTox <- blogits(toxaemia[,4:1], 
#'                      add=0.5, 
#'                      colnames=c("logitH", "logitW"), 
#'                      row.vars=rowlabs)
#' logitsTox
#' 
#' 
#' @export blogits
blogits <- function(Y, add, colnames, row.vars, rev=FALSE) {

  if (ncol(Y) != 4) stop("Y must have 4 columns")
  if (missing(add)) add <- if (any(Y==0)) 0.5 else 0
  Y <- Y + add
  if (rev) Y <- Y[,4:1]
  L <- matrix(0, nrow(Y), 3)
  L[,1] <- log( (Y[,1] + Y[,2]) / (Y[,3] + Y[,4]) )
  L[,2] <- log( (Y[,1] + Y[,3]) / (Y[,2] + Y[,4]) )
  L[,3] <- log( (Y[,1] * Y[,4]) / ((Y[,2] * Y[,3])) )
  cn <- c("logit1", "logit2", "logOR")
  colnames(L) <- if(missing(colnames)) cn else c(colnames, cn[-(1:length(colnames))])
  if(!missing(row.vars)) L <- cbind(L, row.vars)
  L
}

#' Convert frequency, case, or table form data into an array
#' 
#' Converts object (`obj`) in frequency, case or table form into an array. The 
#' column containing the frequencies (`freq`) must be supplied if `obj` is in 
#' frequency form.
#' 
#' @param obj 
#'  Object to be converted to an array.
#' @param freq 
#'  If `obj` is in frequency form, this is the name of the frequency column. 
#'  Leave as `NULL` if `obj` is in any other form.
#' @param dims 
#'  A character vector of dimensions. If not specified, all variables apart from 
#'  `freq` will be used as dimensions.
#' @param prop 
#'  If set to `TRUE`, returns an array of proportions (that sum to 1). May also 
#'  be set to a character or numeric vector of dimensions to be used as margins 
#'  from which proportions will be computed.
#' @return object in array form
#' 
#' @details
#' Unclasses the \code{\link{as_table}} function to return an object in array form.
#' 
#' @author Gavin M. Klorfine
#' 
#' @seealso
#' \code{\link{as_table}}, \code{\link{as_freqform}}, \code{\link{as_caseform}},
#' \code{\link{as_matrix}}
#' 
#' @examples
#' library(vcdExtra)
#' 
#' data("HairEyeColor")
#' 
#' freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
#' tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
#' caseForm <- expand.dft(freqForm) # Generate case form data
#' 
#' # Frequency form -> array form
#' as_array(freqForm, freq = "Freq") |> str()
#' 
#' # Warned if forgot to specify freq
#' as_array(freqForm) |> str()
#' 
#' # Case form -> array form
#' as_array(caseForm) |> str()
#' 
#' # Frequency (tibble) form -> array form
#' as_array(tidy_freqForm, freq = "n") |> str()
#' 
#' # For specific dimensions
#' as_array(tidy_freqForm, freq = "n", dims = c("Hair", "Eye")) |> str()
#' 
#' #-----For proportions-----#
#' 
#' as_array(freqForm, freq = "Freq", prop = TRUE) |> # proportions relative to grand total
#'   head(c(4,4,1)) 
#' 
#' # Marginalize proportions along "Sex" (i.e., male proportions sum to 1, female proportions sum to 1)
#' as_array(freqForm, freq = "Freq", prop = "Sex") |> head(c(4,4,1))
#' 
#' as_array(freqForm, freq = "Freq", prop = 3) |> head(c(4,4,1)) # Same as above
#' 
#' # Marginalize proportions along multiple variables
#' as_array(freqForm, freq = "Freq", prop = c("Hair", "Sex")) |> head(c(4,4,1))
#' 
#' as_array(freqForm, freq = "Freq", prop = c(1, 3)) |> head(c(4,4,1)) # Same as above
#' 
#' # Using dims and prop arguments in tandem
#' as_array(freqForm, freq = "Freq", dims = c("Hair", "Eye"), prop = TRUE)
#' 
#' 
#' @export

as_array <- function(obj, freq = NULL, dims = NULL, prop = NULL){
  return(unclass(as_table(obj, freq, dims, prop))) # Unclass as_table output
}

data("HairEyeColor")

arrayDat <- unclass(HairEyeColor) # Generate array form data
freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
caseForm <- expand.dft(freqForm) # Generate case form data

test_that("Array form input returns an array", {
  ary <- as_array(arrayDat)
  expect_true(is.array(ary))
})

test_that("Table form input returns an array", {
  ary <- as_array(HairEyeColor)
  expect_true(is.array(ary))
})

test_that("Tibble frequency form input returns an array", {
  ary <- as_array(tidy_freqForm, freq = "n")
  expect_true(is.array(ary))
})

test_that("Frequency form input returns an array", {
  ary <- as_array(freqForm, freq = "Freq")
  expect_true(is.array(ary))
})

test_that("Case form input returns an array", {
  ary <- as_array(caseForm)
  expect_true(is.array(ary))
})

test_that("Check if input was modified", {
  ary <- sum(as_array(HairEyeColor))
  expect_equal(ary, sum(HairEyeColor))
})

### return a data.frame giving brief summaries of data sets in packages
#  package:    a character vector giving the package(s) to look in
#  allClass:   include all classes of the item (TRUE) or just the last class (FALSE)
#  incPackage: include package name in result?
#  maxTitle: maximum length of data set Title

# DONE: Make package="vcdExtra" default
# DONE: Add `ndim` arg to also return number of dimensions

#' Information on Data Sets in Packages
#'
#' The \code{\link[utils]{data}} function is used both to load data sets from
#' packages, and give a display of the names and titles of data sets in one or
#' more packages, however it does not return a result that can be easily used
#' to get additional information about the nature of data sets in packages.
#'
#' The `datasets()` function is designed to produce a more useful summary
#' display of data sets in one or more packages.  It extracts the `class`
#' and dimension information (`dim` or codelength) of each item, and
#' formats these to provide additional descriptors.
#'
#' The requested packages must be installed, and are silently loaded in order
#' to extract `class` and size information.
#'
#' @param package     a character vector giving the package(s) to look in
#' @param allClass    a logical variable. Include all classes of the item (`TRUE`) or just the last class (`FALSE`)?
#' @param incPackage  include the package name in result?
#' @param maxTitle    maximum length of data set Title
#' @param ndim        logical; include the number of dimensions in the result?
#'
#' @return A `data.frame` whose rows correspond to data sets found in `package`.
#'
#' The columns (for a single package) are:
#' \item{Item}{data set name, a character variable}
#' \item{class}{class, the object class of the data set, typically one of `"data.frame"`, `"table"`, `"array"` ...}
#' \item{dim}{an abbreviation of the dimensions of the data set, in a form like `"36x3"` for a data.frame or matrix with 36 rows and 3 columns.}
#' \item{Title}{data set title}
#'
#' @note In Rmd documents, `datasets("package") |> knitr::kable()` can be used
#' to create a more pleasing display.
#'
#' @author Michael Friendly, with R-help from Curt Seeliger
#' @seealso \code{\link[utils]{data}}, \code{\link[knitr]{kable}}
#' @keywords package data
#' @examples
#'
#' datasets("vcdExtra")
#' # datasets(c("vcd", "vcdExtra"))
#' datasets("datasets", maxTitle=50)
#'
#' # just list dataset names in a package
#' datasets("vcdExtra")[,"Item"]
#' datasets("vcd")[,"Item"]
#'
#' # Find one-way tables in vcd (dim doesn't contain "x")
#' datasets("vcd") |>
#'   dplyr::filter(class=="table",
#'                 !stringr::str_detect(dim, "x"))
#'
#'
#' @export datasets
datasets <- function(package = "vcdExtra",
                     allClass=FALSE,
                  	 incPackage=length(package) > 1,
                  	 maxTitle=NULL,
                     ndim = FALSE)
{
	# make sure requested packages are available and loaded
	for (i in seq_along(package)) {
		if (!isNamespaceLoaded(package[i]))
			if (requireNamespace(package[i], quietly=TRUE))
				cat(paste("Loading package:", package[i], "\n"))
			else stop(paste("Package", package[i], "is not available"))
	}

	dsitems <- data(package=package)$results
	wanted <- c('Package', 'Item','Title')
	ds <- as.data.frame(dsitems[,wanted], stringsAsFactors=FALSE)

	getData <- function(x, pkg) {
	  # fix items with " (...)" in names, e.g., "BJsales.lead (BJsales)" in datasets
	  objname <- gsub(" .*", "", x)

	  e <- loadNamespace(pkg)
	  if (!exists(x, envir = e)) {
	    dataname <- sub("^.*\\(", "", x)
	    dataname <- sub("\\)$", "", dataname)
	    e <- new.env()
	    data(list = dataname, package = pkg, envir = e)
	  }
	  get(objname, envir = e)
	}

	getDim <- function(i) {
	  data <- getData(ds$Item[i], ds$Package[i])
		if (is.null(dim(data))) length(data) else paste(dim(data), collapse='x')
	}

	getnDim <- function(i) {
	  data <- getData(ds$Item[i], ds$Package[i])
	  length(dim(data))
	}

	getClass <- function(i) {
	  data <- getData(ds$Item[i], ds$Package[i])
		cl <- class(data)
		if (length(cl)>1 && !allClass) cl[length(cl)] else cl
	}

	ds$ndim <- unlist(lapply(seq_len(nrow(ds)), getnDim ))
	ds$dim <- unlist(lapply(seq_len(nrow(ds)), getDim ))
	ds$class <- unlist(lapply(seq_len(nrow(ds)), getClass ))
	if (!is.null(maxTitle)) ds$Title <- substr(ds$Title, 1, maxTitle)

  # what to return?
	fields <- if(ndim) c('Item','class', 'ndim', 'dim','Title') else c('Item','class','dim','Title')
	if (incPackage) fields <- c("Package", fields)
	# return the requested fields
	ds[fields]

}

#' Extract Model Formulas from Model Objects or Model Lists
#'
#' `get_model()` extracts the model formula or bracket notation from a single
#' \code{loglm} or \code{glm} object.
#'
#' `get_models()` does the same for each model in a
#' \code{loglmlist} or \code{glmlist} object.
#' These are useful for labeling models in summaries and plots.
#'
#' @param x For `get_model()`: a \code{loglm} or \code{glm} object.
#'        For `get_models()`: a \code{glmlist} or \code{loglmlist} object.
#' @param type Type of output: \code{"brackets"} for loglinear bracket notation
#'        (e.g., \code{"[AB] [C]"}), or \code{"formula"} for R formula notation.
#'        For \code{glm} and \code{glmlist} objects, only \code{"formula"} is meaningful.
#' @param abbrev Logical or integer. If \code{TRUE} or a positive integer,
#'        abbreviate factor names to that many characters (default 1 when \code{TRUE}).
#'        Only applies to bracket notation.
#' @param \dots Additional arguments passed to \code{loglin2string}
#'        such as \code{sep} and \code{collapse}.
#'
#' @return For `get_model()`: a character string with the model formula or bracket notation.
#'
#'         For `get_models()`: a named character vector with the model formulas or bracket notations.
#'
#' @details
#' For \code{loglm} objects created by \code{\link{seq_loglm}}, the bracket
#' notation is stored in the \code{model.string} component. For other \code{loglm}
#' objects, it is constructed from the \code{margin} component using
#' \code{\link{loglin2string}}.
#'
#' For \code{glm} objects, the formula is extracted using \code{formula()}.
#'
#' **Model notation**
#'
#' For `loglmlist` objects created by \code{\link{seq_loglm}}, the bracket
#' notation distinguishes between models fit to marginal sub-tables and
#' models fit to the full table. Parentheses are used for marginal
#' sub-tables, e.g., `"(Class) (Sex)"`, while square brackets are used
#' for the full table, e.g., `"[Class,Sex,Age] [Survived]"`.
#'
#' The `abbrev` argument can be used to abbreviate factor names
#' for more compact display, e.g., `"[C,S,A] [S]"`.
#'
#' @seealso \code{\link{glmlist}}, \code{\link{loglmlist}},
#'          \code{\link{loglin2string}}, \code{\link{LRstats}},
#'          \code{\link{seq_mosaic}}
#'
#' @family glmlist functions
#' @family loglinear models
#' @export
#' @examples
#' data(Titanic)
#' tit.joint <- seq_loglm(Titanic, type = "joint")
#'
#' # Single model
#' get_model(tit.joint[[2]])
#' get_model(tit.joint[[4]])
#' get_model(tit.joint[[4]], abbrev = TRUE)
#' get_model(tit.joint[[4]], type = "formula")
#'
#' # Model list
#' get_models(tit.joint)
#' get_models(tit.joint, type = "formula")
#'
#' # With abbreviated factor names
#' get_models(tit.joint, abbrev = TRUE)
#' get_models(tit.joint, abbrev = 2)
#'
get_model <- function(x, type = c("brackets", "formula"), abbrev = FALSE, ...) {
  type <- match.arg(type)

  if (inherits(x, "loglm")) {
    if (type == "brackets") {
      if (!is.null(x$model.string)) {
        result <- x$model.string
      } else if (!is.null(x$margin)) {
        result <- loglin2string(x$margin, ...)
      } else {
        result <- deparse(formula(x))
      }

      # Apply abbreviation if requested
      if (!isFALSE(abbrev)) {
        n <- if (isTRUE(abbrev)) 1 else as.integer(abbrev)
        result <- .abbreviate_brackets(result, n)
      }
    } else {
      # Formula type
      if (!is.null(x$call$formula)) {
        result <- deparse(x$call$formula)
      } else if (!is.null(x$margin)) {
        result <- deparse(loglin2formula(x$margin))
      } else {
        result <- NA_character_
      }
    }
  } else if (inherits(x, "glm")) {
    f <- formula(x)
    if (type == "formula") {
      result <- deparse(f)
    } else {
      result <- deparse(f[[3]])  # RHS only
    }
  } else {
    stop("x must be a 'loglm' or 'glm' object")
  }

  result
}


#' @rdname get_model
#' @export
get_models <- function(x, type = c("brackets", "formula"), abbrev = FALSE, ...) {
  type <- match.arg(type)

  if (inherits(x, "loglmlist")) {
    if (type == "brackets") {
      # Try to get model.string first (set by seq_loglm)
      result <- sapply(x, function(mod) {
        if (!is.null(mod$model.string)) {
          mod$model.string
        } else if (!is.null(mod$margin)) {
          loglin2string(mod$margin, ...)
        } else {
          # Fallback to formula
          deparse(formula(mod))
        }
      })

      # Apply abbreviation if requested
      if (!isFALSE(abbrev)) {
        n <- if (isTRUE(abbrev)) 1 else as.integer(abbrev)
        result <- .abbreviate_brackets(result, n)
      }
    } else {
      # Formula type
      result <- sapply(x, function(mod) {
        if (!is.null(mod$call$formula)) {
          deparse(mod$call$formula)
        } else {
          # Try to construct from margin
          if (!is.null(mod$margin)) {
            deparse(loglin2formula(mod$margin))
          } else {
            NA_character_
          }
        }
      })
    }
  } else if (inherits(x, "glmlist")) {
    # For glmlist, extract formulas
    result <- sapply(x, function(mod) {
      f <- formula(mod)
      # Get just the RHS for cleaner display
      if (type == "formula") {
        deparse(f)
      } else {
        # Try to convert to bracket notation (limited support)
        deparse(f[[3]])  # RHS only
      }
    })
  } else {
    stop("x must be a 'glmlist' or 'loglmlist' object")
  }

  names(result) <- names(x)
  result
}


# Helper function to abbreviate factor names in bracket notation
.abbreviate_brackets <- function(strings, n = 1) {
  sapply(strings, function(s) {
    # Find all bracketed terms: [content] or (content)
    m <- gregexpr("([\\[\\(])([^\\]\\)]+)([\\]\\)])", s, perl = TRUE)
    matches <- regmatches(s, m)[[1]]

    if (length(matches) == 0) return(s)

    replacements <- sapply(matches, function(match) {
      bracket_open <- substr(match, 1, 1)
      bracket_close <- substr(match, nchar(match), nchar(match))
      content <- substr(match, 2, nchar(match) - 1)

      # Split by comma (for multi-factor terms like [Class,Sex])
      if (grepl(",", content)) {
        parts <- strsplit(content, ",\\s*")[[1]]
        abbrev_parts <- abbreviate(parts, minlength = n)
        paste0(bracket_open, paste(abbrev_parts, collapse = ","), bracket_close)
      } else {
        paste0(bracket_open, abbreviate(content, minlength = n), bracket_close)
      }
    }, USE.NAMES = FALSE)

    regmatches(s, m) <- list(replacements)
    s
  }, USE.NAMES = FALSE)
}

#' Crossings Interaction of Factors
#'
#' Given two ordered factors in a square, n x n frequency table,
#' `Crossings` creates an n-1 column matrix corresponding to different
#' degrees of difficulty in crossing from one level to the next, as described
#' by Goodman (1972).
#'
#' Instead of treating all mobility as equal, this model posits that the difficulty
#' of moving between categories increases with the number of boundaries (or "crossings")
#' that must be crossed, and that associations between categories decrease
#' with their separation.
#'
#'
#' @param \dots Two factors
#' @return For two factors of `n` levels, returns a binary indicator
#' matrix of `n*n` rows and `n-1` columns.
#' @author Michael Friendly and Heather Turner
#' @seealso
#' \code{\link[stats]{glm}}, \code{\link[gnm]{gnm}} for model fitting
#'      functions for frequency tables;
#' \code{\link[gnm]{Diag}}, \code{\link[gnm]{Mult}}, \code{\link[gnm]{Symm}},
#' \code{\link[gnm]{Topo}} for similar extensions to terms in model formulas.
#'
#' @references
#' Goodman, L. (1972).  Some multiplicative models for the analysis
#' of cross-classified data. In: *Proceedings of the Sixth Berkeley
#' Symposium on Mathematical Statistics and Probability*, Berkeley, CA:
#' University of California Press, pp. 649-696.
#' @keywords models manip
#' @export
#' @examples
#'
#' data(Hauser79)
#' # display table
#' structable(~Father + Son, data=Hauser79)
#'
#' hauser.indep <- gnm(Freq ~ Father + Son,
#'                     data=Hauser79,
#'                     family=poisson)
#'
#' hauser.CR <- update(hauser.indep,
#'                     ~ . + Crossings(Father,Son))
#' LRstats(hauser.CR)
#'
#' hauser.CRdiag <- update(hauser.indep,
#'                         ~ . + Crossings(Father,Son) + Diag(Father,Son))
#' LRstats(hauser.CRdiag)
#'
#' # what does Crossings do?
#' cr <- with(Hauser79, Crossings(Father, Son))
#' head(cr)
#' # Show the codings for varying Crossings levels
#' matrix(cr[,1], nrow=5)
#' matrix(cr[,2], nrow=5)
#' matrix(cr[,3], nrow=5)
#' matrix(cr[,4], nrow=5)
#'
Crossings <- function(...) {
    dots <- list(...)
    if (length(dots) != 2) stop("Crossings() is defined for only two factors")
    if (length(dots[[1]]) != length(dots[[2]]))
    stop("arguments to Crossings() must all have same length")
    dots <- lapply(dots, as.factor)
    n <- nlevels(dots[[1]])
    if (nlevels(dots[[2]]) != n)
        stop("arguments to Crossings() must all have same number of levels")
    result <- crossings(as.numeric(dots[[1]]), as.numeric(dots[[2]]), n)
    rownames(result) <- do.call("paste", c(dots, sep = ""))
    result
}

crossings <- function(i, j, n) {
  npar <- n - 1
  result <- list()
  for(c in 1:npar) {
    overi <- c >= i
    overj <- c >= j
    result[[c]] <- (overi & !overj) + (overj & !overi)
  }
  result <- matrix(unlist(result), length(i), npar)
  colnames(result) <- paste('C', 1:npar, sep='')
  result
}

# Score test for zero inflation in Poisson data
#https://stats.stackexchange.com/questions/118322/how-to-test-for-zero-inflation-in-a-dataset

# References:
# Broek, Jan van den. 1995. ?A Score Test for Zero Inflation in a Poisson Distribution.? Biometrics 51 (2): 738?43. doi:10.2307/2532959.
# Yang, Zhao, James W. Hardin, and Cheryl L. Addy. 2010. ?Score Tests for Zero-Inflation in Overdispersed Count Data.? Communications in Statistics - Theory and Methods 39 (11): 2008?30. doi:10.1080/03610920902948228

# Van den Broek, J. (1995). A Score Test for Zero Inflation in a Poisson Distribution. Biometrics, 51(2), 738-743. doi:10.2307/2532959





#' Score test for zero inflation in Poisson data
#'
#' Carries out a simple score test (van den Broek, 1995) for excess zeros in an
#' otherwise Poisson distribution of counts.  It gives a \eqn{\chi^2_1}
#' statistic on one degree of freedom.
#'
#' The test first calculates the rate estimate from the mean,
#' \eqn{\hat{\lambda} = \bar{x}}. The number of observed zeros, \eqn{n_0} is
#' then compared with the expected number, \eqn{n \hat{p_0}}, where
#' \eqn{\hat{p}_0=\exp[-\hat{\lambda}]}. Then the test statistic is calculated
#' by the formula:
#' \deqn{\frac{(n_0 - n\hat{p}_0)^2}{n\hat{p}_0(1-\hat{p}_0) - n\bar{x}\hat{p}_0^2}} .
#' This test statistic has a \eqn{\chi^2_1} distribution.
#'
#' @param x A vector of non-negative counts, or a one-way frequency table of
#' such counts.
#'
#' @return Returns invisibly a list of three elements:
#' \item{`statistic`}{Value of the test statistic}
#' \item{`df`}{Degrees of freedom}
#' \item{`pvalue`}{Upper tail p-value}
#'
#' @author Michael Friendly
#'
#' @references The original R code came from a Stackexchange question,
#' <https://stats.stackexchange.com/questions/118322/how-to-test-for-zero-inflation-in-a-dataset>
#'
#' Van den Broek, J. (1995).  A Score Test for Zero Inflation in a Poisson
#' Distribution.  *Biometrics*, **51**(2), 738-743.
#' https://www.jstor.org/stable/2532959
#'
#' Yang, Zhao, James W. Hardin, and Cheryl L. Addy (2010).  Score Tests for
#' Zero-Inflation in Overdispersed Count Data.  *Communications in
#' Statistics - Theory and Methods* **39** (11) 2008-2030.
#' DOI: 10.1080/03610920902948228
#'
#' @family association tests
#'
#' @keywords htest
#' @export
#' @examples
#'
#' # synthetic tests
#' zero.test(rpois(100, 1))
#' zero.test(rpois(100, 5))
#' # add some extra zeros
#' zero.test(c(rep(0, 20), rpois(100, 5)))
#'
#' # Articles by Phd candidates
#' data(PhdPubs, package="vcdExtra")
#' zero.test(PhdPubs$articles)
#'
#' phd.tab <- table(PhdPubs$articles)
#' zero.test(phd.tab)
#'
#'
zero.test <- function(x) {

    if(is.table(x)) { # expand to vector of values
        if(length(dim(x)) > 1) stop ("x must be a 1-way table")
        x <- rep(as.numeric(names(x)), unname(c(x)))
    }
		lambda <- mean(x)
		p0_tilde <- exp(-lambda)
		n0 <- sum(1*(!(x >0)))
		n <- length(x)
		numerator <- (n0 - n*p0_tilde)^2
		denominator <- n*p0_tilde*(1-p0_tilde) - n*lambda*(p0_tilde^2)
		stat <- numerator/denominator
		pvalue <- pchisq(stat,df=1, ncp=0, lower.tail=FALSE)
		result <- list(statistic=stat, df=1, prob=pvalue)
		cat(paste("Score test for zero inflation\n\n",
		          "\tChi-square =", round(stat,5), "\n",
		          "\tdf = 1\n",
		          "\tpvalue:", format.pval(pvalue), "\n"))
		invisible(result)
}

# Originally from Marc Schwarz
# Ref: http://tolstoy.newcastle.edu.au/R/e6/help/09/01/1873.html

# 23 Feb 22: Fix warning from type.convert



#' Expand a frequency table to case form
#' 
#' Converts a frequency table, given either as a table object or a data frame
#' in frequency form to a data frame representing individual observations in
#' the table.
#' 
#' `expand.table` is a synonym for `expand.dft`.
#' 
#' @aliases expand.dft expand.table
#' @param x A table object, or a data frame in frequency form containing
#' factors and one numeric variable representing the cell frequency for that
#' combination of factors.
#' @param var.names A list of variable names for the factors, if you wish to
#' override those already in the table
#' @param freq The name of the frequency variable in the table
#' @param \dots Other arguments passed down to `type.convert`.  In
#' particular, pay attention to `na.strings` (default:
#' `na.strings=NA` if there are missing cells) and `as.is` (default:
#' `as.is=FALSE`, converting character vectors to factors).
#' @return A data frame containing the factors in the table and as many
#' observations as are represented by the total of the `freq` variable.
#' @author Mark Schwarz
#' @seealso
#' \code{\link[utils]{type.convert}},
#' \code{\link[gnm]{expandCategorical}}, \code{\link{as_caseform}},
#' \code{\link{as_table}}, \code{\link{as_freqform}},
#' \code{\link{as_array}}, \code{\link{as_matrix}}
#' 
#' @references Originally posted on R-Help, Jan 20, 2009,
#' http://tolstoy.newcastle.edu.au/R/e6/help/09/01/1873.html
#' 
#' Friendly, M. and Meyer, D. (2016).  *Discrete Data Analysis with R:
#' Visualization and Modeling Techniques for Categorical and Count Data*.  Boca
#' Raton, FL: Chapman & Hall/CRC. <http://ddar.datavis.ca>.
#' @keywords manip array
#' @examples
#' 
#' library(vcd)
#' art <- xtabs(~Treatment + Improved, data = Arthritis)
#' art
#' artdf <- expand.dft(art)
#' str(artdf)
#' 
#' # 1D case
#' (tab <- table(sample(head(letters), 20, replace=TRUE)))
#' expand.table(tab, var.names="letter")
#' 
#' 
#' @export expand.dft
expand.dft <- function(x, var.names = NULL, freq = "Freq", ...)
{
  #  allow: a table object, or a data frame in frequency form
  if(inherits(x, "table"))
    x <- as.data.frame.table(x, responseName = freq)

  freq.col <- which(colnames(x) == freq)
  if (length(freq.col) == 0)
      stop(paste(sQuote("freq"), "not found in column names"))

  DF <- sapply(1:nrow(x),
               function(i) x[rep(i, each = x[i, freq.col, drop = TRUE]), ],
               simplify = FALSE)

  DF <- do.call("rbind", DF)[, -freq.col, drop=FALSE]

  for (i in 1:ncol(DF))
  {
    DF[[i]] <- type.convert(as.character(DF[[i]]), as.is=TRUE, ...)
## DONE ##: Generates warning: 
##    1: In type.convert.default(as.character(DF[[i]]), ...) :
##      'as.is' should be specified by the caller; using TRUE
  }

  rownames(DF) <- NULL

  if (!is.null(var.names))
  {
    if (length(var.names) < dim(DF)[2])
    {
      stop(paste("Too few", sQuote("var.names"), "given."))
    } else if (length(var.names) > dim(DF)[2]) {
      stop(paste("Too many", sQuote("var.names"), "given."))
    } else {
      names(DF) <- var.names
    }
  }

  DF
}

# make this a synonym
#' @usage expand.table(x, var.names = NULL, freq = "Freq", ...)
#' @export expand.table
expand.table <- expand.dft

data("HairEyeColor")

arrayDat <- unclass(HairEyeColor) # Generate array form data
freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
caseForm <- expand.dft(freqForm) # Generate case form data

test_that("Matrix returns when array supplied", {
  mat <- as_matrix(arrayDat, dim = c("Hair", "Eye"))
  expect_true(inherits(mat, "matrix"))
})

test_that("Matrix returns when table supplied", {
  mat <- as_matrix(HairEyeColor, dim = c("Hair", "Eye"))
  expect_true(inherits(mat, "matrix"))
})

test_that("Matrix returns when frequency form supplied", {
  mat <- as_matrix(freqForm, freq = "Freq", dim = c("Hair", "Eye"))
  expect_true(inherits(mat, "matrix"))
})

test_that("Matrix returns when (tibble) frequency form supplied", {
  mat <- as_matrix(tidy_freqForm, freq = "n", dim = c("Hair", "Eye"))
  expect_true(inherits(mat, "matrix"))
})

test_that("Matrix returns when case form supplied", {
  mat <- as_matrix(caseForm, dim = c("Hair", "Eye"))
  expect_true(inherits(mat, "matrix"))
})

test_that("Correct dimensions returned", {
  mat <- as_matrix(HairEyeColor, dim = c("Hair", "Eye"))
  expect_true(all(names(dimnames(mat)) == c("Hair", "Eye")))
})

  

test_that("as_table returns table if table is supplied", {
  data("HairEyeColor")
  tab <- as_table(HairEyeColor)
  expect_s3_class(tab, "table")
})

test_that("as_table returns table if array is supplied", {
  data("HairEyeColor")
  tab <- as_table(unclass(HairEyeColor)) # unclass() converts table -> array
  expect_s3_class(tab, "table")
})

test_that("as_table returns table if freqform is supplied", {
  data("HairEyeColor")
  tab <- as_table(as.data.frame(HairEyeColor), freq = "Freq")
  expect_s3_class(tab, "table")
})

test_that("as_table returns table if case form is supplied", {
  data("HairEyeColor")
  tab <- as_table(expand.dft(HairEyeColor))
  expect_s3_class(tab, "table")
})

test_that("as_table does not modify table entries", {
  data("HairEyeColor")
  tab <- as_table(HairEyeColor)
  tab_freq <- as_table(as.data.frame(HairEyeColor), freq = "Freq")
  tab_case <- as_table(expand.dft(HairEyeColor))
  expect_equal(sum(tab), sum(HairEyeColor))
  expect_equal(sum(tab_freq), sum(tab_case))
  expect_equal(sum(tab), sum(tab_case))
})

test_that("Proportions sum to one", {
  data("HairEyeColor")
  tab <- as_table(HairEyeColor, prop = TRUE)
  expect_equal(sum(tab), 1)
})

test_that("Marginal proportions sum to one", {
  data("HairEyeColor")
  tab <- as_table(HairEyeColor, prop = c("Hair", "Sex"))
  expect_equal(sum(tab["Black",,"Male"]), 1)
  tab <- as_table(HairEyeColor, prop = "Hair")
  expect_equal(sum(tab["Black",,]), 1)
})

#' Fit All K-way Models in a GLM
#'
#' Generate and fit all 0-way, 1-way, 2-way, ... k-way terms in a glm.
#'
#' This function is designed mainly for hierarchical loglinear models (or
#' `glm`s in the poisson family), where it is desired to find the
#' highest-order terms necessary to achieve a satisfactory fit.
#'
#' Using \code{\link[stats]{anova}} on the resulting \code{\link{glmlist}}
#' object will then give sequential tests of the pooled contributions of all
#' terms of degree \eqn{k+1} over and above those of degree \eqn{k}.
#'
#' This function is also intended as an example of a generating function for
#' \code{\link{glmlist}} objects, to facilitate model comparison, extraction,
#' summary and plotting of model components, etc., perhaps using `lapply`
#' or similar.
#'
#' With `y` as the response in the `formula`, the 0-way (null) model
#' is `y ~ 1`.  The 1-way ("main effects") model is that specified in the
#' `formula` argument.  The k-way model is generated using the formula
#' `. ~ .^k`. With the default `order = nt`, the final model is the
#' saturated model.
#'
#' As presently written, the function requires a two-sided formula with an
#' explicit response on the LHS. For frequency data in table form (e.g.,
#' produced by `xtabs`) you the `data` argument is coerced to a
#' data.frame, so you should supply the `formula` in the form `Freq ~
#' ` \dots{}.
#'
#' @param formula a two-sided formula for the 1-way effects in the model. The LHS should be the response, and the RHS
#'        should be the first-order terms connected by `+` signs.
#' @param family a description of the error distribution and link function to be used in the model.  This can be a character
#'        string naming a family function, a family function or the result of a call to a family function.
#'        (See \code{\link[stats]{family}} for details of family functions.)
#' @param data an optional data frame, list or environment (or object coercible
#' by \code{\link[base]{as.data.frame}} to a data frame) containing the
#' variables in the model. If not found in data, the variables are taken from
#' `environment(formula)`, typically the environment from which `glm`
#' is called.
#' @param \dots Other arguments passed to `glm`
#' @param order Highest order interaction of the models generated. Defaults to
#' the number of terms in the model formula.
#' @param prefix Prefix used to label the models fit in the `glmlist`
#' object.
#' @return An object of class `glmlist`, of length `order+1`
#' containing the 0-way, 1-way, ...  models up to degree `order`.
#' @author Michael Friendly and Heather Turner
#'
#' @seealso \code{\link{glmlist}}, \code{\link{Summarise}} (soon to be
#' deprecated), \code{\link{LRstats}}
#'
#' @family glmlist functions
#' @keywords models
#' @examples
#'
#' ## artificial data
#' factors <- expand.grid(A=factor(1:3),
#'                        B=factor(1:2),
#'                        C=factor(1:3),
#'                        D=factor(1:2))
#' Freq <- rpois(nrow(factors), lambda=40)
#' df <- cbind(factors, Freq)
#'
#' mods3 <- Kway(Freq ~ A + B + C, data=df, family=poisson)
#' LRstats(mods3)
#' mods4 <- Kway(Freq ~ A + B + C + D, data=df, family=poisson)
#' LRstats(mods4)
#'
#' # JobSatisfaction data
#' data(JobSatisfaction, package="vcd")
#' modSat <- Kway(Freq ~ management+supervisor+own,
#'                data=JobSatisfaction,
#'                family=poisson, prefix="JobSat")
#' LRstats(modSat)
#' anova(modSat, test="Chisq")
#'
#' # Rochdale data: very sparse, in table form
#' data(Rochdale, package="vcd")
#' \dontrun{
#' modRoch <- Kway(Freq~EconActive + Age + HusbandEmployed + Child +
#'                      Education + HusbandEducation + Asian + HouseholdWorking,
#'                 data=Rochdale, family=poisson)
#' LRstats(modRoch)
#' }
#'
#' @export Kway
Kway <- function(formula, family=poisson, data, ..., order=nt, prefix="kway") {

   if (is.character(family))
       family <- get(family, mode = "function", envir = parent.frame())
   if (is.function(family))
       family <- family()
   if (is.null(family$family)) {
       print(family)
       stop("'family' not recognized")
   }
   if (missing(data))
        data <- environment(formula)

   models <- list()
   mod <- glm(formula, family=family, data, ...)
   mod$call$formula <- formula
   terms <- terms(formula)
   tl <- attr(terms, "term.labels")
   nt <- length(tl)
   models[[1]] <- mod
   for(i in 2:order) {
       models[[i]] <- update(mod, substitute(.~.^p, list(p = i)))
   }      # null model
   mod0 <- update(mod, .~1)
   models <- c(list(mod0), models)
   names(models) <- paste(prefix, 0:order, sep = ".")
   class(models) <- "glmlist"
   models
}

# The core algorithm (bd_internal(), below) closely follows the Breslow-Day/Tarone
# implementation originally by Michael Hoehle, packaged as DescTools::BreslowDayTest() by
# Andri Signorell (DescTools is GPL (>= 2), same license as this package):
# https://github.com/AndriSignorell/DescTools/blob/master/R/Tests.r
# Generalized here to handle tables of any dimensionality (not just 2x2xk) and to support
# a Woolf-test-style decomposition for 2x2xRxC tables -- see woolf_test().

#' Breslow-Day Test for Homogeneity of Odds Ratios
#'
#' Tests for homogeneity of odds ratios across strata in \eqn{2 \times 2 \times k}{2 x 2 x k}
#' tables (i.e., whether a common odds ratio fits all strata). Generalized to handle tables
#' of any dimensionality beyond 3. For 4-dimensional tables, optionally provides a two-way
#' decomposition of the homogeneity test into row effects, column effects, and residual,
#' analogous to [woolf_test()] with `decompose = TRUE`.
#'
#' @details
#' The Breslow-Day test (Breslow & Day, 1980) tests the hypothesis that a common odds ratio
#' \eqn{\psi} fits all \eqn{k} strata. Given a common OR (by default the Mantel-Haenszel
#' estimate), the expected cell count \eqn{\tilde{a}_j} in cell \eqn{[1,1]} of stratum
#' \eqn{j} is found by solving the quadratic:
#'
#' \deqn{(\psi - 1)\tilde{a}_j^2 - [n_{2j} - m_{1j} + \psi(n_{1j} + m_{1j})]\tilde{a}_j
#'       + \psi \, m_{1j} n_{1j} = 0}
#'
#' where \eqn{m_{1j}} and \eqn{m_{2j}} are the row margins and \eqn{n_{1j}} and
#' \eqn{n_{2j}} are the column margins of stratum \eqn{j}. The test statistic is:
#'
#' \deqn{\chi^2_{BD} = \sum_{j=1}^{k} \frac{(a_j - \tilde{a}_j)^2}{\widehat{\text{Var}}(a_j)}}
#'
#' where \eqn{\widehat{\text{Var}}(a_j) = (1/\tilde{a}_j + 1/\tilde{b}_j + 1/\tilde{c}_j +
#' 1/\tilde{d}_j)^{-1}} and \eqn{\tilde{b}_j, \tilde{c}_j, \tilde{d}_j} are the remaining
#' expected cell counts. Under the null hypothesis, \eqn{\chi^2_{BD}} follows a chi-squared
#' distribution with \eqn{k - 1} degrees of freedom.
#'
#' The Tarone (1985) correction subtracts a term to account for estimation of the common OR:
#'
#' \deqn{\chi^2_{BD,T} = \chi^2_{BD} -
#'       \frac{(\sum_j a_j - \sum_j \tilde{a}_j)^2}{\sum_j \widehat{\text{Var}}(a_j)}}
#'
#' \strong{Comparison with the Woolf test:}
#' The Woolf test uses log odds ratios and tests deviation from their weighted mean,
#' whereas the Breslow-Day test works on the cell-count scale against a specified common OR.
#' For large samples they agree closely; Breslow-Day is generally preferred when the
#' Mantel-Haenszel common OR is the quantity of interest.
#'
#' \strong{Decomposition for 4-way tables:}
#' For a \eqn{2 \times 2 \times R \times C} table, when `decompose = TRUE`, the overall
#' test is decomposed as:
#'
#' \deqn{\chi^2_{\text{Total}} = \chi^2_{\text{Rows}} + \chi^2_{\text{Cols}} +
#'       \chi^2_{\text{Residual}}}
#'
#' where the row and column components use the row- and column-marginal (pooled) tables,
#' all tested against the same common OR as the overall test. The residual is defined by
#' subtraction and has \eqn{(R-1)(C-1)} degrees of freedom.
#'
#' @param x A \eqn{2 \times 2 \times k}{2 x 2 x k} table, or more generally a
#'   \eqn{2 \times 2 \times \ldots}{2 x 2 x ...} array where the first two dimensions
#'   are both 2.
#' @param OR The common odds ratio to test against. If `NA` (the default), the
#'   Mantel-Haenszel estimate is used.
#' @param correct Logical. If `TRUE`, the Tarone (1985) correction is applied.
#'   Defaults to `FALSE`.
#' @param decompose Logical. If `TRUE` and `x` is 4-dimensional
#'   (a \eqn{2 \times 2 \times R \times C}{2 x 2 x R x C} table), the test is
#'   decomposed into row effects, column effects, and residual. Defaults to `FALSE`.
#'   Ignored for non-4-dimensional tables.
#'
#' @return A list of class `"breslow_day_test"` (also inheriting from `"htest"`)
#'   containing:
#'   \item{statistic}{the chi-squared test statistic.}
#'   \item{parameter}{degrees of freedom.}
#'   \item{p.value}{\eqn{p}-value.}
#'   \item{method}{character string describing the test.}
#'   \item{data.name}{character string giving the name of the data.}
#'   \item{or_vars}{names of the first two dimensions (the 2x2 table variables).}
#'   \item{strata_vars}{names of the stratifying variables (dimensions 3 and beyond).}
#'   \item{OR}{the common odds ratio used (MH estimate if `OR = NA` was supplied).}
#'   \item{correct}{logical indicating whether Tarone correction was applied.}
#'   \item{observed}{observed \eqn{a_j} counts (cell \eqn{[1,1]} of each stratum).}
#'   \item{expected}{expected \eqn{\tilde{a}_j} counts under the common OR.}
#'   \item{decomposed}{logical indicating if decomposition was performed.}
#'
#'   When `decompose = TRUE` (only for 4-dimensional tables), additional components:
#'   \item{rows}{list with `statistic`, `df`, `p.value` for row effects.}
#'   \item{cols}{list with `statistic`, `df`, `p.value` for column effects.}
#'   \item{residual}{list with `statistic`, `df`, `p.value` for residual (interaction).}
#'
#' @importFrom stats mantelhaen.test pchisq
#' @seealso [stats::mantelhaen.test()], [woolf_test()], [DescTools::BreslowDayTest()]
#' @family association tests
#' @author Andri Signorell, Michael Friendly (enhancements)
#'
#' @references
#' Breslow, N. E. & Day, N. E. (1980). \emph{Statistical Methods in Cancer Research.
#' Vol. 1: The Analysis of Case-Control Studies}. IARC Scientific Publications No. 32.
#' Lyon: International Agency for Research on Cancer.
#'
#' Tarone, R. E. (1985). On heterogeneity tests based on efficient scores.
#' \emph{Biometrika}, \bold{72}, 91-95.
#'
#' Lachin, J. M. (2000). \emph{Biostatistical Methods: The Assessment of Relative Risks}.
#' Wiley, p. 124-125.
#'
#' @examples
#' # 3-way table
#' data(CoalMiners, package = "vcd")
#' breslow_day_test(CoalMiners)
#' breslow_day_test(CoalMiners, correct = TRUE)   # Tarone correction
#'
#' # Compare with Woolf test
#' woolf_test(CoalMiners)
#'
#' data(Heart, package = "vcdExtra")
#' breslow_day_test(Heart)
#'
#' # 4-way table without decomposition
#' data(Fungicide, package = "vcdExtra")
#' breslow_day_test(Fungicide)
#'
#' # 4-way table with decomposition
#' breslow_day_test(Fungicide, decompose = TRUE)
#'
#' @export
breslow_day_test <- function(x, OR = NA, correct = FALSE, decompose = FALSE) {
  DNAME <- deparse(substitute(x))

  dims <- dim(x)
  if (length(dims) < 3)
    stop("Array must have at least 3 dimensions")
  if (!all(dims[1:2] == 2))
    stop("First two dimensions must be 2x2")

  dimnames_x <- names(dimnames(x))
  if (is.null(dimnames_x))
    dimnames_x <- paste0("Dim", seq_along(dims))
  or_vars     <- dimnames_x[1:2]
  strata_vars <- dimnames_x[-(1:2)]

  if (decompose && length(dims) != 4) {
    warning("decompose = TRUE only applies to 4-dimensional tables; ignoring decompose")
    decompose <- FALSE
  }

  # Compute common OR once from all data (used for all sub-tests in decomposed case)
  if (is.na(OR)) {
    x_mh <- if (length(dims) > 3)
      array(x, dim = c(dims[1:2], prod(dims[3:length(dims)])))
    else x
    or.hat <- as.numeric(mantelhaen.test(x_mh)$estimate)
  } else {
    or.hat <- OR
  }

  # Handle 4-dimensional case with decomposition
  if (length(dims) == 4 && decompose) {
    R <- dims[3]; C <- dims[4]

    x_all  <- array(x, dim = c(2, 2, R * C))
    overall <- bd_internal(x_all, or.hat)

    x_rows <- apply(x, c(1, 2, 3), sum)   # 2 x 2 x R
    rows   <- bd_internal(x_rows, or.hat)

    x_cols <- apply(x, c(1, 2, 4), sum)   # 2 x 2 x C
    cols   <- bd_internal(x_cols, or.hat)

    stat_overall <- if (correct) overall$X2.tarone else overall$X2
    stat_rows    <- if (correct) rows$X2.tarone    else rows$X2
    stat_cols    <- if (correct) cols$X2.tarone    else cols$X2
    stat_resid   <- stat_overall - stat_rows - stat_cols
    df_resid     <- (R - 1L) * (C - 1L)

    suffix <- if (correct) " (with Tarone correction)" else ""
    METHOD <- paste0("Breslow-Day Test on Homogeneity of Odds Ratios", suffix)

    STATISTIC <- stat_overall; names(STATISTIC) <- "X-squared"
    PARAMETER <- R * C - 1L;   names(PARAMETER) <- "df"

    result <- list(
      statistic   = STATISTIC,
      parameter   = PARAMETER,
      p.value     = 1 - pchisq(stat_overall, R * C - 1L),
      method      = METHOD,
      data.name   = DNAME,
      or_vars     = or_vars,
      strata_vars = strata_vars,
      OR          = or.hat,
      correct     = correct,
      observed    = overall$a,
      expected    = overall$tildea,
      decomposed  = TRUE,
      rows = list(statistic = stat_rows,
                  df        = R - 1L,
                  p.value   = 1 - pchisq(stat_rows, R - 1L)),
      cols = list(statistic = stat_cols,
                  df        = C - 1L,
                  p.value   = 1 - pchisq(stat_cols, C - 1L)),
      residual = list(statistic = stat_resid,
                      df        = df_resid,
                      p.value   = 1 - pchisq(stat_resid, df_resid))
    )
    class(result) <- c("breslow_day_test", "htest")
    return(result)
  }

  # Standard case: collapse higher dimensions into a single stratum dimension
  if (length(dims) > 3)
    x <- array(x, dim = c(dims[1:2], prod(dims[3:length(dims)])))

  k   <- dim(x)[3]
  res <- bd_internal(x, or.hat)

  STATISTIC <- if (correct) res$X2.tarone else res$X2
  PARAMETER <- k - 1L
  PVAL <- 1 - pchisq(STATISTIC, PARAMETER)
  names(STATISTIC) <- "X-squared"
  names(PARAMETER) <- "df"

  suffix <- if (correct) " (with Tarone correction)" else ""
  METHOD <- paste0("Breslow-Day Test on Homogeneity of Odds Ratios", suffix)

  structure(list(
    statistic   = STATISTIC,
    parameter   = PARAMETER,
    p.value     = PVAL,
    method      = METHOD,
    data.name   = DNAME,
    or_vars     = or_vars,
    strata_vars = strata_vars,
    OR          = or.hat,
    correct     = correct,
    observed    = res$a,
    expected    = res$tildea,
    decomposed  = FALSE
  ), class = c("breslow_day_test", "htest"))
}


# Internal helper: Breslow-Day statistic on a 2x2xk array against a fixed OR.
# Returns both the uncorrected (X2) and Tarone-corrected (X2.tarone) statistics,
# plus the per-stratum observed/expected/variance vectors.
bd_internal <- function(y, or) {
  k <- dim(y)[3]
  X2 <- 0
  a <- tildea <- Var.a <- numeric(k)

  for (j in seq_len(k)) {
    mj <- rowSums(y[, , j])   # row margins
    nj <- colSums(y[, , j])   # col margins

    # Solve for tilde_a_j: expected [1,1] cell count under common OR given margins.
    # polyroot() takes coefficients in ascending order (constant term first).
    coef    <- c(-mj[1] * nj[1] * or,
                 nj[2] - mj[1] + or * (nj[1] + mj[1]),
                 1 - or)
    sols    <- Re(polyroot(coef))
    tildeaj <- sols[(sols > 0) & (sols <= min(nj[1], mj[1]))]

    if (length(tildeaj) != 1L)
      stop(sprintf(
        "No unique valid root in stratum %d -- check for zero or very sparse cells", j))

    tildebj <- mj[1] - tildeaj
    tildecj <- nj[1] - tildeaj
    tildedj <- mj[2] - tildecj

    Var.aj <- (1/tildeaj + 1/tildebj + 1/tildecj + 1/tildedj)^(-1)
    aj     <- y[1, 1, j]

    X2        <- X2 + as.numeric((aj - tildeaj)^2 / Var.aj)
    a[j]      <- aj
    tildea[j] <- tildeaj
    Var.a[j]  <- Var.aj
  }

  # Tarone correction: adjusts for estimation of the common OR
  X2.tarone <- as.numeric(X2 - (sum(a) - sum(tildea))^2 / sum(Var.a))

  list(X2 = X2, X2.tarone = X2.tarone,
       a = a, tildea = tildea, Var.a = Var.a)
}


#' Print method for breslow_day_test objects
#'
#' @param x An object of class `"breslow_day_test"`
#' @param digits Number of significant digits for the common OR. Default 4.
#' @param ... Additional arguments (currently unused).
#' @rdname breslow_day_test
#' @export
print.breslow_day_test <- function(x, digits = 4, ...) {
  cat("\n")
  cat(x$method, "\n\n")

  cat("Data:         ", x$data.name, "\n")
  cat("OR variables: ", paste(x$or_vars,     collapse = ", "), "\n")
  cat("Strata:       ", paste(x$strata_vars, collapse = ", "), "\n")
  cat("Common OR:    ", format(x$OR, digits = digits), "\n\n")

  if (!x$decomposed) {
    cat(sprintf("X-squared = %.4f, df = %d, p-value = %.4g\n",
                x$statistic, x$parameter, x$p.value))
  } else {
    cat("Overall homogeneity test:\n")
    cat(sprintf("  X-squared = %.4f, df = %d, p-value = %.4g\n\n",
                x$statistic, x$parameter, x$p.value))

    cat("Decomposition:\n")
    row_label <- sprintf("Rows (%s):", x$strata_vars[1])
    col_label <- sprintf("Cols (%s):", x$strata_vars[2])
    res_label <- "Residual:"
    max_width <- max(nchar(row_label), nchar(col_label), nchar(res_label))

    cat(sprintf("  %-*s X-squared = %.4f, df = %d, p-value = %.4g\n",
                max_width, row_label,
                x$rows$statistic, x$rows$df, x$rows$p.value))
    cat(sprintf("  %-*s X-squared = %.4f, df = %d, p-value = %.4g\n",
                max_width, col_label,
                x$cols$statistic, x$cols$df, x$cols$p.value))
    cat(sprintf("  %-*s X-squared = %.4f, df = %d, p-value = %.4g\n",
                max_width, res_label,
                x$residual$statistic, x$residual$df, x$residual$p.value))

    cat("\nNote: Overall = Rows + Columns + Residual\n")
  }

  invisible(x)
}

#' Update method for a `xtabs` object
#'
#' Provides an `update` method for `"xtabs"` objects, typically by
#' removing terms from the formula to collapse over them.
#'
#'
#' @param object An existing `"xtabs"` object
#' @param formula. Changes to the formula ? see
#' \code{\link[stats]{update.formula}} for details
#' @param \dots Additional arguments to the call, or arguments with changed
#' values.
#' @param evaluate If `TRUE`, evaluate the new call else return the call
#' @return If `evaluate == TRUE`, the new `"xtabs"` object, otherwise
#' the updated call
#' @author Michael Friendly
#' @seealso \code{\link[stats]{update.formula}} for details on updates to model
#' formulae
#'
#' \code{\link[base]{margin.table}} does something similar,
#' \code{\link{collapse.table}} collapses category levels
#' @keywords models
#' @exportS3Method
#' @examples
#'
#' vietnam.tab <- xtabs(Freq ~ sex + year + response, data=Vietnam)
#'
#' update(vietnam.tab, formula = ~ . -year)
#'
#'
#'
#' @export update.xtabs
update.xtabs <-
function (object, formula., ..., evaluate = TRUE)
{
    if (is.null(call<-attr(object, "call")))
        stop("need an object with call component")
    extras <- match.call(expand.dots = FALSE)$...
    if (!missing(formula.))
        call$formula <- update.formula(call$formula, formula.)
    if (length(extras)) {
        existing <- !is.na(match(names(extras), names(call)))
        for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
        if (any(!existing)) {
            call <- c(as.list(call), extras[!existing])
            call <- as.call(call)
        }
    }
    if (evaluate)
        eval(call, parent.frame())
    else call
}

# drop1.loglm()/LRanova()/assoc_strength(): term-level tests and effect
# sizes for a fitted loglm model's generating class.
#
# MASS already supplies update.loglm(), extractAIC.loglm(), and a working
# terms()/formula() for "loglm" objects, so stats::drop1()/MASS::dropterm()
# already dispatch correctly (including data.frame/`Freq ~ ...` fits) via
# their .default methods, refitting through update(). drop1.loglm() below
# does the same -- refits via update(object, ~ . - term) rather than
# reconstructing formulas from $margin by hand, which is both simpler and
# avoids accidentally dropping the response variable on data.frame fits.
# It adds the Pearson X^2 alongside the LR G^2 (loglm objects store both
# $lrt and $pearson directly; extractAIC.loglm()/drop1.default() only
# expose a deviance-derived LRT).

#' Term-level tests and effect sizes for loglm models
#'
#' `drop1.loglm()` performs single-term deletion tests for the generating
#' class of a fitted [MASS::loglm()] model: for each term in that
#' class, it drops that term, refits via `update()`, and reports the change
#' in both the LR statistic (G^2) and the Pearson statistic (X^2).
#'
#' `LRanova()` wraps `drop1.loglm()` and adds a partial R^2 effect-size
#' column: the fraction of a baseline model's G^2 attributable to each
#' dropped term.
#'
#' `assoc_strength()` wraps `drop1.loglm()` and adds a partial Cramer's V or
#' Cohen's w effect-size column, converting each term's partial Pearson X^2
#' into a bounded or semi-bounded measure of association strength.
#'
#' @details
#' All three functions test terms in the model's *generating class* --
#' the set of highest-order terms in a hierarchical loglinear model, from
#' which every lower-order relative (main effects, lower interactions) is
#' implied and so doesn't need to be listed or tested separately.
#'
#' For example, with four factors `A`, `B`, `C`, `D` and the model
#' `~ (A + B + C + D)^2` (all terms up to two-way), the fitted model
#' contains ten terms in all -- four main effects and six two-way
#' interactions -- but its generating class is just the six two-way terms:
#' `A:B`, `A:C`, `A:D`, `B:C`, `B:D`, `C:D` (bracket notation `[A,B] [A,C]
#' [A,D] [B,C] [B,D] [C,D]`).
#'
#' None of the four main effects is maximal on
#' its own -- `A`, say, is already implied by `A:B` (or any of `A:C`,
#' `A:D`) -- so they're excluded from the generating class even though
#' they're very much still part of the fitted model. This is exactly what
#' `MASS::loglm()` stores in `object$margin`, and exactly the set of terms
#' `scope` defaults to and `drop1.loglm()` tests: dropping `A:B` removes
#' only that interaction, leaving `A` and `B`'s main effects in place
#' (still implied by `A:C`/`A:D` and `B:C`/`B:D` respectively), so the
#' reduced model stays hierarchical.
#'
#' ## Comparison with `drop1.glm()`
#' A hierarchical `loglm()` model has an equivalent fit via
#' `glm(family = poisson)`, and `stats::drop1.glm()`'s default `scope`
#' already restricts to the same generating-class terms as `object$margin`,
#' so `Df` and the LR statistic agree exactly between the two. `drop1.loglm()`
#' works directly on table/array or `Freq ~ ...` data, adds the Pearson X^2
#' column, and returns the fitted drop-one models for reuse; its `scope`
#' validation also guards against testing a non-generating-class term, which
#' `drop1.glm()` will silently allow with a misleading result. See
#' `dev/anova-like/drop1-compare.R` for a worked comparison.
#'
#' @param object a fitted `MASS::loglm` model
#' @param scope character vector of term labels (colon-separated, as in
#'   `"A:B"`) to test; must be a subset of the model's generating class.
#'   Defaults to the full generating class. Used by `drop1.loglm()` and
#'   `assoc_strength()`.
#' @param baseline `NULL` (default) to use the model of mutual independence
#'   over the variables in `object`'s generating class, or a fitted `loglm`
#'   model nested within `object` to use as the reference for partial R^2.
#'   Used by `LRanova()` only.
#' @param method `"Cramer"` (default) for partial Cramer's V, generalized to
#'   terms of any order by using the *smallest* factor level count among the
#'   term's variables in place of the usual two-way `min(r,c)`; this reduces
#'   exactly to the ordinary two-way Cramer's V (as used by
#'   [vcd::assocstats()]) when the term has two factors. Bounded to `[0,1]`.
#'   `"Cohen"` for Cohen's w = `sqrt(Delta-X^2 / N)`: unbounded, but requires
#'   no reference to table shape, so it stays well-defined uniformly for
#'   terms of any order. Used by `assoc_strength()` only.
#' @param test `"Chisq"` (default) to include an LR p-value column, or
#'   `"none"` to omit it.
#' @param abbrev passed to [get_model()] for the `Model:`/`Baseline:`
#'   heading line(s); `FALSE` (default) for full factor names, or an integer
#'   (e.g. `4`) to abbreviate each factor name to that many characters --
#'   useful when the generating class has many terms and the heading line
#'   gets long (see the DaytonSurvey example below).
#' @param ... currently unused in `drop1.loglm()`; passed on to it from
#'   `LRanova()`/`assoc_strength()`.
#'
#' @return An object of class `c("anova", "data.frame")` with columns `Df`,
#'   `LR Chisq`, `Pearson Chisq`, and (if `test = "Chisq"`) `Pr(>Chi)`, one
#'   row per tested term plus a `<none>` reference row. The fitted models
#'   underlying each row -- `object` itself as `"<none>"`, plus the refit
#'   dropping each term in `scope` -- are attached as `attr(., "models")`, a
#'   `loglmlist` (see [loglmlist()]) with names matching the table's row
#'   names. This is free (the models are already fit to compute the
#'   Delta-statistics) and makes the whole `*.loglmlist` toolchain available
#'   without refitting, e.g. `mosaic(attr(result, "models"), ask = FALSE)`
#'   for a grid of the full model and every drop-one model, or
#'   `LRstats(attr(result, "models"))` for an AIC/BIC comparison.
#'
#'   `LRanova()` adds a `Partial R2` column. The baseline model is attached
#'   separately as `attr(., "baseline")`, since it's a single reference
#'   model rather than one of the `<none>`/drop-one rows in
#'   `attr(., "models")`.
#'
#'   `assoc_strength()` adds a `"Cramer's V"` or `"Cohen's w"` column. Note
#'   this is a *partial* association -- conditional on the other terms in
#'   `object` -- not the marginal association from `vcd::assocstats()` on
#'   the term's own two-way margin; the two can differ sharply (see the
#'   UCBAdmissions `Admit:Gender` example below, a classic Simpson's-paradox
#'   case).
#'
#' @author Michael Friendly
#' @seealso [mosaic.loglmlist()], [LRstats()], [get_models()]
#'
#' @aliases drop1.loglm LRanova assoc_strength
#' @rdname drop1.loglm
#' @export
#' @rawNamespace export(drop1.loglm)
#'
#' @examples
#' library(MASS)
#' ucb <- loglm(~ (Admit + Gender + Dept)^2, data = UCBAdmissions)
#'
#' drop1.loglm(ucb)
#' drop1.loglm(ucb, scope = "Admit:Gender")   # test only a subset
#' drop1.loglm(ucb, abbrev = 4)               # abbreviate factor names
#' names(attr(drop1.loglm(ucb), "models"))    # the fitted models, for free
#'
#' # data.frame + `Freq ~ ...` form works too, not just array/table data
#' data(DaytonSurvey, package = "vcdExtra")
#' DS <- loglm(Freq ~ (cigarette + alcohol + marijuana + sex + race)^2,
#'             data = DaytonSurvey)
#' drop1.loglm(DS)
#'
#' LRanova(ucb)  # default baseline: mutual independence over Admit, Gender, Dept
#'
#' # supply a specific (nested) baseline other than the default -- here, one
#' # that already includes Admit:Gender, so partial R^2 is now relative to a
#' # smaller remaining G^2 than the mutual-independence baseline above
#' partial_baseline <- loglm(~ Admit + Gender + Dept + Admit:Gender, data = UCBAdmissions)
#' LRanova(ucb, baseline = partial_baseline)
#'
#' assoc_strength(ucb)                     # Cramer's V (default)
#' assoc_strength(ucb, method = "Cohen")   # Cohen's w
#'
#' # partial (conditional on Dept) vs. marginal association -- the small
#' # partial Admit:Gender V here versus the much larger marginal V below is
#' # the classic Simpson's-paradox story for this dataset
#' vcd::assocstats(margin.table(UCBAdmissions, c(1, 2)))
#'
#' # Cramer's V and Cohen's w coincide when every factor in a term is binary,
#' # and diverge once a term involves a factor with more levels
#' hec <- loglm(~ (Hair + Eye + Sex)^2, data = HairEyeColor)
#' assoc_strength(hec)  # Hair, Eye each have 4 levels
drop1.loglm <- function(object, scope, test = c("Chisq", "none"), abbrev = FALSE, ...) {
  if (!inherits(object, "loglm")) {
    stop("'object' must inherit from class 'loglm'.", call. = FALSE)
  }
  test <- match.arg(test)

  if (is.null(object$margin) || !length(object$margin)) {
    stop("'object' has no generating class ($margin); cannot determine ",
         "droppable terms.", call. = FALSE)
  }

  margin_labels <- vapply(object$margin, paste, collapse = ":",
                           FUN.VALUE = character(1L))

  if (missing(scope)) {
    scope <- margin_labels
  } else {
    if (!is.character(scope)) {
      stop("'scope' must be a character vector of term labels.", call. = FALSE)
    }
    if (!all(scope %in% margin_labels)) {
      stop("'scope' must be a subset of the generating class: ",
           paste(margin_labels, collapse = ", "), call. = FALSE)
    }
  }

  env <- environment(stats::formula(object))
  refit_drop <- function(term) {
    upd <- stats::update(object,
                          stats::as.formula(paste("~ . -", term)),
                          evaluate = FALSE)
    eval(upd, envir = env)
  }
  reduced <- lapply(scope, refit_drop)

  # Free: these models are already fit to get the Delta-statistics below, so
  # attaching them costs nothing and makes mosaic.loglmlist()/LRstats.loglmlist()/
  # get_models() available on the drop-one models without refitting.
  models <- c(list(object), reduced)
  names(models) <- c("<none>", scope)
  class(models) <- "loglmlist"

  df      <- c(object$df,      vapply(reduced, `[[`, numeric(1L), "df"))
  lrt     <- c(object$lrt,     vapply(reduced, `[[`, numeric(1L), "lrt"))
  pearson <- c(object$pearson, vapply(reduced, `[[`, numeric(1L), "pearson"))

  delta_df      <- df - df[1L]
  delta_lrt     <- lrt - lrt[1L]
  delta_pearson <- pearson - pearson[1L]
  delta_df[1L] <- delta_lrt[1L] <- delta_pearson[1L] <- NA_real_

  aod <- data.frame(
    Df = delta_df,
    `LR Chisq` = delta_lrt,
    `Pearson Chisq` = delta_pearson,
    check.names = FALSE,
    row.names = c("<none>", scope)
  )

  if (test == "Chisq") {
    p <- rep(NA_real_, length(scope) + 1L)
    p[-1L] <- stats::pchisq(delta_lrt[-1L], delta_df[-1L], lower.tail = FALSE)
    aod[["Pr(>Chi)"]] <- p
  }

  attr(aod, "heading") <- c(
    "Single term deletions",
    sprintf("Model: %s", get_model(object, type = "brackets", abbrev = abbrev))
  )
  attr(aod, "models") <- models
  class(aod) <- c("anova", "data.frame")
  aod
}

#' @rdname drop1.loglm
#' @export
LRanova <- function(object, baseline = NULL, test = c("Chisq", "none"), abbrev = FALSE, ...) {
  test <- match.arg(test)
  aod <- drop1.loglm(object, test = test, ...)

  orig_formula <- stats::formula(object)
  has_response <- length(orig_formula) == 3L
  response <- if (has_response) deparse(orig_formula[[2L]]) else NULL

  if (is.null(baseline)) {
    variables <- unique(unlist(object$margin, use.names = FALSE))
    baseline_call <- object$call
    baseline_call$formula <- stats::reformulate(variables, response = response)
    baseline_call[[1L]] <- quote(MASS::loglm)
    baseline <- eval(baseline_call, envir = environment(orig_formula))
  }
  if (!inherits(baseline, "loglm")) {
    stop("'baseline' must be NULL or a fitted 'loglm' model.", call. = FALSE)
  }

  # Nesting check: terms() on a hierarchical loglm already expands to
  # include implied lower-order relatives, so a plain term.labels subset
  # check is sufficient and cheap.
  base_terms  <- attr(stats::terms(baseline), "term.labels")
  model_terms <- attr(stats::terms(object),   "term.labels")
  extra <- setdiff(base_terms, model_terms)
  if (length(extra)) {
    stop("'baseline' is not nested within 'object': term(s) ",
         paste(extra, collapse = ", "),
         " appear in 'baseline' but not in 'object'.", call. = FALSE)
  }
  if (baseline$df < object$df) {
    stop("'baseline' has fewer residual df (", baseline$df, ") than 'object' (",
         object$df, "); 'baseline' must be the more restrictive model.",
         call. = FALSE)
  }

  baseline_g2 <- baseline$lrt
  is_term_row <- rownames(aod) != "<none>"
  partial_r2 <- rep(NA_real_, nrow(aod))
  if (is.finite(baseline_g2) && baseline_g2 > 0) {
    partial_r2[is_term_row] <- aod[["LR Chisq"]][is_term_row] / baseline_g2
  }
  aod[["Partial R2"]] <- partial_r2

  attr(aod, "heading") <- c(
    "Single term deletions, with partial R^2",
    sprintf("Model:    %s", get_model(object, type = "brackets", abbrev = abbrev)),
    sprintf("Baseline: %s  (G^2 = %.3f, df = %d)",
            get_model(baseline, type = "brackets", abbrev = abbrev), baseline_g2, baseline$df)
  )
  attr(aod, "baseline") <- baseline
  aod
}

#' @rdname drop1.loglm
#' @export
assoc_strength <- function(object, scope, method = c("Cramer", "Cohen"), abbrev = FALSE, ...) {
  method <- match.arg(method)
  aod <- drop1.loglm(object, scope = scope, ...)

  full <- if (!is.null(object$fitted)) object else stats::update(object, fitted = TRUE)
  N <- sum(full$fitted)
  dn <- dimnames(full$fitted)

  is_term_row <- rownames(aod) != "<none>"
  term_vars <- strsplit(rownames(aod)[is_term_row], ":", fixed = TRUE)
  x2 <- aod[["Pearson Chisq"]][is_term_row]

  effect <- rep(NA_real_, nrow(aod))
  if (method == "Cohen") {
    effect[is_term_row] <- sqrt(x2 / N)
    colname <- "Cohen's w"
  } else {
    k <- vapply(term_vars, function(vars) min(lengths(dn[vars])) - 1, numeric(1L))
    effect[is_term_row] <- sqrt(x2 / (N * k))
    colname <- "Cramer's V"
  }

  aod[[colname]] <- effect
  attr(aod, "heading") <- c(
    sprintf("Single term deletions, with %s", colname),
    sprintf("Model: %s", get_model(object, type = "brackets", abbrev = abbrev))
  )
  aod
}

# mosaic plot for a glm object
#  Allow it to use residuals of any type computed by residuals()
#  or to pass residuals calculated by another function, e.g., rstandard(), rstudent()
#
#  Allow to apply to any model with discrete factors
# last modified: 3/6/2009 1:51PM
#  - fixed buggy version using ideas from vcd:::plot.loglm
#  - now use $data component when it is a table

## TODO: move to utility.R
#is.discrete.model <- function(model)
# 	all(attr(terms(model), "dataClasses")[-1] %in% c("factor", "ordered"))





#' Mosaic plots for fitted generalized linear and generalized nonlinear models
#'
#' Produces mosaic plots (and other plots in the \code{\link[vcd]{strucplot}}
#' framework) for a log-linear model fitted with \code{\link[stats]{glm}} or
#' for a generalized nonlinear model fitted with \code{\link[gnm]{gnm}}.
#'
#' These methods extend the range of strucplot visualizations well beyond the
#' models that can be fit with \code{\link[MASS]{loglm}}. They are intended for
#' models for counts using the Poisson family (or quasi-poisson), but should be
#' sensible as long as (a) the response variable is non-negative and (b) the
#' predictors visualized in the `strucplot` are discrete factors.
#'
#' For both poisson family generalized linear models and loglinear models,
#' standardized residuals provided by `rstandard` (sometimes called
#' adjusted residuals) are often preferred because they have constant unit
#' asymptotic variance.
#'
#' The `sieve` and `assoc` methods are simple convenience interfaces
#' to this plot method, setting the panel argument accordingly.
#'
#' @aliases mosaic.glm sieve.glm assoc.glm
#' @param x A `glm` or `gnm` object. The response variable, typically
#' a cell frequency, should be non-negative.
#' @param formula A one-sided formula with the indexing factors of the plot
#' separated by '+', determining the order in which the variables are used in
#' the mosaic.  A formula must be provided unless `x$data` inherits from
#' class `"table"` -- in which case the indexing factors of this table are
#' used, or the factors in `x$data` (or model.frame(x) if `x$data` is
#' an environment) exactly cross-classify the data -- in which case this set of
#' cross-classifying factors are used.
#' @param panel Panel function used to draw the plot for visualizing the
#' observed values, residuals and expected values. Currently, one of
#' `"mosaic"`, `"assoc"`, or `"sieve"` in `vcd`.
#' @param type A character string indicating whether the `"observed"` or
#' the `"expected"` values of the table should be visualized by the area
#' of the tiles or bars.
#' @param residuals An optional array or vector of residuals corresponding to
#' the cells in the data, for example, as calculated by
#' `residuals.glm(x)`, `residuals.gnm(x)`.
#' @param residuals_type If the `residuals` argument is `NULL`,
#' residuals are calculated internally and used in the display.  In this case,
#' `residual_type` can be `"pearson"`, `"deviance"` or
#' `"rstandard"`.  Otherwise (when `residuals` is supplied),
#' `residuals_type` is used as a label for the legend in the plot.
#' @param gp Object of class `"gpar"`, shading function or a corresponding
#' generating function (see \code{\link[vcd]{strucplot}} Details and
#' \code{\link[vcd]{shadings}}).  Ignored if shade = FALSE.
#' @param gp_args A list of arguments for the shading-generating function, if
#' specified.
#' @param \dots Other arguments passed to the `panel` function e.g.,
#' \code{\link[vcd]{mosaic}}
#'
#' @return The `structable` visualized by \code{\link[vcd]{strucplot}} is
#' returned invisibly.
#'
#' @author Heather Turner, Michael Friendly, with help from Achim Zeileis
#'
#' @seealso \code{\link[stats]{glm}}, \code{\link[gnm]{gnm}},
#' \code{\link[vcd]{plot.loglm}}, \code{\link[vcd]{mosaic}}
#' @family mosaic plots
#'
#' @keywords hplot models multivariate
#' @importFrom vcd mosaic
#' @importFrom vcd shading_hcl
#' @importFrom gnm meanResiduals
#' @examples
#' library(vcdExtra)
#'
#' GSStab <- xtabs(count ~ sex + party, data=GSS)
#' # using the data in table form
#' mod.glm1 <- glm(Freq ~ sex + party, family = poisson, data = GSStab)
#' res <- residuals(mod.glm1)
#' std <- rstandard(mod.glm1)
#'
#' # For mosaic.default(), need to re-shape residuals to conform to data
#' stdtab <- array(std,
#'                 dim=dim(GSStab),
#'                 dimnames=dimnames(GSStab))
#'
#' mosaic(GSStab,
#'        gp=shading_Friendly,
#'        residuals=stdtab,
#'        residuals_type="Std\nresiduals",
#'        labeling = labeling_residuals)
#'
#'
#' # Using externally calculated residuals with the glm() object
#' mosaic(mod.glm1,
#'        residuals=std,
#'        labeling = labeling_residuals,
#'        shade=TRUE)
#'
#' # Using residuals_type
#' mosaic(mod.glm1,
#'        residuals_type="rstandard",
#'        labeling = labeling_residuals, shade=TRUE)
#'
#' ## Ordinal factors and structured associations
#' data(Mental)
#' xtabs(Freq ~ mental+ses, data=Mental)
#' long.labels <- list(set_varnames = c(mental="Mental Health Status",
#'                                      ses="Parent SES"))
#'
#' # fit independence model
#' # Residual deviance: 47.418 on 15 degrees of freedom
#' indep <- glm(Freq ~ mental+ses,
#'              family = poisson, data = Mental)
#'
#' long.labels <- list(set_varnames = c(mental="Mental Health Status",
#'                                      ses="Parent SES"))
#' mosaic(indep,
#'        formula = ~ses + mental,
#'        residuals_type="rstandard",
#'        labeling_args = long.labels,
#'        labeling=labeling_residuals)
#'
#' # or, show as a sieve diagram
#' mosaic(indep,
#'        formula = ~ses + mental,
#'        labeling_args = long.labels,
#'        panel=sieve,
#'        gp=shading_Friendly)
#'
#' # fit linear x linear (uniform) association.  Use integer scores for rows/cols
#' Cscore <- as.numeric(Mental$ses)
#' Rscore <- as.numeric(Mental$mental)
#'
#' linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
#'                 family = poisson, data = Mental)
#'
#' mosaic(linlin,
#'        formula = ~ses + mental,
#'        residuals_type="rstandard",
#'        labeling_args = long.labels,
#'        labeling=labeling_residuals,
#'        suppress=1,
#'        gp=shading_Friendly,
#'        main="Lin x Lin model")
#'
#' ##  Goodman Row-Column association model fits even better (deviance 3.57, df 8)
#' if (require(gnm)) {
#' Mental$mental <- C(Mental$mental, treatment)
#' Mental$ses <- C(Mental$ses, treatment)
#' RC1model <- gnm(Freq ~ ses + mental + Mult(ses, mental),
#'                 family = poisson, data = Mental)
#'
#' mosaic(RC1model,
#'        formula = ~ses + mental,
#'        residuals_type="rstandard",
#'        labeling_args = long.labels,
#'        labeling=labeling_residuals,
#'        suppress=1,
#'        gp=shading_Friendly,
#'        main="RC1 model")
#'  }
#'
#'  ############# UCB Admissions data, fit using glm()
#'
#' structable(Dept ~ Admit+Gender,UCBAdmissions)
#'
#' berkeley <- as.data.frame(UCBAdmissions)
#' berk.glm1 <- glm(Freq ~ Dept * (Gender+Admit), data=berkeley, family="poisson")
#' summary(berk.glm1)
#'
#' mosaic(berk.glm1,
#'        gp=shading_Friendly,
#'        labeling=labeling_residuals,
#'        formula=~Admit+Dept+Gender)
#'
#' # the same, displaying studentized residuals;
#' # note use of formula to reorder factors in the mosaic
#' mosaic(berk.glm1,
#'        residuals_type="rstandard",
#'        labeling=labeling_residuals,
#'        shade=TRUE,
#' 	     formula=~Admit+Dept+Gender,
#' 	     main="Model: [DeptGender][DeptAdmit]")
#'
#' ## all two-way model
#' berk.glm2 <- glm(Freq ~ (Dept + Gender + Admit)^2, data=berkeley, family="poisson")
#' summary(berk.glm2)
#'
#' mosaic(berk.glm2,
#'        residuals_type="rstandard",
#'        labeling = labeling_residuals,
#'        shade=TRUE,
#' 	     formula=~Admit+Dept+Gender,
#' 	     main="Model: [DeptGender][DeptAdmit][AdmitGender]")
#'
#' anova(berk.glm1, berk.glm2, test="Chisq")
#'
#' # Add 1 df term for association of [GenderAdmit] only in Dept A
#' berkeley <- within(berkeley,
#'                    dept1AG <- (Dept=='A')*(Gender=='Female')*(Admit=='Admitted'))
#' berkeley[1:6,]
#'
#' berk.glm3 <- glm(Freq ~ Dept * (Gender+Admit) + dept1AG, data=berkeley, family="poisson")
#' summary(berk.glm3)
#'
#' mosaic(berk.glm3,
#'        residuals_type = "rstandard",
#'        labeling = labeling_residuals,
#'        shade=TRUE,
#' 	      formula = ~Admit+Dept+Gender,
#' 	      main = "Model: [DeptGender][DeptAdmit] + DeptA*[GA]")
#'
#' # compare models
#' anova(berk.glm1, berk.glm3, test="Chisq")
#'
#'
#' @export
mosaic.glm <-	function(x, formula = NULL,
                         panel=mosaic, type=c("observed", "expected"),
                         residuals=NULL,
                         residuals_type = c("pearson", "deviance", "rstandard"),
                         gp = shading_hcl, gp_args = list(), ...)
{

	#require(vcd)
	if (!inherits(x,"glm")) stop("mosaic.glm requires a glm object")

	df.residual <- x$df.residual
	observed <- x$data

        if (is.null(formula) && inherits(observed, "table"))
            formula <- reformulate(names(dimnames(observed)))

        if (is.null(formula)) {
            if (is.environment(observed)) observed <- model.frame(x)
            else {
                if (!is.null(x$call$subset))
                    observed <- subset(observed, eval(x$call$subset, observed))
                if (!is.null(x$na.action))
                    observed <- observed[-x$na.action,]
            }
            ## get all factors excluding response
            factors <- sapply(observed, inherits, "factor")
            resp <- as.character(x$formula[[2]])
            factors <- observed[setdiff(colnames(observed[factors]), resp)]
            ## drop unused levels
            for(nm in names(factors)) {
                f <- factors[[nm]]
                if(is.factor(f) &&
                   length(unique(f[!is.na(f)])) < length(levels(f)))
                    factors[[nm]] <- factors[[nm]][, drop = TRUE]
            }
            ok <- TRUE
            ## check cross-classifying
            if (ok <- isTRUE(all(table(factors) == 1))) {
              warning("no formula provided, assuming ",
                      deparse(formula(terms(~ . , data = factors))),
                      "\n", call. = FALSE)
            }
            if (!ok)
              stop("cannot identify indexing factors from ", substitute(x),
                   "$data - please provide formula", call. = FALSE)
        }
        else {
            if (length(formula) == 3) formula <- formula[-2]
            ## get indexing factors allowing for missing data, subset etc
            factors <- do.call("model.frame", list(formula = formula,
                                                   data = observed,
                                                   subset = x$call$subset,
                                                   na.action = na.pass,
                                                   drop.unused.levels = TRUE))
            ## following loop needed due to bug in model.frame.default (fixed for R 2.12)
            for(nm in names(factors)) {
                f <- factors[[nm]]
                if(is.factor(f) && length(unique(f[!is.na(f)])) < length(levels(f)))
                    factors[[nm]] <- factors[[nm]][, drop = TRUE]
            }
            if (!is.null(x$na.action))
                factors <- factors[-x$na.action,]
        }

        if (x$family$family == "poisson") {
            observed <- as.table(tapply(x$y, factors, sum))
            expected <- as.table(tapply(fitted(x), factors, sum))
        }
        else{
            observed <- as.table(tapply(x$prior.weights, factors, sum))
            expected <- as.table(tapply(x$prior.weights * x$weights, factors, sum))
        }
        ## replace any missing values with zero
        observed[is.na(observed)] <- 0 #else strucplot would do this
        expected[is.na(expected)] <- 0

	type <- match.arg(tolower(type), c("observed", "expected"))
	if (any(observed < 0, na.rm = TRUE))
            stop("requires a non-negative response vector")

        ## reshape the residuals to conform to the structure of data

        ## if max one residual per cell, use residuals_type
        if (max(table(factors)) == 1) {
            residuals_type <- match.arg(tolower(residuals_type),
                                        c("pearson", "deviance", "rstandard"))
            if (missing(residuals))
                residuals <- if (residuals_type=="rstandard") rstandard(x)
                else residuals(x, type=residuals_type)
            residuals <- as.table(tapply(residuals, factors, sum))
            df <- x$df.residual
        }
        ## for marginal views, use aggregated working residuals
        else {
            residuals <- meanResiduals(x, factors)
            residuals_type <- "working" #what is this used for?
            df <- attr(residuals, "df")
            if (df == 0) {
                warning("There are zero degrees of freedom ",
                        "for the test of normality")
                df <- NA
            }
        }
        ## replace any missing values with zero
        residuals[is.na(residuals)] <- 0

	gp <- if (inherits(gp, "grapcon_generator"))
            do.call("gp", c(list(observed, residuals, expected, df),
                            as.list(gp_args)))
        else gp

	panel(observed, residuals=residuals, expected=expected, type=type,
              residuals_type=residuals_type, gp=gp, ...)
}

## convenience functions for sieve and assoc plots

#' @rdname mosaic.glm
#' @importFrom vcd sieve
#' @export
sieve.glm <-
		function (x, ...)
{
	mosaic(x, panel = sieve, ...)
}

#' @rdname mosaic.glm
#' @importFrom vcd assoc
#' @export
assoc.glm <-
		function (x, ...)
{
	mosaic(x, panel = assoc, ...)
}

#' Sequential Mosaics and Strucplots for an N-way Table
#'
#' This function takes an n-way contingency table and plots mosaics for series
#' of sequential models to the 1-, 2-, ... n-way marginal tables, corresponding
#' to a variety of types of loglinear models.
#'
#' This function produces similar plots to the use of
#' \code{\link{mosaic.loglmlist}}, called with the result of
#' \code{\link{seq_loglm}}.
#'
#' @param x a contingency table in array form, with optional category labels
#'        specified in the `dimnames(x)` attribute, or else a data.frame in frequency
#'        form, with the frequency variable named `"Freq"`.
#' @param panel a \code{\link[vcd]{strucplot}} panel function, typically
#'        \code{\link[vcd]{mosaic}} or \code{\link[vcd]{sieve}. NOT yet implemented.}
#' @param type type of sequential model to fit, a character string. One of
#'        `"joint"`, `"conditional"`, `"mutual"`, `"markov"`, or
#'         `"saturated"`.
#' @param plots which marginal sub-tables to plot? A vector of a (sub)set of
#'        the integers, `1:nf` where `nf` is the number of factors in the
#'        full n-way table.
#' @param vorder order of variables, a permutation of the integers `1:nf`,
#'        used to reorder the variables in the original table for the purpose of
#'        fitting sequential marginal models.
#' @param k conditioning variable(s) for `type` = `"joint"`,
#'        `"conditional"` or Markov chain order for `type` = `"markov"`
#' @param \dots other arguments passed to \code{\link[vcd]{mosaic}}.
#'
#' @return None. Used for its side-effect of producing plots
#'
#' @author Michael Friendly
#' @seealso \code{\link{loglin-utilities}} for descriptions of sequential
#'         models, \code{\link{conditional}}, \code{\link{joint}}, \code{\link{mutual}}, \dots{}
#'         \code{\link{loglmlist}}, \code{\link{mosaic.loglmlist}},
#'         \code{\link{seq_loglm}}
#'
#'         \code{\link{mosaic.glm}}, \code{\link[vcd]{mosaic}},
#'         \code{\link[vcd]{strucplot}}, for the many parameters that control the details of mosaic plots.
#' @references
#' These functions were inspired by the original SAS implementation
#' of mosaic displays, described in the *User's Guide for Mosaics*,
#' <http://www.datavis.ca/mosaics/mosaics.pdf>
#' @keywords hplots
#' @examples
#'
#'
#' data(Titanic, package="datasets")
#'
#' seq_mosaic(Titanic)  # models of joint independence, Survived last
#' seq_mosaic(Titanic, type="condit")
#' seq_mosaic(Titanic, type="mutual")
#'
#' # other panel functions and options
#' seq_mosaic(Titanic, type="mutual", panel=sieve,
#'    gp=shading_Friendly, labeling=labeling_values)
#'
#'
#' @export seq_mosaic
seq_mosaic <- function(
	x,
	panel = mosaic,
	type = c("joint", "conditional", "mutual", "markov", "saturated"),
	plots = 1:nf,      # which plots to produce?
	vorder = 1:nf,      # order of variables in the sequential plots
	k = NULL,          # conditioning variable(s) for "joint", "conditional" or order for "markov"
	...
	)
{

  if (inherits(x, "data.frame") && "Freq" %in% colnames(x)) {
    x <- xtabs(Freq ~ ., data=x)
  }
  if (!inherits(x, c("table", "array"))) stop("not an xtabs, table, array or data.frame with a 'Freq' variable")

	nf <- length(dim(x))
	x <- aperm(x, vorder)
	factors <- names(dimnames(x))
	indices <- 1:nf

  type = match.arg(type)
	for (i in plots) {
		mtab <- margin.table(x, 1:i)
		df <- NULL
		if (i==1) {
		  expected <- mtab
		  expected[] <- sum(mtab) / length(mtab)
		  df <- length(mtab)-1
		  model.string = paste("=", factors[1])
		  }
		else {
  		expected <- switch(type,
  			'conditional' = conditional(i, mtab, with=if(is.null(k)) i else k),
  			'joint' = joint(i, mtab, with=if(is.null(k)) i else k),
  			'mutual' = mutual(i, mtab),
  			'markov' = markov(i, mtab, order=if(is.null(k)) 1 else k),
  			'saturated' = saturated(i, mtab)
  			)
    model.string <- loglin2string(expected, brackets=if (i<nf) '()' else '[]')
  	}
		panel(mtab, expected=expected, df=df, main=model.string, ...)
	}
}

#' Deprecated functions in vcdExtra
#'
#' These functions are provided for compatibility with older versions of
#' vcdExtra and will be removed in a future release.
#'
#' \itemize{
#'   \item [Summarise()] is deprecated in favor of [LRstats()], which provides
#'     the same brief model-fit comparison under a name that doesn't
#'     near-collide with `dplyr::summarise()`.
#' }
#'
#' @name vcdExtra-deprecated
#' @keywords internal
NULL

#' Cut a Numeric Variable to a Factor
#'
#' @description
#'
#' `cutfac` acts like \code{\link[base]{cut}}, dividing the range of
#' `x` into intervals and coding the values in `x` according in which
#' interval they fall. However, it gives nicer labels for the factor levels and
#' by default chooses convenient breaks among the values based on deciles.
#'
#' It is particularly useful for plots in which one wants to make a numeric
#' variable discrete for the purpose of getting boxplots, spinograms or mosaic
#' plots.
#'
#' @details
#' By default, \code{\link[base]{cut}} chooses breaks by equal lengths of the
#' range of `x`, whereas `cutfac` uses \code{\link[stats]{quantile}}
#' to choose breaks of roughly equal count.
#'
#' @param x a numeric vector which is to be converted to a factor by cutting
#' @param breaks either a numeric vector of two or more unique cut points or a
#'          single number (greater than or equal to 2) giving the number of intervals
#'          into which `x` is to be cut.
#' @param q the number of quantile groups used to define `breaks`, if that
#'          has not been specified.
#'
#' @return A \code{\link[base]{factor}} corresponding to `x` is returned
#'
#' @author Achim Zeileis
#'
#' @seealso \code{\link[base]{cut}}, \code{\link[stats]{quantile}}
#'
#' @references
#' Friendly, M. and Meyer, D. (2016).  *Discrete Data Analysis
#' with R: Visualization and Modeling Techniques for Categorical and Count
#' Data*.  Boca Raton, FL: Chapman & Hall/CRC. <http://ddar.datavis.ca>.
#' @keywords manip
#' @examples
#'
#' if (require(AER)) {
#' data("NMES1988", package="AER")
#' nmes <- NMES1988[, c(1, 6:8, 13, 15, 18)]
#'
#' plot(log(visits+1) ~ cutfac(chronic),
#'   data = nmes,
#'   ylab = "Physician office visits (log scale)",
#'   xlab = "Number of chronic conditions", main = "chronic")
#'
#' plot(log(visits+1) ~ cutfac(hospital, c(0:2, 8)),
#'   data = nmes,
#'   ylab = "Physician office visits (log scale)",
#'   xlab = "Number of hospital stays", main = "hospital")
#' }
#'
#' data("CrabSatellites", package = "vcdExtra")
#'
#' # jittered scatterplot
#' plot(jitter(satellites) ~ width, data=CrabSatellites,
#'   ylab="Number of satellites (jittered)",
#'   xlab="Carapace width",
#'   cex.lab=1.25)
#' with(CrabSatellites,
#'      lines(lowess(width, satellites), col="red", lwd=2))
#'
#' # boxplot, using deciles
#' plot(satellites ~ cutfac(width), data=CrabSatellites,
#'      ylab="Number of satellites",
#'      xlab="Carapace width (deciles)")
#'
#'
#'
#' @export cutfac
cutfac <- function(x, breaks = NULL, q=10) {
  if(is.null(breaks)) breaks <- unique(quantile(x, 0:q/q))
  x <- cut(x, breaks, include.lowest = TRUE, right = FALSE)
  levels(x) <- paste(breaks[-length(breaks)], ifelse(diff(breaks) > 1,
    c(paste("-", breaks[-c(1, length(breaks))] - 1, sep = ""), "+"), ""), sep = "")
  return(x)
}

#' Convert frequency or case form data into table form
#' 
#' Converts object (`obj`) in frequency or case form into table form. The 
#' column containing the frequencies (`freq`) must be supplied if `obj` is in 
#' frequency form. Optionally returns a table of proportions with (optionally) specified margins.
#' 
#' @param obj 
#'  Object to be converted to table form.
#' @param freq 
#'  If `obj` is in frequency form, this is the name of the frequency column. 
#'  Leave as `NULL` if `obj` is in any other form.
#' @param dims 
#'  A character vector of dimensions. If not specified, all variables apart from 
#'  `freq` will be used as dimensions.
#' @param prop 
#'  If set to `TRUE`, returns a table of proportions (that sum to 1). May also 
#'  be set to a character or numeric vector of dimensions to be used as margins 
#'  from which proportions will be computed.
#' @return Object in table form.
#' 
#' @details
#' If `obj` was in table form to begin with, it is returned to the user as-is
#' unless dimensions were specified (in which case it returns a table with
#' entries summed over excluded dimensions). When `prop` is set to `TRUE`, the 
#' returned table will have proportions that sum to one, whereas if a character 
#' or numerical vector of table dimensions is supplied to `prop`, proportions 
#' will be marginalized across the specified dimensions.
#' 
#' @author Gavin M. Klorfine
#' 
#' @seealso
#' \code{\link{as_freqform}}, \code{\link{as_caseform}}, \code{\link{as_array}},
#' \code{\link{as_matrix}}
#' 
#' @importFrom stats reformulate xtabs
#' @importFrom methods is
#' 
#' @examples
#' library(vcdExtra)
#' 
#' data("HairEyeColor")
#' 
#' freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
#' tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
#' caseForm <- expand.dft(freqForm) # Generate case form data
#' 
#' # Frequency form -> table form
#' as_table(freqForm, freq = "Freq") |> str()
#' 
#' # Warned if forgot to specify freq
#' as_table(freqForm) |> str()
#' 
#' # Frequency form (tibble) -> table form
#' as_table(tidy_freqForm, freq = "n") |> str()
#' 
#' # Case form -> table form
#' as_table(caseForm) |> str()
#' 
#' # For specific dimensions
#' as_table(tidy_freqForm, freq = "n", dims = c("Hair", "Eye")) |> str()
#' 
#' #-----For proportions-----#
#' 
#' as_table(freqForm, freq = "Freq", prop = TRUE) |> head(c(4,4,1)) # print only Sex == Male rows
#' 
#' # Marginalize proportions along "Sex" (i.e., male proportions sum to 1, female proportions sum to 1)
#' as_table(freqForm, freq = "Freq", prop = "Sex") |> head(c(4,4,1))
#' 
#' as_table(freqForm, freq = "Freq", prop = 3) |> head(c(4,4,1)) # Same as above
#' 
#' # Marginalize proportions along multiple variables
#' as_table(freqForm, freq = "Freq", prop = c("Hair", "Sex")) |> head(c(4,4,1))
#' 
#' as_table(freqForm, freq = "Freq", prop = c(1, 3)) |> head(c(4,4,1)) # Same as above
#' 
#' # Using dims and prop arguments in tandem
#' as_table(freqForm, freq = "Freq", dims = c("Hair", "Eye"), prop = TRUE)
#' 
#' 
#' @export

as_table <- function(obj, freq = NULL, dims = NULL, prop = NULL){
  
  # If user supplied a table or array, remember that
  tab_or_array <- FALSE
  if (is(obj, "table") || is(obj, "array")){
    
    tab <- as.table(obj) # Handle arrays
    
    # To include dimensions if specified
    if (!is.null(dims)){
      tab <- margin.table(tab, margin = dims)
    }
    
    tab_or_array <- TRUE
  }
  # If obj is a tibble, convert to data frame
  else if (is(obj, "tbl")){
    obj <- as.data.frame(obj)
  }
  
  if (!is.null(dims)){ # If dims supplied by user, use those
    cols <- dims
  }
  else { # If dims NOT supplied by user, use everything else
    cols <- colnames(obj)
  }
  
  if (!tab_or_array){ # If not a table or array...
    if (!is.null(freq)){  # If freq supplied by user, then... (freq form)
      cols <- cols[cols != freq] # Remove freq column
      tab <- xtabs(reformulate(cols, response = freq), data = obj) # freq ~ cols
    }
    else if (is.null(freq)){ # If freq NOT supplied by user, and not array, then... (case form)
      tab <- xtabs(reformulate(cols), data = obj)
      
      # Check if user forgot to supply freq, warn if they potentially forgot
      common <- c("n", "freq", "frequency", "count")
      if (length(intersect(tolower(colnames(obj)), common)) > 0){
        warning("Ensure a value for 'freq' was supplied if your data was in frequency form.")
      }
    }
  }
  
  
  if (!is.null(prop)){ # If user wants proportions
    
    if (is(prop, "logical") && prop == TRUE){ # If margins not specified
      tab <- prop.table(tab)
    }
    else if (is.character(prop) || is.numeric(prop)){ # If proportions are to be marginal
      
      ### Make sure margins are not problematic ###
      if (length(prop) > length(dim(tab))){ # Raise error if # of margin dims exceeds actual # of dims
        stop("Number of specified margins in `prop` exceeds number of dims in table.")
      }
      if (length(unique(prop)) != length(prop)){ # Make sure prop margins are unique
        stop("`prop` margins are not unique (i.e., a duplicate was provided).")
      }
      # If character vector for proportion margins, make sure margins in prop are a subset of dims in table
      if (is.character(prop) && length(prop) != length(intersect(prop, names(dimnames(tab))))){ 
        stop("Ensure all margins specified in `prop` are dims in the table.")
      }
      # If numeric vector for proportion margins, 
      else if(is.numeric(prop)){
        if (!all(prop > 0) || !all(prop %% 1 == 0)){ # Make sure margins in prop are positive, whole nums
          stop("Ensure all margins specified in `prop` are positive whole numbers.")
        }
        if (!all(prop <= length(dim(tab)))){ # Make sure margins in prop do not exceed # of dims in table
          stop("Ensure all margins specified in `prop` do not exceed the total number of dims in the table.")
        }
      }
      ### 
      
      tab <- prop.table(tab, margin = prop)
    }
    else{ # If prop not TRUE or a numeric / character vector
      stop("Argument `prop` must be supplied with either a numeric or character vector")
    }
  }
  
  
  return(tab)
}

# ✔️DONE: fixed buglet when deviance() returns a null
# ✔️DONE: fixed bug: residual df calculated incorrectly but this now depends on objects having a df.residual component. This is TRUE for lm, glm, polr, negbin objects
# ✔️DONE: made generic, adding a glmlist method
# ✔️DONE: Provide a `label` argument to allow use of symbolic formulas as labels for the models, via `get_models()`



#' Brief Summary of Model Fit for glm and loglm Models
#'
#' For `glm` objects, the `print` and `summary` methods give too
#' much information if all one wants to see is a brief summary of model
#' goodness of fit, and there is no easy way to display a compact comparison of
#' model goodness of fit for a collection of models fit to the same data. All
#' `loglm` models have equivalent glm forms, but the `print` and
#' `summary` methods give quite different results.
#'
#' `LRstats` provides a brief summary for one or more models fit to the
#' same dataset for which `logLik` and `nobs` methods exist (e.g.,
#' `glm` and `loglm` models).
#'
#' The function relies on residual degrees of freedom for the LR chisq test
#' being available in the model object.  This is true for objects inheriting
#' from `lm`, `glm`, `loglm`, `polr` and `negbin`.
#'
#' @aliases LRstats LRstats.glmlist LRstats.loglmlist LRstats.default
#'
#' @param object a fitted model object for which there exists a logLik method
#'        to extract the corresponding log-likelihood
#' @param \dots optionally more fitted model objects
#' @param saturated saturated model log likelihood reference value (use 0 if
#'        deviance is not available)
#' @param sortby either a numeric or character string specifying the column in
#'        the result by which the rows are sorted (in decreasing order)
#' @param label character string specifying how to label the rows: `"name"`
#'         (default) uses the model object names; `"formula"` uses model formulas or
#'         bracket notation obtained from \code{\link{get_models}} (for \code{glmlist}
#'         and \code{loglmlist} objects) or \code{\link{get_model}} (for individual
#'         model objects passed to the default method).
#' @param label.args a list of additional arguments passed to \code{\link{get_models}}
#'         when `label = "formula"`. Useful arguments include `abbrev` (logical or integer)
#'         to abbreviate factor names and `sep` to change the separator in bracket notation.
#'
#' @return A data frame (also of class `anova`) with columns
#' `       c("AIC", "BIC", "LR Chisq", "Df", "Pr(>Chisq)")`. Row names are taken
#'         from the names of the model object(s) or their model formulas.
#'
#' @author Achim Zeileis, Michael Friendly
#'
#' @seealso \code{\link[stats]{logLik}}, \code{\link[stats]{glm}},
#' \code{\link[MASS]{loglm}},
#'
#' \code{\link{logLik.loglm}}, \code{\link{modFit}},
#' \code{\link{get_models}}, \code{\link{get_model}}
#'
#' @family glmlist functions
#' @keywords models
#' @examples
#'
#' data(Mental)
#' indep <- glm(Freq ~ mental+ses,
#'                 family = poisson, data = Mental)
#' LRstats(indep)
#' Cscore <- as.numeric(Mental$ses)
#' Rscore <- as.numeric(Mental$mental)
#'
#' coleff <- glm(Freq ~ mental + ses + Rscore:ses,
#'                 family = poisson, data = Mental)
#' roweff <- glm(Freq ~ mental + ses + mental:Cscore,
#'                 family = poisson, data = Mental)
#' linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
#'                 family = poisson, data = Mental)
#'
#' # compare models using object names (default)
#' LRstats(indep, coleff, roweff, linlin)
#'
#' # compare models in a glmlist, using formula labels
#' mods <- glmlist(indep, coleff, roweff, linlin)
#' LRstats(mods, label = "formula")
#'
#' # loglmlist example with bracket notation labels
#' data(Titanic)
#' tit.joint <- seq_loglm(Titanic, type = "joint")
#' LRstats(tit.joint)
#' LRstats(tit.joint, label = "formula")
#' LRstats(tit.joint, label = "formula", label.args = list(abbrev = TRUE))
#'
#' @export
LRstats <- function(object, ...) {
	UseMethod("LRstats")
}

#' @rdname LRstats
#' @export
LRstats.glmlist <- function(object, ..., saturated = NULL, sortby = NULL,
                            label = c("name", "formula"), label.args = list()) {
    label <- match.arg(label)
    ns <- sapply(object, function(x) length(x$residuals))
    if (any(ns != ns[1L]))
        stop("models were not all fitted to the same size of dataset")
    nmodels <- length(object)
    if (nmodels == 1)
        return(LRstats.default(object[[1L]], saturated = saturated))

    rval <- lapply(object, LRstats.default, saturated = saturated)
    rval <- do.call(rbind, rval)

    if (label == "formula") {
        rownames(rval) <- do.call(get_models, c(list(x = object), label.args))
    }

    if (!is.null(sortby)) {
        rval <- rval[order(rval[, sortby], decreasing = TRUE), ]
    }
    rval
}

# could just do LRstats.loglmlist <- LRstats.glmlist

#' @rdname LRstats
#' @export
LRstats.loglmlist <- function(object, ..., saturated = NULL, sortby = NULL,
                              label = c("name", "formula"), label.args = list()) {
    label <- match.arg(label)
    ns <- sapply(object, function(x) length(x$residuals))
    if (any(ns != ns[1L]))
        stop("models were not all fitted to the same size of dataset")
    nmodels <- length(object)
    if (nmodels == 1)
        return(LRstats.default(object[[1L]], saturated = saturated))

    rval <- lapply(object, LRstats.default, saturated = saturated)
    rval <- do.call(rbind, rval)

    if (label == "formula") {
        rownames(rval) <- do.call(get_models, c(list(x = object), label.args))
    }

    if (!is.null(sortby)) {
        rval <- rval[order(rval[, sortby], decreasing = TRUE), ]
    }
    rval
}

#' @rdname LRstats
#' @export
LRstats.default <- function(object, ..., saturated = NULL, sortby = NULL,
                            label = c("name", "formula"), label.args = list())
{
  label <- match.arg(label)

  ## interface methods for logLik() and nobs()
  ## - use S4 methods if loaded
  ## - use residuals() if nobs() is not available
  logLik0 <- if("stats4" %in% loadedNamespaces()) stats4::logLik else logLik
  nobs0   <- function(x, ...) {
    nobs1 <- if("stats4" %in% loadedNamespaces()) stats4::nobs else nobs
    nobs2 <- function(x, ...) NROW(residuals(x, ...))
    rval <- try(nobs1(x, ...), silent = TRUE)
    if(inherits(rval, "try-error") | is.null(rval)) rval <- nobs2(x, ...)
    return(rval)
  }
  dof <- function(x) {
  	if (inherits(x, "loglm")) {
  		rval <- x$df
  		} else {
  		rval <- try(x$df.residual, silent=TRUE)
  		}
  	if (inherits(rval, "try-error") || is.null(rval)) stop(paste("Can't determine residual df for a", class(x), "object"))
  	rval
  	}

  ## collect all objects
  objects <- list(object, ...)
  nmodels <- length(objects)

  ## check sample sizes
  ns <- sapply(objects, nobs0)
  if(any(ns != ns[1L])) stop("models were not all fitted to the same size of dataset")

  ## extract log-likelihood and df (number of parameters)
  ll <- lapply(objects, logLik0)
  par <- as.numeric(sapply(ll, function(x) attr(x, "df")))
	df <- as.numeric(sapply(objects, function(x) dof(x)))
  ll <- sapply(ll, as.numeric)

  ## compute saturated reference value (use 0 if deviance is not available)
  if(is.null(saturated)) {
    dev <- try(sapply(objects, deviance), silent = TRUE)
    if(inherits(dev, "try-error") || any(sapply(dev, is.null))) {
      saturated <- 0
    } else {
      saturated <- ll + dev/2
    }
  }

  ## setup ANOVA-style matrix
  rval <- matrix(rep(NA, 5 * nmodels), ncol = 5)
  colnames(rval) <- c("AIC", "BIC", "LR Chisq", "Df", "Pr(>Chisq)")

  if (label == "formula") {
    rownames(rval) <- sapply(objects, function(mod) {
      do.call(get_model, c(list(x = mod), label.args))
    })
  } else {
    rownames(rval) <- as.character(sapply(match.call(), deparse)[-1L])[1:nmodels]
  }

  rval[,1] <- -2 * ll + 2 * par
  rval[,2] <- -2 * ll + log(ns) * par
  rval[,3] <- -2 * (ll - saturated)
  rval[,4] <- df
  rval[,5] <- pchisq(rval[,3], df, lower.tail = FALSE)

	if (!is.null(sortby)) {
		rval <- rval[order(rval[,sortby], decreasing=TRUE),]
	}

  ## return
  structure(as.data.frame(rval), heading = "Likelihood summary table:", class = c("anova", "data.frame"))
}

#' Collapse the levels of a dataset
#' 
#' Collapses the levels of a dataset (of any form) into those specified. May
#' also be used to re-name levels. Ensure argument \code{freq} is supplied
#' should your data be in frequency form (and the frequency column differs in
#' name from default, "Freq").
#'
#' @param x The dataset to be collapsed.
#' @param freq Supply only if your data is in frequency form AND your frequency 
#'  column differs in name from the default ("Freq").
#' @param \dots A collection of one or more assignments of dataset variables to
#'  a list of levels in the format 
#'  \code{new_level = c("old_level_1", "old_level_2", ..., "old_level_n")}.
#' @return The collapsed dataset in its original form (i.e., the initial form of
#'  \code{x}).
#'  
#' @details
#' First converts the object \code{x} into a frequency form data frame. Then, 
#' \code{\link[forcats]{fct_collapse}} is used to collapse variable levels.
#' Next, duplicate rows (an artefact of collapsing) are aggregated via
#' \code{\link[dplyr]{summarise}}. Last, the frequency form data frame is
#' converted back into the initial form of object \code{x}.
#' 
#' The exceptions to this are objects in case form, which are passed directly
#' to \code{\link[forcats]{fct_collapse}} (and duplicate rows are not aggregated).
#'
#' @author Gavin M. Klorfine
#'
#' @seealso
#' \code{\link[forcats]{fct_collapse}}, \code{\link{collapse.table}}
#' 
#' Tidy conversion functions:
#' \code{link{as_table}}, \code{link{as_freqform}}, \code{link{as_caseform}},
#' \code{link{as_matrix}}, \code{link{as_array}},
#'
#' @importFrom methods is
#' @importFrom rlang set_names .data
#' @importFrom dplyr as_tibble summarise all_of
#' @importFrom forcats fct_collapse
#' 
#' @examples
#' data("HairEyeColor") # Table form data
#' str(HairEyeColor)
#' 
#' collapse_levels(
#'   HairEyeColor,                 # Dataset
#'   Hair = list(                  # List of arguments for first variable
#'     Dark = c("Black", "Brown"), # Collapse "Black" and "Brown" -> "Dark"
#'     Light = c("Blond", "Red")   # Collapse "Blond" and "Red" -> "Light"
#'   ),
#'   Eye = list(                   # List of arguments for second variable
#'     Common = c("Brown"),        # Collapse (rename) "Brown" -> "Common"
#'     Uncommon = c("Blue", "Green", "Hazel")
#'   )
#' ) |> str()
#' 
#' # To illustrate `freq` argument usage, convert Hoyt dataset to frequency form 
#' # (ff) and then rename frequency column to "n"
#' 
#' data("Hoyt", package = "vcdExtra")
#' ff_Hoyt <- as_freqform(Hoyt)
#' names(ff_Hoyt)[length(ff_Hoyt)] <- "n"
#' str(ff_Hoyt)
#' 
#' collapse_levels(
#'   ff_Hoyt,
#'   
#'   # Ensure to supply if data is in frequency form and frequency column name
#'   # differs from "Freq"
#'   freq = "n",
#'   
#'   Occupation = list(
#'     High = c(1, 2),
#'     Middle = 3,
#'     Low = 4,
#'     VeryLow = c(5, 6, 7)
#'   )
#' ) |> str()
#' 
#'
#' @export

collapse_levels <- function(x, freq = "Freq", ...){
  
  argms <- list(...)
  x_class <- NULL
  cf <- FALSE
  
  if (is(x, "array"))
    x_class <- "array"
  else if (is(x, "matrix"))
    x_class <- "matrix"
  else if (is(x, "table"))
    x_class <- "table"
  else if (is(x, "tbl"))
    x_class <- "tbl"
  else if (is(x, "data.frame"))
    x_class <- "data.frame"
  
  # Handle case form data
  if (!(freq %in% names(x)) && (is(x, "data.frame") || is(x, "tbl"))) {
    for (i in 1:length(argms)) {
      var <- names(argms)[i]
      x[[var]] <- forcats::fct_collapse(
        x[[var]], 
        !!!rlang::set_names(argms[[i]], names(argms[[i]]))
      )
    }
    return(x)
  }
  else # If not case form, convert to data frame
    coll_x <- as_freqform(x, freq = freq, tidy = FALSE)
  
  freq <- "Freq"
  
  fact_lvls <- list() # Initialize list
  
  # Iterate through and collapse factors according to specified levels
  for (i in 1:length(names(argms))){
    
    # Gather the specified levels
    for (j in 1:length(argms[[i]])){
      fact_lvls <- append(fact_lvls, list(argms[[i]][[j]]))
    }
    
    # Collapse factor according to specified levels
    coll_x[[names(argms)[i]]] <- forcats::fct_collapse(
      coll_x[[names(argms)[i]]], 
      !!!rlang::set_names(fact_lvls, names(argms[[i]]))
    )
    
    fact_lvls <- list() # Reset list for the next factor
  }
  
  # Combine duplicate rows
  groups = setdiff(names(coll_x), freq)
  coll_x <- coll_x |> 
    dplyr::summarise(Freq = sum(.data[[freq]]), .by = dplyr::all_of(groups))
  
  # Return the user a collapsed object of same type as their input
  if (!is.null(x_class)){
    if (x_class == "array")
      coll_x <- as_array(coll_x, freq = "Freq")
    else if (x_class == "matrix")
      coll_x <- as_matrix(coll_x, freq = "Freq")
    else if (x_class == "table")
      coll_x <- as_table(coll_x, freq = "Freq")
    else if (x_class == "tbl")
      coll_x <- dplyr::as_tibble(coll_x)
  }
  return (coll_x)
}

# Plotting logistic regressions with marginal distributions of Y
#
# `logist_plot()`: plots a `glm(y ~ x, family = binomial)` fit for a single quantitative
# predictor, with a representation of the marginal distribution of cases for which y==0 vs.
# y==1 (histogram, density, or jittered points), optionally conditioning the fit and supported
# marginals on a grouping variable.
#
# Suggested in: Smart et al. (2004), A New Means of Presenting the Results of Logistic Regression,
# Bulletin of the Ecological Society of America, 85(3),
# https://esapubs.org/bulletin/backissues/085-3/bulletinjuly2004_2column.htm#tools1
# See also: vcd::binreg_plot() (similar, using {grid} graphics directly) and
# popbio::logi.hist.plot(). Original sketch: Scott Chamberlain,
# https://recology.info/2012/01/logistic-regression-barplot-fig/
#
# Design decisions and the full review-and-fix history this file went through are recorded in
# logist-plot-history.md, not here.

# ---- public generic + methods ------------------------------------------------------------

#' Plot a fitted logistic regression with marginal distributions of the predictor
#'
#' Plots predicted probabilities from a `glm(y ~ x, family = binomial)` fit for a single
#' quantitative predictor `x` and binary response `y`, and also with the smoothed
#' logistic fit and its confidence band.
#' What this plot method adds is a representation of
#' the marginal distribution of `x` within each `y` group -- mirrored histograms or filled
#' density estimates above and below the curve, or jittered points -- as suggested by Smart et
#' al. (2004). These help you
#' see where the data supporting the fit exist; e.g., where the data are "thin", so the confidence band is wide.
#'
#' `logist_plot()` is generic, with methods for a pair of vectors, a data frame, or a
#' model formula. `logist_hist()`, `logist_point()`, and `logist_density()` are convenience
#' wrappers with `marginal=` fixed to `"hist"`/`"points"`/`"density"`, but otherwise accept
#' the same `x`/`...` as `logist_plot()` -- i.e., they also work with a data frame or a formula.
#' An optional `group` produces separate fits and colour identities in point and density
#' modes. Grouped histograms are deliberately unsupported because stacked or overlapping
#' mirrored bars obscure both the distributions and the fitted curves.
#' Grouped density lanes begin at 0 and 1 and stack outward in fixed, narrow bands. Each
#' group's two response-specific densities are normalized together within that band, so the
#' shapes show conditional distributions but do not encode group sample sizes.
#'
#' `bins`, `adjust`, and `marginal.height` control computation of the marginal geometry;
#' they are not graphical layer arguments and therefore belong at the top level rather than
#' inside `marginal.args`.
#' When a small histogram height would crowd the secondary count axis, its integer ticks are
#' thinned symmetrically while retaining the correct count-to-height mapping.
#'
#' All marginal modes return a native ggplot object. Standard additions such as
#' [ggplot2::labs()] and [ggplot2::theme()] can therefore be applied after construction.
#' Adding another `scale_y_*()` replaces the internally configured probability scale; in
#' histogram mode this can remove or invalidate the secondary count-axis mapping.
#'
#' @param x a numeric predictor vector or a data frame; see `formula` below for the
#'   model-formula interface
#' @param ... arguments passed to methods, or on to `logist_plot()` from the convenience
#'   wrappers. Arguments not consumed by the selected method are an error rather than being
#'   silently ignored. Use `fit.args` and `marginal.args` for layer customization.
#'
#' @return A native `ggplot` object that can be extended with ordinary ggplot2 additions.
#' @author Gavin Klorfine, Michael Friendly
#'
#' @family logistic regression plots
#'
#' @seealso [vcd::binreg_plot()], a similar plot using `grid` graphics directly.
#'
#' @references
#' Smart, J. M. R., Sutherland, W. J., Watkinson, A. R., and Gill, J. A. (2004). A New Means of
#' Presenting the Results of Logistic Regression, *Bulletin of the Ecological Society of
#' America*, 85(3), 100--102. \doi{10.1890/0012-9623(2004)85[100:ANMOPT]2.0.CO;2}
#' <https://esapubs.org/bulletin/backissues/085-3/bulletinjuly2004_2column.htm#tools1>
#'
#' @examples
#' data(Donner, package = "vcdExtra")
#'
#' # three interfaces to the same underlying plot
#' logist_plot(Donner$age, Donner$survived, marginal = "points")
#' logist_plot(Donner[, c("age", "survived")], marginal = "hist")
#' logist_plot(survived ~ age, data = Donner, marginal = "density")
#'
#' # post-hoc labels and themes work in histogram mode
#' logist_plot(survived ~ age, data = Donner, marginal = "hist") +
#'   ggplot2::labs(title = "Survival of the Donner Party") +
#'   ggplot2::theme(plot.title = ggplot2::element_text(face = "bold"))
#'
#' # layer-specific customization; scoped lists override graphical defaults
#' logist_plot(
#'   survived ~ age, data = Donner, marginal = "hist",
#'   fit.args = list(linewidth = 2, fill = "lightblue"),
#'   marginal.args = list(colour = "black", linewidth = 0.2, alpha = 0.8)
#' )
#'
#' # convenience wrappers -- marginal= fixed, still get all calling conventions
#' logist_point(survived ~ age, data = Donner)
#' logist_hist(survived ~ age, data = Donner)
#' logist_density(survived ~ age, data = Donner, adjust = 1.25)
#'
#' # grouped fits and marginals; grouped density lanes extend outward from 0 and 1
#' logist_point(survived ~ age, data = Donner, group = "sex")
#' logist_density(
#'   survived ~ age, data = Donner, group = "sex",
#'   group.colors = c(Female = "#D55E00", Male = "#0072B2"),
#'   marginal.args = list(alpha = 0.35, linewidth = 0.6)
#' )
#'
#' @importFrom rlang .data
#' @export
logist_plot <- function(x, ...) {
  UseMethod("logist_plot")
}

#' @param y a binary (0/1, or 2-level factor/character/logical) response vector
#' @param marginal character string, how to represent the marginal distribution of `x` within
#'   each `y` group: `"hist"`, mirrored histograms (default); `"points"`, jittered points; or
#'   `"density"`, mirrored filled density estimates
#' @param bins number of histogram bins, for `marginal = "hist"`; default: 30
#' @param adjust positive numeric bandwidth adjustment passed to [stats::density()] for
#'   `marginal = "density"`; default: 1
#' @param marginal.height optional finite positive number controlling marginal height. For
#'   histograms and ungrouped densities, this is the maximum proportion of the fixed 0--1
#'   probability panel occupied on each side; defaults are 0.25 and 0.15, respectively, and
#'   values cannot exceed 0.5. For grouped densities, it is the full height of each outward
#'   lane in the original probability-coordinate units; the default is 0.05 and values cannot
#'   exceed 1. `NULL` selects the applicable default. For `marginal = "points"`, a non-`NULL`
#'   value is ignored with a warning; control vertical jitter through `marginal.args` instead.
#' @param xlab,ylab axis labels; default to the deparsed `x`/`y` expressions
#' @param fit.color color of the fitted logistic curve and its confidence band; default:
#'   "steelblue". This scalar is inactive when `group` is supplied; use `group.colors`
#'   instead.
#' @param marginal.color color of the marginal representation of `x` within each `y` group
#'   (histogram/density fill, or point color for `marginal = "points"`); default: "orange"
#'   This scalar is inactive when `group` is supplied; use `group.colors` instead.
#' @param group optional grouping input. For the default method, a vector the same length as
#'   `x` and `y`; for data-frame and formula methods, a single column name or position.
#'   Grouping is supported for `marginal = "points"` and `"density"`, but not `"hist"`.
#' @param group.colors optional character vector of colours for grouped plots. An unnamed
#'   vector is applied in group-level order; a named vector must contain every observed group
#'   label. The same palette is used for fits, marginals, and the legend. The default `NULL`
#'   uses ggplot2's discrete scales.
#' @param fit.args named list of graphical arguments for the fitted curve and confidence band.
#'   Values override the defaults established by `fit.color`. The fit remains a binomial GLM,
#'   so `data`, `mapping`, `stat`, `position`, `inherit.aes`, `method`, `formula`, and
#'   `method.args` cannot be replaced. In grouped mode, `colour` and `fill` must instead be
#'   controlled through `group.colors`.
#' @param marginal.args named list of graphical arguments for the active marginal layer.
#'   Values override the defaults established by `marginal.color`. Valid arguments depend on
#'   `marginal`: point aesthetics and `position` for `"points"`, rectangle aesthetics for
#'   `"hist"`, or ribbon aesthetics for `"density"`. Histogram computation remains controlled
#'   by `bins`; density bandwidth remains controlled by `adjust`; and marginal geometry height
#'   remains controlled by `marginal.height`. In grouped mode, `colour` and `fill` must instead
#'   be controlled through `group.colors`.
#' @rdname logist_plot
#' @export
logist_plot.default <- function(x, y, marginal = c("hist", "points", "density"),
                                 bins = 30, adjust = 1, xlab = NULL, ylab = NULL,
                                 fit.color = "steelblue", marginal.color = "orange",
                                 fit.args = list(), marginal.args = list(),
                                 group = NULL, group.colors = NULL,
                                 marginal.height = NULL, ...) {
  xlab <- xlab %||% deparse(substitute(x))
  ylab <- ylab %||% deparse(substitute(y))
  group.label <- if (is.null(group)) NULL else paste(deparse(substitute(group)), collapse = "")
  .logist_plot_impl(x, y, marginal = marginal, bins = bins, adjust = adjust,
                     xlab = xlab, ylab = ylab,
                     fit.color = fit.color, marginal.color = marginal.color,
                     fit.args = fit.args, marginal.args = marginal.args,
                     group = group, group.label = group.label,
                     group.colors = group.colors, marginal.height = marginal.height, ...)
}

#' @param xvar,yvar which columns of `x` to use as predictor/response -- column name or
#'   position; default to the first two columns (matches the original 2-column-data-frame
#'   calling convention)
#' @rdname logist_plot
#' @export
logist_plot.data.frame <- function(x, xvar = 1L, yvar = 2L,
                                    marginal = c("hist", "points", "density"),
                                    bins = 30, adjust = 1, xlab = NULL, ylab = NULL,
                                    fit.color = "steelblue", marginal.color = "orange",
                                    fit.args = list(), marginal.args = list(),
                                    group = NULL, group.colors = NULL,
                                    marginal.height = NULL, ...) {
  if (ncol(x) < 2L) {
    stop("`x` must have at least 2 columns.", call. = FALSE)
  }
  xres <- .resolve_col(x, xvar, "xvar")
  yres <- .resolve_col(x, yvar, "yvar")
  gres <- if (is.null(group)) NULL else .resolve_col(x, group, "group")
  .logist_plot_impl(xres$value, yres$value, marginal = marginal, bins = bins, adjust = adjust,
                     xlab = xlab %||% xres$name, ylab = ylab %||% yres$name,
                     fit.color = fit.color, marginal.color = marginal.color,
                     fit.args = fit.args, marginal.args = marginal.args,
                     group = if (is.null(gres)) NULL else gres$value,
                     group.label = if (is.null(gres)) NULL else gres$name,
                     group.colors = group.colors, marginal.height = marginal.height, ...)
}

#' @param formula a model formula, `y ~ x` -- exactly one response and one predictor;
#'   `formula` method only. The first argument may be passed positionally or as
#'   `formula = y ~ x` (matching base R's `boxplot()`/`lm()` convention) -- unlike the other
#'   methods, it is not named `x`
#' @param data a data frame -- `formula` method only
#' @rdname logist_plot
#' @export
logist_plot.formula <- function(formula, data, marginal = c("hist", "points", "density"),
                                 bins = 30, adjust = 1, xlab = NULL, ylab = NULL,
                                 fit.color = "steelblue", marginal.color = "orange",
                                 fit.args = list(), marginal.args = list(),
                                 group = NULL, group.colors = NULL,
                                 marginal.height = NULL, ...) {
  gres <- if (is.null(group)) NULL else .resolve_col(data, group, "group")
  mf <- stats::model.frame(formula, data = data, na.action = stats::na.pass)
  if (ncol(mf) != 2L) {
    stop("`formula` must have exactly one response and one predictor (y ~ x); found ",
         ncol(mf) - 1L, " predictor(s).", call. = FALSE)
  }
  .logist_plot_impl(mf[[2]], mf[[1]], marginal = marginal, bins = bins, adjust = adjust,
                     xlab = xlab %||% names(mf)[2], ylab = ylab %||% names(mf)[1],
                     fit.color = fit.color, marginal.color = marginal.color,
                     fit.args = fit.args, marginal.args = marginal.args,
                     group = if (is.null(gres)) NULL else gres$value,
                     group.label = if (is.null(gres)) NULL else gres$name,
                     group.colors = group.colors, marginal.height = marginal.height, ...)
}

# ---- convenience wrappers (fixed marginal=) ------------------------------------------------

#' @rdname logist_plot
#' @export
logist_hist <- function(...) {
  logist_plot(..., marginal = "hist")
}

#' @rdname logist_plot
#' @export
logist_point <- function(...) {
  logist_plot(..., marginal = "points")
}

#' @rdname logist_plot
#' @export
logist_density <- function(...) {
  logist_plot(..., marginal = "density")
}

# if (FALSE) {
#   data(Donner, package = "vcdExtra")
#
#   # three interfaces to the general function
#   logist_plot(Donner$age, Donner$survived, marginal = "hist")
#   logist_plot(Donner[, c("age", "survived")], marginal = "points")
#   logist_plot(survived ~ age, data = Donner, marginal = "hist")
#
#   # xvar=/yvar= to pick columns by name from a wider data frame
#   logist_plot(Donner, xvar = "age", yvar = "survived", marginal = "hist")
#
#   # convenience wrappers -- marginal= fixed, still get all 3 calling conventions
#   logist_hist(survived ~ age, data = Donner)
#   logist_point(survived ~ age, data = Donner)
#   logist_density(survived ~ age, data = Donner, adjust = 1.25)
#   logist_point(Donner[, c("age", "survived")])
# }

# ---- internal helpers (not exported) ----------------------------------------------------

`%||%` <- function(a, b) if (is.null(a)) b else a

# Resolve a xvar=/yvar= selector to a single column. Numeric positions are extracted directly
# by position (never converted to a name and re-looked-up), so this still works correctly when
# `x` has duplicate column names. Character selectors must match exactly one column name.
.resolve_col <- function(x, var, argname) {
  if (length(var) != 1L || is.na(var)) {
    stop("`", argname, "` must be a single column name or position.", call. = FALSE)
  }
  if (is.character(var)) {
    idx <- which(names(x) == var)
    if (length(idx) == 0L) {
      stop("`", argname, "` does not identify an existing column of `x`.", call. = FALSE)
    }
    if (length(idx) > 1L) {
      stop("`", argname, "` (\"", var, "\") matches more than one column of `x`; ",
           "column names must be unique for name-based selection.", call. = FALSE)
    }
    list(value = x[[idx]], name = var)
  } else if (is.numeric(var)) {
    if (var != floor(var) || var < 1L || var > ncol(x)) {
      stop("`", argname, "` must be a whole number between 1 and ", ncol(x), ".", call. = FALSE)
    }
    idx <- as.integer(var)
    list(value = x[[idx]], name = names(x)[idx])
  } else {
    stop("`", argname, "` must be a single column name (character) or position (integer).",
         call. = FALSE)
  }
}

.check_bins <- function(bins) {
  if (length(bins) != 1L || !is.numeric(bins) || is.na(bins) ||
      !is.finite(bins) || bins < 1 || bins != floor(bins)) {
    stop("`bins` must be one positive whole number.", call. = FALSE)
  }
}

.check_adjust <- function(adjust) {
  if (length(adjust) != 1L || !is.numeric(adjust) || is.na(adjust) ||
      !is.finite(adjust) || adjust <= 0) {
    stop("`adjust` must be one finite positive number.", call. = FALSE)
  }
}

.check_marginal_height <- function(marginal.height, grouped = FALSE) {
  if (length(marginal.height) != 1L || !is.numeric(marginal.height) ||
      is.na(marginal.height) || !is.finite(marginal.height) || marginal.height <= 0) {
    stop("`marginal.height` must be one finite positive number.", call. = FALSE)
  }

  upper <- if (grouped) 1 else 0.5
  if (marginal.height > upper) {
    context <- if (grouped) "grouped density lanes" else "histograms and ungrouped densities"
    stop("`marginal.height` cannot exceed ", upper, " for ", context, ".",
         call. = FALSE)
  }
}

# Convert a supported discrete grouping vector to a factor with deterministic levels.
# Existing factor order is meaningful and therefore retained; other supported types are
# sorted independently of row order so colours, legends, and density lanes remain stable.
.as_group_factor <- function(group) {
  if (!is.null(dim(group))) {
    stop("`group` must be a plain vector, not a matrix/array/data.frame.", call. = FALSE)
  }
  if (is.list(group)) {
    stop("`group` must be a plain atomic vector, not a list.", call. = FALSE)
  }
  if (!(is.factor(group) || is.character(group) || is.logical(group) || is.numeric(group))) {
    stop("`group` must be numeric, logical, factor, or character; found class \"",
         paste(class(group), collapse = "/"), "\".", call. = FALSE)
  }

  observed <- group[!is.na(group)]
  levs <- if (is.factor(group)) {
    levels(group)[levels(group) %in% as.character(observed)]
  } else if (is.logical(group)) {
    c(FALSE, TRUE)[c(FALSE, TRUE) %in% observed]
  } else if (is.numeric(group)) {
    sort(unique(observed))
  } else {
    sort(unique(observed), method = "radix")
  }
  factor(as.character(group), levels = as.character(levs))
}

# Validate and order an optional manual palette against the observed group levels.
.check_group_colors <- function(group.colors, group.levels) {
  if (is.null(group.colors)) {
    return(NULL)
  }
  if (!is.character(group.colors) || !length(group.colors) ||
      anyNA(group.colors) || any(group.colors == "")) {
    stop("`group.colors` must be a non-empty character vector of valid colours.",
         call. = FALSE)
  }

  color_names <- names(group.colors)
  if (!is.null(color_names) && any(nzchar(color_names))) {
    if (anyNA(color_names) || any(color_names == "") || anyDuplicated(color_names)) {
      stop("Named `group.colors` must have one unique, non-empty name per colour.",
           call. = FALSE)
    }
    missing_groups <- setdiff(group.levels, color_names)
    if (length(missing_groups)) {
      stop("Named `group.colors` is missing colour(s) for group(s): ",
           paste(missing_groups, collapse = ", "), ".", call. = FALSE)
    }
    group.colors <- group.colors[group.levels]
  } else {
    if (length(group.colors) < length(group.levels)) {
      stop("`group.colors` must provide at least ", length(group.levels),
           " colours, one for each observed group.", call. = FALSE)
    }
    group.colors <- group.colors[seq_along(group.levels)]
    names(group.colors) <- group.levels
  }

  valid <- vapply(group.colors, function(col) {
    tryCatch({
      grDevices::col2rgb(col)
      TRUE
    }, error = function(e) FALSE)
  }, logical(1))
  if (!all(valid)) {
    stop("Invalid colour value(s) in `group.colors`: ",
         paste(group.colors[!valid], collapse = ", "), ".", call. = FALSE)
  }
  group.colors
}

# Apply a shared colour identity and legend title to every grouped layer.
.add_group_scales <- function(plot, group.label, group.levels, group.colors) {
  if (is.null(group.colors)) {
    plot + ggplot2::labs(colour = group.label, fill = group.label)
  } else {
    plot +
      ggplot2::scale_colour_manual(
        values = group.colors,
        breaks = group.levels, limits = group.levels, drop = FALSE
      ) +
      ggplot2::scale_fill_manual(
        values = group.colors,
        breaks = group.levels, limits = group.levels, drop = FALSE
      ) +
      ggplot2::labs(colour = group.label, fill = group.label)
  }
}

# Validate an explicitly scoped ggplot layer-argument list. Structural arguments are kept
# under logist_plot()'s control; the per-layer allowlist makes misspellings fail eagerly
# instead of surfacing later as a ggplot warning during rendering.
.check_layer_args <- function(args, argname, allowed, protected, layer) {
  if (!is.list(args)) {
    stop("`", argname, "` must be a named list.", call. = FALSE)
  }
  if (length(args) == 0L) {
    return(args)
  }

  nms <- names(args)
  if (is.null(nms) || anyNA(nms) || any(nms == "")) {
    stop("Every element of `", argname, "` must be named.", call. = FALSE)
  }
  dup <- unique(nms[duplicated(nms)])
  if (length(dup)) {
    stop("`", argname, "` contains duplicate argument name(s): ",
         paste(dup, collapse = ", "), ".", call. = FALSE)
  }

  # ggplot2 accepts both spellings. Canonicalizing before the merge gives them one
  # deterministic precedence slot instead of potentially passing both to a layer.
  if ("color" %in% nms && "colour" %in% nms) {
    if (!identical(args[["color"]], args[["colour"]])) {
      stop("`", argname, "` supplies conflicting `color` and `colour` values.",
           call. = FALSE)
    }
    args[["color"]] <- NULL
  } else if ("color" %in% nms) {
    names(args)[names(args) == "color"] <- "colour"
  }

  blocked <- intersect(names(args), protected)
  if (length(blocked)) {
    stop("`", argname, "` cannot replace protected ", layer, " argument(s): ",
         paste(blocked, collapse = ", "), ".", call. = FALSE)
  }
  unknown <- setdiff(names(args), allowed)
  if (length(unknown)) {
    stop("Unsupported argument(s) in `", argname, "` for ", layer, ": ",
         paste(unknown, collapse = ", "), ".", call. = FALSE)
  }
  args
}

# Merge named layer arguments with user values taking precedence over defaults.
.merge_layer_args <- function(defaults, user) {
  defaults[names(defaults) %in% names(user)] <- NULL
  c(defaults, user)
}

# Validate and canonicalize a binary response to numeric 0/1, so ggplot2 never sees a
# factor/character/logical y -- which it would treat as *discrete*, silently triggering
# per-level grouping that breaks geom_smooth()'s glm() fit (see dev/loghist-test.R for the
# exact failures this avoids). Which value becomes 1 (the modeled "event") is deterministic,
# not dependent on row order:
#   logical:   FALSE -> 0, TRUE -> 1
#   factor:    the two observed levels, in their existing `levels()` order
#   character: the two observed values, sorted alphabetically (lower -> 0, higher -> 1) --
#              the same convention R itself uses for a factor's default (alphabetical) levels
#   numeric:   must already be coded 0/1; any other numeric coding is rejected rather than
#              guessed at
.to_binary01 <- function(y) {
  if (!is.null(dim(y))) {
    stop("`y` must be a plain vector, not a matrix/array (e.g. not `cbind(success, failure)`).",
         call. = FALSE)
  }
  if (!(is.logical(y) || is.factor(y) || is.character(y) || is.numeric(y))) {
    stop("`y` must be numeric, logical, factor, or character; found class \"",
         paste(class(y), collapse = "/"), "\".", call. = FALSE)
  }
  uy <- unique(y[!is.na(y)])
  if (length(uy) != 2L) {
    stop("`y` must be binary (exactly 2 distinct values); found ", length(uy), ".",
         call. = FALSE)
  }
  levs <- if (is.logical(y)) {
    c(FALSE, TRUE)
  } else if (is.factor(y)) {
    levels(y)[levels(y) %in% as.character(uy)]
  } else if (is.numeric(y)) {
    if (!setequal(uy, c(0, 1))) {
      stop("numeric `y` must be coded 0/1; found ", paste(sort(uy), collapse = ", "), ".",
           call. = FALSE)
    }
    c(0, 1)
  } else {
    # method = "radix" sorts by C-locale byte order, independent of the session locale --
    # otherwise the modeled "event" (which level maps to 1) could differ across systems.
    sort(as.character(uy), method = "radix")
  }
  list(y01 = as.numeric(factor(as.character(y), levels = as.character(levs))) - 1,
       levels = levs)
}

# The one real implementation, shared by all logist_plot() methods and by
# logist_hist()/logist_point()/logist_density().
.logist_plot_impl <- function(x, y, marginal = c("hist", "points", "density"),
                               bins = 30, adjust = 1, xlab = NULL, ylab = NULL,
                               fit.color = "steelblue", marginal.color = "orange",
                               fit.args = list(), marginal.args = list(),
                               group = NULL, group.label = NULL,
                               group.colors = NULL, marginal.height = NULL, ...) {
  rlang::check_dots_empty()
  marginal <- match.arg(marginal)
  grouped <- !is.null(group)
  if (marginal == "points" && !is.null(marginal.height)) {
    warning(
      "`marginal.height` is ignored for `marginal = \"points\"`; ",
      "control vertical jitter through `marginal.args` instead.",
      call. = FALSE
    )
  }
  if (grouped && marginal == "hist") {
    stop("Grouping is not supported for `marginal = \"hist\"`; use ",
         "`marginal = \"points\"` or `marginal = \"density\"`.", call. = FALSE)
  }
  if (!grouped && !is.null(group.colors)) {
    stop("`group.colors` requires a non-NULL `group`.", call. = FALSE)
  }

  if (marginal != "points" && is.list(marginal.args)) {
    misplaced_height <- intersect(
      names(marginal.args),
      c("marginal.height", "height", "lane.height", "lane_height")
    )
    if (length(misplaced_height)) {
      stop("Supply `marginal.height` as a top-level argument, not inside `marginal.args`.",
           call. = FALSE)
    }
  }

  fit.args <- .check_layer_args(
    fit.args,
    argname = "fit.args",
    allowed = c("se", "n", "span", "fullrange", "level", "na.rm", "orientation",
                "show.legend", "colour", "fill", "linewidth", "linetype", "alpha"),
    protected = c("data", "mapping", "stat", "position", "inherit.aes",
                  "method", "formula", "method.args"),
    layer = "fitted-curve layer"
  )
  marginal.args <- if (marginal == "points") {
    .check_layer_args(
      marginal.args,
      argname = "marginal.args",
      allowed = c("position", "na.rm", "show.legend", "colour", "fill", "alpha",
                  "shape", "size", "stroke"),
      protected = c("data", "mapping", "stat", "inherit.aes"),
      layer = '`marginal = "points"` layer'
    )
  } else if (marginal == "hist") {
    .check_layer_args(
      marginal.args,
      argname = "marginal.args",
      allowed = c("na.rm", "show.legend", "colour", "fill", "alpha", "linetype",
                  "linewidth", "lineend", "linejoin"),
      protected = c("data", "mapping", "stat", "position", "inherit.aes",
                    "binwidth", "boundary", "closed"),
      layer = '`marginal = "hist"` rectangle layer'
    )
  } else {
    if (is.list(marginal.args) && "adjust" %in% names(marginal.args)) {
      stop("Supply `adjust` as a top-level argument, not inside `marginal.args`.",
           call. = FALSE)
    }
    .check_layer_args(
      marginal.args,
      argname = "marginal.args",
      allowed = c("na.rm", "show.legend", "colour", "fill", "alpha", "linetype",
                  "linewidth", "lineend", "linejoin", "linemitre", "outline.type"),
      protected = c("data", "mapping", "stat", "position", "orientation", "inherit.aes",
                    "adjust"),
      layer = '`marginal = "density"` ribbon layers'
    )
  }

  if (grouped) {
    fit_color_args <- intersect(names(fit.args), c("colour", "fill"))
    marginal_color_args <- intersect(names(marginal.args), c("colour", "fill"))
    if (length(fit_color_args)) {
      stop("Grouped plots map fit colours to `group`; supply colours through ",
           "`group.colors`, not `fit.args`.", call. = FALSE)
    }
    if (length(marginal_color_args)) {
      stop("Grouped plots map marginal colours to `group`; supply colours through ",
           "`group.colors`, not `marginal.args`.", call. = FALSE)
    }
  }

  if (!is.null(dim(x))) {
    stop("`x` must be a plain vector, not a matrix/array/data.frame (e.g. a multi-column ",
         "formula term such as poly(x, 2) is not supported).", call. = FALSE)
  }
  if (!is.null(dim(y))) {
    stop("`y` must be a plain vector, not a matrix/array/data.frame (e.g. not ",
         "`cbind(success, failure)`).", call. = FALSE)
  }
  if (is.list(x)) {
    stop("`x` must be a plain atomic vector, not a list.", call. = FALSE)
  }
  if (is.list(y)) {
    stop("`y` must be a plain atomic vector, not a list.", call. = FALSE)
  }
  if (length(x) != length(y)) {
    stop("`x` and `y` must be the same length; found ", length(x), " and ", length(y), ".",
         call. = FALSE)
  }
  if (!is.numeric(x)) {
    stop("`x` must be numeric; found class \"", paste(class(x), collapse = "/"), "\".",
         call. = FALSE)
  }

  if (grouped) {
    if (!is.null(dim(group))) {
      stop("`group` must be a plain vector, not a matrix/array/data.frame.", call. = FALSE)
    }
    if (is.list(group)) {
      stop("`group` must be a plain atomic vector, not a list.", call. = FALSE)
    }
    if (length(group) != length(x)) {
      stop("`group` must have the same length as `x` and `y`; found ", length(group),
           " and ", length(x), ".", call. = FALSE)
    }
  }

  data <- if (grouped) {
    data.frame(x = x, y = y, group = group)
  } else {
    data.frame(x = x, y = y)
  }
  data <- data[stats::complete.cases(data) & is.finite(data$x), , drop = FALSE]
  if (nrow(data) == 0L) {
    stop("No complete observations remain after removing missing/non-finite `x` values.",
         call. = FALSE)
  }
  bin <- .to_binary01(data$y)
  data$y <- bin$y01
  uy <- c(0, 1)
  xlab <- xlab %||% "x"
  ylab <- ylab %||% "y"

  group.levels <- NULL
  if (grouped) {
    data$group <- .as_group_factor(data$group)
    group.levels <- levels(data$group)
    if (length(group.levels) < 2L) {
      stop("`group` must contain at least 2 observed groups after filtering; found ",
           length(group.levels), ".", call. = FALSE)
    }
    if (is.null(group.label) || length(group.label) != 1L ||
        is.na(group.label) || !nzchar(group.label)) {
      group.label <- "group"
    }
    group.colors <- .check_group_colors(group.colors, group.levels)

    group_has_both_y <- vapply(group.levels, function(lev) {
      setequal(unique(data$y[data$group == lev]), uy)
    }, logical(1))
    if (!all(group_has_both_y)) {
      stop("Every group must contain both response outcomes; missing an outcome in group(s): ",
           paste(group.levels[!group_has_both_y], collapse = ", "), ".", call. = FALSE)
    }
    group_has_x_range <- vapply(group.levels, function(lev) {
      length(unique(data$x[data$group == lev])) >= 2L
    }, logical(1))
    if (!all(group_has_x_range)) {
      stop("Every group must contain at least 2 distinct predictor values; insufficient ",
           "variation in group(s): ", paste(group.levels[!group_has_x_range], collapse = ", "),
           ".", call. = FALSE)
    }
  }

  min_x <- min(data$x)
  max_x <- max(data$x)
  if (min_x == max_x) {
    stop("`x` has zero range (all values are identical); cannot fit or bin.", call. = FALSE)
  }

  base_mapping <- if (grouped) {
    ggplot2::aes(
      x = .data$x, y = .data$y,
      colour = .data$group, fill = .data$group, group = .data$group
    )
  } else {
    ggplot2::aes(x = .data$x, y = .data$y)
  }
  p_base <- ggplot2::ggplot(data, base_mapping) +
    ggplot2::theme_bw(base_size = 16) +
    ggplot2::theme(panel.grid.major = ggplot2::element_blank(),
                   panel.grid.minor = ggplot2::element_blank(),
                   panel.background = ggplot2::element_blank(),
                   plot.background = ggplot2::element_blank()) +
    ggplot2::labs(y = paste0(ylab, "\n"), x = paste0("\n", xlab))

  fit_defaults <- list(
    method = "glm", formula = y ~ x,
    method.args = list(family = "binomial"),
    se = TRUE, linewidth = 1.5, alpha = 0.3
  )
  if (!grouped) {
    fit_defaults$colour <- fit.color
    fit_defaults$fill <- fit.color
  }
  fit_layer <- do.call(
    ggplot2::geom_smooth,
    .merge_layer_args(
      fit_defaults,
      fit.args
    )
  )

  if (marginal == "points") {
    point_defaults <- list(
      alpha = 0.5,
      position = ggplot2::position_jitter(w = 0, h = 0.02)
    )
    if (!grouped) {
      point_defaults$colour <- marginal.color
    }
    point_layer <- do.call(
      ggplot2::geom_point,
      .merge_layer_args(
        point_defaults,
        marginal.args
      )
    )
    point_plot <- p_base +
      fit_layer +
      point_layer +
      ggplot2::coord_cartesian(xlim = c(min_x, max_x), ylim = c(0, 1))
    if (grouped) {
      point_plot <- .add_group_scales(
        point_plot, group.label, group.levels, group.colors
      )
    }
    point_plot
  } else if (marginal == "hist") {
    .check_bins(bins)
    height <- marginal.height %||% 0.25
    .check_marginal_height(height)

    bin_width <- (max_x - min_x) / bins
    if (!is.finite(bin_width) || bin_width <= 0) {
      stop("Cannot compute histogram bins for `x`: the range of `x` (", min_x, " to ", max_x,
           ") produces a non-finite or non-positive bin width. Try fewer `bins` or check for ",
           "extreme values.", call. = FALSE)
    }
    hist_breaks <- seq(min_x, max_x, length.out = bins + 1)
    if (length(unique(hist_breaks)) != length(hist_breaks)) {
      stop("Cannot compute histogram bins for `x`: the range of `x` is too small relative to ",
           "`bins` (", bins, ") to produce distinct break points. Try fewer `bins`.",
           call. = FALSE)
    }
    hist_counts <- lapply(uy, function(lev) {
      graphics::hist(data$x[data$y == lev], breaks = hist_breaks, right = FALSE,
                      include.lowest = TRUE, plot = FALSE)$counts
    })
    max_count <- max(unlist(hist_counts))
    bin_no <- max_count / height

    # pretty() can return fractional ticks when max_count is small (e.g. 0, 0.2, 0.4, ... for
    # max_count = 1); counts are always whole numbers, so round + dedupe and include both
    # endpoints. If a small marginal height compresses adjacent labels, replace the pretty
    # ticks with evenly spaced integer counts. At extremely small heights, show only the
    # maximum count on each side; zero remains visually implied by the histogram baseline.
    count_ticks <- pretty(c(0, max_count))
    count_ticks <- unique(round(count_ticks[count_ticks >= 0 & count_ticks <= max_count]))
    count_ticks <- sort(unique(c(0L, count_ticks, max_count)))
    min_count_tick_spacing <- 0.05
    count_tick_spacing <- diff(count_ticks / bin_no)
    if (length(count_tick_spacing) && min(count_tick_spacing) < min_count_tick_spacing) {
      count_label_capacity <- floor(
        height / min_count_tick_spacing + sqrt(.Machine$double.eps)
      ) + 1L
      if (count_label_capacity < 2L) {
        count_ticks <- max_count
      } else {
        repeat {
          count_ticks <- unique(round(seq(
            0, max_count, length.out = count_label_capacity
          )))
          count_tick_spacing <- diff(count_ticks / bin_no)
          if (!length(count_tick_spacing) ||
              min(count_tick_spacing) >= min_count_tick_spacing ||
              count_label_capacity <= 2L) {
            break
          }
          count_label_capacity <- count_label_capacity - 1L
        }
      }
    }
    count_positions <- c(count_ticks / bin_no, 1 - count_ticks / bin_no)
    count_labels <- rep(count_ticks, 2L)
    count_order <- order(count_positions)
    count_positions <- count_positions[count_order]
    count_labels <- count_labels[count_order]
    keep_count_position <- !duplicated(count_positions)
    count_positions <- count_positions[keep_count_position]
    count_labels <- count_labels[keep_count_position]

    # Draw both response-specific histograms in the probability coordinate system. Each row
    # describes one bin rectangle: y = 0 grows up from 0; y = 1 hangs down from 1. The tallest
    # bar occupies `height` of the panel on its side. Keeping zero-count rows makes the
    # bin/count correspondence explicit.
    hist_data <- rbind(
      data.frame(
        xmin = utils::head(hist_breaks, -1L),
        xmax = utils::tail(hist_breaks, -1L),
        ymin = 0,
        ymax = hist_counts[[1L]] / bin_no
      ),
      data.frame(
        xmin = utils::head(hist_breaks, -1L),
        xmax = utils::tail(hist_breaks, -1L),
        ymin = 1 - hist_counts[[2L]] / bin_no,
        ymax = 1
      )
    )

    rect_layer <- do.call(
      ggplot2::geom_rect,
      .merge_layer_args(
        list(
          data = hist_data,
          mapping = ggplot2::aes(
            xmin = .data$xmin, xmax = .data$xmax,
            ymin = .data$ymin, ymax = .data$ymax
          ),
          inherit.aes = FALSE,
          fill = marginal.color,
          alpha = 0.67
        ),
        marginal.args
      )
    )

    p_base +
      rect_layer +
      # Draw the fitted curve last so its line and confidence band remain visible over the bars.
      fit_layer +
      ggplot2::scale_y_continuous(
        limits = c(0, 1),
        breaks = seq(0, 1, by = 0.2),
        expand = ggplot2::expansion(mult = 0),
        sec.axis = ggplot2::dup_axis(
          breaks = count_positions,
          labels = count_labels,
          name = "Count",
          guide = ggplot2::guide_axis(check.overlap = TRUE)
        )
      ) +
      ggplot2::coord_cartesian(xlim = c(min_x, max_x))
  } else {
    .check_adjust(adjust)
    density_height <- marginal.height %||% if (grouped) 0.05 else 0.15
    .check_marginal_height(density_height, grouped = grouped)

    if (grouped) {
      density_lane_height <- density_height
      density_lane_gap <- density_lane_height * 0.2
      # The outermost density edge reaches the nominal y limit exactly. Reserve a small
      # display-only margin so geom_ribbon()'s centered outline stroke is not clipped by the
      # panel border. Because linewidth is measured in fixed physical units, retain the
      # canonical 0.0075 padding as a floor when lanes are made unusually thin.
      density_outline_padding <- max(density_lane_height * 0.15, 0.0075)

      estimate_group_density <- function(group_level, response) {
        cell_x <- data$x[data$group == group_level & data$y == response]
        original_level <- bin$levels[response + 1L]
        if (length(cell_x) < 2L) {
          stop("Cannot estimate the marginal density for group ",
               dQuote(as.character(group_level)), " and response level ",
               dQuote(as.character(original_level)),
               ": at least 2 observations are required; found ", length(cell_x), ".",
               call. = FALSE)
        }
        estimate <- tryCatch(
          stats::density(cell_x, from = min_x, to = max_x, adjust = adjust),
          error = function(e) {
            stop("Cannot estimate the marginal density for group ",
                 dQuote(as.character(group_level)), " and response level ",
                 dQuote(as.character(original_level)), ": ", conditionMessage(e),
                 call. = FALSE)
          }
        )
        if (!all(is.finite(estimate$x)) || !all(is.finite(estimate$y))) {
          stop("Density estimation produced non-finite values for group ",
               dQuote(as.character(group_level)), " and response level ",
               dQuote(as.character(original_level)), ".", call. = FALSE)
        }
        if (any(estimate$y < 0)) {
          stop("Density estimation produced negative values for group ",
               dQuote(as.character(group_level)), " and response level ",
               dQuote(as.character(original_level)), ".", call. = FALSE)
        }
        estimate
      }

      density_pairs <- lapply(group.levels, function(group_level) {
        pair <- lapply(uy, function(response) {
          estimate_group_density(group_level, response)
        })
        pair_max <- max(vapply(pair, function(z) max(z$y), numeric(1)))
        if (!is.finite(pair_max) || pair_max <= 0) {
          stop("Density estimation produced a non-finite or non-positive maximum density ",
               "for group ", dQuote(as.character(group_level)), ".", call. = FALSE)
        }
        list(estimates = pair, headroom = pair_max / density_lane_height)
      })
      density_limits <- c(
        -length(group.levels) * density_lane_height -
          (length(group.levels) - 1L) * density_lane_gap,
        1 + length(group.levels) * density_lane_height +
          (length(group.levels) - 1L) * density_lane_gap
      )

      make_group_density_data <- function(group_index, response) {
        pair <- density_pairs[[group_index]]
        estimate <- pair$estimates[[response + 1L]]
        # Clamp at the nominal lane height to prevent floating-point overshoot at the
        # highest peak from being removed by the exact outer y-scale limit.
        scaled_density <- pmin(estimate$y / pair$headroom, density_lane_height)
        lane_index <- group_index - 1L
        lane_offset <- lane_index * (density_lane_height + density_lane_gap)
        if (response == 0) {
          baseline <- -lane_offset
          ymin <- pmax(baseline - scaled_density, density_limits[1L])
          ymax <- baseline
        } else {
          baseline <- 1 + lane_offset
          ymin <- baseline
          ymax <- pmin(baseline + scaled_density, density_limits[2L])
        }
        data.frame(
          x = estimate$x,
          ymin = ymin,
          ymax = ymax,
          group = factor(
            rep(group.levels[group_index], length(estimate$x)),
            levels = group.levels
          )
        )
      }

      density_y0 <- do.call(rbind, lapply(seq_along(group.levels), function(i) {
        make_group_density_data(i, 0)
      }))
      density_y1 <- do.call(rbind, lapply(seq_along(group.levels), function(i) {
        make_group_density_data(i, 1)
      }))

      grouped_density_mapping <- ggplot2::aes(
        x = .data$x,
        ymin = .data$ymin,
        ymax = .data$ymax,
        fill = .data$group,
        colour = .data$group,
        group = .data$group
      )
      grouped_density_layer <- function(layer_data, outline_type) {
        do.call(
          ggplot2::geom_ribbon,
          .merge_layer_args(
            list(
              data = layer_data,
              mapping = grouped_density_mapping,
              inherit.aes = FALSE,
              alpha = 0.35,
              linewidth = 0.5,
              outline.type = outline_type
            ),
            marginal.args
          )
        )
      }

      density_plot <- p_base +
        grouped_density_layer(density_y0, "lower") +
        grouped_density_layer(density_y1, "upper") +
        fit_layer +
        ggplot2::scale_y_continuous(
          limits = density_limits,
          breaks = seq(0, 1, by = 0.2),
          expand = ggplot2::expansion(add = density_outline_padding)
        ) +
        ggplot2::coord_cartesian(xlim = c(min_x, max_x))

      .add_group_scales(density_plot, group.label, group.levels, group.colors)
    } else {

    densities <- lapply(uy, function(lev) {
      group_x <- data$x[data$y == lev]
      original_level <- bin$levels[lev + 1L]
      if (length(group_x) < 2L) {
        stop("Cannot estimate the marginal density for response level ",
             dQuote(as.character(original_level)), ": at least 2 observations are required; ",
             "found ", length(group_x), ".", call. = FALSE)
      }
      tryCatch(
        stats::density(group_x, from = min_x, to = max_x, adjust = adjust),
        error = function(e) {
          stop("Cannot estimate the marginal density for response level ",
               dQuote(as.character(original_level)), ": ", conditionMessage(e),
               call. = FALSE)
        }
      )
    })

    density_is_finite <- vapply(
      densities,
      function(z) all(is.finite(z$x)) && all(is.finite(z$y)),
      logical(1)
    )
    if (!all(density_is_finite)) {
      bad <- which(!density_is_finite)[1L]
      stop("Density estimation produced non-finite values for response level ",
           dQuote(as.character(bin$levels[bad])), ".", call. = FALSE)
    }
    if (any(vapply(densities, function(z) any(z$y < 0), logical(1)))) {
      stop("Density estimation produced negative values; cannot map the marginal density ",
           "into probability coordinates.", call. = FALSE)
    }

    max_density <- max(vapply(densities, function(z) max(z$y), numeric(1)))
    # Continuous filled ribbons are visually heavier than histogram bars, so the default caps
    # the tallest density at 15% of the panel on each side (histograms default to 25%).
    density_headroom <- max_density / density_height
    if (!is.finite(density_headroom) || density_headroom <= 0) {
      stop("Density estimation produced a non-finite or non-positive maximum density.",
           call. = FALSE)
    }

    density_y0 <- data.frame(
      x = densities[[1L]]$x,
      ymin = 0,
      ymax = densities[[1L]]$y / density_headroom
    )
    density_y1 <- data.frame(
      x = densities[[2L]]$x,
      ymin = 1 - densities[[2L]]$y / density_headroom,
      ymax = 1
    )

    density_mapping <- ggplot2::aes(
      x = .data$x,
      ymin = .data$ymin,
      ymax = .data$ymax
    )
    density_layer <- function(layer_data, outline_type) {
      do.call(
        ggplot2::geom_ribbon,
        .merge_layer_args(
          list(
            data = layer_data,
            mapping = density_mapping,
            inherit.aes = FALSE,
            fill = marginal.color,
            colour = NA,
            alpha = 0.67,
            outline.type = outline_type
          ),
          marginal.args
        )
      )
    }

    density_y0_layer <- density_layer(density_y0, "upper")
    density_y1_layer <- density_layer(density_y1, "lower")

    p_base +
      density_y0_layer +
      density_y1_layer +
      # As in histogram mode, keep the fit visible above the filled marginal layers.
      fit_layer +
      ggplot2::scale_y_continuous(
        limits = c(0, 1),
        breaks = seq(0, 1, by = 0.2),
        expand = ggplot2::expansion(mult = 0)
      ) +
      ggplot2::coord_cartesian(xlim = c(min_x, max_x))
    }
  }
}

data(Donner, package = "vcdExtra")

# Small helpers -------------------------------------------------------------------------------

# geom_rect layer is used for histogram mode regardless of layer position, so locate it by
# geom class rather than a hardcoded index.
rect_layer_data <- function(p) {
  is_rect <- vapply(p$layers, function(l) inherits(l$geom, "GeomRect"), logical(1))
  ggplot2::layer_data(p, which(is_rect)[1L])
}

expect_built <- function(p) {
  expect_s3_class(p, "ggplot")
  expect_no_error(ggplot2::ggplot_build(p))
}

# ---- three calling conventions + convenience wrappers --------------------------------------

test_that("vector, data.frame, and formula interfaces all build", {
  expect_built(logist_plot(Donner$age, Donner$survived, marginal = "hist"))
  expect_built(logist_plot(Donner[, c("age", "survived")], marginal = "points"))
  expect_built(logist_plot(survived ~ age, data = Donner, marginal = "hist"))
})

test_that("data.frame method resolves xvar/yvar by name or position", {
  p_name <- logist_plot(Donner, xvar = "age", yvar = "survived", marginal = "hist")
  p_pos  <- logist_plot(Donner[, c("age", "survived")], xvar = 1L, yvar = 2L, marginal = "hist")
  expect_built(p_name)
  expect_built(p_pos)
})

test_that("named formula dispatch works, matching boxplot()/lm() convention", {
  expect_built(logist_plot(formula = survived ~ age, data = Donner))
})

test_that("convenience wrappers fix marginal= but keep all three interfaces", {
  expect_built(logist_hist(survived ~ age, data = Donner))
  expect_built(logist_point(survived ~ age, data = Donner))
  expect_built(logist_density(survived ~ age, data = Donner))
  expect_built(logist_hist(Donner$age, Donner$survived))
  expect_built(logist_point(Donner[, c("age", "survived")]))
})

test_that("formula with more than one predictor is rejected", {
  expect_error(
    logist_plot(survived ~ age + sex, data = Donner),
    "exactly one response and one predictor"
  )
})

test_that("data.frame method requires at least 2 columns", {
  expect_error(
    logist_plot(Donner["age"], marginal = "hist"),
    "at least 2 columns"
  )
})

test_that("unknown xvar/yvar selectors error clearly", {
  expect_error(logist_plot(Donner, xvar = "nope", yvar = "survived"), "does not identify")
  expect_error(logist_plot(Donner, xvar = "age", yvar = 99L), "whole number between 1 and")
})

# ---- marginal modes ---------------------------------------------------------------------

test_that("all three marginal modes build and render", {
  expect_built(logist_plot(survived ~ age, data = Donner, marginal = "hist"))
  expect_built(logist_plot(survived ~ age, data = Donner, marginal = "points"))
  expect_built(logist_plot(survived ~ age, data = Donner, marginal = "density"))
})

test_that("marginal defaults to \"hist\" when omitted", {
  p1 <- logist_plot(survived ~ age, data = Donner)
  p2 <- logist_plot(survived ~ age, data = Donner, marginal = "hist")
  expect_equal(ggplot2::layer_data(p1), ggplot2::layer_data(p2))
})

# ---- accepted binary-response encodings -----------------------------------------------------

test_that("numeric, logical, factor, and character y all build", {
  y_num  <- Donner$survived
  y_log  <- as.logical(Donner$survived)
  y_fac  <- factor(Donner$survived, labels = c("died", "survived"))
  y_char <- ifelse(Donner$survived == 1, "survived", "died")

  for (y in list(y_num, y_log, y_fac, y_char)) {
    expect_built(logist_plot(Donner$age, y, marginal = "hist"))
    expect_built(logist_plot(Donner$age, y, marginal = "points"))
  }
})

test_that("numeric y must be coded 0/1", {
  expect_error(
    logist_plot(1:10, rep(c(1, 2), 5)),
    "must be coded 0/1"
  )
})

test_that("y must be binary (exactly 2 distinct values)", {
  expect_error(
    logist_plot(1:9, rep(c(0, 1, 2), 3)),
    "must be binary"
  )
})

test_that("non-numeric/list/matrix x and y are rejected", {
  expect_error(logist_plot(as.character(1:10), rep(0:1, 5)), "`x` must be numeric")
  expect_error(logist_plot(1:10, list(1:10)), "plain atomic vector")
  expect_error(logist_plot(matrix(1:10, 5), rep(0:1, 5)), "matrix/array/data.frame")
})

test_that("mismatched x/y lengths are rejected", {
  expect_error(logist_plot(1:10, rep(0:1, 4)), "same length")
})

test_that("zero-range x is rejected", {
  expect_error(logist_plot(rep(1, 10), rep(0:1, 5)), "zero range")
})

test_that("NA/Inf x rows are dropped consistently across calling conventions", {
  x <- c(Donner$age, NA, Inf)
  y <- c(Donner$survived, 1, 0)
  p_vec <- logist_plot(x, y, marginal = "hist")
  p_form <- logist_plot(survived ~ age, data = Donner, marginal = "hist")
  expect_equal(nrow(ggplot2::ggplot_build(p_vec)$data[[1]]),
               nrow(ggplot2::ggplot_build(p_form)$data[[1]]))
})

# ---- grouping ---------------------------------------------------------------------------

test_that("group= builds for points and density but errors for hist", {
  expect_built(logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "points"))
  expect_built(logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "density"))
  expect_error(
    logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "hist"),
    "Grouping is not supported"
  )
})

test_that("group order is stable regardless of row order", {
  set.seed(1)
  shuffled <- Donner[sample(nrow(Donner)), ]
  p1 <- logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "points")
  p2 <- logist_plot(survived ~ age, data = shuffled, group = "sex", marginal = "points")
  expect_identical(levels(ggplot2::ggplot_build(p1)$plot$data$group),
                    levels(ggplot2::ggplot_build(p2)$plot$data$group))
})

test_that("every group must contain both response outcomes", {
  bad <- data.frame(
    x = c(1, 2, 3, 4, 5, 6), y = c(0, 1, 0, 1, 1, 1),
    g = c("a", "a", "a", "b", "b", "b")
  )
  expect_error(
    logist_plot(y ~ x, data = bad, group = "g", marginal = "points"),
    "must contain both response outcomes"
  )
})

test_that("every group must have at least 2 distinct x values", {
  bad <- data.frame(
    x = c(1, 1, 1, 2, 3, 4), y = c(0, 1, 0, 1, 0, 1),
    g = c("a", "a", "a", "b", "b", "b")
  )
  expect_error(
    logist_plot(y ~ x, data = bad, group = "g", marginal = "points"),
    "at least 2 distinct predictor values"
  )
})

test_that("group.colors validates against observed group levels", {
  expect_error(
    logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "points",
                group.colors = "red"),
    "must provide at least"
  )
  expect_error(
    logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "points",
                group.colors = c(Female = "red")),
    "missing colour"
  )
  expect_error(
    logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "points",
                group.colors = c("not-a-colour", "red")),
    "Invalid colour"
  )
  expect_built(
    logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "points",
                group.colors = c(Female = "purple", Male = "darkgreen"))
  )
})

test_that("group.colors requires a non-NULL group", {
  expect_error(
    logist_plot(survived ~ age, data = Donner, group.colors = c("red", "blue")),
    "requires a non-NULL"
  )
})

test_that("grouped plots reject colour overrides through fit.args/marginal.args", {
  expect_error(
    logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "points",
                fit.args = list(colour = "black")),
    "map fit colours to `group`"
  )
  expect_error(
    logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "points",
                marginal.args = list(fill = "black")),
    "map marginal colours to `group`"
  )
})

# ---- marginal.height --------------------------------------------------------------------

test_that("default marginal.height reproduces the documented per-mode defaults", {
  p_hist_default <- logist_plot(survived ~ age, data = Donner, marginal = "hist")
  p_hist_explicit <- logist_plot(survived ~ age, data = Donner, marginal = "hist",
                                  marginal.height = 0.25)
  expect_equal(rect_layer_data(p_hist_default), rect_layer_data(p_hist_explicit))

  p_dens_default <- logist_plot(survived ~ age, data = Donner, marginal = "density")
  p_dens_explicit <- logist_plot(survived ~ age, data = Donner, marginal = "density",
                                  marginal.height = 0.15)
  expect_equal(ggplot2::layer_data(p_dens_default), ggplot2::layer_data(p_dens_explicit))
})

test_that("a custom marginal.height changes the rendered extents", {
  p_small <- logist_plot(survived ~ age, data = Donner, marginal = "hist",
                          marginal.height = 0.05)
  p_large <- logist_plot(survived ~ age, data = Donner, marginal = "hist",
                          marginal.height = 0.4)
  small_max <- max(abs(rect_layer_data(p_small)$ymax - rect_layer_data(p_small)$ymin))
  large_max <- max(abs(rect_layer_data(p_large)$ymax - rect_layer_data(p_large)$ymin))
  expect_lt(small_max, large_max)
})

test_that("marginal.height validation rejects bad values", {
  expect_error(
    logist_plot(survived ~ age, data = Donner, marginal.height = -1),
    "finite positive number"
  )
  expect_error(
    logist_plot(survived ~ age, data = Donner, marginal.height = 0.9, marginal = "hist"),
    "cannot exceed 0.5"
  )
  expect_error(
    logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "density",
                marginal.height = 1.5),
    "cannot exceed 1"
  )
})

test_that("marginal.height is ignored (with a warning) for marginal = \"points\"", {
  expect_warning(
    logist_plot(survived ~ age, data = Donner, marginal = "points", marginal.height = 0.3),
    "ignored for `marginal = \"points\"`"
  )
})

test_that("grouped density stays clear of the panel border at extreme lane heights", {
  expect_built(
    logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "density",
                marginal.height = 0.01)
  )
  expect_built(
    logist_plot(survived ~ age, data = Donner, group = "sex", marginal = "density",
                marginal.height = 0.99)
  )
})

test_that("marginal.height cannot be smuggled through marginal.args", {
  expect_error(
    logist_plot(survived ~ age, data = Donner, marginal = "hist",
                marginal.args = list(height = 0.1)),
    "Supply `marginal.height` as a top-level argument"
  )
})

test_that("histogram count-axis ticks stay symmetric and thin out at small heights", {
  p <- logist_plot(survived ~ age, data = Donner, marginal = "hist", marginal.height = 0.03)
  built <- ggplot2::ggplot_build(p)
  breaks <- built$layout$panel_params[[1]]$y$scale$secondary.axis$breaks
  breaks <- breaks[is.finite(breaks)]
  # symmetric around 0.5 (mirrored histogram)
  expect_equal(sort(breaks), sort(1 - breaks), tolerance = 1e-6)
  # no two adjacent labels closer than the documented minimum spacing
  expect_true(all(diff(sort(breaks)) >= 0.05 - 1e-6))
})

# ---- fit.args / marginal.args forwarding ----------------------------------------------------

test_that("fit.color and marginal.color retain their existing behavior", {
  p <- logist_plot(survived ~ age, data = Donner, marginal = "points",
                    fit.color = "navy", marginal.color = "goldenrod")
  built <- ggplot2::layer_data(p, 1)
  expect_true(all(built$colour %in% c("navy") | built$fill %in% c("navy")))
})

test_that("fit.args overrides fit.color for the same aesthetic", {
  p <- logist_plot(survived ~ age, data = Donner, marginal = "points",
                    fit.color = "navy", fit.args = list(colour = "red"))
  fit_data <- ggplot2::layer_data(p, 1)
  expect_true(all(fit_data$colour == "red"))
})

test_that("marginal.args overrides marginal.color for the same aesthetic", {
  p <- logist_plot(survived ~ age, data = Donner, marginal = "hist",
                    marginal.color = "goldenrod", marginal.args = list(fill = "grey70"))
  expect_true(all(rect_layer_data(p)$fill == "grey70"))
})

test_that("point shape/size/alpha/jitter are customizable via marginal.args", {
  p <- logist_plot(
    survived ~ age, data = Donner, marginal = "points",
    marginal.args = list(shape = 21, size = 3, alpha = 0.9,
                          position = ggplot2::position_jitter(h = 0.1))
  )
  expect_built(p)
})

test_that("histogram fill/outline/linewidth/linetype/alpha are customizable", {
  p <- logist_plot(
    survived ~ age, data = Donner, marginal = "hist",
    marginal.args = list(colour = "black", linewidth = 0.2, linetype = "dashed", alpha = 0.8)
  )
  expect_built(p)
})

test_that("color/colour alias is deduplicated deterministically", {
  p <- logist_plot(survived ~ age, data = Donner, marginal = "points",
                    marginal.args = list(color = "purple"))
  expect_true(all(ggplot2::layer_data(p, 2)$colour == "purple"))
  expect_error(
    logist_plot(survived ~ age, data = Donner, marginal = "points",
                marginal.args = list(color = "purple", colour = "green")),
    "conflicting `color` and `colour`"
  )
})

test_that("protected arguments are rejected with a clear error", {
  expect_error(
    logist_plot(survived ~ age, data = Donner, fit.args = list(data = Donner)),
    "cannot replace protected"
  )
  expect_error(
    logist_plot(survived ~ age, data = Donner, marginal = "hist",
                marginal.args = list(binwidth = 1)),
    "cannot replace protected"
  )
})

test_that("unsupported arguments are rejected per marginal mode", {
  expect_error(
    logist_plot(survived ~ age, data = Donner, marginal = "points",
                marginal.args = list(binwidth = 1)),
    "Unsupported argument"
  )
})

test_that("unnamed, duplicated, or non-list layer args are rejected", {
  expect_error(
    logist_plot(survived ~ age, data = Donner, fit.args = list(1, 2)),
    "must be named"
  )
  expect_error(
    logist_plot(survived ~ age, data = Donner, fit.args = "not a list"),
    "must be a named list"
  )
  expect_error(
    logist_plot(survived ~ age, data = Donner, fit.args = list(alpha = 0.5, alpha = 0.9)),
    "duplicate argument name"
  )
})

test_that("adjust cannot be smuggled through marginal.args", {
  expect_error(
    logist_plot(survived ~ age, data = Donner, marginal = "density",
                marginal.args = list(adjust = 2)),
    "Supply `adjust` as a top-level argument"
  )
})

test_that("misspelled top-level arguments still fail via check_dots_empty()", {
  expect_error(
    logist_plot(survived ~ age, data = Donner, fitcolor = "red"),
    "unused argument|fitcolor"
  )
})

test_that("all marginal modes still build after graphical customization", {
  expect_built(logist_plot(survived ~ age, data = Donner, marginal = "hist",
                            marginal.args = list(alpha = 0.5)))
  expect_built(logist_plot(survived ~ age, data = Donner, marginal = "points",
                            marginal.args = list(alpha = 0.5)))
  expect_built(logist_plot(survived ~ age, data = Donner, marginal = "density",
                            marginal.args = list(alpha = 0.5)))
})

# ---- adjust (density bandwidth) ----------------------------------------------------------

test_that("adjust must be one finite positive number", {
  expect_error(logist_plot(survived ~ age, data = Donner, marginal = "density", adjust = -1),
               "finite positive number")
})

test_that("adjust changes the rendered density curve", {
  p1 <- logist_plot(survived ~ age, data = Donner, marginal = "density", adjust = 0.3)
  p2 <- logist_plot(survived ~ age, data = Donner, marginal = "density", adjust = 3)
  expect_false(isTRUE(all.equal(ggplot2::layer_data(p1, 1), ggplot2::layer_data(p2, 1))))
})

# ---- bins -----------------------------------------------------------------------------------

test_that("bins must be one positive whole number", {
  expect_error(logist_plot(survived ~ age, data = Donner, bins = 0), "positive whole number")
  expect_error(logist_plot(survived ~ age, data = Donner, bins = 2.5), "positive whole number")
})

test_that("too many bins for the data range errors clearly", {
  expect_error(
    logist_plot(c(1, 1 + .Machine$double.eps), c(0, 1), bins = 2),
    "too small relative to"
  )
})

#' Diagonal Panel Function for Pairs Plots of Contingency Tables
#'
#' An enhanced replacement for \code{\link[vcd]{pairs_diagonal_mosaic}} from the
#' \pkg{vcd} package. This version fixes two bugs in the original (the
#' \code{labeling} and \code{alternate_labels} arguments were hardcoded and
#' ignored) and changes the default labeling to 
#' \code{\link[vcd]{labeling_border}}.
#'
#' A companion \code{pairs.table} method is also provided that uses this
#' improved function as the default diagonal panel.
#'
#' @param split_vertical Logical; passed to \code{\link[vcd]{mosaic}}.
#'   Default is \code{TRUE}.
#' @param margins A \code{\link[grid]{unit}} object giving the margins around
#'   the mosaic. Default is \code{unit(0, "lines")}.
#' @param offset_labels Numeric; offset for the cell labels. Default is
#'   \code{-0.4}.
#' @param offset_varnames Numeric; offset for the variable name labels.
#'   Default is \code{0}.
#' @param gp A \code{\link[grid]{gpar}} object for the mosaic tiles, or
#'   \code{NULL} (default) to use \code{fill}.
#' @param fill Either a color name or a function mapping an integer to a
#'   vector of colors (e.g. a shading function). Default is \code{"grey"}.
#' @param labeling A labeling function such as
#'   \code{\link[vcd]{labeling_border}} or \code{\link[vcd]{labeling_values}}.
#'   Default is \code{\link[vcd]{labeling_border}} (no cell counts shown).
#' @param alternate_labels Logical; whether to alternate label positions on
#'   the axes. Default is \code{TRUE}.
#' @param ... Additional arguments passed to \code{\link[vcd]{mosaic}}.
#'
#' @details
#' The function follows the \code{"grapcon_generator"} pattern used throughout
#' \pkg{vcd}: calling \code{pairs_diagonal_mosaic(...)} returns a
#' \emph{function} suitable for passing as the \code{diag_panel} argument of
#' \code{\link[vcd]{pairs.table}}.
#'
#' The original \code{vcd::pairs_diagonal_mosaic} has two bugs: (1) the
#' \code{labeling} argument is accepted but then hardcoded to
#' \code{labeling_values} inside the returned function, and (2)
#' \code{alternate_labels} is similarly hardcoded to \code{TRUE}.  This version
#' fixes both, making those arguments work as documented.
#' 
#' The default labeling scheme was changed from \code{\link[vcd]{labeling_values}} to 
#' \code{\link[vcd]{labeling_border}}, so as to disable cell counts by default. To
#' enable cell counts, use \code{\link[vcd]{labeling_values}}.
#'
#' @return A function of class \code{"grapcon_generator"} suitable for use as
#'   the \code{diag_panel} argument in \code{\link[vcd]{pairs.table}}.
#'
#' @seealso
#' \code{\link[vcd]{pairs.table}}, \code{\link[vcd]{pairs_diagonal_mosaic}},
#' \code{\link[vcd]{labeling_border}}, \code{\link[vcd]{labeling_values}}
#'
#' @examples
#' data(Hoyt, package = "vcdExtra")
#' pred_tab <- margin.table(Hoyt, c("Rank", "Occupation", "Sex"))
#'
#' # Default: no cell counts in diagonal panels
#' pairs(pred_tab)
#'
#' # Show cell counts in diagonal panels
#' pairs(pred_tab, diag_panel_args = list(labeling = labeling_values))
#'
#' # Use residual shading for off-diagonal panels
#' pairs(pred_tab, gp = shading_Friendly)
#'
#' @importFrom vcd labeling_border labeling_values mosaic
#' @importFrom grid gpar unit
#' @export
pairs_diagonal_mosaic <- function(split_vertical = TRUE,
                                  margins = unit(0, "lines"),
                                  offset_labels = -0.4,
                                  offset_varnames = 0,
                                  gp = NULL,
                                  fill = "grey",
                                  labeling = labeling_border,
                                  alternate_labels = TRUE,
                                  ...) {

  function(x, i) {
    if (is.function(fill))
      fill <- rev(fill(dim(x)[i]))
    if (is.null(gp))
      gp <- gpar(fill = fill)
    mosaic(margin.table(x, i),
           newpage = FALSE,
           split_vertical = split_vertical,
           margins = margins,
           offset_labels = offset_labels,
           offset_varnames = offset_varnames,
           prefix = "diag",
           gp = gp,
           labeling = labeling,
           labeling_args = list(alternate_labels = alternate_labels),
           ...)
  }
}

class(pairs_diagonal_mosaic) <- "grapcon_generator"


#' @rdname pairs_diagonal_mosaic
#'
#' @param x A contingency table (object of class \code{"table"}).
#' @param diag_panel A \code{"grapcon_generator"} function or a panel function
#'   for the diagonal cells. Defaults to the vcdExtra version of
#'   \code{\link{pairs_diagonal_mosaic}}.
#' @param ... Additional arguments passed to the \pkg{vcd} \code{pairs.table} method.
#'
#' @details
#' \code{pairs.table} is an S3 method for \code{\link[graphics]{pairs}} that
#' overrides the version in \pkg{vcd} solely to change the default
#' \code{diag_panel} to the improved \code{pairs_diagonal_mosaic} defined in
#' this package. All actual rendering is delegated to the \pkg{vcd} implementation,
#' retrieved via \code{utils::getFromNamespace()} to avoid a \code{:::} check NOTE.
#'
#' @export
pairs.table <- function(x, diag_panel = pairs_diagonal_mosaic, ...) {
  vcd_pairs_table <- utils::getFromNamespace("pairs.table", "vcd")
  vcd_pairs_table(x, diag_panel = diag_panel, ...)
}

# Functions for Hosmer Lemeshow test
# original function downloaded from 
# http://sas-and-r.blogspot.com/2010/09/example-87-hosmer-and-lemeshow-goodness.html
#
# see also: MKmisc::gof.test for more general versions





#' Hosmer-Lemeshow Goodness of Fit Test
#' 
#' The `HLtest` function computes the classical Hosmer-Lemeshow (1980)
#' goodness of fit test for a binomial `glm` object in logistic regression
#' 
#' The general idea is to assesses whether or not the observed event rates
#' match expected event rates in subgroups of the model population. The
#' Hosmer-Lemeshow test specifically identifies subgroups as the deciles of
#' fitted event values, or other quantiles as determined by the `g`
#' argument. Given these subgroups, a simple chisquare test on `g-2` df is
#' used.
#' 
#' In addition to `print` and `summary` methods, a `plot` method
#' is supplied to visualize the discrepancies between observed and fitted
#' frequencies.
#' 
#' 
#' @aliases HosmerLemeshow HLtest plot.HLtest print.HLtest rootogram.HLtest summary.HLtest
#' @param model A `glm` model object in the `binomial` family
#' @param g Number of groups used to partition the fitted values for the GOF
#' test.
#' @param x,object A `HLtest` object
#' @param \dots Other arguments passed down to methods
#' @return A class `HLtest` object with the following components:
#' \item{table}{A data.frame describing the results of partitioning the data
#' into `g` groups with the following columns: `cut`, `total`,
#' `obs`, `exp`, `chi`} \item{chisq}{The chisquare statistics}
#' \item{df}{Degrees of freedom} \item{p.value}{p value} \item{groups}{Number
#' of groups} \item{call}{`model` call} %% ...
#' @author Michael Friendly
#' @seealso \code{\link[vcd]{rootogram}}, ~~~
#' @family association tests
#' @references
#' 
#' Hosmer, David W., Lemeshow, Stanley (1980).  A goodness-of-fit test for
#' multiple logistic regression model. *Communications in Statistics,
#' Series A*, 9, 1043-1069.
#' 
#' Hosmer, David W., Lemeshow, Stanley (2000).  *Applied Logistic
#' Regression*, New York: Wiley, ISBN 0-471-61553-6
#' 
#' Lemeshow, S. and Hosmer, D.W.  (1982).  A review of goodness of fit
#' statistics for use in the development of logistic regression models.
#' *American Journal of Epidemiology*, 115(1), 92-106.
#' 
#' @importFrom vcd rootogram
#' @keywords htest
#' @examples
#' 
#' 
#' data(birthwt, package="MASS")
#' # how to do this without attach?
#' attach(birthwt)
#' 	race = factor(race, labels = c("white", "black", "other"))
#' 	ptd = factor(ptl > 0)
#' 	ftv = factor(ftv)
#' 	levels(ftv)[-(1:2)] = "2+"
#' 	bwt <- data.frame(low = factor(low), age, lwt, race,
#'     	smoke = (smoke > 0), ptd, ht = (ht > 0), ui = (ui > 0), ftv)
#' detach(birthwt)
#' 
#' options(contrasts = c("contr.treatment", "contr.poly"))
#' BWmod <- glm(low ~ ., family=binomial, data=bwt)
#' 
#' (hlt <- HLtest(BWmod))
#' str(hlt)
#' summary(hlt)
#' plot(hlt)
#' 
#' # basic model
#' BWmod0 <- glm(low ~ age, family=binomial, data=bwt)
#' (hlt0 <- HLtest(BWmod0))
#' str(hlt0)
#' summary(hlt0)
#' plot(hlt0)
#' 
#' 
#' 
#' @export HLtest
HLtest <- HosmerLemeshow <- function(model, g=10) {
	if (!inherits(model, "glm")) stop("requires a binomial family glm")
	if (!family(model)$family == 'binomial') stop("requires a binomial family glm")
	y <- model$y
	yhat <- model$fitted.values
  cutyhat = cut(yhat,
     breaks = quantile(yhat, probs=seq(0, 1, 1/g)), 
     include.lowest=TRUE)
  obs = xtabs(cbind(1 - y, y) ~ cutyhat)
  exp = xtabs(cbind(1 - yhat, yhat) ~ cutyhat)

  chi = (obs - exp)/sqrt(exp)
	# browser()
  table <- data.frame(cut=dimnames(obs)$cutyhat,
                      total= as.numeric(apply(obs, 1, sum)),
                      obs=as.numeric(as.character(obs[,1])),
                      exp=as.numeric(as.character(exp[,1])), 
                      chi=as.numeric(as.character(chi[,1]))
                         )

  rownames(table) <- 1:g
  chisq = sum(chi^2)
  p = 1 - pchisq(chisq, g - 2)
  result <- list(table=table, chisq=chisq, df=g-2, p.value=p, groups=g, call=model$call)
  class(result) <- "HLtest"
  return(result)
}

#' @rdname HLtest
#' @exportS3Method 
print.HLtest <- function(x, ...) {
	heading <- "Hosmer and Lemeshow Goodness-of-Fit Test"
	df <- data.frame("ChiSquare"=x$chisq, df=x$df, "P_value"= x$p.value)
	cat(heading,"\n\n")
	cat("Call:\n")
	print(x$call)
	print(df, row.names=FALSE)
	invisible(x)
}

# Q: how to print **s next to larg chisq components?
#' @rdname HLtest
#' @exportS3Method 
summary.HLtest <- function(object, ...) {
	heading <- "Partition for Hosmer and Lemeshow Goodness-of-Fit Test"
	cat(heading,"\n\n")
	print(object$table)
	print(object)
}

## Q: how to display any large chi residuals on the bars??
#' @rdname HLtest
#' @exportS3Method 
rootogram.HLtest <- function(x, ...) {
	rootogram(as.numeric(x$table$obs), as.numeric(x$table$exp), 
		xlab="Fitted value group", names=1:x$groups, ...) 
}

#' @rdname HLtest
#' @exportS3Method 
plot.HLtest <- function(x, ...) {
	rootogram.HLtest(x, ...)
}

#' Plot an Association Graph
#'
#' Plot method for \code{\link{assoc_graph}} objects, displaying the association
#' structure of a loglinear model as a network diagram.
#'
#' @param x An \code{assoc_graph} object, as returned by \code{\link{assoc_graph}}.
#' @param layout Layout function or coordinate matrix for node positions.
#'   Defaults to \code{\link[igraph]{layout_in_circle}} for up to 6 nodes,
#'   \code{\link[igraph]{layout_with_fr}} otherwise.
#' @param groups Optional named list assigning variables to groups for coloring,
#'   e.g., \code{list(response = "Survived", predictors = c("Class", "Sex", "Age"))}.
#' @param colors Character vector of colors for groups. Recycled as needed.
#' @param vertex.size Vertex size (default 30).
#' @param vertex.label.cex Label size for vertex names (default 1.2).
#' @param edge.width Edge width (default 2). If edge weights are present, widths are
#'   scaled from the weights automatically.
#' @param edge.label Optional edge labels. If \code{TRUE} and edge weights are present,
#'   the weight values are used as labels.
#' @param \dots Additional arguments passed to \code{\link[igraph]{plot.igraph}},
#'   such as \code{main} for a title.
#'
#' @return The \code{assoc_graph} object \code{x}, returned invisibly.
#'
#' @seealso \code{\link{assoc_graph}}, \code{\link[igraph]{plot.igraph}}
#'
#' @family loglinear models
#' @export
#' @examples
#' # Basic structural plot
#' g <- conditional(3, factors = c("A", "B", "C")) |> assoc_graph()
#' plot(g, main = "Conditional independence: [AC] [BC]")
#'
#' # With grouped node colors
#' g <- saturated(4, factors = c("A", "B", "C", "D")) |> assoc_graph()
#' plot(g, groups = list(c("A", "B"), c("C", "D")),
#'      main = "Saturated model")
#'
plot.assoc_graph <- function(x, layout = NULL,
                             groups = NULL,
                             colors = c("lightblue", "lightyellow", "lightgreen",
                                        "lightsalmon", "plum"),
                             vertex.size = 30,
                             vertex.label.cex = 1.2,
                             edge.width = 2,
                             edge.label = NULL,
                             ...) {

  if (is.null(layout)) {
    layout <- if (igraph::vcount(x) <= 6) igraph::layout_in_circle(x)
              else igraph::layout_with_fr(x)
  }

  # Node colors from groups
  vcol <- rep(colors[1], igraph::vcount(x))
  if (!is.null(groups)) {
    vnames <- igraph::V(x)$name
    if (is.list(groups)) {
      # named list: list(response = "Survived", explanatory = c("Class", "Sex"))
      for (i in seq_along(groups)) {
        idx <- match(groups[[i]], vnames)
        idx <- idx[!is.na(idx)]
        vcol[idx] <- colors[((i - 1) %% length(colors)) + 1]
      }
    }
  }

  # Edge weights -> width scaling
  ew <- edge.width
  if (!is.null(igraph::E(x)$weight)) {
    w <- igraph::E(x)$weight
    # Scale to range [1, 6]
    if (max(w) > min(w)) {
      ew <- 1 + 5 * (w - min(w)) / (max(w) - min(w))
    } else {
      ew <- rep(3, length(w))
    }
  }

  # Edge labels from weights
  el <- edge.label
  if (isTRUE(el) && !is.null(igraph::E(x)$weight)) {
    el <- round(igraph::E(x)$weight, 2)
  }

  igraph::plot.igraph(x,
                      layout = layout,
                      vertex.size = vertex.size,
                      vertex.color = vcol,
                      vertex.label = igraph::V(x)$name,
                      vertex.label.cex = vertex.label.cex,
                      vertex.frame.color = "gray40",
                      edge.width = ew,
                      edge.label = el,
                      edge.color = "gray30",
                      ...)

  invisible(x)
}

#' Association Graph for a Loglinear Model
#'
#' Construct an undirected graph representing the associations in a loglinear model.
#' Nodes represent variables and edges represent pairwise associations fitted in the model.
#' If two variables are not connected by an edge, they are conditionally independent
#' given the other variables.
#'
#' @param x An object specifying the model. This can be:
#'   \itemize{
#'     \item A \code{list} of character vectors (a margin/generating class list, as produced by
#'           \code{\link{joint}}, \code{\link{conditional}}, etc.)
#'     \item A fitted \code{\link[MASS]{loglm}} object
#'     \item A fitted \code{\link[stats]{glm}} object (poisson family loglinear model)
#'   }
#' @param result Type of result to return: \code{"igraph"} (default) returns an
#'   \code{\link[igraph:igraph-package]{igraph}} object; \code{"matrix"} returns the
#'   adjacency matrix; \code{"edge_list"} returns a two-column character matrix of edges.
#' @param measure Type of association measure for edge weights (only for \code{glm} method):
#'   \code{"none"} (default) produces an unweighted graph;
#'   \code{"chisq"} computes partial chi-squared statistics (deviance change when each
#'   edge is removed from the model);
#'   \code{"cramer"} computes Cramer's V from the marginal two-way table for each edge.
#' @param \dots Additional arguments (currently unused).
#'
#' @return Depending on \code{result}:
#'   \itemize{
#'     \item \code{"igraph"}: An \code{igraph} undirected graph object of class
#'           \code{c("assoc_graph", "igraph")}, with vertex names corresponding to
#'           the variable names. When \code{measure != "none"}, edge weights are
#'           stored as \code{E(g)$weight} and the measure name as \code{g$measure}.
#'     \item \code{"matrix"}: A symmetric adjacency matrix with variable names as
#'           row and column names. Contains 0/1 when unweighted, or association
#'           strength values when \code{measure} is specified.
#'     \item \code{"edge_list"}: When unweighted, a two-column character matrix (from, to).
#'           When \code{measure} is specified, a data frame with columns from, to, and weight.
#'   }
#'
#' @details
#' Each high-order term (margin) in a hierarchical loglinear model defines a clique
#' in the association graph. For example, the term \code{c("A", "B", "C")} generates
#' edges A--B, A--C, and B--C. Single-variable terms (as in mutual independence)
#' yield isolated nodes with no edges.
#'
#' For \code{loglm} objects, the margins are extracted from the \code{$margin} component.
#' For \code{glm} objects, the interaction terms are extracted from the model formula.
#'
#' @references
#' Khamis, H. J. (2011). \emph{The Association Graph and the Multigraph for Loglinear Models}.
#' SAGE Publications. \doi{10.4135/9781452226521}
#'
#' Darroch, J. N., Lauritzen, S. L., & Speed, T. P. (1980). Markov Fields and Log-Linear
#' Interaction Models for Contingency Tables. \emph{The Annals of Statistics}, 8(3), 522--539.
#' \doi{10.1214/aos/1176345006}
#'
#' Whittaker, J. (1990). \emph{Graphical Models in Applied Multivariate Statistics}.
#' John Wiley & Sons, Chichester.
#'
#' @seealso \code{\link{joint}}, \code{\link{conditional}}, \code{\link{mutual}},
#'   \code{\link{saturated}}, \code{\link{loglin2string}}, \code{\link{seq_loglm}},
#'   \code{\link{plot.assoc_graph}}
#'
#' @family loglinear models
#' @export
#' @examples
#' # Structural graphs from margin lists (3-way: A, B, C)
#' mutual(3, factors = c("A", "B", "C"))      |> assoc_graph()
#' joint(3, factors = c("A", "B", "C"))       |> assoc_graph()
#' conditional(3, factors = c("A", "B", "C")) |> assoc_graph()
#' saturated(3, factors = c("A", "B", "C"))   |> assoc_graph()
#'
#' # Adjacency matrix form
#' conditional(3, factors = c("A", "B", "C")) |> assoc_graph(result = "matrix")
#'
#' # From a fitted loglm model (Berkeley admissions)
#' \dontrun{
#' mod <- MASS::loglm(~ (Admit + Gender) * Dept, data = UCBAdmissions)
#' assoc_graph(mod)
#' plot(assoc_graph(mod), main = "Berkeley: [AD] [GD]")
#' }
#'
#' # From glm models (Dayton Survey: cigarette, alcohol, marijuana, sex, race)
#' data(DaytonSurvey)
#'
#' # Mutual independence + sex*race: one edge only
#' mod.SR <- glm(Freq ~ . + sex*race, data = DaytonSurvey, family = poisson)
#' assoc_graph(mod.SR)
#' plot(assoc_graph(mod.SR), main = "Mutual indep. + [SR]")
#'
#' # [AM][AC][MC][AR][AS][RS]: {race, sex} indep {marijuana, cigs} | alcohol
#' mod.cond <- glm(Freq ~ (cigarette + alcohol + marijuana)^2 +
#'                         (alcohol + sex + race)^2,
#'                 data = DaytonSurvey, family = poisson)
#'
#' # define groups for the model
#' gps <- list(c("cigarette", "marijuana"),
#'             "alcohol",
#'             c("sex", "race"))
#'
#' assoc_graph(mod.cond)
#' plot(assoc_graph(mod.cond),
#'      groups = gps,
#'      layout = igraph::layout_nicely,
#'      main = "{R,S} indep {M,C} | A")
#'
#' # Weighted graph: partial chi-squared
#' g <- assoc_graph(mod.cond, measure = "chisq")
#' g
#' plot(g, edge.label = TRUE,
#'      groups = gps,
#'      layout = igraph::layout_nicely,
#'      main = "Partial chi-squared weights")
#'
#' # Cramer's V (marginal)
#' g2 <- assoc_graph(mod.cond, measure = "cramer")
#' g2
#' plot(g2, edge.label = TRUE,
#'      groups = gps,
#'      layout = igraph::layout_nicely,
#'      main = "Cramer's V weights")
#'
assoc_graph <- function(x, ...) {
  UseMethod("assoc_graph")
}

#' @rdname assoc_graph
#' @export
assoc_graph.list <- function(x, result = c("igraph", "matrix", "edge_list"), ...) {
  result <- match.arg(result)
  .margins_to_assoc_graph(x, result = result)
}

#' @rdname assoc_graph
#' @export
assoc_graph.loglm <- function(x, result = c("igraph", "matrix", "edge_list"), ...) {
  result <- match.arg(result)
  if (is.null(x$margin)) {
    stop("Cannot extract margins from this loglm object")
  }
  .margins_to_assoc_graph(x$margin, result = result)
}

#' @rdname assoc_graph
#' @importFrom vcd assocstats
#' @export
assoc_graph.glm <- function(x, result = c("igraph", "matrix", "edge_list"),
                            measure = c("none", "chisq", "cramer"), ...) {
  result <- match.arg(result)
  measure <- match.arg(measure)
  margins <- .glm_to_margins(x)

  if (measure == "none") {
    return(.margins_to_assoc_graph(margins, result = result))
  }

  # Compute weights — need the igraph object regardless of result type

  g <- .margins_to_assoc_graph(margins, result = "igraph")

  weights <- switch(measure,
    chisq  = .compute_chisq_weights(g, x),
    cramer = .compute_cramer_weights(g, x)
  )

  g$measure <- measure

  if (igraph::ecount(g) > 0) {
    igraph::E(g)$weight <- weights
  }

  if (result == "igraph") {
    return(g)
  }

  if (result == "matrix") {
    all_vars <- igraph::V(g)$name
    nv <- length(all_vars)
    adj <- matrix(0, nv, nv, dimnames = list(all_vars, all_vars))
    if (igraph::ecount(g) > 0) {
      el <- igraph::as_edgelist(g)
      for (i in seq_len(nrow(el))) {
        adj[el[i, 1], el[i, 2]] <- weights[i]
        adj[el[i, 2], el[i, 1]] <- weights[i]
      }
    }
    return(adj)
  }

  if (result == "edge_list") {
    if (igraph::ecount(g) == 0) {
      return(matrix(character(0), ncol = 2, dimnames = list(NULL, c("from", "to"))))
    }
    el <- igraph::as_edgelist(g)
    out <- data.frame(from = el[, 1], to = el[, 2], weight = weights,
                      stringsAsFactors = FALSE)
    return(out)
  }
}


# --- Core helper: margin list -> assoc_graph ---

.margins_to_assoc_graph <- function(margins, result = "igraph") {

  # all variable names (including isolated ones from single-variable terms)
  all_vars <- unique(unlist(margins))

  # pairwise edges from each clique
  edge_list <- do.call(rbind, lapply(margins, function(m) {
    if (length(m) >= 2) t(utils::combn(m, 2)) else NULL
  }))

  if (!is.null(edge_list) && nrow(edge_list) > 0) {
    # deduplicate edges (sort each pair so A-B and B-A are treated the same)
    edge_list <- unique(edge_list)
  }

  if (result == "edge_list") {
    if (is.null(edge_list) || nrow(edge_list) == 0) {
      return(matrix(character(0), ncol = 2, dimnames = list(NULL, c("from", "to"))))
    }
    colnames(edge_list) <- c("from", "to")
    return(edge_list)
  }

  if (result == "matrix") {
    nv <- length(all_vars)
    adj <- matrix(0L, nv, nv, dimnames = list(all_vars, all_vars))
    if (!is.null(edge_list) && nrow(edge_list) > 0) {
      for (i in seq_len(nrow(edge_list))) {
        adj[edge_list[i, 1], edge_list[i, 2]] <- 1L
        adj[edge_list[i, 2], edge_list[i, 1]] <- 1L
      }
    }
    return(adj)
  }

  # result == "igraph"
  if (is.null(edge_list) || nrow(edge_list) == 0) {
    g <- igraph::make_empty_graph(n = 0, directed = FALSE)
    g <- igraph::add_vertices(g, length(all_vars), name = all_vars)
  } else {
    g <- igraph::graph_from_edgelist(edge_list, directed = FALSE)
    # add any isolated nodes not covered by edges
    missing <- setdiff(all_vars, igraph::V(g)$name)
    if (length(missing) > 0) {
      g <- igraph::add_vertices(g, length(missing), name = missing)
    }
  }

  class(g) <- c("assoc_graph", class(g))
  g
}


# --- Helper: margins -> glm formula with hierarchy (using * notation) ---

.margins_to_glm_formula <- function(margins, response) {
  rhs_terms <- vapply(margins, function(m) {
    if (length(m) == 1) m else paste(m, collapse = " * ")
  }, character(1))
  stats::as.formula(paste(response, "~", paste(rhs_terms, collapse = " + ")))
}


# --- Helper: partial chi-squared weights (deviance change per edge) ---

.compute_chisq_weights <- function(g, model) {
  el <- igraph::as_edgelist(g)
  if (nrow(el) == 0) return(numeric(0))

  dev_full <- stats::deviance(model)
  resp <- as.character(stats::formula(model)[[2]])

  weights <- numeric(nrow(el))
  for (i in seq_len(nrow(el))) {
    # Remove this edge
    eid <- igraph::get.edge.ids(g, c(el[i, 1], el[i, 2]))
    g_reduced <- igraph::delete_edges(g, eid)

    # Convert reduced graph -> margins -> glm formula
    margins_reduced <- .graph_to_margins(g_reduced)
    new_formula <- .margins_to_glm_formula(margins_reduced, resp)

    # Refit and compute deviance change
    mod_reduced <- stats::update(model, formula. = new_formula)
    weights[i] <- stats::deviance(mod_reduced) - dev_full
  }
  weights
}


# --- Helper: Cramer's V weights (marginal 2-way tables) ---

.compute_cramer_weights <- function(g, model) {
  el <- igraph::as_edgelist(g)
  if (nrow(el) == 0) return(numeric(0))

  mf <- stats::model.frame(model)
  freq <- stats::model.response(mf)

  weights <- numeric(nrow(el))
  for (i in seq_len(nrow(el))) {
    tab <- stats::xtabs(freq ~ mf[[el[i, 1]]] + mf[[el[i, 2]]])
    weights[i] <- vcd::assocstats(tab)$cramer
  }
  weights
}


# --- Helper: extract generating class (margins) from a glm formula ---

.glm_to_margins <- function(object) {
  tt <- stats::terms(object)
  factors <- attr(tt, "factors")
  order <- attr(tt, "order")

  if (is.null(factors)) {
    stop("Cannot extract model terms from this glm object")
  }

  # Get variable names involved in each term
  # Only keep the highest-order terms (generating class for hierarchical model)
  var_names <- rownames(factors)
  term_names <- colnames(factors)

  # Build list of variable sets for each term
  term_vars <- lapply(seq_along(term_names), function(j) {
    var_names[factors[, j] > 0]
  })

  # Filter to the generating class: remove terms that are subsets of other terms
  is_maximal <- vapply(seq_along(term_vars), function(i) {
    ti <- term_vars[[i]]
    !any(vapply(seq_along(term_vars), function(j) {
      if (i == j) return(FALSE)
      all(ti %in% term_vars[[j]]) && length(term_vars[[j]]) > length(ti)
    }, logical(1)))
  }, logical(1))

  margins <- term_vars[is_maximal]
  names(margins) <- paste0("term", seq_along(margins))
  margins
}


# --- Print method ---

#' @rdname assoc_graph
#' @export
print.assoc_graph <- function(x, ...) {
  nv <- igraph::vcount(x)
  ne <- igraph::ecount(x)
  vnames <- igraph::V(x)$name

  cat("Association graph: ", nv, " variables, ", ne, " edges\n", sep = "")
  cat("Variables:", paste(vnames, collapse = ", "), "\n")

  if (ne > 0) {
    el <- igraph::as_edgelist(x)
    w <- igraph::E(x)$weight
    if (!is.null(w)) {
      edge_strings <- paste0(el[, 1], " -- ", el[, 2], " (", round(w, 2), ")")
    } else {
      edge_strings <- paste0(el[, 1], " -- ", el[, 2])
    }
    cat("Edges:", paste(edge_strings, collapse = ", "), "\n")
  } else {
    cat("Edges: (none -- mutual independence)\n")
  }

  # Show measure if present
  if (!is.null(x$measure) && x$measure != "none") {
    cat("Measure:", x$measure, "\n")
  }

  # Show bracket notation
  margins <- .graph_to_margins(x)
  cat("Model:", loglin2string(margins), "\n")

  invisible(x)
}


# --- Helper: recover generating class from the graph (maximal cliques) ---

.graph_to_margins <- function(g) {
  if (igraph::ecount(g) == 0) {
    # Mutual independence: each variable is its own term
    margins <- as.list(igraph::V(g)$name)
  } else {
    # Maximal cliques give the generating class
    cliques <- igraph::max_cliques(g)
    margins <- lapply(cliques, function(cl) igraph::V(g)$name[cl])
  }
  names(margins) <- paste0("term", seq_along(margins))
  margins
}

#' Sequential Loglinear Models for an N-way Table
#'
#' This function takes an n-way contingency table and fits a series of
#' sequential models to the 1-, 2-, ... n-way marginal tables, corresponding to
#' a variety of types of loglinear models.
#'
#' Sequential marginal models for an n-way tables begin with the model of
#' equal-probability for the one-way margin (equivalent to a
#' \code{\link[stats]{chisq.test}}) and add successive variables one at a time
#' in the order specified by `vorder`.
#'
#' All model types give the same result for the two-way margin, namely the test
#' of independence for the first two factors.
#'
#' Sequential models of *joint independence* (`type="joint"`) have a
#' particularly simple interpretation, because they decompose the likelihood
#' ratio test for the model of mutual independence in the full n-way table, and
#' hence account for "total" association in terms of portions attributable to
#' the conditional probabilities of each new variable, given all prior
#' variables.
#'
#' @param x a contingency table in array form, with optional category labels specified in the dimnames(x) attribute,
#'          or else a data.frame in frequency form, with the frequency variable named `"Freq"`.
#' @param type type of sequential model to fit, a character string. One of `"joint"`, `"conditional"`,
#'          `"mutual"`, `"markov"`, or `"saturated"`.
#' @param marginals which marginal sub-tables to fit? A vector of a (sub)set of the integers, `1:nf` where
#'          `nf` is the number of factors in the full n-way table.
#' @param vorder order of variables, a permutation of the integers `1:nf`, used to reorder the variables in
#'          the original table for the purpose of fitting sequential marginal models.
#' @param k conditioning variable(s) for `type` = `"joint"`, `"conditional"` or Markov chain order
#'          for `type` = `"markov"`
#' @param prefix prefix used to give names to the sequential models. If `NULL` (the default),
#'          uses an abbreviation of `type`: `"joint"`, `"cond"`, `"mutual"`, `"markov"`, or `"sat"`.
#' @param fitted argument passed to `loglm` to store the fitted values in the model objects
#' @param \dots other arguments, passed down
#'
#' @return An object of class `"loglmlist"`, each of which is a class `"loglm"` object
#'
#' @note One-way marginal tables are a bit of a problem here, because they
#' cannot be fit directly using \code{\link[MASS]{loglm}}. The present version
#' uses \code{\link[stats]{loglin}}, and repairs the result to look like a
#' `loglm` object (sort of).
#'
#' @author Michael Friendly
#'
#' @seealso \code{\link{loglin-utilities}} for descriptions of sequential
#' models, \code{\link{conditional}}, \code{\link{joint}},
#' \code{\link{mutual}}, \dots{}
#'
#' \code{\link{loglmlist}}
#' @family loglinear models
#'
#' @references These functions were inspired by the original SAS implementation
#' of mosaic displays, described in the *User's Guide*,
#' <http://www.datavis.ca/mosaics/mosaics.pdf>
#'
#' @keywords models
#' @examples
#'
#' data(Titanic, package="datasets")
#' # variables are in the order Class, Sex, Age, Survived
#'
#' # Models of joint independence
#' tit.joint <- seq_loglm(Titanic, type = "joint")
#'
#' # compare the models
#' LRstats(tit.joint)
#'
#' #' # Models of conditional independence
#' tit.cond <- seq_loglm(Titanic, type = "conditional")
#'
#' # compare the models
#' LRstats(tit.cond)
#'

#'
#'
#' @export seq_loglm
seq_loglm <- function(
	x,
	type = c("joint", "conditional", "mutual", "markov", "saturated"),
	marginals = 1:nf,  # which marginals to fit?
	vorder = 1:nf,     # order of variables in the sequential models
	k = NULL,          # conditioning variable(s) for "joint", "conditional" or order for "markov"
	prefix = NULL,
	fitted = TRUE,     # keep fitted values?
	...
	)
{
  if (inherits(x, "data.frame") && "Freq" %in% colnames(x)) {
    x <- xtabs(Freq ~ ., data=x)
  }
  if (!inherits(x, c("table", "array"))) stop("not an xtabs, table, array or data.frame with a 'Freq' variable")

	nf <- length(dim(x))
	x <- aperm(x, vorder)
	factors <- names(dimnames(x))
	indices <- 1:nf

  type = match.arg(type)

  # Set default prefix based on type if not specified
  if (is.null(prefix)) {
    prefix <- switch(type,
      joint = "joint",
      conditional = "cond",
      mutual = "mutual",
      markov = "markov",
      saturated = "sat"
    )
  }
#  models <- as.list(rep(NULL, length(marginals)))
  models <- list()
  for (i in marginals) {
		mtab <- margin.table(x, 1:i)
		if (i==1) {
			# KLUDGE: use loglin, but try to make it look like a loglm object
			mod <- loglin(mtab, margin=NULL, print=FALSE)
		  mod$model.string = paste("=", factors[1])
		  mod$margin <- list(factors[1])
#		  mod$margin <- names(dimnames(mtab))
#		  names(mod$margin) <- factors[1]
      if (fitted) {
        fit <- mtab
  		  fit[] <- (sum(mtab) / length(mtab))
  		  mod$fitted <- fit
		  }
		  mod$nobs <- length(mtab)
		  mod$frequencies <- mtab
		  mod$deviance <- mod$lrt
		  class(mod) <- c("loglin", "loglm")
		  }
		else {
  		expected <- switch(type,
  			'conditional' = conditional(i, mtab, with=if(is.null(k)) i else k),
  			'joint' = joint(i, mtab, with=if(is.null(k)) i else k),
  			'mutual' = mutual(i, mtab),
  			'markov' = markov(i, mtab, order=if(is.null(k)) 1 else k),
  			'saturated' = saturated(i, mtab)
  			)

  		form <- loglin2formula(expected)
#  		mod <- loglm(formula=form, data=mtab, fitted=TRUE)
      mod <- eval(bquote(MASS::loglm(.(form), data=mtab, fitted=fitted)))
  		mod$model.string <- loglin2string(expected, brackets=if (i<nf) '()' else '[]')

		}
#  	cat(i, "  model.string: ", mod$model.string, "\n")
#  	cat("model:\n"); print(mod)
  	models[[i]] <- mod
	}
	names(models) <- paste(prefix, '.', indices, sep='')
	models <- models[marginals]
	class(models) <- "loglmlist"
	invisible(models)
}

#' Convert frequency, case, or table form data into a matrix.
#' 
#' Converts object (`obj`) in frequency, case or table form into a matrix of
#' specified dimensions (`dims`). The column containing the frequencies (`freq`) 
#' must be supplied if `obj` is in frequency form.
#' 
#' @param obj 
#'  Object to be converted into a matrix.
#' @param freq 
#'  If `obj` is in frequency form, this is the name of the frequency column. 
#'  Leave as `NULL` if `obj` is in any other form.
#' @param dims 
#'  A character vector of dimensions. If not specified, all variables apart from 
#'  `freq` will be used as dimensions.
#' @param prop 
#'  If set to `TRUE`, returns a matrix of proportions (that sum to 1). May also 
#'  be set to a character or numeric vector of dimensions to be used as margins 
#'  from which proportions will be computed.
#' @return Object in matrix form.
#' 
#' @details
#' First converts `obj` into an array using \code{\link{as_array}}. Then a
#' check is made to ensure the user inputted a 2D `obj`. If `obj` is not 2D, an 
#' error is returned. If `obj` is 2D, \code{\link[base]{as.matrix}} is applied.
#' 
#' @author Gavin M. Klorfine
#' 
#' @seealso
#' \code{\link{as_array}}, \code{\link{as_table}}, \code{\link{as_freqform}},
#' \code{\link{as_caseform}}
#' 
#' @examples
#' library(vcdExtra)
#' 
#' data("HairEyeColor")
#' 
#' freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
#' tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
#' caseForm <- expand.dft(freqForm) # Generate case form data
#' arrayDat <- as_array(HairEyeColor) # Generate an array
#' 
#' # Table form -> matrix
#' as_matrix(HairEyeColor, dims = c("Hair", "Sex")) |> str()
#' 
#' # Frequency form -> matrix
#' as_matrix(freqForm, freq = "Freq", dims = c("Hair", "Sex")) |> str()
#' 
#' # Case form -> matrix form
#' as_matrix(caseForm, dims = c("Hair", "Sex")) |> str()
#' 
#' # Frequency (tibble) form -> matrix form
#' as_matrix(tidy_freqForm, freq = "n", dims = c("Hair", "Sex")) |> str()
#' 
#' #-----For proportions-----#
#' 
#' # Proportions relative to grand total
#' as_matrix(HairEyeColor, dims = c("Hair", "Sex"), prop = TRUE)
#' 
#' # Marginalize proportions along "Sex" (i.e., male proportions sum to 1, 
#' # female proportions sum to 1)
#' as_matrix(HairEyeColor, dims = c("Hair", "Sex"), prop = "Sex")
#' 
#' as_matrix(HairEyeColor, dims = c("Hair", "Sex"), prop = 2) # Same as above
#' 
#' 
#' @export

as_matrix <- function(obj, freq = NULL, dims = NULL, prop = NULL){
  
  tab <- as_array(obj, freq = freq, dims = dims, prop = prop)
  
  if (length(dim(tab)) == 2){ # If number of dimensions equal 2
    return(as.matrix(tab))
  }
  else{
    stop("Please supply an object with two dimensions. You may use the `dims` argument.")
  }
}

#' Horseshoe Crab Mating
#'
#' @description
#' Determinants of mating for male satellites to nesting horseshoe crabs.
#' The number of `satellites` is a natural outcome variable. This dataset is useful for exploring various count data models.
#'
#' @name CrabSatellites
#' @docType data
#' @md
#'
#' @format
#' A data frame containing 173 observations on 5 variables.
#' \describe{
#'   \item{`color`}{Ordered factor indicating color (light medium, medium, dark medium, dark).}
#'   \item{`spine`}{Ordered factor indicating spine condition (both good, one worn or broken, both worn or broken).}
#'   \item{`width`}{Carapace width (cm).}
#'   \item{`weight`}{Weight (kg).}
#'   \item{`satellites`}{Number of satellites.}
#' }
#'
#' @details
#' Brockmann (1996) investigates horseshoe crab mating. The crabs arrive on the beach in pairs to spawn. Furthermore, unattached males
#' also come to the beach, crowd around the nesting couples and compete with attached males for fertilizations. These so-called
#' satellite males form large groups around some couples while ignoring others. Brockmann (1996) shows that the groupings are
#' not driven by environmental factors but by properties of the nesting female crabs. Larger females that are in better condition
#' attract more satellites.
#'
#' Agresti (2002, 2013) reanalyzes the number of satellites using count models. Explanatory variables are the female crab's
#' color, spine condition, weight, and carapace width. Color and spine condition are ordered factors but are treated as numeric in some analyses.

#' @source
#' Table 4.3 in Agresti (2002). This dataset was taken from the \pkg{countreg} package, which is not on CRAN
#'
#' @references
#' Agresti A (2002). Categorical Data Analysis, 2nd ed., John Wiley & Sons, Hoboken.
#'
#' Agresti A (2013). Categorical Data Analysis, 3rd ed., John Wiley & Sons, Hoboken.

#' Brockmann HJ (1996). “Satellite Male Groups in Horseshoe Crabs, Limulus polyphemus”, Ethology, 102(1), 1–21.
#'
#' @examples
#' ## load data, use ordered factors as numeric, and grouped factor version of width
#' data("CrabSatellites", package = "vcdExtra")
#' CrabSatellites <- transform(CrabSatellites,
#'   color = as.numeric(color),
#'   spine = as.numeric(spine),
#'   cwidth = cut(width, c(-Inf, seq(23.25, 29.25), Inf))
#' )
#'
#' ## Agresti, Table 4.4
#' aggregate(CrabSatellites$satellites,
#'           list(CrabSatellites$cwidth), function(x)
#'   round(c(Number = length(x), Sum = sum(x), Mean = mean(x), Var = var(x)), digits = 2))
#'
#' ## Agresti, Figure 4.4
#' plot(tapply(satellites, cwidth, mean) ~ tapply(width, cwidth, mean),
#'   data = CrabSatellites,
#'   ylim = c(0, 6), pch = 19, cex = 1.5,
#'   xlab = "Mean carapace width (cm)",
#'   ylab = "Mean number of satellites")
#'
#' ## More examples: ?countreg::CrabSatellites` has examples of other plots and count data models
NULL

#' Loglinear Model Utilities
#'
#' @description
#' These functions generate lists of terms to specify a loglinear model in a
#' form compatible with \code{\link[stats]{loglin}} and also provide for
#' conversion to an equivalent \code{\link[MASS]{loglm}} specification or a
#' shorthand character string representation.
#'
#' They allow for a more conceptual way to specify such models by a function
#' for their type, as opposed to just an uninterpreted list of model terms and
#' also allow easy specification of marginal models for a given contingency
#' table.
#' They are intended to be used as tools in higher-level modeling and graphics
#' functions, but can also be used directly.
#'
#' @details
#' The main model specification functions, `conditional`, `joint`,
#' `markov`, \dots{}, `saturated`, return a list of vectors
#' indicating the marginal totals to be fit, via the `margin` argument to
#' \code{\link[stats]{loglin}}.
#' Each element of this list corresponds to a
#' high-order term in a hierarchical loglinear model, where, e.g., a term like
#' `c("A", "B")` is equivalent to the \code{\link[MASS]{loglm}} term
#' `"A:B"` and hence automatically includes all low-order terms.
#'
#' Note that these can be used to supply the `expected` argument for the
#' default \code{\link[vcd]{mosaic}} function, when the data is supplied as a
#' contingency table.
#'
#' The table below shows some typical results in terms of the standard
#' shorthand notation for loglinear models, with factors A, B, C, \dots{},
#' where brackets are used to delimit the high-order terms in the loglinear
#' model.
#' \tabular{llll}{
#'   **function** \tab **3-way** \tab **4-way** \tab **5-way** \cr
#'   `mutual` \tab  \[A\]  \[B\]  \[C\]  \tab  \[A\]  \[B\]  \[C\]  \[D\]  \tab \[A\]  \[B\]  \[C\]  \[D\]  \[E\] \cr
#'   `joint`  \tab  \[AB\]  \[C\]  \tab  \[ABC\]  \[D\]  \tab  \[ABCE\]  \[E\]  \cr
#'   `joint (with=1)` \tab   \[A\]  \[BC\]  \tab   \[A\]  \[BCD\]  \tab  \[A\]  \[BCDE\]  \cr
#'   `conditional`  \tab  \[AC\]  \[BC\]  \tab  \[AD\]  \[BD\]  \[CD\]  \tab   \[AE\]  \[BE\]  \[CE\]  \[DE\] \cr
#'   `condit (with=1)`  \tab  \[AB\]  \[AC\]  \tab   \[AB\]  \[AC\]  \[AD\]  \tab  \[AB\]  \[AC\]  \[AD\]  \[AE\] \cr
#'   `markov (order=1)`  \tab   \[AB\]  \[BC\]  \tab   \[AB\]  \[BC\]  \[CD\]  \tab  \[AB\]  \[BC\]  \[CD\]  \[DE\] \cr   
#'   `markov (order=2)`  \tab   \[A\]  \[B\]  \[C\]  \tab  \[ABC\]  \[BCD\]  \tab   \[ABC\]  \[BCD\]  \[CDE\]  \cr
#'   `saturated`  \tab \[ABC\] \tab \[ABCD\] \tab \[ABCDE\] \cr
#' }
#'
#'
#' `loglin2formula` converts the output of one of these to a model formula
#' suitable as the `formula` for of \code{\link[MASS]{loglm}}.
#'
#' `loglin2string` converts the output of one of these to a string
#' describing the loglinear model in the shorthand bracket notation,
#' e.g., `"[A,B] [A,C]"`.
#'
#' @aliases loglin-utilities conditional joint loglin2formula loglin2string markov mutual saturated
#' @param nf number of factors for which to generate model
#' @param table a contingency table used for factor names, typically the output from \code{\link[base]{table}}
#' @param factors names of factors used in the model when `table` is not specified
#' @param with    indices of the factors against which others are considered jointly independent
#' @family loglinear models
#' @rdname loglin-utilities
#' @export

joint <- function(nf,
                  table=NULL,
                  factors=1:nf,
                  with=nf) {
    if (!is.null(table)) factors <- names(dimnames(table))
    if (nf == 1) return (list(term1=factors[1]))
    if (nf == 2) return (list(term1=factors[1], term2=factors[2]))
    others <- setdiff(1:nf, with)
    result <- list(term1=factors[others], term2=factors[with])
    result
  }


#' models of conditional independence of some factors wrt one or more other factors

#' @param nf number of factors for which to generate model
#' @param table a contingency table used for factor names, typically the output from \code{\link[base]{table}}
#' @param factors names of factors used in the model when `table` is not specified
#' @param with    indices of the factors against which others are considered conditionally independent
#' @rdname loglin-utilities
#' @export

conditional <- function(nf,
                        table=NULL,
                        factors=1:nf,
                        with=nf) {
    if (!is.null(table)) factors <- names(dimnames(table))
    if (nf == 1) return (list(term1=factors[1]))
    if (nf == 2) return (list(term1=factors[1], term2=factors[2]))
    main <- setdiff(1:nf, with)
    others <- matrix(factors[with], length(with), length(main))
    result <- rbind(factors[main], others)
    result <- as.list(as.data.frame(result, stringsAsFactors=FALSE))
    names(result) <- paste('term', 1:length(result), sep='')
    result
  }

#' models of mutual independence of all factors

#' @param nf number of factors for which to generate model
#' @param table a contingency table used for factor names, typically the output from \code{\link[base]{table}}
#' @param factors names of factors used in the model when `table` is not specified
#' @rdname loglin-utilities
#' @export

mutual <- function(nf,
                   table=NULL,
                   factors=1:nf) {
   if (!is.null(table)) factors <- names(dimnames(table))
   result <- sapply(factors[1:nf], list)
   names(result) <- paste('term', 1:length(result), sep='')
   result
  }


#' saturated model: highest-order interaction

#' @param nf number of factors for which to generate model
#' @param table a contingency table used for factor names, typically the output from \code{\link[base]{table}}
#' @param factors names of factors used in the model when `table` is not specified
#' @rdname loglin-utilities
#' @export

saturated <- function(nf,
                      table=NULL,
                      factors=1:nf) {
	if (!is.null(table)) factors <- names(dimnames(table))
	list(term1=factors[1:nf])
}

# models of conditional independence, given one pair of variables
## Not needed: handled by condit, with length(with)>1

#condit2 <- function(nf, factors=1:nf, with=1:2) {
#    if (nf == 1) return (list(term1=factors[1]))
#    if (nf == 2) return (list(term1=factors[1], term2=factors[2]))
#    others <- setdiff(1:nf, with)
#    result <- rbind(factors[with], cbind(factors[others], factors[others]))
#    result <- as.list(as.data.frame(result, stringsAsFactors=FALSE))
#    names(result) <- paste('term', 1:length(result), sep='')
#    result
#}

#' markov models of a given order

#' @param nf number of factors for which to generate model
#' @param table a contingency table used for factor names, typically the output from \code{\link[base]{table}}
#' @param factors names of factors used in the model when `table` is not specified
#' @param order   order of the markov chain
#' @rdname loglin-utilities
#' @export

markov <- function(nf,
                   factors=1:nf,
                   order=1) {
    if (nf == 1) return (list(term1=factors[1]))
    if (nf == 2) return (list(term1=factors[1], term2=factors[2]))
    if (length(factors) < order+2) {
      warning(paste('Not enough factors for order', order, 'Markov chain; using order=1'))
      order <-1
      result <- rbind(factors[1:(nf-1)], factors[2:nf])
    }
    else {
      if (nf <= order+1) result <- factors[1:nf]
      else {
        result <- NULL
        for (i in 1:(order+1))
          result <- rbind(result, factors[i:(nf-order+i-1)])
      }
    }

    result <- as.list(as.data.frame(result, stringsAsFactors=FALSE))
    names(result) <- paste('term', 1:length(result), sep='')
    result
}

#' convert a loglin model to a model formula for loglm

#' @param  x a list of terms in a loglinear model, such as returned by `joint`, `conditional`, \dots
#' @param  env environment in which to evaluate the formula
#' @source Code from Henrique Dallazuanna, <wwwhsd@gmail.com>, R-help 7-4-2013
#' @rdname loglin-utilities
#' @export

loglin2formula <- function(x,
                           env = parent.frame()) {
	terms <- lapply(x, paste, collapse = ":")
	formula(sprintf(" ~ %s", do.call(paste, c(terms, sep = "+"))), env=env)
}

#' convert a loglin model to a string, using bracket notation for the high-order terms

#' @param x a list of terms in a loglinear model, such as returned by `joint`, `conditional`, \dots
#' @param brackets characters to use to surround model terms.  Either a single character string containing two characters
#'        or a character vector of length two.
#' @param sep characters used to separate factor names within a term
#' @param collapse  characters used to separate terms
#' @param abbrev   Unused as yet
#' @rdname loglin-utilities
#' @export

loglin2string <- function(x,
                          brackets = c('[', ']'),
                          sep=',',
                          collapse=' ',
                          abbrev) {
	if (length(brackets)==1 && (nchar(brackets)>1)) brackets <- unlist(strsplit(brackets, ""))
	terms <- lapply(x, paste, collapse=sep)
	terms <- paste(brackets[1], terms, brackets[2], sep='')
	paste(terms, collapse= ' ')
}

#' Mosaic Displays for glmlist and loglmlist Objects
#'
#' This function provides a convenient interface for viewing mosaic displays
#' associated with a collection of glm models for frequency tables that have
#' been stored in a `glmlist` or `loglmlist` object.  You can plot
#' either selected models individually, or mosaics for all models in an array
#' of viewports.
#'
#' Most details of the plots produced can be controlled via \dots{} arguments
#' as shown in some of the examples below.  In particular, with
#' `panel=sieve` you need to also pass `gp=shading_Friendly` to get a
#' color version.
#'
#' @aliases mosaic.glmlist mosaic.loglmlist
#' @param x         a `glmlist` or `loglmlist` object
#' @param selection the index or name of one `glm` or `loglm` object
#'        in `x`.  If no selection is specified, a menu of models is presented or
#'        all models are plotted.
#' @param panel     a \code{\link[vcd]{strucplot}} panel function, typically
#'        \code{\link[vcd]{mosaic}} or \code{\link[vcd]{sieve}}
#' @param type      a character string indicating whether the `"observed"` or
#'        the `"expected"` values of the table should be visualized
#' @param legend    logical: show a legend for residuals in the mosaic display(s)?
#'        The default behavior is to include a legend when only a single plot is
#'        shown, i.e., if `ask` is `TRUE` or a `selection` has been
#'        specified.
#' @param main either a logical, or a vector of character strings used for
#'        plotting the main title.  If main is a logical and `TRUE`, the name of
#'        the selected glm object is used.
#' @param ask logical: should the function display a menu of models, when one
#'        is not specified in `selection`? If `selection` is not supplied
#'        and `ask` is `TRUE` (the default), a menu of model names is
#'        presented; if `ask` is `FALSE`, mosaics for all models are plotted
#'        in an array.
#' @param graphics logical: use a graphic dialog box when `ask=TRUE`?
#' @param rows,cols when `ask=FALSE`, the number of rows and columns in
#'        which to plot the mosaics.
#' @param newpage start a new page? (only applies to `ask=FALSE`)
#' @param \dots other arguments passed to \code{\link{mosaic.glm}} and
#'        ultimately to \code{\link[vcd]{mosaic}}.
#' @return Returns the result of \code{\link{mosaic.glm}}.
#'
#' @author Michael Friendly
#' @importFrom grid viewport pushViewport
#' @seealso \code{\link{glmlist}}, \code{\link{loglmlist}}, \code{\link{Kway}}
#'
#' \code{\link{mosaic.glm}}, \code{\link[vcd]{mosaic}},
#' \code{\link[vcd]{strucplot}}, for the many parameters that control the
#' details of mosaic plots.
#' @family mosaic plots
#' @family glmlist functions
#'
#' @references
#' David Meyer, Achim Zeileis, and Kurt Hornik (2006). The
#' Strucplot Framework: Visualizing Multi-Way Contingency Tables with vcd.
#' *Journal of Statistical Software*, 17(3), 1-48.
#' <https://www.jstatsoft.org/v17/i03/>,
#' available as `vignette("strucplot", package="vcd")`.
#'
#' @keywords hplot
#' @examples
#'
#' data(JobSatisfaction, package="vcd")
#'
#' # view all pairwise mosaics
#' pairs(xtabs(Freq~management+supervisor+own, data=JobSatisfaction),
#'     shade=TRUE, diag_panel=pairs_diagonal_mosaic)
#'
#' modSat <- Kway(Freq ~ management+supervisor+own, data=JobSatisfaction,
#'                family=poisson, prefix="JobSat")
#' names(modSat)
#'
#' \dontrun{
#' mosaic(modSat)              # uses menu, if interactive()
#' }
#' mosaic(modSat, "JobSat.1")  # model label
#' mosaic(modSat, 2)           # model index
#'
#' # supply a formula to determine the order of variables in the mosaic
#' mosaic(modSat, 2, formula=~own+supervisor+management)
#'
#' mosaic(modSat, ask=FALSE)   # uses viewports
#'
#' # use a different panel function, label the observed valued in the cells
#' mosaic(modSat, 1, main=TRUE, panel=sieve, gp=shading_Friendly, labeling=labeling_values)
#'
#' data(Mental)
#' indep <- glm(Freq ~ mental+ses,
#'                 family = poisson, data = Mental)
#' Cscore <- as.numeric(Mental$ses)
#' Rscore <- as.numeric(Mental$mental)
#'
#' coleff <- glm(Freq ~ mental + ses + Rscore:ses,
#'                 family = poisson, data = Mental)
#' roweff <- glm(Freq ~ mental + ses + mental:Cscore,
#'                 family = poisson, data = Mental)
#' linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
#'                 family = poisson, data = Mental)
#'
#' # assign names for the plot labels
#' modMental <- glmlist(Indep=indep, ColEff=coleff, RowEff=roweff, `Lin x Lin`=linlin)
#' mosaic(modMental, ask=FALSE, margins=c(3,1,1,2), labeling_args=list(abbreviate_labs=5))
#'
#'
#'
#' @importFrom grid grid.newpage grid.layout pushViewport popViewport
#' @export
mosaic.glmlist <- function(x, selection,
		panel=mosaic,
		type=c("observed", "expected"),
		legend=ask | !missing(selection),
		main=NULL,
		ask=TRUE, graphics=TRUE, rows, cols, newpage=TRUE,
		...) {

#	calls <- sapply(x, mod.call)  # get model calls as strings
	models <- names(x)
	if (!is.null(main)) {
		if (is.logical(main) && main)
			main <- models
	}
	else main <- rep(main, length(x))

	type=match.arg(type)
	if (!missing(selection)){
		if (is.character(selection)) selection <- gsub(" ", "", selection)
		return(panel(x[[selection]], type=type, main=main[selection], legend=legend, ...))
	}
	# perhaps make these model labels more explicit for the menu
	if (ask & interactive()){
		repeat {
			selection <- menu(models, graphics=graphics, title="Select Model to Plot")
			if (selection == 0) break
			else panel(x[[selection]], type=type, main=main[selection], legend=legend, ...)
		}
	}
	else {
		nmodels <- length(x)
		mfrow <- mfrow(nmodels)
		if (missing(rows) || missing(cols)){
			rows <- mfrow[1]
			cols <- mfrow[2]
		}

		if (newpage) grid.newpage()
		lay <- grid.layout(nrow=rows, ncol = cols)
		pushViewport(viewport(layout = lay, y = 0, just = "bottom"))
		for (i in 1:rows) {
			for (j in 1:cols){
				if ((sel <-(i-1)*cols + j) > nmodels) break
				pushViewport(viewport(layout.pos.row=i, layout.pos.col=j))
				panel(x[[sel]], type=type, main=main[sel], newpage=FALSE, legend=legend, ...)
				popViewport()
			}
		}
	}
}

#' @rdname mosaic.glmlist
#' @export
mosaic.loglmlist <- function(x, selection,
		panel=mosaic,
		type=c("observed", "expected"),
		legend=ask | !missing(selection),
		main=NULL,
		ask=TRUE, graphics=TRUE, rows, cols, newpage=TRUE,
		...) {

	models <- names(x)
	strings <- as.vector(sapply(x, function(x) x$model.string))
	if (!is.null(main)) {
		if (is.logical(main) && main)
			main <- ifelse(as.vector(sapply(strings, is.null)), models, strings)
	}
	else main <- rep(main, length(x))

	type=match.arg(type)
	if (!missing(selection)){
		if (is.character(selection)) selection <- gsub(" ", "", selection)
		return(panel(x[[selection]], type=type, main=main[selection], legend=legend, ...))
	}
	# perhaps make these model labels more explicit for the menu
	if (ask & interactive()){
		repeat {
			selection <- menu(models, graphics=graphics, title="Select Model to Plot")
			if (selection == 0) break
			else panel(x[[selection]], type=type, main=main[selection], legend=legend, ...)
		}
	}
	else {
		nmodels <- length(x)
		mfrow <- mfrow(nmodels)
		if (missing(rows) || missing(cols)){
			rows <- mfrow[1]
			cols <- mfrow[2]
		}

		if (newpage) grid.newpage()
		lay <- grid.layout(nrow=rows, ncol = cols)
		pushViewport(viewport(layout = lay, y = 0, just = "bottom"))
		for (i in 1:rows) {
			for (j in 1:cols){
				if ((sel <-(i-1)*cols + j) > nmodels) break
				pushViewport(viewport(layout.pos.row=i, layout.pos.col=j))
				panel(x[[sel]], type=type, main=main[sel], newpage=FALSE, legend=legend, ...)
				popViewport()
			}
		}
	}
}


# from effects::utilities.R
mfrow <- function(n, max.plots=0){
	# number of rows and columns for array of n plots
	if (max.plots != 0 & n > max.plots)
		stop(paste("number of plots =",n," exceeds maximum =", max.plots))
	rows <- round(sqrt(n))
	cols <- ceiling(n/rows)
	c(rows, cols)
}

# from plot.lm: get model call as a string
# TODO: should use abbreviate()
mod.call <- function(x) {
        cal <- x$call
        if (!is.na(m.f <- match("formula", names(cal)))) {
            cal <- cal[c(1, m.f)]
            names(cal)[2L] <- ""
        }
        cc <- deparse(cal, 80)
        nc <- nchar(cc[1L], "c")
        abbr <- length(cc) > 1 || nc > 75
        cap <- if (abbr)
            paste(substr(cc[1L], 1L, min(75L, nc)), "...")
        else cc[1L]
		cap
}

# collapse a contingency table or ftable by re-assigning levels of table variables
# revised to accept an array also



#' Collapse Levels of a Table
#' 
#' Collapse (or re-label) variables in a a contingency table, array or
#' `ftable` object by re-assigning levels of the table variables.
#' 
#' Each of the \code{\dots{}} arguments must be of the form `variable =
#' levels`, where `variable` is the name of one of the table dimensions,
#' and `levels` is a character or numeric vector of length equal to the
#' corresponding dimension of the table.
#' 
#' @param table A \code{\link[base]{table}}, \code{\link[base]{array}} or
#' \code{\link[stats]{ftable}} object
#' @param \dots A collection of one or more assignments of factors of the table
#' to a list of levels
#' @return A `xtabs` and `table` object, representing the original
#' table with one or more of its factors collapsed or rearranged into other
#' levels.
#' @author Michael Friendly
#' @seealso 
#' \code{\link{expand.dft}} and \code{\link{as_caseform}}: expands a frequency 
#' data frame to case form.
#' 
#' \code{\link[base]{margin.table}} "collapses" a table in a different way, by
#' summing over table dimensions.
#' 
#' \code{\link{collapse_levels}} collapses in the same manner as 
#' \code{collapse.table} but also works for frequency and case form data.
#' @keywords manip attribute
#' @examples
#' 
#' # create some sample data in table form
#' sex <- c("Male", "Female")
#' age <- letters[1:6]
#' education <- c("low", 'med', 'high')
#' data <- expand.grid(sex=sex, age=age, education=education)
#' counts <- rpois(36, 100) 
#' data <- cbind(data, counts)
#' t1 <- xtabs(counts ~ sex + age + education, data=data)
#' structable(t1)
#' 
#' ##                  age   a   b   c   d   e   f
#' ## sex    education                            
#' ## Male   low           119 101 109  85  99  93
#' ##        med            94  98 103 108  84  84
#' ##        high           81  88  96 110 100  92
#' ## Female low           107 104  95  86 103  96
#' ##        med           104  98  94  95 110 106
#' ##        high           93  85  90 109  99  86
#' 
#' 
#' # collapse age to 3 levels
#' t2 <- collapse.table(t1, age=c("A", "A", "B", "B", "C", "C"))
#' structable(t2)
#' 
#' ##                  age   A   B   C
#' ## sex    education                
#' ## Male   low           220 194 192
#' ##        med           192 211 168
#' ##        high          169 206 192
#' ## Female low           211 181 199
#' ##        med           202 189 216
#' ##        high          178 199 185
#' 
#' 
#' # collapse age to 3 levels and pool education: "low" and "med" to "low"
#' t3 <- collapse.table(t1, age=c("A", "A", "B", "B", "C", "C"), 
#'     education=c("low", "low", "high"))
#' structable(t3)
#' 
#' ##                  age   A   B   C
#' ## sex    education                
#' ## Male   low           412 405 360
#' ##        high          169 206 192
#' ## Female low           413 370 415
#' ##        high          178 199 185
#' 
#' 
#' 
#' # change labels for levels of education to 1:3
#' t4 <- collapse.table(t1,  education=1:3)
#' structable(t4)
#' 
#' structable(t4)
#' ##                  age   a   b   c   d   e   f
#' ## sex    education                            
#' ## Male   1             119 101 109  85  99  93
#' ##        2              94  98 103 108  84  84
#' ##        3              81  88  96 110 100  92
#' ## Female 1             107 104  95  86 103  96
#' ##        2             104  98  94  95 110 106
#' ##        3              93  85  90 109  99  86
#' 
#' 
#' 
#' 
#' @export collapse.table
collapse.table <- function(table, ...) {
	nargs <- length(args <- list(...))
	if (!nargs) 
		return(table)
	if (inherits(table, "ftable"))
		table <- as.table(table)
	if (inherits(table, "array"))
		table <- as.table(table)
	if (inherits(table, "table")) {
		tvars <- names(dimnames(table))
		table <- as.data.frame.table(table)
		freq <- table[,"Freq"]
	}
	else stop("Argument must be a table, array or ftable object")
	
	names <- names(args)
	for (i in 1:nargs) {
		vals <- args[[i]]
		nm <- names[[i]]
		if(any(nm==tvars)) levels(table[[nm]]) <- vals
		else warning(nm, " is not among the table variables.")
	}
#    term <- paste(tvars, collapse = '+')
#    form <- as.formula(paste("freq ~", term))
#    cat("term: ", term, "\n")
	xtabs(as.formula(paste("freq ~", paste(tvars, collapse = '+'))), data=table)
}

#' The Logarithmic Series Distribution
#'
#' The logarithmic series distribution is a long-tailed distribution introduced
#' by Fisher etal. (1943) in connection with data on the abundance of
#' individuals classified by species.
#'
#' These functions provide the density, distribution function, quantile
#' function and random generation for the logarithmic series distribution with
#' parameter `prob`.
#'
#' The logarithmic series distribution with `prob` = \eqn{p} has density
#' \deqn{ p ( x ) = \alpha p^x / x } for \eqn{x = 1, 2, \dots},
#' where
#' \eqn{\alpha= -1 / \log(1 - p)} and \eqn{0 < p < 1}.
#' % Note that counts `x==2` cannot occur.
#'
#' @aliases logseries dlogseries plogseries qlogseries rlogseries
#' @param x,q vector of quantiles representing the number of events.
#' @param prob parameter for the distribution, `0 < prob < 1`
#' @param log,log.p logical; if TRUE, probabilities `p` are given as `log(p)`
#' @param lower.tail logical; if TRUE (default), probabilities are \eqn{P[X \le x]}{P[X <= x]}, otherwise, \eqn{P[X > x]}{P[X > x]}.
#' @param p vector of probabilities
#' @param max.value maximum value returned by `qlogseries`
#' @param n number of observations for `rlogseries`
#'
#' @return `dlogseries` gives the density,
#'         `plogseries` gives the cumulative distribution function,
#'         `qlogseries` gives the quantile function, and
#'         `rlogseries` generates random deviates.
#'
#'
#' @author Michael Friendly, using original code modified from the
#'         `gmlss.dist` package by Mikis Stasinopoulos.
#' @seealso \code{\link[stats]{Distributions}}
#'
#' @references
#' <https://en.wikipedia.org/wiki/Logarithmic_distribution>
#'
#' Fisher, R. A. and Corbet, A. S. and Williams, C. B. (1943). The relation
#' between the number of species and the number of individuals *Journal of
#' Animal Ecology*, 12, 42-58.
#'
#' @keywords distribution
#' @examples
#'
#' XL <-expand.grid(x=1:5, p=c(0.33, 0.66, 0.99))
#' lgs.df <- data.frame(XL, prob=dlogseries(XL[,"x"], XL[,"p"]))
#' lgs.df$p = factor(lgs.df$p)
#' str(lgs.df)
#'
#' require(lattice)
#' mycol <- palette()[2:4]
#' xyplot( prob ~ x, data=lgs.df, groups=p,
#' 	xlab=list('Number of events (k)', cex=1.25),
#' 	ylab=list('Probability',  cex=1.25),
#' 	type='b', pch=15:17, lwd=2, cex=1.25, col=mycol,
#' 	key = list(
#' 					title = 'p',
#' 					points = list(pch=15:17, col=mycol, cex=1.25),
#' 					lines = list(lwd=2, col=mycol),
#' 					text = list(levels(lgs.df$p)),
#' 					x=0.9, y=0.98, corner=c(x=1, y=1)
#' 					)
#' 	)
#'
#'
#' # random numbers
#' hist(rlogseries(200, prob=.4), xlab='x')
#' hist(rlogseries(200, prob=.8), xlab='x')
#'
#'
#'
#-----------------------------------------------------------------------------------------
#' @rdname logseries
#' @export
dlogseries<-function(x, prob = 0.5, log = FALSE)
 {
          if (any(prob <= 0) | any(prob >= 1) )  stop(paste("prob must be greater than 0 and less than 1", "\n", ""))
          if (any(x <= 0) )  stop(paste("x must be >0", "\n", ""))
       logfy <- x*log(prob)-log(x)-log(-log(1-prob))
      if(log == FALSE) fy <- exp(logfy) else fy <- logfy
          fy
  }

#----------------------------------------------------------------------------------------
#' @rdname logseries
#' @export
plogseries <- function(q, prob = 0.5, lower.tail = TRUE, log.p = FALSE)
  {
          if (any(prob <= 0) | any(prob >= 1) )  stop(paste("prob must be greater than 0 and less than 1", "\n", ""))
   if (any(q <= 0) )  stop(paste("q must be >0", "\n", ""))
        ly <- length(q)
       FFF <- rep(0,ly)
       nmu <- rep(prob, length = ly)
         j <- seq(along=q)
   for (i in j)
      {
        y.y <- q[i]
         mm <- nmu[i]
     allval <- seq(1,y.y)
     pdfall <- dlogseries(allval, prob = mm, log = FALSE)
     FFF[i] <- sum(pdfall)
      }
      cdf <- FFF
      cdf <- if(lower.tail==TRUE) cdf else 1-cdf
      cdf <- if(log.p==FALSE) cdf else log(cdf)
      cdf
  }
#----------------------------------------------------------------------------------------

#' @rdname logseries
#' @export
qlogseries <- function(p, prob=0.5,  lower.tail = TRUE, log.p = FALSE,
                 max.value = 10000)
  {
          if (any(prob <= 0) | any(prob >= 1) )  stop(paste("prob must be greater than 0 and less than 1", "\n", ""))
          if (any(p < 0) | any(p > 1.0001))  stop(paste("p must be between 0 and 1", "\n", ""))
          if (log.p==TRUE) p <- exp(p) else p <- p
          if (lower.tail==TRUE) p <- p else p <- 1-p
           ly <- length(p)
          QQQ <- rep(0,ly)
          nmu <- rep(prob, length = ly)
       for (i in seq(along=p))
      {
       cumpro <- 0
     if (p[i]+0.000000001 >= 1) QQQ[i] <- Inf
     else
        {
            for (j in seq(from = 1, to = max.value))
            {
            cumpro <-  plogseries(j, prob = nmu[i], log.p = FALSE)
           QQQ[i] <- j
       if  (p[i] <= cumpro ) break
            }
        }
      }
          QQQ
   }
#----------------------------------------------------------------------------------------

#' @rdname logseries
#' @export
rlogseries <- function(n, prob = 0.5)
  {
          if (any(prob <= 0) | any(prob >= 1) )  stop(paste("prob must be greater than 0 and less than 1", "\n", ""))
          if (any(n <= 0))  stop(paste("n must be a positive integer", "\n", ""))
          n <- ceiling(n)
          p <- runif(n)
          r <- qlogseries(p, prob=prob)
          r
  }

#' Abortion Opinion Data
#'
#' Opinions about abortion classified by gender and SES
#'
#' `Support_Abortion` is a natural response variable.
#'
#' The combinations of `Sex` and `Status` represent four independent
#' samples, having fixed `Sex`-`Status` marginal totals.  There were
#' 500 females and 600 males. Within the female group, 250 of low status and
#' 250 of high status were sampled. Similarly for the males, with 300 in each
#' of the low and hgh status sub-groups.
#'
#' This is an example of a product-multinomial sampling scheme. the
#' `Sex:Status` association must be included in any loglinear model where
#' the goal is to determine how attitude toward abortion depends on the others.
#'
#' Alternatively, a logit model for abortion support may provide a simpler
#' analysis.
#'
#' @name Abortion
#' @docType data
#' @format A 3-dimensional array resulting from cross-tabulating 3 variables
#' for 1100 observations. The variable names and their levels are:
#'
#' \tabular{rll}{
#'   No \tab Name                    \tab Levels \cr
#'   1  \tab `Sex`              \tab `"Female", "Male"`\cr
#'   2  \tab `Status`           \tab `"Lo", "Hi"`\cr
#'   3  \tab `Support_Abortion` \tab `"Yes", "No"`\cr
#' }
#'
#' @source
#'
#' Christensen, R. (1990).  *Log-Linear Models*, New York, NY: Springer-Verlag, p. 92, Example 3.5.2.
#'
#' Christensen, R. (1997).  *Log-Linear Models and Logistic Regression*,
#' New York, NY: Springer, p. 100, Example 3.5.2.
#' @keywords datasets
#' @examples
#'
#' data(Abortion)
#'
#'
#' ftable(Abortion)
#' mosaic(Abortion, shade=TRUE)
#'
#' # stratified by Sex
#' fourfold(aperm(Abortion, 3:1))
#' # stratified by Status
#' fourfold(aperm(Abortion, c(3,1,2)))
#'
#'
NULL





#' Traffic Accident Victims in France in 1958
#'
#' Bertin (1983) used these data to illustrate the cross-classification of data
#' by numerous variables, each of which could have various types and could be
#' assigned to various visual attributes.
#'
#' For modeling and visualization purposes, the data can be treated as a 4-way
#' table using loglinear models and mosaic displays, or as a frequency-weighted
#' data frame using a binomial response for `result` (`"Died"` vs.
#' `"Injured"`) and plots of predicted probabilities.
#'
#' `age` is an ordered factor, but arguably, `mode` should be treated
#' as ordered, with levels `Pedestrian` < `Bicycle` <
#' `Motorcycle` < `4-Wheeled` as Bertin does.  This affects the
#' parameterization in models, so we don't do this directly in the data frame.
#'
#' @name Accident
#' @docType data
#' @format A data frame in frequency form (comprising a 5 x 2 x 4 x 2 table)
#' with 80 observations on the following 5 variables.
#' \describe{
#'   \item{`age`}{an ordered factor with levels `0-9` < `10-19` <`20-29`
#'         < `30-49` < `50+`}
#'   \item{`result`}{a factor with levels `Died` `Injured`}
#'   \item{`mode`}{mode of transportation, a factor with levels `4-Wheeled` `Bicycle`
#'         `Motorcycle` `Pedestrian`}
#'   \item{`gender`}{a factor with levels `Female` `Male`}
#'   \item{`Freq`}{a numeric vector}
#' }
#'
#' @references Bertin, J. (1983), *Semiology of Graphics*, University of
#' Wisconsin Press.
#' @source Bertin (1983), p. 30; original data from the Ministere des Travaux
#' Publics
#' @keywords datasets
#' @examples
#'
#' # examples
#' data(Accident)
#' head(Accident)
#'
#' # for graphs, reorder mode
#' Accident$mode <- ordered(Accident$mode,
#'    levels=levels(Accident$mode)[c(4,2,3,1)])
#'
#' # Bertin's table
#' accident_tab <- xtabs(Freq ~ gender + mode + age + result, data=Accident)
#' structable(mode + gender ~ age + result, data=accident_tab)
#'
#' ## Loglinear models
#' ## ----------------
#'
#' # mutual independence
#' acc.mod0 <- glm(Freq ~ age + result + mode + gender,
#'                 data=Accident,
#'                 family=poisson)
#' LRstats(acc.mod0)
#'
#' mosaic(acc.mod0, ~mode + age + gender + result)
#'
#' # result as a response
#' acc.mod1 <- glm(Freq ~ age*mode*gender + result,
#'                 data=Accident,
#'                 family=poisson)
#' LRstats(acc.mod1)
#'
#' mosaic(acc.mod1, ~mode + age + gender + result,
#'     labeling_args = list(abbreviate = c(gender=1, result=4)))
#'
#' # allow two-way association of result with each explanatory variable
#' acc.mod2 <- glm(Freq ~ age*mode*gender + result*(age+mode+gender),
#'                 data=Accident,
#'                 family=poisson)
#' LRstats(acc.mod2)
#' mosaic(acc.mod2, ~mode + age + gender + result,
#'     labeling_args = list(abbreviate = c(gender=1, result=4)))
#'
#' acc.mods <- glmlist(acc.mod0, acc.mod1, acc.mod2)
#' LRstats(acc.mods)
#'
#' ## Binomial (logistic regression) models for result
#' ## ------------------------------------------------
#' library(car)  # for Anova()
#' acc.bin1 <- glm(result=='Died' ~ age + mode + gender,
#'     weights=Freq, data=Accident, family=binomial)
#' Anova(acc.bin1)
#'
#' acc.bin2 <- glm(result=='Died' ~ (age + mode + gender)^2,
#'     weights=Freq, data=Accident, family=binomial)
#' Anova(acc.bin2)
#'
#' acc.bin3 <- glm(result=='Died' ~ (age + mode + gender)^3,
#'     weights=Freq, data=Accident, family=binomial)
#' Anova(acc.bin3)
#'
#' # compare models
#' anova(acc.bin1, acc.bin2, acc.bin3, test="Chisq")
#'
#' # visualize probability of death with effect plots
#' \dontrun{
#' library(effects)
#' plot(allEffects(acc.bin1), ylab='Pr (Died)')
#'
#' plot(allEffects(acc.bin2), ylab='Pr (Died)')
#' }
#'
#'
#' #
NULL





#' Air Crash Data
#'
#' Data on all fatal commercial airplane crashes from 1993--2015. Excludes
#' small planes (less than 6 passengers) and non-commercial (cargo, military,
#' private) aircraft.
#'
#' `Phase` of the flight was cleaned by combining related variants,
#' spelling, etc.
#'
#' @name AirCrash
#' @docType data
#' @format A data frame with 439 observations on the following 5 variables.
#' \describe{
#'   \item{`Phase`}{phase of the flight, a factor with levels `en route` `landing`
#'         `standing` `take-off`
#' `unknown`}
#'   \item{`Cause`}{a factor with levels `criminal` `human error` `mechanical`
#'         `unknown` `weather`}
#'   \item{`date`}{date of crash, a Date}
#'   \item{`Fatalities`}{number of fatalities, a numeric vector}
#'   \item{`Year`}{year, a numeric vector}
#' }
#' @references Rick Wicklin,
#' <http://blogs.sas.com/content/iml/2015/03/30/visualizing-airline-crashes.html>
#' @source Originally from David McCandless,
#' <https://web.archive.org/web/2024/https://informationisbeautiful.net/visualizations/plane-truth-every-single-commercial-plane-crash-visualized/>,
#' with the data at
#' <https://docs.google.com/spreadsheets/d/1OvDq4_BtbR6nSnnHnjD5hVC3HQ-ulZPGbo0RDGbzM3Q/edit?usp=drive_web>,
#' downloaded April 14, 2015.
#' @keywords datasets
#' @examples
#'
#' data(AirCrash)
#' aircrash.tab <- xtabs(~Phase + Cause, data=AirCrash)
#' mosaic(aircrash.tab, shade=TRUE)
#'
#' # fix label overlap
#' mosaic(aircrash.tab, shade=TRUE,
#'        labeling_args=list(rot_labels=c(30, 30, 30, 30)))
#'
#' # reorder by Phase
#' phase.ord <- rev(c(3,4,1,2,5))
#' mosaic(aircrash.tab[phase.ord,], shade=TRUE,
#'        labeling_args=list(rot_labels=c(30, 30, 30, 30)),
#'        offset_varnames=0.5)
#'
#' # reorder by frequency
#' phase.ord <- order(rowSums(aircrash.tab), decreasing=TRUE)
#' cause.ord <- order(colSums(aircrash.tab), decreasing=TRUE)
#' mosaic(aircrash.tab[phase.ord,cause.ord], shade=TRUE,
#'        labeling_args=list(rot_labels=c(30, 30, 30, 30)))
#'
#'
#' library(ca)
#' aircrash.ca <- ca(aircrash.tab)
#' plot(aircrash.ca)
#'
NULL





#' Alligator Food Choice
#'
#' The Alligator data, from Agresti (2002), comes from a study of the primary
#' food choices of alligators in four Florida lakes. Researchers classified the
#' stomach contents of 219 captured alligators into five categories: Fish (the
#' most common primary food choice), Invertebrate (snails, insects, crayfish,
#' etc.), Reptile (turtles, alligators), Bird, and Other (amphibians, plants,
#' household pets, stones, and other debris).
#'
#' The table contains a fair number of 0 counts.
#'
#' `food` is the response variable.  `fish` is the most frequent
#' choice, and often taken as a baseline category in multinomial response
#' models.
#'
#' @name Alligator
#' @docType data
#' @format A frequency data frame with 80 observations on the following 5
#' variables.
#' \describe{
#'   \item{`lake`}{a factor with levels `George` `Hancock` `Oklawaha` `Trafford`}
#'   \item{`sex`}{a factor with levels `female` `male`}
#'   \item{`size`}{alligator size, a factor with levels `large` (>2.3m) `small` (<=2.3m)}
#'   \item{`food`}{primary food choice, a factor with levels `bird` `fish`
#'         `invert` `other` `reptile`}
#'   \item{`count`}{cell frequency, a numeric vector} }
#' @source Agresti, A. (2002). *Categorical Data Analysis*, New York:
#' Wiley, 2nd Ed., Table 7.1
#' @keywords datasets
#' @examples
#'
#' data(Alligator)
#'
#' # change from frequency data.frame to table
#' allitable <- xtabs(count ~ lake + sex + size + food, data=Alligator)
#' # Agresti's Table 7.1
#' structable(food ~ lake + sex + size, allitable)
#'
#'
#' plot(allitable, shade=TRUE)
#'
#' # mutual independence model
#' mosaic(~ food + lake + size, allitable, shade=TRUE)
#'
#' # food jointly independent of lake and size
#' mosaic(~ food + lake + size, allitable, shade=TRUE,
#'        expected = ~lake:size + food)
#'
#' if (require(nnet)) {
#' 	# multinomial logit model
#' 	mod1 <- multinom(food ~ lake + size + sex, data=Alligator, weights=count)
#' }
#'
#'
NULL





#' Effect of Exposure to Asbestos
#'
#' A two-way contingency table formed from the cross-classification of the
#' number of years of occupational exposure to asbestos and the diagnosed
#' severity of asbestosis of 1117 New York workers. Asbestosis is a chronic
#' lung disease that results in the lung tissue being scared due to contact
#' with the fibers which can lead to severe breathing difficulties.
#'
#' `exposure` and `grade` should be regarded as ordered factors. Beh
#' and Lombardo (2022) use this data to illustrate a polynomial biplot for
#' ordered categories.
#'
#' The data summarized here was studied by Beh and Smith (2011) and comes from
#' the original data collected and published by Selikoff (1981) who examined
#' the link between asbestos exposure and asbestosis severity in 1963.
#'
#' @name Asbestos
#' @docType data
#' @format The format is:
#' \preformatted{
#'  num [1:5, 1:4] 310 212 21 25 7 36 158 35 102 35 ...
#'  - attr(*, "dimnames")=List of 2
#'   ..$ exposure: chr [1:5] "0-9" "10-19" "20-29" "30-39" ...
#'   ..$ grade   : chr [1:4] "None" "Grade 1" "Grade 2" "Grade 3"#'
#'   }
#' @references Beh, E. J., and D. R. Smith (2011b), Real World Occupational
#' Epidemiology, Part 2: A Visual Interpretation of Statistical Significance,
#' *Archives of Environmental & Occupational Health*, **66**, 245-248.
#'
#' Selikoff, I. J. (1981), Household Risks With Inorganic Fibers,
#' *Bulletin of the New York Academy of Medicine*, **57**, 947-961.
#' @source Beh, E. J. & Lombardo, R. (2022). Features of the Polynomial Biplot
#' for Ordered Contingency Tables, *Journal of Computational and Graphical
#' Statistics*, 31:2, 403-412, DOI: 10.1080/10618600.2021.1990773, Table 1.
#' @keywords datasets
#' @examples
#'
#' data(Asbestos)
#' # mosaic plot
#' vcd::mosaic(Asbestos, shade=TRUE, legend=FALSE)
#'
#' # do the correspondence analysis
#' library(ca)
#' Asbestos.ca <- ca(Asbestos)
#'
#' plot(Asbestos.ca, lines=TRUE)
#'
#'
NULL





#' Bartlett Data on Plum Root Cuttings
#'
#' In an experiment to investigate the effect of cutting length (two levels)
#' and planting time (two levels) on the survival of plum root cuttings, 240
#' cuttings were planted for each of the 2 x 2 combinations of these factors,
#' and their survival was later recorded.
#'
#' Bartlett (1935) used these data to illustrate a method for testing for no
#' three-way interaction in a contingency table.
#'
#'
#' @name Bartlett
#' @docType data
#' @format A 3-dimensional array resulting from cross-tabulating 3 variables
#' for 960 observations. The variable names and their levels are:
#'
#' \tabular{rll}{
#'  dim \tab Name          \tab Levels                \cr
#'    1  \tab `Alive`  \tab `"Alive", "Dead"`\cr
#'    2  \tab `Time`   \tab `"Now", "Spring"`\cr
#'    3  \tab `Length` \tab `"Long", "Short"`\cr
#' }

#'
#' @references
#' Bartlett, M. S. (1935).  Contingency Table Interactions *Journal of the Royal Statistical Society*, Supplement,
#' 1935, 2, 248-252.
#'
#' @source
#'
#' Hand, D. and Daly, F. and Lunn, A. D.and McConway, K. J. and Ostrowski, E. (1994). *A Handbook of Small Data Sets*.
#' London: Chapman & Hall, p. 15, # 19.
#' @keywords datasets
#' @examples
#'
#' data(Bartlett)
#'
#' # measures of association
#' assocstats(Bartlett)
#' oddsratio(Bartlett)
#'
#' # Test models
#'
#' ## Independence
#' MASS::loglm(formula = ~Alive + Time + Length, data = Bartlett)
#'
#' ## No three-way association
#' MASS::loglm(formula = ~(Alive + Time + Length)^2, data = Bartlett)
#'
#' # Use woolf_test() for a formal test of homogeneity of odds ratios
#' vcd::woolf_test(Bartlett)
#'
#'
#' # Plots
#' fourfold(Bartlett, mfrow=c(1,2))
#'
#' mosaic(Bartlett, shade=TRUE)
#' pairs(Bartlett, gp=shading_Friendly)
#'
NULL





#' Burt (1950) Data on Hair, Eyes, Head and Stature
#'
#' Cyril Burt (1950) gave these data, on a sample of 100 people from Liverpool,
#' to illustrate the application of a method of factor analysis (later called
#' multiple correspondence analysis) applied to categorical data.
#'
#' He presented these data initially in the form that has come to be called a
#' "Burt table", giving the univariate and bivariate frequencies for an n-way
#' frequency table.
#'
#' Burt says: "In all, 217 individuals were examined, about two-thirds of them
#' males. But, partly to simplify the calculations and partly because the later
#' observations were rather more trustworthy, I shall here restrict my analysis
#' to the data obtained from the last hundred males in the series."
#'
#' `Head` and `Stature` reflect a binary coding where people are
#' classified according to whether they are below or above the average for the
#' population.
#'
#' @name Burt
#' @docType data
#' @format A frequency data frame (representing a 3 x 3 x 2 x 2 frequency
#' table) with 36 cells on the following 5 variables.
#' \describe{
#'   \item{`Hair`}{hair color, a factor with levels `Fair` `Red` `Dark`}
#'   \item{`Eyes`}{eye color, a factor with levels `Light` `Mixed` `Dark`}
#'   \item{`Head`}{head shape, a factor with levels `Narrow` `Wide`}
#'   \item{`Stature`}{height, a factor with levels `Tall` `Short`}
#'   \item{`Freq`}{a numeric vector}
#' }
#'
#' @source Burt, C. (1950). The factorial analysis of qualitative data,
#' *British Journal of Statistical Psychology*, **3**(3), 166-185.
#' Table IX.
#' @keywords datasets
#' @examples
#'
#' data(Burt)
#' mosaic(Freq ~ Hair + Eyes + Head + Stature, data=Burt, shade=TRUE)
#'
#' #or
#' burt.tab <- xtabs(Freq ~ Hair + Eyes + Head + Stature, data=Burt)
#' mosaic(burt.tab, shade=TRUE)
#'
NULL





#' Risk Factors for Infection in Caesarian Births
#'
#' Data from infection from birth by Caesarian section, classified by
#' `Risk` (two levels), whether `Antibiotics` were used (two levels)
#' and whether the Caesarian section was `Planned` or not.  The outcome is
#' `Infection` (three levels).
#'
#' @details `Infection` is regarded as the response variable here.  There are quite
#' a few 0 cells here, particularly when `Risk` is absent and the
#' Caesarian section was unplanned. Should these be treated as structural or
#' sampling zeros?
#'
#' @name Caesar
#' @docType data
#' @format A 4-dimensional array resulting from cross-tabulating 4 variables
#' for 251 observations. The variable names and their levels are:
#'
#'   \tabular{rll}{
#'   dim \tab Name \tab Levels \cr
#'     1 \tab `Infection`\tab `"Type 1", "Type 2", "None"`\cr
#'     2 \tab `Risk`\tab `"Yes", "No"` (presence of risk factors)\cr
#'     3 \tab `Antibiotics`\tab `"Yes", "No"` (were antibiotics given?)\cr
#'     4 \tab `Planned`\tab `"Yes", "No"` (was the C section planned?)\cr
#'}
#'
#'
#' @seealso \code{\link[Fahrmeir]{caesar}} for the same data recorded as a
#' frequency data frame with other variables.
#' @source
#'
#' % \cite{Fahrmeir:94}
#' Fahrmeir, L. & Tutz, G. (1994). Multivariate
#' Statistical Modelling Based on Generalized Linear Models New York: Springer
#' Verlag, Table 1.1.
#' @keywords datasets
#' @examples
#'
#' data(Caesar)
#' #display table;  note that there are quite a few 0 cells
#' structable(Caesar)
#' require(MASS)
#'
#' # baseline model, Infection as response
#' Caesar.mod0 <- loglm(~Infection + (Risk*Antibiotics*Planned),
#'                      data=Caesar)
#'
#' # NB: Pearson chisq cannot be computed due to the 0 cells
#' Caesar.mod0
#'
#' mosaic(Caesar.mod0, main="Baseline model")
#'
#' # Illustrate handling structural zeros
#' zeros <- 0+ (Caesar >0)
#' zeros[1,,1,1] <- 1
#' structable(zeros)
#'
#' # fit model excluding possible structural zeros
#' Caesar.mod0s <- loglm(~Infection + (Risk*Antibiotics*Planned),
#'                       data=Caesar,
#' 	                    start=zeros)
#' Caesar.mod0s
#'
#' anova(Caesar.mod0, Caesar.mod0s, test="Chisq")
#'
#' mosaic (Caesar.mod0s)
#'
#' # what terms to add?
#' add1(Caesar.mod0, ~.^2, test="Chisq")
#'
#' # add Association of Infection:Antibiotics
#' Caesar.mod1 <- update(Caesar.mod0, ~ . + Infection:Antibiotics)
#' anova(Caesar.mod0, Caesar.mod1, test="Chisq")
#'
#' mosaic(Caesar.mod1,
#'        gp=shading_Friendly,
#'        main="Adding Infection:Antibiotics")
#'
#'
NULL





#' Survival of Breast Cancer Patients
#'
#' Three year survival of 474 breast cancer patients according to nuclear grade
#' and diagnostic center.
#'
#'
#' @name Cancer
#' @docType data
#' @format A 3-dimensional array resulting from cross-tabulating 3 variables
#' for 474 observations. The variable names and their levels are:
#'
#' \tabular{rll}{
#'   dim \tab Name \tab Levels \cr
#'     1\tab `Survival`\tab  `"Died", "Surv"`\cr
#'     2\tab `Grade`\tab `"Malignant", "Benign"`\cr
#'     3\tab `Center`\tab `"Boston", "Glamorgan"`\cr
#'     }
#' @source
#'
#' Lindsey, J. K. (1995).
#' Analysis of Frequency and Count Data Oxford, UK: Oxford University Press. p.
#' 38, Table 2.5.
#'
#' Whittaker, J. (1990) Graphical Models in Applied Multivariate Statistics New
#' York: John Wiley and Sons, p. 220.
#' @keywords datasets
#' @examples
#'
#' data(Cancer)
#'
#' MASS::loglm(~Survival + Grade + Center, data = Cancer)
#'
#' vcd::mosaic(Cancer, shade=TRUE)
#'
NULL





#' Advertising Behavior by Males Cormorants
#'
#' Male double-crested cormorants use advertising behavior to attract females
#' for breeding. In this study by Meagan McRae (2015), cormorants were observed
#' two or three times a week at six stations in a tree-nesting colony for an
#' entire season, April 10, 2014-July 10, 2014. The number of advertising birds
#' was counted and these observations were classified by characteristics of the
#' trees and nests.
#'
#' The goal is to determine how this behavior varies temporally over the season
#' and spatially, as well as with characteristics of nesting sites.
#'
#' Observations were made on only 2 days in weeks 3 and 4, but 3 days in all
#' other weeks. One should use log(days) as an offset, so that the response
#' measures rate.
#'
#' `Cormorants$days <- ifelse(Cormorants$week \%in\% 3:4, 2, 3)`
#'
#' @name Cormorants
#' @docType data
#' @format A data frame with 343 observations on the following 8 variables.
#'
#' \describe{
#'   \item{`category`}{Time of season, divided into 3 categories based on breeding chronology, an ordered factor with levels `Pre` < `Incubation` < `Chicks Present`}
#'   \item{`week`}{Week of the season}
#'   \item{`station`}{Station of observations on two different peninsulas in a park, a factor with levels `B1` `B2` `C1` `C2` `C3` `C4`}
#'   \item{`nest`}{Type of nest, an ordered factor with levels `no` < `partial` < `full`}
#'   \item{`height`}{Relative height of bird in the tree, an ordered factor with levels `low` < `mid` < `high`}
#'   \item{`density`}{Number of other nests in the tree, an ordered factor with levels `zero` < `few` < `moderate` < `high`}
#'   \item{`tree_health`}{Health of the tree the bird is advertising in, a factor with levels `dead` `healthy`}
#'   \item{`count`}{Number of birds advertising, a numeric vector}
#' }

#' @source McRae, M. (2015). Spatial, Habitat and Frequency Changes in
#' Double-crested Cormorant Advertising Display in a Tree-nesting Colony.
#' Unpublished MA project, Environmental Studies, York University.
#' @keywords datasets
#' @examples
#'
#' data(Cormorants)
#' str(Cormorants)
#'
#' if (require("ggplot2")) {
#'   print(ggplot(Cormorants, aes(count)) +
#'     geom_histogram(binwidth=0.5) +
#' 	  labs(x="Number of birds advertising"))
#'
#' # Quick look at the data, on the log scale, for plots of `count ~ week`,
#' #   stratified by something else.
#'
#'   print(ggplot(Cormorants, aes(week, count, color=height)) +
#'     geom_jitter() +
#' 	  stat_smooth(method="loess", size=2) +
#' 	  scale_y_log10(breaks=c(1,2,5,10)) +
#' 	  geom_vline(xintercept=c(4.5, 9.5)))
#' }
#'
#' # ### models using week
#' fit1 <-glm(count ~ week + station + nest + height + density + tree_health,
#'            data=Cormorants,
#'            family =  poisson)
#'
#' if (requireNamespace("car"))
#'   car::Anova(fit1)
#'
#' # plot fitted effects
#' if (requireNamespace("effects"))
#'   plot(effects::allEffects(fit1))
#'
#'
NULL





#' London Cycling Deaths
#'
#' A data frame containing the number of deaths of cyclists in London from 2005
#' through 2012 in each fortnightly period.  Aberdein & Spiegelhalter (2013)
#' discuss these data in relation to the observation that six cyclists died in
#' London between Nov. 5 and Nov. 13, 2013.
#'
#'
#' @name CyclingDeaths
#' @docType data
#' @format A data frame with 208 observations on the following 2 variables.
#' \describe{
#'   \item{`date`}{a Date}
#'   \item{`deaths`}{number of deaths, a numeric vector}
#'   }
#' @references Aberdein, Jody and Spiegelhalter, David (2013). Have London's
#' roads become more dangerous for cyclists? *Significance*, 10(6),
#' 46--48.
#' @source
#' <https://www.data.gov.uk/dataset/cb7ae6f0-4be6-4935-9277-47e5ce24a11f/road-accidents-safety-data>,
#' STATS 19 data, 2005-2012, using the files `Casualty0512.csv` and
#' `Accidents0512.csv`
#' @keywords datasets
#' @examples
#'
#' data(CyclingDeaths)
#'
#' plot(deaths ~ date, data=CyclingDeaths,
#'   type="h",
#' 	lwd=3,
#' 	ylab="Number of deaths",
#' 	axes=FALSE)
#' axis(1, at=seq(as.Date('2005-01-01'),
#'                by='years',
#'                length.out=9),
#'      labels=2005:2013)
#' axis(2, at=0:3)
#'
#' # make a one-way frequency table
#' CyclingDeaths.tab <- table(CyclingDeaths$deaths)
#'
#' gf <- goodfit(CyclingDeaths.tab)
#' gf
#' summary(gf)
#'
#' rootogram(gf, xlab="Number of Deaths")
#' distplot(CyclingDeaths.tab)
#'
#' # prob of 6 or more deaths in one fortnight
#' lambda <- gf$par$lambda
#' ppois(5, lambda, lower.tail=FALSE)
#'
NULL





#' Dayton Student Survey on Substance Use
#'
#' This data, from Agresti (2002), Table 9.1, gives the result of a 1992 survey
#' in Dayton Ohio of 2276 high school seniors on whether they had ever used
#' alcohol, cigarettes and marijuana.
#'
#' Agresti uses the letters G (`sex`), R (`race`), A
#' (`alcohol`), C (`cigarette`), M (`marijuana`) to refer to the
#' table variables, and this usage is followed in the examples below.
#'
#' Background variables include `sex` and `race` of the respondent
#' (GR), typically treated as explanatory, so that any model for the full table
#' should include the term `sex:race`. Models for the reduced table,
#' collapsed over `sex` and `race` are not entirely unreasonable, but
#' don't permit the estimation of the effects of these variables on the
#' responses.
#'
#' The full 5-way table contains a number of cells with counts of 0 or 1, as
#' well as many cells with large counts, and even the ACM table collapsed over
#' GR has some small cell counts.  Consequently, residuals for these models in
#' mosaic displays are best represented as standardized (adjusted) residuals.
#'
#' @name DaytonSurvey
#' @docType data
#' @format A frequency data frame with 32 observations on the following 6
#' variables.
#' \describe{
#'   \item{`cigarette`}{a factor with levels `Yes` `No`}
#'   \item{`alcohol`}{a factor with levels `Yes` `No`}
#'   \item{`marijuana`}{a factor with levels `Yes` `No`}
#'   \item{`sex`}{a factor with levels `female` `male`}
#'   \item{`race`}{a factor with levels `white` `other`}
#'   \item{`Freq`}{a numeric vector}
#' }

#' @references Thompson, L. (2009). *R (and S-PLUS) Manual to Accompany
#' Agresti's Categorical Data*,
#' http://www.stat.ufl.edu/~aa/cda/Thompson_manual.pdf
#' @source Agresti, A. (2002). *Categorical Data Analysis*, 2nd Ed., New
#' York: Wiley-Interscience, Table 9.1, p. 362.
#' @keywords datasets
#' @examples
#'
#' data(DaytonSurvey)
#'
#' # mutual independence
#' mod.0  <- glm(Freq ~ ., data=DaytonSurvey, family=poisson)
#'
#' # mutual independence + GR
#' mod.GR <- glm(Freq ~ . + sex*race, data=DaytonSurvey, family=poisson)
#' anova(mod.GR, test = "Chisq")
#'
#' # all two-way terms
#' mod.all2way <- glm(Freq ~ .^2, data=DaytonSurvey, family=poisson)
#' anova(mod.all2way, test = "Chisq")
#'
#' # compare models
#' LRstats(mod.0, mod.GR, mod.all2way)
#'
#' # collapse over sex and race
#' Dayton.ACM <- aggregate(Freq ~ cigarette+alcohol+marijuana,
#'                         data=DaytonSurvey,
#'                         FUN=sum)
#' Dayton.ACM
#'
NULL





#' Dependencies of R Packages
#'
#' This one-way table gives the type-token distribution of the number of
#' dependencies declared in 4983 packages listed on CRAN on January 17, 2014.
#'
#'
#' @name Depends
#' @docType data
#' @format The format is a one-way frequency table of counts of packages with
#' 0, 1, 2, ... dependencies.
#'
#' \preformatted{
#'  table' int [1:15(1d)] 986 1347 993 685 375 298 155 65 32 19 ...
#'  - attr(*, "dimnames")=List of 1
#'  ..$ Depends: chr [1:15] "0" "1" "2" "3" ...
#' }
#' @source Using code from
#' <https://www.r-bloggers.com/2013/12/a-look-at-the-distribution-of-r-package-dependencies/>
#' @keywords datasets
#' @examples
#'
#' data(Depends)
#' plot(Depends,
#'      xlab="Number of Dependencies",
#'      ylab="Number of R Packages",
#'      lwd=8)
#'
#' # what type of distribution?
#' # Ord_plot can't classify this!
#' Ord_plot(Depends)
#'
#' \dontrun{
#' # The code below, from Joseph Rickert, downloads and tabulates the data
#' p <- as.data.frame(available.packages(),stringsAsFactors=FALSE)
#' names(p)
#'
#' pkgs <- data.frame(p[,c(1,4)])                  # Pick out Package names and Depends
#' row.names(pkgs) <- NULL                         # Get rid of row names
#' pkgs <- pkgs[complete.cases(pkgs[,2]),]         # Remove NAs
#'
#' pkgs$Depends2 <-strsplit(pkgs$Depends,",")      # split list of Depends
#' pkgs$numDepends <- as.numeric(lapply(pkgs$Depends2,length)) # Count number of dependencies in list
#' zeros <- c(rep(0,dim(p)[1] - dim(pkgs)[1]))     # Account for packages with no dependencies
#' Deps <- as.vector(c(zeros,pkgs$numDepends))     # Set up to tablate
#' Depends <- table(Deps)
#'
#' }
#'
NULL





#' Detergent Preference Data
#'
#' Cross-classification of a sample of 1008 consumers according to (a) the
#' softness of the laundry water used, (b) previous use of detergent Brand M,
#' (c) the temperature of laundry water used and (d) expressed preference for
#' Brand X or Brand M in a blind trial.
#'
#'
#' @name Detergent
#' @docType data
#' @format A 4-dimensional array resulting from cross-tabulating 4 variables
#' for 1008 observations. The variable names and their levels are:
#'
#'   \tabular{rll}{
#' dim \tab Name \tab Levels \cr
#' 1\tab `Temperature`\tab `"High", "Low"`\cr
#' 2\tab `M_User`\tab `"Yes", "No"`\cr
#' 3\tab `Preference`\tab `"Brand X", "Brand M"`\cr
#' 4\tab `Water_softness`\tab `"Soft", "Medium", "Hard"`\cr
#' }

#' @references
#' Ries, P. N. & Smith, H. (1963). The use of
#' chi-square for preference testing in multidimensional problems.
#' *Chemical Engineering Progress*, 59, 39-43.
#' @source
#'
#' Fienberg, S. E. (1980). *The Analysis of
#' Cross-Classified Categorical Data* Cambridge, MA: MIT Press, p. 71.
#' @keywords datasets
#' @examples
#'
#' data(Detergent)
#'
#' # basic mosaic plot
#' mosaic(Detergent, shade=TRUE)
#'
#' require(MASS)
#' (det.mod0 <- loglm(~ Preference + Temperature + M_User + Water_softness,
#'                    data=Detergent))
#' # examine addition of two-way terms
#' add1(det.mod0, ~ .^2, test="Chisq")
#'
#' # model for Preference as a response
#' (det.mod1 <- loglm(~ Preference + (Temperature * M_User * Water_softness),
#'                    data=Detergent))
#' mosaic(det.mod0)
#'
#'
#'
NULL





#' Survival in the Donner Party
#'
#' This data frame contains information on the members of the Donner Party, a
#' group of people who attempted to migrate to California in 1846. They were
#' trapped by an early blizzard on the eastern side of the Sierra Nevada
#' mountains, and before they could be rescued, nearly half of the party had
#' died.
#'
#' What factors affected who lived and who died?
#'
#' This data frame uses the person's name as row labels. `family` reflects
#' a recoding of the last names of individuals to reduce the number of factor
#' levels. The main families in the Donner party were: Donner, Graves, Breen
#' and Reed. The families of Murphy, Foster and Pike are grouped as
#' `'MurFosPik'`, those of Fosdick and Wolfinger are coded as
#' `'FosdWolf'`, and all others as `'Other'`.
#'
#' `survived` is the response variable. What kind of models should be used
#' here?
#'
#' @name Donner
#' @docType data
#' @format A data frame with 90 observations on the following 5 variables.
#'
#' \describe{
#'   \item{`family`}{family name, a factor with 10 levels }
#'   \item{`age`}{age of person, a numeric vector}
#'   \item{`sex`}{a factor with levels `Female` `Male`}
#'   \item{`survived`}{a numeric vector, 0 or 1}
#'   \item{`death`}{date of death for those who died before rescue, a POSIXct}
#' }
#'
#' @seealso `donner` in \pkg{alr3}, \code{\link[Sleuth2]{case2001}} in
#' \pkg{Sleuth2}(adults only) provide similar data sets.
#'
#' @references Ramsey, F.L. and Schafer, D.W. (2002).  *The Statistical
#' Sleuth: A Course in Methods of Data Analysis*, (2nd ed), Duxbury.
#'
#' Friendly, M. and Meyer, D. (2016).  *Discrete Data Analysis with R:
#' Visualization and Modeling Techniques for Categorical and Count Data*.  Boca
#' Raton, FL: Chapman & Hall/CRC. <http://ddar.datavis.ca>.
#' @source D. K. Grayson, 1990, "Donner party deaths: A demographic
#' assessment", *J. Anthropological Research*, **46**, 223-242.
#'
#' Johnson, K. (1996). *Unfortunate Emigrants: Narratives of the Donner
#' Party*.  Logan, UT: Utah State University Press.  Additions, and dates of
#' death from <http://user.xmission.com/~octa/DonnerParty/Roster.htm>.
#' @keywords datasets
#' @examples
#'
#' # conditional density plots
#' op <- par(mfrow=c(1,2), cex.lab=1.5)
#' cdplot(factor(survived) ~ age,
#'        subset=sex=='Male',
#'        data=Donner,
#'        main="Donner party: Males",
#'        ylevels=2:1,
#'        ylab="Survived",
#'        yaxlabels=c("yes", "no"))
#' with(Donner, rug(jitter(age[sex=="Male"]),
#'                  col="white", quiet=TRUE))
#'
#' cdplot(factor(survived) ~ age,
#'        subset=sex=='Female',
#'        data=Donner,
#'        main="Donner party: Females",
#'        ylevels=2:1,
#'        ylab="Survived",
#'        yaxlabels=c("yes", "no"))
#' with(Donner, rug(jitter(age[sex=="Female"]),
#'                  col="white", quiet=TRUE))
#' par(op)
#'
#'
#' # fit some models
#' (mod1 <- glm(survived ~ age + sex, data=Donner, family=binomial))
#' (mod2 <- glm(survived ~ age * sex, data=Donner, family=binomial))
#' anova(mod2, test="Chisq")
#'
#' (mod3 <- glm(survived ~ poly(age,2) * sex, data=Donner, family=binomial))
#' anova(mod3, test="Chisq")
#' LRstats(glmlist(mod1, mod2, mod3))
#'
#' # plot fitted probabilities from mod2 and mod3
#' # idea from: http://www.ling.upenn.edu/~joseff/rstudy/summer2010_ggplot2_intro.html
#' if (require("ggplot2")) {
#'
#' # separate linear fits on age for M/F
#' print(ggplot(Donner, aes(age, survived, color = sex)) +
#'   geom_point(position = position_jitter(height = 0.02, width = 0)) +
#'   stat_smooth(method = "glm",
#'               method.args = list(family = binomial),
#'               formula = y ~ x,
#'               alpha = 0.2,
#'               size=2,
#'               aes(fill = sex)))
#'
#' # separate quadratics
#' print(ggplot(Donner, aes(age, survived, color = sex)) +
#'   geom_point(position = position_jitter(height = 0.02, width = 0)) +
#'   stat_smooth(method = "glm",
#'               method.args = list(family = binomial),
#'               formula = y ~ poly(x,2),
#'               alpha = 0.2,
#'               size=2,
#'               aes(fill = sex)))
#'
#' }
#'
NULL





#' USA 1970 Draft Lottery Data
#'
#' This data set gives the results of the 1970 US draft lottery, in the form of
#' a data frame.
#'
#' The draft lottery was used to determine the order in which eligible men
#' would be called to the Selective Service draft. The days of the year
#' (including February 29) were represented by the numbers 1 through 366
#' written on slips of paper. The slips were placed in separate plastic
#' capsules that were mixed in a shoebox and then dumped into a deep glass jar.
#' Capsules were drawn from the jar one at a time.
#'
#' The first number drawn was 258 (September 14), so all registrants with that
#' birthday were assigned lottery number `Rank` 1. The second number drawn
#' corresponded to April 24, and so forth.  All men of draft age (born 1944 to
#' 1950) who shared a birthdate would be called to serve at once. The first 195
#' birthdates drawn were later called to serve in the order they were drawn;
#' the last of these was September 24.
#'
#' @name Draft1970
#' @docType data
#' @format A data frame with 366 observations on the following 3 variables.
#' \describe{
#'   \item{`Day`}{day of the year, 1:366}
#'   \item{`Rank`}{draft priority rank of people born on that day}
#'   \item{`Month`}{an ordered factor with levels `Jan` < `Feb` \dots < `Dec`}
#' }
#'
#' @seealso \code{\link{Draft1970table}}
#' @references Fienberg, S. E. (1971), "Randomization and Social Affairs: The
#' 1970 Draft Lottery," *Science*, 171, 255-261.
#'
#' <https://en.wikipedia.org/wiki/Draft_lottery_(1969)>
#'
#' Hanley, J. A., "US Selective Service Lotteries 1917-1975: Datasets,"
#' <https://jhanley.biostat.mcgill.ca/DraftLotteries/>
#' @source Starr, N. (1997). Nonrandom Risk: The 1970 Draft Lottery,
#' *Journal of Statistics Education*, v.5, n.2
#' \doi{10.1080/10691898.1997.11910534}
#' @keywords datasets
#' @examples
#'
#' data(Draft1970)
#'
#' # scatterplot
#' plot(Rank ~ Day, data=Draft1970)
#' with(Draft1970, lines(lowess(Day, Rank), col="red", lwd=2))
#' abline(lm(Rank ~ Day, data=Draft1970), col="blue")
#'
#' # boxplots
#' plot(Rank ~ Month, data=Draft1970, col="bisque")
#'
#' lm(Rank ~ Month, data=Draft1970)
#' anova(lm(Rank ~ Month, data=Draft1970))
#'
#' # make the table version
#' Draft1970$Risk <- cut(Draft1970$Rank, breaks=3, labels=c("High", "Med", "Low"))
#' with(Draft1970, table(Month, Risk))
#'
NULL





#' USA 1970 Draft Lottery Table
#'
#' This data set gives the results of the 1970 US draft lottery, in the form of
#' a frequency table. The rows are months of the year, Jan--Dec and columns
#' give the number of days in that month which fall into each of three draft
#' risk categories High, Medium, and Low, corresponding to the chances of being
#' called to serve in the US army.
#'
#' The lottery numbers are divided into three categories of risk of being
#' called for the draft -- High, Medium, and Low -- each representing roughly
#' one third of the days in a year.  Those birthdays having the highest risk
#' have lottery numbers 1-122, medium risk have numbers 123-244, and the lowest
#' risk category contains lottery numbers 245-366.
#'
#' @name Draft1970table
#' @docType data
#' @format The format is:
#'
#' \preformatted{
#' 'table' int [1:12, 1:3] 9 7 5 8 9 11 12 13 10 9 ...
#' - attr(*, "dimnames")=List of 2
#' ..$ Month: chr [1:12] "Jan" "Feb" "Mar" "Apr" ...
#' ..$ Risk : chr [1:3] "High" "Med" "Low"
#' }
#'
#' @seealso \code{\link{Draft1970}}
#' @references Fienberg, S. E. (1971), "Randomization and Social Affairs: The
#' 1970 Draft Lottery," *Science*, 171, 255-261.
#'
#' Starr, N. (1997). Nonrandom Risk: The 1970 Draft Lottery, *Journal of
#' Statistics Education*, v.5, n.2
#' \doi{10.1080/10691898.1997.11910534}
#' @source This data is available in several forms, but the table version was
#' obtained from
#'
#' <https://sas.uwaterloo.ca/~rwoldfor/software/eikosograms/data/draft-70>
#' @keywords datasets
#' @examples
#'
#' data(Draft1970table)
#' chisq.test(Draft1970table)
#'
#' # plot.table -> graphics:::mosaicplot
#' plot(Draft1970table, shade=TRUE)
#' mosaic(Draft1970table, gp=shading_Friendly)
#'
#' # correspondence analysis
#' if(require(ca)) {
#'   ca(Draft1970table)
#'   plot(ca(Draft1970table))
#' }
#'
#' # convert to a frequency data frame with ordered factors
#' Draft1970df <- as.data.frame(Draft1970table)
#'
#' Draft1970df <- within(Draft1970df, {
#'   Month <- ordered(Month)
#'   Risk <- ordered(Risk, levels=rev(levels(Risk)))
#'   })
#' str(Draft1970df)
#'
#' # similar model, as a Poisson GLM
#' indep <- glm(Freq ~ Month + Risk, family = poisson, data = Draft1970df)
#'
#' mosaic(indep, residuals_type="rstandard", gp=shading_Friendly)
#'
#' # numeric scores for tests of ordinal factors
#' Cscore <- as.numeric(Draft1970df$Risk)
#' Rscore <- as.numeric(Draft1970df$Month)
#'
#' # linear x linear association between Month and Risk
#' linlin <- glm(Freq ~ Month + Risk + Rscore:Cscore, family = poisson, data = Draft1970df)
#'
#' # compare models
#' anova(indep, linlin, test="Chisq")
#' mosaic(linlin, residuals_type="rstandard", gp=shading_Friendly)
#'
#'
#'
NULL





#' Sources of Knowledge of Cancer
#'
#' Observational data on a sample of 1729 individuals, cross-classified in a
#' 2^5 table according to their sources of information (read newspapers, listen
#' to the radio, do 'solid' reading, attend lectures) and whether they have
#' good or poor knowledge regarding cancer.  Knowledge of cancer is often
#' treated as the response.
#'
#'
#' @name Dyke
#' @docType data
#' @format A 5-dimensional array resulting from cross-tabulating 5 variables
#' for 1729 observations. The variable names and their levels are:
#'
#' \tabular{rll}{
#'  dim \tab Name \tab Levels \cr
#'   1\tab `Knowledge`\tab `"Good", "Poor"`\cr
#'   2\tab `Reading`\tab `"No", "Yes"`\cr
#'   3\tab `Radio`\tab `"No", "Yes"`\cr
#'   4\tab `Lectures`\tab `"No", "Yes"`\cr
#'   5\tab `Newspaper`\tab `"No", "Yes"`\cr
#' }

#' @references
#' Dyke, G. V. and Patterson, H. D. (1952). Analysis of factorial
#' arrangements when the data are proportions. *Biometrics*, 8, 1-12.
#'
#' Lindsey, J. K. (1993).  *Models for Repeated Measurements* Oxford, UK:
#' Oxford University Press, p. 57.
#'
#' @source Fienberg, S. E. (1980). *The Analysis of Cross-Classified
#' Categorical Data* Cambridge, MA: MIT Press, p. 85, Table 5-6.
#' @keywords datasets
#' @examples
#'
#' data(Dyke)
#'
#' # independence model
#' mosaic(Dyke, shade=TRUE)
#'
#' # null model, Knowledge as response, independent of others
#' require(MASS)
#' dyke.mod0 <- loglm(~ Knowledge + (Reading * Radio * Lectures * Newspaper), data=Dyke)
#' dyke.mod0
#' mosaic(dyke.mod0)
#'
#' # view as doubledecker plot
#' Dyke <- Dyke[2:1,,,,]    # make Good the highlighted value of Knowledge
#' doubledecker(Knowledge ~ ., data=Dyke)
#'
#' # better version, with some options
#' doubledecker(Knowledge ~ Lectures + Reading + Newspaper + Radio,
#'   data=Dyke,
#' 	margins = c(1,6, length(dim(Dyke)) + 1, 1),
#' 	fill_boxes=list(rep(c("white", gray(.90)),4))
#' 	)
#'
#' # separate (conditional) plots for those who attend lectures and those who do not
#' doubledecker(Knowledge ~ Reading + Newspaper + Radio,
#'   data=Dyke[,,,1,],
#' 	main="Do not attend lectures",
#' 	margins = c(1,6, length(dim(Dyke)) + 1, 1),
#' 	fill_boxes=list(rep(c("white", gray(.90)),3))
#' 	)
#' doubledecker(Knowledge ~ Reading + Newspaper + Radio,
#'   data=Dyke[,,,2,],
#' 	main="Attend lectures",
#' 	margins = c(1,6, length(dim(Dyke)) + 1, 1),
#' 	fill_boxes=list(rep(c("white", gray(.90)),3))
#' 	)
#'
#'
#' drop1(dyke.mod0, test="Chisq")
#'
#'
NULL





#' Carcinogenic Effects of a Fungicide
#'
#' Data from Gart (1971) on the carcinogenic effects of a certain fungicide in
#' two strains of mice. Of interest is how the association between `group`
#' (Control, Treated) and `outcome` (Tumor, No Tumor) varies with
#' `sex` and `strain` of the mice.
#'
#' Breslow (1976) used this data to illustrate the application of linear models
#' to log odds ratios.
#'
#' All tables have some small cells, so a continuity correction is recommended.
#'
#' @name Fungicide
#' @docType data
#' @format The data comprise a set of four 2 x 2 tables classifying 403 mice,
#' either Control or Treated and whether or not a tumor was later observed.
#' The four groups represent the combinations of sex and strain of mice.
#'
#' The format is:
#' \preformatted{
#' num [1:2, 1:2, 1:2, 1:2] 5 4 74 12 3 2 84 14 10 4 ...
#' - attr(*, "dimnames")=List of 4
#' ..$ group  : chr [1:2] "Control" "Treated"
#' ..$ outcome: chr [1:2] "Tumor" "NoTumor"
#' ..$ sex    : chr [1:2] "M" "F"
#' ..$ strain : chr [1:2] "1" "2"
#' }
#
#' @references Breslow, N. (1976), Regression analysis of the log odds ratio: A
#' method for retrospective studies, *Biometrics*, 32(3), 409-416.
#'
#' @source Gart, J. J. (1971). The comparison of proportions: a review of
#' significance tests, confidence intervals and adjustments for stratification.
#' *International Statistical Review*, 39, 148-169.
#'
#' @keywords datasets
#' @examples
#'
#' data(Fungicide)
#' # loddsratio was moved to vcd; requires vcd_1.3-3+
#' fung.lor <- loddsratio(Fungicide, correct=TRUE)
#' fung.lor
#' confint(fung.lor)
#'
#' # visualize odds ratios in fourfold plots
#' cotabplot(Fungicide, panel=cotab_fourfold)
#' #  -- fourfold() requires vcd >= 1.2-10
#' fourfold(Fungicide, p_adjust_method="none")
#'
#'
#'
NULL





#' Geissler's Data on the Human Sex Ratio
#'
#' Geissler (1889) published data on the distributions of boys and girls in
#' families in Saxony, collected for the period 1876-1885. The `Geissler`
#' data tabulates the family composition of 991,958 families by the number of
#' boys and girls listed in the table supplied by Edwards (1958, Table 1).
#'
#' The data on family composition was available because, on the birth of a
#' child, the parents had to state the sex of all their children on the birth
#' certificate. These family records are not necessarily independent, because a
#' given family may have had several children during this 10 year period,
#' included as multiple records.
#'
#' @name Geissler
#' @docType data
#' @format A data frame with 90 observations on the following 4 variables.  The
#' rows represent the non-NA entries in Edwards' table.
#' \describe{
#'   \item{`boys`}{number of boys in the family, `0:12`}
#'   \item{`girls`}{number of girls in the family, `0:12`}
#'   \item{`size`}{family size: `boys+girls`}
#'   \item{`Freq`}{number of families with this sex composition}
#' }

#' @seealso \code{\link[vcd]{Saxony}}, containing the data for families of size
#' 12.
#' @references
#' Friendly, M. and Meyer, D. (2016).  *Discrete Data Analysis
#' with R: Visualization and Modeling Techniques for Categorical and Count
#' Data*.  Boca Raton, FL: Chapman & Hall/CRC. <http://ddar.datavis.ca>.
#'
#' Geissler, A. (1889). *Beitrage zur Frage des Geschlechts verhaltnisses
#' der Geborenen* Z. K. Sachsischen Statistischen Bureaus, 35, n.p.
#'
#' Lindsey, J. K. & Altham, P. M. E. (1998).  Analysis of the human sex ratio
#' by using overdispersion models. *Journal of the Royal Statistical
#' Society: Series C (Applied Statistics)*, 47, 149-157.
#'
#' @source
#' Edwards, A. W. F. (1958). An Analysis Of Geissler's Data On The
#' Human Sex Ratio. *Annals of Human Genetics*, 23, 6-15.
#' @keywords datasets
#' @examples
#'
#' data(Geissler)
#' str(Geissler)
#'
#' # reproduce Saxony data, families of size 12
#' Saxony12 <- subset(Geissler, size==12, select=c(boys, Freq))
#' rownames(Saxony12)<-NULL
#'
#' # make a 1-way table
#' xtabs(Freq~boys, Saxony12)
#'
#' # extract data for other family sizes
#' Saxony11 <- subset(Geissler, size==11, select=c(boys, Freq))
#' rownames(Saxony11)<-NULL
#'
#' Saxony10 <- subset(Geissler, size==10, select=c(boys, Freq))
#' rownames(Saxony10)<-NULL
#'
#'
NULL





#' Clothing and Intelligence Rating of Children
#'
#' Schoolboys were classified according to their clothing and to their teachers
#' rating of "dullness" (lack of intelligence), in a 5 x 7 table originally
#' from Gilby (1911). Anscombe (1981) presents a slightly collapsed 4 x 6
#' table, used here, where the last two categories of clothing were pooled as
#' were the first two categories of dullness due to small counts.
#'
#' Both `Dullness` and `Clothing` are ordered categories, so models
#' and methods that examine their association in terms of ordinal categories
#' are profitable.
#'
#'
#' @name Gilby
#' @docType data
#' @format A 2-dimensional array resulting from cross-tabulating 2 variables
#' for 1725 observations. The variable names and their levels are:
#'
#' \tabular{rll}{
#'   No \tab Name \tab Levels \cr
#'   1\tab `Dullness`\tab `"Ment. defective", "Slow", "Slow Intell", "Fairly Intell", "Capable", "V.Able"`\cr
#'   2\tab `Clothing`\tab `"V.Well clad", "Well clad", "Passable", "Insufficient"`\cr
#' }

#' @references
#' Gilby, W. H. (1911).
#' On the significance of the teacher's appreciation of general
#' intelligence.  *Biometrika*, 8, 93-108 (esp. p. 94).  (Quoted by Kendall (1943,..., 1953) Table 13.1, p 320.)
#'
#' @source Anscombe, F. J. (1981). *Computing in Statistical Science Through APL*. New York: Springer-Verlag, p. 302
#' @keywords datasets
#' @examples
#'
#' data(Gilby)
#'
#' # CMH tests treating row/column variables as ordinal
#' CMHtest(Gilby)
#'
#' mosaic(Gilby, shade=TRUE)
#'
#' # correspondence analysis to see relations among categories
#' if(require(ca)){
#' 	ca(Gilby)
#' 	plot(ca(Gilby), lines=TRUE)
#'
#' }
#'
#'
#'
NULL





#' British Social Mobility from Glass(1954)
#'
#' Glass(1954) gave this 5 x 5 table on the occupations of 3500 British fathers
#' and their sons.
#'
#' The occupational categories in order of status are: (1) Professional & High
#' Administrative (2) Managerial, Executive & High Supervisory (3) Low
#' Inspectional & Supervisory (4) Routine Nonmanual & Skilled Manual (5) Semi-
#' & Unskilled Manual
#'
#' However, to make the point that factors are ordered alphabetically by
#' default, Friendly & Meyer (2016) introduce this data set in the form given
#' here.
#'
#' @name Glass
#' @docType data
#' @format A frequency data frame with 25 observations on the following 3
#' variables representing a 5 x 5 table with 3500 cases.
#' \describe{
#'   \item{`father`}{a factor with levels `Managerial` `Professional` `Skilled` `Supervisory` `Unskilled`}
#'   \item{`son`}{a factor with levels `Managerial` `Professional` `Skilled` `Supervisory` `Unskilled`}
#'   \item{`Freq`}{a numeric vector}
#' }
#'
#' @references
#' Bishop, Y. M. M. and Fienberg, S. E. and Holland, P. W. (1975).
#' *Discrete Multivariate Analysis: Theory and Practice*, MIT Press.
#'
#' Friendly, M. and Meyer, D. (2016).  *Discrete Data Analysis with R:
#' Visualization and Modeling Techniques for Categorical and Count Data*.  Boca
#' Raton, FL: Chapman & Hall/CRC. <http://ddar.datavis.ca>.
#'
#' @source Glass, D. V. (1954), *Social Mobility in Britain*. The Free
#' Press.
#' @keywords datasets
#' @examples
#'
#' data(Glass)
#' glass.tab <- xtabs(Freq ~ father + son, data=Glass)
#'
#' largs <- list(set_varnames=list(father="Father's Occupation",
#'                                 son="Son's Occupation"),
#'               abbreviate=10)
#' gargs <- list(interpolate=c(1,2,4,8))
#'
#' mosaic(glass.tab,
#'   shade=TRUE,
#'   labeling_args=largs,
#'   gp_args=gargs,
#'   main="Alphabetic order",
#'   legend=FALSE,
#'   rot_labels=c(20,90,0,70))
#'
#' # reorder by status
#' ord <- c(2, 1, 4, 3, 5)
#' mosaic(glass.tab[ord, ord],
#'   shade=TRUE,
#'   labeling_args=largs,
#'   gp_args=gargs,
#'   main="Effect order",
#'   legend=FALSE,
#'   rot_labels=c(20,90,0,70))
#'
#'
NULL





#' General Social Survey-- Sex and Party affiliation
#'
#' Data from the General Social Survey, 1991, on the relation between sex and
#' party affiliation.
#'
#'
#' @name GSS
#' @docType data
#' @format A data frame in frequency form with 6 observations on the following
#' 3 variables.
#'
#' \describe{
#'   \item{`sex`}{a factor with levels `female` `male`}
#'   \item{`party`}{a factor with levels `dem` `indep` `rep`}
#'   \item{`count`}{a numeric vector}
#' }

#' @source Agresti, A. *Categorical Data Analysis*, 2nd E., John Wiley &
#' Sons, 2002, Table 3.11, p. 106.
#' @keywords datasets
#' @examples
#'
#' data(GSS)
#' str(GSS)
#'
#' # use xtabs to show the table in a compact form
#' (GSStab <- xtabs(count ~ sex + party, data=GSS))
#'
#' # fit the independence model
#' (mod.glm <- glm(count ~ sex + party, family = poisson, data = GSS))
#'
#' # display all the residuals in a mosaic plot
#' mosaic(mod.glm,
#'   formula = ~ sex + party,
#'   labeling = labeling_residuals,
#'   suppress=0)
#'
NULL





#' Hair Color and Eye Color in Caithness and Aberdeen
#'
#' A three-way frequency table crossing eye color and hair color in two places,
#' Caithness and Aberdeen, Scotland. These data were of interest to Fisher
#' (1940) and others because there are mixtures of people of Nordic, Celtic and
#' Anglo-Saxon origin.
#'
#' One or both tables have been widely analyzed in conjunction with RC and
#' canonical correlation models for categorical data, e.g., Becker and Clogg
#' (1989).
#'
#' @details
#' The hair and eye colors are ordered as in the original source, suggesting
#' that they form ordered categories.
#'
#' @name HairEyePlace
#' @docType data
#' @format
#' The format is:
#' \preformatted{
#'   num [1:4, 1:5, 1:2] 326 688 343 98 38 116 84 48 241 584 ...
#' - attr(*, "dimnames")=List of 3
#' ..$ Eye  : chr [1:4] "Blue" "Light" "Medium" "Dark"
#' ..$ Hair : chr [1:5] "Fair" "Red" "Medium" "Dark" ...
#' ..$ Place: chr [1:2] "Caithness" "Aberdeen"
#' }
#'
#' @references Becker, M. P., and Clogg, C. C. (1989).  Analysis of Sets of
#' Two-Way Contingency Tables Using Association Models.  *Journal of the
#' American Statistical Association*, 84(405), 142-151.
#'
#' Fisher, R.A. (1940) The precision of discriminant functions.  *Annals
#' of Eugenics*, 10, 422-429.
#' @source This data was taken from the `colors` data in \pkg{logmult}.
#' @keywords datasets
#' @examples
#'
#' data(HairEyePlace)
#'
#' # separate mosaics
#' mosaic(HairEyePlace[,,1], shade=TRUE, main="Caithness")
#' mosaic(HairEyePlace[,,2], shade=TRUE, main="Aberdeen")
#'
#' # condition on Place
#' mosaic(~Hair + Eye |Place, data=HairEyePlace, shade=TRUE, legend=FALSE)
#'
#' cotabplot(~Hair+Eye|Place, data=HairEyePlace, shade=TRUE, legend=FALSE)
#'
NULL





#' Hauser (1979) Data on Social Mobility
#'
#' Hauser (1979) presented this two-way frequency table, cross-classifying
#' occupational categories of sons and fathers in the United States.
#'
#' It is a good example for exploring a variety of models for square tables:
#' quasi-independence, quasi-symmetry, row/column effects, uniform association,
#' etc., using the facilities of the \pkg{gnm}.
#'
#' Hauser's data was first presented in 1979, and then published in 1980. The
#' name of the dataset reflects the earliest use.
#'
#' It reflects the "frequencies in a classification of son's first full-time
#' civilian occupation by father's (or other family head's) occupation at son's
#' sixteenth birthday among American men who were aged 20 to 64 in 1973 and
#' were not currently enrolled in school".
#'
#' As noted in Hauser's Table 1, "Counts are based on observations weighted to
#' estimate population counts and compensate for departures of the sampling
#' design from simple random sampling. Broad occupation groups are upper
#' nonmanual: professional and kindred workers, managers and officials, and
#' non-retail sales workers; lower nonmanual: proprietors, clerical and kindred
#' workers, and retail sales workers; upper manual: craftsmen, foremen, and
#' kindred workers; lower manual: service workers, operatives and kindred
#' workers, and laborers (except farm); farm: farmers and farm managers, farm
#' laborers, and foremen. density of mobility or immobility in the cells to
#' which they refer."
#'
#' The table levels for `Son` and `Father` have been arranged in
#' order of decreasing status as is common for mobility tables.
#'
#' @name Hauser79
#' @docType data
#' @format A frequency data frame with 25 observations on the following 3
#' variables, representing the cross-classification of 19912 individuals by
#' father's occupation and son's first occupation.
#' \describe{
#'   \item{`Son`}{a factor with levels `UpNM` `LoNM` `UpM` `LoM` `Farm`}
#'   \item{`Father`}{a factor with levels `UpNM` `LoNM` `UpM` `LoM` `Farm`}
#'   \item{`Freq`}{a numeric vector}
#' }
#'
#' @references
#' Powers, D.A. and Xie, Y. (2008). *Statistical Methods for
#' Categorical Data Analysis*, Bingley, UK: Emerald.
#' @source
#' R.M. Hauser (1979), Some exploratory methods for modeling mobility
#' tables and other cross-classified data.  In: K.F. Schuessler (Ed.),
#' *Sociological Methodology*, 1980, Jossey-Bass, San Francisco, pp.
#' 413-458. Table 1.
#' @keywords datasets
#' @examples
#'
#' data(Hauser79)
#' str(Hauser79)
#'
#' # display table
#' structable(~Father+Son, data=Hauser79)
#'
#' #Examples from Powers & Xie, Table 4.15
#' # independence model
#' mosaic(Freq ~ Father + Son, data=Hauser79, shade=TRUE)
#'
#' hauser.indep <- gnm(Freq ~ Father + Son,
#'   data=Hauser79,
#'   family=poisson)
#'
#' mosaic(hauser.indep, ~Father+Son,
#'        main="Independence model",
#'        gp=shading_Friendly)
#'
#' # Quasi-independence
#' hauser.quasi <-  update(hauser.indep,
#'                         ~ . + Diag(Father,Son))
#' mosaic(hauser.quasi, ~Father+Son,
#'        main="Quasi-independence model",
#'        gp=shading_Friendly)
#'
#' # Quasi-symmetry
#' hauser.qsymm <-  update(hauser.indep,
#'                         ~ . + Diag(Father,Son) + Symm(Father,Son))
#'
#' mosaic(hauser.qsymm, ~Father+Son,
#'        main="Quasi-symmetry model",
#'        gp=shading_Friendly)
#'
#'
#' # numeric scores for row/column effects
#' Sscore <- as.numeric(Hauser79$Son)
#' Fscore <- as.numeric(Hauser79$Father)
#'
#' # row effects model
#' hauser.roweff <- update(hauser.indep, ~ . + Father*Sscore)
#' LRstats(hauser.roweff)
#'
#' # uniform association
#' hauser.UA <- update(hauser.indep, ~ . + Fscore*Sscore)
#' LRstats(hauser.UA)
#'
#' # uniform association, omitting diagonals
#' hauser.UAdiag <- update(hauser.indep, ~ . + Fscore*Sscore + Diag(Father,Son))
#' LRstats(hauser.UAdiag)
#'
#' # Levels for Hauser 5-level model
#' levels <- matrix(c(
#'   2,  4,  5,  5,  5,
#'   3,  4,  5,  5,  5,
#'   5,  5,  5,  5,  5,
#'   5,  5,  5,  4,  4,
#'   5,  5,  5,  4,  1
#'   ), 5, 5, byrow=TRUE)
#'
#' hauser.topo <- update(hauser.indep,
#'                       ~ . + Topo(Father, Son, spec=levels))
#'
#' mosaic(hauser.topo, ~Father+Son,
#'        main="Topological model", gp=shading_Friendly)
#'
#' # RC model
#' hauser.RC <- update(hauser.indep, ~ . + Mult(Father, Son), verbose=FALSE)
#' mosaic(hauser.RC, ~Father+Son, main="RC model", gp=shading_Friendly)
#' LRstats(hauser.RC)
#'
#' # crossings models
#' hauser.CR <- update(hauser.indep, ~ . + Crossings(Father,Son))
#' mosaic(hauser.topo, ~Father+Son, main="Crossings model", gp=shading_Friendly)
#' LRstats(hauser.CR)
#'
#' hauser.CRdiag <- update(hauser.indep, ~ . + Crossings(Father,Son) + Diag(Father,Son))
#' LRstats(hauser.CRdiag)
#'
#'
#' # compare model fit statistics
#' modlist <- glmlist(hauser.indep, hauser.roweff, hauser.UA, hauser.UAdiag,
#'                    hauser.quasi, hauser.qsymm,  hauser.topo,
#'                    hauser.RC, hauser.CR, hauser.CRdiag)
#' sumry <- LRstats(modlist)
#' sumry[order(sumry$AIC, decreasing=TRUE),]
#' # or, more simply
#' LRstats(modlist, sortby="AIC")
#'
#' mods <- substring(rownames(sumry),8)
#' with(sumry,
#' 	{plot(Df, AIC, cex=1.3, pch=19, xlab='Degrees of freedom', ylab='AIC')
#' 	text(Df, AIC, mods, adj=c(0.5,-.5), col='red', xpd=TRUE)
#' 	})
#'
#'
#'
NULL





#' Sex, Occupation and Heart Disease
#'
#' Classification of individuals by gender, occupational category and
#' occurrence of heart disease
#'
#'
#' @name Heart
#' @docType data
#' @format A 3-dimensional array resulting from cross-tabulating 3 variables
#' for 21522 observations. The variable names and their levels are:
#'
#' \tabular{rll}{
#'   No \tab Name \tab Levels \cr
#'   1\tab `Disease`\tab `"Disease", "None"`\cr
#'   2\tab `Gender`\tab `"Male", "Female"`\cr
#'   3\tab `Occup`\tab `"Unempl", "WhiteCol", "BlueCol"`\cr
#' }

#' @source
#'
#' % \cite{Karger, 1980}
#' Karger, (1980).
#' @keywords datasets
#' @examples
#'
#' data(Heart)
#' str(Heart)
#'
#' # Display the frequencies for occupational categories.
#' # Each row is a 2 x 2 table
#' vcd::structable(Disease + Gender ~ Occup, data=Heart)
#'
#' # display as fourfold plots
#' vcd::cotabplot(~ Disease + Gender | Occup, data=Heart, panel = cotab_fourfold)
#'
NULL





#' Labour Force Participation of Married Women 1967-1971
#'
#' 1583 married women were surveyed over the years 1967-1971, recording whether
#' or not they were employed in the labor force.
#'
#' The data, originally from Heckman & Willis (1977) provide an example of
#' modeling longitudinal categorical data, e.g., with markov chain models for
#' dependence over time.
#'
#' Lindsey (1993) fits an initial set of logistic regression models examining
#' the dependence of employment in 1971 (`e1971`) on successive subsets of
#' the previous years, `e1970`, `e1969`, \dots{} `e1967`.
#'
#' Alternatively, one can examine markov chain models of first-order
#' (dependence on previous year), second-order (dependence on previous two
#' years), etc.
#'
#' @name Heckman
#' @docType data
#' @format A 5-dimensional \eqn{2^5} array resulting from cross-tabulating 5
#' binary variables for 1583 observations. The variable names and their levels
#' are:
#'
#' \tabular{rll}{
#'   No \tab Name \tab Levels \cr
#'   1\tab `e1971`\tab `"71Yes", "No"`\cr
#'   2\tab `e1970`\tab `"70Yes", "No"`\cr
#'   3\tab `e1969`\tab `"69Yes", "No"`\cr
#'   4\tab `e1968`\tab `"68Yes", "No"`\cr
#'   5\tab `e1967`\tab `"67Yes", "No"`\cr
#' }

#' @references
#' % \cite{HeckmanWillis:77}
#' Heckman, J.J. & Willis, R.J. (1977).
#' "A beta-logistic model for the analysis of sequential labor force
#' participation by married women."  *Journal of Political Economy*, 85:
#' 27-58
#' @source
#'
#' Lindsey, J. K. (1993).  *Models for
#' Repeated Measurements* Oxford, UK: Oxford University Press, p. 185.
#' @keywords datasets
#' @examples
#'
#' data(Heckman)
#'
#' # independence model
#' mosaic(Heckman, shade=TRUE)
#' # same, as a loglm()
#' require(MASS)
#' (heckman.mod0 <- loglm(~ e1971+e1970+e1969+e1968+e1967, data=Heckman))
#' mosaic(heckman.mod0, main="Independence model")
#'
#' # first-order markov chain: bad fit
#' (heckman.mod1 <- loglm(~ e1971*e1970 + e1970*e1969 +e1969*e1968 + e1968*e1967, data=Heckman))
#' mosaic(heckman.mod1, main="1st order markov chain model")
#'
#' # second-order markov chain: bad fit
#' (heckman.mod2 <- loglm(~ e1971*e1970*e1969 + e1970*e1969*e1968 +e1969*e1968*e1967, data=Heckman))
#' mosaic(heckman.mod2, main="2nd order markov chain model")
#'
#' # third-order markov chain: fits OK
#' (heckman.mod3 <- loglm(~ e1971*e1970*e1969*e1968 + e1970*e1969*e1968*e1967, data=Heckman))
#' mosaic(heckman.mod2, main="3rd order markov chain model")
#'
#'
NULL





#' Hospital Visits Data
#'
#' Length of stay in hospital for 132 schizophrenic patients, classified by
#' visiting patterns, originally from Wing (1962).
#'
#' Both table variables can be considered ordinal. The variable `visit`
#' refers to visiting patterns recorded hospital.  The category labels are
#' abbreviations of those given by Goodman (1983); e.g., `"Regular"` is
#' short for \dQuote{received visitors regularly or patient went home}. The
#' variable `stay` refers to length of stay in hospital, in year groups.
#'
#' @name HospVisits
#' @docType data
#' @format A 3 by 3 frequency table, with format:
#' \preformatted{
#' table [1:3, 1:3] 43 6 9 16 11 18 3 10 16
#' - attr(*, "dimnames")=List of 2
#' ..$ visit: chr [1:3] "Regular" "Infrequent" "Never"
#' ..$ stay : chr [1:3] "2-9" "10-19" "20+"
#' }
#
#' @seealso \code{\link[ca]{ca}}
#' @references
#' Wing, J. K. (1962). Institutionalism in Mental Hospitals,
#' *British Journal of Social and Clinical Psychology*, 1 (1), 38-51.
#'
#' @source
#' Goodman, L. A. (1983) The analysis of dependence in
#' cross-classifications having ordered categories, using log-linear models for
#' frequencies and log-linear models for odds.  *Biometrics*, 39, 149-160.
#'
#' @keywords datasets
#' @examples
#'
#' data(HospVisits)
#' mosaic(HospVisits, gp=shading_Friendly)
#'
#' if(require(ca)){
#'   ca(HospVisits)
#'   # surprisingly 1D !
#'   plot(ca(HospVisits))
#'   }
#'
NULL





#' Household Tasks Performed by Husbands and Wives
#'
#' A 13 x 4 table of frequencies of household tasks performed by couples,
#' either by the `Husband`, `Wife`, `Alternating` or `Jointly`.
#'
#'
#' @name HouseTasks
#' @docType data
#' @format The format is:
#' \preformatted{
#'   'table' int [1:13, 1:4] 36 11 24 51 13 1 1 14 20 46 ...
#' - attr(*, "dimnames")=List of 2
#' ..$ Task: chr [1:13] "Breakfast" "Dinner" "Dishes" "Driving" ...
#' ..$ Who : chr [1:4] "Alternating" "Husband" "Jointly" "Wife"
#' }
#'
#' @source This data set was taken from \code{\link[factoextra]{housetasks}}, a
#' 13 x 4 data.frame. In this table version, the rows and columns were sorted
#' alphabetically (and a typo was corrected).
#' @keywords datasets
#' @examples
#'
#' data(HouseTasks)
#' str(HouseTasks)
#'
#' chisq.test(HouseTasks)
#'
#' # mosaic plot, illustrating some tweaks to handle overlapping labels
#' require(vcd)
#' mosaic(HouseTasks, shade = TRUE,
#'        labeling = labeling_border(rot_labels = c(45,0, 0, 0),
#'                                   offset_label =c(.5,5,0, 0),
#'                                   varnames = c(FALSE, TRUE),
#'                                   just_labels=c("center","right"),
#'                                   tl_varnames = FALSE),
#'        legend = FALSE)
#'
#' # use seriation package to permute rows & cols using correspondence analysis
#' if(require(seriation)) {
#' order <- seriate(HouseTasks, method = "CA")
#' # the permuted row and column labels
#' rownames(HouseTasks)[order[[1]]]
#' colnames(HouseTasks)[order[[2]]]
#'
#' # do the permutation
#' HT_perm <- permute(HouseTasks, order, margin=1)
#'
#' mosaic(HT_perm, shade = TRUE,
#'        labeling = labeling_border(rot_labels = c(45,0, 0, 0),
#'                                   offset_label =c(.5,5,0, 0),
#'                                   varnames = c(FALSE, TRUE),
#'                                   just_labels=c("center","right"),
#'                                   tl_varnames = FALSE),
#'        legend = FALSE)
#' }
#'
NULL





#' Minnesota High School Graduates
#'
#' Minnesota high school graduates of June 1930 were classified with respect to
#' (a) `Rank` by thirds in their graduating class, (b) post-high school
#' `Status` in April 1939 (4 levels), (c) `Sex`, (d) father's
#' `Occupation`al status (7 levels, from 1=High to 7=Low).
#'
#' The data were first presented by Hoyt et al. (1959) and have been analyzed
#' by Fienberg(1980), Plackett(1974) and others.
#'
#' Post high-school `Status` is natural to consider as the response.
#' `Rank` and father's `Occupation` are ordinal variables.
#'
#' @name Hoyt
#' @docType data
#' @format A 4-dimensional array resulting from cross-tabulating 4 variables
#' for 13968 observations. The variable names and their levels are:
#'
#' \tabular{rll}{
#'   No \tab Name \tab Levels \cr
#'   1\tab `Status`\tab `"College", "School", "Job", "Other"`\cr
#'   2\tab `Rank`\tab `"Low", "Middle", "High"`\cr
#'   3\tab `Occupation`\tab `"1", "2", "3", "4", "5", "6", "7"`\cr
#'   4\tab `Sex`\tab `"Male", "Female"`\cr
#' }
#
#' @seealso \code{\link[MASS]{minn38}} provides the same data as a data frame.
#' @references
#' Hoyt, C. J., Krishnaiah, P. R. and Torrance, E. P. (1959)
#' Analysis of complex contingency tables, *Journal of Experimental
#' Education* 27, 187-194.
#' @source
#'
#' Fienberg, S. E. (1980). *The Analysis of Cross-Classified Categorical
#' Data*. Cambridge, MA: MIT Press, p. 91-92.
#'
#' R. L. Plackett, (1974). *The Analysis of Categorical Data*. London: Griffin.
#' @keywords datasets
#' @examples
#'
#' data(Hoyt)
#'
#' # display the table
#' structable(Status + Sex ~ Rank + Occupation, data=Hoyt)
#'
#' # mosaic for independence model
#' plot(Hoyt, shade=TRUE)
#'
#' # examine all pairwise mosaics
#' pairs(Hoyt, shade=TRUE)
#'
#' # collapse Status to College vs. Non-College
#' Hoyt1 <- collapse.table(Hoyt, Status=c("College", rep("Non-College",3)))
#' plot(Hoyt1, shade=TRUE)
#'
#' #################################################
#' # fitting models with loglm, plotting with mosaic
#' #################################################
#'
#' # fit baseline log-linear model for Status as response
#' require(MASS)
#' hoyt.mod0 <- loglm(~ Status + (Sex*Rank*Occupation),
#'   data=Hoyt1)
#' hoyt.mod0
#'
#' mosaic(hoyt.mod0,
#'   gp=shading_Friendly,
#'   main="Baseline model: Status + (Sex*Rank*Occ)")
#'
#' # add one-way association of Status with factors
#' hoyt.mod1 <- loglm(~ Status * (Sex + Rank + Occupation) + (Sex*Rank*Occupation),
#'   data=Hoyt1)
#' hoyt.mod1
#'
#' mosaic(hoyt.mod1,
#'   gp=shading_Friendly,
#'   main="Status * (Sex + Rank + Occ)")
#'
#' # can we drop any terms?
#' drop1(hoyt.mod1, test="Chisq")
#'
#' # assess model fit
#' anova(hoyt.mod0, hoyt.mod1)
#'
#' # what terms to add?
#' add1(hoyt.mod1, ~.^2, test="Chisq")
#'
#' # add interaction of Sex:Occupation on Status
#' hoyt.mod2 <- update(hoyt.mod1, ~ . + Status:Sex:Occupation)
#'
#' mosaic(hoyt.mod2,
#'   gp=shading_Friendly,
#'   main="Adding Status:Sex:Occupation")
#'
#' # compare model fits
#' anova(hoyt.mod0, hoyt.mod1, hoyt.mod2)
#'
#' # Alternatively, try stepwise analysis, heading toward the saturated model
#' steps <- step(hoyt.mod0,
#'   direction="forward",
#'   scope=~Status*Sex*Rank*Occupation)
#'
#' # display anova
#' steps$anova
#'
#'
NULL





#' ICU data set
#'
#' The ICU data set consists of a sample of 200 subjects who were part of a
#' much larger study on survival of patients following admission to an adult
#' intensive care unit (ICU), derived from Hosmer, Lemeshow and Sturdivant
#' (2013) and Friendly (2000).
#'
#' The major goal of this study was to develop a logistic regression model to
#' predict the probability of survival to hospital discharge of these patients
#' and to study the risk factors associated with ICU mortality. The clinical
#' details of the study are described in Lemeshow, Teres, Avrunin, and Pastides
#' (1988).
#'
#' This data set is often used to illustrate model selection methods for
#' logistic regression.
#'
#' Patient ID numbers are the rownames of the data frame.
#'
#' Note that the last two variables `white` and `uncons` are a
#' recoding of respectively `race` and `coma` to binary variables.
#'
#' @name ICU
#' @docType data
#' @format A data frame with 200 observations on the following 22 variables.
#' \describe{
#'   \item{`died`}{Died before discharge?, a factor with levels `No` `Yes`}
#'   \item{`age`}{Patient age, a numeric vector}
#'   \item{`sex`}{Patient sex, a factor with levels `Female` `Male`}
#'   \item{`race`}{Patient race, a factor with levels `Black` `Other` `White`.  Also represented here as `white`.}
#'   \item{`service`}{Service at ICU Admission, a factor with levels `Medical` `Surgical`}
#'   \item{`cancer`}{Cancer part of present problem?, a factor with levels `No` `Yes`}
#'   \item{`renal`}{History of chronic renal failure?, a factor with levels `No` `Yes`}
#'   \item{`infect`}{Infection probable at ICU admission?, a factor with levels `No` `Yes`}
#'   \item{`cpr`}{Patient received CPR prior to ICU admission?, a factor with levels `No` `Yes`}
#'   \item{`systolic`}{Systolic blood pressure at admission (mm Hg), a numeric vector}
#'   \item{`hrtrate`}{Heart rate at ICU Admission (beats/min), a numeric vector}
#'   \item{`previcu`}{Previous admission to an ICU within 6 Months?, a factor with levels `No` `Yes`}
#'   \item{`admit`}{Type of admission, a factor with levels `Elective` `Emergency`}
#'   \item{`fracture`}{Admission with a long bone, multiple, neck, single area, or hip fracture?
#'         a factor with levels `No` `Yes`}
#'   \item{`po2`}{PO2 from initial blood gases, a factor with levels `>60` `<=60`}
#'   \item{`ph`}{pH from initial blood gases, a factor with levels `>=7.25` `<7.25`}
#'   \item{`pco`}{PCO2 from initial blood gases, a factor with levels `<=45` `>45`}
#'   \item{`bic`}{Bicarbonate (HCO3) level from initial blood gases, a factor with levels `>=18` `<18`}
#'   \item{`creatin`}{Creatinine, from initial blood gases, a factor with levels `<=2` `>2`}
#'   \item{`coma`}{Level of unconsciousness at admission to ICU,	a factor with levels `None` `Stupor` `Coma`}
#'   \item{`white`}{a recoding of `race`,  a factor with levels `White` `Non-white`}
#'   \item{`uncons`}{a recoding of `coma` a factor with levels `No` `Yes`}
#' }
#'
#' @references
#'
#' Lemeshow, S., Teres, D., Avrunin, J. S., Pastides, H. (1988). Predicting the
#' Outcome of Intensive Care Unit Patients. *Journal of the American
#' Statistical Association*, 83, 348-356.
#' @source M. Friendly (2000), *Visualizing Categorical Data*, Appendix
#' B.4. SAS Institute, Cary, NC.
#'
#' Hosmer, D. W. Jr., Lemeshow, S. and Sturdivant, R. X. (2013) *Applied
#' Logistic Regression*, NY: Wiley, Third Edition.
#' @keywords datasets
#' @examples
#'
#' data(ICU)
#' # remove redundant variables (race, coma)
#' ICU1 <- ICU[,-c(4,20)]
#'
#' # fit full model
#' icu.full <- glm(died ~ ., data=ICU1, family=binomial)
#' summary(icu.full)
#'
#' # simpler model (found from a "best" subsets procedure)
#' icu.mod1 <- glm(died ~ age + sex + cancer + systolic + admit + uncons,
#'   data=ICU1,
#'   family=binomial)
#' summary(icu.mod1)
#'
#' # even simpler model
#' icu.mod2 <- glm(died ~ age + cancer  + admit + uncons,
#'   data=ICU1,
#'   family=binomial)
#' summary(icu.mod2)
#'
#' anova(icu.mod2, icu.mod1, icu.full, test="Chisq")
#'
#' ## Reproduce Fig 6.12 from VCD
#'
#' icu.fit <- data.frame(ICU, prob=predict(icu.mod2, type="response"))
#'
#' # combine categorical risk factors to a single string
#' risks <- ICU[, c("cancer", "admit", "uncons")]
#' risks[,1] <- ifelse(risks[,1]=="Yes", "Cancer", "")
#' risks[,2] <- ifelse(risks[,2]=="Emergency", "Emerg", "")
#' risks[,3] <- ifelse(risks[,3]=="Yes", "Uncons", "")
#' risks <- apply(risks, 1, paste, collapse="")
#' risks[risks==""] <- "(none)"
#' icu.fit$risks <- risks
#'
#' if (require("ggplot2")) {
#' print(ggplot(icu.fit, aes(x=age, y=prob, color=risks)) +
#' 	geom_point(size=2) +
#' 	geom_line(size=1.25, alpha=0.5) +
#' 	theme_bw() + ylab("Probability of death"))
#' }
#'
#'
NULL





#' Cross-classification of job satisfaction by income
#'
#' This data set is a contingency table of job satisfaction by income for a
#' small sample of black males from the 1996 General Social Survey, as used by
#' Agresti (2002) for an example.
#'
#' Both `income` and `satisfaction` are ordinal variables, and are so
#' ordered in the table.  Measures of association, visualizations, and models
#' should take ordinality into account.
#'
#' @name JobSat
#' @docType data
#' @format A 4 x 4 contingency table of `income` by `satisfaction`,
#' with the following structure:
#' \preformatted{
#'   table [1:4, 1:4] 1 2 1 0 3 3 6 1 10 10 ...
#'   - attr(*, "dimnames")=List of 2
#'   ..$ income      : chr [1:4] "< 15k" "15-25k" "25-40k" "> 40k"
#'   ..$ satisfaction: chr [1:4] "VeryD" "LittleD" "ModerateS" "VeryS"
#' }
#'
#' @source Agresti, A. Categorical Data Analysis John Wiley & Sons, 2002, Table
#' 2.8, p. 57.
#' @keywords datasets
#' @examples
#'
#' data(JobSat)
#' assocstats(JobSat)
#' GKgamma(JobSat)
#'
NULL





# Loglinear Model Utilities
#
#
# @aliases loglin-utilities conditional joint loglin2formula loglin2string
#          markov mutual saturated
# @param nf number of factors for which to generate the model
# @param table a contingency table used only for factor names in the model,
# typically the output from \code{\link[base]{table}} and possibly permuted
# with \code{aperm}
# @param factors names of factors used in the model formula when \code{table}
# is not specified
# @param with For \code{joint} and \code{conditional} models, \code{with}
# gives the indices of the factors against which all others are considered
# jointly or conditionally independent
# @param order For \code{markov}, this gives the order of the Markov chain
# model for the factors.  An \code{order=1} Markov chain allows associations
# among sequential pairs of factors, e.g., \code{[A,B], [B,C], [C,D]} \dots{}.
# An \code{order=2} Markov chain allows associations among sequential triples.
# @param x For the \code{loglin2*} functions, a list of terms in a loglinear
# model, such as returned by \code{conditional}, \code{joint}, \dots{}
# @param env For \code{loglin2formula}, environment in which to evaluate the
# formula
# @param brackets For \code{loglin2string}, characters to use to surround
# model terms.  Either a single character string containing two characters
# (e.g., \code{'[]'} or a character vector of length two.
# @param sep For \code{loglin2string}, the separator character string used for
# factor names within a given model term
# @param collapse For \code{loglin2string}, the character string used between
# terms in the the model string
# @param abbrev For \code{loglin2string}, whether and how to abbreviate the
# terms in the string representation. This has not yet been implemented.
# @return For the main model specification functions, \code{conditional},
# \code{joint}, \code{markov}, \dots{}, the result is a list of vectors
# (terms), where the elements in each vector are the names of the factors. The
# elements of the list are given names \code{term1, term2, \dots{}}.
# @author Michael Friendly
# @seealso \code{\link[stats]{loglin}}, \code{\link[MASS]{loglm}}
# @references These functions were inspired by the original SAS implementation
# of mosaic displays, described in the \emph{User's Guide},
# \url{http://www.datavis.ca/mosaics/mosaics.pdf}
# @keywords models
# EXAMPLES NOT COPIED
# @examples
#
# joint(3, table=HairEyeColor)
# # as a formula or string
# loglin2formula(joint(3, table=HairEyeColor))
# loglin2string(joint(3, table=HairEyeColor))
#
# joint(2, HairEyeColor)  # marginal model for [Hair] [Eye]
#
# # other possibilities
# joint(4, factors=letters, with=1)
# joint(5, factors=LETTERS)
# joint(5, factors=LETTERS, with=4:5)
#
# conditional(4)
# conditional(4, with=3:4)
#
# # use in mosaic displays or other strucplots
# mosaic(HairEyeColor, expected=joint(3))
# mosaic(HairEyeColor, expected=conditional(3))
#
# # use with MASS::loglm
# cond3 <- loglin2formula(conditional(3, table=HairEyeColor))
# cond3 <- loglin2formula(conditional(3))  # same, with factors 1,2,3
# require(MASS)
# loglm(cond3, data=HairEyeColor)
#
# saturated(3, HairEyeColor)
# loglin2formula(saturated(3, HairEyeColor))
# loglin2string(saturated(3, HairEyeColor))
# loglin2string(saturated(3, HairEyeColor), brackets='{}', sep=', ')
#
#
#NULL





# The Logarithmic Series Distribution
#
# The logarithmic series distribution is a long-tailed distribution introduced
# by Fisher etal. (1943) in connection with data on the abundance of
# individuals classified by species.
#
# These functions provide the density, distribution function, quantile
# function and random generation for the logarithmic series distribution with
# parameter \code{prob}.
#
# The logarithmic series distribution with \code{prob} = \eqn{p} has density
# \deqn{ p ( x ) = \alpha p^x / x } for \eqn{x = 1, 2, \dots}, where
# \eqn{\alpha= -1 / \log(1 - p)} and \eqn{0 < p <1}.  Note that counts
# \code{x==2} cannot occur.
#
# @aliases Logseries dlogseries plogseries qlogseries rlogseries
# @param x,q vector of quantiles representing the number of events.
# @param prob parameter for the distribution, \code{0 < prob < 1}
# @param log,log.p logical; if TRUE, probabilities \code{p} are given as
# \code{log(p)}
# @param lower.tail logical; if TRUE (default), probabilities are \eqn{P[X \le
# x]}{P[X <= x]}, otherwise, \eqn{P[X > x]}{P[X > x]}.
# @param p vector of probabilities
# @param max.value maximum value returned by \code{qlogseries}
# @param n number of observations for \code{rlogseries}
# @return \code{dlogseries} gives the density, \code{plogseries} gives the
# distribution function, \code{qlogseries} gives the quantile function, and
# \code{rlogseries} generates random deviates.
#
# %% ~Describe the value returned %% If it is a LIST, use %% \item{comp1
# }{Description of 'comp1'} %% \item{comp2 }{Description of 'comp2'} %% ...
# @author Michael Friendly, using original code modified from the
# \code{gmlss.dist} package by Mikis Stasinopoulos.
# @seealso \code{\link[stats]{Distributions}}, ~~~
# @references \url{https://en.wikipedia.org/wiki/Logarithmic_distribution}
#
# Fisher, R. A. and Corbet, A. S. and Williams, C. B. (1943). The relation
# between the number of species and the number of individuals \emph{Journal of
# Animal Ecology}, 12, 42-58.
# @keywords distribution
# @examples
#
# XL <-expand.grid(x=1:5, p=c(0.33, 0.66, 0.99))
# lgs.df <- data.frame(XL, prob=dlogseries(XL[,"x"], XL[,"p"]))
# lgs.df$p = factor(lgs.df$p)
# str(lgs.df)
#
# require(lattice)
# mycol <- palette()[2:4]
# xyplot( prob ~ x, data=lgs.df, groups=p,
# 	xlab=list('Number of events (k)', cex=1.25),
# 	ylab=list('Probability',  cex=1.25),
# 	type='b', pch=15:17, lwd=2, cex=1.25, col=mycol,
# 	key = list(
# 					title = 'p',
# 					points = list(pch=15:17, col=mycol, cex=1.25),
# 					lines = list(lwd=2, col=mycol),
# 					text = list(levels(lgs.df$p)),
# 					x=0.9, y=0.98, corner=c(x=1, y=1)
# 					)
# 	)
#
#
# # random numbers
# hist(rlogseries(200, prob=.4), xlab='x')
# hist(rlogseries(200, prob=.8), xlab='x')
#
#
# NULL





#' Mammogram Ratings
#'
#' Kundel & Polansky (2003) give (possibly contrived) data on a set of 110
#' mammograms rated by two readers.
#'
#'
#' @name Mammograms
#' @docType data
#' @format
#' A frequency table in matrix form.  The format is:
#' \preformatted{
#'   num [1:4, 1:4] 34 6 2 0 10 8 5 1 2 8 ...
#' - attr(*, "dimnames")=List of 2
#' ..$ Reader2: chr [1:4] "Absent" "Minimal" "Moderate" "Severe"
#' ..$ Reader1: chr [1:4] "Absent" "Minimal" "Moderate" "Severe"
#' }
#'
#' @source
#' Kundel, H. L. & Polansky, M. (2003), "Measurement of Observer
#' Agreement", *Radiology*, **228**, 303-308, Table A1
#' @keywords datasets
#' @examples
#'
#' data(Mammograms)
#' B <- agreementplot(Mammograms, main="Mammogram ratings")
#' # agreement measures
#' B
#' Kappa(Mammograms)
#'
#' ## other displays
#' mosaic(Mammograms, shade=TRUE)
#'
#' sieve(Mammograms, pop = FALSE, shade = TRUE)
#' labeling_cells(text = Mammograms,
#'   gp_text = gpar(fontface = 2, cex=1.75))(as.table(Mammograms))
#'
NULL





#' Mental Impairment and Parents SES
#'
#' A 6 x 4 contingency table representing the cross-classification of mental
#' health status (`mental`) of 1660 young New York residents by their
#' parents' socioeconomic status (`ses`).
#'
#' @details
#' Both `ses` and `mental` can be treated as ordered factors or
#' integer scores.  For `ses`, 1="High" and 6="Low".
#'
#' @name Mental
#' @docType data
#' @format A data frame frequency table with 24 observations on the following 3
#' variables.
#' \describe{
#'   \item{`ses`}{an ordered factor with levels `1` < `2` < `3` < `4` < `5` < `6`}
#'   \item{`mental`}{an ordered factor with levels `Well` < `Mild` < `Moderate` < `Impaired`}
#'   \item{`Freq`}{cell frequency: a numeric vector}
#' }
#'
#' @references
#' Friendly, M. *Visualizing Categorical Data*, Cary, NC: SAS
#' Institute, 2000, Appendix B.7.
#' @source
#' Haberman, S. J.  *The Analysis of Qualitative Data: New
#' Developments*, Academic Press, 1979, Vol. II, p. 375.
#'
#' Srole, L.; Langner, T. S.; Michael, S. T.; Kirkpatrick, P.; Opler, M. K. &
#' Rennie, T. A. C.  *Mental Health in the Metropolis: The Midtown
#' Manhattan Study*, NYU Press, 1978, p. 289
#'
#' @keywords datasets
#' @examples
#'
#' data(Mental)
#' str(Mental)
#' (Mental.tab <- xtabs(Freq ~ ses + mental, data=Mental))
#'
#' # mosaic and sieve plots
#' mosaic(Mental.tab, gp=shading_Friendly)
#' sieve(Mental.tab, gp=shading_Friendly)
#'
#' if(require(ca)){
#'   plot(ca(Mental.tab), main="Mental impairment & SES", lines=TRUE)
#' }
#'
#' # fit linear x linear (uniform) association model, using integer scores
#' # for rows/cols
#' indep <- glm(Freq ~ mental + ses, family = poisson, data = Mental)
#' Cscore <- as.numeric(Mental$ses)
#' Rscore <- as.numeric(Mental$mental)
#'
#' linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
#'               family = poisson, data = Mental)
#' anova(linlin, test = "Chisq")
#'
#' # use update.glm method to fit other models
#' linlin <- update(indep, . ~ . + Rscore:Cscore)
#' roweff <- update(indep, . ~ . + mental:Cscore)
#' coleff <- update(indep, . ~ . + Rscore:ses)
#' rowcol <- update(indep, . ~ . + Rscore:ses + mental:Cscore)
#'
#' # compare models
#' LRstats(indep, linlin, roweff, coleff, rowcol)
#'
#' # tests of nested models
#' anova(indep, linlin, roweff, test = "Chisq")
#'
NULL





#' Mice Depletion Data
#'
#' Data from Kastenbaum and Lamphiear (1959). The table gives the number of
#' depletions (deaths) in 657 litters of mice, classified by litter size and
#' treatment.  This data set has become a classic in the analysis of
#' contingency tables, yet unfortunately little information on the details of
#' the experiment has been published.
#'
#'
#' @name Mice
#' @docType data
#' @format A frequency data frame with 30 observations on the following 4
#' variables, representing a 5 x 2 x 3 contingency table.
#' \describe{
#'   \item{`litter`}{litter size, a numeric vector}
#'   \item{`treatment`}{treatment, a factor with levels `A` `B`}
#'   \item{`deaths`}{number of depletions, a factor with levels `0` `1` `2+`}
#'   \item{`Freq`}{cell frequency, a numeric vector}
#' }
#'
#' @references
#' Kastenbaum, M. A. & Lamphiear, D. E. (1959) Calculation of
#' chi-square to calculate the no three-factor interaction hypothesis.
#' *Biometrics*, 15, 107-115.
#' @source
#' Goodman, L. A. (1983) The analysis of dependence in
#' cross-classifications having ordered categories, using log-linear models for
#' frequencies and log-linear models for odds. *Biometrics*, 39, 149-160.
#' @keywords datasets
#' @examples
#'
#' data(Mice)
#' # make a table
#' ftable(mice.tab <- xtabs(Freq ~ litter + treatment + deaths, data=Mice))
#'
#' #library(vcd)
#' vcd::mosaic(mice.tab, shade=TRUE)
#'
#'
NULL





#' Social Mobility data
#'
#' Data on social mobility, recording the occupational category of fathers and
#' their sons.
#'
#'
#' @name Mobility
#' @docType data
#' @format A 2-dimensional array resulting from cross-tabulating 2 variables
#' for 19912 observations. The variable names and their levels are:
#'
#' \tabular{rll}{
#'   No \tab Name \tab Levels \cr
#'   1\tab `Son's_Occupation`\tab `"UpNonMan", "LoNonMan", "UpManual", "LoManual", "Farm"`\cr
#'     2\tab `Father's_Occupation`\tab `"UpNonMan", "LoNonMan", "UpManual", "LoManual", "Farm"`\cr
#' }
#'
#' @seealso \code{\link{Glass}}, \code{\link{Hauser79}},
#'      \code{\link{Yamaguchi87}} for other examples of mobility data.
#' @source
#'
#' Falguerolles, A. de and Mathieu, J. R. (1988).  *Proceedings of
#' COMPSTAT 88*, Copenhagen, Denmark, Springer-Verlag.
#'
#' % \cite{FeathermanHauser:78}
#'
#' Featherman, D. L. and Hauser, R. M. Occupations and social mobility in the
#' United States.  *Sociological Microjournal*, 12, Fiche 62. Copenhagen:
#' Sociological Institute.
#' @keywords datasets
#' @examples
#'
#' data(Mobility)
#' Mobility
#'
#' # independence model
#' MASS::loglm(~Father_Occupation + Son_Occupation, data = Mobility)
#'
#' vcd::mosaic(Mobility, shade=TRUE, legend = FALSE)
#'
#'
#'
NULL





#' Publications of PhD Candidates
#'
#' A data set giving the number of publications by doctoral candidates in
#' biochemistry in relation to various predictors, originally from Long (1997).
#'
#' There is a large number of zero counts. Is there evidence for a separate
#' group of non-publishers?
#'
#' In this version of the data set, `phdprestige` had been rounded to the
#' nearest integer. A Stata version with the continuous values was subsequently
#' found at <https://www.stata-press.com/data/lf2/couart2.dta>
#'
#' @name PhdPubs
#' @docType data
#' @format A data frame with 915 observations on the following 6 variables.
#' \describe{
#'   \item{`articles`}{number of articles published in the final three years of PhD studies}
#'   \item{`female`}{dummy variable for gender, coded `1` for female}
#'   \item{`married`}{dummy variable for marital status, coded `1` for married}
#'   \item{`kid5`}{number of young children, age 5 and under}
#'   \item{`phdprestige`}{prestige of the PhD department.  The higher the number the more prestigious the program.}
#'   \item{`mentor`}{number of publications by the mentor in the preceeding three years}
#' }
#'
#' @source
#' Long, J. S. (1997). *Regression Models for Categorical and
#' Limited Dependent Variables*, Sage.
#'
#' Long, J. S. & Freese, J. (2006). *Regression Models for Categorical
#' Dependent Variables Using Stata*, 2nd Ed., Stata Press.
#' @keywords datasets
#' @examples
#'
#' data(PhdPubs)
#' # very uninformative
#' hist(PhdPubs$articles,
#'      breaks=0:19, col="pink", xlim=c(0,20),
#'      xlab="Number of Articles")
#'
#' library(vcd)
#' rootogram(goodfit(PhdPubs$articles), xlab="Number of Articles")
#'
#' # compare with negative binomial
#' rootogram(goodfit(PhdPubs$articles, type="nbinomial"),
#' 	xlab="Number of Articles", main="Negative binomial")
#'
#'
#'
NULL


#' Risk Factors for Coronary Heart Disease
#'
#' Data from the beginning of a 15-year follow-up study of probable risk factors
#' for coronary thrombosis. The data are from 1841 men employed in a Czechoslovakian
#' car factory and represent all possible combinations of the six risk factors.
#'
#' The study was conducted to examine risk factors for coronary heart disease (CHD) and
#' collected information on smoking habits, mental and physical work strain, systolic
#' blood pressure, ratio of lipoproteins, and family history of CHD.
#'
#' The six variables form a \eqn{2^6 = 64} contingency table. The data have been used
#' extensively to illustrate model search procedures for high-dimensional contingency tables.
#'
#' @name Reinis
#' @docType data
#' @format A 6-dimensional array resulting from cross-tabulating 6 binary variables
#' for 1841 observations. The variable names and their levels are:
#'
#' \tabular{rll}{
#'   No \tab Name        \tab Levels (meaning) \cr
#'   1  \tab `smoke`   \tab `"y", "n"` (smoking)\cr
#'   2  \tab `mental`  \tab `"y", "n"` (strenuous mental work)\cr
#'   3  \tab `phys`    \tab `"y", "n"` (strenuous physical work)\cr
#'   4  \tab `systol`  \tab `"y", "n"` (systolic blood pressure > 140)\cr
#'   5  \tab `protein` \tab `"y", "n"` (ratio of beta to alpha lipoproteins > 3)\cr
#'   6  \tab `family`  \tab `"y", "n"` (family history of CHD)\cr
#' }
#'
#' @references
#' Edwards, D. and Havranek, T. (1985).
#' A fast procedure for model search in multidimensional contingency tables.
#' *Biometrika*, 72, 339-351.
#'
#' Gauraha, N. and Parui, S. K. (2020).
#' Mutual conditional independence and its applications to model selection in Markov networks.
#' *Annals of Mathematics and Artificial Intelligence*, 88, 951-972.
#' \doi{10.1007/s10472-020-09690-7}
#'
#' Reinis, Z., Pokorny, J., Basika, V., Tiserova, J., Gorican, K., Horakova, D.,
#' Stuchlikova, E., Havranek, T., and Hrabovsky, F. (1981).
#' Prognostic significance of the risk profile in the prevention of coronary heart disease.
#' *Bratis. lek. Listy*, 76, 137-150.
#'
#' @source Originally from the \pkg{gRbase} package as `data(reinis, package = "gRbase")`
#' @keywords datasets
#' @examples
#' data(Reinis)
#' str(Reinis)
#' ftable(Reinis, row.vars = 1:3)
#'
#' # Fit all 0-way through 6-way models using Kway()
#' Reinis.gmodels <- Kway(Freq ~ smoke + mental + phys + systol + protein + family,
#'                        data = Reinis, family = poisson)
#'
#' # Examine fit statistics
#' LRstats(Reinis.gmodels)
#'
#' # Sequential tests for k vs k+1 way effects
#' anova(Reinis.gmodels, test = "Chisq")
#'
#' # Fit sequential models of joint independence
#' Reinis.seqjoint <- seq_loglm(Reinis, type = "joint", prefix="joint")
#' LRstats(Reinis.seqjoint)
#'
#' # Fit sequential models of conditional independence
#' Reinis.seqcond <- seq_loglm(Reinis, type = "conditional", prefix = "cond")
#' LRstats(Reinis.seqcond)
#'
NULL


#' Shakespeare's Word Type Frequencies
#'
#' This data set, from Efron and Thisted (1976), gives the number of distinct
#' words types (`Freq`) of words that appeared exactly once, twice, etc.
#' up to 100 times (`count`) in the complete works of Shakespeare.  In
#' these works, Shakespeare used 31,534 distinct words (types), comprising
#' 884,647 words in total.
#'
#' Efron & Thisted used this data to ask the question, "How many words did
#' Shakespeare know?"  Put another way, suppose another new corpus of works
#' Shakespeare were discovered, also with 884,647 words. How many new word
#' types would appear? The answer to the main question involves contemplating
#' an infinite number of such new corpora.
#'
#' In addition to the words that appear `1:100` times, there are 846 words
#' that appear more than 100 times, not listed in this data set.
#'
#' @name ShakeWords
#' @docType data
#' @format A data frame with 100 observations on the following 2 variables.
#' \describe{
#'    \item{`count`}{the number of times a word type appeared in Shakespeare's written works}
#'    \item{`Freq`}{the number of different words (types) appearing with this count.}
#'   }
#
#' @source
#' Bradley Efron and Ronald Thisted (1976). Estimating the Number of
#' Unseen Species: How Many Words Did Shakespeare Know? *Biometrika*, Vol.
#' 63, No. 3, pp. 435-447,
#' %<http://www.jstor.org/stable/2335721>
#' @keywords datasets
#' @examples
#'
#' data(ShakeWords)
#' str(ShakeWords)
#'
#' plot(sqrt(Freq) ~ count, data=ShakeWords)
#'
NULL





#' Passengers on the Titanic
#'
#' Data on passengers on the RMS Titanic, excluding the Crew and some
#' individual identifier variables.
#'
#' There are a number of related versions of the Titanic data, in various
#' formats. This version was derived from `ptitanic` in the
#' \pkg{rpart.plot} package, modifying it to remove the `Class 'labelled'`
#' attributes for some variables (inherited from Frank Harrell's
#' `titanic3` version) which caused problems with some applications,
#' notably `ggplot2`.
#'
#' Other versions:
#'
#' \code{\link[datasets]{Titanic}} is the 4-way frequency table of all 2201
#' people aboard the Titanic, including passengers and crew.
#'
#' @name Titanicp
#' @docType data
#' @format A data frame with 1309 observations on the following 6 variables.
#' \describe{
#'   \item{`pclass`}{a factor with levels `1st` `2nd` `3rd`}
#'   \item{`survived`}{a factor with levels `died` `survived`}
#'   \item{`sex`}{a factor with levels `female` `male`}
#'   \item{`age`}{passenger age in years (or fractions of a year, for children), a numeric vector; age is missing for 263 of the passengers}
#'   \item{`sibsp`}{number of siblings or spouses aboard, integer: `0:8`}
#'   \item{`parch`}{number of parents or children aboard, integer: `0:6`}
#' }
#
#' @source
#'
#' The original R source for this dataset was compiled by Frank Harrell and
#' Robert Dawson:
#' <https://hbiostat.org/data/repo/titanic.txt>,
#' described in more detail in
#' <https://hbiostat.org/data/repo/titanic>
#'
#' For this version of the Titanic data, passenger details were deleted,
#' survived was cast as a factor, and the name changed to `Titanicp` to
#' minimize confusion with other versions.
#' @keywords datasets
#' @examples
#'
#' data(Titanicp)
#' ## maybe str(Titanicp) ; plot(Titanicp) ...
#'
NULL





#' Toxaemia Symptoms in Pregnancy
#'
#' Brown et al (1983) gave these data on two signs of toxaemia, an abnormal
#' condition during pregnancy characterized by high blood pressure
#' (hypertension) and high levels of protein in the urine.  If untreated, both
#' the mother and baby are at risk of complications or death.
#'
#' The data frame `Toxaemia` represents 13384 expectant mothers in
#' Bradford, England in their first pregnancy, who were also classified
#' according to social class and the number of cigarettes smoked per day.
#'
#'
#' @name Toxaemia
#' @docType data
#' @format A data frame in frequency form representing a 5 x 3 x 2 x 2
#' contingency table, with 60 observations on the following 5 variables.
#' \describe{
#'   \item{`class`}{Social class of mother, a factor with levels `1` `2` `3` `4` `5`}
#'   \item{`smoke`}{Cigarettes smoked per day during pregnancy, a factor with levels `0` `1-19` `20+`}
#'   \item{`hyper`}{Hypertension level, a factor with levels `Low` `High`}
#'   \item{`urea`}{Protein urea level, a factor with levels `Low` `High`}
#'   \item{`Freq`}{frequency in each cell, a numeric vector}
#' }
#'
#' @references Friendly, M.  (2000), *Visualizing Categorical Data*, SAS
#' Institute, Cary, NC, Example 7.15.
#'
#' Friendly, M. and Meyer, D. (2016).  *Discrete Data Analysis with R:
#' Visualization and Modeling Techniques for Categorical and Count Data*.  Boca
#' Raton, FL: Chapman & Hall/CRC. <http://ddar.datavis.ca>. Example 10.10.
#' @source
#' Brown, P. J., Stone, J. and Ord-Smith, C. (1983), Toxaemic signs
#' during pregnancy. *JRSS, Series C, Applied Statistics*, 32, 69-72
#' @keywords datasets
#' @examples
#'
#' data(Toxaemia)
#'
#' tox.tab <- xtabs(Freq ~ class + smoke + hyper + urea, Toxaemia)
#' ftable(tox.tab, row.vars=1)
#'
#'
#' # symptoms by smoking
#' mosaic(~smoke + hyper + urea, data=tox.tab, shade=TRUE)
#'
#' # symptoms by social class
#' mosaic(~class + hyper + urea, data=tox.tab, shade=TRUE)
#'
#' # predictors
#' mosaic(~smoke + class, data=tox.tab, shade=TRUE)
#'
#' # responses
#' mosaic(~hyper + urea, data=tox.tab, shade=TRUE)
#'
#' # log odds ratios for urea and hypertension, by class and smoke
#' LOR <-loddsratio(aperm(tox.tab))
#' LOR
#'
#'
NULL





#' TV Viewing Data
#'
#' This data set `TV` comprises a 5 x 11 x 3 contingency table based on
#' audience viewing data from Neilsen Media Research for the week starting
#' November 6, 1995.
#'
#' The original data, `tv.dat`, contains two additional networks: "Fox"
#' and "Other", with small frequencies. These levels were removed in the
#' current version. There is also a fourth factor, transition State transition
#' (turn the television Off, Switch channels, or Persist in viewing the current
#' channel). The `TV` data here includes only the Persist observations.
#'
#' @name TV
#' @docType data
#' @format A 5 x 11 x 3 array of cell frequencies with the following structure:
#' \preformatted{
#'   int [1:5, 1:11, 1:3] 146 244 233 174 294 151 181 161 183 281 ...
#'   - attr(*, "dimnames")=List of 3
#'   ..$ Day    : chr [1:5] "Monday" "Tuesday" "Wednesday" "Thursday" ...
#'   ..$ Time   : chr [1:11] "8:00" "8:15" "8:30" "8:45" ...
#'   ..$ Network: chr [1:3] "ABC" "CBS" "NBC"
#' }
#'
#' @references
#' Friendly, M. and Meyer, D. (2016).  *Discrete Data Analysis
#' with R: Visualization and Modeling Techniques for Categorical and Count
#' Data*.  Boca Raton, FL: Chapman & Hall/CRC. <http://ddar.datavis.ca>.
#'
#' Emerson, John W. Mosaic Displays in S-PLUS: A General Implementation and a
#' Case Study. *Statistical Graphics and Computing Newsletter*, 1998,
#' 9(1), 17--23, <http://www.stat.yale.edu/~jay/R/mosaic/v91.pdf>
#'
#' Hartigan, J. A. & Kleiner, B. A Mosaic of Television Ratings. *The
#' American Statistician*, 1984, 38, 32-35.
#'
#' @source The original data, `tv.dat`, came from the initial
#' implementation of mosaic displays in R by Jay Emerson (1998). Similar data
#' had been used by Hartigan and Kleiner (1984) as an illustration.
#' @keywords datasets
#' @examples
#'
#' data(TV)
#' structable(TV)
#' doubledecker(TV)
#'
#' # reduce number of levels of Time
#' TV.df <- as.data.frame.table(TV)
#' levels(TV.df$Time) <- rep(c("8:00-8:59", "9:00-9:59", "10:00-10:44"),
#'                           c(4, 4, 3))
#'
#' TV2 <- xtabs(Freq ~ Day + Time + Network, TV.df)
#'
#' # re-label for mosaic display
#' levels(TV.df$Time) <- c("8", "9", "10")
#' # fit mode of joint independence, showing association of Network with Day*Time
#' mosaic(~ Day + Network + Time,
#'   data = TV.df,
#'   expected = ~ Day:Time + Network,
#'   legend = FALSE)
#'
#'
#' # with doubledecker arrangement
#' mosaic(~ Day + Network + Time,
#'   data = TV.df,
#'   expected = ~ Day:Time + Network,
#'   split = c(TRUE, TRUE, FALSE),
#'   spacing = spacing_highlighting,
#'   legend = FALSE)
#'
NULL



#' Student Opinion about the Vietnam War
#'
#' A survey of student opinion on the Vietnam War was taken at the University
#' of North Carolina at Chapel Hill in May 1967 and published in the student
#' newspaper. Students were asked to fill in ballot papers stating which policy
#' out of A,B,C or D they supported. Responses were cross-classified by
#' gender/year.
#'
#' The response categories were:
#' \describe{
#'   \item{`A`}{Defeat North Vietnam by widespread bombing and land invasion}
#'   \item{`B`}{Maintain the present policy}
#'   \item{`C`}{De-escalate military activity, stop bombing and begin negotiations}
#'   \item{`D`}{Withdraw military forces Immediately}
#' }
#'
#'
#' For some analyses, it is useful to treat `year` as numeric, and
#' possibly assign grad students a value `year=7`.
#'
#' @name Vietnam
#' @docType data
#' @format A frequency data frame with 40 observations representing a 2 x 5 x 4
#' contingency table on the following 4 variables.
#' \describe{
#'   \item{`sex`}{a factor with levels `Female` `Male`}
#'   \item{`year`}{year of study, an ordered factor with levels
#'     `Freshmen`, `Sophomore`, `Junior`, `Senior`, `Grad student`}
#'   \item{`response`}{a factor with levels `A` `B` `C` `D`}
#'   \item{`Freq`}{cell frequency, a numeric vector}
#' }
#
#' @references Friendly, M.  (2000), *Visualizing Categorical Data*, SAS
#' Institute, Cary, NC, Example 7.9.
#' @source Aitken, M. etal, 1989, *Statistical Modelling in GLIM*
#' @keywords datasets
#' @examples
#'
#' data(Vietnam)
#' ## maybe str(Vietnam) ; plot(Vietnam) ...
#'
NULL





#' Race and Politics in the 1980 Presidential Vote
#'
#' Data from the 1982 General Social Survey on votes in the 1980 U.S.
#' presidential election in relation to race and political conservatism.
#'
#' The data contains a number of sampling zeros in the frequencies of NonWhites
#' voting for Ronald Reagan.
#'
#' @name Vote1980
#' @docType data
#' @format A frequency data frame representing a 2 x 7 x 2 table, with 28
#' observations on the following 4 variables.
#' \describe{
#'   \item{`race`}{a factor with levels `NonWhite` `White`}
#'   \item{`conservatism`}{a factor with levels `1` `2` `3` `4` `5` `6` `7`,
#'        `1`=most liberal, `7`=most conservative}
#'   \item{`votefor`}{a factor with levels `Carter` `Reagan`; `Carter`represents Jimmy Carter or other.}
#'   \item{`Freq`}{a numeric vector}
#' }
#'
#' @references
#' Agresti, A. (1990) *Categorical Data Analysis*, Table 4.12
#' New York: Wiley-Interscience.
#'
#' Friendly, M. (2000) *Visualizing Categorical Data*, Example 7.5 Cary,
#' NC: SAS Institute.
#' @source
#' Clogg, C. & Shockey, J. W. (1988). In Nesselroade, J. R. & Cattell,
#' R. B. (ed.)  Multivariate Analysis of Discrete Data, *Handbook of
#' Multivariate Experimental Psychology*, New York: Plenum Press.
#' @keywords datasets
#' @examples
#'
#' data(Vote1980)
#' fourfold(xtabs(Freq ~ race + votefor + conservatism,
#'   data=Vote1980),
#'   mfrow=c(2,4))
#'
#'
NULL





#' Worker Satisfaction Data
#'
#' Blue collar workers job satisfaction from large scale investigation in
#' Denmark in 1968 (Andersen, 1991).
#'
#'
#' @name WorkerSat
#' @docType data
#' @format A frequency data frame with 8 observations on the following 4
#' variables, representing the 2 x 2 x 2 classification of 715 cases.
#' \describe{
#'   \item{`Manage`}{Quality of management, an ordered factor with levels `bad` < `good`}
#'   \item{`Super`}{Supervisor satisfaction, an ordered factor with levels `low` < `high`}
#'   \item{`Worker`}{Worker job satisfaction, an ordered factor with levels `low` < `high`}
#'   \item{`Freq`}{a numeric vector}
#' }
#' @references Andersen, E. B. (1991) Statistical Analysis of Categorical Data,
#' 2nd Ed., Springer-Verlag.
#' @source
#' Originally from <https://online.stat.psu.edu/stat504/Lesson10>
#' @keywords datasets
#' @examples
#'
#' data(WorkerSat)
#'
#' worker.tab <- xtabs(Freq ~ Worker + Super + Manage, data=WorkerSat)
#' fourfold(worker.tab)
#' mosaic(worker.tab, shade=TRUE)
#'
#'
NULL





#' Occupational Mobility in Three Countries
#'
#' Yamaguchi (1987) presented this three-way frequency table, cross-classifying
#' occupational categories of sons and fathers in the United States, United
#' Kingdom and Japan.  This data set has become a classic for models comparing
#' two-way mobility tables across layers corresponding to countries, groups or
#' time (e.g., Goodman and Hout, 1998; Xie, 1992).
#'
#' The US data were derived from the 1973 OCG-II survey; those for the UK from
#' the 1972 Oxford Social Mobility Survey; those for Japan came from the 1975
#' Social Stratification and Mobility survey. They pertain to men aged 20-64.
#'
#' Five status categories -- upper and lower nonmanuals (`UpNM`,
#' `LoNM`), upper and lower manuals (`UpM`, `LoM`), and
#' `Farm`) are used for both fathers' occupations and sons' occupations.
#'
#' Upper nonmanuals are professionals, managers, and officials; lower
#' nonmanuals are proprietors, sales workers, and clerical workers; upper
#' manuals are skilled workers; lower manuals are semi-skilled and unskilled
#' nonfarm workers; and farm workers are farmers and farm laborers.
#'
#' Some of the models from Xie (1992), Table 1, are fit in
#' `demo(yamaguchi-xie)`.
#'
#' @name Yamaguchi87
#' @docType data
#' @format A frequency data frame with 75 observations on the following 4
#' variables. The total sample size is 28887.
#'
#' \describe{
#'   \item{`Son`}{a factor with levels `UpNM` `LoNM` `UpM` `LoM` `Farm`}
#'   \item{`Father`}{a factor with levels `UpNM` `LoNM` `UpM` `LoM` `Farm`}
#'   \item{`Country`}{a factor with levels `US` `UK` `Japan`}
#'   \item{`Freq`}{a numeric vector}
#' }
#'
#' @references Goodman, L. A. and Hout, M. (1998). Statistical Methods and
#' Graphical Displays for Analyzing How the Association Between Two Qualitative
#' Variables Differs Among Countries, Among Groups, Or Over Time: A Modified
#' Regression-Type Approach. *Sociological Methodology*, 28 (1), 175-230.
#'
#' Xie, Yu (1992). The log-multiplicative layer effect model for comparing
#' mobility tables. *American Sociological Review*, 57 (June), 380-395.
#' @source Yamaguchi, K. (1987).  Models for comparing mobility tables: toward
#' parsimony and substance, *American Sociological Review*, vol. 52
#' (Aug.), 482-494, Table 1
#' @keywords datasets
#' @examples
#'
#' data(Yamaguchi87)
#' # reproduce Table 1
#' structable(~ Father + Son + Country, Yamaguchi87)
#' # create table form
#' Yama.tab <- xtabs(Freq ~ Son + Father + Country, data=Yamaguchi87)
#'
#' # define mosaic labeling_args for convenient reuse in 3-way displays
#' largs <- list(rot_labels=c(right=0), offset_varnames = c(right = 0.6),
#'               offset_labels = c(right = 0.2),
#'               set_varnames = c(Son="Son's status", Father="Father's status")
#'              )
#'
#' ###################################
#' # Fit some models & display mosaics
#'
#' # Mutual independence
#' yama.indep <- glm(Freq ~ Son + Father + Country,
#'   data=Yamaguchi87,
#'   family=poisson)
#' anova(yama.indep)
#'
#' mosaic(yama.indep, ~Son+Father, main="[S][F] ignoring country")
#'
#' mosaic(yama.indep, ~Country + Son + Father, condvars="Country",
#'        labeling_args=largs,
#'        main='[S][F][C] Mutual independence')
#'
#' # no association between S and F given country ('perfect mobility')
#' # asserts same associations for all countries
#' yama.noRC <- glm(Freq ~ (Son + Father) * Country,
#'   data=Yamaguchi87,
#'   family=poisson)
#' anova(yama.noRC)
#'
#' mosaic(yama.noRC, ~~Country + Son + Father, condvars="Country",
#'        labeling_args=largs,
#'        main="[SC][FC] No [SF] (perfect mobility)")
#'
#' # ignore diagonal cells
#' yama.quasi <- update(yama.noRC, ~ . + Diag(Son,Father):Country)
#' anova(yama.quasi)
#'
#' mosaic(yama.quasi, ~Son + Father, main="Quasi [S][F]")
#'
#' ## see also:
#' # demo(yamaguchi-xie)
#' ##
#'
NULL

#' Extensions and additions to vcd: Visualizing Categorical Data
#'
#' \if{html}{\figure{man/figures/logo.png}{options: alt='logo' width='100'}}
#'
#' This package provides additional data sets, documentation, and a few
#' functions designed to extend the `vcd` package for Visualizing
#' Categorical Data and the `gnm` package for Generalized Nonlinear
#' Models. In particular, vcdExtra extends mosaic, assoc and sieve plots from
#' vcd to handle glm() and gnm() models and adds a 3D version in
#' \code{\link{mosaic3d}}.
#'
#' This package is also a support package for the book, *Discrete Data
#' Analysis with R* by Michael Friendly and David Meyer, Chapman & Hall/CRC,
#' 2016,
#' <https://www.routledge.com/Discrete-Data-Analysis-with-R-Visualization-and-Modeling-Techniques-for-Categorical-and-Count-Data/Friendly-Meyer/p/book/9781498725835>
#' with a number of additional data sets, and functions. The web site for the
#' book is <http://ddar.datavis.ca>.
#'
#' In addition, I teach a course, *Psy 6136: Categorical Data Analysis*,
#' <https://friendly.github.io/psy6136/> using this package.
#'
#' The main purpose of this package is to serve as a sandbox for introducing
#' extensions of mosaic plots and related graphical methods that apply to
#' loglinear models fitted using `glm()` and related, generalized
#' nonlinear models fitted with `gnm()` in the
#' \code{\link[gnm]{gnm-package}} package. A related purpose is to fill in some
#' holes in the analysis of categorical data in R, not provided in base R, the
#' \pkg{vcd}, or other commonly used packages.
#'
#' The method \code{\link{mosaic.glm}} extends the
#' \code{\link[vcd]{mosaic.loglm}} method in the \pkg{vcd} package to this
#' wider class of models.  This method also works for the generalized nonlinear
#' models fit with the \code{\link[gnm]{gnm-package}} package, including models
#' for square tables and models with multiplicative associations.
#'
#' \code{\link{mosaic3d}} introduces a 3D generalization of mosaic displays
#' using the \pkg{rgl} package.
#'
#' In addition, there are several new data sets, a tutorial vignette,
#' \describe{ \item{vcd-tutorial}{Working with categorical data with R and the
#' vcd package, `vignette("vcd-tutorial", package = "vcdExtra") `} } and a
#' few functions for manipulating categorical data sets and working with models
#' for categorical data.
#'
#' A new class, \code{\link{glmlist}}, is introduced for working with
#' collections of `glm` objects, e.g., \code{\link{Kway}} for fitting all
#' K-way models from a basic marginal model, and \code{\link{LRstats}} for
#' brief statistical summaries of goodness-of-fit for a collection of models.
#'
#' For square tables with ordered factors, \code{\link{Crossings}} supplements
#' the specification of terms in model formulas using \code{\link[gnm]{Symm}},
#' \code{\link[gnm]{Diag}}, \code{\link[gnm]{Topo}}, etc. in the
#' \code{\link[gnm]{gnm-package}}.
#'
#' Some of these extensions may be migrated into \pkg{vcd} or \pkg{gnm}.
#'
#' A collection of demos is included to illustrate fitting and visualizing a
#' wide variety of models: \describe{ \item{mental-glm}{Mental health data:
#' mosaics for glm() and gnm() models} \item{occStatus}{Occupational status
#' data: Compare mosaic using expected= to mosaic.glm}
#' \item{ucb-glm}{UCBAdmissions data: Conditional independence via loglm() and
#' glm()} \item{vision-quasi}{VisualAcuity data: Quasi- and Symmetry models}
#' \item{yaish-unidiff}{Yaish data: Unidiff model for 3-way table}
#' \item{Wong2-3}{Political views and support for women to work (U, R, C, R+C
#' and RC(1) models)} \item{Wong3-1}{Political views, support for women to work
#' and national welfare spending (3-way, marginal, and conditional independence
#' models)} \item{housing}{Visualize glm(), multinom() and polr() models from
#' `example(housing, package="MASS")`} } Use `
#' demo(package="vcdExtra")` for a complete current list.
#'
#' The \pkg{vcdExtra} package now contains a large number of data sets
#' illustrating various forms of categorical data analysis and related
#' visualizations, from simple to advanced. Use `data(package="vcdExtra")`
#' for a complete list, or `datasets(package="vcdExtra")` for an annotated
#' one showing the `class` and `dim` for each data set.
#'
#' @name vcdExtra-package
#' @aliases vcdExtra-package vcdExtra _PACKAGE
#' @author Michael Friendly
#'
#' Maintainer: Michael Friendly 
#' (\href{https://orcid.org/0000-0002-3237-0941}{ORCID})
#' 
#' @seealso \code{\link[gnm]{gnm-package}}, for an extended range of models for
#' contingency tables
#'
#' \code{\link[vcd]{mosaic}} for details on mosaic displays within the
#' strucplot framework.
#' @references 
#' Friendly, M. *Visualizing Categorical Data*, Cary NC: SAS
#' Institute, 2000. Web materials: <http://www.datavis.ca/books/vcd/>.
#'
#' Friendly, M. and Meyer, D. (2016). *Discrete Data Analysis with R:
#' Visualization and Modeling Techniques for Categorical and Count Data*. Boca
#' Raton, FL: Chapman & Hall/CRC. <http://ddar.datavis.ca>.
#'
#' Meyer, D.; Zeileis, A. & Hornik, K. The Strucplot Framework: Visualizing
#' Multi-way Contingency Tables with vcd *Journal of Statistical
#' Software*, 2006, **17**, 1-48. Available in R via
#' `vignette("strucplot", package = "vcd")`
#'
#' Turner, H. and Firth, D. *Generalized nonlinear models in R: An
#' overview of the gnm package*, 2007, <http://eprints.ncrm.ac.uk/472/>.
#' Available in R via `vignette("gnmOverview", package = "gnm")`.
#' 
#' @importFrom grDevices hsv rgb
#' @importFrom stats as.formula deviance family fitted formula glm logLik loglin model.frame na.pass nobs pchisq poisson qnorm quantile reformulate residuals rstandard runif terms update update.formula xtabs
#' @importFrom utils data menu type.convert
#' @importFrom ca cacoord multilines
#' @importFrom graphics abline plot points text
#' @importFrom MASS loglm
#' 
#' @keywords package
#'
#' @examples
#'
#' example(mosaic.glm)
#'
#' demo("mental-glm")
#'
NULL

# logLik method for loglm objects, to allow use of AIC() and BIC()
# with MASS::loglm, giving comparable results to the use of these
# functions with glm(..., family=poisson) models.

# allow for non-integer frequencies
# allow for zero frequencies, with a zero= argument



#' Log-Likelihood of a loglm Object
#'
#' Calculates the log-likelihood value of the `loglm` model represented by
#' `object` evaluated at the estimated coefficients.
#'
#' It allows the use of \code{\link[stats]{AIC}} and \code{\link[stats]{BIC}},
#' which require that a `logLik` method exists to extract the
#' corresponding log-likelihood for the model.
#'
#' If cell frequencies have not been stored with the `loglm` object (via
#' the argument `keep.frequencies = TRUE`), they are obtained using
#' `update`.
#'
#' This function calculates the log-likelihood in a way that allows for
#' non-integer frequencies, such as the case where 0.5 has been added to all
#' cell frequencies to allow for sampling zeros.  If the frequencies still
#' contain zero values, those are replaced by the value of `start`.
#'
#' For integer frequencies, it gives the same result as the corresponding model
#' fit using \code{\link[stats]{glm}}, whereas \code{\link[stats]{glm}} returns
#' `-Inf` if there are any non-integer frequencies.
#'
#' @param object A `loglm` object
#' @param \dots For compatibility with the S3 generic; not used here
#' @param zero value used to replace zero frequencies in calculating the
#' log-likelihood
#' @return Returns an object of class `logLik`.  This is a number with one
#' attribute, `"df"` (degrees of freedom), giving the number of
#' (estimated) parameters in the model.
#' @author Achim Zeileis
#' @seealso \code{\link[MASS]{loglm}}, \code{\link[stats]{AIC}},
#' \code{\link[stats]{BIC}},
#' @keywords models htest
#' @examples
#'
#' data(Titanic, package="datasets")
#'
#' require(MASS)
#' titanic.mod1 <- loglm(~ (Class * Age * Sex) + Survived, data=Titanic)
#' titanic.mod2 <- loglm(~ (Class * Age * Sex) + Survived*(Class + Age + Sex), data=Titanic)
#' titanic.mod3 <- loglm(~ (Class * Age * Sex) + Survived*(Class + Age * Sex), data=Titanic)
#'
#' logLik(titanic.mod1)
#' AIC(titanic.mod1, titanic.mod2, titanic.mod3)
#' BIC(titanic.mod1, titanic.mod2, titanic.mod3)
#'
#' # compare with models fit using glm()
#' titanic <- as.data.frame(Titanic)
#' titanic.glm1 <- glm(Freq ~ (Class * Age * Sex) + Survived,
#'                     data=titanic, family=poisson)
#' titanic.glm2 <- glm(Freq ~ (Class * Age * Sex) + Survived*(Class + Age + Sex),
#'                     data=titanic, family=poisson)
#' titanic.glm3 <- glm(Freq ~ (Class * Age * Sex) + Survived*(Class + Age * Sex),
#'                     data=titanic, family=poisson)
#'
#' logLik(titanic.glm1)
#' AIC(titanic.glm1, titanic.glm2, titanic.glm3)
#' BIC(titanic.glm1, titanic.glm2, titanic.glm3)
#'
#'
#' @export
logLik.loglm <- function(object, ..., zero=1E-10) {
	fr <- if(!is.null(object$frequencies)) unclass(object$frequencies) else {
				unclass(update(object, keep.frequencies = TRUE)$frequencies)
			}
	df <- prod(dim(fr)) - object$df
	if (any(fr==0)) {
		fr <- as.vector(fr)
		fr[fr==0] <- zero
	}
	structure(sum((log(fr) - 1) * fr - lgamma(fr + 1))  - object$deviance/2,
			df = df, class = "logLik")
}

##
## One-line summary of model fit for a glm/loglm object
##



#' Brief Summary of Model Fit for a glm or loglm Object
#'
#' Formats a brief summary of model fit for a `glm` or `loglm`
#' object, showing the likelihood ratio Chisq (df) value and or AIC.  Useful
#' for inclusion in a plot title or annotation.
#'
#'
#' @name modFit
#' @aliases modFit modFit.loglm modFit.glm
#' @param x A `glm` or `loglm` object
#' @param \dots Arguments passed down
#' @param stats One or more of `chisq` or `aic`, determining the statistics displayed.
#' @param digits Number of digits after the decimal point in displayed statistics.
#'
#' @return A character string containing the formatted values of the chosen statistics.
#' @author Michael Friendly
#'
#' @seealso  \code{\link{LRstats}}
#'
#' @keywords utilities models
#' @examples
#'
#' data(Mental)
#' require(MASS)
#' (Mental.tab <- xtabs(Freq ~ ses + mental, data=Mental))
#' (Mental.mod <- loglm(~ses + mental, Mental.tab))
#' Mental.mod
#' modFit(Mental.mod)
#'
#' # use to label mosaic()
#' mosaic(Mental.mod, main=paste("Independence model,", modFit(Mental.mod)))
#'
#' @export
`modFit` <-
function(x, ...) UseMethod("modFit")

#' @param stats statistics to print: one or more of `"chisq"`, `"aic"`
#' @param digits number to digits to use in the print method
#' @rdname modFit
#' @export
modFit.glm <- function(x, stats="chisq", digits=2, ...) {
	if (!inherits(x,"glm")) stop("modFit requires a glm object")
	result <- NULL
	if ("chisq" %in% stats)
		result <- paste("G^2(",x$df.residual,")=",
				formatC(x$deviance,digits=digits,format="f"),sep="")
	if ("aic" %in% stats)
		result <- paste(result, " AIC=", formatC(x$aic,digits=digits,format="f"),sep="")
	result
}


#' @rdname modFit
#' @export
modFit.loglm <- function(x, stats="chisq", digits=2, ...) {
	if (!inherits(x,"loglm")) stop("modFit requires a loglm object")
	result <- NULL
	if ("chisq" %in% stats)
		result <- paste("G^2(",x$df,")=",
				formatC(x$deviance,digits=digits,format="f"),sep="")
	if ("aic" %in% stats) {
	    aic<-x$deviance-x$df*2
		result <- paste(result, " AIC=", formatC(aic,digits=digits,format="f"),sep="")
		}
	result
}

#' Include an HTML-renderable object in any knitr output format
#'
#' A pipe-friendly helper in Rmarkdown, Quarto, and other documents for objects that render natively in HTML output
#' (RStudio Viewer, HTML documents) but need a convenient image fallback for PDF, Word,
#' and other non-HTML formats. It was designed to solve a problem in rendering the result of `color_table()`,
#' and then generalized to provide for other similar cases.
#'
#' Supported classes (handled internally) solve this problem by saving the image to a file and then inserting it in the document
#' using `knitr::include_graphics()`:
#'   - `"gt_tbl"`     (gt package)       -- saved via \code{\link[gt]{gtsave}}
#'   - `"htmlwidget"` (htmlwidgets family: plotly, DT, leaflet, dygraphs, ...)
#'                                     -- saved via
#'                                        \code{\link[htmlwidgets]{saveWidget}}
#'                                        + \code{\link[webshot2]{webshot}}
#'
#' Packages that handle cross-format output natively (\pkg{flextable}, \pkg{kableExtra},
#' \pkg{huxtable}) do NOT need this helper.
#'
#' For any other class, \code{x} is returned as-is in all output formats and
#' knitr's normal printing applies.  This means the function is safe to use in
#' a pipe on any object: it only intervenes when it knows how to help!
#'
#' @param x       Any R object. Specialised handling for \code{gt_tbl} and
#'                \code{htmlwidget}; all other classes are passed through
#'                unchanged.
#' @param file    Base filename (no extension) for temporary files written when
#'                output is not HTML.  Defaults to a session-unique temp file in
#'                the current directory so multiple calls don't collide.
#' @param width   Viewport / screenshot width in pixels (non-HTML only).
#' @param height  Viewport / screenshot height in pixels (non-HTML only).
#' @param ...     Additional arguments forwarded to \code{\link[gt]{gtsave}} or
#'                \code{\link[webshot2]{webshot}}.
#'
#' @return For \code{gt_tbl} / \code{htmlwidget} in non-HTML output: the result
#'         of \code{\link[knitr]{include_graphics}}. Otherwise: \code{x}
#'         unchanged.
#'
#' @examples
#' \dontrun{
#' data(HairEyeColor)
#' HEC <- margin.table(HairEyeColor, 1:2)
#'
#' ## gt_tbl from color_table()
#' color_table(HEC, title = "Hair x Eye Color") |> knit_include()
#'
#' ## DT htmlwidget
#' DT::datatable(mtcars) |> knit_include(width = 900, height = 500)
#'
#' ## plotly htmlwidget
#' plotly::plot_ly(mtcars, x = ~wt, y = ~mpg) |> knit_include()
#'
#' ## Any other object passes through unchanged
#' lm(mpg ~ wt, data = mtcars) |> knit_include()
#' }
#'
#' @importFrom knitr is_html_output include_graphics
#' @importFrom webshot2 webshot
#' @importFrom htmlwidgets saveWidget
#' @export
knit_include <- function(x,
                         file   = tempfile("ki_", tmpdir = "."),
                         width  = 700,
                         height = 400,
                         ...) {

  if (knitr::is_html_output()) {
    return(x)
  }

  if (inherits(x, "gt_tbl")) {
    img <- paste0(file, ".png")
    gt::gtsave(x, filename = img, vwidth = width, vheight = height, ...)
    return(knitr::include_graphics(img))
  }

  if (inherits(x, "htmlwidget")) {
    if (!requireNamespace("htmlwidgets", quietly = TRUE))
      stop("Package 'htmlwidgets' is required for htmlwidget objects.")
    if (!requireNamespace("webshot2", quietly = TRUE))
      stop("Package 'webshot2' is required for htmlwidget objects in non-HTML output.")

    html <- paste0(file, ".html")
    img  <- paste0(file, ".png")
    htmlwidgets::saveWidget(x, html, selfcontained = TRUE)
    webshot2::webshot(html, img, vwidth = width, vheight = height, ...)
    return(knitr::include_graphics(img))
  }

  # Unknown class: return x and let knitr's normal printing handle it,
  # just as it would outside a pipe.
  x
}

# Unified Woolf test for 2 x 2 x ... tables of any dimensionality
# Combines functionality from woolf_test.R, woolf_test_general.R, and woolf_test_2way.R
#
# Main changes compared to vcd::woolf_test():
#
# DONE: Generalized to handle tables of any dimensionality >= 3 (not just 2x2xk)
#       For tables with more than 3 dimensions, collapses dimensions 3+ into single stratum dimension
#
# DONE: Added decompose parameter for 4-dimensional tables (2x2xRxC)
#       When decompose=TRUE, provides two-way decomposition into row effects, column effects, and residual
#       Decomposition analogous to two-way ANOVA: Overall = Rows + Columns + Residual
#
# DONE: Unified return structure for both decomposed and non-decomposed cases
#       Both return class c("woolf_test", "htest") with consistent base components
#       Decomposed case includes additional components: rows, cols, residual
#
# DONE: Added dimension name extraction and display
#       New components: or_vars (names of 2x2 table variables), strata_vars (stratifying variables)
#       If array has no dimension names, creates default names (Dim1, Dim2, etc.)
#
# DONE: Created unified print.woolf_test() method
#       Single print method handles both decomposed and non-decomposed cases
#       Displays data name, OR variables, and strata variables in formatted output
#       For decomposed case, shows which stratum variable corresponds to rows/columns
#
# DONE: Added comprehensive roxygen2 documentation
#       Includes @param, @return, @examples, @references, @seealso
#       Documents all return components including new ones
#
# DONE: Added decomposed logical flag to return object
#       Allows print method and other functions to distinguish between cases
#
# TODO: Do we still need to include class "htest"?

#' Woolf Test for Homogeneity of Odds Ratios
#'
#' Test for homogeneity on \eqn{2 \times 2 \times k}{2 x 2 x k} tables
#' over strata (i.e., whether the log odds ratios are the same in all
#' strata). Generalized to handle tables of any dimensionality beyond 3.
#' For 4-dimensional tables, optionally provides a two-way decomposition
#' of the homogeneity test into row effects, column effects, and residual.
#'
#' @details
#' The Woolf test (Woolf, 1955) tests the hypothesis that the odds ratios
#' \eqn{\theta_i} are equal across all \eqn{k} strata. The test statistic
#' is computed as the weighted sum of squared deviations of the log odds ratios
#' from their weighted mean:
#'
#' \deqn{\chi^2_W = \sum_{i=1}^{k} w_i [\log(\theta_i) - \log(\bar{\theta}_w)]^2
#'       = \sum_{i=1}^{k} w_i \log^2(\theta_i / \bar{\theta}_w)}
#'
#' where \eqn{\theta_i = (n_{11i} n_{22i}) / (n_{12i} n_{21i})} is the odds ratio
#' in stratum \eqn{i}, and \eqn{\bar{\theta}_w} is the weighted average odds ratio
#' (computed as the exponential of the weighted mean of the log odds ratios).
#'
#' The weights \eqn{w_i} are the inverse variances of the log odds ratios:
#' \deqn{w_i = 1 / \text{Var}(\log \theta_i) = 1 / (1/n_{11i} + 1/n_{12i} + 1/n_{21i} + 1/n_{22i})}
#'
#' Under the null hypothesis of homogeneous odds ratios, \eqn{\chi^2_W} follows
#' a chi-squared distribution with \eqn{k - 1} degrees of freedom.
#'
#' \strong{Decomposition for 4-way tables:}
#' For a \eqn{2 \times 2 \times R \times C} table, the strata
#' form an \eqn{R \times C} two-way layout with odds ratios
#' \eqn{\theta_{ij}} for row \eqn{i} and column \eqn{j}. This suggests
#' that overall Woolf test
#' of homogeneity can be decomposed into three components, conceptually
#' analogous to a two-way ANOVA with one observation per cell:
#'
#' \deqn{\chi^2_{\text{W:Total}} = \chi^2_{\text{W:Rows}} + \chi^2_{\text{W:Cols}} + \chi^2_{\text{W:Residual}}}
#'
#' where:
#' \itemize{
#'   \item \eqn{\chi^2_{\text{W:Rows}} } tests whether the odds ratios differ among
#'     row levels (pooling over columns), with \eqn{R - 1} df
#'   \item \eqn{\chi^2_{\text{W:Cols}}} tests whether the odds ratios differ among
#'     column levels (pooling over rows), with \eqn{C - 1} df
#'   \item \eqn{\chi^2_{\text{W:Residual}}} tests the row \eqn{\times}{x} column interaction
#'     (deviation from additivity on the log odds scale), with \eqn{(R-1)(C-1)} df
#' }
#'
#' The row effect test compares the marginal log odds ratios
#' \eqn{\log \bar{\theta}_{i+}} (pooled over columns) to the overall weighted mean.
#' Similarly, the column effect test compares \eqn{\log \bar{\theta}_{+j}}
#' (pooled over rows). The residual tests whether the cell log odds ratios
#' \eqn{\log \theta_{ij}} are additive in the row and column effects.
#'
#' \strong{Note:} The two-way ANOVA-like decomposition for 4-dimensional tables
#' appears to be a novel extension introduced in this package. The existing
#' literature on the Woolf test (and related Breslow-Day test) treats strata
#' as unstructured, testing only whether all \eqn{k} odds ratios are equal.
#' This decomposition exploits the factorial structure of \eqn{R \times C}{R x C}
#' strata to provide more detailed insight into the sources of heterogeneity.
#'
#' @param x A \eqn{2 \times 2 \times k}{2 x 2 x k} table, or more generally,
#'   a \eqn{2 \times 2 \times \ldots}{2 x 2 x ...} array where the first
#'   two dimensions are both 2.
#' @param decompose Logical. If \code{TRUE} and \code{x} is 4-dimensional
#'   (a \eqn{2 \times 2 \times R \times C}{2 x 2 x R x C} table), the test
#'   is decomposed into row effects, column effects, and residual (interaction).
#'   Defaults to \code{FALSE}. Ignored for non-4-dimensional tables.
#'
#' @return A list of class \code{"woolf_test"} (also inheriting from \code{"htest"})
#'   containing the following components:
#'   \item{statistic}{the chi-squared test statistic.}
#'   \item{parameter}{degrees of freedom of the approximate chi-squared
#'     distribution of the test statistic.}
#'   \item{p.value}{\eqn{p}-value for the test.}
#'   \item{method}{a character string indicating the type of test performed.}
#'   \item{data.name}{a character string giving the name of the data.}
#'   \item{or_vars}{names of the first two dimensions (the 2x2 table variables).}
#'   \item{strata_vars}{names of the stratifying variables (dimensions 3 and beyond).}
#'   \item{observed}{the observed log odds ratios.}
#'   \item{expected}{the expected log odds ratio under the null hypothesis (weighted mean).}
#'   \item{decomposed}{logical indicating if decomposition was performed.}
#'
#'   When \code{decompose = TRUE} (only for 4-dimensional tables), additional components:
#'   \item{rows}{list with statistic, df, p.value, observed, expected for row effects.}
#'   \item{cols}{list with statistic, df, p.value, observed, expected for column effects.}
#'   \item{residual}{list with statistic, df, p.value for residual (interaction).}
#'
#' @importFrom stats weighted.mean
#' @seealso \code{\link[stats]{mantelhaen.test}}, \code{\link[DescTools]{BreslowDayTest}}
#' @family association tests
#'
#' @references
#' Woolf, B. (1955). On estimating the relation between blood group and disease.
#' \emph{Annals of Human Genetics}, \bold{19}, 251-253.
#'
#' @examples
#' # 3-way tables
#' data(CoalMiners, package = "vcd")
#' woolf_test(CoalMiners)
#'
#' data(Heart, package = "vcdExtra")
#' woolf_test(Heart)
#'
#' # 4-way table without decomposition
#' data(Fungicide, package = "vcdExtra")
#' woolf_test(Fungicide)
#'
#' # 4-way table with decomposition
#' woolf_test(Fungicide, decompose = TRUE)
#'
#' # 4-way table, but need to rearrange dimensions
#' # How does association between `Preference` and `M_User` vary over strata?
#' data(Detergent, package = "vcdExtra")
#' dimnames(Detergent) |> names()
#' Detergent <- aperm(Detergent, c(3, 2, 1, 4))
#' woolf_test(Detergent)
#'
#'
#' @export
woolf_test <- function(x, decompose = FALSE) {
  DNAME <- deparse(substitute(x))

  # Check that first two dimensions are 2x2
  dims <- dim(x)
  if (length(dims) < 3) {
    stop("Array must have at least 3 dimensions")
  }
  if (!all(dims[1:2] == 2)) {
    stop("First two dimensions must be 2x2")
  }

  # Extract dimension names
  dimnames_x <- names(dimnames(x))
  if (is.null(dimnames_x)) {
    dimnames_x <- paste0("Dim", 1:length(dims))
  }
  or_vars <- dimnames_x[1:2]
  strata_vars <- dimnames_x[-(1:2)]

  # Check decompose argument validity
  if (decompose && length(dims) != 4) {
    warning("decompose = TRUE only applies to 4-dimensional tables; ignoring decompose")
    decompose <- FALSE
  }

  # Apply continuity correction if needed
  if (any(x == 0))
    x <- x + 1 / 2

  # Internal helper function to compute Woolf test on 2x2xk array
  woolf_internal <- function(y) {
    k <- dim(y)[3]
    or <- apply(y, 3, function(x) (x[1,1] * x[2,2]) / (x[1,2] * x[2,1]))
    w <-  apply(y, 3, function(x) 1 / sum(1 / x))
    o <- log(or)
    e <- weighted.mean(log(or), w)
    statistic <- sum(w * (o - e)^2)
    df <- k - 1
    pval <- 1 - pchisq(statistic, df)
    list(statistic = statistic, df = df, p.value = pval,
         observed = o, expected = e, weights = w)
  }

  # Handle 4-dimensional case with decomposition
  if (length(dims) == 4 && decompose) {
    R <- dims[3]
    C <- dims[4]

    # Overall test (all R*C strata)
    x_all <- array(x, dim = c(2, 2, R * C))
    overall <- woolf_internal(x_all)

    # Row effects: pool across columns (dimension 4) for each row (dimension 3)
    # Result: 2 x 2 x R array
    x_rows <- apply(x, c(1, 2, 3), sum)
    rows <- woolf_internal(x_rows)

    # Column effects: pool across rows (dimension 3) for each column (dimension 4)
    # Result: 2 x 2 x C array
    x_cols <- apply(x, c(1, 2, 4), sum)
    cols <- woolf_internal(x_cols)

    # Residual (interaction): difference between overall and main effects
    residual_stat <- overall$statistic - rows$statistic - cols$statistic
    residual_df <- (R - 1) * (C - 1)
    residual_pval <- 1 - pchisq(residual_stat, residual_df)

    # Create output structure (unified with standard case)
    names(overall$statistic) <- "X-squared"
    names(overall$df) <- "df"

    result <- list(
      statistic = overall$statistic,
      parameter = overall$df,
      p.value = overall$p.value,
      method = "Woolf-test on Homogeneity of Odds Ratios (no 4-way association)",
      data.name = DNAME,
      or_vars = or_vars,
      strata_vars = strata_vars,
      observed = overall$observed,
      expected = overall$expected,
      decomposed = TRUE,
      rows = list(
        statistic = rows$statistic,
        df = rows$df,
        p.value = rows$p.value,
        observed = rows$observed,
        expected = rows$expected
      ),
      cols = list(
        statistic = cols$statistic,
        df = cols$df,
        p.value = cols$p.value,
        observed = cols$observed,
        expected = cols$expected
      ),
      residual = list(
        statistic = residual_stat,
        df = residual_df,
        p.value = residual_pval
      )
    )

    class(result) <- c("woolf_test", "htest")
    return(result)
  }

  # Standard case: general k-way test
  # If more than 3 dimensions, reshape to collapse dimensions 3+ into one
  if (length(dims) > 3) {
    new_dim <- c(dims[1:2], prod(dims[3:length(dims)]))
    x <- array(x, dim = new_dim)
  }

  # Compute standard Woolf test
  k <- dim(x)[3]
  or <- apply(x, 3, function(x) (x[1,1] * x[2,2]) / (x[1,2] * x[2,1]))
  w <-  apply(x, 3, function(x) 1 / sum(1 / x))
  o <- log(or)
  e <- weighted.mean(log(or), w)
  STATISTIC <- sum(w * (o - e)^2)
  PARAMETER <- k - 1
  PVAL <- 1 - pchisq(STATISTIC, PARAMETER)

  # Set method description based on dimensionality
  if (length(dims) == 3) {
    METHOD <- "Woolf-test on Homogeneity of Odds Ratios (no 3-way association)"
  } else {
    METHOD <- paste0("Woolf-test on Homogeneity of Odds Ratios (no ",
                     length(dims), "-way association)")
  }

  names(STATISTIC) <- "X-squared"
  names(PARAMETER) <- "df"

  structure(list(statistic = STATISTIC,
                 parameter = PARAMETER,
                 p.value = PVAL,
                 method = METHOD,
                 data.name = DNAME,
                 or_vars = or_vars,
                 strata_vars = strata_vars,
                 observed = o,
                 expected = e,
                 decomposed = FALSE),
            class = c("woolf_test", "htest"))
}

#' Print method for woolf_test objects
#'
#' @param x An object of class \code{"woolf_test"}
#' @param ... Additional arguments (currently unused)
#' @rdname woolf_test
#' @export
print.woolf_test <- function(x, ...) {
  cat("\n")
  cat(x$method, "\n\n")

  # Display data information
  cat("Data:         ", x$data.name, "\n")
  cat("OR variables: ", paste(x$or_vars, collapse = ", "), "\n")
  cat("Strata:       ", paste(x$strata_vars, collapse = ", "), "\n\n")

  # Standard output (always shown)
  if (!x$decomposed) {
    # Simple case: just show overall test
    cat(sprintf("X-squared = %.4f, df = %d, p-value = %.4g\n",
                x$statistic, x$parameter, x$p.value))
  } else {
    # Decomposed case: show overall + decomposition
    cat("Overall homogeneity test:\n")
    cat(sprintf("  X-squared = %.4f, df = %d, p-value = %.4g\n\n",
                x$statistic, x$parameter, x$p.value))

    cat("Decomposition:\n")
    # Calculate label widths for alignment
    row_label <- sprintf("Rows (%s):", x$strata_vars[1])
    col_label <- sprintf("Cols (%s):", x$strata_vars[2])
    res_label <- "Residual:"
    max_width <- max(nchar(row_label), nchar(col_label), nchar(res_label))

    cat(sprintf("  %-*s X-squared = %.4f, df = %d, p-value = %.4g\n",
                max_width, row_label, x$rows$statistic, x$rows$df, x$rows$p.value))
    cat(sprintf("  %-*s X-squared = %.4f, df = %d, p-value = %.4g\n",
                max_width, col_label, x$cols$statistic, x$cols$df, x$cols$p.value))
    cat(sprintf("  %-*s X-squared = %.4f, df = %d, p-value = %.4g\n",
                max_width, res_label, x$residual$statistic, x$residual$df, x$residual$p.value))

    cat("\nNote: Overall = Rows + Columns + Residual\n")
  }

  invisible(x)
}

# Example usage:
#
# # For a 2 x 2 x k table (3-way):
# woolf_test(x)  # Standard test
#
# # For a 2 x 2 x R x C table (4-way):
# woolf_test(x)                  # Standard test (treats as R*C strata)
# woolf_test(x, decompose = TRUE) # Two-way decomposition
#
# # For a 2 x 2 x ... table (5+ dimensions):
# woolf_test(x)  # Standard test (collapses all dimensions beyond 2)

#' Simple and enhanced plot of MCA solutions
#'
#' This function is intended as an alternative to \code{\link[ca]{plot.mjca}}
#' for plotting multiple correspondence analysis solutions. It provides more
#' flexibility for labeling factor levels and connecting them with lines. It
#' does not support some features of `plot.mjca` (centroids, supplementary
#' points, arrows, etc.)
#'
#'
#' @param obj An `"mjca"` object
#' @param map Character string specifying the map type, i.e., the scaling
#'        applied to coordinates for different types of MCA representations. Allowed
#'        options include: `"symmetric"` (default), `"rowprincipal"`,
#'        `"colprincipal"`,
#'        `"symbiplot"`,
#'        `"rowgab"`,
#'        `"colgab"`,
#'        `"rowgreen"`,
#'        `"colgreen"`.
#'        See \code{\link[ca]{mjca}} for details.
#' @param dim Dimensions to plot, an integer vector of length 2
#' @param col Vector of colors, one for each factor in the MCA
#' @param pch Vector of point symbols for the category levels, one for each factor
#' @param cex Character size for points and level labels
#' @param pos Position of level labels relative to the category points; either
#'        a single number or a vector of length equal to the number of category points.
#' @param lines A logical or an integer vector indicating which factors are to
#'        be joined with lines using \code{\link[ca]{multilines}}
#' @param lwd Line width(s) for the lines
#' @param legend Logical; draw a legend for the factor names?
#' @param legend.pos Position of the legend in the plot, as in \code{\link[graphics]{legend}}
#' @param xlab,ylab Labels for horizontal and vertical axes.  The default,
#'        `"_auto_"` means that the function auto-generates a label of the form
#'        `"Dimension X (xx.x \%)"`
#' @param rev.axes A logical vector of length 2, where TRUE reverses the
#'        direction of the corresponding axis
#' @param \dots Arguments passed down to `plot`
#' @return Returns the coordinates of the category points invisibly
#'
#' @author Michael Friendly
#' @seealso \code{\link[ca]{mjca}}, \code{\link[ca]{plot.mjca}}
#'          \code{\link[ca]{cacoord}} returns CA and MCA coordinates,
#'          \code{\link[ca]{multilines}} draw multiple lines according to a factor,
#' @keywords hplot
#' @importFrom ca cacoord multilines
#' @examples
#'
#' require(ca)
#' data(Titanic)
#' titanic.mca <- suppressWarnings(mjca(Titanic))     # ca pkg uses old version of expand.dft()
#' mcaplot(titanic.mca, legend=TRUE, legend.pos="topleft")
#'
#' data(HairEyeColor)
#' haireye.mca <- suppressWarnings(mjca(HairEyeColor))
#' mcaplot(haireye.mca, legend=TRUE, cex.lab=1.3)
#'
#'
#'
#' @export mcaplot
mcaplot <- function(obj,
                    map="symmetric",
                    dim=1:2,
	col=c("blue", "red", "brown", "black", "green3", "purple"),
	pch=15:20, cex=1.2, pos=3,
	lines=TRUE, lwd=2,
	legend=FALSE, legend.pos="topright",
  xlab      = "_auto_",
  ylab      = "_auto_",
  rev.axes  = c(FALSE, FALSE),
	...) {

  if(!requireNamespace("ca", quietly=TRUE)) stop("The ca package is required")
	if(!inherits(obj, "mjca")) stop("Only defined for mjca objects")

	# nd=obj$nd ensures the number of coordinate dimensions matches obj$sv,
	# preventing a non-conformable error in rsc %*% diag(sv) inside cacoord()
	# when colcoord/rowcoord have more dimensions than the mjca sv vector.
	coords <- cacoord(obj, type=map, rows=FALSE, dim=seq_len(obj$nd))
	coords <- data.frame(coords, obj$factors)     # extract factor names & levels
	nlev <- obj$levels.n
	nfac <- length(nlev)
	pch <- rep_len(pch, nfac)
	col <- rep_len(col, nfac)
	lwd <- rep_len(lwd, nfac)
	rev.axes <- rep(rev.axes, length.out=2)

	if(any(dim > ncol(coords)))
      stop("dim must be valid dimensions of the coordinates")
	labs <- pctlab(obj)
	if (xlab == "_auto_") xlab <- labs[dim[1]]
	if (ylab == "_auto_") ylab <- labs[dim[2]]

  if(isTRUE(rev.axes[1])) coords[, dim[1]] <- -coords[, dim[1]]
  if(isTRUE(rev.axes[2])) coords[, dim[2]] <- -coords[, dim[2]]

	plot(coords[, dim], type='n', asp=1, xlab=xlab, ylab=ylab, ...)
	points(coords[,dim], pch=rep(pch, nlev), col=rep(col, nlev), cex=cex)
	text(coords[,dim], labels=coords$level, col=rep(col, nlev), pos=pos, cex=cex, xpd=TRUE)
	if (is.logical(lines)) lines <- if(lines) 1:nfac else NULL
	if(length(lines))
		multilines(coords[, dim], group=coords$factor, which=lines, col=col, lwd=lwd)
	abline(h = 0, v = 0, lty = "longdash", col="gray")

	if (legend) {
		factors <- coords$factor
		factors <- factors[!duplicated(factors)]
		legend(legend.pos, legend=factors,
			title="Factor", title.col="black",
			col=col, text.col=col, pch=pch,
			bg=rgb(.95, .95, .95, .3), cex=cex)
	}
	invisible(coords)
}

pctlab <- function(obj, prefix="Dimension ", decimals=1) {
  values <- obj$sv^2
  if (obj$lambda == "JCA"){
    pct <- rep_len(NA, length(values))
    } else {
	if (obj$lambda == "adjusted") {
      values <- obj$inertia.e
      pct <- round(100 * values, decimals)
      } else {
      pct <- round(100 * values / sum(values), decimals)
      }
    }
  pctval <- ifelse(is.na(pct), NULL, paste0(" (", pct, "%)"))
  paste0(prefix, 1:length(values), pctval)
}

---
title: "4. Mosaic plots"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{4. Mosaic plots}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tut04-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")


# Load packages
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
library(seriation)

data(HairEyeColor)
data(PreSex)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
#if(!file.exists("fig")) dir.create("fig")
```

Mosaic plots provide an ideal method both for visualizing contingency tables and for 
visualizing the fit--- or more importantly--- **lack of fit** of a loglinear model.

For a two-way table, `mosaic()`, by default, fits a model of independence, $[A][B]$
or `~A + B` as an R formula. The `vcdExtra` package extends this to models fit 
using `glm(..., family=poisson)`, which can include specialized models for
ordered factors, or square tables that are intermediate between the saturated model,
$[A B]$ = `A * B`, and the independence model $[A][B]$.

For $n$-way tables, `vcd::mosaic()` can fit any loglinear model, and can also be
used to plot a model fit with `MASS:loglm()`. The `vcdExtra` package extends this
to models fit using `stats::glm()` and, by extension, to non-linear models fit
using the [gnm package](https://cran.r-project.org/package=gnm).

See @vcd:Friendly:1994, @vcd:Friendly:1999 for the statistical ideas behind these
uses of mosaic displays in connection with loglinear models. Our book @FriendlyMeyer:2016:DDAR
gives a detailed discussion of mosaic plots and many more examples.

The essential ideas are to: 

* recursively sub-divide a unit square into rectangular "tiles" for the
cells of the table, such that the area of each tile is proportional to the cell frequency. Tiles are split in a sequential order: 

  + First according to the **marginal** proportions of a first variable, V1
  + Next according to the **conditional** proportions of a 2nd variable, V2 | V1
  + Next according to the **conditional** proportions of a 3rd variable, V3 | {V1, V2}
  + ...

* For a given loglinear model, the tiles can then be shaded in various ways to reflect
the residuals (lack of fit) for a given model.  

* The pattern of residuals can then
be used to suggest a better model or understand *where* a given model fits or
does not fit.

`mosaic()` provides a wide range of options for the directions of splitting,
the specification of shading, labeling, spacing, legend and many other details.
It is actually implemented as a special case of a more general
class of displays for $n$-way tables called `strucplot`, including
sieve diagrams, association plots, double-decker plots as well as mosaic
plots.  

For details, see `help(strucplot)` and the "See also" links therein,
and also @vcd:Meyer+Zeileis+Hornik:2006b, which is available as
an R vignette via `vignette("strucplot", package="vcd")`.

***Example***:
A mosaic plot for the Arthritis treatment data fits the model of independence,
`~ Treatment + Improved` and displays the association in the pattern of
residual shading. The goal is to visualize the difference in the proportions
of `Improved` for the two levels of `Treatment` : "Placebo" and "Treated".

The plot below is produced with the following call to `mosaic()`.
With the first split by `Treatment` and the shading used, it is easy to see
that more people given the placebo experienced no improvement, while more
people given the active treatment reported marked improvement.


```{r}
#| Arthritis1,
#| fig.height = 6,
#| fig.width = 7,
#| fig.cap = "Mosaic plot for the `Arthritis` data, using `shading_max`"
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
mosaic(art, gp = shading_max, 
            split_vertical = TRUE, 
            main="Arthritis: [Treatment] [Improved]")
```

`gp = shading_max` specifies that color in the plot signals a significant residual at a 90% or 99% significance level,
with the more intense shade for 99%.
Note that the residuals for the independence model were not large
(as shown in the legend),
yet the association between `Treatment` and `Improved`
is highly significant.
```{r, art1}
summary(art)
```

In contrast, one of the other shading schemes, from @vcd:Friendly:1994
(use: `gp = shading_Friendly`), 
uses fixed cutoffs of $\pm 2, \pm 4$,
to shade cells which are *individually* significant
at approximately $\alpha = 0.05$ and $\alpha = 0.001$ levels, respectively.
The plot below uses `gp = shading_Friendly`.

```{r}
#| Arthritis2,
#| fig.height = 6,
#| fig.width = 7,
#| fig.cap = "Mosaic plot for the `Arthritis` data, using `shading_Friendly`"
mosaic(art, gp = shading_Friendly, 
            split_vertical = TRUE, 
            main="Arthritis: gp = shading_Friendly")
```

## Permuting variable levels

Mosaic plots using tables or frequency data frames as input typically take the levels of the
table variables in the order presented in the dataset.  For character variables, this is often
alphabetical order. That might be helpful for looking up a value, but is unhelpful for seeing
and understanding the pattern of association.

It is usually much better to order the levels of the row and column variables to help reveal
the nature of their association. This is an example of **effect ordering for data display**
[@FriendlyKwan:02:effect].

***Example***:

Data from @Glass:54 gave this 5 x 5 table on the occupations of 3500 British fathers and their sons, where the occupational categories are listed in alphabetic order.

```{r glass}
data(Glass, package="vcdExtra")
(glass.tab <- xtabs(Freq ~ father + son, data=Glass))
```

The mosaic display shows very strong association, but aside from the 
diagonal cells, the pattern is unclear. Note the use of
`set_varnames` to give more descriptive labels for the variables
and abbreviate the occupational category labels.
and `interpolate` to set the shading levels for the mosaic.
```{r glass-mosaic1}
largs <- list(set_varnames=list(father="Father's Occupation", 
                                son="Son's Occupation"),
              abbreviate=10)
gargs <- list(interpolate=c(1,2,4,8))

mosaic(glass.tab, 
  shade=TRUE, 
  labeling_args=largs, 
  gp_args=gargs,
  main="Alphabetic order", 
  legend=FALSE, 
  rot_labels=c(20,90,0,70))
```

The occupational categories differ in **status**, and can be reordered correctly as follows,
from `Professional` down to `Unskilled`.
```{r glass-order}
# reorder by status
ord <- c(2, 1, 4, 3, 5) 
row.names(glass.tab)[ord]
```
The revised mosaic plot can be produced by indexing the rows and columns of
the table using `ord`.
```{r glass-mosaic2}
mosaic(glass.tab[ord, ord], 
  shade=TRUE, 
  labeling_args=largs,  
  gp_args=gargs,
  main="Effect order", 
  legend=FALSE, 
  rot_labels=c(20,90,0,70))
```

From this, and for the examples in the next section, it is useful to re-define
`father` and `son` as **ordered** factors in the original `Glass` frequency data.frame.

```{r glass-ord}
Glass.ord <- Glass
Glass.ord$father <- ordered(Glass.ord$father, levels=levels(Glass$father)[ord])
Glass.ord$son    <- ordered(Glass.ord$son,    levels=levels(Glass$son)[ord])
str(Glass.ord)
```

## Square tables

For mobility tables such as this, where the rows and columns refer to the same
occupational categories it comes as no surprise that there is a strong association
in the diagonal cells: most often, sons remain in the same occupational categories
as their fathers.

However, the re-ordered mosaic display also reveals something subtler:
when a son differs in occupation from the father, it is more likely that
he will appear in a category one-step removed than more steps removed.
The residuals seem to decrease with the number of steps from the diagonal.

For such tables, specialized loglinear models provide interesting cases
intermediate between the independence model, [A] [B],
and the saturated model, [A B]. These can be fit using `glm()`, with the data
in frequency form,

```
glm(Freq ~ A + B + assoc, data = ..., family = poisson)
```

where `assoc` is a special term to handle a restricted form of association, different from
`A:B` which specifies the saturated model in this notation.

* **Quasi-independence**: Asserts independence, but ignores the diagonal cells by
fitting them exactly.  The loglinear model is:
$\log m_{ij} = \mu + \lambda^A_i + \lambda^B_j + \delta_i I(i = j)$,
where $I()$ is the indicator function. 

* **Symmetry**: This model asserts that the joint distribution of the row and column variables
is symmetric, that is $\pi_{ij} = \pi_{ji}$: 
A son is equally likely to move from their father's occupational category $i$ to another
category, $j$, as the reverse, moving from $j$ to $i$.
Symmetry is quite strong, because it also implies
**marginal homogeneity**, that the marginal probabilities of the row and column variables
are equal,
$\pi{i+} = \sum_j \pi_{ij} = \sum_j \pi_{ji} = \pi_{+i}$ for all $i$.

* **Quasi-symmetry**: This model uses the standard main-effect terms in the loglinear
model, but asserts that the association parameters are symmetric,
$\log m_{ij} = \mu + \lambda^A_i + \lambda^B_j + \lambda^{AB}_{ij}$,
where $\lambda^{AB}_{ij} = \lambda^{AB}_{ji}$.

The [gnm package](https://cran.r-project.org/package=gnm) provides a variety of these functions:
`gnm::Diag()`, `gnm::Symm()` and `gnm::Topo()` for an  interaction factor as specified by an array of levels, which may be arbitrarily structured.

For example, the following generates a term for a diagonal factor in a $4 \times 4$
table. The diagonal values reflect parameters fitted for each diagonal cell. Off-diagonal
values, "." are ignored.

```{r diag}
rowfac <- gl(4, 4, 16)
colfac <- gl(4, 1, 16)
diag4by4 <- Diag(rowfac, colfac)
matrix(Diag(rowfac, colfac, binary = FALSE), 4, 4)
```

`Symm()` constructs parameters for symmetric cells. The particular values don't matter.
All that does matter is that the same value, e.g., `1:2` appears in both the (1,2) and
(2,1) cells. 
```{r symm}
symm4by4 <- Symm(rowfac, colfac)
matrix(symm4by4, 4, 4)
```



***Example***:
To illustrate, we fit the four models below, starting with the independence model
`Freq ~ father + son` and then adding terms to reflect the restricted
forms of association, e.g., `Diag(father, son)` for diagonal terms and
`Symm(father, son)` for symmetry.

```{r glass-models}
library(gnm)
glass.indep <- glm(Freq ~ father + son, 
                   data = Glass.ord, family=poisson)
glass.quasi <- glm(Freq ~ father + son + Diag(father, son),  
                   data = Glass.ord, family=poisson)
glass.symm  <- glm(Freq ~ Symm(father, son),  
                   data = Glass.ord, family=poisson)
glass.qsymm <- glm(Freq ~ father + son + Symm(father, son),  
                   data = Glass.ord, family=poisson)
```

We can visualize these using the `vcdExtra::mosaic.glm()` method, which extends
mosaic displays to handle fitted `glm` objects. *Technical note*: for
models fitted using `glm()`, standardized residuals, `residuals_type="rstandard"`
have better statistical properties than the default Pearson residuals in mosaic
plots and analysis.

```{r glass-quasi}
mosaic(glass.quasi, 
  residuals_type="rstandard", 
  shade=TRUE, 
  labeling_args=largs,  
  gp_args=gargs,
  main="Quasi-Independence",
  legend=FALSE, 
  rot_labels=c(20,90,0,70)
  )
```

Mosaic plots for the other models would give further visual assessment of these models,
however we can also test differences among them. For nested models, `anova()` gives tests
of how much better a more complex model is compared to the previous one.

```{r glass-anova}
# model comparisons: for *nested* models
anova(glass.indep, glass.quasi, glass.qsymm, test="Chisq")
```

Alternatively, `vcdExtra::LRstats()` gives model summaries for a collection of
models, not necessarily nested, with AIC and BIC statistics reflecting
model parsimony.

```{r glass-lrstats}
models <- glmlist(glass.indep, glass.quasi, glass.symm, glass.qsymm)
LRstats(models)
```

By all criteria, the model of quasi symmetry fits best. The residual deviance $G^2
is not significant. The mosaic is largely unshaded, indicating a good fit, but there
are a few shaded cells that indicate the remaining positive and negative residuals.
For comparative mosaic displays, it is sometimes useful to show the $G^2$ statistic
in the main title, using `vcdExtra::modFit()` for this purpose.

```{r glass-qsymm}
mosaic(glass.qsymm, 
  residuals_type="rstandard", 
  shade=TRUE, 
  labeling_args=largs,  
  gp_args=gargs,
  main = paste("Quasi-Symmetry", modFit(glass.qsymm)),
  legend=FALSE, 
  rot_labels=c(20,90,0,70)
  )
```



## Correspondence analysis ordering
When natural orders for row and column levels are not given a priori, we can find
orderings that make more sense using correspondence analysis.

The general ideas are that:

* Correspondence analysis assigns scores to the row and column variables to best account for the association in 1, 2, ... dimensions

* The first CA dimension accounts for largest proportion of the Pearson $\chi^2$

* Therefore, permuting the levels of the row and column variables by the CA Dim1 scores gives a more coherent mosaic plot, more clearly showing the nature of the association.

* The [seriation package](https://cran.r-project.org/package=seriation) now has a method to order variables in frequency tables using CA.

***Example***:
As an example, consider the `HouseTasks` dataset, a 13 x 4 table of frequencies of household tasks performed by couples, either by the `Husband`, `Wife`, `Alternating` or `Jointly`.
You can see from the table that some tasks (Repairs) are done largely by the husband; some (laundry, main meal)
are largely done by the wife, while others are done jointly or alternating between husband and
wife. But the `Task` and `Who` levels are both in alphabetical order.

```{r housetasks}
data("HouseTasks", package = "vcdExtra")
HouseTasks
```

The naive mosaic plot for this dataset is shown below, splitting first by `Task` and then by `Who`. Due to the length of the factor labels, some features of `labeling` were used to make the display more readable.

```{r housetasks-mos1}
require(vcd)
mosaic(HouseTasks, shade = TRUE,
       labeling = labeling_border(rot_labels = c(45,0, 0, 0), 
                                  offset_label =c(.5,5,0, 0),
                                  varnames = c(FALSE, TRUE),
                                  just_labels=c("center","right"),
                                  tl_varnames = FALSE),
       legend = FALSE)

```

Correspondence analysis, using the [ca package](https://cran.r-project.org/package=ca),
shows that nearly 89% of the $\chi^2$ can be accounted for in two dimensions.

```{r housetasks-ca}
require(ca)
HT.ca <- ca(HouseTasks)
summary(HT.ca, rows=FALSE, columns=FALSE)
```

The CA plot has a fairly simple interpretation: Dim1 is largely the distinction between
tasks primarily done by the wife vs. the husband. Dim2 distinguishes tasks that are done
singly vs. those that are done jointly.
```{r housetasks-ca-plot}
plot(HT.ca, lines = TRUE)
```

So, we can use the `CA` method of `seriation::seriate()` to find the order of permutations of
`Task` and `Who` along the CA dimensions.
```{r housetasks-seriation}
require(seriation)
order <- seriate(HouseTasks, method = "CA")
# the permuted row and column labels
rownames(HouseTasks)[order[[1]]]
colnames(HouseTasks)[order[[2]]]
```

Now, use `seriation::permute()` to use `order` for the permutations of `Task` and `Who`,
and plot the resulting mosaic:

```{r housetasks-mos2}
# do the permutation
HT_perm <- permute(HouseTasks, order, margin=1)

mosaic(HT_perm, shade = TRUE,
       labeling = labeling_border(rot_labels = c(45,0, 0, 0), 
                                  offset_label =c(.5,5,0, 0),
                                  varnames = c(FALSE, TRUE),
                                  just_labels=c("center","right"),
                                  tl_varnames = FALSE),
       legend = FALSE)
```

It is now easy to see the cluster of tasks (laundry and cooking) done largely by the wife
at the top, and those (repairs, driving) done largely by the husband at the bottom.

## References

---
title: "1a. Steps Toward Tidy Categorical Data Analysis"
subtitle: "May the Forms Be with You: Novel Functions to Intuitively Convert Among Forms and Collapse Variable Levels Presented Using the `starwars` Data."
author: "Gavin M. Klorfine"
output: rmarkdown::html_vignette
package: vcdExtra
vignette: >
  %\VignetteIndexEntry{1a. Steps Toward Tidy Categorical Data Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  dev = "png",
  comment = "##"
)

library(vcdExtra)
library(dplyr)
library(tidyr)
```

<p>
<img align="right" src="fig/formhex.png" height="200">
</p>

# Overview

While R provides many intuitive facilities for the manipulation of continuous
variables (such as those in the 
[`tidyverse`](https://CRAN.R-project.org/package=tidyverse) collection of 
packages), it somewhat lacks the equivalent for categorical data. Two such areas
include the collapsing of variable levels (e.g., combining hair 
colours of "Brown" and "Black" into a "Dark" category) and the conversion
between forms of categorical data (e.g., from a `table` of entries to a 
`data.frame` containing frequencies for each combination of variable levels).

## Tidy Collapsing

In R, when trying to collapse levels of a variable in a dataset (e.g., combining 
hair colours of "Brown" and "Black" into a "Dark" category), it was often the 
case that one would need to first convert amongst forms, "collapse" their data, aggregate the duplicate rows, and finally convert back to the initial form.

`collapse_levels()` simplifies this process, allowing for the intuitive 
collapsing of variable levels for datasets of any form. One just needs to ensure 
that an argument of `freq = "the frequency column name"` is supplied when the
inputted dataset is in frequency form.

Functionality of `collapse_levels()` is demonstrated below 
using the `starwars` data from the 
[`dplyr`](https://CRAN.R-project.org/package=dplyr) package. This dataset
contains case form data on various characters in the Star Wars franchise.
Variables considered in this vignette are a character's `hair_color`, 
`skin_color`, and `eye_color`. Taken as is, this would correspond to an 
$11 \times 28 \times 15$ contingency table... Time to collapse!

Here I load the `starwars` data and select the variables of interest. For
simplicity, I then remove rows containing `NA` values.

```{r overoll_loadselect}
data("starwars", package = "dplyr")

star_case <- starwars |>
  dplyr::select(c("hair_color", "skin_color", "eye_color")) |> 
  tidyr::drop_na()

str(star_case)
```

First, taking a look at the levels of variable `hair_color`, there are many
ways one might want to collapse these categories:

```{r overcoll_hairunique}
unique(star_case$hair_color)
```

***Example***:
Likely the most natural of these ways is the following:

1. Collapse different spellings of `"blond"` (i.e., `"blond"` and `"blonde"` become `Blonde`).
1. Collapse different shades of `"brown"` (i.e., `"brown"` and `"brown, grey"` become `Brown`).
1. Collapse different shades of `"auburn"` (i.e., `"auburn, white"`, `"auburn, grey"`, and `"auburn"` become `Auburn`).
1. Keep `"none"` as-is.
1. Keep `"white"` as-is.
1. Keep `"grey"` as-is.
1. Keep `"Black"` as-is.

Here is how to do this using `collapse_levels()`:

```{r overcoll_ex1}
collapsed.star_case <- collapse_levels(
  star_case,             # The dataset
  hair_color = list(     # Assign the variable to be collapsed to a list
    
    # Format the list as NewLevel = c("old1", "old2", ..., "oldn")
    Blonde = c("blond", "blonde"), 
    Brown = c("brown", "brown, grey"),
    Auburn = c("auburn, white", "auburn, grey", "auburn")
  )
)
str(collapsed.star_case)
unique(collapsed.star_case$hair_color)
```

Second, one might also want to collapse levels of variable `skin_color`:

```{r overcoll_skinunique}
unique(star_case$skin_color)
```

***Example***:
I decided to arbitrarily collapse these as follows:

1. Keep `"none"` as-is. 
1. Keep`"unknown"` as-is.
1. `Shades`, comprising all levels that begin with `"white"`, `"grey"`, `"dark"`, `"light"`, and `"fair"`.
1. `Rainbows`, comprising all other levels.

Note that when working with real data, arbitrary decisions involving the
collapsing of variable levels are a *VERY* bad idea. Collapses should be
grounded in strong, data-driven justification. Arbitrary collapsing is employed
in this vignette purely for pedagogical and illustrative purposes.

```{r overcoll_ex2}
collapsed.star_case <- collapse_levels(
  collapsed.star_case,
  skin_color = list(
    Shades = c(
      "fair", "white", "light", "dark", "grey", "grey, red", 
      "grey, blue", "white, blue", "grey, green, yellow", "fair, green, yellow"
    ), 
    Rainbows = c(
      "green", "pale", "metal", "brown mottle", "brown", "mottled green", 
      "orange", "blue, grey", "red", "blue", "yellow", "tan", "silver, red",
      "green, grey", "red, blue, white", "brown, white"
    )
  )
)
str(collapsed.star_case)
unique(collapsed.star_case$skin_color)
```

Third, one may also want to collapse levels of variable `eye_color`:

```{r overcoll_eyeunique}
unique(star_case$eye_color)
```

***Example***:
Again, I decided to arbitrarily collapse these as follows:

1. `Normal`, with levels of typical human eye color (i.e., `"blue"`, `"blue-gray"`, `"brown"`, `"hazel"`, and `"dark"`).
1. `Abnormal`, with levels of eye colours that would be abnormal for humans (e.g., `"red"`, `"pink"`, `"gold"`, etc.).
1. Keep `unknown` as-is.

```{r overcoll_ex3}
collapsed.star_case <- collapse_levels(
  collapsed.star_case,
  eye_color = list(
    Normal = c("blue", "brown", "blue-gray", "hazel", "dark"), 
    Abnormal = c(
      "yellow", "red", "orange", "black", "pink", "red, blue", "gold", 
      "green, yellow", "white"
    )
  )
)
str(collapsed.star_case)
unique(collapsed.star_case$eye_color)
```

In addition, one may want (and is able) to collapse levels of multiple variables
in a single call to `collapse_levels()`.

***Example***:
To illustrate this (and to provide an easy working example for the following 
"Tidy Conversions" section), the `collapsed.star_case` data is arbitrarily 
collapsed as follows to correspond to a $3 \times 3 \times 3$ contingency table:

1. Variable `hair_color`:
    a. `Dark` corresponding to levels `"Brown"`, `"black"`, and `"Auburn"`.
    b. `Light` corresponding to levels `"Blonde"`, `"white"`, and `"grey"`.
    c. Keep `"none"` as-is.
1. Variable `skin_color`:
    a. `Other` corresponding to levels `"none"` and `"unknown"`.
    b. Keep `Rainbows` as-is.
    c. Keep `Shades` as-is.
1. Variable `eye_color` kept as-is.

```{r overcoll_ex4}
collapsed.star_case <- collapse_levels(
  collapsed.star_case,
  hair_color = list(    # First variable
    Dark = c("Brown", "black", "Auburn"),
    Light = c("Blonde", "white", "grey")
  ),
  skin_color = list(    # Second variable
    Other = c("none", "unknown")
  )
)
unique(collapsed.star_case$hair_color)
unique(collapsed.star_case$skin_color)
str(collapsed.star_case)
```

## Tidy Conversions

Until now, converting amongst forms of categorical data in R has been somewhat
onerous. As outlined in 
[1. Creating and manipulating frequency tables]( a1-creating.html), 
the below table shows the typical process for converting among forms 
(`A`, `B`, and `C`  represent categorical variables, `X` represents an R data 
object):

| **From this**    |                     | **To this**          |                   |
|:-----------------|:--------------------|:---------------------|-------------------|
|	                 |    _Case form_      | _Frequency form_     | _Table form_      |
|	_Case form_      |   noop              | `xtabs(~A+B)`        |  `table(A,B)`     |
|	_Frequency form_ |  `expand.dft(X)`    | noop                 | `xtabs(count~A+B)`|
|	_Table form_     |  `expand.dft(X)`    | `as.data.frame(X)`   |  noop             |

Instead, one may simply use `as_table(X)` to convert to table form, 
`as_freqform(X)` to convert to frequency form, and `as_caseform(X)` to convert 
to case form. These are illustrated in the network (node/edge) diagram below:

<p>
<img align="center" src="fig/convnetwork.png" height="400">
</p>

Additionally, there are functions `as_array(X)` and `as_matrix(X)`
for converting to those respective types.

Like `collapse_levels()`, the single thing to keep in mind when employing these functions is the following:
when your object `X` is in frequency form, an argument of 
`freq = "your frequency column name"` must be supplied. Besides this, the rote
memory work of having to remember which function to use to convert form X to
form Y is now completely removed.

Functionality of these "tidy" conversion functions are demonstrated below
using the `collapsed.star_case` data from the most recent example (i.e., the 
data corresponding to a $3 \times 3 \times 3$ contingency table).

***Example***:
Convert the `collapsed.star_case` data into frequency form. Name this data
`star_freqform`.

```{r overconv-ex1}
star_freqform <- as_freqform(collapsed.star_case)

str(star_freqform)
```

Note that if one would like a data frame instead of a tibble, an argument of
`tidy = FALSE` needs to be provided. Naturally, this `tidy` argument is present 
only in functions `as_freqform()` and `as_caseform()`.

***Example***:
Convert the `collapsed.star_case` data into a data frame in frequency form.

```{r overconv-ex2}
as_freqform(collapsed.star_case, tidy = FALSE) |> str()
```

***Example***:
Convert the frequency form data, `star_freqform`, into table form. Name this
data `star_tab`. Because we are converting *from* frequency form, the
`freq = "frequency column name"` argument must be supplied.

```{r overconv-ex3}
star_tab <- as_table(star_freqform, freq = "Freq")

str(star_tab)
```

***Example***:
Convert the table form data, `star_tab`, into an array. Name this
data `star_array`.

```{r overconv-ex4}
star_array <- as_array(star_tab)

class(star_array)
str(star_array)
```

To convert to a matrix, one also needs to specify row and column dimensions.
This is done using the `dims = c("dim1", "dim2", ..., "dim_n")` argument, which
works by summing over the dimensions excluded from this call. The first provided
dimension is taken as the row dimension, with the second dimension taken as the 
column dimension.

***Example***:
Convert the array form data, `star_array`, into a matrix with dimensions
`"hair_color"` and `"eye_color"`. Name this data `star_mat`.

```{r overconv-ex5}
star_mat <- as_matrix(star_array, dims = c("hair_color", "eye_color"))

class(star_mat)
str(star_mat)
```

Note that the `dims` argument works the same way for all other tidy conversion
functions.

***Example***:
Convert the table form data, `star_tab`, into frequency form with dimensions
`"hair_color"` and `"eye_color"`.

```{r overconv-ex6}
as_freqform(star_tab, dims = c("hair_color", "eye_color")) |> str()
```

#### Proportions

The last piece of these conversion functions is the `prop` argument, allowing
users to convert cells/frequencies to proportions. Calculated proportions may 
either be relative to the grand total (`prop = TRUE`) or to one or more margins
(`prop = c("margin1", "margin2", ... "margin_n")`). 

Note that `as_caseform()` is the only of the tidy conversion functions to not 
include a `prop` argument. Also, `as_caseform()` will not convert proportional
data.^[This was a deliberate choice, as once proportions are relative to
margins, it becomes unclear how to convert these proportions back to
the original entries.]

***Example***:
Convert `star_mat` into a table of proportions that are relative to the grand 
total.

```{r propconv-ex1}
star_mat # To view the original

as_table(star_mat, prop = TRUE)
```

***Example***:
Convert `star_mat` into a table of proportions that are relative to the marginal
sums of `hair_color`.

```{r propconv-ex2}
as_table(star_mat, prop = "hair_color")
```

***Example***:
Convert `star_mat` into a table of proportions that are relative to the marginal
sums of both `hair_color` and `eye_color`. Since these are the only two
dimensions, cell proportions will all be equal to $1.0$ (except for cells where
no data exists).

```{r propconv-ex3}
as_table(star_mat, prop = c("hair_color", "eye_color"))
```

# Taken Together

Taking `collapse_levels()` and the tidy conversion functions together, one now
has an intuitive framework for manipulating categorical data.

***Example***:
The `starwars` data also has a variable named `homeworld`, specifying the planet
that a given character was from. The below code does the following: 

1. Create data `home_star` from dataset `starwars`. The new data includes both `homeworld` and the previous variables of interest (`hair_color`, `eye_color`, and `skin_color`). Missing values are then omitted.
1. Sort `homeworld` alphabetically.
1. Collapse the first half of the sorted `homeworld`s into a level named `abc`.
1. Collapse the second half of the sorted `homeworld`s into a level named `xyz`.
1. Collapse `eye_color` according to the previous `Abnormal`, `Normal`, and `"unknown"` conventions.
1. Convert the collapsed data into a table with dimensions `homeworld` and 
`eye_color`. Call this table `tab.home_star` and plot the result in a mosaic 
display.
1. Convert `tab.home_star` into a matrix of proportions (relative to the grand total).

```{r tt-ex1}
home_star <- starwars |>
  dplyr::select(c("hair_color", "skin_color", "eye_color", "homeworld")) |> 
  tidyr::drop_na()

# Sort unique levels of homeworld
lvls <- home_star$homeworld |> unique() |> sort()
lvls

# Collapse variable levels
collapsed.home_star <- collapse_levels(
  home_star,
  homeworld = list(
    abc = lvls[1:(length(lvls)/2)],
    xyz = lvls[(length(lvls)/2 + 1):length(lvls)]
  ),
  eye_color = list(
    Normal = c("blue", "brown", "blue-gray", "hazel", "dark"), 
    Abnormal = c(
      "yellow", "red", "orange", "black", "pink", "red, blue", "gold", 
      "green, yellow", "white"
    )
  )
)
# Convert to table of dimensions 'homeworld' and 'eye_color'
tab.home_star <- as_table(collapsed.home_star, dims = c("homeworld", "eye_color"))

# Plot as mosaic display
mosaic(tab.home_star, shading = TRUE, gp = shading_Friendly)

# Convert table into matrix of proportions. Note argument 'dims' was not supplied
# as we already know that there are exactly 2 dimensions.
as_matrix(tab.home_star, prop = TRUE)
```

Thus, this constitutes a pipeline for working with categorical data:

1. Gather data and clean it.
1. Collapse levels when substantively necessary.
1. Convert forms, select dimensions, and/or take proportions if necessary.

```{r ttpipeline, eval=FALSE}
dataset |>                             # Gather the data
  select(...) |> drop_na() |> ... |>   # Clean the data
  collapse_levels(...) |>              # Collapse levels as necessary
  as_form(...)   # Convert forms, select dimensions, take proportions
```

When viewed this way, these functions appear to be the start of a grammar of
categorical data analysis.

---
title: "Datasets for categorical data analysis"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Datasets for categorical data analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/datasets-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")
```

The `vcdExtra` package contains `r nrow(vcdExtra::datasets("vcdExtra"))` datasets, taken
from the literature on categorical data analysis, and selected to illustrate various
methods of analysis and data display. These are in addition to the 
`r nrow(vcdExtra::datasets("vcd"))` datasets in the [vcd package](https://cran.r-project.org/package=vcd).

To make it easier to find those which illustrate a particular method, the datasets in `vcdExtra` have been classified
using method tags. This vignette creates an "inverse table", listing the datasets that apply to each method.
It also illustrates a general method for classifying datasets in R packages.

```{r load}
library(dplyr)
library(tidyr)
library(readxl)
```

## Processing tags

Using the result of `vcdExtra::datasets(package="vcdExtra")` I created a spreadsheet, `vcdExtra-datasets.xlsx`,
and then added method tags.
```{r read-datasets}
dsets_tagged <- read_excel(here::here("inst", "extdata", "vcdExtra-datasets.xlsx"), 
                           sheet="vcdExtra-datasets")

dsets_tagged <- dsets_tagged |>
  dplyr::select(-Title, -dim) |>
  dplyr::rename(dataset = Item)

head(dsets_tagged)
```


To invert the table, need to split tags into separate observations, then collapse the rows for the same tag.

```{r split-tags}
dset_split <- dsets_tagged |>
  tidyr::separate_longer_delim(tags, delim = ";") |>
  dplyr::mutate(tag = stringr::str_trim(tags)) |>
  dplyr::select(-tags)

#' ## collapse the rows for the same tag
tag_dset <- dset_split |>
  arrange(tag) |>
  dplyr::group_by(tag) |>
  dplyr::summarise(datasets = paste(dataset, collapse = "; ")) |> ungroup()

# get a list of the unique tags
unique(tag_dset$tag)
```

## Make this into a nice table

Another sheet in the spreadsheet gives a more descriptive `topic` for corresponding to each tag.

```{r read-tags}
tags <- read_excel(here::here("inst", "extdata", "vcdExtra-datasets.xlsx"), 
                   sheet="tags")
head(tags)
```

Now, join this with the `tag_dset` created above.

```{r join-tags}
tag_dset <- tag_dset |>
  dplyr::left_join(tags, by = "tag") |>
  dplyr::relocate(topic, .after = tag)

tag_dset |>
  dplyr::select(-tag) |>
  head()
```

### Add links to `help()`

We're almost there. It would be nice if the dataset names could be linked to their documentation.
This function is designed to work with the `pkgdown` site. There are different ways this can be
done, but what seems to work is a link to `../reference/{dataset}.html`
Unfortunately, this won't work in the actual vignette.

```{r add-links}
add_links <- function(dsets, 
                      style = c("reference", "help", "rdrr.io"),
                      sep = "; ") {

  style <- match.arg(style)
  names <- stringr::str_split_1(dsets, sep)

  names <- dplyr::case_when(
    style == "help"      ~ glue::glue("[{names}](help({names}))"),
    style == "reference" ~ glue::glue("[{names}](../reference/{names}.html)"),
    style == "rdrr.io"   ~ glue::glue("[{names}](https://rdrr.io/cran/vcdExtra/man/{names}.html)")
  )  
  glue::glue_collapse(names, sep = sep)
}

```

## Make the table {#table}

Use `purrr::map()` to apply `add_links()` to all the datasets for each tag.
(`mutate(datasets = add_links(datasets))` by itself doesn't work.)

```{r kable}
tag_dset |>
  dplyr::select(-tag) |>
  dplyr::mutate(datasets = purrr::map(datasets, add_links)) |>
  knitr::kable()
```

Voila!

---
title: "tidyCat: Tidy Methods For Categorical Data Analysis"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{tidyCat: Tidy Methods For Categorical Data Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tidycat-",
  dev = "png",
  comment = "##"
)
```


<center>
<img src="fig/tidyCat-logo.png" height=160>
</center>

> **Note**: This vignette was an initial statement of the problem of "tidy" methods for
> categorical data analysis. Much of this has now been accomplished by the work of Gavin
> Klorfine. It is described in the vignette,
> [Steps Toward Tidy Categorical Data Analysis](https://friendly.github.io/vcdExtra/articles/a1a-convert-collapse.html).

## Frequency tables need some Tidy Love ❤️

Tidy methods for quantitative data & models have advanced considerably, but there hasn't been much
development of similar ideas for "categorical data", by which I mean data that is often compactly represented
as $n$-way frequency tables, cross classified by one or more discrete factors. 

What would it take to implement a tidy framework for such data? These notes are, in effect, a call for 
participation in developing a `tidyCat` package for this purpose. Other possible names for this:
`tidyCDA`, `tidyfreq` but `tidyCat` makes for a nice logo!

```{r load-pkgs}
library(MASS)
library(vcdExtra)
```

I see three areas that could be developed here:

## Constructing categorical data sets 
Current non-tidy data forms and operations, following [@FriendlyMeyer:2016:DDAR] are described in
the vignette [Creating and manipulating frequency tables](https://friendly.github.io/vcdExtra/articles/a1-creating.html)

It seems clear that the most flexible and general form, and one that most closely matches a tidy
data frame is **case form**, because this allows for numeric variables as well. Thus:

> Among these, tidy categorical data should best be represented as a `tibble` in **case form**. A tibble is convenient for its' printing.

### Manipulating categorical data sets

The methods `xtabs()`, `table()` and `expand.dft()` described in that vignette allow conversion from one form to another.
Tidy equivalents might be:

* `as_table()`, `as_matrix()`, `as_array()` to convert from any form to table/array form

* Similarly, perhaps `as_caseform()`, `as_freqform()` to convert to those. There is already `as.data.frame(table)` to convert to frequency form, and `expand.dft()` converts that to case form

```{r}
data("HairEyeColor")
hec.df <- as.data.frame(HairEyeColor)
head(hec.df)
# expand to case form
expand.dft(hec.df) |> head()
```


* `vcd::structable()` (and `stats::ftable()`) produce a 'flat' representation of a high-dimensional contingency table constructed by recursive splits (similar to the construction of mosaic displays). One can be constructed from a table or from a data frame with
a formula method,

```{r}
structable(Titanic)
structable(Sex + Class ~ Survived + Age, data = Titanic)
```

and there are a suite of methods for indexing and selecting parts of an $n$-way table.

* The methods in the `plyr` package (now retired) provided a coherent set of tools for a split-apply-combine
strategy that works nicely with multidimensional arrays. Perhaps there are some useful ideas for frequency tables
that could be resurrected here.

* There is also a role for `purrr` methods and thinking here: $n$-way tables as nested lists/arrays?
The ideas of mapping over these?

## Manipulating factor levels 

Also needed: 

* methods for **recoding and collapsing** the levels of a factor: `forcats::fct_recode()`, `forcats::fct_collapse()`,  `forcats::fct_lump_min()` are useful here.

* methods for **reordering the levels** of a factor, either manually or for some analysis purpose. For example, Data from @Glass:54 gave this 5 x 5 table on the occupations of 3500 British fathers and their sons, where the occupational categories are listed in alphabetic order.
  
```{r glass}
data(Glass, package="vcdExtra")
str(Glass)
(glass.tab <- xtabs(Freq ~ father + son, data=Glass))
```

This can be reordered manually by indexing, to arrange the categories by **status**, giving an order  `Professional` down to `Unskilled`:

```{r glass-order}
# reorder by status
ord <- c(2, 1, 4, 3, 5) 
glass.tab[ord, ord]
```

A more general method is to permute the row and column categories in the order implied by correspondence analysis dimensions. This is implemented in the [`seriation` package](https://cran.r-project.org/package=seriation) using the `CA` method of `seriation::seriate()` and applying `permute()` to the result.

```{r housetasks-seriation}
library(seriation)
order <- seriate(glass.tab, method = "CA")
# the permuted row and column labels
rownames(glass.tab)[order[[1]]]

# reorder rows and columns
permute(glass.tab, order)
```
What are tidy ways to do these things?


## Models

The standard analysis of frequency data is in the form of loglinear models fit by `MASS::loglm()`
or with more flexible versions fit with `glm()` or `gnm::gnm()`. These are essentially linear models for the log
of frequency. In `vcdExtra`, there are several methods for a list of such models, of class `"glmlist"` and `"loglmlist"`,
and these should be accommodated in tidy methods. 

What is needed are `broom` methods for `loglm` models. The information required is accessible from standard functions, but not in a tidy form.

* `glance.loglm()` -- **model level** statistics. These are given in the output of the `print()` method, and available from the `print()` method. A complication is both LR and Pearson $\chi^2$ are reported, so these would need to be made to appear in separate columns. There are also related `LRstats()` functions in `vcdExtra`, which report `AIC` and `BIC`.

```{r}
hec.indep <- loglm(~Hair+Eye+Sex, data=HairEyeColor)
hec.indep
# extract test statistics
summary(hec.indep)$tests
LRstats(hec.indep)
```

* `tidy.loglm()` --- **coefficient level** statistics. These are available from `coef.loglm()`. They would need to assembled into a long format. Standard errors & p-values might be a problem.

```{r}
coef(hec.indep)
```

* `augment.loglm()` --- should give **case level** statistics: fitted values, residuals, ...

```{r}
fitted(hec.indep)
residuals(hec.indep)
```

What about `hatvalues`? Not implemented, but shouldn't be too hard.

```{r error=TRUE}
hatvalues(hec.indep)
```


## Graphical methods

The most common graphical methods are those implemented in `vcd`: `mosaic()` association plots (`assoc()`), ..., which
rely on `vcd::strucplot()` described in [@vcd:Meyer+Zeileis+Hornik:2006b].

Is there a tidy analog that might work with `ggplot2`? The [`ggmosaic` package](https://github.com/haleyjeppson/ggmosaic) implements basic marimeko-style mosaic plots. They are not very general, in that they cannot do residual-based shading to show the patterns of association.

However, they are based on a [productplots](https://github.com/hadley/productplots) package by Hadley, which seems to
provide some basic structure for constructing such displays of nested rectangles.

## Some tidy stuff

* The [infer package](https://infer.netlify.app/) has some interesting stuff for CDA. 
The vignette [Tidy Chi-Squared Tests with infer](https://infer.netlify.app/articles/chi_squared)
expresses do a Chisq test of independences as:

```
observed_indep_statistic <- gss |>
  specify(college ~ finrela) |>
  hypothesize(null = "independence") |>
  calculate(stat = "Chisq")
```
You can do simulation tests like this:

```
null_dist_sim <- gss |>
  specify(college ~ finrela) |>
  hypothesize(null = "independence") |>
  generate(reps = 1000, type = "permute") |>
  calculate(stat = "Chisq")
```

## References

---
title: "5. Demo - Housing Data"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{5. Demo - Housing Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/demo-housing-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# colorize text
colorize <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}
```

This vignette was one of a series of `demo()` files in the package. It is still there
as `demo("housing")`, but is now presented here with additional commentary and analysis,
designed to highlight some aspects of analysis of categorical data and graphical display.

## Load packages
I'll use the following packages in this vignette.

```{r}
library(vcdExtra)
library(MASS)
library(effects)
```


## Housing data

The content here is the dataset `MASS::housing`, giving a 4-way,
$3 \times 3 \times 4 \times 2$
frequency table of 1681 individuals from the *Copenhagen Housing Conditions
Survey*, classified by their:

* Satisfaction (`Sat`) with their housing circumstances (low, medium or high),
* `Type` of rental dwelling (Tower, Apartment, Atrium or Terrace) 
* perceived influence (`Infl`) on management of the property (low, medium, high), and 
* degree of contact (`Cont`) with other residents (low or high)

Load the data:
```{r housing}
data(housing, package="MASS")
str(housing)
```


### Variables, levels and models

Satisfaction (`Sat`) of these householders with their present
housing circumstances is the **outcome variable** here. 

For purposes of analysis,
note that `Sat` is an ordered factor with levels `"Low" < "Medium" < "High"`.

Note also that Influence, with the same levels is just a "Factor", not an
ordered one.

I consider here just models using `glm(..., family=poisson)` or
the equivalent in `MASS::loglm()`. 

The ordering of factor levels is important in graphical displays. We
don't want to see them ordered alphabetically, "High", "Low", "Medium".
The `housing` data.frame was constructed so that the levels of
`Sat` and `Infl` appear in the dataset in their appropriate order.

```{r}
levels(housing$Sat)
levels(housing$Infl)
```



Other models, e.g., the **proportional odds** model, fit using
`MASS:polr()` can take the ordinal nature of satisfaction into
account. In `glm()` one could re-assign `Infl` as an ordered
factor and examine linear vs. non-linear associations for this factor.
But I don't do this here.


## Null model

The most ignorant model asserts that all the table factors are mutually independent.
In symbolic notation, this is `[S] [I] [T] [C]` where all terms in separate `[ ]` are
supposed to be independent. This is `Freq ~ Sat + Infl + Type + Cont` as a formula for
`glm()`.

```{r house.null}
house.null <- glm(Freq ~ Sat + Infl + Type + Cont, family = poisson,
                  data = housing)

```


## Baseline model

When `Sat` is the outcome variable,  a minimal **baseline model** should allow
for all associations among the predictors, symbolized as `[S] [I T C]`.
That is,
Influence, Type and Contact may be associated in arbitrary ways, just
as multiple predictors can be correlated in regression models.

In this framework, what remains to be explained is whether/how `Sat`
depends on the combinations of the other variables. The baseline
model therefore includes the full three-way term for the predictors.
```{r house.glm0}
house.glm0 <- glm(Freq ~ Sat + Infl*Type*Cont, family = poisson,
                  data = housing)
```

Both of these models fit terribly, but we can always use `anova(mod1, mod2,...)` to
compare the *relative* fits of **nested** models.

```{r anova}
anova(house.null, house.glm0, test = "Chisq")
```

## Visualising model fit
The baseline model is shown in the mosaic plot below. Note that
this is applied not to the `housing` data, but rather to the 
`house.glm0` object (of class `glm`) 
resulting to a call to `vcdExtra::mosaic.glm()`.

With four variables in the mosaic, labeling of the variable names
and factor levels is a bit tricky, because labels must appear on all four
sides of the plot.
The `labeling_args` argument can be used to set more informative variable
names and abbreviate factor levels where necessary.

```{r}
#| label= mosaic-glm0a,
#| warning = TRUE
# labeling_args for mosaic()
largs <- list(set_varnames = c(
      Infl="Influence on management", 
			Cont="Contact among residents", 
			Type="Type of dwelling", 
			Sat="Satisfaction"),
	abbreviate=c(Type=3))

mosaic(house.glm0, 
       labeling_args=largs, 
       main='Baseline model: [ITC][Sat]')
```

In this plot we can see largish 
`r colorize("positive residuals", "blue")`
in the blocks corresponding
to (low satisfaction, low influence) and (high satisfaction, high influence)
and clusters of largish 
`r colorize("negative residuals", "red")`
in the opposite corners.

By default, variables are used in the mosaic display in their order in the data
table or frequency data.frame. The `r colorize("warning", "red")` reminds us that the
order of conditioning used is `~Sat + Infl + Type + Cont`.

### Ordering the variables in the mosaic

For `mosaic.glm()`, the conditioning order of variables in the mosaic can be set using
the `formula` argument.  Here, I rearrange the variables to put `Sat`
as the last variable in the splitting / conditioning sequence.
I also use `vcdExtra::modFit()` to add the LR $G^2$ fit statistic to
the plot title.

```{r mosaic-glm0b}
mosaic(house.glm0, 
       formula = ~ Type + Infl + Cont + Sat, 
       labeling_args=largs, 
       main=paste('Baseline model: [ITC][Sat],', modFit(house.glm0))
  )
```

## Adding association terms

Clearly, satisfaction depends on one or more of the predictors, `Infl`, `Type` and `Cont`
and possibly their interactions.

As a first step it is useful to consider sequentially adding the association terms
`Infl:Sat`, `Type:Sat`, `Cont:Sat` one at a time.
This analysis is carried out using `MASS::addterm()`.

```{r addterm}
MASS::addterm(house.glm0, 
              ~ . + Sat:(Infl + Type + Cont), 
              test = "Chisq")
```

Based on this, it is useful to consider a "main-effects" model for satisfaction,
adding all three two-way terms involving satisfaction.

The `update()` method provides an easy way to add (or subtract) terms from a fitted
model object. In the model formula, `.` stands for whatever was on the left
side (`Freq`) or on the right side (`Sat + Infl*Type*Cont`) of the model
(`house.glm0`) that is being updated.

```{r house-glm1}
house.glm1 <- update(house.glm0, 
                     . ~ . + Sat*(Infl + Type + Cont))

```

For comparison, we note that the same model can be fit using the
iterative proportional scaling algorithm of `MASS::loglm()`.

```{r house-loglm1}
(house.loglm1 <- MASS::loglm(Freq ~ Infl * Type * Cont + 
                              Sat*(Infl + Type + Cont), data = housing))

```


## Did the model get better? 

As before, `anova()` tests the added contribution of
each more complex model over the one before.
The residual deviance $G^2$ has been reduced from 
$G^2 (46) = 217.46$ for the baseline model `house.glm0` to
$G^2 (34) = 38.66$ for the revised model `house.glm1`.
The difference, $G^2(M1 | M0) = G^2 (12) = 178.79$ tests the collective additional fit
provided by the two-way association of satisfaction with the predictors.

```{r}
anova(house.glm0, house.glm1, test="Chisq")
```


## Visualize model `glm1`
The model `house.glm1` fits reasonably well, `r modFit(house.glm1)`, so most residuals
are small. In the mosaic below, I use `gp=shading_Friendly` to shade the tiles
so that positive and negative residuals are distinguished by color,
and they are filled when the absolute value of the residual is outside $\pm 2, 4$.

```{r mosaic-glm1}
mosaic(house.glm1, 
       labeling_args=largs, 
       main=paste('Model [IS][TS][CS],', modFit(house.glm1) ), 
       gp=shading_Friendly)
```

One cell is highlighted here: The combination of medium influence, low contact and tower type,
is more likely to give low satisfaction than the model predicts.
Is this just an outlier, or is there something that can be interpreted and perhaps
improve the model fit?
It is hard tell, but the virtues of mosaic displays are that they help to:

* diagnose overall patterns of associations,
* spot unusual cells in relation to lack of fit of a given model.



## Can we drop any terms?

When we add terms using `MASS::addterm()`, they are added sequentially.
It might be the case that once some term is added, a previously added term
is no longer important. Running `MASS::dropterm()` on the `housel.glm1` model
checks for this.

```{r dropterm}
MASS::dropterm(house.glm1, test = "Chisq")

```

Note that the three-way term `Infl:Type:Cont` is not significant.  However,
with `Sat` as the response, the associations of all predictors must be included
in the model.

## What about two-way interactions?

The model so far says that each of influence, type and control have separate,
additive effects on the level of satisfaction, what I called a "main-effects"
model.  It might be the case that some of the predictors have *interaction*
effects, e.g., that the effect of influence on satisfaction might vary with the 
type of dwelling or the level of control.

An easy way to test for these is to update the main-effects model, adding
all possible two-way interactions for `Sat`, one at a time, with `addterm()`.

```{r addterm1}
MASS::addterm(house.glm1,
               ~. + Sat:(Infl + Type + Cont)^2, 
              test  =  "Chisq")
```

The result shows that adding the term `Infl:Type:Sat`
reduces the deviance $G^2$ from 38.66 to 16.11.
The difference, $G^2(M1 + ITS | M1) = G^2 (12) = 22.55$
reflects a substantial improvement.
The remaining two-way interaction terms reduce the deviance
by smaller and non-significant amounts, relative to `house.glm1`.

Model fitting should be guided by substance, not just statistical
machinery. Nonetheless, it seems arguably sensible to add one
two-way term to the model, giving `house.glm2`.

```{r}
house.glm2 <- update(house.glm1,
                     . ~ . + Sat:Infl:Type)

```

## Model parsimony: AIC & BIC

Adding more association terms to a model will always improve it.
The question is, whether that is "worth it"?

"Worth it" concerns the trade-off between model fit and parsimony.
Sometimes we might prefer a model with fewer parameters to one
that has a slightly better fit, but requires more model terms
and parameters.  

The AIC and BIC statistics are designed to 
adjust our assessment of model fit by penalizing it for using more
parameters. Equivalently, they deduct from the 
likelihood ratio $G^2$ a term proportional to the residual $\text{df}$ of the model.
In any case -- **smaller is better** for both AIC and BIC.

$$AIC = G^2 - 2 \: \text{df}$$
$$BIC = G^2 - \log(n) \: \text{df}$$


These measures are provided 
by `AIC()`, `BIC()`, and can be used to compare models using
`vcdExtra::LRstats()`. 

```{r lrstats}
LRstats(house.glm0, house.glm1, house.glm2)
```

By these metrics, model `house.glm1` is best on both AIC and BIC.
The increased goodness-of-fit (smaller $G^2$) of model `house.glm2`
is not worth the extra cost of parameters in the `house.glm2` model.

<!-- ## Effect plots -->

<!-- ```{r} -->
<!-- house.eff <-allEffects(house.glm1) -->

<!-- ``` -->

---
title: "tidyCat: Tidy Methods For Categorical Data Analysis"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{tidyCat: Tidy Methods For Categorical Data Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tidycat-",
  dev = "png",
  comment = "##"
)
```


<center>
<img src="fig/tidyCat-logo.png" height=160>
</center>

> Frequency tables need some Tidy Love ❤️

Tidy methods for quantitative data & models have advanced considerably, but there hasn't been much
development of similar ideas for "categorical data", by which I mean data that is often compactly represented
as $n$-way frequency tables, cross classified by one or more discrete factors. 

What would it take to implement a tidy framework for such data? These notes are, in effect, a call for 
participation in developing a `tidyCat` package for this purpose. Other possible names for this:
`tidyCDA`, `tidyfreq` but `tidyCat` makes for a nice logo!

```{r load-pkgs}
library(MASS)
library(vcdExtra)
```

I see three areas that could be developed here:

## Constructing categorical data sets 
Current non-tidy data forms and operations, following [@FriendlyMeyer:2016:DDAR] are described in
the vignette [Creating and manipulating frequency tables](https://friendly.github.io/vcdExtra/articles/a1-creating.html)

It seems clear that the most flexible and general form, and one that most closely matches a tidy
data frame is **case form**, because this allows for numeric variables as well. Thus:

> Among these, tidy categorical data should best be represented as a `tibble` in **case form**. A tibble is convenient for its' printing.

### Manipulating categorical data sets

The methods `xtabs()`, `table()` and `expand.dft()` described in that vignette allow conversion from one form to another.
Tidy equivalents might be:

* `as_table()`, `as_matrix()`, `as_array()` to convert from any form to table/array form

* Similarly, perhaps `as_caseform()`, `as_freqform()` to convert to those. There is already `as.data.frame(table)` to convert to frequency form, and `expand.dft()` converts that to case form

```{r}
data("HairEyeColor")
hec.df <- as.data.frame(HairEyeColor)
head(hec.df)
# expand to case form
expand.dft(hec.df) |> head()
```


* `vcd::structable()` (and `stats::ftable()`) produce a 'flat' representation of a high-dimensional contingency table constructed by recursive splits (similar to the construction of mosaic displays). One can be constructed from a table or from a data frame with
a formula method,

```{r}
structable(Titanic)
structable(Sex + Class ~ Survived + Age, data = Titanic)
```

and there are a suite of methods for indexing and selecting parts of an $n$-way table.

* The methods in the `plyr` package (now retired) provided a coherent set of tools for a split-apply-combine
strategy that works nicely with multidimensional arrays. Perhaps there are some useful ideas for frequency tables
that could be resurrected here.

* There is also a role for `purrr` methods and thinking here: $n$-way tables as nested lists/arrays?
The ideas of mapping over these?

## Manipulating factor levels 

Also needed: 

* methods for **recoding and collapsing** the levels of a factor: `forcats::fct_recode()`, `forcats::fct_collapse()`,  `forcats::fct_lump_min()` are useful here.

* methods for **reordering the levels** of a factor, either manually or for some analysis purpose. For example, Data from @Glass:54 gave this 5 x 5 table on the occupations of 3500 British fathers and their sons, where the occupational categories are listed in alphabetic order.
  
```{r glass}
data(Glass, package="vcdExtra")
str(Glass)
(glass.tab <- xtabs(Freq ~ father + son, data=Glass))
```

This can be reordered manually by indexing, to arrange the categories by **status**, giving an order  `Professional` down to `Unskilled`:

```{r glass-order}
# reorder by status
ord <- c(2, 1, 4, 3, 5) 
glass.tab[ord, ord]
```

A more general method is to permute the row and column categories in the order implied by correspondence analysis dimensions. This is implemented in the [`seriation` package](https://cran.r-project.org/package=seriation) using the `CA` method of `seriation::seriate()` and applying `permute()` to the result.

```{r housetasks-seriation}
library(seriation)
order <- seriate(glass.tab, method = "CA")
# the permuted row and column labels
rownames(glass.tab)[order[[1]]]

# reorder rows and columns
permute(glass.tab, order)
```
What are tidy ways to do these things?


## Models

The standard analysis of frequency data is in the form of loglinear models fit by `MASS::loglm()`
or with more flexible versions fit with `glm()` or `gnm::gnm()`. These are essentially linear models for the log
of frequency. In `vcdExtra`, there are several methods for a list of such models, of class `"glmlist"` and `"loglmlist"`,
and these should be accommodated in tidy methods. 

What is needed are `broom` methods for `loglm` models. The information required is accessible from standard functions, but not in a tidy form.

* `glance.loglm()` -- **model level** statistics. These are given in the output of the `print()` method, and available from the `print()` method. A complication is both LR and Pearson $\chi^2$ are reported, so these would need to be made to appear in separate columns. There are also related `LRstats()` functions in `vcdExtra`, which report `AIC` and `BIC`.

```{r}
hec.indep <- loglm(~Hair+Eye+Sex, data=HairEyeColor)
hec.indep
# extract test statistics
summary(hec.indep)$tests
LRstats(hec.indep)
```

* `tidy.loglm()` --- **coefficient level** statistics. These are available from `coef.loglm()`. They would need to assembled into a long format. Standard errors & p-values might be a problem.

```{r}
coef(hec.indep)
```

* `augment.loglm()` --- should give **case level** statistics: fitted values, residuals, ...

```{r}
fitted(hec.indep)
residuals(hec.indep)
```

What about `hatvalues`? Not implemented, but shouldn't be too hard.

```{r error=TRUE}
hatvalues(hec.indep)
```


## Graphical methods

The most common graphical methods are those implemented in `vcd`: `mosaic()` association plots (`assoc()`), ..., which
rely on `vcd::strucplot()` described in [@vcd:Meyer+Zeileis+Hornik:2006b].

Is there a tidy analog that might work with `ggplot2`? The [`ggmosaic` package](https://github.com/haleyjeppson/ggmosaic) implements basic marimeko-style mosaic plots. They are not very general, in that they cannot do residual-based shading to show the patterns of association.

However, they are based on a [productplots](https://github.com/hadley/productplots) package by Hadley, which seems to
provide some basic structure for constructing such displays of nested rectangles.

## Some tidy stuff

* The [infer package](https://infer.netlify.app/) has some interesting stuff for CDA. 
The vignette [Tidy Chi-Squared Tests with infer](https://infer.netlify.app/articles/chi_squared)
expresses do a Chisq test of independences as:

```
observed_indep_statistic <- gss |>
  specify(college ~ finrela) |>
  hypothesize(null = "independence") |>
  calculate(stat = "Chisq")
```
You can do simulation tests like this:

```
null_dist_sim <- gss |>
  specify(college ~ finrela) |>
  hypothesize(null = "independence") |>
  generate(reps = 1000, type = "permute") |>
  calculate(stat = "Chisq")
```

## References

---
title: "2. Tests of Independence"
author: "Michael Friendly"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
vignette: >
  %\VignetteIndexEntry{2. Tests of Independence}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tut02-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# Old Sweave options
# \SweaveOpts{engine=R,eps=TRUE,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{engine=R,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{prefix.string=fig/vcd-tut,eps=FALSE}
# \SweaveOpts{keep.source=TRUE}


# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(HairEyeColor)
data(PreSex)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
#if(!file.exists("fig")) dir.create("fig")

```

OK, now we're ready to do some analyses.  This vignette focuses on relatively simple non-parametric
tests and measures of association.

## CrossTable

For tabular displays,
the  `CrossTable()`  function in  the `gmodels`  package produces  cross-tabulations
modeled after `PROC FREQ` in SAS or `CROSSTABS` in SPSS. 
It has a wealth of options for the quantities that can be shown in each cell.

Recall the GSS data used earlier.
```{r, GSStab}
# Agresti (2002), table 3.11, p. 106
GSS <- data.frame(
  expand.grid(sex = c("female", "male"), 
              party = c("dem", "indep", "rep")),
  count = c(279,165,73,47,225,191))

(GSStab <- xtabs(count ~ sex + party, data=GSS))
```

Generate a cross-table showing cell frequency and the cell contribution to $\chi^2$.
```{r, xtabs-ex2}
# 2-Way Cross Tabulation
library(gmodels)
CrossTable(GSStab, prop.t=FALSE, prop.r=FALSE, prop.c=FALSE)
```

There are  options to  report percentages  (row, column,  cell), specify decimal
places, produce Chi-square,  Fisher, and McNemar  tests of independence,  report
expected  and residual  values (pearson,  standardized, adjusted  standardized),
include missing values as valid, annotate with row and column titles, and format
as SAS or SPSS style output! See `help(CrossTable)` for details.

## Chi-square test

For 2-way tables you can use `chisq.test()` to test independence of the row
and column variable. By default,  the $p$-value is calculated from  the asymptotic
chi-squared distribution of the test  statistic. Optionally, the $p$-value can  be
derived via Monte Carlo simulation. 

```{r, chisq}
(HairEye <- margin.table(HairEyeColor, c(1, 2)))

chisq.test(HairEye)

chisq.test(HairEye, simulate.p.value = TRUE)
```

## Fisher Exact Test {#sec:Fisher}

`fisher.test(X)` provides an  **exact test** of  independence. `X` must be  a two-way
contingency table in table form.  Another form, 
`fisher.test(X, Y)` takes two
categorical vectors of the same length.  
For tables larger than $2 \times 2$ the method can be computationally intensive (or can fail) if
the frequencies are not small.

```{r fisher}
fisher.test(GSStab)
```

Fisher's test is meant for tables with small total sample size.
It generates an error for the `HairEye` data with $n$=592 total frequency.

```{r fisher-error, error=TRUE}
fisher.test(HairEye)
```
## Mantel-Haenszel test and conditional association {#sec:mantel}

Use the  `mantelhaen.test(X)` function  to perform  a Cochran-Mantel-Haenszel 
$\chi^2$ chi
test  of   the  null  hypothesis   that  two  nominal   variables  are
*conditionally independent*, $A \perp B \; | \; C$, in each stratum,  assuming that there is no  three-way
interaction. `X` is  a 3 dimensional  contingency table, where  the last dimension
refers to the strata.

The `UCBAdmissions` serves as an example of a $2 \times 2 \times 6$ table,
with `Dept` as the stratifying variable.
```{r, mantel1}
# UC Berkeley Student Admissions
mantelhaen.test(UCBAdmissions)
```

The results show no evidence for association between admission and gender
when adjusted for department.  However, we can easily see that the assumption
of equal association across the strata (no 3-way association) is probably
violated. For $2 \times 2 \times k$ tables, this can be examined
from the odds ratios for each $2 \times 2$ table (`oddsratio()`), and
tested by  using `woolf_test()` in `vcd`.

```{r, mantel2}
oddsratio(UCBAdmissions, log=FALSE)

lor <- oddsratio(UCBAdmissions)  # capture log odds ratios
summary(lor)

woolf_test(UCBAdmissions) 
``` 

## Some plot methods

### Fourfold displays

We  can visualize the  odds ratios of  Admission for
each  department  with  fourfold  displays  using  `fourfold()`.  The cell
frequencies $n_{ij}$  of each  $2 \times  2$ table  are shown  as a  quarter circle whose
radius is proportional to $\sqrt{n_{ij}}$, so  that its area is proportional to  the
cell frequency.

```{r, reorder3}
UCB <- aperm(UCBAdmissions, c(2, 1, 3))
dimnames(UCB)[[2]] <- c("Yes", "No")
names(dimnames(UCB)) <- c("Sex", "Admit?", "Department")
```
Confidence rings for the odds ratio allow a visual test of the null of no association; 
the rings for adjacent quadrants overlap *iff* the observed counts are consistent 
with the null hypothesis.  In the extended version (the default), brighter colors
are used where the odds ratio is significantly different from 1.
The following lines produce @ref(fig:fourfold1).
<!-- \footnote{The color values `col[3:4]` were modified from their default values -->
<!-- to show a greater contrast between significant and insignificant associations here.} -->



```{r}
#| fourfold1, 
#| h=5, w=7.5, 
#| cap = "Fourfold display for the `UCBAdmissions` data. Where the odds ratio differs
#|   	significantly from 1.0, the confidence bands do not overlap, and the circle quadrants are
#|  	shaded more intensely."
col <- c("#99CCFF", "#6699CC", "#F9AFAF", "#6666A0", "#FF0000", "#000080")
fourfold(UCB, mfrow=c(2,3), color=col)
```
 

Another `vcd` function, `cotabplot()`, provides a more general approach
to visualizing conditional associations in contingency tables,
similar to trellis-like plots produced by `coplot()` and lattice graphics.
The `panel` argument supplies a function used to render each conditional 
subtable. The following gives a display (not shown) similar to @ref(fig:fourfold1).
```{r fourfold2, eval=FALSE}
cotabplot(UCB, panel = cotab_fourfold)
```

### Doubledecker plots

When we want to view the conditional
probabilities of a response variable (e.g., `Admit`)
in relation to several factors,
an alternative visualization is a `doubledecker()` plot.
This plot is a specialized version of a mosaic plot, which
highlights the levels of a response variable (plotted vertically)
in relation to the factors (shown horizontally). The following
call produces @ref(fig:doubledecker), where we use indexing
on the first factor (`Admit`) to make `Admitted`
the highlighted level.

In this plot, the
association between `Admit` and `Gender` is shown
where the heights of the highlighted conditional probabilities
do not align. The excess of females admitted in Dept A stands out here.

```{r}
#| doubledecker, 
#| h=5, w=8,
#| out.width = "75%",
#| cap = "Doubledecker display for the `UCBAdmissions` data. The heights
#|     of the highlighted bars show the conditional probabilities of `Admit`,
#|     given `Dept` and `Gender`."
doubledecker(Admit ~ Dept + Gender, data=UCBAdmissions[2:1,,])
```

### Odds ratio plots
Finally, the there is a `plot()` method for `oddsratio` objects.
By default, it shows the 95% confidence interval for the log odds ratio.
@ref(fig:oddsratio) is produced by:

```{r}
#| oddsratio,
#| h=6, w=6,
#| out.width = "60%",
#| cap = "Log odds ratio plot for the `UCBAdmissions` data."
plot(lor, 
     xlab="Department", 
     ylab="Log Odds Ratio (Admit | Gender)")
```
 {#fig:oddsratio}

## Cochran-Mantel-Haenszel tests for ordinal factors {#sec:CMH}

The standard $\chi^2$ tests for association in a two-way table
treat both table factors as nominal (unordered) categories.
When one or both factors of a two-way table are
quantitative or ordinal, more powerful tests of association
may be obtained by taking ordinality into account, using
row and or column scores to test for linear trends or differences
in row or column means.

More general versions of the CMH tests (Landis etal., 1978) 
[@Landis-etal:1978] are provided by assigning
numeric scores to the row and/or column variables. 
For example, with two ordinal factors (assumed to be equally spaced), assigning
integer scores, `1:R` and `1:C` tests the linear $\times$ linear component
of association. This is statistically equivalent to the Pearson correlation between the
integer-scored table variables, with $\chi^2 = (n-1) r^2$, with only 1 $df$
rather than $(R-1)\times(C-1)$ for the test of general association.

When only one table
variable is ordinal, these general CMH tests are analogous to an ANOVA, testing
whether the row mean scores or column mean scores are equal, again consuming
fewer $df$ than the test of general association.

The `CMHtest()` function in `vcdExtra` calculates these various
CMH tests for two possibly ordered factors, optionally stratified other factor(s).

***Example***:
```{r, table-form2, include=FALSE}
## A 4 x 4 table  Agresti (2002, Table 2.8, p. 57) Job Satisfaction
JobSat <- matrix(c(1,2,1,0, 3,3,6,1, 10,10,14,9, 6,7,12,11), 4, 4)
dimnames(JobSat) = list(income=c("< 15k", "15-25k", "25-40k", "> 40k"),
                satisfaction=c("VeryD", "LittleD", "ModerateS", "VeryS"))
JobSat <- as.table(JobSat)
```

Recall the $4 \times 4$ table, `JobSat` introduced in \@ref(sec:creating),
```{r, jobsat}
JobSat
```

Treating the `satisfaction` levels as equally spaced, but using
midpoints of the `income` categories as row scores gives the following results:
```{r, cmh1}
CMHtest(JobSat, rscores=c(7.5,20,32.5,60))
```

Note that with the relatively small cell frequencies, the test for general 
give no evidence for association. However, the the `cor` test for linear x linear
association on 1 df is nearly significant. The `coin` package contains the
functions `cmh_test()` and `lbl_test()`
for CMH tests of general association and linear x linear association respectively.

## Measures of Association

There are a variety of statistical measures of *strength* of association for
contingency tables--- similar in spirit to $r$ or $r^2$ for continuous variables.
With a large sample size, even a small degree of association can show a 
significant $\chi^2$, as in the example below for the `GSS` data.

The  `assocstats()`  function in `vcd`  calculates  the   $\phi$
contingency coefficient,  and Cramer's  V for  an $r \times c$  table. 
The input must be in table form, a two-way $r \times c$ table.
It won't work with `GSS` in frequency form, but by now you should know how
to convert.
```{r, assoc1}
assocstats(GSStab)
```

For tables with ordinal variables, like `JobSat`, some people prefer the
Goodman-Kruskal $\gamma$ statistic
[@vcd:Agresti:2002, \S 2.4.3]
based on a comparison of concordant
and discordant pairs of observations in the case-form equivalent of a two-way table.
```{r, gamma}
GKgamma(JobSat)
```

<!-- A web article by Richard Darlington, -->
<!-- [http://node101.psych.cornell.edu/Darlington/crosstab/TABLE0.HTM] -->
<!-- gives further description of these and other measures of association. -->

## Measures of Agreement
The
`Kappa()` function in the `vcd` package calculates Cohen's $\kappa$ and weighted
$\kappa$ for a square two-way table with the same row and column categories [@Cohen:60].
\footnote{ 
Don't confuse this with `kappa()` in base R that computes something
entirely different (the condition number of a matrix).
}
Normal-theory $z$-tests are obtained by dividing $\kappa$ by its asymptotic standard
error (ASE).  A `confint()` method for `Kappa` objects provides confidence intervals.
```{r, kappa}
data(SexualFun, package = "vcd")
(K <- Kappa(SexualFun))
confint(K)
```

A visualization of agreement [@Bangdiwala:87], both unweighted and weighted for degree of departure
from exact agreement is provided by the `agreementplot()` function.
@fig(fig:agreesex) shows the agreementplot for the `SexualFun` data,
produced as shown below. 

The Bangdiwala measures (returned by the function)
represent the proportion of the
shaded areas of the diagonal rectangles, using weights $w_1$ for exact agreement,
and $w_2$ for partial agreement one step from the main diagonal.

```{r}
#| agreesex, 
#| h=6, w=7,
#| out.width = "70%",
#| cap = "Agreement plot for the `SexualFun` data."
agree <- agreementplot(SexualFun, main="Is sex fun?")
unlist(agree)
```


In other examples, the agreement plot can help to show *sources*
of disagreement.  For example, when the shaded boxes are above or below the diagonal
(red) line, a lack of exact agreement can be attributed in part to
different frequency of use of categories by the two raters-- lack of
*marginal homogeneity*.
	
## Correspondence analysis

Correspondence analysis is a technique for visually exploring relationships
between rows and columns in contingency tables. The `ca` package gives one implementation.
For an $r \times c$ table,
the method provides a breakdown of the Pearson $\chi^2$ for association in up to $M = \min(r-1, c-1)$
dimensions, and finds scores for the row ($x_{im}$) and column ($y_{jm}$) categories
such that the observations have the maximum possible correlations.%
^[Related methods are the non-parametric CMH tests using assumed row/column scores (\@ref(sec:CMH),
the analogous `glm()` model-based methods (\@ref(sec:CMH), and the more general RC models which can be fit using `gnm()`. Correspondence analysis differs in that it is a primarily descriptive/exploratory method (no significance tests), but is directly tied to informative graphic displays of the row/column categories.]



Here, we carry out a simple correspondence analysis of the `HairEye` data.
The printed results show that nearly 99% of the association between hair color and eye color
can be accounted for in 2 dimensions, of which the first dimension accounts for 90%.
```{r, ca1}
library(ca)
ca(HairEye)
```


The resulting `ca` object can be plotted just by running the `plot()`
method on the `ca` object, giving the result in
\@ref(fig:ca-haireye).  `plot.ca()` does not allow labels for dimensions;
these can be added with `title()`.
It can be seen that most of the association is accounted for by the ordering
of both hair color and eye color along Dimension 1, a dark to light dimension.

```{r ca-haireye, cap = "Correspondence analysis plot for the `HairEye` data"}
plot(ca(HairEye), main="Hair Color and Eye Color")
```


## References

---
title: "7. Continuous predictors"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{7. Continuous predictors}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tut05-",
  fig.align = "center",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
#if(!file.exists("fig")) dir.create("fig")

```



When continuous predictors are available---and potentially important---in explaining a
categorical outcome, models for that outcome include:
logistic regression (binary response), 
the proportional odds model (ordered polytomous response),
multinomial (generalized) logistic regression.
Many of these are special cases of the generalized linear model using the
`"poisson"` or `"binomial"` family and their relatives.

## Spine and conditional density plots {#sec:spine}
I don't go into fitting such models here, but I would be remiss not to illustrate some
visualizations in `vcd` that are helpful here.
The first of these is the spine plot or spinogram [@vcd:Hummel:1996], produced with `spine()`.
These are special cases of mosaic plots with 
specific spacing and shading to show how a categorical response varies with
a continuous or categorical predictor.

They are also a generalization of stacked bar plots where not the heights  but
the *widths* of the bars corresponds to the relative frequencies of `x`. The heights
of the  bars then  correspond to  the conditional  relative frequencies  of `y` in
every `x` group.

***Example***:
For the `Arthritis` data, we can see how `Improved` varies with `Age`
as follows.  `spine()` takes a formula of the form 
`y ~ x` with a single dependent factor and a single explanatory variable `x`
(a numeric variable or a factor).
The range of a numeric variable`x` is divided into intervals based on the
`breaks` argument, and stacked bars are drawn to show the distribution of
`y` as `x` varies.  As shown below, the discrete table that is visualized
is returned by the function.

```{r, spine1}
#| fig.height = 6,
#| fig.width = 6,
#| fig.show = "hold",
#| out.width = "46%",
#| fig.align = "center",
#| cap = "Spine plots for the `Arthritis` data"
(spine(Improved ~ Age, data = Arthritis, breaks = 3))
(spine(Improved ~ Age, data = Arthritis, breaks = "Scott"))
```


The conditional density plot [@vcd:Hofmann+Theus] is a further generalization.
This visualization technique is similar to spinograms, but uses a smoothing approach
rather than discretizing the explanatory variable.  As well, it uses 
the original `x` axis and not a distorted one.


```{r}
#| cdplot,
#| fig.height = 5,
#| fig.width = 5,
#| cap = "Conditional density plot for the `Arthritis` data showing the variation of Improved with Age."
cdplot(Improved ~ Age, data = Arthritis)
```



In such plots, it is useful to also see the distribution of the observations
across the horizontal axis, e.g., with a `rug()` plot.
\@ref{fig:cd-plot} uses `cdplot()` from the `graphics` package
rather than `cd_plot()` from `vcd`, and is produced with

```{r}
#| cdplot1,
#| fig.height = 5,
#| fig.width = 5,
cdplot(Improved ~ Age, data = Arthritis)
with(Arthritis, rug(jitter(Age), col="white", quiet=TRUE))
```

From this figure it can be easily seen that the proportion
of patients reporting Some or Marked improvement increases with Age,
but there are some peculiar bumps in the distribution.
These may be real or artifactual, but they would be hard to see
with most other visualization methods.
When we switch from non-parametric data exploration to parametric
statistical models, such effects are easily missed.

## Model-based plots: effect plots and `ggplot2 plots` {#sec:modelplots}

The nonparametric conditional density plot uses smoothing methods
to convey the distributions of the response variable,
but displays that are simpler to interpret can often be obtained by plotting
the predicted response from a parametric model.

For complex `glm()` models with interaction effects, the `effects`
package  provides the most useful displays,
plotting  the predicted values for a given term, averaging over other
predictors not included in that term.  I don't illustrate this here, but
see @effects:1,@effects:2 and `help(package="effects")`.

Here I just briefly illustrate the capabilities of the `ggplot2` package
for model-smoothed plots of categorical responses in `glm()` models.

***Example***:
The `Donner` data frame in `vcdExtra` gives details on the survival
of 90 members of the Donner party,
a group of people who attempted to migrate to California in 1846.
They were trapped by an early blizzard on the eastern side of the
Sierra Nevada mountains, and before they could be rescued, 
nearly half of the party had died.
What factors affected who lived and who died?

```{r, donner1}
data(Donner, package="vcdExtra")
str(Donner)
```

A potential model of interest
is the logistic regression model for $Pr(survived)$, allowing separate
fits for males and females as a function of `age`.
The key to this is the `stat_smooth()` function, using
`method = "glm", method.args = list(family = binomial)`. The `formula = y ~ x`
specifies a linear fit on the logit scale (\@ref{fig:donner3}, left)

```{r, donner2a, fig=FALSE, eval=FALSE}
# separate linear fits on age for M/F
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ x,
              alpha = 0.2, size=2, aes(fill = sex))
```

Alternatively, we can allow a quadratic relation with `age`
by specifying `formula = y ~ poly(x,2)` (@ref(fig:donner3), right).

```{r, donner2b, fig=FALSE, eval=FALSE}
# separate quadratics
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ poly(x,2), 
              alpha = 0.2, size=2, aes(fill = sex))
```


```{r}
#| donner3a,
#| echo = FALSE,
#| fig.height = 6,
#| fig.width = 6,
#| fig.show = "hold",
#| out.width = "46%",
#| cap = "Logistic regression plots for the  `Donner` data	showing survival vs. age, by sex. Left: linear logistic model; right: quadratic model {#fig:donner3}"
# separate linear fits on age for M/F
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ x,
              alpha = 0.2, size=2, aes(fill = sex))

# separate quadratics
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ poly(x,2),
              alpha = 0.2, size=2, aes(fill = sex))
```


These plots very nicely show (a) the fitted $Pr(survived)$ for males and females;
(b) confidence bands around the smoothed model fits and (c) the individual
observations by jittered points at 0 and 1 for those who died and survived, respectively.

# References

---
title: "1. Creating and manipulating frequency tables"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{1. Creating and manipulating frequency tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tut01-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# Old Sweave options
# \SweaveOpts{engine=R,eps=TRUE,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{engine=R,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{prefix.string=fig/vcd-tut,eps=FALSE}
# \SweaveOpts{keep.source=TRUE}


# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(HairEyeColor)
data(PreSex)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
#if(!file.exists("fig")) dir.create("fig")

```


R provides many methods for creating frequency and contingency tables. Several are
described below. In the examples below, we use some real examples and some anonymous
ones, where the variables `A`, `B`, and `C`  represent
categorical variables, and `X` represents an arbitrary R data object.

## Forms of frequency data
The first thing you need to know is that categorical data can be represented in
three different forms in R, and it is sometimes necessary to convert
from one form to another, for carrying out statistical tests, fitting models
or visualizing the results.  Once a data object exists in R, 
you can examine its complete structure with
the `str()` function, or view the names of its components with the
`names()` function.
 

### Case form
Categorical data in case form are simply data frames containing individual observations, with one or
more factors, used as the classifying variables. In case form, there may also
be numeric covariates. 
The total number of observations is `nrow(X)`, and the number of variables is `ncol(X)`.
  	
***Example***:

The `Arthritis` data is available in case form in the `vcd` package. 
There are two explanatory factors: `Treatment` and `Sex`. `Age`
is a numeric covariate, and `Improved` is the response--- an ordered factor,
with levels
`r paste(levels(Arthritis$Improved),collapse=' < ')`.
Excluding `Age`, this represents
a $2 \times 2 \times 3$ contingency table for `Treatment`, `Sex` and `Improved`, but in case form.

```{r, case-form}
names(Arthritis)      # show the variables

str(Arthritis)        # show the structure

head(Arthritis,5)     # first 5 observations, same as Arthritis[1:5,] 
```
  
### Frequency form
Data in frequency form is also a data frame containing one or more factors, and a frequency 
variable, often called `Freq` or `count`.  The total number of observations
is: `sum(X$Freq)`,  `sum(X[,"Freq"])` or some equivalent form.

The number of cells in the table is given by `nrow(X)`.

***Example***: 
For small frequency tables, it is often convenient to enter them in frequency form
using `expand.grid()` for the factors and `c()` to list the counts in a vector.
The example below, from [@vcd:Agresti:2002] gives results for the 1991 General Social Survey,
with respondents classified by sex and party identification.
```{r, frequency-form}
# Agresti (2002), table 3.11, p. 106
GSS <- data.frame(
  expand.grid(sex = c("female", "male"), 
              party = c("dem", "indep", "rep")),
  count = c(279,165,73,47,225,191))

GSS
names(GSS)
str(GSS)

sum(GSS$count)
```

### Table form 
Table form data is represented by a `matrix`, `array` or `table` object, whose elements are the frequencies
in an $n$-way table.  The variable names (factors) and their levels are given by
`dimnames(X)`. The total number of observations
is `sum(X)`.  The number of dimensions of the table is `length(dimnames(X))`,
and the table sizes are given by `sapply(dimnames(X), length)`.
  	
***Example***: 
The `HairEyeColor` is stored in table form in `vcd`.  
```{r, table-form1}
str(HairEyeColor)                      # show the structure

sum(HairEyeColor)                      # number of cases

sapply(dimnames(HairEyeColor), length) # table dimension sizes
```
***Example***: 
Enter frequencies in a matrix, and assign `dimnames`,
giving the variable names and category labels.  Note that, by default,
`matrix()` uses the elements supplied by *columns* in the
result, unless you specify `byrow=TRUE`.
```{r, table-form2}
# A 4 x 4 table  Agresti (2002, Table 2.8, p. 57) Job Satisfaction
JobSat <- matrix(c( 1, 2, 1, 0, 
                    3, 3, 6, 1, 
                   10,10,14, 9, 
                    6, 7,12,11), 4, 4)

dimnames(JobSat) = list(
  income = c("< 15k", "15-25k", "25-40k", "> 40k"),
  satisfaction = c("VeryD", "LittleD", "ModerateS", "VeryS")
  )

JobSat
```

`JobSat` is a **matrix**, not an object of `class("table")`, and some functions
are happier with tables than matrices.
You can coerce it to a table with `as.table()`,
```{r, table-form3}
JobSat <- as.table(JobSat)
str(JobSat)
```

## Ordered factors and reordered tables {#sec:ordered-factors}

In table form, the values of the table factors are ordered by their position in the table.
Thus in the `JobSat` data, both `income` and `satisfaction` represent ordered
factors, and the *positions* of the values in the rows and columns reflects their
ordered nature.

Yet, for analysis, there are times when you need *numeric* values for the levels
of ordered factors in a table, e.g., to treat a factor as a quantitative variable.
In such cases, you can simply re-assign the `dimnames` attribute of the table
variables.  For example, here, we assign numeric values to `income` as the middle of their
ranges, and treat `satisfaction` as equally spaced with integer scores.

```{r, relevel, eval=FALSE}
dimnames(JobSat)$income <- c(7.5,20,32.5,60)
dimnames(JobSat)$satisfaction <- 1:4
```

For the  `HairEyeColor` data, hair color and eye color are ordered arbitrarily.
For visualizing the data using mosaic plots and other methods described below, it 
turns out to be more useful to assure that both hair color and eye color are
ordered from dark to light.
Hair colors are actually ordered this way already, and it is easiest to re-order
eye colors by indexing. Again `str()` is your friend.

```{r, reorder1}
HairEyeColor <- HairEyeColor[, c(1,3,4,2), ]
str(HairEyeColor)
```
This is also the order for both hair color and eye color shown in 
the result of a correspondence analysis (@ref(fig:ca-haireye) below.

With data in  case form or frequency form, when you have ordered factors
represented with character values, you must ensure that they are treated
as ordered in R.
<!-- \footnote{In SAS, many procedures offer the option -->
<!-- 	`order = data | internal | formatted` to allow character values -->
<!-- 	to be ordered according to (a) their order in the data set, (b) -->
<!-- 	sorted internal value, or (c) sorted formatted representation -->
<!-- 	provided by a SAS format. -->
<!-- } -->

Imagine that the `Arthritis` data was read from a text file.  
By default the `Improved` will be ordered alphabetically:
`Marked`,
`None`,
`Some` --- not what we want.  In this case, the function
`ordered()` (and others) can be useful. 

```{r, reorder2, echo=TRUE, eval=FALSE}
Arthritis <- read.csv("arthritis.txt",header=TRUE)
Arthritis$Improved <- ordered(Arthritis$Improved, 
                              levels=c("None", "Some", "Marked")
                              )
```
The dataset `Arthritis` in the `vcd` package is a data.frame in this form
With this order of  `Improved`, the response in this data,
a mosaic display of `Treatment` and `Improved` (@ref(fig:arthritis) shows a clearly
interpretable pattern.

The original version of `mosaic` in the `vcd` package required the input to be 
a contingency table in array form, so we convert using `xtabs()`.

<!-- ```{r Arthritis, height=6, width=7, fig.cap="Mosaic plot for the `Arthritis` data ..."} -->

```{r}
#| Arthritis,
#| fig.height = 6,
#| fig.width = 6,
#| fig.cap = "Mosaic plot for the `Arthritis` data, showing the marginal model of independence for Treatment and Improved.  Age, a covariate, and Sex are ignored here."
#| 
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
mosaic(art, gp = shading_max, split_vertical = TRUE, main="Arthritis: [Treatment] [Improved]")
```

Several data sets in the package illustrate the salutary effects of reordering factor levels
in mosaic displays and other analyses. See:

* `help(AirCrash)`
* `help(Glass)`
* `help(HouseTasks)`

The [seriate](https://CRAN.R-project.org/package=seriation) package now contains a general method
to permute the row and column variables in a table according to the result of a correspondence
analysis, using scores on the first CA dimension.

### Re-ordering dimensions

Finally, there are situations where, particularly for display purposes, you
want to re-order the *dimensions* of an $n$-way table, or change the
labels for the variables or levels.
This is easy when the data are in table form: `aperm()` permutes
the dimensions, and assigning to `names` and `dimnames`
changes variable names and level labels respectively.
We will use the following version of `UCBAdmissions` in
\@ref(sec:mantel) below.
^[Changing `Admit` to `Admit?` might be useful for display purposes, but is
dangerous--- because it is then difficult to use that variable name in a model formula.
See \@ref(sec:tips) for options `labeling_args` and `set_labels`to change variable and level names for displays in the `strucplot` framework.]

```{r, reorder3}
UCB <- aperm(UCBAdmissions, c(2, 1, 3))
dimnames(UCB)[[2]] <- c("Yes", "No")
names(dimnames(UCB)) <- c("Sex", "Admit?", "Department")

# display as a flattened table
stats::ftable(UCB)
```

## `structable()` {#sec:structable}

For 3-way and larger tables
the `structable()` function in `vcd` provides a convenient and flexible tabular display.
The variables assigned to the rows and columns of a two-way display can be specified
by a model formula.
```{r, structable}
structable(HairEyeColor)                   # show the table: default

structable(Hair+Sex ~ Eye, HairEyeColor)   # specify col ~ row variables
```
It also returns an object of class `"structable"` which may be plotted with 
`mosaic()` (not shown here).
```{r, structable1,eval=FALSE}
HSE < - structable(Hair+Sex ~ Eye, HairEyeColor)   # save structable object
mosaic(HSE)                                        # plot it
```

## `table()` and friends {#sec:table}

You  can  generate frequency  tables from factor variables  using the  `table()` function,  tables  of
proportions using  the `prop.table()` function,  and marginal  frequencies using
`margin.table()`.

For these examples, create some categorical vectors:
```{r, table-setup}
 n=500
 A <- factor(sample(c("a1","a2"), n, rep=TRUE))
 B <- factor(sample(c("b1","b2"), n, rep=TRUE))
 C <- factor(sample(c("c1","c2"), n, rep=TRUE))
 mydata <- data.frame(A,B,C)
```

These lines illustrate `table`-related functions:
```{r, table-ex1}
# 2-Way Frequency Table
attach(mydata)
mytable <- table(A,B)   # A will be rows, B will be columns
mytable                 # print table

margin.table(mytable, 1) # A frequencies (summed over B)
margin.table(mytable, 2) # B frequencies (summed over A)

prop.table(mytable)    # cell percentages
prop.table(mytable, 1) # row percentages
prop.table(mytable, 2) # column percentages
```

`table()` can  also  generate  multidimensional  tables  based  on  3  or  more
categorical variables. In  this case, you can use  the `ftable()`  or `structable()`
function to print the results more attractively.

```{r, table-ex2}
# 3-Way Frequency Table
mytable <- table(A, B, C)
ftable(mytable)
```

`table()`  ignores missing values by default. 
To include `NA` as a category in counts, include the
table option  `exclude=NULL` if  the variable  is a  vector. If  the variable is a
factor you  have to  create a  new factor  using \code{newfactor  <- factor(oldfactor,
exclude=NULL)}. 

## `xtabs()` {#sec:xtabs}

The `xtabs()` function allows you to create cross-tabulations of data using formula style input.
This typically works with case-form data supplied in a data frame or a matrix.
The result is a contingency table in array format, whose dimensions are determined by
the terms on the right side of the formula.

```{r, xtabs-ex1}
# 3-Way Frequency Table
mytable <- xtabs(~A+B+C, data=mydata)

ftable(mytable)    # print table

summary(mytable)   # chi-square test of indepedence
```

If a variable is included on the left side of the formula, it is assumed to be a
vector  of  frequencies  (useful if the data have already been tabulated in frequency form).

```{r, xtabs-ex2}
(GSStab <- xtabs(count ~ sex + party, data=GSS))

summary(GSStab)
```

## Collapsing over table factors: `aggregate()`, `margin.table()` and `apply()`

It sometimes happens that we have a data set with more variables or factors than 
we want to analyse, or else, having done some initial analyses, we decide that
certain factors are not important, and so should be excluded from graphic displays
by collapsing (summing) over them.  For example, mosaic plots and fourfold displays
are often simpler to construct from versions of the data collapsed over
the factors which are not shown in the plots.

The appropriate tools to use again depend on
the form in which the data are represented--- a case-form data frame, a
frequency-form data frame (`aggregate()`), or a table-form array or 
table object (`margin.table()` or `apply()`).

When the data are in frequency form, and we want to produce another
frequency data frame, `aggregate()` is a handy tool, using
the argument `FUN=sum` to sum the frequency variable over the 
factors *not* mentioned in the formula.

***Example***: 
The data frame `DaytonSurvey` in the `vcdExtra` package represents a
$2^5$ table giving the frequencies of reported use (``ever used?'') of 
alcohol, cigarettes and marijuana in a sample of high school seniors,
also classified by sex and race.  

```{r, dayton1}
data("DaytonSurvey", package="vcdExtra")
str(DaytonSurvey)
head(DaytonSurvey)
```

To focus on the associations among the
substances, we want to collapse over sex and race. The right-hand side of the formula
used in the call to `aggregate()` gives the factors to be retained in the
new frequency data frame, `Dayton.ACM.df`.

```{r, dayton2}
# data in frequency form
# collapse over sex and race
Dayton.ACM.df <- aggregate(Freq ~ cigarette+alcohol+marijuana, 
                           data=DaytonSurvey, 
                           FUN=sum)
Dayton.ACM.df
```

When the data are in table form, and we want to produce another
table, `apply()` with `FUN=sum` can be used in a similar way
to sum the table over dimensions not mentioned in the `MARGIN`
argument.  `margin.table()` is just a wrapper for `apply()`
using the `sum()` function.


***Example***: 
To illustrate, we first convert the `DaytonSurvey` to a 5-way
table using `xtabs()`, giving `Dayton.tab`.  

```{r, dayton3}
# in table form
Dayton.tab <- xtabs(Freq ~ cigarette+alcohol+marijuana+sex+race, 
                    data=DaytonSurvey)
structable(cigarette+alcohol+marijuana ~ sex+race, 
           data=Dayton.tab)
```

Then, use `apply()` on `Dayton.tab` to give the
3-way table `Dayton.ACM.tab` summed over sex and race.
The elements in this new table are the column sums for 
`Dayton.tab` shown by `structable()` just above.

```{r, dayton4}
# collapse over sex and race
Dayton.ACM.tab <- apply(Dayton.tab, MARGIN=1:3, FUN=sum)
Dayton.ACM.tab <- margin.table(Dayton.tab, 1:3)   # same result

structable(cigarette+alcohol ~ marijuana, data=Dayton.ACM.tab)
```


Many of these operations can be performed using the `**ply()` functions
in the [`plyr`]( https://CRAN.R-project.org/package=plyr) package.
For example, with the data in a frequency form data frame, use `ddply()`
to collapse over unmentioned factors, and `plyr::summarise()`
as the function to be applied to each piece.
<!-- \footnote{ -->
<!-- Ugh. This `plyr` function clashes with a function of the same name in `vcdExtra`. -->
<!-- In this document I will use the explicit double-colon notation to keep them -->
<!-- separate. -->
<!-- } -->

```{r, dayton5}
library(plyr)
Dayton.ACM.df <- plyr::ddply(DaytonSurvey, 
                             .(cigarette, alcohol, marijuana), 
                             plyr::summarise, Freq=sum(Freq))

Dayton.ACM.df
```

## Collapsing table levels: `collapse.table()`

A related problem arises when we have a table or array and for some purpose
we want to reduce the number of levels of some factors by summing subsets
of the frequencies.  For example, we may have initially coded Age in 10-year
intervals, and decide that, either for analysis or display purposes, we
want to reduce Age to 20-year intervals.  The `collapse.table()` function
in `vcdExtra` was designed for this purpose.

***Example***:
Create a 3-way table, and collapse Age from 10-year to 20-year intervals. 
First, we generate a $2 \times 6 \times 3$ table of random counts from a 
Poisson distribution with mean of 100.
```{r, collapse1}
# create some sample data in frequency form
sex <- c("Male", "Female")
age <- c("10-19", "20-29",  "30-39", "40-49", "50-59", "60-69")
education <- c("low", 'med', 'high')
data <- expand.grid(sex=sex, age=age, education=education)
counts <- rpois(36, 100)   # random Possion cell frequencies
data <- cbind(data, counts)

# make it into a 3-way table
t1 <- xtabs(counts ~ sex + age + education, data=data)
structable(t1)
``` 
Now collapse `age` to 20-year intervals, and `education`
to 2 levels. In the arguments, levels of `age` and `education`
given the same label are summed in the resulting smaller table.
```{r, collapse2}
# collapse age to 3 levels, education to 2 levels
t2 <- collapse.table(t1, 
         age=c("10-29", "10-29",  "30-49", "30-49", "50-69", "50-69"),
         education=c("<high", "<high", "high"))
structable(t2)
```

## Collapsing table levels: `dplyr`

For data sets in frequency form or case form, factor levels can be collapsed by recoding the levels
to some grouping. One handy function for this is `dplyr::case_match()`

***Example***:

The `vcdExtra::Titanicp` data set contains information on 1309 passengers on the _RMS Titanic_,
including `sibsp`, the number of (0:8) siblings or spouses aboard, and `parch` (0:6), the
number of parents or children aboard, but the table is quite sparse.

```{r titanicSAVE, include=FALSE, echo=FALSE}
# Save unmodified Titanicp data for collapse_levels() example
saveDat <- Titanicp
```

```{r titanicp1}
table(Titanicp$sibsp, Titanicp$parch)
```

For purposes of analysis, we might want to collapse both of these to the levels `0, 1, 2+`.
Here's how:

```{r titanicp2}
library(dplyr)
Titanicp <- Titanicp |>
  mutate(sibspF = case_match(sibsp,
                            0 ~ "0",
                            1 ~ "1",
                            2:max(sibsp) ~ "2+")) |>
  mutate(sibspF = ordered(sibspF)) |>
  mutate(parchF = case_match(parch,
                             0 ~ "0",
                             1 ~ "1",
                             2:max(parch) ~ "2+")) |>
  mutate(parchF = ordered(parchF)) 

table(Titanicp$sibspF, Titanicp$parchF)
```

`car::recode()` is a similar function, but with a less convenient interface.

The [`forcats`]( https://CRAN.R-project.org/package=forcats) package provides a collection of functions for reordering the levels of a factor
or grouping categories according to their frequency:


*    `forcats::fct_reorder()`: Reorder a factor by another variable.
*    `forcats::fct_infreq()`: Reorder a factor by the frequency of values.
*    `forcats::fct_relevel()`: Change the order of a factor by hand.
*    `forcats::fct_lump()`: Collapse the least/most frequent values of a factor into “other”.
*    `forcats::fct_collapse()`: Collapse factor levels into manually defined groups.
*    `forcats::fct_recode()`: Change factor levels by hand.

## Collapsing table levels: `collapse_levels()`

As seen in the previous section, collapsing variables from datasets in frequency 
or case form (in R) is not as intuitive/easy as it should be. Thus, the
`collapse_levels()` function was written to easily collapse variables contained
in datasets of *any form*.

Note that when your object `X` is in frequency form, an argument of 
`freq = "your frequency column name"` must be supplied. This argument defaults
to a value of `freq = "Freq"`.

The use of this function is illustrated below using the same `Titanicp` data.

***Example***
First, factorize the `sibsp` and `parch` variables. Then collapse variables into
categories of `0`, `1`, and `2+`.

```{r titanicLOAD, include=FALSE, echo=FALSE}
# Load unmodified Titanicp data for collapse_levels() example
Titanicp <- saveDat
```

```{r collapse_level-ex1}
str(Titanicp) # The original dataset

Titanicp <- within(Titanicp, {
  parchF <- factor(parch)
  sibspF <- factor(sibsp)
})

Titanicp <- collapse_levels(
  Titanicp,
  parchF = list(
    "2+" = c("2", "3", "4", "5", "6", "9")
  ),
  sibspF = list(
    "2+" = c("2", "3", "4", "5", "8")
  )
)

table(Titanicp$sibspF, Titanicp$parchF)
```

For a more thorough overview of `collapse_levels()`, see
[1a. Steps Toward Tidy Categorical Data Analysis](a1a-convert-collapse.html).

## Converting among frequency tables and data frames

As we've seen, a given contingency table can be represented 
equivalently in different forms,
but some R functions were designed for one particular representation.

The table below shows some handy tools for converting from one form to another.

<!-- [htb] -->
<!--  \caption{Tools for converting among different forms for categorical data} {#tab:convert} -->
 
<!--    {llll} -->
<!-- 	\hline -->
<!--                  & \multicolumn{3}{c}{**To this**} \\ -->
<!-- 	**From this**      &     Case form         & Frequency form             &  Table form \\ -->
<!-- 	\hline -->
<!-- 	Case form      &   noop                 & `xtabs(~A+B)`        &  `table(A,B)`  \\  -->
<!-- 	Frequency form &  `expand.dft(X)`  & noop                      & `xtabs(count~A+B)`\\ -->
<!-- 	Table form     &  `expand.dft(X)`  & `as.data.frame(X)`   &  noop \\ -->
<!-- 	\hline -->
   
| **From this**    |                     | **To this**          |                   |
|:-----------------|:--------------------|:---------------------|-------------------|
|	                 |    _Case form_      | _Frequency form_     | _Table form_      |
|	_Case form_      |   noop              | `xtabs(~A+B)`        |  `table(A,B)`     |
|	_Frequency form_ |  `expand.dft(X)`    | noop                 | `xtabs(count~A+B)`|
|	_Table form_     |  `expand.dft(X)`    | `as.data.frame(X)`   |  noop             |


For example, a contingency table in table form (an object of `class(table)`) can be converted
to a data.frame with `as.data.frame()`.
^[Because R is object-oriented, this is actually a short-hand for the function `as.data.frame.table()`.] 
The resulting
`data.frame` contains columns
representing the classifying factors and the table entries (as a column named by
the `responseName` argument, defaulting to `Freq`. This is the inverse of `xtabs()`.

***Example***: 
Convert the `GSStab` in table form to a data.frame in frequency form.
```{r, convert-ex1}
as.data.frame(GSStab)
```

***Example***: Convert the `Arthritis` data in case form to a 3-way table of
`Treatment` $\times$ `Sex` $\times$ `Improved`.
Note the use of `with()` to avoid having to use `Arthritis\$Treatment` etc. within the call to `table()`.%
^[`table()` does not allow a `data` argument to provide an environment in which the table variables are to be found.  In the examples in \@ref(sec:table) I used `attach(mydata)` for this purpose,
but `attach()` leaves the variables in the global environment, while `with()` just evaluates the `table()` expression in a temporary environment of the data.]

```{r, convert-ex2}
Art.tab <- with(Arthritis, table(Treatment, Sex, Improved))
str(Art.tab)

ftable(Art.tab)
```

There may also be times that you will need an equivalent case form `data.frame`
with factors  representing the table variables
rather than the frequency  table.
For example, the `mca()` function in package `MASS`
only operates on data in this format. 
Marc Schwartz initially provided code for `expand.dft()` on the  Rhelp
mailing list for converting a table back into a case form `data.frame`.
This function is included in `vcdExtra`.

***Example***: Convert the `Arthritis` data in table form (`Art.tab`) back to a `data.frame`
in case form, with factors
`Treatment`, `Sex` and `Improved`.
```{r, convert-ex3}
Art.df <- expand.dft(Art.tab)
str(Art.df)
```

### Tidy conversions

To make conversions more intuitive, `vcdExtra` includes tidy conversion
functions `as_table()`, `as_matrix()`, `as_array()`, `as_freqform()`, and `as_caseform()` to easily convert any form to the form of the function's
namesake (i.e., `as_table()` converts any form into table form). Arguments in
these functions are of similar format, with conversions *from* frequency form
requiring the column containing frequencies to be specified via the `freq`
argument. Likewise, `dims` (dimensions) may be specified, with output summing
over excluded variables. For a more thorough overview of these functions, see
[1a. Steps Toward Tidy Categorical Data Analysis](a1a-convert-collapse.html).

***Example***: 
Convert the `GSStab` in table form to a `data.frame` in frequency form.
```{r, tidyconvert-ex1}
# tidy = TRUE (default) returns a tibble
GSS.ff <- as_freqform(GSStab, tidy = FALSE)
GSS.ff
```

***Example***: 
Convert the `GSStab` data in frequency form (`GSS.ff`) back to table form.
```{r, tidyconvert-ex2}
as_table(GSS.ff, freq = "Freq") # When present, freq column must be specified
```

***Example***: Convert the `Arthritis` data in case form to a 3-way table of
`Treatment` $\times$ `Sex` $\times$ `Improved`.
```{r, tidyconvert-ex3}
tidy_Art.tab <- as_table(Arthritis, dims = c("Treatment", "Sex", "Improved"))
str(tidy_Art.tab)

ftable(tidy_Art.tab)
```

***Example***: Convert the `Arthritis` data in table form (`tidy_Art.tab`) back 
to a `data.frame` in case form, with factors `Treatment`, `Sex` and `Improved`.
```{r, tidyconvert-ex4}
tidy_Art.df <- as_caseform(tidy_Art.tab, tidy = FALSE)
str(tidy_Art.df)
```

## A complex example {#sec:complex}

If you've followed so far, you're ready for a more complicated example.
The data file, `tv.dat` represents a 4-way table of size
$5 \times 11 \times 5 \times 3$ where the table variables (unnamed in the file)
are read as `V1` -- `V4`, and the cell frequency is read
as `V5`.  The file, stored in the `doc/extdata` directory
of `vcdExtra`, can be read as follows:
```{r, tv1}
tv.data<-read.table(system.file("extdata","tv.dat", package="vcdExtra"))
head(tv.data,5)
```
For a local file, just use `read.table()` in this form:
```{r, tv2,eval=FALSE}
tv.data<-read.table("C:/R/data/tv.dat")
```
The data `tv.dat` came from the initial implementation of 
mosaic displays in R by Jay Emerson. 
In turn, they came from the initial development of mosaic displays 
[@vcd:Hartigan+Kleiner:1984]
that illustrated the method with data on a large sample of TV viewers
whose behavior had been recorded for the Neilsen ratings.
This data set contains sample television audience data from Neilsen
Media Research for the week starting November 6, 1995.

The table variables are:

* `V1`-- values 1:5 correspond to the days Monday--Friday;
* `V2`-- values 1:11 correspond to the quarter hour times 8:00PM through 10:30PM;
* `V3`-- values 1:5 correspond to ABC, CBS, NBC, Fox, and non-network choices;
* `V4`-- values 1:3 correspond to transition states: turn the television Off, Switch channels, or Persist in viewing the current channel.

We are interested just the cell frequencies, and rely on the facts that the

(a) the table is complete--- there are no missing cells, so `nrow(tv.data)` = `r nrow(tv.data)`;
(b) the observations are ordered so that `V1` varies most rapidly and `V4` most slowly.  From this, we can just extract the frequency column and reshape it into an array. [That would be dangerous if any observations were out of order.]

```{r, tv3}
TV <- array(tv.data[,5], dim=c(5,11,5,3))                                        
dimnames(TV) <- list(c("Monday","Tuesday","Wednesday","Thursday","Friday"), 
                     c("8:00","8:15","8:30","8:45","9:00","9:15","9:30",         
                       "9:45","10:00","10:15","10:30"),                            
                     c("ABC","CBS","NBC","Fox","Other"), 
                     c("Off","Switch","Persist"))

names(dimnames(TV))<-c("Day", "Time", "Network", "State")
```

More generally (even if there are missing cells), we can 
use `xtabs()` (or `plyr::daply()`)
to do the cross-tabulation, using `V5` as the
frequency variable.  Here's how to do this same operation with `xtabs()`:
```{r, tv3a,eval=FALSE}
TV <- xtabs(V5 ~ ., data=tv.data)
dimnames(TV) <- list(Day = c("Monday","Tuesday","Wednesday","Thursday","Friday"), 
                     Time = c("8:00","8:15","8:30","8:45","9:00","9:15","9:30",         
                              "9:45","10:00","10:15","10:30"),                            
                     Network = c("ABC","CBS","NBC","Fox","Other"), 
                     State = c("Off","Switch","Persist"))

# table dimensions
dim(TV)
```

But this 4-way table is too large and awkward to work with. Among the networks,
Fox and Other occur infrequently. 
We can also cut it down to a 3-way table by considering only viewers who persist
with the current station.
^[This relies on the fact that that indexing an array drops dimensions of length 1 by default, using the argument `drop=TRUE`; the result is coerced to the lowest possible dimension.]


```{r, tv4}
TV2 <- TV[,,1:3,]      # keep only ABC, CBS, NBC
TV2 <- TV2[,,,3]       # keep only Persist -- now a 3 way table
structable(TV2)
```

Finally, for some purposes, we might want to collapse the 11 times into a smaller number.
Half-hour time slots make more sense.
Here, we use `as.data.frame.table()` to convert the table back to a data frame,
 `levels()` to re-assign the values of `Time`,
and finally, `xtabs()` to give a new, collapsed frequency table.

```{r, tv5}
TV.df <- as.data.frame.table(TV2)
levels(TV.df$Time) <- c(rep("8:00", 2),
                        rep("8:30", 2),
                        rep("9:00", 2), 
                        rep("9:30", 2), 
                        rep("10:00",2),
                            "10:30"
                        )

TV3 <- xtabs(Freq ~ Day + Time + Network, TV.df)

structable(Day ~ Time+Network, TV3)
```

We've come this far, so we might as well show a mosaic display. This is analogous to that used by
@vcd:Hartigan+Kleiner:1984. 
```{r tv-mosaic1, fig.height=6, fig.width=7}
mosaic(TV3, shade = TRUE,
       labeling = labeling_border(rot_labels = c(0, 0, 0, 90)))
```

This mosaic displays can be read at several levels, corresponding to the successive splits of
the tiles and the residual shading.
Several trends are clear for viewers who persist:

* Overall, there are about the same number of viewers on each weekday, with slightly more on Thursday.
* Looking at time slots, viewership is slightly greater from 9:00 - 10:00 overall and also
8:00 - 9:00 on Thursday and Friday

From the residual shading of the tiles:

* Monday: CBS dominates in all time slots.
* Tuesday" ABC and CBS  dominate after 9:00
* Thursday: is a largely NBC day
* Friday: ABC dominates in the early evening

<!-- ```{r, tv4} -->
<!-- TV.df <- as.data.frame.table(TV) -->
<!-- levels(TV.df$Time) <- c(rep("8:00-8:59",4), -->
<!--                         rep("9:00-9:59",4),  -->
<!--                         rep("10:00-10:44",3)) -->
<!-- TV3 <- xtabs(Freq ~ Day + Time + Network, TV.df) -->

<!-- structable(Day ~ Time+Network, TV3) -->
<!-- ``` -->


<!-- Whew! See \figref{fig:TV-mosaic} for a mosaic plot of the `TV3` data. -->

<!-- The table is too large to display conveniently, but we can show a subtable -->
<!-- by selecting the indices.  This call to `ftable()` subsets the first three -->
<!-- levels of each factor. -->

<!-- ```{r tv4} -->
<!-- ftable(TV[1:3,1:3,1:3,1:3], col.vars = 3:4) -->
<!-- ``` -->

<!-- We've come this far, so we might as well show a mosaic display. This is analogous to that used by -->
<!-- @vcd:Hartigan+Kleiner:1984. The result is too complex to see very much.  It would be useful to simplify -->
<!-- the table by collapsing one or more of the table dimensions, e.g., `Time`. -->

<!-- ```{r tv-mosaic2, fig.height=7, fig.width=7} -->
<!-- mosaic(TV[,,1:4,], shade=TRUE,  -->
<!--        labeling_args = list(abbreviate_labs = c(8,5,1,1))) -->

<!-- ``` -->





# References

---
title: "3. Loglinear Models"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{3. Loglinear Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/tut03-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(HairEyeColor)
data(PreSex)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
#if(!file.exists("fig")) dir.create("fig")

```

You can  use the  `loglm()`  function in  the `MASS`  package to fit log-linear
models.  Equivalent models can also be fit (from a different perspective) as generalized
linear models with the `glm()`  function using the `family='poisson'` argument,
and the `gnm` package provides a wider range of generalized *nonlinear* models,
particularly for testing structured associations.

The visualization methods for these models were originally developed for models fit using `loglm()`,
so this approach is emphasized here.  Some extensions of these methods for models
fit using `glm()` and `gnm()` are contained in the `vcdExtra` package
and illustrated in @ref(sec:glm).

Assume we  have a 3-way  contingency table based  on
variables A, B, and C.
The possible different forms of loglinear models for a 3-way table are shown in the table below.
\@(tab:loglin-3way)
The **Model formula** column shows how to express each model for `loglm()` in R.
^[For `glm()`, or `gnm()`, with the data in the form of a frequency data.frame, the same model is specified
in the form `glm(Freq` $\sim$ `..., family="poisson")`, where `Freq` is  the name
of the cell frequency variable and `...` specifies the *Model formula*.]
In the **Interpretation** column, the symbol "$\perp$" is to be read as "is independent of,"
and "$\;|\;$" means "conditional on," or "adjusting for," or just "given".

| **Model**                | **Model formula**  | **Symbol**     | **Interpretation**     |
|:-------------------------|:-------------------|:---------------|:-----------------------|
| Mutual independence      | `~A + B + C`       | $[A][B][C]$    | $A \perp B \perp C$    |
| Joint independence       | `~A*B + C`         | $[AB][C]$      | $(A \: B) \perp C$     | 
| Conditional independence | `~(A+B)*C`         | $[AC][BC]$     | $(A \perp B) \;|\; C$  | 
| All two-way associations | `~A*B + A*C + B*C` | $[AB][AC][BC]$ | homogeneous association|
| Saturated model          | `~A*B*C`           | $[ABC]$        | 3-way association      |




For example, the formula `~A + B + C` specifies the model of *mutual independence* with
no associations among the three factors.  In standard notation for the expected frequencies
$m_{ijk}$, this corresponds to

$$	\log ( m_{ijk} ) = \mu + \lambda_i^A + \lambda_j^B + \lambda_k^C \equiv  A + B + C $$

The parameters $\lambda_i^A , \lambda_j^B$ and  $\lambda_k^C$ pertain to the differences among the
one-way marginal frequencies for the factors A, B and C. 

Similarly, the model of *joint independence*,  $(A \: B) \perp C$, allows an association between A and B, but specifies that
C is independent of both of these and their combinations,

$$	\log ( m_{ijk} ) = \mu + \lambda_i^A + \lambda_j^B + \lambda_k^C + \lambda_{ij}^{AB} \equiv A * B + C $$

where the parameters $\lambda_{ij}^{AB}$ pertain to the overall association between A and B (collapsing over C).

In the literature or text books, you will often find these models expressed in shorthand symbolic notation,
using brackets, `[ ]` to enclose the *high-order terms* in the model.
Thus, the joint independence model can be denoted `[AB][C]`, as shown in the **Symbol** column in the table.
\@(tab:loglin-3way).

Models of *conditional independence* allow (and fit)  two of the three possible
two-way associations.  There are three such models, depending on which variable is conditioned upon.
For a given conditional independence model, e.g., `[AB][AC]`, the given variable is the one
common to all terms, so this example has the interpretation $(B \perp C) \;|\; A$.


## Fitting with `loglm()` {#sec:loglm}

For example, we can fit the model of mutual independence among hair color, eye color and sex 
in `HairEyeColor` as

```{r, loglm-hec1}
library(MASS)
## Independence model of hair and eye color and sex.  
hec.1 <- loglm(~Hair+Eye+Sex, data=HairEyeColor)
hec.1
```

Similarly, the models of conditional independence and joint independence are specified as
```{r, loglm-hec2}
## Conditional independence
hec.2 <- loglm(~(Hair + Eye) * Sex, data=HairEyeColor)
hec.2
```
```{r, loglm-hec3}
## Joint independence model.  
hec.3 <- loglm(~Hair*Eye + Sex, data=HairEyeColor)
hec.3
```
Note that printing the model gives a brief summary of the goodness of fit.
A set of models can be compared using the `anova()` function.

```{r, loglm-anova}
anova(hec.1, hec.2, hec.3)
```
## Fitting with `glm()` and `gnm()` {#sec:glm}

The `glm()` approach, and extensions of this in the `gnm` package allows a
much wider class of models for frequency data to be fit than can be handled by
`loglm()`.  Of particular importance are models for ordinal factors and for
square tables, where we can test more structured hypotheses about the patterns
of association than are provided in the tests of general association under
`loglm()`. These are similar in spirit to the 
non-parametric CMH tests described in \@ref(sec:CMH).

***Example***:
The data `Mental` in the `vcdExtra` package gives a two-way table in frequency form
classifying young people by
their mental health status and parents' socioeconomic status (SES), where
both of these variables are ordered factors.

```{r, mental1}
data(Mental, package = "vcdExtra")
str(Mental)
xtabs(Freq ~ mental + ses, data=Mental)   # display the frequency table
```
Simple ways of handling ordinal variables involve assigning scores to the table
categories, and the simplest cases are to use integer scores, either for the row variable (``column
effects'' model), the column variable (``row effects'' model), or both (``uniform association''
model).

```{r, mental2}
indep <- glm(Freq ~ mental + ses, family = poisson, data = Mental)  # independence model
```

To fit more parsimonious models than general association, we can define
numeric scores for the row and column categories
```{r, mental3}
# Use integer scores for rows/cols 
Cscore <- as.numeric(Mental$ses)
Rscore <- as.numeric(Mental$mental)	
```

Then, the row effects model, the column effects model, and the uniform association
model can be fit as follows. The essential idea is to replace a factor variable
with its numeric equivalent in the model formula for the association term.

```{r, mental4}
# column effects model (ses)
coleff <- glm(Freq ~ mental + ses + Rscore:ses, family = poisson, data = Mental)

# row effects model (mental)
roweff <- glm(Freq ~ mental + ses + mental:Cscore, family = poisson, data = Mental)

# linear x linear association
linlin <- glm(Freq ~ mental + ses + Rscore:Cscore, family = poisson, data = Mental)
```

The `LRstats()` function in `vcdExtra` provides a nice, compact summary of
the fit statistics for a set of models, collected into a *glmlist* object.
Smaller is better for AIC and BIC.

```{r, mental4a}
# compare models using AIC, BIC, etc
vcdExtra::LRstats(glmlist(indep, roweff, coleff, linlin))
```

For specific model comparisons, we can also carry out tests of *nested* models with
`anova()` when those models are listed from smallest to largest.
Here, there are two separate paths from the most restrictive (independence) model
through the model of uniform association, to those that allow only one of
row effects or column effects.


```{r, mental5}
anova(indep, linlin, coleff, test="Chisq")	
anova(indep, linlin, roweff, test="Chisq")	
```
The model of linear by linear association seems best on all accounts.
For comparison, one might try the CMH tests on these data:

```{r, mental6}
CMHtest(xtabs(Freq~ses+mental, data=Mental))
```
## Non-linear terms

The strength of the `gnm` package is that it handles a wide variety of models
that handle non-linear terms, where the parameters enter the model beyond a simple
linear function.
The simplest example is the Goodman RC(1) model [@Goodman:79],
which allows a multiplicative
term to account for the association of the table variables.
In the notation of generalized linear models with a log link, this can be expressed as

$$ \log \mu_{ij} = \alpha_i + \beta_j + \gamma_{i} \delta_{j}	,$$

where the row-multiplicative effect parameters $\gamma_i$ and 
corresponding column parameters $\delta_j$ are estimated from the data.%
^[This is similar in spirit to a correspondence analysis with a single dimension, but as a statistical model.]

Similarly, the RC(2) model adds two multiplicative terms to
the independence model,

$$ \log \mu_{ij} = \alpha_i + \beta_j + \gamma_{i1} \delta_{j1} + \gamma_{i2} \delta_{j2} . $$

In the `gnm` package, these models may be fit using the `Mult()`
to specify the multiplicative term, and `instances()` to specify several
such terms.

***Example***:
For the `Mental` data, we fit the RC(1) and RC(2) models, and compare
these with the independence model.

```{r, mental7}
RC1 <- gnm(Freq ~ mental + ses + Mult(mental,ses), data=Mental, 
             family=poisson, verbose=FALSE)
RC2 <- gnm(Freq ~ mental+ses + instances(Mult(mental,ses),2), data=Mental, 
             family=poisson, verbose=FALSE)
anova(indep, RC1, RC2, test="Chisq")
```




## References

---
title: "Mobility tables"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib", "vignettes.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Mobility tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/mobility-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# colorize text
colorize <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}
```

## Social mobility

Social mobility is an important concept in sociology, and its' study has led to
a wide range of developments in categorical data analysis in what are often called
_mobility tables_. 

The idea is to study the movement of individuals, families, households or other categories of people within or between social strata in a society, across time or space. This refers to
a change in social status relative to one's current social location within a given society.

Using survey data, the most frequent examples relate to changes in income or wealth, but most often this is studied via classification in occupational categories ("professional, "managerial", "skilled manual", ...). Most often this is studied _intergenerationaly_ using the occupational
categories of fathers and sons.

Mobility tables are nearly always _square_ tables, with the same categories for the row
and column variables. As such, they nearly always exhibit positive associations
along the diagonal cells.  What is of interest are specialized models, intermediate
between the null model of independence and the saturated model.

### Models

These models include important special cases:

- **quasi-independence**: Ignoring diagonal cells, are the row and column variables independent?
- **symmetry**: Are associations above the diagonal equal to the corresponding ones below the diagonal?
- **row effects, col effects, linear x linear**: Typically, the factors in such tables are ordinal. To what extent can the models be simplified by assigning integer scores to the
row, column categories or both?
- **multiplicative RC**: RC models attempt to estimate the scores for the row and column categories. 
- **topographical models**: It is possible that the associations among occupational categories
exhibit consistent patterns according to their nature. These models allow specifying a
theoretically interesting pattern.
- **crossings models**: assert that there are different difficulty parameters for crossing from category to the next and associations between categories decrease with their separation.

While standard loglinear models can be fit using `MASS::loglm`, these models require
use of ``stats::glm()` or `gnm::gnm()`, as I illustrate below.

## Hauser data

This vignette uses the `vcdExtra::Hauser79` dataset, a cross-classification of 19,912 individuals by father's occupation and son's first occupation for U.S. men aged 20-64 in 1973.
The data comes from @Hauser:79 and has been also analysed by @PowersXie:2008.
The discussion draws on @FriendlyMeyer:2016:DDAR, Ch. 10.

```{r hauser-data}
data("Hauser79", package="vcdExtra")
str(Hauser79)
(Hauser_tab <- xtabs(Freq ~ Father + Son, data=Hauser79))
```

As can be seen, `Hauser79` is a data.frame in frequency form.
The factor levels in this table are a coarse grouping of occupational categories, so:

- `UpNM` = professional and kindred workers, managers and officials, and non-retail sales workers;
- `LoNM` = proprietors, clerical and kindred workers, and retail sales workers; 

- `UpM` = craftsmen, foremen, and kindred workers;

- `LoM` =  service workers, operatives and kindred workers, and laborers (except farm); 

- `Farm` = farmers and farm managers, farm laborers, and foremen.

### Load packages

```{r load}
library(vcdExtra)
library(gnm)
library(dplyr)
```

### Mosaic plots

`Hauser_tab` is a `table` object, and the
simplest plot for the frequencies is the default `plot()` method, giving a `graphics::mosaicplot()`.

```{r mosaicplot}
plot(Hauser_tab, shade=TRUE)
```

The frequencies are first split according to father's occupational category 
(the first table dimension) and then by sons' occupation.
The most common category for fathers is lower manual, followed by farm.

`mosaicplot()`, using `shade=TRUE` colors the tiles according to the 
sign and magnitude of the residuals from an independence model: shades of 
`r colorize("positive", "blue")` for positive residuals and
`r colorize("negative", "red")` red for negative residuals.

`vcd::mosaic()` gives a similar display, but is much more flexible in the labeling of
the row and column variable, labels for the categories, and the scheme used for shading
the tiles. Here, I simply assign longer labels for the row and column variables,
using the `labeling_args` argument to `mosaic()`.

```{r mosaic1}
labels <- list(set_varnames = c(Father="Father's occupation", 
                                Son="Son's occupation"))

mosaic(Freq ~ Father + Son, data=Hauser79,
       labeling_args = labels,
       shade=TRUE,
       legend = FALSE)
```

### Fitting and graphing models

The call to `vcd::mosaic()` above takes the `Hauser79` dataset as input.
Internally, it fits the model of independence and displays the result,
but for more complex tables, control of the fitted model is limited.

Unlike `mosaicplot()` and even the [`ggmosaic`]( https://CRAN.R-project.org/package=ggmosaic) package,
`vcdExtra::mosaic.glm()` is a `mosaic` **method** for `glm` objects.
This means you can fit any model, and supply the model object to `mosaic()`.
(Note that in `mosaic()`, the `formula` argument determines the order of
splitting in the mosaic, not a loglinear formula.)

```{r indep}
hauser.indep <- glm(Freq ~ Father + Son, 
  data=Hauser79, 
  family=poisson)

# the same mosaic, using the fitted model
mosaic(hauser.indep, formula = ~ Father + Son, 
       labeling_args = labels,
       legend = FALSE,
       main="Independence model")
```

## Quasi-independence

Among the most important advances from the social mobility literature is the idea that
associations between row and column variables in square tables can be explored in greater
depth if we ignore the obvious association in the diagonal cells.
The result is a model of _quasi-independence_, asserting that fathers' and sons' occupations
are independent, ignoring the diagonal cells. 

For a two-way table, quasi-independence can be expressed as
$$
 \pi_{ij} = \pi_{i+} \pi_{+j} \quad\quad \mbox{for } i\ne j
$$
or in loglinear form as:
$$
 \log m_{ij} = \mu + \lambda_i^A + \lambda_j^B + \delta_i I(i=j)
 \quad .
$$
This model effectively adds one parameter, $\delta_i$, for each main diagonal cell
and fits those frequencies perfectly.


In the [`gnm`]( https://CRAN.R-project.org/package=gnm) package,
`gnm::Diag()` creates the appropriate term in the model formula,
using a symbol in the diagonal cells and "." otherwise.

```{r Diag}
# with symbols
with(Hauser79, Diag(Father, Son)) |> matrix(nrow=5)
```

We proceed to fit and plot the quasi-independence model by updating the
independence model, adding the term `Diag(Father, Son)`.
```{r quasi}
hauser.quasi <-  update(hauser.indep, 
                        ~ . + Diag(Father, Son))

mosaic(hauser.quasi, ~ Father+Son, 
       labeling_args = labels,
       legend = FALSE,
       main="Quasi-independence model")

```

Note that the pattern of residuals shows a systematic pattern of 
`r colorize("positive", "blue")` and 
`r colorize("negative", "red")` residuals above and below the diagonal tiles.
We turn to this next.

### Symmetry and quasi-symmetry

Another advance from the social mobility literature was the idea of how to test for 
_differences_ in occupational categories between fathers and sons.
The null hypothesis of no systematic differences can be formulated as a 
test of **symmetry** in the table, 
$$
 \pi_{ij} = \pi_{ji} \quad\quad \mbox{for } i\ne j \quad ,
$$
which asserts that sons are as likely to move from their father's occupation $i$
to another category $j$ as they were to move in the reverse direction, $j$ to $i$.
An alternative, "Upward mobility",
i.e., that sons who did not stay in their father's occupational category moved
to a higher category on average would mean that 
$$
 \pi_{ij} < \pi_{ji} \quad\quad \mbox{for } i\ne j
$$
Yet this model is overly strong, because it also asserts **marginal homogeneity**,
that the marginal probabilities of row and column values are equal,
$\pi_{i+} = \pi_{+i}$ for all $i$. Consequently, this hypothesis is most often
tested as a model for **quasi-symmetry**, that also ignores the diagonal cells.

Symmetry is modeled by the function `gnm::Symm()`. It returns a factor with
the same labels for positions above and below the diagonal.

```{r symm}
with(Hauser79, Symm(Father, Son)) |> matrix(nrow=5)
```

To fit the model of quasi-symmetry, add both `Diag()` and `Symm()` to the model of independence.
```{r qsymm}
hauser.qsymm <-  update(hauser.indep, 
                        ~ . + Diag(Father,Son) + Symm(Father,Son))
```

To compare the models so far, we can use `anova()` or `vcdExtra::LRstats():

```{r anova1}
anova(hauser.indep, hauser.quasi, hauser.qsymm, test="Chisq")

LRstats(hauser.indep, hauser.quasi, hauser.qsymm)
```

This `hauser.qsymm` model represents a huge improvement in goodness of fit.
With such a large sample size, it might be considered an acceptable
fit.

But, this model of quasi-symmetry still shows some residual lack of fit.
To visualize this in the mosaic, we can label the cells with their
standardized residuals.
```{r qsymm-mosaic}
mosaic(hauser.qsymm, ~ Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Quasi-symmetry model")
```

The cells with the largest lack of symmetry (using standardized residuals)
are those for the upper and lower non-manual occupations, where the son of
an upper manual worker is less likely to move to lower non-manual
work than the reverse.

### Topological models

It is also possible that there are more subtle patterns of association one
might want to model, with specific parameters for particular combinations of the
occupational categories (beyond the idea of symmetry).
@Hauser:79 developed this idea in what are now called
**topological** models or **levels** models, where an arbitrary pattern of
associations can be specified, implemented in `gnm::Topo()`.

```{r topo-levels}
# Levels for Hauser 5-level model
levels <- matrix(c(
      2,  4,  5,  5,  5,
      3,  4,  5,  5,  5,
      5,  5,  5,  5,  5,
      5,  5,  5,  4,  4,
      5,  5,  5,  4,  1), 
      nrow = 5, ncol = 5, 
      byrow=TRUE)
```

```{r topo-mosaic}
hauser.topo <- update(hauser.indep, 
                      ~ . + Topo(Father, Son, spec=levels))

mosaic(hauser.topo, ~Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Topological model")
```

Comparing models, we can see that the model of quasi-symmetry is the best
so far, using AIC as the measure:

```{r}
LRstats(hauser.indep, hauser.quasi, hauser.qsymm, hauser.topo, sortby = "AIC")

```

## Ordinal tables

Because the factors in mobility tables are ordered, another
path to simplifying the saturated model is to consider assigning
numerical scores (typically consecutive integers) to the categories. 

When both variables are assigned scores, this gives the **linear-by-linear model**,
$$
\log ( m_{ij} ) = \mu  +  \lambda_i^A
+  \lambda_j^B  +  \gamma \: a_i b_j \quad ,
$$
where $a_i$ and $b_j$ are the row and column numeric scores.
This model is also called the model of **uniform association** [@Goodman:79]
because,
for integer scores, $a_i=i$, $b_j=j$, 
this model has only one extra parameter, $\gamma$, which is the
common odds local ratio.
The independence model is the special case, $\gamma=0$.  In contrast,
the saturated model, allowing general association $\lambda_{ij}^{AB}$,
uses $(I-1)(J-1)$ additional parameters.

For square tables, like mobility tables, this model can be amended to include
a diagonal term, `Diag()`


```{r scores}
Sscore <- as.numeric(Hauser79$Son)
Fscore <- as.numeric(Hauser79$Father)

Hauser79 |> cbind(Fscore, Fscore) |> head()
```

To fit this model, I use `Fscore * Sscore` for the linear x linear
association and add `Diag(Father, Son)` to fit the diagonal cells
exactly.
```{r hauser-UAdiag}
hauser.UAdiag <- update(hauser.indep,
                        . ~ . + Fscore : Sscore + Diag(Father, Son))

LRstats(hauser.UAdiag)
```

In this model, the estimated common local log odds ratio---the
coefficient $\gamma$ for the linear-by-linear term
`Fscore:Sscore`, is given by:
```{r}
coef(hauser.UAdiag)[["Fscore:Sscore"]]
```

For comparisons not involving the diagonal cells,
each step down the scale of occupational categories for the father
multiplies the odds that the son will also be in one lower
category by $\exp (0.158) = 1.172$, an increase of 17%.

But this model does not seem to be any improvement over quasi-symmetry.
From the pattern of residuals in the mosaic, we see a number of large
residuals of various signs in the lower triangular, where the son's
occupation is of a higher level than that of the father.
```{r UAdiag-mosaic}
mosaic(hauser.UAdiag, ~ Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Uniform association + Diag()")
```

## Model comparison plots

Finally, for comparing a largish collection of models, a model comparison plot
can show the trade-off between goodness-of-fit and parsimony by plotting
measures like $G^2/df$, AIC, or BIC against
degrees of freedom. The plot below, including quite a few more models, uses a log scale for BIC to emphasize differences among better fitting models. (The code for this plot is shown
on p. 399 of @FriendlyMeyer:2016:DDAR).

![](fig/hauser-model-plot.png){width=80%}



## References

---
title: "6. Mobility tables"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib", "vignettes.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{6. Mobility tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
#  fig.path = "fig/mobility-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# colorize text
colorize <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}
```

## Social mobility

Social mobility is an important concept in sociology, and its' study has led to
a wide range of developments in categorical data analysis in what are often called
_mobility tables_. 

The idea is to study the movement of individuals, families, households or other categories of people within or between social strata in a society, across time or space. This refers to
a change in social status relative to one's current social location within a given society.

Using survey data, the most frequent examples relate to changes in income or wealth, but most often this is studied via classification in occupational categories ("professional, "managerial", "skilled manual", ...). Most often this is studied _intergenerationaly_ using the occupational
categories of fathers and sons.

Mobility tables are nearly always _square_ tables, with the same categories for the row
and column variables. As such, they nearly always exhibit positive associations
along the diagonal cells.  What is of interest are specialized models, intermediate
between the null model of independence and the saturated model.

### Models

These models include important special cases:

- **quasi-independence**: Ignoring diagonal cells, are the row and column variables independent?
- **symmetry**: Are associations above the diagonal equal to the corresponding ones below the diagonal?
- **row effects, col effects, linear x linear**: Typically, the factors in such tables are ordinal. To what extent can the models be simplified by assigning integer scores to the
row, column categories or both?
- **multiplicative RC**: RC models attempt to estimate the scores for the row and column categories. 
- **topographical models**: It is possible that the associations among occupational categories
exhibit consistent patterns according to their nature. These models allow specifying a
theoretically interesting pattern.
- **crossings models**: assert that there are different difficulty parameters for crossing from category to the next and associations between categories decrease with their separation.

While standard loglinear models can be fit using `MASS::loglm`, these models require
use of ``stats::glm()` or `gnm::gnm()`, as I illustrate below.

## Hauser data

This vignette uses the `vcdExtra::Hauser79` dataset, a cross-classification of 19,912 individuals by father's occupation and son's first occupation for U.S. men aged 20-64 in 1973.
The data comes from @Hauser:79 and has been also analysed by @PowersXie:2008.
The discussion draws on @FriendlyMeyer:2016:DDAR, Ch. 10.

```{r hauser-data}
data("Hauser79", package="vcdExtra")
str(Hauser79)
(Hauser_tab <- xtabs(Freq ~ Father + Son, data=Hauser79))
```

As can be seen, `Hauser79` is a data.frame in frequency form.
The factor levels in this table are a coarse grouping of occupational categories, so:

- `UpNM` = professional and kindred workers, managers and officials, and non-retail sales workers;
- `LoNM` = proprietors, clerical and kindred workers, and retail sales workers; 

- `UpM` = craftsmen, foremen, and kindred workers;

- `LoM` =  service workers, operatives and kindred workers, and laborers (except farm); 

- `Farm` = farmers and farm managers, farm laborers, and foremen.

### Load packages

```{r load}
library(vcdExtra)
library(gnm)
library(dplyr)
```

### Mosaic plots

`Hauser_tab` is a `table` object, and the
simplest plot for the frequencies is the default `plot()` method, giving a `graphics::mosaicplot()`.

```{r mosaicplot}
plot(Hauser_tab, shade=TRUE)
```

The frequencies are first split according to father's occupational category 
(the first table dimension) and then by sons' occupation.
The most common category for fathers is lower manual, followed by farm.

`mosaicplot()`, using `shade=TRUE` colors the tiles according to the 
sign and magnitude of the residuals from an independence model: shades of 
`r colorize("positive", "blue")` for positive residuals and
`r colorize("negative", "red")` red for negative residuals.

`vcd::mosaic()` gives a similar display, but is much more flexible in the labeling of
the row and column variable, labels for the categories, and the scheme used for shading
the tiles. Here, I simply assign longer labels for the row and column variables,
using the `labeling_args` argument to `mosaic()`.

```{r mosaic1}
labels <- list(set_varnames = c(Father="Father's occupation", 
                                Son="Son's occupation"))

mosaic(Freq ~ Father + Son, data=Hauser79,
       labeling_args = labels,
       shade=TRUE,
       legend = FALSE)
```

### Fitting and graphing models

The call to `vcd::mosaic()` above takes the `Hauser79` dataset as input.
Internally, it fits the model of independence and displays the result,
but for more complex tables, control of the fitted model is limited.

Unlike `mosaicplot()` and even the [`ggmosaic`]( https://CRAN.R-project.org/package=ggmosaic) package,
`vcdExtra::mosaic.glm()` is a `mosaic` **method** for `glm` objects.
This means you can fit any model, and supply the model object to `mosaic()`.
(Note that in `mosaic()`, the `formula` argument determines the order of
splitting in the mosaic, not a loglinear formula.)

```{r indep}
hauser.indep <- glm(Freq ~ Father + Son, 
  data=Hauser79, 
  family=poisson)

# the same mosaic, using the fitted model
mosaic(hauser.indep, formula = ~ Father + Son, 
       labeling_args = labels,
       legend = FALSE,
       main="Independence model")
```

## Quasi-independence

Among the most important advances from the social mobility literature is the idea that
associations between row and column variables in square tables can be explored in greater
depth if we ignore the obvious association in the diagonal cells.
The result is a model of _quasi-independence_, asserting that fathers' and sons' occupations
are independent, ignoring the diagonal cells. 

For a two-way table, quasi-independence can be expressed as
$$
 \pi_{ij} = \pi_{i+} \pi_{+j} \quad\quad \mbox{for } i\ne j
$$
or in loglinear form as:
$$
 \log m_{ij} = \mu + \lambda_i^A + \lambda_j^B + \delta_i I(i=j)
 \quad .
$$
This model effectively adds one parameter, $\delta_i$, for each main diagonal cell
and fits those frequencies perfectly.


In the [`gnm`]( https://CRAN.R-project.org/package=gnm) package,
`gnm::Diag()` creates the appropriate term in the model formula,
using a symbol in the diagonal cells and "." otherwise.

```{r Diag}
# with symbols
with(Hauser79, Diag(Father, Son)) |> matrix(nrow=5)
```

We proceed to fit and plot the quasi-independence model by updating the
independence model, adding the term `Diag(Father, Son)`.
```{r quasi}
hauser.quasi <-  update(hauser.indep, 
                        ~ . + Diag(Father, Son))

mosaic(hauser.quasi, ~ Father+Son, 
       labeling_args = labels,
       legend = FALSE,
       main="Quasi-independence model")

```

Note that the pattern of residuals shows a systematic pattern of 
`r colorize("positive", "blue")` and 
`r colorize("negative", "red")` residuals above and below the diagonal tiles.
We turn to this next.

### Symmetry and quasi-symmetry

Another advance from the social mobility literature was the idea of how to test for 
_differences_ in occupational categories between fathers and sons.
The null hypothesis of no systematic differences can be formulated as a 
test of **symmetry** in the table, 
$$
 \pi_{ij} = \pi_{ji} \quad\quad \mbox{for } i\ne j \quad ,
$$
which asserts that sons are as likely to move from their father's occupation $i$
to another category $j$ as they were to move in the reverse direction, $j$ to $i$.
An alternative, "Upward mobility",
i.e., that sons who did not stay in their father's occupational category moved
to a higher category on average would mean that 
$$
 \pi_{ij} < \pi_{ji} \quad\quad \mbox{for } i\ne j
$$
Yet this model is overly strong, because it also asserts **marginal homogeneity**,
that the marginal probabilities of row and column values are equal,
$\pi_{i+} = \pi_{+i}$ for all $i$. Consequently, this hypothesis is most often
tested as a model for **quasi-symmetry**, that also ignores the diagonal cells.

Symmetry is modeled by the function `gnm::Symm()`. It returns a factor with
the same labels for positions above and below the diagonal.

```{r symm}
with(Hauser79, Symm(Father, Son)) |> matrix(nrow=5)
```

To fit the model of quasi-symmetry, add both `Diag()` and `Symm()` to the model of independence.
```{r qsymm}
hauser.qsymm <-  update(hauser.indep, 
                        ~ . + Diag(Father,Son) + Symm(Father,Son))
```

To compare the models so far, we can use `anova()` or `vcdExtra::LRstats():

```{r anova1}
anova(hauser.indep, hauser.quasi, hauser.qsymm, test="Chisq")

LRstats(hauser.indep, hauser.quasi, hauser.qsymm)
```

This `hauser.qsymm` model represents a huge improvement in goodness of fit.
With such a large sample size, it might be considered an acceptable
fit.

But, this model of quasi-symmetry still shows some residual lack of fit.
To visualize this in the mosaic, we can label the cells with their
standardized residuals.
```{r qsymm-mosaic}
mosaic(hauser.qsymm, ~ Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Quasi-symmetry model")
```

The cells with the largest lack of symmetry (using standardized residuals)
are those for the upper and lower non-manual occupations, where the son of
an upper manual worker is less likely to move to lower non-manual
work than the reverse.

### Topological models

It is also possible that there are more subtle patterns of association one
might want to model, with specific parameters for particular combinations of the
occupational categories (beyond the idea of symmetry).
@Hauser:79 developed this idea in what are now called
**topological** models or **levels** models, where an arbitrary pattern of
associations can be specified, implemented in `gnm::Topo()`.

```{r topo-levels}
# Levels for Hauser 5-level model
levels <- matrix(c(
      2,  4,  5,  5,  5,
      3,  4,  5,  5,  5,
      5,  5,  5,  5,  5,
      5,  5,  5,  4,  4,
      5,  5,  5,  4,  1), 
      nrow = 5, ncol = 5, 
      byrow=TRUE)
```

```{r topo-mosaic}
hauser.topo <- update(hauser.indep, 
                      ~ . + Topo(Father, Son, spec=levels))

mosaic(hauser.topo, ~Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Topological model")
```

Comparing models, we can see that the model of quasi-symmetry is the best
so far, using AIC as the measure:

```{r}
LRstats(hauser.indep, hauser.quasi, hauser.qsymm, hauser.topo, sortby = "AIC")

```

## Ordinal tables

Because the factors in mobility tables are ordered, another
path to simplifying the saturated model is to consider assigning
numerical scores (typically consecutive integers) to the categories. 

When both variables are assigned scores, this gives the **linear-by-linear model**,
$$
\log ( m_{ij} ) = \mu  +  \lambda_i^A
+  \lambda_j^B  +  \gamma \: a_i b_j \quad ,
$$
where $a_i$ and $b_j$ are the row and column numeric scores.
This model is also called the model of **uniform association** [@Goodman:79]
because,
for integer scores, $a_i=i$, $b_j=j$, 
this model has only one extra parameter, $\gamma$, which is the
common odds local ratio.
The independence model is the special case, $\gamma=0$.  In contrast,
the saturated model, allowing general association $\lambda_{ij}^{AB}$,
uses $(I-1)(J-1)$ additional parameters.

For square tables, like mobility tables, this model can be amended to include
a diagonal term, `Diag()`


```{r scores}
Sscore <- as.numeric(Hauser79$Son)
Fscore <- as.numeric(Hauser79$Father)

Hauser79 |> cbind(Fscore, Fscore) |> head()
```

To fit this model, I use `Fscore * Sscore` for the linear x linear
association and add `Diag(Father, Son)` to fit the diagonal cells
exactly.
```{r hauser-UAdiag}
hauser.UAdiag <- update(hauser.indep,
                        . ~ . + Fscore : Sscore + Diag(Father, Son))

LRstats(hauser.UAdiag)
```

In this model, the estimated common local log odds ratio---the
coefficient $\gamma$ for the linear-by-linear term
`Fscore:Sscore`, is given by:
```{r}
coef(hauser.UAdiag)[["Fscore:Sscore"]]
```

For comparisons not involving the diagonal cells,
each step down the scale of occupational categories for the father
multiplies the odds that the son will also be in one lower
category by $\exp (0.158) = 1.172$, an increase of 17%.

But this model does not seem to be any improvement over quasi-symmetry.
From the pattern of residuals in the mosaic, we see a number of large
residuals of various signs in the lower triangular, where the son's
occupation is of a higher level than that of the father.
```{r UAdiag-mosaic}
mosaic(hauser.UAdiag, ~ Father+Son, 
       labeling_args = labels,
       labeling = labeling_residuals,
       residuals_type ="rstandard",
       legend = FALSE,
       main="Uniform association + Diag()")
```

## Model comparison plots

Finally, for comparing a largish collection of models, a model comparison plot
can show the trade-off between goodness-of-fit and parsimony by plotting
measures like $G^2/df$, AIC, or BIC against
degrees of freedom. The plot below, including quite a few more models, uses a log scale for BIC to emphasize differences among better fitting models. (The code for this plot is shown
on p. 399 of @FriendlyMeyer:2016:DDAR).

![](fig/hauser-model-plot.png){width=80%}



## References

---
title: "Creating and manipulating frequency tables"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Creating and manipulating frequency tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/tut01-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# Old Sweave options
# \SweaveOpts{engine=R,eps=TRUE,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{engine=R,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{prefix.string=fig/vcd-tut,eps=FALSE}
# \SweaveOpts{keep.source=TRUE}


# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(HairEyeColor)
data(PreSex)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
if(!file.exists("fig")) dir.create("fig")

```


R provides many methods for creating frequency and contingency tables. Several are
described below. In the examples below, we use some real examples and some anonymous
ones, where the variables `A`, `B`, and `C`  represent
categorical variables, and `X` represents an arbitrary R data object.

## Forms of frequency data
The first thing you need to know is that categorical data can be represented in
three different forms in R, and it is sometimes necessary to convert
from one form to another, for carrying out statistical tests, fitting models
or visualizing the results.  Once a data object exists in R, 
you can examine its complete structure with
the `str()` function, or view the names of its components with the
`names()` function.
 

### Case form
Categorical data in case form are simply data frames containing individual observations, with one or
more factors, used as the classifying variables. In case form, there may also
be numeric covariates. 
The total number of observations is `nrow(X)`, and the number of variables is `ncol(X)`.
  	
***Example***:

The `Arthritis` data is available in case form in the `vcd` package. 
There are two explanatory factors: `Treatment` and `Sex`. `Age`
is a numeric covariate, and `Improved` is the response--- an ordered factor,
with levels
`r paste(levels(Arthritis$Improved),collapse=' < ')`.
Excluding `Age`, this represents
a $2 \times 2 \times 3$ contingency table for `Treatment`, `Sex` and `Improved`, but in case form.

```{r, case-form}
names(Arthritis)      # show the variables

str(Arthritis)        # show the structure

head(Arthritis,5)     # first 5 observations, same as Arthritis[1:5,] 
```
  
### Frequency form
Data in frequency form is also a data frame containing one or more factors, and a frequency 
variable, often called `Freq` or `count`.  The total number of observations
is: `sum(X$Freq)`,  `sum(X[,"Freq"])` or some equivalent form.

The number of cells in the table is given by `nrow(X)`.

***Example***: 
For small frequency tables, it is often convenient to enter them in frequency form
using `expand.grid()` for the factors and `c()` to list the counts in a vector.
The example below, from [@vcd:Agresti:2002] gives results for the 1991 General Social Survey,
with respondents classified by sex and party identification.
```{r, frequency-form}
# Agresti (2002), table 3.11, p. 106
GSS <- data.frame(
  expand.grid(sex = c("female", "male"), 
              party = c("dem", "indep", "rep")),
  count = c(279,165,73,47,225,191))

GSS
names(GSS)
str(GSS)

sum(GSS$count)
```

### Table form 
Table form data is represented by a `matrix`, `array` or `table` object, whose elements are the frequencies
in an $n$-way table.  The variable names (factors) and their levels are given by
`dimnames(X)`. The total number of observations
is `sum(X)`.  The number of dimensions of the table is `length(dimnames(X))`,
and the table sizes are given by `sapply(dimnames(X), length)`.
  	
***Example***: 
The `HairEyeColor` is stored in table form in `vcd`.  
```{r, table-form1}
str(HairEyeColor)                      # show the structure

sum(HairEyeColor)                      # number of cases

sapply(dimnames(HairEyeColor), length) # table dimension sizes
```
***Example***: 
Enter frequencies in a matrix, and assign `dimnames`,
giving the variable names and category labels.  Note that, by default,
`matrix()` uses the elements supplied by *columns* in the
result, unless you specify `byrow=TRUE`.
```{r, table-form2}
# A 4 x 4 table  Agresti (2002, Table 2.8, p. 57) Job Satisfaction
JobSat <- matrix(c( 1, 2, 1, 0, 
                    3, 3, 6, 1, 
                   10,10,14, 9, 
                    6, 7,12,11), 4, 4)

dimnames(JobSat) = list(
  income = c("< 15k", "15-25k", "25-40k", "> 40k"),
  satisfaction = c("VeryD", "LittleD", "ModerateS", "VeryS")
  )

JobSat
```

`JobSat` is a **matrix**, not an object of `class("table")`, and some functions
are happier with tables than matrices.
You can coerce it to a table with `as.table()`,
```{r, table-form3}
JobSat <- as.table(JobSat)
str(JobSat)
```

## Ordered factors and reordered tables {#sec:ordered-factors}

In table form, the values of the table factors are ordered by their position in the table.
Thus in the `JobSat` data, both `income` and `satisfaction` represent ordered
factors, and the *positions* of the values in the rows and columns reflects their
ordered nature.

Yet, for analysis, there are time when you need *numeric* values for the levels
of ordered factors in a table, e.g., to treat a factor as a quantitative variable.
In such cases, you can simply re-assign the `dimnames` attribute of the table
variables.  For example, here, we assign numeric values to `income` as the middle of their
ranges, and treat `satisfaction` as equally spaced with integer scores.

```{r, relevel, eval=FALSE}
dimnames(JobSat)$income <- c(7.5,20,32.5,60)
dimnames(JobSat)$satisfaction <- 1:4
```

For the  `HairEyeColor` data, hair color and eye color are ordered arbitrarily.
For visualizing the data using mosaic plots and other methods described below, it 
turns out to be more useful to assure that both hair color and eye color are
ordered from dark to light.
Hair colors are actually ordered this way already, and it is easiest to re-order
eye colors by indexing. Again `str()` is your friend.

```{r, reorder1}
HairEyeColor <- HairEyeColor[, c(1,3,4,2), ]
str(HairEyeColor)
```
This is also the order for both hair color and eye color shown in 
the result of a correspondence analysis (\figref{fig:ca-haireye}) below.

With data in  case form or frequency form, when you have ordered factors
represented with character values, you must ensure that they are treated
as ordered in R.
<!-- \footnote{In SAS, many procedures offer the option -->
<!-- 	`order = data | internal | formatted` to allow character values -->
<!-- 	to be ordered according to (a) their order in the data set, (b) -->
<!-- 	sorted internal value, or (c) sorted formatted representation -->
<!-- 	provided by a SAS format. -->
<!-- } -->

Imagine that the `Arthritis` data was read from a text file.  
By default the `Improved` will be ordered alphabetically:
`Marked`,
`None`,
`Some` --- not what we want.  In this case, the function
`ordered()` (and others) can be useful. 

```{r, reorder2, echo=TRUE, eval=FALSE}
Arthritis <- read.csv("arthritis.txt",header=TRUE)
Arthritis$Improved <- ordered(Arthritis$Improved, 
                              levels=c("None", "Some", "Marked")
                              )
```
The dataset `Arthritis` in the `vcd` package is a data.frame in this form
With this order of  `Improved`, the response in this data,
a mosaic display of `Treatment` and `Improved` (\figref{fig:arthritis}) shows a clearly
interpretable pattern.

The original version of `mosaic` in the `vcd` package required the input to be 
a contingency table in array form, so we convert using `xtabs()`.

<!-- ```{r Arthritis, height=6, width=7, fig.cap="Mosaic plot for the `Arthritis` data ..."} -->

```{r}
#| Arthritis,
#| fig.height = 6,
#| fig.width = 6,
#| fig.cap = "Mosaic plot for the `Arthritis` data, showing the marginal model of independence for Treatment and Improved.  Age, a covariate, and Sex are ignored here."
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
mosaic(art, gp = shading_max, split_vertical = TRUE, main="Arthritis: [Treatment] [Improved]")
```


Finally, there are situations where, particularly for display purposes, you
want to re-order the *dimensions* of an $n$-way table, or change the
labels for the variables or levels.
This is easy when the data are in table form: `aperm()` permutes
the dimensions, and assigning to `names` and `dimnames`
changes variable names and level labels respectively.
We will use the following version of `UCBAdmissions` in
\@(sec:mantel) below.
^[Changing `Admit` to `Admit?` might be useful for display purposes, but is
dangerous--- because it is then difficult to use that variable name in a model formula.
See \@(sec:tips) for options `labeling_args` and `set_labels`to change variable and level names for displays in the `strucplot` framework.]

```{r, reorder3}
UCB <- aperm(UCBAdmissions, c(2, 1, 3))
dimnames(UCB)[[2]] <- c("Yes", "No")
names(dimnames(UCB)) <- c("Sex", "Admit?", "Department")

# display as a flattened table
stats::ftable(UCB)
```

## `structable()` {#sec:structable}

For 3-way and larger tables
the `structable()` function in `vcd` provides a convenient and flexible tabular display.
The variables assigned to the rows and columns of a two-way display can be specified
by a model formula.
```{r, structable}
structable(HairEyeColor)                   # show the table: default

structable(Hair+Sex ~ Eye, HairEyeColor)   # specify col ~ row variables
```
It also returns an object of class `"structable"` which may be plotted with 
`mosaic()` (not shown here).
```{r, structable1,eval=FALSE}
HSE < - structable(Hair+Sex ~ Eye, HairEyeColor)   # save structable object
mosaic(HSE)                                        # plot it
```

## `table()` and friends {#sec:table}

You  can  generate frequency  tables from factor variables  using the  `table()` function,  tables  of
proportions using  the `prop.table()` function,  and marginal  frequencies using
`margin.table()`.

For these examples, create some categorical vectors:
```{r, table-setup}
 n=500
 A <- factor(sample(c("a1","a2"), n, rep=TRUE))
 B <- factor(sample(c("b1","b2"), n, rep=TRUE))
 C <- factor(sample(c("c1","c2"), n, rep=TRUE))
 mydata <- data.frame(A,B,C)
```

These lines illustrate `table`-related functions:
```{r, table-ex1}
# 2-Way Frequency Table
attach(mydata)
mytable <- table(A,B)   # A will be rows, B will be columns
mytable                 # print table

margin.table(mytable, 1) # A frequencies (summed over B)
margin.table(mytable, 2) # B frequencies (summed over A)

prop.table(mytable)    # cell percentages
prop.table(mytable, 1) # row percentages
prop.table(mytable, 2) # column percentages
```

`table()` can  also  generate  multidimensional  tables  based  on  3  or  more
categorical variables. In  this case, you can use  the `ftable()`  or `structable()`
function to print the results more attractively.

```{r, table-ex2}
# 3-Way Frequency Table
mytable <- table(A, B, C)
ftable(mytable)
```

`table()`  ignores missing values by default. 
To include `NA` as a category in counts, include the
table option  `exclude=NULL` if  the variable  is a  vector. If  the variable is a
factor you  have to  create a  new factor  using \code{newfactor  <- factor(oldfactor,
exclude=NULL)}. 

## `xtabs()` {#sec:xtabs}

The `xtabs()` function allows you to create cross-tabulations of data using formula style input.
This typically works with case-form data supplied in a data frame or a matrix.
The result is a contingency table in array format, whose dimensions are determined by
the terms on the right side of the formula.

```{r, xtabs-ex1}
# 3-Way Frequency Table
mytable <- xtabs(~A+B+C, data=mydata)

ftable(mytable)    # print table

summary(mytable)   # chi-square test of indepedence
```

If a variable is included on the left side of the formula, it is assumed to be a
vector  of  frequencies  (useful if the data have already been tabulated in frequency form).

```{r, xtabs-ex2}
(GSStab <- xtabs(count ~ sex + party, data=GSS))

summary(GSStab)
```

## Collapsing over table factors: `aggregate()`, `margin.table()` and `apply()`

It sometimes happens that we have a data set with more variables or factors than 
we want to analyse, or else, having done some initial analyses, we decide that
certain factors are not important, and so should be excluded from graphic displays
by collapsing (summing) over them.  For example, mosaic plots and fourfold displays
are often simpler to construct from versions of the data collapsed over
the factors which are not shown in the plots.

The appropriate tools to use again depend on
the form in which the data are represented--- a case-form data frame, a
frequency-form data frame (`aggregate()`), or a table-form array or 
table object (`margin.table()` or `apply()`).

When the data are in frequency form, and we want to produce another
frequency data frame, `aggregate()` is a handy tool, using
the argument `FUN=sum` to sum the frequency variable over the 
factors *not* mentioned in the formula.

***Example***: 
The data frame `DaytonSurvey` in the `vcdExtra` package represents a
$2^5$ table giving the frequencies of reported use (``ever used?'') of 
alcohol, cigarettes and marijuana in a sample of high school seniors,
also classified by sex and race.  

```{r, dayton1}
str(DaytonSurvey)
head(DaytonSurvey)
```

To focus on the associations among the
substances, we want to collapse over sex and race. The right-hand side of the formula
used in the call to `aggregate()` gives the factors to be retained in the
new frequency data frame, `Dayton.ACM.df`.

```{r, dayton2}
# data in frequency form
# collapse over sex and race
Dayton.ACM.df <- aggregate(Freq ~ cigarette+alcohol+marijuana, 
                           data=DaytonSurvey, 
                           FUN=sum)
Dayton.ACM.df
```

When the data are in table form, and we want to produce another
table, `apply()` with `FUN=sum` can be used in a similar way
to sum the table over dimensions not mentioned in the `MARGIN`
argument.  `margin.table()` is just a wrapper for `apply()`
using the `sum()` function.


***Example***: 
To illustrate, we first convert the `DaytonSurvey` to a 5-way
table using `xtabs()`, giving `Dayton.tab`.  

```{r, dayton3}
# in table form
Dayton.tab <- xtabs(Freq ~ cigarette+alcohol+marijuana+sex+race, 
                    data=DaytonSurvey)
structable(cigarette+alcohol+marijuana ~ sex+race, 
           data=Dayton.tab)
```

Then, use `apply()` on `Dayton.tab` to give the
3-way table `Dayton.ACM.tab` summed over sex and race.
The elements in this new table are the column sums for 
`Dayton.tab` shown by `structable()` just above.

```{r, dayton4}
# collapse over sex and race
Dayton.ACM.tab <- apply(Dayton.tab, MARGIN=1:3, FUN=sum)
Dayton.ACM.tab <- margin.table(Dayton.tab, 1:3)   # same result

structable(cigarette+alcohol ~ marijuana, data=Dayton.ACM.tab)
```


Many of these operations can be performed using the `**ply()` functions
in the `plyr` package.
For example, with the data in a frequency form data frame, use `ddply()`
to collapse over unmentioned factors, and `plyr::summarise()`
as the function to be applied to each piece.
<!-- \footnote{ -->
<!-- Ugh. This `plyr` function clashes with a function of the same name in `vcdExtra`. -->
<!-- In this document I will use the explicit double-colon notation to keep them -->
<!-- separate. -->
<!-- } -->

```{r, dayton5}
library(plyr)
Dayton.ACM.df <- plyr::ddply(DaytonSurvey, 
                             .(cigarette, alcohol, marijuana), 
                             plyr::summarise, Freq=sum(Freq))

Dayton.ACM.df
```

## Collapsing table levels: `collapse.table()`

A related problem arises when we have a table or array and for some purpose
we want to reduce the number of levels of some factors by summing subsets
of the frequencies.  For example, we may have initially coded Age in 10-year
intervals, and decide that, either for analysis or display purposes, we
want to reduce Age to 20-year intervals.  The `collapse.table()` function
in `vcdExtra` was designed for this purpose.

***Example***:
Create a 3-way table, and collapse Age from 10-year to 20-year intervals. 
First, we generate a $2 \times 6 \times 3$ table of random counts from a 
Poisson distribution with mean of 100.
```{r, collapse1}
# create some sample data in frequency form
sex <- c("Male", "Female")
age <- c("10-19", "20-29",  "30-39", "40-49", "50-59", "60-69")
education <- c("low", 'med', 'high')
data <- expand.grid(sex=sex, age=age, education=education)
counts <- rpois(36, 100)   # random Possion cell frequencies
data <- cbind(data, counts)

# make it into a 3-way table
t1 <- xtabs(counts ~ sex + age + education, data=data)
structable(t1)
``` 
Now collapse `age` to 20-year intervals, and `education`
to 2 levels. In the arguments, levels of `age` and `education`
given the same label are summed in the resulting smaller table.
```{r, collapse2}
# collapse age to 3 levels, education to 2 levels
t2 <- collapse.table(t1, 
         age=c("10-29", "10-29",  "30-49", "30-49", "50-69", "50-69"),
         education=c("<high", "<high", "high"))
structable(t2)
```

## Converting among frequency tables and data frames

As we've seen, a given contingency table can be represented 
equivalently in different forms,
but some R functions were designed for one particular representation.

The table below shows some handy tools for converting from one form to another.

<!-- [htb] -->
<!--  \caption{Tools for converting among different forms for categorical data} {#tab:convert} -->
 
<!--    {llll} -->
<!-- 	\hline -->
<!--                  & \multicolumn{3}{c}{**To this**} \\ -->
<!-- 	**From this**      &     Case form         & Frequency form             &  Table form \\ -->
<!-- 	\hline -->
<!-- 	Case form      &   noop                 & `xtabs(~A+B)`        &  `table(A,B)`  \\  -->
<!-- 	Frequency form &  `expand.dft(X)`  & noop                      & `xtabs(count~A+B)`\\ -->
<!-- 	Table form     &  `expand.dft(X)`  & `as.data.frame(X)`   &  noop \\ -->
<!-- 	\hline -->
   
| **From this**    |                     | **To this**          |                   |
|:-----------------|:--------------------|:---------------------|-------------------|
|	                 |    _Case form_      | _Frequency form_     | _Table form_      |
|	_Case form_      |   noop              | `xtabs(~A+B)`        |  `table(A,B)`     |
|	_Frequency form_ |  `expand.dft(X)`    | noop                 | `xtabs(count~A+B)`|
|	_Table form_     |  `expand.dft(X)`    | `as.data.frame(X)`   |  noop             |


For example, a contingency table in table form (an object of `class(table)`) can be converted
to a data.frame with `as.data.frame()`.
^[Because R is object-oriented, this is actually a short-hand for the function `as.data.frame.table()`.] 
The resulting
`data.frame` contains columns
representing the classifying factors and the table entries (as a column named by
the `responseName` argument, defaulting to `Freq`. This is the inverse of `xtabs()`.

***Example***: 
Convert the `GSStab` in table form to a data.frame in frequency form.
```{r, convert-ex1}
as.data.frame(GSStab)
```

***Example***: Convert the `Arthritis` data in case form to a 3-way table of
`Treatment` $\times$ `Sex` $\times$ `Improved`.
Note the use of `with()` to avoid having to use `Arthritis\$Treatment` etc. within the call to `table()`.%
^[`table()` does not allow a `data` argument to provide an environment in which the table variables are to be found.  In the examples in \@(sec:table) I used `attach(mydata)` for this purpose,
but `attach()` leaves the variables in the global environment, while `with()` just evaluates the `table()` expression in a temporary environment of the data.]

```{r, convert-ex2}
Art.tab <- with(Arthritis, table(Treatment, Sex, Improved))
str(Art.tab)

ftable(Art.tab)
```

There may also be times that you will need an equivalent case form `data.frame`
with factors  representing the table variables
rather than the frequency  table.
For example, the `mca()` function in package `MASS`
only operates on data in this format. 
Marc Schwartz initially provided code for `expand.dft()` on the  Rhelp
mailing list for converting a table back into a case form `data.frame`.
This function is included in `vcdExtra`.

***Example***: Convert the `Arthritis` data in table form (`Art.tab`) back to a `data.frame`
in case form, with factors
`Treatment`, `Sex` and `Improved`.
```{r, convert-ex3}
Art.df <- expand.dft(Art.tab)
str(Art.df)
```

## A complex example {#sec:complex}

If you've followed so far, you're ready for a more complicated example.
The data file, `tv.dat` represents a 4-way table of size
$5 \times 11 \times 5 \times 3$ where the table variables (unnamed in the file)
are read as `V1` -- `V4`, and the cell frequency is read
as `V5`.  The file, stored in the `doc/extdata` directory
of `vcdExtra`, can be read as follows:
```{r, tv1}
tv.data<-read.table(system.file("extdata","tv.dat", package="vcdExtra"))
head(tv.data,5)
```
For a local file, just use `read.table()` in this form:
```{r, tv2,eval=FALSE}
tv.data<-read.table("C:/R/data/tv.dat")
```
The data `tv.dat` came from the initial implementation of 
mosaic displays in R by Jay Emerson. 
In turn, they came from the initial development of mosaic displays 
[@vcd:Hartigan+Kleiner:1984]
that illustrated the method with data on a large sample of TV viewers
whose behavior had been recorded for the Neilsen ratings.
This data set contains sample television audience data from Neilsen
Media Research for the week starting November 6, 1995.

The table variables are:

* `V1`-- values 1:5 correspond to the days Monday--Friday;
* `V2`-- values 1:11 correspond to the quarter hour times 8:00PM through 10:30PM;
* `V3`-- values 1:5 correspond to ABC, CBS, NBC, Fox, and non-network choices;
* `V4`-- values 1:3 correspond to transition states: turn the television Off, Switch channels, or Persist in viewing the current channel.

We are interested just the cell frequencies, and rely on the facts that the

(a) the table is complete--- there are no missing cells, so `nrow(tv.data)` = `r nrow(tv.data)`;
(b) the observations are ordered so that `V1` varies most rapidly and `V4` most slowly.  From this, we can just extract the frequency column and reshape it into an array. [That would be dangerous if any observations were out of order.]

```{r, tv3}
TV <- array(tv.data[,5], dim=c(5,11,5,3))                                        
dimnames(TV) <- list(c("Monday","Tuesday","Wednesday","Thursday","Friday"), 
                     c("8:00","8:15","8:30","8:45","9:00","9:15","9:30",         
                       "9:45","10:00","10:15","10:30"),                            
                     c("ABC","CBS","NBC","Fox","Other"), 
                     c("Off","Switch","Persist"))

names(dimnames(TV))<-c("Day", "Time", "Network", "State")
```

More generally (even if there are missing cells), we can 
use `xtabs()` (or `plyr::daply()`)
to do the cross-tabulation, using `V5` as the
frequency variable.  Here's how to do this same operation with `xtabs()`:
```{r, tv3a,eval=FALSE}
TV <- xtabs(V5 ~ ., data=tv.data)
dimnames(TV) <- list(Day = c("Monday","Tuesday","Wednesday","Thursday","Friday"), 
                     Time = c("8:00","8:15","8:30","8:45","9:00","9:15","9:30",         
                              "9:45","10:00","10:15","10:30"),                            
                     Network = c("ABC","CBS","NBC","Fox","Other"), 
                     State = c("Off","Switch","Persist"))

# table dimensions
dim(TV)
```

But this 4-way table is too large and awkward to work with. Among the networks,
Fox and Other occur infrequently. 
We can also cut it down to a 3-way table by considering only viewers who persist
with the current station.
^[This relies on the fact that that indexing an array drops dimensions of length 1 by default, using the argument `drop=TRUE`; the result is coerced to the lowest possible dimension.]


```{r, tv4}
TV2 <- TV[,,1:3,]      # keep only ABC, CBS, NBC
TV2 <- TV2[,,,3]       # keep only Persist -- now a 3 way table
structable(TV2)
```

Finally, for some purposes, we might want to collapse the 11 times into a smaller number.
Half-hour time slots make more sense.
Here, we use `as.data.frame.table()` to convert the table back to a data frame,
 `levels()` to re-assign the values of `Time`,
and finally, `xtabs()` to give a new, collapsed frequency table.

```{r, tv5}
TV.df <- as.data.frame.table(TV2)
levels(TV.df$Time) <- c(rep("8:00", 2),
                        rep("8:30", 2),
                        rep("9:00", 2), 
                        rep("9:30", 2), 
                        rep("10:00",2),
                            "10:30"
                        )

TV3 <- xtabs(Freq ~ Day + Time + Network, TV.df)

structable(Day ~ Time+Network, TV3)
```

We've come this far, so we might as well show a mosaic display. This is analogous to that used by
@vcd:Hartigan+Kleiner:1984. 
```{r tv-mosaic1, fig.height=6, fig.width=7}
mosaic(TV3, shade = TRUE,
       labeling = labeling_border(rot_labels = c(0, 0, 0, 90)))
```

This mosaic displays can be read at several levels, corresponding to the successive splits of
the tiles and the residual shading.
Several trends are clear for viewers who persist:

* Overall, there are about the same number of viewers on each weekday, with slightly more on Thursday.
* Looking at time slots, viewership is slightly greater from 9:00 - 10:00 overall and also
8:00 - 9:00 on Thursday and Friday

From the residual shading of the tiles:

* Monday: CBS dominates in all time slots.
* Tuesday" ABC and CBS  dominate after 9:00
* Thursday: is a largely NBC day
* Friday: ABC dominates in the early evening

<!-- ```{r, tv4} -->
<!-- TV.df <- as.data.frame.table(TV) -->
<!-- levels(TV.df$Time) <- c(rep("8:00-8:59",4), -->
<!--                         rep("9:00-9:59",4),  -->
<!--                         rep("10:00-10:44",3)) -->
<!-- TV3 <- xtabs(Freq ~ Day + Time + Network, TV.df) -->

<!-- structable(Day ~ Time+Network, TV3) -->
<!-- ``` -->


<!-- Whew! See \figref{fig:TV-mosaic} for a mosaic plot of the `TV3` data. -->

<!-- The table is too large to display conveniently, but we can show a subtable -->
<!-- by selecting the indices.  This call to `ftable()` subsets the first three -->
<!-- levels of each factor. -->

<!-- ```{r tv4} -->
<!-- ftable(TV[1:3,1:3,1:3,1:3], col.vars = 3:4) -->
<!-- ``` -->

<!-- We've come this far, so we might as well show a mosaic display. This is analogous to that used by -->
<!-- @vcd:Hartigan+Kleiner:1984. The result is too complex to see very much.  It would be useful to simplify -->
<!-- the table by collapsing one or more of the table dimensions, e.g., `Time`. -->

<!-- ```{r tv-mosaic2, fig.height=7, fig.width=7} -->
<!-- mosaic(TV[,,1:4,], shade=TRUE,  -->
<!--        labeling_args = list(abbreviate_labs = c(8,5,1,1))) -->

<!-- ``` -->



<!-- ## Styles -->

<!-- The `html_vignette` template includes a basic CSS theme. To override this theme you can specify your own CSS in the document metadata as follows: -->

<!--     output:  -->
<!--       rmarkdown::html_vignette: -->
<!--         css: mystyles.css -->

<!-- ## Figures -->

<!-- The figure sizes have been customised so that you can easily put two images side-by-side.  -->

<!-- ```{r, fig.show='hold'} -->
<!-- plot(1:10) -->
<!-- plot(10:1) -->
<!-- ``` -->


# References

---
title: "Tests of Independence"
author: "Michael Friendly"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
vignette: >
  %\VignetteIndexEntry{Tests of Independence}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/tut02-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# Old Sweave options
# \SweaveOpts{engine=R,eps=TRUE,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{engine=R,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{prefix.string=fig/vcd-tut,eps=FALSE}
# \SweaveOpts{keep.source=TRUE}


# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(HairEyeColor)
data(PreSex)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
if(!file.exists("fig")) dir.create("fig")

```

OK, now we're ready to do some analyses.  This vignette focuses on relatively simple non-parametric
tests and measures of association.

## CrossTable

For tabular displays,
the  `CrossTable()`  function in  the `gmodels`  package produces  cross-tabulations
modeled after `PROC FREQ` in SAS or `CROSSTABS` in SPSS. 
It has a wealth of options for the quantities that can be shown in each cell.

Recall the GSS data used earlier.
```{r, GSStab}
# Agresti (2002), table 3.11, p. 106
GSS <- data.frame(
  expand.grid(sex = c("female", "male"), 
              party = c("dem", "indep", "rep")),
  count = c(279,165,73,47,225,191))

(GSStab <- xtabs(count ~ sex + party, data=GSS))
```

Generate a cross-table showing cell frequency and the cell contribution to $\chi^2$.
```{r, xtabs-ex2}
# 2-Way Cross Tabulation
library(gmodels)
CrossTable(GSStab, prop.t=FALSE, prop.r=FALSE, prop.c=FALSE)
```

There are  options to  report percentages  (row, column,  cell), specify decimal
places, produce Chi-square,  Fisher, and McNemar  tests of independence,  report
expected  and residual  values (pearson,  standardized, adjusted  standardized),
include missing values as valid, annotate with row and column titles, and format
as SAS or SPSS style output! See `help(CrossTable)` for details.

## Chi-square test

For 2-way tables you can use `chisq.test()` to test independence of the row
and column variable. By default,  the $p$-value is calculated from  the asymptotic
chi-squared distribution of the test  statistic. Optionally, the $p$-value can  be
derived via Monte Carlo simulation. 

```{r, chisq}
(HairEye <- margin.table(HairEyeColor, c(1, 2)))

chisq.test(HairEye)

chisq.test(HairEye, simulate.p.value = TRUE)
```

## Fisher Exact Test {#sec:Fisher}

`fisher.test(X)` provides an  **exact test** of  independence. `X` must be  a two-way
contingency table in table form.  Another form, 
`fisher.test(X, Y)` takes two
categorical vectors of the same length.  
For tables larger than $2 \times 2$ the method can be computationally intensive (or can fail) if
the frequencies are not small.

```{r fisher}
fisher.test(GSStab)
```

Fisher's test is meant for tables with small total sample size.
It generates an error for the `HairEye` data with $n$=592 total frequency.

```{r fisher-error, error=TRUE}
fisher.test(HairEye)
```
## Mantel-Haenszel test and conditional association {#sec:mantel}

Use the  `mantelhaen.test(X)` function  to perform  a Cochran-Mantel-Haenszel 
$\chi^2$ chi
test  of   the  null  hypothesis   that  two  nominal   variables  are
*conditionally independent*, $A \perp B \; | \; C$, in each stratum,  assuming that there is no  three-way
interaction. `X` is  a 3 dimensional  contingency table, where  the last dimension
refers to the strata.

The `UCBAdmissions` serves as an example of a $2 \times 2 \times 6$ table,
with `Dept` as the stratifying variable.
```{r, mantel1}
# UC Berkeley Student Admissions
mantelhaen.test(UCBAdmissions)
```

The results show no evidence for association between admission and gender
when adjusted for department.  However, we can easily see that the assumption
of equal association across the strata (no 3-way association) is probably
violated. For $2 \times 2 \times k$ tables, this can be examined
from the odds ratios for each $2 \times 2$ table (`oddsratio()`), and
tested by  using `woolf_test()` in `vcd`.

```{r, mantel2}
oddsratio(UCBAdmissions, log=FALSE)

lor <- oddsratio(UCBAdmissions)  # capture log odds ratios
summary(lor)

woolf_test(UCBAdmissions) 
``` 

## Some plot methods

### Fourfold displays

We  can visualize the  odds ratios of  Admission for
each  department  with  fourfold  displays  using  `fourfold()`.  The cell
frequencies $n_{ij}$  of each  $2 \times  2$ table  are shown  as a  quarter circle whose
radius is proportional to $\sqrt{n_{ij}}$, so  that its area is proportional to  the
cell frequency.

```{r, reorder3}
UCB <- aperm(UCBAdmissions, c(2, 1, 3))
dimnames(UCB)[[2]] <- c("Yes", "No")
names(dimnames(UCB)) <- c("Sex", "Admit?", "Department")
```
Confidence rings for the odds ratio allow a visual test of the null of no association; 
the rings for adjacent quadrants overlap *iff* the observed counts are consistent 
with the null hypothesis.  In the extended version (the default), brighter colors
are used where the odds ratio is significantly different from 1.
The following lines produce \figref{fig:fourfold1}.
<!-- \footnote{The color values `col[3:4]` were modified from their default values -->
<!-- to show a greater contrast between significant and insignificant associations here.} -->



```{r}
#| fourfold1, 
#| h=5, w=7.5, 
#| cap = "Fourfold display for the `UCBAdmissions` data. Where the odds ratio differs
#|   	significantly from 1.0, the confidence bands do not overlap, and the circle quadrants are
#|  	shaded more intensely."
col <- c("#99CCFF", "#6699CC", "#F9AFAF", "#6666A0", "#FF0000", "#000080")
fourfold(UCB, mfrow=c(2,3), color=col)
```
 

Another `vcd` function, `cotabplot()`, provides a more general approach
to visualizing conditional associations in contingency tables,
similar to trellis-like plots produced by `coplot()` and lattice graphics.
The `panel` argument supplies a function used to render each conditional 
subtable. The following gives a display (not shown) similar to \figref{fig:fourfold1}.
```{r fourfold2, eval=FALSE}
cotabplot(UCB, panel = cotab_fourfold)
```

### Doubledecker plots

When we want to view the conditional
probabilities of a response variable (e.g., `Admit`)
in relation to several factors,
an alternative visualization is a `doubledecker()` plot.
This plot is a specialized version of a mosaic plot, which
highlights the levels of a response variable (plotted vertically)
in relation to the factors (shown horizontally). The following
call produces \figref{fig:doubledecker}, where we use indexing
on the first factor (`Admit`) to make `Admitted`
the highlighted level.

In this plot, the
association between `Admit` and `Gender` is shown
where the heights of the highlighted conditional probabilities
do not align. The excess of females admitted in Dept A stands out here.

```{r}
#| doubledecker, 
#| h=5, w=8,
#| out.width = "75%",
#| cap = "Doubledecker display for the `UCBAdmissions` data. The heights
#|     of the highlighted bars show the conditional probabilities of `Admit`,
#|     given `Dept` and `Gender`."
doubledecker(Admit ~ Dept + Gender, data=UCBAdmissions[2:1,,])
```

### Odds ratio plots
Finally, the there is a `plot()` method for `oddsratio` objects.
By default, it shows the 95% confidence interval for the log odds ratio.
\figref{fig:oddsratio} is produced by:

```{r}
#| oddsratio,
#| h=6, w=6,
#| out.width = "60%",
#| cap = "Log odds ratio plot for the `UCBAdmissions` data."
plot(lor, 
     xlab="Department", 
     ylab="Log Odds Ratio (Admit | Gender)")
```
 {#fig:oddsratio}

## Cochran-Mantel-Haenszel tests for ordinal factors {#sec:CMH}

The standard $\chi^2$ tests for association in a two-way table
treat both table factors as nominal (unordered) categories.
When one or both factors of a two-way table are
quantitative or ordinal, more powerful tests of association
may be obtained by taking ordinality into account, using
row and or column scores to test for linear trends or differences
in row or column means.

More general versions of the CMH tests (Landis etal., 1978) 
[@Landis-etal:1978] are provided by assigning
numeric scores to the row and/or column variables. 
For example, with two ordinal factors (assumed to be equally spaced), assigning
integer scores, `1:R` and `1:C` tests the linear $\times$ linear component
of association. This is statistically equivalent to the Pearson correlation between the
integer-scored table variables, with $\chi^2 = (n-1) r^2$, with only 1 $df$
rather than $(R-1)\times(C-1)$ for the test of general association.

When only one table
variable is ordinal, these general CMH tests are analogous to an ANOVA, testing
whether the row mean scores or column mean scores are equal, again consuming
fewer $df$ than the test of general association.

The `CMHtest()` function in `vcdExtra` calculates these various
CMH tests for two possibly ordered factors, optionally stratified other factor(s).

***Example***:
```{r, table-form2, include=FALSE}
## A 4 x 4 table  Agresti (2002, Table 2.8, p. 57) Job Satisfaction
JobSat <- matrix(c(1,2,1,0, 3,3,6,1, 10,10,14,9, 6,7,12,11), 4, 4)
dimnames(JobSat) = list(income=c("< 15k", "15-25k", "25-40k", "> 40k"),
                satisfaction=c("VeryD", "LittleD", "ModerateS", "VeryS"))
JobSat <- as.table(JobSat)
```

Recall the $4 \times 4$ table, `JobSat` introduced in \@(sec:creating),
```{r, jobsat}
JobSat
```

Treating the `satisfaction` levels as equally spaced, but using
midpoints of the `income` categories as row scores gives the following results:
```{r, cmh1}
CMHtest(JobSat, rscores=c(7.5,20,32.5,60))
```

Note that with the relatively small cell frequencies, the test for general 
give no evidence for association. However, the the `cor` test for linear x linear
association on 1 df is nearly significant. The `coin` package contains the
functions `cmh_test()` and `lbl_test()`
for CMH tests of general association and linear x linear association respectively.

## Measures of Association

There are a variety of statistical measures of *strength* of association for
contingency tables--- similar in spirit to $r$ or $r^2$ for continuous variables.
With a large sample size, even a small degree of association can show a 
significant $\chi^2$, as in the example below for the `GSS` data.

The  `assocstats()`  function in `vcd`  calculates  the   $\phi$
contingency coefficient,  and Cramer's  V for  an $r \times c$  table. 
The input must be in table form, a two-way $r \times c$ table.
It won't work with `GSS` in frequency form, but by now you should know how
to convert.
```{r, assoc1}
assocstats(GSStab)
```

For tables with ordinal variables, like `JobSat`, some people prefer the
Goodman-Kruskal $\gamma$ statistic
[@vcd:Agresti:2002, \S 2.4.3]
based on a comparison of concordant
and discordant pairs of observations in the case-form equivalent of a two-way table.
```{r, gamma}
GKgamma(JobSat)
```

A web article by Richard Darlington,
[http://node101.psych.cornell.edu/Darlington/crosstab/TABLE0.HTM]
gives further description of these and other measures of association.

## Measures of Agreement
The
`Kappa()` function in the `vcd` package calculates Cohen's $\kappa$ and weighted
$\kappa$ for a square two-way table with the same row and column categories [@Cohen:60].
\footnote{ 
Don't confuse this with `kappa()` in base R that computes something
entirely different (the condition number of a matrix).
}
Normal-theory $z$-tests are obtained by dividing $\kappa$ by its asymptotic standard
error (ASE).  A `confint()` method for `Kappa` objects provides confidence intervals.
```{r, kappa}
data(SexualFun, package = "vcd")
(K <- Kappa(SexualFun))
confint(K)
```

A visualization of agreement [@Bangdiwala:87], both unweighted and weighted for degree of departure
from exact agreement is provided by the `agreementplot()` function.
\figref{fig:agreesex} shows the agreementplot for the `SexualFun` data,
produced as shown below. 

The Bangdiwala measures (returned by the function)
represent the proportion of the
shaded areas of the diagonal rectangles, using weights $w_1$ for exact agreement,
and $w_2$ for partial agreement one step from the main diagonal.

```{r}
#| agreesex, 
#| h=6, w=7,
#| out.width = "70%",
#| cap = "Agreement plot for the `SexualFun` data."
agree <- agreementplot(SexualFun, main="Is sex fun?")
unlist(agree)
```


In other examples, the agreement plot can help to show *sources*
of disagreement.  For example, when the shaded boxes are above or below the diagonal
(red) line, a lack of exact agreement can be attributed in part to
different frequency of use of categories by the two raters-- lack of
*marginal homogeneity*.
	
## Correspondence analysis

Correspondence analysis is a technique for visually exploring relationships
between rows and columns in contingency tables. The `ca` package gives one implementation.
For an $r \times c$ table,
the method provides a breakdown of the Pearson $\chi^2$ for association in up to $M = \min(r-1, c-1)$
dimensions, and finds scores for the row ($x_{im}$) and column ($y_{jm}$) categories
such that the observations have the maximum possible correlations.%
^[Related methods are the non-parametric CMH tests using assumed row/column scores (\secref{sec:CMH}),
the analogous `glm()` model-based methods (\secref{sec:CMH}), and the more general RC models which can be fit using `gnm()`. Correspondence analysis differs in that it is a primarily descriptive/exploratory method (no significance tests), but is directly tied to informative graphic displays of the row/column categories.]



Here, we carry out a simple correspondence analysis of the `HairEye` data.
The printed results show that nearly 99% of the association between hair color and eye color
can be accounted for in 2 dimensions, of which the first dimension accounts for 90%.
```{r, ca1}
library(ca)
ca(HairEye)
```


The resulting `ca` object can be plotted just by running the `plot()`
method on the `ca` object, giving the result in
\figref{fig:ca-haireye}.  `plot.ca()` does not allow labels for dimensions;
these can be added with `title()`.
It can be seen that most of the association is accounted for by the ordering
of both hair color and eye color along Dimension 1, a dark to light dimension.

```{r ca-haireye, cap = "Correspondence analysis plot for the `HairEye` data"}
plot(ca(HairEye), main="Hair Color and Eye Color")
```


## References

% !Rnw weave = Sweave
%\VignetteEngine{Sweave}
%\VignetteIndexEntry{Tutorial: Working with categorical data with R and the vcd package}
%\VignetteDepends{vcd,gmodels,ca}
%\VignetteKeywords{contingency tables, mosaic plots, sieve plots, categorical data, independence, conditional independence, R}
%\VignettePackage{vcdExtra}

\documentclass[10pt,twoside]{article}
\usepackage{Sweave}
\usepackage{bm}
\usepackage[toc]{multitoc}   % for table of contents

% from Z.cls
\usepackage[authoryear,round,longnamesfirst]{natbib}
\bibpunct{(}{)}{;}{a}{}{,}
\bibliographystyle{jss}

\usepackage{hyperref}
\usepackage{color}
%% colors
\definecolor{Red}{rgb}{0.7,0,0}
\definecolor{Blue}{rgb}{0,0,0.8}
\hypersetup{%
  hyperindex = {true},
  colorlinks = {true},
%  linktocpage = {true},
  plainpages = {false},
  linkcolor = {Blue},
  citecolor = {Blue},
  urlcolor = {Red},
  pdfstartview = {Fit},
  pdfpagemode = {UseOutlines},
  pdfview = {XYZ null null null}
}
%\AtBeginDocument{
%  \hypersetup{%
%    pdfauthor = {Michael Friendly},
%    pdftitle = {Tutorial: Working with categorical data with R and the vcd package},
%    pdfkeywords = {contingency tables, mosaic plots, sieve plots, categorical data, independence, conditional independence, R}
%  }
%}


% math stuff
\newcommand*{\given}{\ensuremath{\, | \,}}
\renewcommand*{\vec}[1]{\ensuremath{\bm{#1}}}
\newcommand{\mat}[1]{\ensuremath{\bm{#1}}}
\newcommand{\trans}{\ensuremath{^\mathsf{T}}}
\newcommand{\diag}[1]{\ensuremath{\mathrm{diag} (#1)}}
\def\binom#1#2{{#1 \choose #2}}%
\newcommand{\implies}{ \ensuremath{\mapsto} }

\newenvironment{equation*}{\displaymath}{\enddisplaymath}%

\newcommand{\tabref}[1]{Table~\ref{#1}}
\newcommand{\figref}[1]{Figure~\ref{#1}}
\newcommand{\secref}[1]{Section~\ref{#1}}
\newcommand{\loglin}{loglinear }

%\usepackage{thumbpdf}

% page dimensions
\addtolength{\hoffset}{-1.5cm}
\addtolength{\textwidth}{3cm}
\addtolength{\voffset}{-1cm}
\addtolength{\textheight}{2cm}

% Vignette examples
\newcommand*{\Example}{\fbox{\textbf{\emph{Example}}:} }

% R stuff
\newcommand{\var}[1]{\textit{\texttt{#1}}}
\newcommand{\data}[1]{\texttt{#1}}
\newcommand{\class}[1]{\textsf{"#1"}}
%% \code without `-' ligatures
\def\nohyphenation{\hyphenchar\font=-1 \aftergroup\restorehyphenation}
\def\restorehyphenation{\hyphenchar\font=`-}
{\catcode`\-=\active%
  \global\def\code{\bgroup%
    \catcode`\-=\active \let-\codedash%
    \Rd@code}}
\def\codedash{-\discretionary{}{}{}}
\def\Rd@code#1{\texttt{\nohyphenation#1}\egroup}

\newcommand{\codefun}[1]{\code{#1()}}
\let\proglang=\textsf
\newcommand{\pkg}[1]{{\normalfont\fontseries{b}\selectfont #1}}
\newcommand{\Rpackage}[1]{{\textsf{#1}}}

%% almost as usual
\author{Michael Friendly\\York University, Toronto}
\title{Working with categorical data with \proglang{R} and the \pkg{vcd} and \pkg{vcdExtra} packages}
\date{\footnotesize{Using \Rpackage{vcdExtra} version \Sexpr{packageDescription("vcdExtra")[["Version"]]}
  and \Rpackage{vcd} version \Sexpr{packageDescription("vcd")[["Version"]]}; Date: \Sexpr{Sys.Date()}}}


%% for pretty printing and a nice hypersummary also set:
%\Plainauthor{Michael Friendly} %% comma-separated
%\Shorttitle{vcd tutorial} %% a short title (if necessary)
%\Plaintitle{Tutorial: Working with categorical data with R and the vcd package}

%\SweaveOpts{engine=R,eps=TRUE,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
\SweaveOpts{engine=R,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
\SweaveOpts{prefix.string=fig/vcd-tut,eps=FALSE}
\SweaveOpts{keep.source=TRUE}
%\SweaveOpts{concordance=TRUE}
\setkeys{Gin}{width=0.7\textwidth}

<<preliminaries,echo=FALSE,results=hide>>=
set.seed(1071)
#library(vcd)
library(vcdExtra)
library(ggplot2)
#data(Titanic)
data(HairEyeColor)
data(PreSex)
data(Arthritis)
art <- xtabs(~Treatment + Improved, data = Arthritis)
if(!file.exists("fig")) dir.create("fig")
@



%% end of declarations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\begin{document}
\SweaveOpts{concordance=TRUE}
\maketitle

%% an abstract and keywords
\begin{abstract}
This tutorial describes the creation and manipulation of frequency and contingency tables
from categorical variables, along with tests of independence, measures
of association, and methods for graphically displaying results. 
The framework is provided by the \proglang{R} package \pkg{vcd},
but other packages are used to help with various tasks.  
The \pkg{vcdExtra} package extends the graphical and statistical methods provided by
\pkg{vcd}.  

This package is now the main support package for the book
\emph{Discrete Data Analysis with R: Visualizing and Modeling Techniques for Categorical
and Count Data} \citep{FriendlyMeyer:2016:DDAR}.  The web page for the book,
\href{http://ddar.datavis.ca}{ddar.datavis.ca}, gives further details.
\end{abstract}

%\keywords{contingency tables, mosaic plots, sieve plots, 
%categorical data, independence, conditional independence, generalized linear models,
%\proglang{R}}
%\Plainkeywords{contingency tables, mosaic plots, 
%  sieve plots, categorical data, independence, 
%  conditional independence, generalized linear models, R}

{\small
% \sloppy
% \begin{multicols}{2}
 \tableofcontents
% \end{multicols}
}


\section[Introduction]{Introduction}\label{sec:intro}
%% Note: If there is markup in \(sub)section, then it has to be escape as above.


This tutorial, part of the \pkg{vcdExtra} package,
describes how to work with categorical data in the context of
fitting statistical models in \proglang{R} and visualizing
the results using the \pkg{vcd} and \pkg{vcdExtra} packages.
It focuses first on methods and tools for creating
and manipulating \proglang{R} data objects which represent
frequency and contingency tables involving
categorical variables.

Further sections describe some simple methods for calculating 
tests of independence and measures of association amomg 
categorial variables, and also
methods for graphically displaying results.

There is much more to the analysis of categorical data than is described here,
where the emphasis is on cross-tabulated tables of frequencies (``contingency tables''),
statistical tests, associated \loglin\ models, and visualization of \emph{how}
variables are related.

A more general treatment of graphical methods for categorical data is contained
in the book, \emph{Discrete Data Analysis with R: Visualizing and Modeling Techniques for Categorical
and Count Data} \citep{FriendlyMeyer:2016:DDAR}. 
An earlier book using SAS is
\emph{Visualizing Categorical Data} \citep{vcd:Friendly:2000}, for which
\pkg{vcd} is a partial \proglang{R} companion, covering topics not otherwise
available in \proglang{R}.  
On the other hand, the implementation of graphical
methods in \pkg{vcd} is more general in many respects than what I provided in
\proglang{SAS}. Statistical models for categorical data in \proglang{R} have
been extended considerably with the \pkg{gnm} package for generalized \emph{nonlinear}
models.  The \pkg{vcdExtra} package extends \pkg{vcd} methods to models fit using
\codefun{glm} and \codefun{gnm}.

A more complete theoretical description of these statistical
methods is provided in Agresti's \citeyearpar{vcd:Agresti:2002,Agresti:2013}
\emph{Categorical Data Analysis}. For this, see
the \proglang{Splus/R} companion by Laura Thompson,
\url{http://www.stat.ufl.edu/~aa/cda/Thompson_manual.pdf}
and Agresti's support web page,
\url{http://www.stat.ufl.edu/~aa/cda/cda.html}.



\section[Creating frequency tables]{Creating and manipulating frequency tables}\label{sec:creating}

\proglang{R} provides many methods for creating frequency and contingency tables. Several are
described below. In the examples below, we use some real examples and some anonymous
ones, where the variables \code{A}, \code{B}, and \code{C}  represent
categorical variables, and \code{X} represents an arbitrary \proglang{R} data object.

The first thing you need to know is that categorical data can be represented in
three different forms in \proglang{R}, and it is sometimes necessary to convert
from one form to another, for carrying out statistical tests, fitting models
or visualizing the results.  Once a data object exists in \proglang{R}, 
you can examine its complete structure with
the \codefun{str} function, or view the names of its components with the
\codefun{names} function.
 
\begin{description}
  \item[case form] a data frame containing individual observations, with one or
  more factors, used as the classifying variables. In case form, there may also
  be numeric covariates. 
  The total number of observations
  is \code{nrow(X)}, and the number of variables is \code{ncol(X)}.
  	
\Example
The \data{Arthritis} data is available in case form in the \pkg{vcd} package. 
There are two explanatory factors: \code{Treatment} and \code{Sex}. \code{Age}
is a numeric covariate, and \code{Improved} is the response--- an ordered factor,
with levels 
\code{\Sexpr{paste(levels(Arthritis$Improved),collapse=' < ')}}.  
Excluding \code{Age}, we would have
a $2 \times 2 \times 3$ contingency table for \code{Treatment}, \code{Sex} and \code{Improved}.
%\code{"None" < "Some" < "Marked"}.
<<case-form,results=verbatim>>=
names(Arthritis)      # show the variables
str(Arthritis)        # show the structure
head(Arthritis,5)     # first 5 observations, same as Arthritis[1:5,] 
@
  
  \item[frequency form] a data frame containing one or more factors, and a frequency 
  variable, often called \code{Freq} or \code{count}.  The total number of observations
  is 
  \verb|sum(X$Freq)|,  
  \code{sum(X[,"Freq"])} or some equivalent form.
  The number of cells in the table is \code{nrow(X)}.

\Example 
For small frequency tables, it is often convenient to enter them in frequency form
using \codefun{expand.grid} for the factors and \codefun{c} to list the counts in a vector.
The example below, from \cite{vcd:Agresti:2002} gives results for the 1991 General Social Survey,
with respondents classified by sex and party identification.
<<frequency-form,results=verbatim>>=
# Agresti (2002), table 3.11, p. 106
GSS <- data.frame(
  expand.grid(sex=c("female", "male"), 
              party=c("dem", "indep", "rep")),
  count=c(279,165,73,47,225,191))
GSS
names(GSS)
str(GSS)
sum(GSS$count)
@

  \item[table form]  a matrix, array or table object, whose elements are the frequencies
  in an $n$-way table.  The variable names (factors) and their levels are given by
  \code{dimnames(X)}. The total number of observations
  is \code{sum(X)}.  The number of dimensions of the table is \code{length(dimnames(X))},
  and the table sizes are given by \code{sapply(dimnames(X), length)}.
  	
\Example 
The \data{HairEyeColor} is stored in table form in \pkg{vcd}.  
<<table-form1,results=verbatim>>=
str(HairEyeColor)                      # show the structure
sum(HairEyeColor)                      # number of cases
sapply(dimnames(HairEyeColor), length) # table dimension sizes
@

\Example 
Enter frequencies in a matrix, and assign \code{dimnames},
giving the variable names and category labels.  Note that, by default,
\codefun{matrix} uses the elements supplied by \emph{columns} in the
result, unless you specify \code{byrow=TRUE}.
<<table-form2,results=verbatim>>=
## A 4 x 4 table  Agresti (2002, Table 2.8, p. 57) Job Satisfaction
JobSat <- matrix(c(1,2,1,0, 3,3,6,1, 10,10,14,9, 6,7,12,11), 4, 4)
dimnames(JobSat) = list(income=c("< 15k", "15-25k", "25-40k", "> 40k"),
                satisfaction=c("VeryD", "LittleD", "ModerateS", "VeryS"))
JobSat
@
\data{JobSat} is a matrix, not an object of \code{class("table")}, and some functions
are happier with tables than matrices.
You can coerce it to a table with \codefun{as.table},
<<table-form3,results=verbatim>>=
JobSat <- as.table(JobSat)
str(JobSat)
@

\end{description}

\subsection[Ordered factors]{Ordered factors and reordered tables}\label{sec:ordered-factors}
In table form, the values of the table factors are ordered by their position in the table.
Thus in the \data{JobSat} data, both \code{income} and \code{satisfaction} represent ordered
factors, and the \emph{positions} of the values in the rows and columns reflects their
ordered nature.

Yet, for analysis, there are time when you need \emph{numeric} values for the levels
of ordered factors in a table, e.g., to treat a factor as a quantitative variable.
In such cases, you can simply re-assign the \code{dimnames} attribute of the table
variables.  For example, here, we assign numeric values to \code{income} as the middle of their
ranges, and treat \code{satisfaction} as equally spaced with integer scores.

<<relevel,results=hide,eval=FALSE>>=
dimnames(JobSat)$income<-c(7.5,20,32.5,60)
dimnames(JobSat)$satisfaction<-1:4
@

For the  \data{HairEyeColor} data, hair color and eye color are ordered arbitrarily.
For visualizing the data using mosaic plots and other methods described below, it 
turns out to be more useful to assure that both hair color and eye color are
ordered from dark to light.
Hair colors are actually ordered this way already, and it is easiest to re-order
eye colors by indexing. Again \codefun{str} is your friend.

<<reorder1,results=verbatim>>=
HairEyeColor <- HairEyeColor[, c(1,3,4,2), ]
str(HairEyeColor)
@
This is also the order for both hair color and eye color shown in 
the result of a correspondence analysis (\figref{fig:ca-haireye}) below.

With data in  case form or frequency form, when you have ordered factors
represented with character values, you must ensure that they are treated
as ordered in \proglang{R}.%
\footnote{In \proglang{SAS}, many procedures offer the option
	\code{order = data | internal | formatted} to allow character values
	to be ordered according to (a) their order in the data set, (b)
	sorted internal value, or (c) sorted formatted representation
	provided by a \proglang{SAS} format.
}

Imagine that the \data{Arthritis} data was read from a text file.  
By default the \code{Improved} will be ordered alphabetically:
\code{Marked},
\code{None},
\code{Some}--- not what we want.  In this case, the function
\codefun{ordered} (and others) can be useful. 

<<reorder2,echo=TRUE,eval=FALSE>>=
Arthritis <- read.csv("arthritis.txt",header=TRUE)
Arthritis$Improved <- ordered(Arthritis$Improved, levels=c("None", "Some", "Marked"))
@

With this order of  \code{Improved}, the response in this data,
a mosaic display of \code{Treatment} and \code{Improved} (\figref{fig:arthritis})shows a clearly
interpretable pattern.
<<Arthritis,fig=TRUE,echo=FALSE,height=6,width=7,results=hide,include=FALSE>>=
mosaic(art, gp = shading_max, split_vertical = TRUE, main="Arthritis: [Treatment] [Improved]")
@

%\setkeys{Gin}{width=0.7\textwidth}
\begin{figure}[htb]
\begin{center}
%<<Arthritis,fig=TRUE,echo=FALSE,height=6,width=7,results=hide>>=
%mosaic(art, gp = shading_max, split_vertical = TRUE, main="Arthritis: [Treatment] [Improved]")
%@
\includegraphics[width=0.7\textwidth]{fig/vcd-tut-Arthritis}
\caption{Mosaic plot for the \data{Arthritis} data, showing the marginal model of independence
	for Treatment and Improved.  Age, a covariate, and Sex are ignored here.}
\label{fig:arthritis}
\end{center}
\end{figure}

Finally, there are situations where, particularly for display purposes, you
want to re-order the \emph{dimensions} of an $n$-way table, or change the
labels for the variables or levels.
This is easy when the data are in table form: \codefun{aperm} permutes
the dimensions, and assigning to \code{names} and \code{dimnames}
changes variable names and level labels respectively.
We will use the following version of \data{UCBAdmissions} in
\secref{sec:mantel} below.%
\footnote{
Changing \code{Admit} to \code{Admit?} might be useful for display purposes, but is
dangerous--- because it is then
difficult to use that variable name in a model formula.
See \secref{sec:tips} for options \code{labeling\_args} and \code{set\_labels}
to change variable and level names 
for displays in the \code{strucplot} framework.
}
<<reorder3,results=verbatim>>=
UCB <- aperm(UCBAdmissions, c(2, 1, 3))
dimnames(UCB)[[2]] <- c("Yes", "No")
names(dimnames(UCB)) <- c("Sex", "Admit?", "Department")
ftable(UCB)
@

%There is one subtle ``gotcha'' here:  \codefun{aperm} returns an object of class \class{"array"},
%whereas \data{UCBAdmissions} is of class \class{"table"}, so methods defined for \code{table}
%objects will not work on the permuted array.
%The solution is to reassign the \code{class} of the result of  \codefun{aperm}.
%
%<<reorder4,results=verbatim, keep.source=TRUE>>=
%class(UCBAdmissions)
%class(UCB)
%str(as.data.frame(UCBAdmissions))  # OK
%str(as.data.frame(UCB))            # wrong
%
%class(UCB) <- "table"
%str(as.data.frame(UCB))            # now OK
%@
%
\subsection[structable()]{\codefun{structable}}\label{sec:structable}
For 3-way and larger tables
the \codefun{structable} function in \pkg{vcd} provides a convenient and flexible tabular display.
The variables assigned to the rows and columns of a two-way display can be specified
by a model formula.
<<structable,results=verbatim>>=
structable(HairEyeColor)                   # show the table: default
structable(Hair+Sex ~ Eye, HairEyeColor)   # specify col ~ row variables
@
It also returns an object of class \code{"structable"} which may be plotted with 
\codefun{mosaic} (not shown here).
<<structable1,eval=FALSE>>=
HSE < - structable(Hair+Sex ~ Eye, HairEyeColor)   # save structable object
mosaic(HSE)                                        # plot it
@

\subsection[table() and friends]{\codefun{table} and friends}\label{sec:table}

You  can  generate frequency  tables from factor variables  using the  \codefun{table} function,  tables  of
proportions using  the \codefun{prop.table} function,  and marginal  frequencies using
\codefun{margin.table}.

<<setup>>=
 n=500
 A <- factor(sample(c("a1","a2"), n, rep=TRUE))
 B <- factor(sample(c("b1","b2"), n, rep=TRUE))
 C <- factor(sample(c("c1","c2"), n, rep=TRUE))
 mydata <- data.frame(A,B,C)
@

<<table-ex1>>=
# 2-Way Frequency Table
attach(mydata)
mytable <- table(A,B)   # A will be rows, B will be columns
mytable                 # print table

margin.table(mytable, 1) # A frequencies (summed over B)
margin.table(mytable, 2) # B frequencies (summed over A)

prop.table(mytable)    # cell percentages
prop.table(mytable, 1) # row percentages
prop.table(mytable, 2) # column percentages
@

\codefun{table} can  also  generate  multidimensional  tables  based  on  3  or  more
categorical variables. In  this case, use  the \codefun{ftable}  or \codefun{structable}
function to print  the
results more attractively.

<<table-ex2>>=
# 3-Way Frequency Table
mytable <- table(A, B, C)
ftable(mytable)
@

\codefun{table}  ignores missing values by default. 
To include \code{NA} as a category in counts, include the
table option  \code{exclude=NULL} if  the variable  is a  vector. If  the variable is a
factor you  have to  create a  new factor  using \code{newfactor  <- factor(oldfactor,
exclude=NULL)}. 

\subsection[xtabs()]{\codefun{xtabs}}\label{sec:xtabs}

The \codefun{xtabs} function allows you to create crosstabulations of data using formula style input.
This typically works with case-form data supplied in a data frame or a matrix.
The result is a contingency table in array format, whose dimensions are determined by  
the terms on the right side of the formula.

<<xtabs-ex1>>=
# 3-Way Frequency Table
mytable <- xtabs(~A+B+C, data=mydata)
ftable(mytable)    # print table
summary(mytable)   # chi-square test of indepedence
@

If a variable is included on the left side of the formula, it is assumed to be a
vector  of  frequencies  (useful if the data have already been tabulated in frequency form).

<<xtabs-ex2,results=verbatim>>=
(GSStab <- xtabs(count ~ sex + party, data=GSS))
summary(GSStab)
@

\subsection[Collapsing over factors]{Collapsing over table factors: \codefun{aggregate}, \codefun{margin.table} and \codefun{apply}}
It sometimes happens that we have a data set with more variables or factors than 
we want to analyse, or else, having done some initial analyses, we decide that
certain factors are not important, and so should be excluded from graphic displays
by collapsing (summing) over them.  For example, mosaic plots and fourfold displays
are often simpler to construct from versions of the data collapsed over
the factors which are not shown in the plots.

The appropriate tools to use again depend on
the form in which the data are represented--- a case-form data frame, a
frequency-form data frame (\codefun{aggregate}), or a table-form array or 
table object (\codefun{margin.table} or \codefun{apply}).

When the data are in frequency form, and we want to produce another
frequency data frame, \codefun{aggregate} is a handy tool, using
the argument \code{FUN=sum} to sum the frequency variable over the 
factors \emph{not} mentioned in the formula.

\Example 
The data frame \data{DaytonSurvey} in the \pkg{vcdExtra} package represents a
$2^5$ table giving the frequencies of reported use (``ever used?'') of 
alcohol, cigarettes and marijuana in a sample of high school seniors,
also classified by sex and race.  

<<dayton1, results=verbatim>>=
str(DaytonSurvey)
head(DaytonSurvey)
@

To focus on the associations among the
substances, we want to collapse over sex and race. The right-hand side of the formula
used in the call to \codefun{aggregate} gives the factors to be retained in the
new frequency data frame, \code{Dayton.ACM.df}.

<<dayton2, results=verbatim>>=
# data in frequency form
# collapse over sex and race
Dayton.ACM.df <- aggregate(Freq ~ cigarette+alcohol+marijuana, 
                           data=DaytonSurvey, FUN=sum)
Dayton.ACM.df
@

When the data are in table form, and we want to produce another
table, \codefun{apply} with \code{FUN=sum} can be used in a similar way
to sum the table over dimensions not mentioned in the \code{MARGIN}
argument.  \codefun{margin.table} is just a wrapper for \codefun{apply}
using the \codefun{sum} function.


\Example 
To illustrate, we first convert the \data{DaytonSurvey} to a 5-way
table using \codefun{xtabs}, giving \code{Dayton.tab}.  

<<dayton3, results=verbatim>>==
# in table form
Dayton.tab <- xtabs(Freq~cigarette+alcohol+marijuana+sex+race, data=DaytonSurvey)
structable(cigarette+alcohol+marijuana ~ sex+race, data=Dayton.tab)
@
Then, use \codefun{apply} on \code{Dayton.tab} to give the
3-way table \code{Dayton.ACM.tab} summed over sex and race.
The elements in this new table are the column sums for 
\code{Dayton.tab} shown by \codefun{structable} just above.

<<dayton4, results=verbatim>>==
# collapse over sex and race
Dayton.ACM.tab <- apply(Dayton.tab, MARGIN=1:3, FUN=sum)
Dayton.ACM.tab <- margin.table(Dayton.tab, 1:3)   # same result
structable(cigarette+alcohol ~ marijuana, data=Dayton.ACM.tab)
@



Many of these operations can be performed using the \verb|**ply()| functions
in the \pkg{plyr} package.
For example, with the data in a frequency form data frame, use \codefun{ddply}
to collapse over unmentioned factors, and \codefun{plyr::summarise}%
\footnote{
Ugh. This \pkg{plyr} function clashes with a function of the same name in \pkg{vcdExtra}.
In this document I will use the explicit double-colon notation to keep them
separate.
}
as the function to be applied to each piece.
<<dayton5, eval=FALSE>>==
Dayton.ACM.df <- ddply(DaytonSurvey, .(cigarette, alcohol, marijuana), 
                       plyr::summarise, Freq=sum(Freq))
@

\subsection[Collapsing levels]{Collapsing table levels: \codefun{collapse.table}}

A related problem arises when we have a table or array and for some purpose
we want to reduce the number of levels of some factors by summing subsets
of the frequencies.  For example, we may have initially coded Age in 10-year
intervals, and decide that, either for analysis or display purposes, we
want to reduce Age to 20-year intervals.  The \codefun{collapse.table} function
in \pkg{vcdExtra} was designed for this purpose.

\Example
Create a 3-way table, and collapse Age from 10-year to 20-year intervals. 
First, we generate a $2 \times 6 \times 3$ table of random counts from a 
Poisson distribution with mean of 100.
<<collapse1,results=verbatim>>=
# create some sample data in frequency form
sex <- c("Male", "Female")
age <- c("10-19", "20-29",  "30-39", "40-49", "50-59", "60-69")
education <- c("low", 'med', 'high')
data <- expand.grid(sex=sex, age=age, education=education)
counts <- rpois(36, 100)   # random Possion cell frequencies
data <- cbind(data, counts)

# make it into a 3-way table
t1 <- xtabs(counts ~ sex + age + education, data=data)
structable(t1)
@ 
Now collapse \code{age} to 20-year intervals, and \code{education}
to 2 levels. In the arguments, levels of \code{age} and \code{education}
given the same label are summed in the resulting smaller table.
<<collapse2, results=verbatim>>=
# collapse age to 3 levels, education to 2 levels
t2 <- collapse.table(t1, 
         age=c("10-29", "10-29",  "30-49", "30-49", "50-69", "50-69"),
         education=c("<high", "<high", "high"))
structable(t2)
@


\subsection[Converting]{Converting among frequency tables and data frames}

As we've seen, a given contingency table can be represented 
equivalently in different forms,
but some \proglang{R} functions were designed for one particular representation.
\tabref{tab:convert} shows some handy tools for converting from one form to another.

\begin{table}[htb]
 \caption{Tools for converting among different forms for categorical data}\label{tab:convert}
 \begin{center}
   \begin{tabular}{llll}
	\hline
                 & \multicolumn{3}{c}{\textbf{To this}} \\
	\textbf{From this}      &     Case form         & Frequency form             &  Table form \\
	\hline
	Case form      &   noop                 & \verb|xtabs(~A+B)|        &  \verb|table(A,B)|  \\ 
	Frequency form &  \verb|expand.dft(X)|  & noop                      & \verb|xtabs(count~A+B)|\\
	Table form     &  \verb|expand.dft(X)|  & \verb|as.data.frame(X)|   &  noop \\
	\hline
   \end{tabular}
 \end{center}
\end{table}

A contingency table in table form (an object of \code{class(table)}) can be converted
to a data.frame with \codefun{as.data.frame}.%
\footnote{
Because \proglang{R} is object-oriented, this is actually a short-hand for
the function \codefun{as.data.frame.table}.
}  
The resulting
\code{data.frame} contains columns
representing the classifying factors and the table entries (as a column named by
the \code{responseName} argument, defaulting to \code{Freq}. This is the inverse of \codefun{xtabs}.

\Example 
Convert the \code{GSStab} in table form to a data.frame in frequency form.
<<convert-ex1,results=verbatim>>=
as.data.frame(GSStab)
@

\Example Convert the \code{Arthritis} data in case form to a 3-way table of
\code{Treatment} $\times$ \code{Sex} $\times$ \code{Improved}.
Note the use of \codefun{with} to avoid having to use \code{Arthritis\$Treatment} etc. within the call to \codefun{table}.%
\footnote{
\codefun{table} does not allow a \code{data} argument to provide
an environment in which the table variables are to be found.  In the 
examples in \secref{sec:table} I used \code{attach(mydata)} for this purpose,
but \codefun{attach} leaves the variables in the global environment,
while \codefun{with} just evaluates the \codefun{table} expression in a
temporary environment of the data.
}
<<convert-ex2,results=verbatim>>=
Art.tab <-with(Arthritis, table(Treatment, Sex, Improved))
str(Art.tab)
ftable(Art.tab)
@


There may also be times that you will need an equivalent case form \code{data.frame}
with factors  representing the table variables
rather than the frequency  table.
For example, the \codefun{mca} function in package \pkg{MASS}
only operates on data in this format. 
Marc Schwartz provided code for \codefun{expand.dft} on the  Rhelp
mailing list for converting a table back into a case form \code{data.frame}.
This function is included in \pkg{vcdExtra}.

\Example Convert the \data{Arthritis} data in table form (\code{Art.tab}) back to a \code{data.frame}
in case form, with factors
\code{Treatment}, \code{Sex} and \code{Improved}.
<<convert-ex3,results=verbatim>>=
Art.df <- expand.dft(Art.tab)
str(Art.df)
@

\subsection{A complex example}\label{sec:complex}

If you've followed so far, you're ready for a more complicated example.
The data file, \code{tv.dat} represents a 4-way table of size
$5 \times 11 \times 5 \times 3$ where the table variables (unnamed in the file)
are read as \code{V1} -- \code{V4}, and the cell frequency is read
as \code{V5}.  The file, stored in the \code{doc/extdata} directory
of \pkg{vcdExtra}, can be read as follows:
<<tv1,results=verbatim>>=
tv.data<-read.table(system.file("extdata","tv.dat", package="vcdExtra"))
head(tv.data,5)
@
For a local file, just use \codefun{read.table} in this form:
<<tv2,eval=FALSE>>=
tv.data<-read.table("C:/R/data/tv.dat")
@
The data \code{tv.dat} came from the initial implementation of 
mosaic displays in \proglang{R} by Jay Emerson. 
In turn, they came from the initial development of mosaic displays 
\citep{vcd:Hartigan+Kleiner:1984}
that illustrated the method with data on a large sample of TV viewers
whose behavior had been recorded for the Neilson ratings.
This data set contains sample television audience data from Neilsen
Media Research for the week starting November 6, 1995.

\begin{flushleft}
The table variables are:\\
~~~\code{V1}-- values 1:5 correspond to the days Monday--Friday;\\
~~~\code{V2}-- values 1:11 correspond to the quarter hour times 8:00PM through 10:30PM;\\
~~~\code{V3}-- values 1:5 correspond to ABC, CBS, NBC, Fox, and non-network choices;\\
~~~\code{V4}-- values 1:3 correspond to transition states: turn the television Off, Switch channels, 
 or Persist in viewing the current channel.
\end{flushleft}
	 
We are interested just the cell frequencies, and rely on the facts that the
(a) the table is complete--- there are no missing cells,
so \code{nrow(tv.data)}=\Sexpr{nrow(tv.data)};
(b) the observations are ordered so that \code{V1} varies most rapidly and
\code{V4} most slowly.  From this, we can just extract the frequency column
and reshape it into an array.
<<tv2,results=hide>>=
TV <- array(tv.data[,5], dim=c(5,11,5,3))                                        
dimnames(TV) <- list(c("Monday","Tuesday","Wednesday","Thursday","Friday"), 
                c("8:00","8:15","8:30","8:45","9:00","9:15","9:30",         
                  "9:45","10:00","10:15","10:30"),                            
                c("ABC","CBS","NBC","Fox","Other"), c("Off","Switch","Persist"))
names(dimnames(TV))<-c("Day", "Time", "Network", "State")
@

More generally (even if there are missing cells), we can 
use \codefun{xtabs} (or \codefun{plyr::daply})
to do the cross-tabulation, using \code{V5} as the
frequency variable.  Here's how to do this same operation with \codefun{xtabs}:
<<tv2a,eval=FALSE>>=
TV <- xtabs(V5 ~ ., data=tv.data)
dimnames(TV) <- list(Day=c("Monday","Tuesday","Wednesday","Thursday","Friday"), 
                Time=c("8:00","8:15","8:30","8:45","9:00","9:15","9:30",         
                       "9:45","10:00","10:15","10:30"),                            
                Network=c("ABC","CBS","NBC","Fox","Other"), 
                State=c("Off","Switch","Persist"))
@

But this 4-way table is too large and awkward to work with. Among the networks,
Fox and Other occur infrequently. 
We can also cut it down to a 3-way table by considering only viewers who persist
with the current station.%
\footnote{This relies on the fact that that indexing
an array drops dimensions of length 1 by default,
using the argument \code{drop=TRUE};
the result is coerced to the lowest possible dimension.
}

<<tv3,results=verbatim>>=
TV <- TV[,,1:3,]     # keep only ABC, CBS, NBC
TV <- TV[,,,3]       # keep only Persist -- now a 3 way table
structable(TV)
@

Finally, for some purposes, we might want to collapse the 11 times into a smaller number.
Here, we use \codefun{as.data.frame.table} to convert the table back to a data frame,
 \codefun{levels} to re-assign the values of \code{Time},
 and finally, \codefun{xtabs} to give a new, collapsed frequency table.

<<tv4,results=verbatim>>=
TV.df <- as.data.frame.table(TV)
levels(TV.df$Time) <- c(rep("8:00-8:59",4),rep("9:00-9:59",4), rep("10:00-10:44",3))
TV2 <- xtabs(Freq ~ Day + Time + Network, TV.df)
structable(Day ~ Time+Network,TV2)
@
Whew! See \figref{fig:TV-mosaic} for a mosaic plot of the \code{TV2} data.


\section{Tests of Independence}

\subsection{CrossTable}

OK, now we're ready to do some analyses.  For tabular displays,
the  \codefun{CrossTable}  function in  the \pkg{gmodels}  package produces  cross-tabulations
modeled after \code{PROC FREQ} in \proglang{SAS} or \code{CROSSTABS} in \proglang{SPSS}. 
It has a wealth of options for the quantities that can be shown in each cell.

<<xtabs-ex2, results=verbatim>>=
# 2-Way Cross Tabulation
library(gmodels)
CrossTable(GSStab,prop.t=FALSE,prop.r=FALSE,prop.c=FALSE)
@
There are  options to  report percentages  (row, column,  cell), specify decimal
places, produce Chi-square,  Fisher, and McNemar  tests of independence,  report
expected  and residual  values (pearson,  standardized, adjusted  standardized),
include missing values as valid, annotate with row and column titles, and format
as \proglang{SAS} or \proglang{SPSS} style output! See \code{help(CrossTable)} for details.


\subsection{Chi-square test}

For 2-way tables you can use \codefun{chisq.test} to test independence of the row
and column variable. By default,  the $p$-value is calculated from  the asymptotic
chi-squared distribution of the test  statistic. Optionally, the $p$-value can  be
derived via Monte Carlo simulation. 

<<chisq,results=verbatim>>=
(HairEye <- margin.table(HairEyeColor, c(1, 2)))
chisq.test(HairEye)
@

\subsection{Fisher Exact Test}\label{sec:Fisher}

\code{fisher.test(X)} provides an  exact test of  independence. \code{X} must be  a two-way
contingency table in table form.  Another form, 
\code{fisher.test(X, Y)} takes two
categorical vectors of the same length.  
For tables larger than $2 \times 2$ the method can be computationally intensive (or can fail) if
the frequencies are not small.

<<fisher,results=verbatim>>=
fisher.test(GSStab)
@

But this does not work because \data{HairEye} data has $n$=592 total frequency.
An exact test is unnecessary in this case.
<<echo=TRUE,eval=FALSE>>=
fisher.test(HairEye)
@
%# <<echo=TRUE,eval=FALSE,results=verbatim>>=
%# #cat(try(fisher.test(HairEye)))
%# @
\begin{Soutput}
Error in fisher.test(HairEye) : FEXACT error 6.
LDKEY is too small for this problem.
Try increasing the size of the workspace.
\end{Soutput}

\subsection[Mantel-Haenszel test]{Mantel-Haenszel test and conditional association}\label{sec:mantel}

Use the  \code{mantelhaen.test(X)} function  to perform  a Cochran-Mantel-Haenszel 
$\chi^2$ chi
test  of   the  null  hypothesis   that  two  nominal   variables  are
\emph{conditionally independent}, $A \perp B \given C$, in each stratum,  assuming that there is no  three-way
interaction. \code{X} is  a 3 dimensional  contingency table, where  the last dimension
refers to the strata.

The \data{UCBAdmissions} serves as an example of a $2 \times 2 \times 6$ table,
with \code{Dept} as the stratifying variable.
<<mantel1,results=verbatim>>=
## UC Berkeley Student Admissions
mantelhaen.test(UCBAdmissions)
@
The results show no evidence for association between admission and gender
when adjusted for department.  However, we can easily see that the assumption
of equal association across the strata (no 3-way association) is probably
violated. For $2 \times 2 \times k$ tables, this can be examimed
from the odds ratios for each $2 \times 2$ table (\codefun{oddsratio}), and
tested 
by  using
\verb|woolf_test()| in \pkg{vcd}.

%<<mantel2,results=verbatim>>=
%oddsRatio <- function(x) (x[1,1]*x[2,2])/(x[1,2]*x[2,1])
%apply(UCBAdmissions, 3, oddsRatio)
%
%woolf_test(UCBAdmissions) 
%@ 
<<mantel2,results=verbatim>>=
oddsratio(UCBAdmissions, log=FALSE)
lor <- oddsratio(UCBAdmissions)  # capture log odds ratios
summary(lor)
woolf_test(UCBAdmissions) 
@ 
We  can visualize the  odds ratios of  Admission for
each  department  with  fourfold  displays  using  \codefun{fourfold}.  The cell
frequencies $n_{ij}$  of each  $2 \times  2$ table  are shown  as a  quarter circle whose
radius is proportional to $\sqrt{n_{ij}}$, so  that its area is proportional to  the
cell frequency.
Confidence rings for the odds ratio allow a visual test of the null of no association; 
the rings for adjacent quadrants overlap \emph{iff} the observed counts are consistent 
with the null hypothesis.  In the extended version (the default), brighter colors
are used where the odds ratio is significantly different from 1.
The following lines produce \figref{fig:fourfold1}.%
\footnote{The color values \code{col[3:4]} were modified from their default values
to show a greater contrast between significant and insignifcant associations here.}
<<fourfold1,fig=TRUE,height=6,width=9,results=hide,include=FALSE>>=
col <- c("#99CCFF", "#6699CC", "#F9AFAF", "#6666A0", "#FF0000", "#000080")
fourfold(UCB,mfrow=c(2,3), color=col)
@
 
%\setkeys{Gin}{width=0.8\textwidth}
\begin{figure}[htb]
\begin{center}
%<<fourfold1,fig=TRUE,height=6,width=9,results=hide>>=
%col <- c("#99CCFF", "#6699CC", "#F9AFAF", "#6666A0", "#FF0000", "#000080")
%fourfold(UCB,mfrow=c(2,3), color=col)
%@
\includegraphics[width=0.8\textwidth,trim=80 50 80 50]{fig/vcd-tut-fourfold1}
\caption{Fourfold display for the \data{UCBAdmissions} data. Where the odds ratio differs
	significantly from 1.0, the confidence bands do not overlap, and the circle quadrants are
	shaded more intensely.}
\label{fig:fourfold1}
\end{center}
\end{figure}

Another \pkg{vcd} function, \codefun{cotabplot}, provides a more general approach
to visualizing conditional associations in contingency tables,
similar to trellis-like plots produced by \codefun{coplot} and lattice graphics.
The \code{panel} argument supplies a function used to render each conditional 
subtable. The following gives a display (not shown) similar to \figref{fig:fourfold1}.
<<fourfold2,eval=FALSE>>=
cotabplot(UCB, panel = cotab_fourfold)
@

When we want to view the conditional
probabilities of a response variable (e.g., \code{Admit})
in relation to several factors,
an alternative visualization is a \codefun{doubledecker} plot.
This plot is a specialized version of a mosaic plot, which
highlights the levels of a response variable (plotted vertically)
in relation to the factors (shown horizontally). The following
call produces \figref{fig:doubledecker}, where we use indexing
on the first factor (\code{Admit}) to make \code{Admitted}
the highlighted level.

In this plot, the
association between \code{Admit} and \code{Gender} is shown
where the heights of the highlighted conditional probabilities
do not align. The excess of females admitted in Dept A stands out here.

<<doubledecker,fig=TRUE, height=5,width=9,include=FALSE>>=
doubledecker(Admit ~ Dept + Gender, data=UCBAdmissions[2:1,,])
@

\begin{figure}[htb]
\begin{center}
\includegraphics[width=0.9\textwidth]{fig/vcd-tut-doubledecker}
\caption{Doubledecker display for the \data{UCBAdmissions} data. The heights
of the highlighted bars show the conditional probabilities of \texttt{Admit},
given \texttt{Dept} and \texttt{Gender}.}
\label{fig:doubledecker}
\end{center}
\end{figure}

Finally, the there is a \codefun{plot} method for \code{oddsratio} objects.
By default, it shows the 95\% confidence interval for the log odds ratio.
\figref{fig:oddsratio} is produced by:
<<oddsratio0,eval=FALSE>>=
plot(lor, xlab="Department", ylab="Log Odds Ratio (Admit | Gender)")
@

\setkeys{Gin}{width=0.5\textwidth}
\begin{figure}[htb]
\begin{center}
<<oddsratio,fig=TRUE,height=6,width=6,echo=FALSE>>=
plot(lor, xlab="Department", ylab="Log Odds Ratio (Admit | Gender)")
@
\caption{Log odds ratio plot for the \data{UCBAdmissions} data.}
\label{fig:oddsratio}
\end{center}
\end{figure}

\subsection[CMH tests: ordinal factors]{Cochran-Mantel-Haenszel tests for ordinal factors}\label{sec:CMH}
The standard $\chi^2$ tests for association in a two-way table
treat both table factors as nominal (unordered) categories.
When one or both factors of a two-way table are
quantitative or ordinal, more powerful tests of association
may be obtaianed by taking ordinality into account, using
row and or column scores to test for linear trends or differences
in row or column means.

More general versions of the CMH tests (Landis etal., 1978) are provided by assigning
numeric scores to the row and/or column variables. 
For example, with two ordinal factors (assumed to be equally spaced), assigning
integer scores, \code{1:R} and \code{1:C} tests the linear $\times$ linear component
of association. This is statistically equivalent to the Pearson correlation between the
integer-scored table variables, with $\chi^2 = (n-1) r^2$, with only 1 $df$
rather than $(R-1)\times(C-1)$ for the test of general association.

When only one table
variable is ordinal, these general CMH tests are analogous to an ANOVA, testing
whether the row mean scores or column mean scores are equal, again consuming
fewer $df$ than the test of general association.

The \codefun{CMHtest} function in \pkg{vcdExtra} now calculates these various
CMH tests for two possibly ordered factors, optionally stratified other factor(s).

\Example
Recall the $4 \times 4$ table, \code{JobSat} introduced in \secref{sec:creating},
<<jobsat, results=verbatim>>=
JobSat
@

Treating the \code{satisfaction} levels as equally spaced, but using
midpoints of the \code{income} categories as row scores gives the following results:
<<cmh1, results=verbatim>>=
CMHtest(JobSat, rscores=c(7.5,20,32.5,60))
@

Note that with the relatively small cell frequencies, the test for general 
give no evidence for association. However, the the \code{cor} test for linear x linear
association on 1 df is nearly significant. The \pkg{coin} contains the
functions \verb|cmh_test()| and \verb|lbl_test()|
for CMH tests of general association and linear x linear association respectively.

\subsection{Measures of Association}

There are a variety of statistical measures of \emph{strength} of association for
contingency tables--- similar in spirit to $r$ or $r^2$ for continuous variables.
With a large sample size, even a small degree of association can show a 
significant $\chi^2$, as in the example below for the \data{GSS} data.

The  \codefun{assocstats}  function in \pkg{vcd}  calculates  the   $\phi$
contingency coefficient,  and Cramer's  V for  an $r \times c$  table. 
The input must be in table form, a two-way $r \times c$ table.  
It won't work with \data{GSS} in frequency form, but by now you should know how
to convert.
<<assoc1,results=verbatim>>=
assocstats(GSStab)
@

For tables with ordinal variables, like \data{JobSat}, some people prefer the
Goodman-Kruskal $\gamma$ statistic (\citet[\S 2.4.3]{vcd:Agresti:2002})
based on a comparison of concordant
and discordant pairs of observations in the case-form equivalent of a two-way table.
<<gamma,results=verbatim>>=
GKgamma(JobSat)
@

A web article by Richard Darlington,
\url{http://www.psych.cornell.edu/Darlington/crosstab/TABLE0.HTM}
gives further description of these and other measures of association.

\subsection{Measures of Agreement}
The
\codefun{Kappa} function in the \pkg{vcd} package calculates Cohen's $\kappa$ and weighted
$\kappa$ for a square two-way table with the same row and column categories \citep{Cohen:60}.%
\footnote{ 
Don't confuse this with \codefun{kappa} in base \proglang{R} that computes something
entirely different (the condition number of a matrix).
}
Normal-theory $z$-tests are obtained by dividing $\kappa$ by its asymptotic standard
error (ASE).  A \codefun{confint} method for \code{Kappa} objects provides confidence intervals.
<<kappa,results=verbatim>>=
(K <- Kappa(SexualFun))
confint(K)
@

A visualization of agreement, both unweighted and weighted for degree of departure
from exact agreement is provided by the \codefun{agreementplot} function.
\figref{fig:agreesex} shows the agreementplot for the \data{SexualFun} data,
produced as shown below. The Bangdiwala measures represent the proportion of the
shaded areas of the diagonal rectangles, using weights $w_1$ for exact agreement,
and $w_2$ for partial agreement one step from the main diagonal.
<<agreesex,fig=TRUE,height=6,width=7,results=verbatim,include=FALSE>>=
agree <- agreementplot(SexualFun, main="Is sex fun?")
unlist(agree)
@

%\setkeys{Gin}{width=0.5\textwidth}
\begin{figure}[htb]
\begin{center}
%<<agreesex,fig=TRUE,height=6,width=7,results=verbatim>>=
%agree <- agreementplot(SexualFun, main="Is sex fun?")
%agree
%@
\includegraphics[width=0.4\textwidth,trim=50 25 50 25]{fig/vcd-tut-agreesex}
\caption{Agreement plot for the \data{SexualFun} data.}
\label{fig:agreesex}
\end{center}
\end{figure}
In other examples, the agreement plot can help to show \emph{sources}
of disagreement.  For example, when the shaded boxes are above or below the diagonal
(red) line, a lack of exact agreement can be attributed in part to
different frequency of use of categories by the two raters-- lack of
\emph{marginal homogeneity}.
	
\subsection{Correspondence analysis}
Use the \pkg{ca} package for correspondence analysis for visually exploring relationships
between rows and columns in contingency tables.  For an $r \times c$ table,
the method provides a breakdown of the Pearson $\chi^2$ for association in up to $M = \min(r-1, c-1)$
dimensions, and finds scores for the row ($x_{im}$) and column ($y_{jm}$) categories
such that the observations have the maximum possible correlations.%
\footnote{
	Related methods are the non-parametric
	CMH tests using assumed row/column scores (\secref{sec:CMH}),
	the analogous \codefun{glm} model-based methods (\secref{sec:CMH}),
	and the more general RC models which can be fit using \codefun{gnm}.
	Correspondence analysis differs in that it is a primarily
	descriptive/exploratory method (no significance tests), but is
	directly tied to informative graphic displays of the row/column categories.
}	
	


Here, we carry out a simple correspondence analysis of the \data{HairEye} data.
The printed results show that nearly 99\% of the association between hair color and eye color
can be accounted for in 2 dimensions, of which the first dimension accounts for 90\%.
<<ca1,results=verbatim>>=
library(ca)
ca(HairEye)
@

The resulting \code{ca} object can be plotted just by running the \codefun{plot}
method on the \code{ca} object, giving the result in
\figref{fig:ca-haireye}.  \codefun{plot.ca} does not allow labels for dimensions;
these can be added with \codefun{title}.
It can be seen that most of the association is accounted for by the ordering
of both hair color and eye color along Dimension 1, a dark to light dimension.
<<ca-haireye0,echo=TRUE,eval=FALSE>>=
plot(ca(HairEye), main="Hair Color and Eye Color")
title(xlab="Dim 1 (89.4%)", ylab="Dim 2 (9.5%)")
@

\setkeys{Gin}{width=0.7\textwidth}
\begin{figure}[htb]
\begin{center}
<<ca-haireye,results=hide,fig=TRUE,echo=FALSE>>=
plot(ca(HairEye), main="Hair Color and Eye Color")
title(xlab="Dim 1 (89.4%)", ylab="Dim 2 (9.5%)")
@
\caption{Correspondence analysis plot for the \data{HairEye} data.}
\label{fig:ca-haireye}
\end{center}
\end{figure}

\section{Loglinear Models}\label{sec:loglin}

You can  use the  \codefun{loglm}  function in  the \pkg{MASS}  package to fit log-linear
models.  Equivalent models can also be fit (from a different perspective) as generalized
linear models with the \codefun{glm}  function using the \code{family='poisson'} argument,
and the \pkg{gnm} package provides a wider range of generalized \emph{nonlinear} models,
particularly for testing structured associations.
The visualization methods for these models were originally developed for models fit using \codefun{loglm},
so this approach is emphasized here.  Some extensions of these methods for models
fit using \codefun{glm} and \codefun{gnm} are contained in the \pkg{vcdExtra} package
and illustrated in \secref{sec:glm}.

Assume we  have a 3-way  contingency table based  on
variables A, B, and C.  
The possible different forms of \loglin\ models for a 3-way table are shown in \tabref{tab:loglin-3way}.
The \textbf{Model formula} column shows how to express each model for \codefun{loglm} in \proglang{R}.%
\footnote{
For \codefun{glm}, or \codefun{gnm}, with the data in the form of a frequency data.frame, the same model is specified
in the form \code{glm(Freq} $\sim$ \code{..., family="poisson")}, where \texttt{Freq} is  the name
of the cell frequency variable and \texttt{...} specifies the \textbf{Model formula}.
}
In the \textbf{Interpretation} column, the symbol ``$\perp$'' is to be read as ``is independent of,''
and ``$\given$'' means ``conditional on,'' or ``adjusting for,'' or just ``given''.

\begin{table}[htb]
 \caption{Log-linear Models for Three-Way Tables}\label{tab:loglin-3way}
 \begin{center}
 \begin{tabular}{llll}
  \hline
  \textbf{Model}           & \textbf{Model formula}  & \textbf{Symbol}& \textbf{Interpretation} \\
  \hline\hline 
  Mutual independence      & \verb|~A + B + C|       & $[A][B][C]$    & $A \perp B \perp C$ \\ 
  Joint independence       & \verb|~A*B + C|         & $[AB][C]$      & $(A \: B) \perp C$ \\ 
  Conditional independence & \verb|~(A+B)*C|         & $[AC][BC]$     & $(A \perp B) \given C$ \\ 
  All two-way associations & \verb|~A*B + A*C + B*C| & $[AB][AC][BC]$ & homogeneous association  \\ 
  Saturated model          & \verb|~A*B*C|           & $[ABC]$        & 3-way association \\ 
  \hline
 \end{tabular}
 \end{center}
\end{table}



For example, the formula \verb|~A + B + C| specifies the model of \emph{mutual independence} with
no associations among the three factors.  In standard notation for the expected frequencies
$m_{ijk}$, this corresponds to
\begin{equation*}
	\log ( m_{ijk} ) = \mu + \lambda_i^A + \lambda_j^B + \lambda_k^C \equiv \texttt{A + B + C}
\end{equation*}
The parameters $\lambda_i^A , \lambda_j^B$ and  $\lambda_k^C$ pertain to the differences among the
one-way marginal frequencies for the factors A, B and C. 

Similarly, the model of \emph{joint independence},  $(A \: B) \perp C$,
allows an association between A and B, but specifies that
C is independent of both of these and their combinations,
\begin{equation*}
	\log ( m_{ijk} ) = \mu + \lambda_i^A + \lambda_j^B + \lambda_k^C + \lambda_{ij}^{AB} \equiv \texttt{A * B + C}
\end{equation*}
where the parameters $\lambda_{ij}^{AB}$ pertain to the overall association between A and B (collapsing over C).

In the literature or text books, you will often find these models expressed in shorthand symbolic notation,
using brackets, \texttt{[ ]} to enclose the \emph{high-order terms} in the model.
Thus, the joint independence model can be denoted \texttt{[AB][C]}, as shown in the \textbf{Symbol} column in 
\tabref{tab:loglin-3way}.

Models of \emph{conditional independence} allow (and fit)  two of the three possible
two-way associations.  There are three such models, depending on which variable is conditioned upon.
For a given conditional independence model, e.g., \texttt{[AB][AC]}, the given variable is the one
common to all terms, so this example has the interpretation $(B \perp C) \given A$.


\subsection[Fitting with loglm()]{Fitting with \codefun{loglm}}\label{sec:loglm}
For example, we can fit the model of mutual independence among hair color, eye color and sex 
in \data{HairEyeColor} as

<<loglm-hec1,results=verbatim>>=
library(MASS)
## Independence model of hair and eye color and sex.  
hec.1 <- loglm(~Hair+Eye+Sex, data=HairEyeColor)
hec.1
@

Similarly, the models of conditional independence and joint independence are specified as
<<loglm-hec2,results=verbatim>>=
## Conditional independence
hec.2 <- loglm(~(Hair + Eye) * Sex, data=HairEyeColor)
hec.2
@
<<loglm-hec3,results=verbatim>>=
## Joint independence model.  
hec.3 <- loglm(~Hair*Eye + Sex, data=HairEyeColor)
hec.3
@
Note that printing the model gives a brief summary of the goodness of fit.
A set of models can be compared using the \codefun{anova} function.

<<loglm-anova,results=verbatim>>=
anova(hec.1, hec.2, hec.3)
@

%Martin Theus and Stephan Lauer have written an excellent article on  Visualizing
%Loglinear Models, using  mosaic plots. There  is also great  tutorial example by
%Kevin Quinn on analyzing loglinear models via glm.

\subsection[Fitting with glm() and gnm()]{Fitting with \codefun{glm} and \codefun{gnm}}\label{sec:glm}
The \codefun{glm} approach, and extensions of this in the \pkg{gnm} package allows a
much wider class of models for frequency data to be fit than can be handled by
\codefun{loglm}.  Of particular importance are models for ordinal factors and for
square tables, where we can test more structured hypotheses about the patterns
of association than are provided in the tests of general assosiation under
\codefun{loglm}. These are similar in spirit to the 
non-parametric CMH tests described in \secref{sec:CMH}.

\Example
The data \code{Mental} in the \pkg{vcdExtra} package gives a two-way table in frequency form
classifying young people by
their mental health status and parents' socioeconomic status (SES), where
both of these variables are ordered factors.
<<mental1,results=verbatim>>=
str(Mental)
xtabs(Freq ~ mental+ses, data=Mental)   # display the frequency table
@
Simple ways of handling ordinal variables involve assigning scores to the table
categories, and the simplest cases are to use integer scores, either for the row variable (``column
effects'' model), the column variable (``row effects'' model), or both (``uniform association''
model).

<<mental2>>=
indep <- glm(Freq ~ mental + ses, family = poisson, data = Mental)  # independence model
@
To fit more parsimonious models than general association, we can define
numeric scores for the row and column categories
<<mental3>>=
# Use integer scores for rows/cols 
Cscore <- as.numeric(Mental$ses)
Rscore <- as.numeric(Mental$mental)	
@
Then, the row effects model, the column effects model, and the uniform association
model can be fit as follows:
<<mental4,results=verbatim>>=
# column effects model (ses)
coleff <- glm(Freq ~ mental + ses + Rscore:ses, family = poisson, data = Mental)

# row effects model (mental)
roweff <- glm(Freq ~ mental + ses + mental:Cscore, family = poisson, data = Mental)

# linear x linear association
linlin <- glm(Freq ~ mental + ses + Rscore:Cscore, family = poisson, data = Mental)
@
The \codefun{Summarize} in \pkg{vcdExtra} provides a nice, compact summary of
the fit statistics for a set of models, collected into a \class{glmlist} object.
Smaller is better for AIC and BIC.
<<mental4a,results=verbatim>>=
# compare models using AIC, BIC, etc
vcdExtra::LRstats(glmlist(indep, roweff, coleff, linlin))
@
For specific model comparisons, we can also carry out tests of \emph{nested} models with
\codefun{anova} when those models are listed from smallest to largest.
Here, there are two separate paths from the most restrictive (independence) model
through the model of uniform association, to those that allow only one of
row effects or column effects.
<<mental5,results=verbatim>>=
anova(indep, linlin, coleff, test="Chisq")	
anova(indep, linlin, roweff, test="Chisq")	
@
The model of linear by linear association seems best on all accounts.
For comparison, one might try the CMH tests on these data:

<<mental6,results=verbatim>>=
CMHtest(xtabs(Freq~ses+mental, data=Mental))
@

\subsection{Non-linear terms}
The strength of the \pkg{gnm} package is that it handles a wide variety of models
that handle non-linear terms, where the parameters enter the model beyond a simple
linear function.
The simplest example is the Goodman RC(1) model, which allows a multiplicative
term to account for the association of the table variables.
In the notation of generalized linear models with a log link, this can be expressed as
\begin{equation*}
\log \mu_{ij} = \alpha_i + \beta_j + \gamma_{i} \delta_{j}	
\end{equation*}
where the row-multiplicative effect parameters $\gamma_i$ and 
corresponding column parameters $\delta_j$ are estimated from the data.%
\footnote{
This is similar in spirit to a correspondence analysis with a single
dimension, but as a statistical model.
}
Similarly, the RC(2) model adds two multiplicative terms to
the independence model,
\begin{equation*}
\log \mu_{ij} = \alpha_i + \beta_j + \gamma_{i1} \delta_{j1} + \gamma_{i2} \delta_{j2}
\end{equation*}

In the \pkg{gnm} package, these models may be fit using the \codefun{Mult}
to specify the multiplicative term, and \codefun{instances} to specify several
such terms.

\Example
For the \code{Mental} data, we fit the RC(1) and RC(2) models, and compare
these with the independence model.

<<mental7,results=verbatim>>=
RC1 <- gnm(Freq ~ mental + ses + Mult(mental,ses), data=Mental, 
             family=poisson, , verbose=FALSE)
RC2 <- gnm(Freq ~ mental+ses + instances(Mult(mental,ses),2), data=Mental, 
             family=poisson, verbose=FALSE)
anova(indep, RC1, RC2, test="Chisq")
@

\section{Mosaic plots}\label{sec:mosaic}

Mosaic plots provide an ideal method both for visualizing contingency tables and for 
visualizing the fit--- or more importantly--- lack of fit of a \loglin\ model.
For a two-way table, \codefun{mosaic} fits a model of independence, $[A][B]$
or \verb|~A+B| as an \proglang{R} formula.
For $n$-way tables, \codefun{mosaic} can fit any \loglin\ model, and can also be
used to plot a model fit with \codefun{loglm}.
See \citet{vcd:Friendly:1994,vcd:Friendly:1999} for the statistical ideas behind these
uses of mosaic displays in connection with \loglin\ models.

The essential idea is to recursively sub-divide a unit square into rectangular ``tiles'' for the
cells of the table, such
that the are area of each tile is proportional to the cell frequency.
For a given \loglin\ model, the tiles can then be shaded in various ways to reflect
the residuals (lack of fit) for a given model.  The pattern of residuals can then
be used to suggest a better model or understand \emph{where} a given model fits or
does not fit.

\codefun{mosaic} provides a wide range of options for the directions of splitting,
the specification of shading, labeling, spacing, legend and many other details.
It is actually implemented as a special case of a more general
class of displays for $n$-way tables called \code{strucplot}, including
sieve diagrams, association plots, double-decker plots as well as mosaic
plots.  For details, see \code{help(strucplot)} and the ``See also'' links,
and also \citet{vcd:Meyer+Zeileis+Hornik:2006b}, which is available as
an \proglang{R} vignette via \code{vignette("strucplot", package="vcd")}.

\figref{fig:arthritis}, showing the association between
\code{Treatment} and \code{Improved} was produced with the following
call to \codefun{mosaic}.
<<Arthritis1,echo=TRUE,eval=FALSE>>=
mosaic(art, gp = shading_max, split_vertical = TRUE, 
       main="Arthritis: [Treatment] [Improved]")
@
Note that the residuals for the independence model were not large
(as shown in the legend),
yet the association between \code{Treatment} and \code{Improved}
is highly significant.
<<art1,results=verbatim>>=
summary(art)
@
In contrast, one of the other shading schemes, from \citet{vcd:Friendly:1994}
(use: \verb|gp = shading_Friendly|), 
uses fixed cutoffs of $\pm 2, \pm 4$,
to shade cells which are \emph{individually} significant
at approximately $\alpha = 0.05$ and $\alpha = 0.001$ levels, respectively.
The right panel below uses \verb|gp = shading_Friendly|.

\setkeys{Gin}{width=0.5\textwidth}
<<art21,fig=TRUE,height=6,width=7,echo=FALSE>>=
mosaic(art, gp = shading_max, split_vertical = TRUE, 
       main="Arthritis: gp = shading_max")
@
<<art22,fig=TRUE,height=6,width=7,echo=FALSE>>=
mosaic(art, gp = shading_Friendly, split_vertical = TRUE, 
       main="Arthritis: gp = shading_Friendly")
@

\subsection[Mosaics for loglinear models]{Mosaics for \loglin\ models}\label{sec:mosaic-llm}

When you have fit a \loglin\ model using \codefun{loglm}, 
and saved the result (as a \code{loglm} object) the simplest way to display the
results is to use the \codefun{plot} method for the \code{loglm} object.
Calling \code{mosaic(loglm.object)} has the same result.
In \secref{sec:loglm} above, we fit several different models to the
\data{HairEyeColor} data.  We can produce mosaic displays of each just
by plotting them:

<<hec-mosaic,eval=FALSE>>=
# mosaic plots, using plot.loglm() method
plot(hec.1, main="model: [Hair][Eye][Sex]")
plot(hec.2, main="model: [HairSex][EyeSex]")
plot(hec.3, main="model: [HairEye][Sex]")
@

\setkeys{Gin}{width=0.32\textwidth}
<<hec1,fig=TRUE,height=6,width=7,echo=FALSE>>=
plot(hec.1, main="model: [Hair][Eye][Sex]")
@
<<hec2,fig=TRUE,height=6,width=7,echo=FALSE>>=
plot(hec.2, main="model: [HairSex][EyeSex]")
@
<<hec3,fig=TRUE,height=6,width=7,echo=FALSE>>=
plot(hec.3, main="model: [HairSex][EyeSex]")
@

Alternatively, you can supply the model formula to \codefun{mosaic}
with the \code{expected} argument.  This is passed to \codefun{loglm},
which fits the model, and returns residuals used for shading in the plot. 

For example, here we examine the \data{TV2} constructed in \secref{sec:complex}
above.  The goal is to see how Network choice depends on (varies with)
Day and Time. To do this: 
\begin{itemize}
	\item We fit a model of joint independence of
\code{Network} on the combinations of \code{Day} and \code{Time},
with the model formula \verb|~Day:Time + Network|.
  \item To make the display more easily read, we place \code{Day} and \code{Time}
on the vertical axis and \code{Network} on the horizontal,
  \item The \code{Time} values overlap on the right vertical axis, so we use
  \codefun{level} to abbreviate them.  \codefun{mosaic} also supports a 
  more sophisticated set of labeling functions.  Instead of changing the data
  table, we could have used 
  \verb|labeling_args = list(abbreviate = c(Time = 2))| for a similar effect.
\end{itemize}

The following call to \codefun{mosaic} produces \figref{fig:TV-mosaic}:
<<TV-mosaic0,eval=FALSE>>=
dimnames(TV2)$Time <- c("8", "9", "10")     # re-level for mosaic display
mosaic(~ Day + Network + Time, data=TV2, expected=~Day:Time + Network, 
         legend=FALSE, gp=shading_Friendly)
@

\setkeys{Gin}{width=0.75\textwidth}
\begin{figure}[htb]
\begin{center}
<<TV-mosaic,fig=TRUE,height=6,width=6,echo=FALSE>>=
dimnames(TV2)$Time <- c("8", "9", "10")     # re-level for mosaic display
mosaic(~ Day + Network + Time, data=TV2, expected=~Day:Time + Network, 
         legend=FALSE, gp=shading_Friendly)
@
\caption{Mosaic plot for the \data{TV} data
	showing model of joint independence, \texttt{Day:Time + Network} .}
\label{fig:TV-mosaic}
\end{center}
\end{figure}

From this, it is easy to read from the display how network choice varies with day and
time. For example, CBS dominates in all time slots on Monday;
ABC and NBC dominate on Tuesday, particularly in the later time slots;
Thursday is an NBC day, while on Friday, ABC gets the greatest share.

In interpreting this mosaic and other plots, it is important to understand that
associations included in the model---here, that between day and time---are \emph{not}
shown in the shading of the cells, because they have been fitted (taken into account)
in the \loglin\ model. 

For comparison, you might want to try fitting the model of homogeneous association.
This allows all pairs of factors to be associated, but asserts that each pairwise
association is the same across the levels of the remaining factor.
The resulting plot displays the contributions to a 3-way association, but is not shown here.
<<TV-mosaic1,eval=FALSE>>=
mosaic(~ Day + Network + Time, data=TV2, 
         expected=~Day:Time + Day:Network + Time:Network, 
         legend=FALSE, gp=shading_Friendly)
@
\subsection[Mosaics for glm() and gnm() models]{Mosaics for \codefun{glm} and \codefun{gnm} models}\label{sec:mosglm}
The \pkg{vcdExtra} package provides an additional method, \codefun{mosaic.glm} 
for models fit with \codefun{glm} and \codefun{gnm}.%
\footnote{
Models fit with \codefun{gnm} are of \code{class = c("gnm", "glm", "lm")},
so all \code{*.glm} methods apply, unless overridden in the \pkg{gnm} package.
}
These are not restricted to the 
Poisson family, but only apply to cases where the response variable is non-negative. 

\Example
Here, we plot the independence and the linear-by-linear association model
for the Mental health data from \secref{sec:glm}.
These examples illustrate some of the options for labeling (variable names and
residuals printed in cells).  Note that the \code{formula} supplied to \codefun{mosaic}
for \class{glm} objects refers to the order of factors displayed in the plot, not the model.
<<mental-plots,eval=FALSE>>=
long.labels <- list(set_varnames = c(mental="Mental Health Status", ses="Parent SES"))
mosaic(indep, ~ses+mental, residuals_type="rstandard",  
     labeling_args = long.labels, labeling=labeling_residuals,
     main="Mental health data: Independence")

mosaic(linlin, ~ses+mental, residuals_type="rstandard", 
     labeling_args = long.labels, labeling=labeling_residuals, suppress=1, 
     gp=shading_Friendly, main="Mental health data: Linear x Linear")
@

\setkeys{Gin}{width=0.49\textwidth}
<<mental-plots1,fig=TRUE,echo=FALSE,height=6,width=6>>=
long.labels <- list(set_varnames = c(mental="Mental Health Status", ses="Parent SES"))
mosaic(indep, ~ses+mental, residuals_type="rstandard",  
     labeling_args = long.labels, labeling=labeling_residuals,
     main="Mental health data: Independence")
@
<<mental-plots2,fig=TRUE,echo=FALSE,height=6,width=6>>=
long.labels <- list(set_varnames = c(mental="Mental Health Status", ses="Parent SES"))
mosaic(linlin, ~ses+mental, residuals_type="rstandard", 
     labeling_args = long.labels, labeling=labeling_residuals, suppress=1, 
     gp=shading_Friendly, main="Mental health data: Linear x Linear")
@

The \pkg{gnm} package also fits a wide variety of models with nonlinear terms or terms for
structured associations of table variables.  In the following, we fit the RC(1)
model
\begin{equation*}
	\log ( m_{ij} ) = \mu + \lambda_i^A + \lambda_j^B + \phi \mu_i \nu_j 
\end{equation*}
This is similar to the linear by linear model, except that the row effect
parameters ($\mu_i$) and column parameters ($\nu_j$) are estimated from the data
rather than given assigned equally-spaced values.  The multiplicative terms
are specified by the \codefun{Mult}.

<<mental-RC1, eval=FALSE>>=
Mental$mental <- C(Mental$mental, treatment)
Mental$ses <- C(Mental$ses, treatment)
RC1model <- gnm(Freq ~ mental + ses + Mult(mental, ses),
                family = poisson, data = Mental)
mosaic(RC1model, residuals_type="rstandard", labeling_args = long.labels, 
       labeling=labeling_residuals, suppress=1, gp=shading_Friendly,
       main="Mental health data: RC(1) model")
@
Other forms of nonlinear terms are provided for the inverse of a predictor
(\codefun{Inv}) and the exponential of a predictor (\codefun{Exp}).
You should read \code{vignette("gnmOverview", package="gnm")} for further details.

\subsection{Mosaic tips and techniques}\label{sec:tips}

The \pkg{vcd} package implements an extremely general collection of graphical methods for
$n$-way frequency tables within the strucplot framework, which includes mosaic plots
(\codefun{mosaic}), as well as association plots (\codefun{assoc}), sieve diagrams
(\codefun{sieve}), as well as tabular displays (\codefun{structable}).

The graphical methods in \pkg{vcd}  support a wide of options that control almost all of the details
of the plots, but it is often difficult to determine what arguments you need to supply
to achieve a given effect from the \code{help()}.  As a first step, you should read the 
\code{vignette("strucplot")} in \pkg{vcd} to understand the overall structure of
these plot methods.  The notes below describe a few useful things that may not be obvious,
or can be done in different ways.

\subsubsection[Changing labels]{Changing the labels for variables and levels}
With data in contingency table form or as a frequency data frame, it often happens
that the variable names and/or the level values of the factors, while suitable
for analysis, are less than adequate when used in mosaic plots and other strucplot
displays.  

For example, we might prefer that a variable named \code{ses} appear
as \code{"Socioeconomic Status"}, or a factor with levels \code{c("M", "F")}
be labeled using \code{c("Male", "Female")} in a plot.  Or, sometimes we
start with a factor whose levels are fully spelled out
(e.g., \code{c("strongly disagree", "disagree", "neutral", "agree", "strongly agree")}),
only to find that the level labels overlap in graphic displays.

The structplot framework in \pkg{vcd} provides an extremely large variety of
functions and options for controlling almost all details of text labels in mosaics
and other plots.  See \code{help(labelings)} for an overview. 

For example, in \secref{sec:ordered-factors} we showed how to rearrange the dimensions
of the \code{UCBAdmissions} table, change the names of the table variables, and
relabel the levels of one of the table variables.
The code below changes the actual table for plotting purposes, but we pointed out that
these changes can create other problems in analysis.

<<reorder3a,results=hide>>=
UCB <- aperm(UCBAdmissions, c(2, 1, 3))
names(dimnames(UCB)) <- c("Sex", "Admit?", "Department")
dimnames(UCB)[[2]] <- c("Yes", "No")
@

The same effects can be achieved \emph{without} modifying the data
using the \verb|set_varnames| and
\verb|set_labels| options in \codefun{mosaic} as follows:

<<reorder3b,results=hide, keep.source=TRUE>>=
vnames <- list(set_varnames = c(Admit="Admission", Gender="Sex", Dept="Department"))

lnames <- list(Admit = c("Yes", "No"),
              Gender = c("Males", "Females"),
              Dept = LETTERS[1:6])

mosaic(UCBAdmissions, labeling_args=vnames, set_labels=lnames)
@

In some cases, it may be sufficient to abbreviate (or clip, or rotate) level names to avoid 
overlap.  For example, the statements below produce another version of
\figref{fig:TV-mosaic} with days of the week abbreviated to their first three letters.
Section 4 in the \code{vignette("strucplot")} provides many other examples.

<<TV-mosaic2,results=hide>>=
dimnames(TV2)$Time <- c("8", "9", "10")     # re-level for mosaic display
mosaic(~ Day + Network + Time, data=TV2, expected=~Day:Time + Network, 
         legend=FALSE, gp=shading_Friendly, 
         labeling_args=list(abbreviate=c(Day=3)) )
@

%\subsubsection{Fitting complex models with glm() and gnm()}



\section[Continuous predictors]{Continuous predictors}\label{sec:contin}
When continuous predictors are available---and potentially important--- in explaining a
categorical outcome, models for that outcome include:
logistic regression (binary response), 
the proportional odds model (ordered polytomous response),
multinomial (generalized) logistic regression.
Many of these are special cases of the generalized linear model using the
\code{"poisson"} or \code{"binomial"} family and their relatives.

\subsection{Spine and conditional density plots}\label{sec:spine}
I don't go into fitting such models here, but I would be remiss not to illustrate some
visualizations in \pkg{vcd} that are helpful here.
The first of these is the spine plot or spinogram \citep{vcd:Hummel:1996}
 (produced with \codefun{spine}).
These are special cases of mosaic plots with 
specific spacing and shading to show how a categorical response varies with
a continuous or categorical predictor.

They are also a generalization of stacked bar plots where not the heights  but
the \emph{widths} of the bars corresponds to the relative frequencies of \code{x}. The heights
of the  bars then  correspond to  the conditional  relative frequencies  of {y} in
every \code{x} group.

\Example
For the \data{Arthritis} data, we can see how \code{Improved} varies with \code{Age}
as follows.  \codefun{spine} takes a formula of the form 
\verb|y ~ x| with a single dependent factor and a single explanatory variable \code{x}
(a numeric variable or a factor).
The range of a numeric variable\code{x} is divided into intervals based on the
\code{breaks} argument, and stacked bars are drawn to show the distribution of
\code{y} as \code{x} varies.  As shown below, the discrete table that is visualized
is returned by the function.

<<spine1,fig=FALSE,results=verbatim>>=
(spine(Improved ~ Age, data = Arthritis, breaks = 3))
(spine(Improved ~ Age, data = Arthritis, breaks = "Scott"))
@

\setkeys{Gin}{width=0.49\textwidth}
<<spine2,fig=TRUE,echo=FALSE,height=6,width=6>>=
(spine(Improved ~ Age, data = Arthritis, breaks = 3))
@
<<spine3,fig=TRUE,echo=FALSE,height=6,width=6>>=
(spine(Improved ~ Age, data = Arthritis, breaks = "Scott"))
@


The conditional density plot \citep{vcd:Hofmann+Theus} is a further generalization.
This visualization technique is similar to spinograms, but uses a smoothing approach
rather than discretizing the explanatory variable.  As well, it uses 
the original \code{x} axis and not a distorted one.

\setkeys{Gin}{width=0.6\textwidth}
\begin{figure}[htb]
\begin{center}
<<cdplot,fig=TRUE,height=6,width=6>>=
cdplot(Improved ~ Age, data = Arthritis)
with(Arthritis, rug(jitter(Age), col="white", quiet=TRUE))
@
\caption{Conditional density plot for the \data{Arthritis} data
	showing the variation of Improved with Age.}
\label{fig:cd-plot}
\end{center}
\end{figure}

In such plots, it is useful to also see the distribution of the observations
across the horizontal axis, e.g., with a \codefun{rug} plot.
\figref{fig:cd-plot} uses \codefun{cdplot} from the \pkg{graphics} package
rather than \verb|cd_plot()| from \pkg{vcd}, and is produced with
<<cdplot1,echo=TRUE>>=
cdplot(Improved ~ Age, data = Arthritis)
with(Arthritis, rug(jitter(Age), col="white", quiet=TRUE))
@
From \figref{fig:cd-plot} it can be easily seen that the proportion
of patients reporting Some or Marked improvement increases with Age,
but there are some peculiar bumps in the distribution.
These may be real or artifactual, but they would be hard to see
with most other visualization methods.
When we switch from non-parametric data exploration to parametric
statistical models, such effects are easily missed.

\subsection[Model-based plots]{Model-based plots: effect plots and \pkg{ggplot2} plots}\label{sec:modelplots}
The nonparametric conditional density plot uses smoothing methods
to convey the distributions of the response variable,
but displays that are simpler to interpret can often be obtained by plotting
the predicted response from a parametric model.

For complex \codefun{glm} models with interaction effects, the \pkg{effects}
package  provides the most useful displays,
plotting  the predicted values for a given term, averaging over other
predictors not included in that term.  I don't illustrate this here, but
see \citet{effects:1,effects:2} and \code{help(package="effects")}.

Here I just briefly illustrate the capabilities of the \pkg{ggplot2} package
for model-smoothed plots of categorical responses in \codefun{glm} models.

\Example
The \data{Donner} data frame in \pkg{vcdExtra} gives details on the survival
of 90 members of the Donner party,
a group of people who attempted to migrate to California in 1846.
They were trapped by an early blizzard on the eastern side of the
Sierra Nevada mountains, and before they could be rescued, 
nearly half of the party had died.
What factors affected who lived and who died?

<<donner1, results=verbatim>>=
data(Donner, package="vcdExtra")
str(Donner)
@
A potential model of interest
is the logistic regression model for $Pr(survived)$, allowing separate
fits for males and females as a function of \code{age}.
The key to this is the \verb|stat_smooth()| function, using
	\code{method = "glm", method.args = list(family = binomial)}. The \verb|formula = y ~ x|
	specifies a linear fit on the logit scale (\figref{fig:donner3}, left)

<<donner2a, fig=FALSE, eval=FALSE>>=
# separate linear fits on age for M/F
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", method.args = list(family = binomial), formula = y ~ x,
           alpha = 0.2, size=2, aes(fill = sex))
@
Alternatively, we can allow a quadratic relation with \code{age}
by specifying \verb|formula = y ~ poly(x,2)| (\figref{fig:donner3}, right).
<<donner2b, fig=FALSE, eval=FALSE>>=
# separate quadratics
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", method.args = list(family = binomial), formula = y ~ poly(x,2),
           alpha = 0.2, size=2, aes(fill = sex))
@

\setkeys{Gin}{width=0.49\textwidth}
\begin{figure}[htb]
\begin{center}
<<donner3a,fig=TRUE,echo=FALSE,height=6,width=6, results=hide>>=
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", method.args = list(family = binomial), formula = y ~ x,
           alpha = 0.2, size=2, aes(fill = sex))

@
<<donner3b,fig=TRUE,echo=FALSE,height=6,width=6, results=hide>>=
# separate quadratics
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", method.args = list(family = binomial), formula = y ~ poly(x,2),
           alpha = 0.2, size=2, aes(fill = sex))
@
\caption{Logistic regression plots for the  \data{Donner} data
	showing survival vs. age, by sex. Left: linear logistic model; right: quadratic model}
\label{fig:donner3}
\end{center}
\end{figure}
These plots very nicely show (a) the fitted $Pr(survived)$ for males and females;
(b) confidence bands around the smoothed model fits and (c) the individual
observations by jittered points at 0 and 1 for those who died and survided, respectively.

\bibliography{vcd,vcdExtra}

\end{document}

---
title: "Continuous predictors"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Continuous predictors}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/tut05-",
  fig.align = "center",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# Old Sweave options
# \SweaveOpts{engine=R,eps=TRUE,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{engine=R,height=6,width=7,results=hide,fig=FALSE,echo=TRUE}
# \SweaveOpts{prefix.string=fig/vcd-tut,eps=FALSE}
# \SweaveOpts{keep.source=TRUE}


# preload datasets ???
set.seed(1071)
library(vcd)
library(vcdExtra)
library(ggplot2)
data(Arthritis, package="vcd")
art <- xtabs(~Treatment + Improved, data = Arthritis)
if(!file.exists("fig")) dir.create("fig")

```



When continuous predictors are available---and potentially important---in explaining a
categorical outcome, models for that outcome include:
logistic regression (binary response), 
the proportional odds model (ordered polytomous response),
multinomial (generalized) logistic regression.
Many of these are special cases of the generalized linear model using the
`"poisson"` or `"binomial"` family and their relatives.

## Spine and conditional density plots {#sec:spine}
I don't go into fitting such models here, but I would be remiss not to illustrate some
visualizations in `vcd` that are helpful here.
The first of these is the spine plot or spinogram [@vcd:Hummel:1996], produced with `spine()`.
These are special cases of mosaic plots with 
specific spacing and shading to show how a categorical response varies with
a continuous or categorical predictor.

They are also a generalization of stacked bar plots where not the heights  but
the *widths* of the bars corresponds to the relative frequencies of `x`. The heights
of the  bars then  correspond to  the conditional  relative frequencies  of `y` in
every `x` group.

***Example***:
For the `Arthritis` data, we can see how `Improved` varies with `Age`
as follows.  `spine()` takes a formula of the form 
`y ~ x` with a single dependent factor and a single explanatory variable `x`
(a numeric variable or a factor).
The range of a numeric variable`x` is divided into intervals based on the
`breaks` argument, and stacked bars are drawn to show the distribution of
`y` as `x` varies.  As shown below, the discrete table that is visualized
is returned by the function.

```{r, spine1}
#| spine1,
#| fig.height = 6,
#| fig.width = 6,
#| fig.show = "hold",
#| out.width = "46%",
#| fig.align = "center",
#| cap = "Spine plots for the `Arthritis` data"
(spine(Improved ~ Age, data = Arthritis, breaks = 3))
(spine(Improved ~ Age, data = Arthritis, breaks = "Scott"))
```


The conditional density plot [@vcd:Hofmann+Theus] is a further generalization.
This visualization technique is similar to spinograms, but uses a smoothing approach
rather than discretizing the explanatory variable.  As well, it uses 
the original `x` axis and not a distorted one.


```{r}
#| cdplot,
#| fig.height = 5,
#| fig.width = 5,
#| cap = "Conditional density plot for the `Arthritis` data showing the variation of Improved with Age."
cdplot(Improved ~ Age, data = Arthritis)
```



In such plots, it is useful to also see the distribution of the observations
across the horizontal axis, e.g., with a `rug()` plot.
\@ref{fig:cd-plot} uses `cdplot()` from the `graphics` package
rather than `cd_plot()` from `vcd`, and is produced with

```{r}
#| cdplot1,
#| fig.height = 5,
#| fig.width = 5,
cdplot(Improved ~ Age, data = Arthritis)
with(Arthritis, rug(jitter(Age), col="white", quiet=TRUE))
```

From this figure it can be easily seen that the proportion
of patients reporting Some or Marked improvement increases with Age,
but there are some peculiar bumps in the distribution.
These may be real or artifactual, but they would be hard to see
with most other visualization methods.
When we switch from non-parametric data exploration to parametric
statistical models, such effects are easily missed.

## Model-based plots: effect plots and `ggplot2 plots` {#sec:modelplots}

The nonparametric conditional density plot uses smoothing methods
to convey the distributions of the response variable,
but displays that are simpler to interpret can often be obtained by plotting
the predicted response from a parametric model.

For complex `glm()` models with interaction effects, the `effects`
package  provides the most useful displays,
plotting  the predicted values for a given term, averaging over other
predictors not included in that term.  I don't illustrate this here, but
see @effects:1,@effects:2 and `help(package="effects")`.

Here I just briefly illustrate the capabilities of the `ggplot2` package
for model-smoothed plots of categorical responses in `glm()` models.

***Example***:
The `Donner` data frame in `vcdExtra` gives details on the survival
of 90 members of the Donner party,
a group of people who attempted to migrate to California in 1846.
They were trapped by an early blizzard on the eastern side of the
Sierra Nevada mountains, and before they could be rescued, 
nearly half of the party had died.
What factors affected who lived and who died?

```{r, donner1}
data(Donner, package="vcdExtra")
str(Donner)
```

A potential model of interest
is the logistic regression model for $Pr(survived)$, allowing separate
fits for males and females as a function of `age`.
The key to this is the `stat_smooth()` function, using
`method = "glm", method.args = list(family = binomial)`. The `formula = y ~ x`
specifies a linear fit on the logit scale (\@ref{fig:donner3}, left)

```{r, donner2a, fig=FALSE, eval=FALSE}
# separate linear fits on age for M/F
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ x,
              alpha = 0.2, size=2, aes(fill = sex))
```

Alternatively, we can allow a quadratic relation with `age`
by specifying `formula = y ~ poly(x,2)` (@ref(fig:donner3), right).

```{r, donner2b, fig=FALSE, eval=FALSE}
# separate quadratics
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ poly(x,2), 
              alpha = 0.2, size=2, aes(fill = sex))
```


```{r}
#| donner3a,
#| echo = FALSE,
#| fig.height = 6,
#| fig.width = 6,
#| fig.show = "hold",
#| out.width = "46%",
#| cap = "Logistic regression plots for the  `Donner` data	showing survival vs. age, by sex. Left: linear logistic model; right: quadratic model {#fig:donner3}"
# separate linear fits on age for M/F
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ x,
              alpha = 0.2, size=2, aes(fill = sex))

# separate quadratics
ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm", 
              method.args = list(family = binomial), 
              formula = y ~ poly(x,2),
              alpha = 0.2, size=2, aes(fill = sex))
```


These plots very nicely show (a) the fitted $Pr(survived)$ for males and females;
(b) confidence bands around the smoothed model fits and (c) the individual
observations by jittered points at 0 and 1 for those who died and survived, respectively.

# References

---
title: "Demo - Housing Data"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: vcdExtra
output: 
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["vcd.bib", "vcdExtra.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Demo - Housing Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/demo-housing-",
  dev = "png",
  comment = "##"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# colorize text
colorize <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}
```

This vignette was one of a series of `demo()` files in the package. It is still there
as `demo("housing")`, but is now presented here with additional commentary and analysis,
designed to highlight some aspects of analysis of categorical data and graphical display.

## Load packages
I'll use the following packages in this vignette.

```{r}
library(vcdExtra)
library(MASS)
library(effects)
```


## Housing data

The content here is the dataset `MASS::housing`, giving a 4-way,
$3 \times 3 \times 4 \times 2$
frequency table of 1681 individuals from the *Copenhagen Housing Conditions
Survey*, classified by their:

* Satisfaction (`Sat`) with their housing circumstances (low, medium or high),
* `Type` of rental dwelling (Tower, Apartment, Atrium or Terrace) 
* perceived influence (`Infl`) on management of the property (low, medium, high), and 
* degree of contact (`Cont`) with other residents (low or high)

Load the data:
```{r housing}
data(housing, package="MASS")
str(housing)
```


### Variables, levels and models

Satisfaction (`Sat`) of these householders with their present
housing circumstances is the **outcome variable** here. 

For purposes of analysis,
note that `Sat` is an ordered factor with levels `"Low" < "Medium" < "High"`.

Note also that Influence, with the same levels is just a "Factor", not an
ordered one.

I consider here just models using `glm(..., family=poisson)` or
the equivalent in `MASS::loglm()`. 

The ordering of factor levels is important in graphical displays. We
don't want to see them ordered alphabetically, "High", "Low", "Medium".
The `housing` data.frame was constructed so that the levels of
`Sat` and `Infl` appear in the dataset in their appropriate order.

```{r}
levels(housing$Sat)
levels(housing$Infl)
```



Other models, e.g., the **proportional odds** model, fit using
`MASS:polr()` can take the ordinal nature of satisfaction into
account. In `glm()` one could re-assign `Infl` as an ordered
factor and examine linear vs. non-linear associations for this factor.
But I don't do this here.


## Null model

The most ignorant model asserts that all the table factors are mutually independent.
In symbolic notation, this is `[S] [I] [T] [C]` where all terms in separate `[ ]` are
supposed to be independent. This is `Freq ~ Sat + Infl + Type + Cont` as a formula for
`glm()`.

```{r house.null}
house.null <- glm(Freq ~ Sat + Infl + Type + Cont, family = poisson,
                  data = housing)

```


## Baseline model

When `Sat` is the outcome variable,  a minimal **baseline model** should allow
for all associations among the predictors, symbolized as `[S] [I T C]`.
That is,
Influence, Type and Contact may be associated in arbitrary ways, just
as multiple predictors can be correlated in regression models.

In this framework, what remains to be explained is whether/how `Sat`
depends on the combinations of the other variables. The baseline
model therefore includes the full three-way term for the predictors.
```{r house.glm0}
house.glm0 <- glm(Freq ~ Sat + Infl*Type*Cont, family = poisson,
                  data = housing)
```

Both of these models fit terribly, but we can always use `anova(mod1, mod2,...)` to
compare the *relative* fits of **nested** models.

```{r anova}
anova(house.null, house.glm0, test = "Chisq")
```

## Visualising model fit
The baseline model is shown in the mosaic plot below. Note that
this is applied not to the `housing` data, but rather to the 
`house.glm0` object (of class `glm`) 
resulting to a call to `vcdExtra::mosaic.glm()`.

With four variables in the mosaic, labeling of the variable names
and factor levels is a bit tricky, because labels must appear on all four
sides of the plot.
The `labeling_args` argument can be used to set more informative variable
names and abbreviate factor levels where necessary.

```{r}
#| label= mosaic-glm0a,
#| warning = TRUE
# labeling_args for mosaic()
largs <- list(set_varnames = c(
      Infl="Influence on management", 
			Cont="Contact among residents", 
			Type="Type of dwelling", 
			Sat="Satisfaction"),
	abbreviate=c(Type=3))

mosaic(house.glm0, 
       labeling_args=largs, 
       main='Baseline model: [ITC][Sat]')
```

In this plot we can see largish 
`r colorize("positive residuals", "blue")`
in the blocks corresponding
to (low satisfaction, low influence) and (high satisfaction, high influence)
and clusters of largish 
`r colorize("negative residuals", "red")`
in the opposite corners.

By default, variables are used in the mosaic display in their order in the data
table or frequency data.frame. The `r colorize("warning", "red")` reminds us that the
order of conditioning used is `~Sat + Infl + Type + Cont`.

### Ordering the variables in the mosaic

For `mosaic.glm()`, the conditioning order of variables in the mosaic can be set using
the `formula` argument.  Here, I rearrange the variables to put `Sat`
as the last variable in the splitting / conditioning sequence.
I also use `vcdExtra::modFit()` to add the LR $G^2$ fit statistic to
the plot title.

```{r mosaic-glm0b}
mosaic(house.glm0, 
       formula = ~ Type + Infl + Cont + Sat, 
       labeling_args=largs, 
       main=paste('Baseline model: [ITC][Sat],', modFit(house.glm0))
  )
```

## Adding association terms

Clearly, satisfaction depends on one or more of the predictors, `Infl`, `Type` and `Cont`
and possibly their interactions.

As a first step it is useful to consider sequentially adding the association terms
`Infl:Sat`, `Type:Sat`, `Cont:Sat` one at a time.
This analysis is carried out using `MASS::addterm()`.

```{r addterm}
MASS::addterm(house.glm0, 
              ~ . + Sat:(Infl + Type + Cont), 
              test = "Chisq")
```

Based on this, it is useful to consider a "main-effects" model for satisfaction,
adding all three two-way terms involving satisfaction.

The `update()` method provides an easy way to add (or subtract) terms from a fitted
model object. In the model formula, `.` stands for whatever was on the left
side (`Freq`) or on the right side (`Sat + Infl*Type*Cont`) of the model
(`house.glm0`) that is being updated.

```{r house-glm1}
house.glm1 <- update(house.glm0, 
                     . ~ . + Sat*(Infl + Type + Cont))

```

For comparison, we note that the same model can be fit using the
iterative proportional scaling algorithm of `MASS::loglm()`.

```{r house-loglm1}
(house.loglm1 <- MASS::loglm(Freq ~ Infl * Type * Cont + 
                              Sat*(Infl + Type + Cont), data = housing))

```


## Did the model get better? 

As before, `anova()` tests the added contribution of
each more complex model over the one before.
The residual deviance $G^2$ has been reduced from 
$G^2 (46) = 217.46$ for the baseline model `house.glm0` to
$G^2 (34) = 38.66$ for the revised model `house.glm1`.
The difference, $G^2(M1 | M0) = G^2 (12) = 178.79$ tests the collective additional fit
provided by the two-way association of satisfaction with the predictors.

```{r}
anova(house.glm0, house.glm1, test="Chisq")
```


## Visualize model `glm1`
The model `house.glm1` fits reasonably well, `r modFit(house.glm1)`, so most residuals
are small. In the mosaic below, I use `gp=shading_Friendly` to shade the tiles
so that positive and negative residuals are distinguished by color,
and they are filled when the absolute value of the residual is outside $\pm 2, 4$.

```{r mosaic-glm1}
mosaic(house.glm1, 
       labeling_args=largs, 
       main=paste('Model [IS][TS][CS],', modFit(house.glm1) ), 
       gp=shading_Friendly)
```

One cell is highlighted here: The combination of medium influence, low contact and tower type,
is more likely to give low satisfaction than the model predicts.
Is this just an outlier, or is there something that can be interpreted and perhaps
improve the model fit?
It is hard tell, but the virtues of mosaic displays are that they help to:

* diagnose overall patterns of associations,
* spot unusual cells in relation to lack of fit of a given model.



## Can we drop any terms?

When we add terms using `MASS::addterm()`, they are added sequentially.
It might be the case that once some term is added, a previously added term
is no longer important. Running `MASS::dropterm()` on the `housel.glm1` model
checks for this.

```{r dropterm}
MASS::dropterm(house.glm1, test = "Chisq")

```

Note that the three-way term `Infl:Type:Cont` is not significant.  However,
with `Sat` as the response, the associations of all predictors must be included
in the model.

## What about two-way interactions?

The model so far says that each of influence, type and control have separate,
additive effects on the level of satisfaction, what I called a "main-effects"
model.  It might be the case that some of the predictors have *interaction*
effects, e.g., that the effect of influence on satisfaction might vary with the 
type of dwelling or the level of control.

An easy way to test for these is to update the main-effects model, adding
all possible two-way interactions for `Sat`, one at a time, with `addterm()`.

```{r addterm1}
MASS::addterm(house.glm1,
               ~. + Sat:(Infl + Type + Cont)^2, 
              test  =  "Chisq")
```

The result shows that adding the term `Infl:Type:Sat`
reduces the deviance $G^2$ from 38.66 to 16.11.
The difference, $G^2(M1 + ITS | M1) = G^2 (12) = 22.55$
reflects a substantial improvement.
The remaining two-way interaction terms reduce the deviance
by smaller and non-significant amounts, relative to `house.glm1`.

Model fitting should be guided by substance, not just statistical
machinery. Nonetheless, it seems arguably sensible to add one
two-way term to the model, giving `house.glm2`.

```{r}
house.glm2 <- update(house.glm1,
                     . ~ . + Sat:Infl:Type)

```

## Model parsimony: AIC & BIC

Adding more association terms to a model will always improve it.
The question is, whether that is "worth it"?

"Worth it" concerns the trade-off between model fit and parsimony.
Sometimes we might prefer a model with fewer parameters to one
that has a slightly better fit, but requires more model terms
and parameters.  

The AIC and BIC statistics are designed to 
adjust our assessment of model fit by penalizing it for using more
parameters. Equivalently, they deduct from the 
likelihood ratio $G^2$ a term proportional to the residual $\text{df}$ of the model.
In any case -- **smaller is better** for both AIC and BIC.

$$AIC = G^2 - 2 \: \text{df}$$
$$BIC = G^2 - \log(n) \: \text{df}$$


These measures are provided 
by `AIC()`, `BIC()`, and can be used to compare models using
`vcdExtra::LRstats()`. 

```{r lrstats}
LRstats(house.glm0, house.glm1, house.glm2)
```

By these metrics, model `house.glm1` is best on both AIC and BIC.
The increased goodness-of-fit (smaller $G^2$) of model `house.glm2`
is not worth the extra cost of parameters in the `house.glm2` model.

<!-- ## Effect plots -->

<!-- ```{r} -->
<!-- house.eff <-allEffects(house.glm1) -->

<!-- ``` -->

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Glass}
\alias{Glass}
\title{British Social Mobility from Glass(1954)}
\format{
A frequency data frame with 25 observations on the following 3
variables representing a 5 x 5 table with 3500 cases.
\describe{
\item{\code{father}}{a factor with levels \code{Managerial} \code{Professional} \code{Skilled} \code{Supervisory} \code{Unskilled}}
\item{\code{son}}{a factor with levels \code{Managerial} \code{Professional} \code{Skilled} \code{Supervisory} \code{Unskilled}}
\item{\code{Freq}}{a numeric vector}
}
}
\source{
Glass, D. V. (1954), \emph{Social Mobility in Britain}. The Free
Press.
}
\description{
Glass(1954) gave this 5 x 5 table on the occupations of 3500 British fathers
and their sons.
}
\details{
The occupational categories in order of status are: (1) Professional & High
Administrative (2) Managerial, Executive & High Supervisory (3) Low
Inspectional & Supervisory (4) Routine Nonmanual & Skilled Manual (5) Semi-
& Unskilled Manual

However, to make the point that factors are ordered alphabetically by
default, Friendly & Meyer (2016) introduce this data set in the form given
here.
}
\examples{

data(Glass)
glass.tab <- xtabs(Freq ~ father + son, data=Glass)

largs <- list(set_varnames=list(father="Father's Occupation",
                                son="Son's Occupation"),
              abbreviate=10)
gargs <- list(interpolate=c(1,2,4,8))

mosaic(glass.tab,
  shade=TRUE,
  labeling_args=largs,
  gp_args=gargs,
  main="Alphabetic order",
  legend=FALSE,
  rot_labels=c(20,90,0,70))

# reorder by status
ord <- c(2, 1, 4, 3, 5)
mosaic(glass.tab[ord, ord],
  shade=TRUE,
  labeling_args=largs,
  gp_args=gargs,
  main="Effect order",
  legend=FALSE,
  rot_labels=c(20,90,0,70))


}
\references{
Bishop, Y. M. M. and Fienberg, S. E. and Holland, P. W. (1975).
\emph{Discrete Multivariate Analysis: Theory and Practice}, MIT Press.

Friendly, M. and Meyer, D. (2016).  \emph{Discrete Data Analysis with R:
Visualization and Modeling Techniques for Categorical and Count Data}.  Boca
Raton, FL: Chapman & Hall/CRC. \url{http://ddar.datavis.ca}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{PhdPubs}
\alias{PhdPubs}
\title{Publications of PhD Candidates}
\format{
A data frame with 915 observations on the following 6 variables.
\describe{
\item{\code{articles}}{number of articles published in the final three years of PhD studies}
\item{\code{female}}{dummy variable for gender, coded \code{1} for female}
\item{\code{married}}{dummy variable for marital status, coded \code{1} for married}
\item{\code{kid5}}{number of young children, age 5 and under}
\item{\code{phdprestige}}{prestige of the PhD department.  The higher the number the more prestigious the program.}
\item{\code{mentor}}{number of publications by the mentor in the preceeding three years}
}
}
\source{
Long, J. S. (1997). \emph{Regression Models for Categorical and
Limited Dependent Variables}, Sage.

Long, J. S. & Freese, J. (2006). \emph{Regression Models for Categorical
Dependent Variables Using Stata}, 2nd Ed., Stata Press.
}
\description{
A data set giving the number of publications by doctoral candidates in
biochemistry in relation to various predictors, originally from Long (1997).
}
\details{
There is a large number of zero counts. Is there evidence for a separate
group of non-publishers?

In this version of the data set, \code{phdprestige} had been rounded to the
nearest integer. A Stata version with the continuous values was subsequently
found at \url{https://www.stata-press.com/data/lf2/couart2.dta}
}
\examples{

data(PhdPubs)
# very uninformative
hist(PhdPubs$articles,
     breaks=0:19, col="pink", xlim=c(0,20),
     xlab="Number of Articles")

library(vcd)
rootogram(goodfit(PhdPubs$articles), xlab="Number of Articles")

# compare with negative binomial
rootogram(goodfit(PhdPubs$articles, type="nbinomial"),
	xlab="Number of Articles", main="Negative binomial")



}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expand.dft.R
\name{expand.dft}
\alias{expand.dft}
\alias{expand.table}
\title{Expand a frequency table to case form}
\usage{
expand.dft(x, var.names = NULL, freq = "Freq", ...)
}
\arguments{
\item{x}{A table object, or a data frame in frequency form containing
factors and one numeric variable representing the cell frequency for that
combination of factors.}

\item{var.names}{A list of variable names for the factors, if you wish to
override those already in the table}

\item{freq}{The name of the frequency variable in the table}

\item{\dots}{Other arguments passed down to \code{type.convert}.  In
particular, pay attention to \code{na.strings} (default:
\code{na.strings=NA} if there are missing cells) and \code{as.is} (default:
\code{as.is=FALSE}, converting character vectors to factors).}
}
\value{
A data frame containing the factors in the table and as many
observations as are represented by the total of the \code{freq} variable.
}
\description{
Converts a frequency table, given either as a table object or a data frame
in frequency form to a data frame representing individual observations in
the table.
}
\details{
\code{expand.table} is a synonym for \code{expand.dft}.
}
\examples{

library(vcd)
art <- xtabs(~Treatment + Improved, data = Arthritis)
art
artdf <- expand.dft(art)
str(artdf)

# 1D case
(tab <- table(sample(head(letters), 20, replace=TRUE)))
expand.table(tab, var.names="letter")


}
\references{
Originally posted on R-Help, Jan 20, 2009,
http://tolstoy.newcastle.edu.au/R/e6/help/09/01/1873.html

Friendly, M. and Meyer, D. (2016).  \emph{Discrete Data Analysis with R:
Visualization and Modeling Techniques for Categorical and Count Data}.  Boca
Raton, FL: Chapman & Hall/CRC. \url{http://ddar.datavis.ca}.
}
\seealso{
\code{\link[utils]{type.convert}},
\code{\link[gnm]{expandCategorical}}, \code{\link{as_caseform}},
\code{\link{as_table}}, \code{\link{as_freqform}},
\code{\link{as_array}}, \code{\link{as_matrix}}
}
\author{
Mark Schwarz
}
\keyword{array}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmlist.R
\name{glmlist}
\alias{glmlist}
\alias{loglmlist}
\alias{coef.glmlist}
\title{Create a Model List Object}
\usage{
glmlist(...)

loglmlist(...)

\method{coef}{glmlist}(object, result = c("list", "matrix", "data.frame"), ...)
}
\arguments{
\item{\dots}{One or more model objects, as appropriate to the function, optionally assigned names as in \code{\link{list}}.}

\item{object}{a \code{"glmlist"} object}

\item{result}{type of the result to be returned}
}
\value{
An object of class \code{glmlist} \code{loglmlist}, just like a \code{list}, except that each model is given a \code{name} attribute.
}
\description{
\code{glmlist} creates a \code{glmlist} object containing a list of fitted
\code{glm} objects with their names. \code{loglmlist} does the same for
\code{loglm} objects.
}
\details{
The intention is to provide object classes to facilitate model comparison,
extraction, summary and plotting of model components, etc., perhaps using
\code{\link[base]{lapply}} or similar.

There exists a \code{\link[stats]{anova.glm}} method for \code{glmlist}
objects.  Here, a \code{coef} method is also defined, collecting the
coefficients from all models in a single object of type determined by
\code{result}.

The arguments to \code{glmlist} or \code{loglmlist} are of the form
\code{value} or \code{name=value}.

Any objects which do not inherit the appropriate class \code{glm} or
\code{loglm} are excluded, with a warning.

In the \code{coef} method, coefficients from the different models are
matched by name in the list of unique names across all models.
}
\examples{

data(Mental)
indep <- glm(Freq ~ mental+ses,
                family = poisson, data = Mental)
Cscore <- as.numeric(Mental$ses)
Rscore <- as.numeric(Mental$mental)

coleff <- glm(Freq ~ mental + ses + Rscore:ses,
                family = poisson, data = Mental)
roweff <- glm(Freq ~ mental + ses + mental:Cscore,
                family = poisson, data = Mental)
linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
                family = poisson, data = Mental)

# use object names
mods <- glmlist(indep, coleff, roweff, linlin)
names(mods)

# assign new names
mods <- glmlist(Indep=indep, Col=coleff, Row=roweff, LinxLin=linlin)
names(mods)

LRstats(mods)

coef(mods, result='data.frame')

#extract model components
unlist(lapply(mods, deviance))

res <- lapply(mods, residuals)
boxplot(as.data.frame(res), main="Residuals from various models")

}
\seealso{
The function \code{\link[Hmisc]{llist}} in package \code{Hmisc} is
similar, but perplexingly more general.

The function \code{\link[stats]{anova.glm}} also handles \verb{glmlist objects}

\code{\link{LRstats}} gives LR statistics and tests for a \code{glmlist}
object.

Other glmlist functions:
\code{\link[=Kway]{Kway()}},
\code{\link[=LRstats]{LRstats()}},
\code{\link[=get_model]{get_model()}},
\code{\link[=mosaic.glmlist]{mosaic.glmlist()}}

Other loglinear models:
\code{\link[=assoc_graph]{assoc_graph()}},
\code{\link[=get_model]{get_model()}},
\code{\link[=joint]{joint()}},
\code{\link[=plot.assoc_graph]{plot.assoc_graph()}},
\code{\link[=seq_loglm]{seq_loglm()}}
}
\author{
Michael Friendly; \code{coef} method by John Fox
}
\concept{glmlist functions}
\concept{loglinear models}
\keyword{models}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Summarise.R
\name{Summarise}
\alias{Summarise}
\alias{Summarise.glmlist}
\alias{Summarise.loglmlist}
\alias{Summarise.default}
\title{Brief Summary of Model Fit for glm and loglm Models}
\usage{
Summarise(object, ...)

\method{Summarise}{glmlist}(object, ..., saturated = NULL, sortby = NULL)

\method{Summarise}{loglmlist}(object, ..., saturated = NULL, sortby = NULL)

\method{Summarise}{default}(object, ..., saturated = NULL, sortby = NULL)
}
\arguments{
\item{object}{a fitted model object for which there exists a logLik method
to extract the corresponding log-likelihood}

\item{\dots}{optionally more fitted model objects}

\item{saturated}{saturated model log likelihood reference value (use 0 if deviance is not available)}

\item{sortby}{either a numeric or character string specifying the column in the result by which the rows are sorted (in decreasing order)}
}
\value{
A data frame (also of class \code{anova}) with columns
\code{c("AIC", "BIC", "LR Chisq", "Df", "Pr(>Chisq)")}. Row names are taken
from the names of the model object(s).
}
\description{
\strong{Deprecated}: use \code{\link[=LRstats]{LRstats()}} instead, which provides the same brief
model-fit comparison. \code{Summarise()} predates \code{LRstats()} and its capitalized
name reads as an easy-to-mistype near-collision with \code{dplyr::summarise()}
(which this package also imports internally) -- \code{LRstats()} avoids the
naming clash entirely and is otherwise a drop-in replacement.
}
\details{
For \code{glm} objects, the \code{print} and \code{summary} methods give too
much information if all one wants to see is a brief summary of model
goodness of fit, and there is no easy way to display a compact comparison of
model goodness of fit for a collection of models fit to the same data. All
\code{loglm} models have equivalent glm forms, but the \code{print} and
\code{summary} methods give quite different results.

\code{Summarise} provides a brief summary for one or more models fit to the
same dataset for which \code{logLik} and \code{nobs} methods exist (e.g.,
\code{glm} and \code{loglm} models).

The function relies on residual degrees of freedom for the LR chisq test
being available in the model object.  This is true for objects inheriting
from \code{lm}, \code{glm}, \code{loglm}, \code{polr} and \code{negbin}.
}
\examples{

data(Mental)
indep <- glm(Freq ~ mental+ses,
                family = poisson, data = Mental)
Cscore <- as.numeric(Mental$ses)
Rscore <- as.numeric(Mental$mental)

coleff <- glm(Freq ~ mental + ses + Rscore:ses,
                family = poisson, data = Mental)
roweff <- glm(Freq ~ mental + ses + mental:Cscore,
                family = poisson, data = Mental)
linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
                family = poisson, data = Mental)

# Summarise() is deprecated; see LRstats() for the same comparison
\dontrun{
Summarise(indep)

# compare models
Summarise(indep, coleff, roweff, linlin)
}


}
\seealso{
\code{\link[stats]{logLik}}, \code{\link[stats]{glm}},
\code{\link[MASS]{loglm}},

\code{\link{logLik.loglm}}, \code{\link{modFit}},  \code{\link{LRstats}}
}
\author{
Achim Zeileis
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Yamaguchi87}
\alias{Yamaguchi87}
\title{Occupational Mobility in Three Countries}
\format{
A frequency data frame with 75 observations on the following 4
variables. The total sample size is 28887.

\describe{
\item{\code{Son}}{a factor with levels \code{UpNM} \code{LoNM} \code{UpM} \code{LoM} \code{Farm}}
\item{\code{Father}}{a factor with levels \code{UpNM} \code{LoNM} \code{UpM} \code{LoM} \code{Farm}}
\item{\code{Country}}{a factor with levels \code{US} \code{UK} \code{Japan}}
\item{\code{Freq}}{a numeric vector}
}
}
\source{
Yamaguchi, K. (1987).  Models for comparing mobility tables: toward
parsimony and substance, \emph{American Sociological Review}, vol. 52
(Aug.), 482-494, Table 1
}
\description{
Yamaguchi (1987) presented this three-way frequency table, cross-classifying
occupational categories of sons and fathers in the United States, United
Kingdom and Japan.  This data set has become a classic for models comparing
two-way mobility tables across layers corresponding to countries, groups or
time (e.g., Goodman and Hout, 1998; Xie, 1992).
}
\details{
The US data were derived from the 1973 OCG-II survey; those for the UK from
the 1972 Oxford Social Mobility Survey; those for Japan came from the 1975
Social Stratification and Mobility survey. They pertain to men aged 20-64.

Five status categories -- upper and lower nonmanuals (\code{UpNM},
\code{LoNM}), upper and lower manuals (\code{UpM}, \code{LoM}), and
\code{Farm}) are used for both fathers' occupations and sons' occupations.

Upper nonmanuals are professionals, managers, and officials; lower
nonmanuals are proprietors, sales workers, and clerical workers; upper
manuals are skilled workers; lower manuals are semi-skilled and unskilled
nonfarm workers; and farm workers are farmers and farm laborers.

Some of the models from Xie (1992), Table 1, are fit in
\code{demo(yamaguchi-xie)}.
}
\examples{

data(Yamaguchi87)
# reproduce Table 1
structable(~ Father + Son + Country, Yamaguchi87)
# create table form
Yama.tab <- xtabs(Freq ~ Son + Father + Country, data=Yamaguchi87)

# define mosaic labeling_args for convenient reuse in 3-way displays
largs <- list(rot_labels=c(right=0), offset_varnames = c(right = 0.6),
              offset_labels = c(right = 0.2),
              set_varnames = c(Son="Son's status", Father="Father's status")
             )

###################################
# Fit some models & display mosaics

# Mutual independence
yama.indep <- glm(Freq ~ Son + Father + Country,
  data=Yamaguchi87,
  family=poisson)
anova(yama.indep)

mosaic(yama.indep, ~Son+Father, main="[S][F] ignoring country")

mosaic(yama.indep, ~Country + Son + Father, condvars="Country",
       labeling_args=largs,
       main='[S][F][C] Mutual independence')

# no association between S and F given country ('perfect mobility')
# asserts same associations for all countries
yama.noRC <- glm(Freq ~ (Son + Father) * Country,
  data=Yamaguchi87,
  family=poisson)
anova(yama.noRC)

mosaic(yama.noRC, ~~Country + Son + Father, condvars="Country",
       labeling_args=largs,
       main="[SC][FC] No [SF] (perfect mobility)")

# ignore diagonal cells
yama.quasi <- update(yama.noRC, ~ . + Diag(Son,Father):Country)
anova(yama.quasi)

mosaic(yama.quasi, ~Son + Father, main="Quasi [S][F]")

## see also:
# demo(yamaguchi-xie)
##

}
\references{
Goodman, L. A. and Hout, M. (1998). Statistical Methods and
Graphical Displays for Analyzing How the Association Between Two Qualitative
Variables Differs Among Countries, Among Groups, Or Over Time: A Modified
Regression-Type Approach. \emph{Sociological Methodology}, 28 (1), 175-230.

Xie, Yu (1992). The log-multiplicative layer effect model for comparing
mobility tables. \emph{American Sociological Review}, 57 (June), 380-395.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seq_mosaic.R
\name{seq_mosaic}
\alias{seq_mosaic}
\title{Sequential Mosaics and Strucplots for an N-way Table}
\usage{
seq_mosaic(
  x,
  panel = mosaic,
  type = c("joint", "conditional", "mutual", "markov", "saturated"),
  plots = 1:nf,
  vorder = 1:nf,
  k = NULL,
  ...
)
}
\arguments{
\item{x}{a contingency table in array form, with optional category labels
specified in the \code{dimnames(x)} attribute, or else a data.frame in frequency
form, with the frequency variable named \code{"Freq"}.}

\item{panel}{a \code{\link[vcd]{strucplot}} panel function, typically
\code{\link[vcd]{mosaic}} or \code{\link[vcd]{sieve}. NOT yet implemented.}}

\item{type}{type of sequential model to fit, a character string. One of
\code{"joint"}, \code{"conditional"}, \code{"mutual"}, \code{"markov"}, or
\code{"saturated"}.}

\item{plots}{which marginal sub-tables to plot? A vector of a (sub)set of
the integers, \code{1:nf} where \code{nf} is the number of factors in the
full n-way table.}

\item{vorder}{order of variables, a permutation of the integers \code{1:nf},
used to reorder the variables in the original table for the purpose of
fitting sequential marginal models.}

\item{k}{conditioning variable(s) for \code{type} = \code{"joint"},
\code{"conditional"} or Markov chain order for \code{type} = \code{"markov"}}

\item{\dots}{other arguments passed to \code{\link[vcd]{mosaic}}.}
}
\value{
None. Used for its side-effect of producing plots
}
\description{
This function takes an n-way contingency table and plots mosaics for series
of sequential models to the 1-, 2-, ... n-way marginal tables, corresponding
to a variety of types of loglinear models.
}
\details{
This function produces similar plots to the use of
\code{\link{mosaic.loglmlist}}, called with the result of
\code{\link{seq_loglm}}.
}
\examples{


data(Titanic, package="datasets")

seq_mosaic(Titanic)  # models of joint independence, Survived last
seq_mosaic(Titanic, type="condit")
seq_mosaic(Titanic, type="mutual")

# other panel functions and options
seq_mosaic(Titanic, type="mutual", panel=sieve,
   gp=shading_Friendly, labeling=labeling_values)


}
\references{
These functions were inspired by the original SAS implementation
of mosaic displays, described in the \emph{User's Guide for Mosaics},
\url{http://www.datavis.ca/mosaics/mosaics.pdf}
}
\seealso{
\code{\link{loglin-utilities}} for descriptions of sequential
models, \code{\link{conditional}}, \code{\link{joint}}, \code{\link{mutual}}, \dots{}
\code{\link{loglmlist}}, \code{\link{mosaic.loglmlist}},
\code{\link{seq_loglm}}

\if{html}{\out{<div class="sourceCode">}}\preformatted{    \code{\link{mosaic.glm}}, \code{\link[vcd]{mosaic}},
    \code{\link[vcd]{strucplot}}, for the many parameters that control the details of mosaic plots.
}\if{html}{\out{</div>}}
}
\author{
Michael Friendly
}
\keyword{hplots}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Crossings.R
\name{Crossings}
\alias{Crossings}
\title{Crossings Interaction of Factors}
\usage{
Crossings(...)
}
\arguments{
\item{\dots}{Two factors}
}
\value{
For two factors of \code{n} levels, returns a binary indicator
matrix of \code{n*n} rows and \code{n-1} columns.
}
\description{
Given two ordered factors in a square, n x n frequency table,
\code{Crossings} creates an n-1 column matrix corresponding to different
degrees of difficulty in crossing from one level to the next, as described
by Goodman (1972).
}
\details{
Instead of treating all mobility as equal, this model posits that the difficulty
of moving between categories increases with the number of boundaries (or "crossings")
that must be crossed, and that associations between categories decrease
with their separation.
}
\examples{

data(Hauser79)
# display table
structable(~Father + Son, data=Hauser79)

hauser.indep <- gnm(Freq ~ Father + Son,
                    data=Hauser79,
                    family=poisson)

hauser.CR <- update(hauser.indep,
                    ~ . + Crossings(Father,Son))
LRstats(hauser.CR)

hauser.CRdiag <- update(hauser.indep,
                        ~ . + Crossings(Father,Son) + Diag(Father,Son))
LRstats(hauser.CRdiag)

# what does Crossings do?
cr <- with(Hauser79, Crossings(Father, Son))
head(cr)
# Show the codings for varying Crossings levels
matrix(cr[,1], nrow=5)
matrix(cr[,2], nrow=5)
matrix(cr[,3], nrow=5)
matrix(cr[,4], nrow=5)

}
\references{
Goodman, L. (1972).  Some multiplicative models for the analysis
of cross-classified data. In: \emph{Proceedings of the Sixth Berkeley
Symposium on Mathematical Statistics and Probability}, Berkeley, CA:
University of California Press, pp. 649-696.
}
\seealso{
\code{\link[stats]{glm}}, \code{\link[gnm]{gnm}} for model fitting
functions for frequency tables;
\code{\link[gnm]{Diag}}, \code{\link[gnm]{Mult}}, \code{\link[gnm]{Symm}},
\code{\link[gnm]{Topo}} for similar extensions to terms in model formulas.
}
\author{
Michael Friendly and Heather Turner
}
\keyword{manip}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HLtest.R
\name{HLtest}
\alias{HLtest}
\alias{HosmerLemeshow}
\alias{plot.HLtest}
\alias{print.HLtest}
\alias{rootogram.HLtest}
\alias{summary.HLtest}
\title{Hosmer-Lemeshow Goodness of Fit Test}
\usage{
HLtest(model, g = 10)

\method{print}{HLtest}(x, ...)

\method{summary}{HLtest}(object, ...)

\method{rootogram}{HLtest}(x, ...)

\method{plot}{HLtest}(x, ...)
}
\arguments{
\item{model}{A \code{glm} model object in the \code{binomial} family}

\item{g}{Number of groups used to partition the fitted values for the GOF
test.}

\item{x, object}{A \code{HLtest} object}

\item{\dots}{Other arguments passed down to methods}
}
\value{
A class \code{HLtest} object with the following components:
\item{table}{A data.frame describing the results of partitioning the data
into \code{g} groups with the following columns: \code{cut}, \code{total},
\code{obs}, \code{exp}, \code{chi}} \item{chisq}{The chisquare statistics}
\item{df}{Degrees of freedom} \item{p.value}{p value} \item{groups}{Number
of groups} \item{call}{\code{model} call} \%\% ...
}
\description{
The \code{HLtest} function computes the classical Hosmer-Lemeshow (1980)
goodness of fit test for a binomial \code{glm} object in logistic regression
}
\details{
The general idea is to assesses whether or not the observed event rates
match expected event rates in subgroups of the model population. The
Hosmer-Lemeshow test specifically identifies subgroups as the deciles of
fitted event values, or other quantiles as determined by the \code{g}
argument. Given these subgroups, a simple chisquare test on \code{g-2} df is
used.

In addition to \code{print} and \code{summary} methods, a \code{plot} method
is supplied to visualize the discrepancies between observed and fitted
frequencies.
}
\examples{


data(birthwt, package="MASS")
# how to do this without attach?
attach(birthwt)
	race = factor(race, labels = c("white", "black", "other"))
	ptd = factor(ptl > 0)
	ftv = factor(ftv)
	levels(ftv)[-(1:2)] = "2+"
	bwt <- data.frame(low = factor(low), age, lwt, race,
    	smoke = (smoke > 0), ptd, ht = (ht > 0), ui = (ui > 0), ftv)
detach(birthwt)

options(contrasts = c("contr.treatment", "contr.poly"))
BWmod <- glm(low ~ ., family=binomial, data=bwt)

(hlt <- HLtest(BWmod))
str(hlt)
summary(hlt)
plot(hlt)

# basic model
BWmod0 <- glm(low ~ age, family=binomial, data=bwt)
(hlt0 <- HLtest(BWmod0))
str(hlt0)
summary(hlt0)
plot(hlt0)



}
\references{
Hosmer, David W., Lemeshow, Stanley (1980).  A goodness-of-fit test for
multiple logistic regression model. \emph{Communications in Statistics,
Series A}, 9, 1043-1069.

Hosmer, David W., Lemeshow, Stanley (2000).  \emph{Applied Logistic
Regression}, New York: Wiley, ISBN 0-471-61553-6

Lemeshow, S. and Hosmer, D.W.  (1982).  A review of goodness of fit
statistics for use in the development of logistic regression models.
\emph{American Journal of Epidemiology}, 115(1), 92-106.
}
\seealso{
\code{\link[vcd]{rootogram}}, ~~~

Other association tests:
\code{\link[=CMHtest]{CMHtest()}},
\code{\link[=GKgamma]{GKgamma()}},
\code{\link[=breslow_day_test]{breslow_day_test()}},
\code{\link[=woolf_test]{woolf_test()}},
\code{\link[=zero.test]{zero.test()}}
}
\author{
Michael Friendly
}
\concept{association tests}
\keyword{htest}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Alligator}
\alias{Alligator}
\title{Alligator Food Choice}
\format{
A frequency data frame with 80 observations on the following 5
variables.
\describe{
\item{\code{lake}}{a factor with levels \code{George} \code{Hancock} \code{Oklawaha} \code{Trafford}}
\item{\code{sex}}{a factor with levels \code{female} \code{male}}
\item{\code{size}}{alligator size, a factor with levels \code{large} (>2.3m) \code{small} (<=2.3m)}
\item{\code{food}}{primary food choice, a factor with levels \code{bird} \code{fish}
\code{invert} \code{other} \code{reptile}}
\item{\code{count}}{cell frequency, a numeric vector} }
}
\source{
Agresti, A. (2002). \emph{Categorical Data Analysis}, New York:
Wiley, 2nd Ed., Table 7.1
}
\description{
The Alligator data, from Agresti (2002), comes from a study of the primary
food choices of alligators in four Florida lakes. Researchers classified the
stomach contents of 219 captured alligators into five categories: Fish (the
most common primary food choice), Invertebrate (snails, insects, crayfish,
etc.), Reptile (turtles, alligators), Bird, and Other (amphibians, plants,
household pets, stones, and other debris).
}
\details{
The table contains a fair number of 0 counts.

\code{food} is the response variable.  \code{fish} is the most frequent
choice, and often taken as a baseline category in multinomial response
models.
}
\examples{

data(Alligator)

# change from frequency data.frame to table
allitable <- xtabs(count ~ lake + sex + size + food, data=Alligator)
# Agresti's Table 7.1
structable(food ~ lake + sex + size, allitable)


plot(allitable, shade=TRUE)

# mutual independence model
mosaic(~ food + lake + size, allitable, shade=TRUE)

# food jointly independent of lake and size
mosaic(~ food + lake + size, allitable, shade=TRUE,
       expected = ~lake:size + food)

if (require(nnet)) {
	# multinomial logit model
	mod1 <- multinom(food ~ lake + size + sex, data=Alligator, weights=count)
}


}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc_graph.R
\name{assoc_graph}
\alias{assoc_graph}
\alias{assoc_graph.list}
\alias{assoc_graph.loglm}
\alias{assoc_graph.glm}
\alias{print.assoc_graph}
\title{Association Graph for a Loglinear Model}
\usage{
assoc_graph(x, ...)

\method{assoc_graph}{list}(x, result = c("igraph", "matrix", "edge_list"), ...)

\method{assoc_graph}{loglm}(x, result = c("igraph", "matrix", "edge_list"), ...)

\method{assoc_graph}{glm}(
  x,
  result = c("igraph", "matrix", "edge_list"),
  measure = c("none", "chisq", "cramer"),
  ...
)

\method{print}{assoc_graph}(x, ...)
}
\arguments{
\item{x}{An object specifying the model. This can be:
\itemize{
\item A \code{list} of character vectors (a margin/generating class list, as produced by
\code{\link{joint}}, \code{\link{conditional}}, etc.)
\item A fitted \code{\link[MASS]{loglm}} object
\item A fitted \code{\link[stats]{glm}} object (poisson family loglinear model)
}}

\item{\dots}{Additional arguments (currently unused).}

\item{result}{Type of result to return: \code{"igraph"} (default) returns an
\code{\link[igraph:igraph-package]{igraph}} object; \code{"matrix"} returns the
adjacency matrix; \code{"edge_list"} returns a two-column character matrix of edges.}

\item{measure}{Type of association measure for edge weights (only for \code{glm} method):
\code{"none"} (default) produces an unweighted graph;
\code{"chisq"} computes partial chi-squared statistics (deviance change when each
edge is removed from the model);
\code{"cramer"} computes Cramer's V from the marginal two-way table for each edge.}
}
\value{
Depending on \code{result}:
\itemize{
\item \code{"igraph"}: An \code{igraph} undirected graph object of class
\code{c("assoc_graph", "igraph")}, with vertex names corresponding to
the variable names. When \code{measure != "none"}, edge weights are
stored as \code{E(g)$weight} and the measure name as \code{g$measure}.
\item \code{"matrix"}: A symmetric adjacency matrix with variable names as
row and column names. Contains 0/1 when unweighted, or association
strength values when \code{measure} is specified.
\item \code{"edge_list"}: When unweighted, a two-column character matrix (from, to).
When \code{measure} is specified, a data frame with columns from, to, and weight.
}
}
\description{
Construct an undirected graph representing the associations in a loglinear model.
Nodes represent variables and edges represent pairwise associations fitted in the model.
If two variables are not connected by an edge, they are conditionally independent
given the other variables.
}
\details{
Each high-order term (margin) in a hierarchical loglinear model defines a clique
in the association graph. For example, the term \code{c("A", "B", "C")} generates
edges A--B, A--C, and B--C. Single-variable terms (as in mutual independence)
yield isolated nodes with no edges.

For \code{loglm} objects, the margins are extracted from the \code{$margin} component.
For \code{glm} objects, the interaction terms are extracted from the model formula.
}
\examples{
# Structural graphs from margin lists (3-way: A, B, C)
mutual(3, factors = c("A", "B", "C"))      |> assoc_graph()
joint(3, factors = c("A", "B", "C"))       |> assoc_graph()
conditional(3, factors = c("A", "B", "C")) |> assoc_graph()
saturated(3, factors = c("A", "B", "C"))   |> assoc_graph()

# Adjacency matrix form
conditional(3, factors = c("A", "B", "C")) |> assoc_graph(result = "matrix")

# From a fitted loglm model (Berkeley admissions)
\dontrun{
mod <- MASS::loglm(~ (Admit + Gender) * Dept, data = UCBAdmissions)
assoc_graph(mod)
plot(assoc_graph(mod), main = "Berkeley: [AD] [GD]")
}

# From glm models (Dayton Survey: cigarette, alcohol, marijuana, sex, race)
data(DaytonSurvey)

# Mutual independence + sex*race: one edge only
mod.SR <- glm(Freq ~ . + sex*race, data = DaytonSurvey, family = poisson)
assoc_graph(mod.SR)
plot(assoc_graph(mod.SR), main = "Mutual indep. + [SR]")

# [AM][AC][MC][AR][AS][RS]: {race, sex} indep {marijuana, cigs} | alcohol
mod.cond <- glm(Freq ~ (cigarette + alcohol + marijuana)^2 +
                        (alcohol + sex + race)^2,
                data = DaytonSurvey, family = poisson)

# define groups for the model
gps <- list(c("cigarette", "marijuana"),
            "alcohol",
            c("sex", "race"))

assoc_graph(mod.cond)
plot(assoc_graph(mod.cond),
     groups = gps,
     layout = igraph::layout_nicely,
     main = "{R,S} indep {M,C} | A")

# Weighted graph: partial chi-squared
g <- assoc_graph(mod.cond, measure = "chisq")
g
plot(g, edge.label = TRUE,
     groups = gps,
     layout = igraph::layout_nicely,
     main = "Partial chi-squared weights")

# Cramer's V (marginal)
g2 <- assoc_graph(mod.cond, measure = "cramer")
g2
plot(g2, edge.label = TRUE,
     groups = gps,
     layout = igraph::layout_nicely,
     main = "Cramer's V weights")

}
\references{
Khamis, H. J. (2011). \emph{The Association Graph and the Multigraph for Loglinear Models}.
SAGE Publications. \doi{10.4135/9781452226521}

Darroch, J. N., Lauritzen, S. L., & Speed, T. P. (1980). Markov Fields and Log-Linear
Interaction Models for Contingency Tables. \emph{The Annals of Statistics}, 8(3), 522--539.
\doi{10.1214/aos/1176345006}

Whittaker, J. (1990). \emph{Graphical Models in Applied Multivariate Statistics}.
John Wiley & Sons, Chichester.
}
\seealso{
\code{\link{joint}}, \code{\link{conditional}}, \code{\link{mutual}},
\code{\link{saturated}}, \code{\link{loglin2string}}, \code{\link{seq_loglm}},
\code{\link{plot.assoc_graph}}

Other loglinear models:
\code{\link[=get_model]{get_model()}},
\code{\link[=glmlist]{glmlist()}},
\code{\link[=joint]{joint()}},
\code{\link[=plot.assoc_graph]{plot.assoc_graph()}},
\code{\link[=seq_loglm]{seq_loglm()}}
}
\concept{loglinear models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Toxaemia}
\alias{Toxaemia}
\title{Toxaemia Symptoms in Pregnancy}
\format{
A data frame in frequency form representing a 5 x 3 x 2 x 2
contingency table, with 60 observations on the following 5 variables.
\describe{
\item{\code{class}}{Social class of mother, a factor with levels \code{1} \code{2} \code{3} \code{4} \code{5}}
\item{\code{smoke}}{Cigarettes smoked per day during pregnancy, a factor with levels \code{0} \code{1-19} \verb{20+}}
\item{\code{hyper}}{Hypertension level, a factor with levels \code{Low} \code{High}}
\item{\code{urea}}{Protein urea level, a factor with levels \code{Low} \code{High}}
\item{\code{Freq}}{frequency in each cell, a numeric vector}
}
}
\source{
Brown, P. J., Stone, J. and Ord-Smith, C. (1983), Toxaemic signs
during pregnancy. \emph{JRSS, Series C, Applied Statistics}, 32, 69-72
}
\description{
Brown et al (1983) gave these data on two signs of toxaemia, an abnormal
condition during pregnancy characterized by high blood pressure
(hypertension) and high levels of protein in the urine.  If untreated, both
the mother and baby are at risk of complications or death.
}
\details{
The data frame \code{Toxaemia} represents 13384 expectant mothers in
Bradford, England in their first pregnancy, who were also classified
according to social class and the number of cigarettes smoked per day.
}
\examples{

data(Toxaemia)

tox.tab <- xtabs(Freq ~ class + smoke + hyper + urea, Toxaemia)
ftable(tox.tab, row.vars=1)


# symptoms by smoking
mosaic(~smoke + hyper + urea, data=tox.tab, shade=TRUE)

# symptoms by social class
mosaic(~class + hyper + urea, data=tox.tab, shade=TRUE)

# predictors
mosaic(~smoke + class, data=tox.tab, shade=TRUE)

# responses
mosaic(~hyper + urea, data=tox.tab, shade=TRUE)

# log odds ratios for urea and hypertension, by class and smoke
LOR <-loddsratio(aperm(tox.tab))
LOR


}
\references{
Friendly, M.  (2000), \emph{Visualizing Categorical Data}, SAS
Institute, Cary, NC, Example 7.15.

Friendly, M. and Meyer, D. (2016).  \emph{Discrete Data Analysis with R:
Visualization and Modeling Techniques for Categorical and Count Data}.  Boca
Raton, FL: Chapman & Hall/CRC. \url{http://ddar.datavis.ca}. Example 10.10.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seq_loglm.R
\name{seq_loglm}
\alias{seq_loglm}
\title{Sequential Loglinear Models for an N-way Table}
\usage{
seq_loglm(
  x,
  type = c("joint", "conditional", "mutual", "markov", "saturated"),
  marginals = 1:nf,
  vorder = 1:nf,
  k = NULL,
  prefix = NULL,
  fitted = TRUE,
  ...
)
}
\arguments{
\item{x}{a contingency table in array form, with optional category labels specified in the dimnames(x) attribute,
or else a data.frame in frequency form, with the frequency variable named \code{"Freq"}.}

\item{type}{type of sequential model to fit, a character string. One of \code{"joint"}, \code{"conditional"},
\code{"mutual"}, \code{"markov"}, or \code{"saturated"}.}

\item{marginals}{which marginal sub-tables to fit? A vector of a (sub)set of the integers, \code{1:nf} where
\code{nf} is the number of factors in the full n-way table.}

\item{vorder}{order of variables, a permutation of the integers \code{1:nf}, used to reorder the variables in
the original table for the purpose of fitting sequential marginal models.}

\item{k}{conditioning variable(s) for \code{type} = \code{"joint"}, \code{"conditional"} or Markov chain order
for \code{type} = \code{"markov"}}

\item{prefix}{prefix used to give names to the sequential models. If \code{NULL} (the default),
uses an abbreviation of \code{type}: \code{"joint"}, \code{"cond"}, \code{"mutual"}, \code{"markov"}, or \code{"sat"}.}

\item{fitted}{argument passed to \code{loglm} to store the fitted values in the model objects}

\item{\dots}{other arguments, passed down}
}
\value{
An object of class \code{"loglmlist"}, each of which is a class \code{"loglm"} object
}
\description{
This function takes an n-way contingency table and fits a series of
sequential models to the 1-, 2-, ... n-way marginal tables, corresponding to
a variety of types of loglinear models.
}
\details{
Sequential marginal models for an n-way tables begin with the model of
equal-probability for the one-way margin (equivalent to a
\code{\link[stats]{chisq.test}}) and add successive variables one at a time
in the order specified by \code{vorder}.

All model types give the same result for the two-way margin, namely the test
of independence for the first two factors.

Sequential models of \emph{joint independence} (\code{type="joint"}) have a
particularly simple interpretation, because they decompose the likelihood
ratio test for the model of mutual independence in the full n-way table, and
hence account for "total" association in terms of portions attributable to
the conditional probabilities of each new variable, given all prior
variables.
}
\note{
One-way marginal tables are a bit of a problem here, because they
cannot be fit directly using \code{\link[MASS]{loglm}}. The present version
uses \code{\link[stats]{loglin}}, and repairs the result to look like a
\code{loglm} object (sort of).
}
\examples{

data(Titanic, package="datasets")
# variables are in the order Class, Sex, Age, Survived

# Models of joint independence
tit.joint <- seq_loglm(Titanic, type = "joint")

# compare the models
LRstats(tit.joint)

#' # Models of conditional independence
tit.cond <- seq_loglm(Titanic, type = "conditional")

# compare the models
LRstats(tit.cond)



}
\references{
These functions were inspired by the original SAS implementation
of mosaic displays, described in the \emph{User's Guide},
\url{http://www.datavis.ca/mosaics/mosaics.pdf}
}
\seealso{
\code{\link{loglin-utilities}} for descriptions of sequential
models, \code{\link{conditional}}, \code{\link{joint}},
\code{\link{mutual}}, \dots{}

\code{\link{loglmlist}}

Other loglinear models:
\code{\link[=assoc_graph]{assoc_graph()}},
\code{\link[=get_model]{get_model()}},
\code{\link[=glmlist]{glmlist()}},
\code{\link[=joint]{joint()}},
\code{\link[=plot.assoc_graph]{plot.assoc_graph()}}
}
\author{
Michael Friendly
}
\concept{loglinear models}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapse_levels.R
\name{collapse_levels}
\alias{collapse_levels}
\title{Collapse the levels of a dataset}
\usage{
collapse_levels(x, freq = "Freq", ...)
}
\arguments{
\item{x}{The dataset to be collapsed.}

\item{freq}{Supply only if your data is in frequency form AND your frequency
column differs in name from the default ("Freq").}

\item{\dots}{A collection of one or more assignments of dataset variables to
a list of levels in the format
\code{new_level = c("old_level_1", "old_level_2", ..., "old_level_n")}.}
}
\value{
The collapsed dataset in its original form (i.e., the initial form of
\code{x}).
}
\description{
Collapses the levels of a dataset (of any form) into those specified. May
also be used to re-name levels. Ensure argument \code{freq} is supplied
should your data be in frequency form (and the frequency column differs in
name from default, "Freq").
}
\details{
First converts the object \code{x} into a frequency form data frame. Then,
\code{\link[forcats]{fct_collapse}} is used to collapse variable levels.
Next, duplicate rows (an artefact of collapsing) are aggregated via
\code{\link[dplyr]{summarise}}. Last, the frequency form data frame is
converted back into the initial form of object \code{x}.

The exceptions to this are objects in case form, which are passed directly
to \code{\link[forcats]{fct_collapse}} (and duplicate rows are not aggregated).
}
\examples{
data("HairEyeColor") # Table form data
str(HairEyeColor)

collapse_levels(
  HairEyeColor,                 # Dataset
  Hair = list(                  # List of arguments for first variable
    Dark = c("Black", "Brown"), # Collapse "Black" and "Brown" -> "Dark"
    Light = c("Blond", "Red")   # Collapse "Blond" and "Red" -> "Light"
  ),
  Eye = list(                   # List of arguments for second variable
    Common = c("Brown"),        # Collapse (rename) "Brown" -> "Common"
    Uncommon = c("Blue", "Green", "Hazel")
  )
) |> str()

# To illustrate `freq` argument usage, convert Hoyt dataset to frequency form 
# (ff) and then rename frequency column to "n"

data("Hoyt", package = "vcdExtra")
ff_Hoyt <- as_freqform(Hoyt)
names(ff_Hoyt)[length(ff_Hoyt)] <- "n"
str(ff_Hoyt)

collapse_levels(
  ff_Hoyt,
  
  # Ensure to supply if data is in frequency form and frequency column name
  # differs from "Freq"
  freq = "n",
  
  Occupation = list(
    High = c(1, 2),
    Middle = 3,
    Low = 4,
    VeryLow = c(5, 6, 7)
  )
) |> str()


}
\seealso{
\code{\link[forcats]{fct_collapse}}, \code{\link{collapse.table}}

Tidy conversion functions:
\code{link{as_table}}, \code{link{as_freqform}}, \code{link{as_caseform}},
\code{link{as_matrix}}, \code{link{as_array}},
}
\author{
Gavin M. Klorfine
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_freqform.R
\name{as_freqform}
\alias{as_freqform}
\title{Convert any form (case or table form) into frequency form.}
\usage{
as_freqform(obj, freq = NULL, dims = NULL, prop = NULL, tidy = TRUE)
}
\arguments{
\item{obj}{Object to be converted to frequency form.}

\item{freq}{If \code{obj} is already in frequency form, this is the name of the frequency
column. If \code{obj} is in any other form, do not supply an argument (see "Details").}

\item{dims}{A character vector of dimensions. If not specified, all variables apart from
\code{freq} will be used as dimensions.}

\item{prop}{If set to \code{TRUE}, the resulting "frequency" column will contain proportions
(that sum to 1). May also be set to a character or numeric vector of
dimensions to be used as margins from which proportions will be computed.
The resulting "frequency" column is renamed to "Prop."}

\item{tidy}{Returns a tibble if set to \code{TRUE}.}
}
\value{
Object in frequency form.
}
\description{
A wrapper for \code{\link[base]{as.data.frame}} that is able to properly handle arrays.
Converts object (\code{obj}) in case or table form into frequency form. The
column containing the frequencies (\code{freq}) must be supplied if \code{obj} is
already in frequency form (and you are using this function to select
dimensions). Returns a tibble if \code{tidy} is set to \code{TRUE}.
}
\details{
Converts \code{obj} to a table using \code{\link{as_table}} before converting to
frequency form
}
\examples{
library(vcdExtra)

data("HairEyeColor")

freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
tableForm <- as_table(HairEyeColor) # Generate table form data
arrayDat <- as_array(HairEyeColor) # Generate an array
caseForm <- as_caseform(HairEyeColor) # Generate case form data

# array -> frequency form
as_freqform(arrayDat) |> str()

# table -> frequency form
as_freqform(tableForm) |> str()

# case -> frequency form
as_freqform(caseForm) |> str()

# Selecting dimensions (optional)
as_freqform(freqForm, freq = "Freq", dims = c("Hair", "Eye")) |> str()

as_freqform(tableForm, dims = c("Hair", "Eye")) |> str()

#-----For proportions-----#

as_freqform(tableForm, prop = TRUE) |> head() # print only Sex == Male rows

# Marginalize proportions along "Sex" (i.e., male proportions sum to 1, female proportions sum to 1)
as_freqform(tableForm, prop = "Sex") |> head()

as_freqform(tableForm, prop = 3) |> head() # Same as above

# Marginalize proportions along multiple variables
as_freqform(tableForm, prop = c("Hair", "Sex")) |> head()

as_freqform(tableForm, prop = c(1, 3)) |> head() # Same as above

# Using dims and prop arguments in tandem
as_freqform(tableForm, dims = c("Hair", "Eye"), prop = TRUE)

}
\seealso{
\code{\link{as_table}}, \code{\link{as_caseform}}, \code{\link{as_array}},
\code{\link{as_matrix}}
}
\author{
Gavin M. Klorfine
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modFit.R
\name{modFit}
\alias{modFit}
\alias{modFit.loglm}
\alias{modFit.glm}
\title{Brief Summary of Model Fit for a glm or loglm Object}
\usage{
modFit(x, ...)

\method{modFit}{glm}(x, stats = "chisq", digits = 2, ...)

\method{modFit}{loglm}(x, stats = "chisq", digits = 2, ...)
}
\arguments{
\item{x}{A \code{glm} or \code{loglm} object}

\item{\dots}{Arguments passed down}

\item{stats}{statistics to print: one or more of \code{"chisq"}, \code{"aic"}}

\item{digits}{number to digits to use in the print method}
}
\value{
A character string containing the formatted values of the chosen statistics.
}
\description{
Formats a brief summary of model fit for a \code{glm} or \code{loglm}
object, showing the likelihood ratio Chisq (df) value and or AIC.  Useful
for inclusion in a plot title or annotation.
}
\examples{

data(Mental)
require(MASS)
(Mental.tab <- xtabs(Freq ~ ses + mental, data=Mental))
(Mental.mod <- loglm(~ses + mental, Mental.tab))
Mental.mod
modFit(Mental.mod)

# use to label mosaic()
mosaic(Mental.mod, main=paste("Independence model,", modFit(Mental.mod)))

}
\seealso{
\code{\link{LRstats}}
}
\author{
Michael Friendly
}
\keyword{models}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ShakeWords}
\alias{ShakeWords}
\title{Shakespeare's Word Type Frequencies}
\format{
A data frame with 100 observations on the following 2 variables.
\describe{
\item{\code{count}}{the number of times a word type appeared in Shakespeare's written works}
\item{\code{Freq}}{the number of different words (types) appearing with this count.}
}
}
\source{
Bradley Efron and Ronald Thisted (1976). Estimating the Number of
Unseen Species: How Many Words Did Shakespeare Know? \emph{Biometrika}, Vol.
63, No. 3, pp. 435-447,
\%\url{http://www.jstor.org/stable/2335721}
}
\description{
This data set, from Efron and Thisted (1976), gives the number of distinct
words types (\code{Freq}) of words that appeared exactly once, twice, etc.
up to 100 times (\code{count}) in the complete works of Shakespeare.  In
these works, Shakespeare used 31,534 distinct words (types), comprising
884,647 words in total.
}
\details{
Efron & Thisted used this data to ask the question, "How many words did
Shakespeare know?"  Put another way, suppose another new corpus of works
Shakespeare were discovered, also with 884,647 words. How many new word
types would appear? The answer to the main question involves contemplating
an infinite number of such new corpora.

In addition to the words that appear \code{1:100} times, there are 846 words
that appear more than 100 times, not listed in this data set.
}
\examples{

data(ShakeWords)
str(ShakeWords)

plot(sqrt(Freq) ~ count, data=ShakeWords)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Caesar}
\alias{Caesar}
\title{Risk Factors for Infection in Caesarian Births}
\format{
A 4-dimensional array resulting from cross-tabulating 4 variables
for 251 observations. The variable names and their levels are:

\tabular{rll}{
dim \tab Name \tab Levels \cr
1 \tab \code{Infection}\tab \verb{"Type 1", "Type 2", "None"}\cr
2 \tab \code{Risk}\tab \verb{"Yes", "No"} (presence of risk factors)\cr
3 \tab \code{Antibiotics}\tab \verb{"Yes", "No"} (were antibiotics given?)\cr
4 \tab \code{Planned}\tab \verb{"Yes", "No"} (was the C section planned?)\cr
}
}
\source{
\% \cite{Fahrmeir:94}
Fahrmeir, L. & Tutz, G. (1994). Multivariate
Statistical Modelling Based on Generalized Linear Models New York: Springer
Verlag, Table 1.1.
}
\description{
Data from infection from birth by Caesarian section, classified by
\code{Risk} (two levels), whether \code{Antibiotics} were used (two levels)
and whether the Caesarian section was \code{Planned} or not.  The outcome is
\code{Infection} (three levels).
}
\details{
\code{Infection} is regarded as the response variable here.  There are quite
a few 0 cells here, particularly when \code{Risk} is absent and the
Caesarian section was unplanned. Should these be treated as structural or
sampling zeros?
}
\examples{

data(Caesar)
#display table;  note that there are quite a few 0 cells
structable(Caesar)
require(MASS)

# baseline model, Infection as response
Caesar.mod0 <- loglm(~Infection + (Risk*Antibiotics*Planned),
                     data=Caesar)

# NB: Pearson chisq cannot be computed due to the 0 cells
Caesar.mod0

mosaic(Caesar.mod0, main="Baseline model")

# Illustrate handling structural zeros
zeros <- 0+ (Caesar >0)
zeros[1,,1,1] <- 1
structable(zeros)

# fit model excluding possible structural zeros
Caesar.mod0s <- loglm(~Infection + (Risk*Antibiotics*Planned),
                      data=Caesar,
	                    start=zeros)
Caesar.mod0s

anova(Caesar.mod0, Caesar.mod0s, test="Chisq")

mosaic (Caesar.mod0s)

# what terms to add?
add1(Caesar.mod0, ~.^2, test="Chisq")

# add Association of Infection:Antibiotics
Caesar.mod1 <- update(Caesar.mod0, ~ . + Infection:Antibiotics)
anova(Caesar.mod0, Caesar.mod1, test="Chisq")

mosaic(Caesar.mod1,
       gp=shading_Friendly,
       main="Adding Infection:Antibiotics")


}
\seealso{
\code{\link[Fahrmeir]{caesar}} for the same data recorded as a
frequency data frame with other variables.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\name{datasets}
\alias{datasets}
\title{Information on Data Sets in Packages}
\usage{
datasets(
  package = "vcdExtra",
  allClass = FALSE,
  incPackage = length(package) > 1,
  maxTitle = NULL,
  ndim = FALSE
)
}
\arguments{
\item{package}{a character vector giving the package(s) to look in}

\item{allClass}{a logical variable. Include all classes of the item (\code{TRUE}) or just the last class (\code{FALSE})?}

\item{incPackage}{include the package name in result?}

\item{maxTitle}{maximum length of data set Title}

\item{ndim}{logical; include the number of dimensions in the result?}
}
\value{
A \code{data.frame} whose rows correspond to data sets found in \code{package}.

The columns (for a single package) are:
\item{Item}{data set name, a character variable}
\item{class}{class, the object class of the data set, typically one of \code{"data.frame"}, \code{"table"}, \code{"array"} ...}
\item{dim}{an abbreviation of the dimensions of the data set, in a form like \code{"36x3"} for a data.frame or matrix with 36 rows and 3 columns.}
\item{Title}{data set title}
}
\description{
The \code{\link[utils]{data}} function is used both to load data sets from
packages, and give a display of the names and titles of data sets in one or
more packages, however it does not return a result that can be easily used
to get additional information about the nature of data sets in packages.
}
\details{
The \code{datasets()} function is designed to produce a more useful summary
display of data sets in one or more packages.  It extracts the \code{class}
and dimension information (\code{dim} or codelength) of each item, and
formats these to provide additional descriptors.

The requested packages must be installed, and are silently loaded in order
to extract \code{class} and size information.
}
\note{
In Rmd documents, \code{datasets("package") |> knitr::kable()} can be used
to create a more pleasing display.
}
\examples{

datasets("vcdExtra")
# datasets(c("vcd", "vcdExtra"))
datasets("datasets", maxTitle=50)

# just list dataset names in a package
datasets("vcdExtra")[,"Item"]
datasets("vcd")[,"Item"]

# Find one-way tables in vcd (dim doesn't contain "x")
datasets("vcd") |>
  dplyr::filter(class=="table",
                !stringr::str_detect(dim, "x"))


}
\seealso{
\code{\link[utils]{data}}, \code{\link[knitr]{kable}}
}
\author{
Michael Friendly, with R-help from Curt Seeliger
}
\keyword{data}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{TV}
\alias{TV}
\title{TV Viewing Data}
\format{
A 5 x 11 x 3 array of cell frequencies with the following structure:
\preformatted{
  int [1:5, 1:11, 1:3] 146 244 233 174 294 151 181 161 183 281 ...
  - attr(*, "dimnames")=List of 3
  ..$ Day    : chr [1:5] "Monday" "Tuesday" "Wednesday" "Thursday" ...
  ..$ Time   : chr [1:11] "8:00" "8:15" "8:30" "8:45" ...
  ..$ Network: chr [1:3] "ABC" "CBS" "NBC"
}
}
\source{
The original data, \code{tv.dat}, came from the initial
implementation of mosaic displays in R by Jay Emerson (1998). Similar data
had been used by Hartigan and Kleiner (1984) as an illustration.
}
\description{
This data set \code{TV} comprises a 5 x 11 x 3 contingency table based on
audience viewing data from Neilsen Media Research for the week starting
November 6, 1995.
}
\details{
The original data, \code{tv.dat}, contains two additional networks: "Fox"
and "Other", with small frequencies. These levels were removed in the
current version. There is also a fourth factor, transition State transition
(turn the television Off, Switch channels, or Persist in viewing the current
channel). The \code{TV} data here includes only the Persist observations.
}
\examples{

data(TV)
structable(TV)
doubledecker(TV)

# reduce number of levels of Time
TV.df <- as.data.frame.table(TV)
levels(TV.df$Time) <- rep(c("8:00-8:59", "9:00-9:59", "10:00-10:44"),
                          c(4, 4, 3))

TV2 <- xtabs(Freq ~ Day + Time + Network, TV.df)

# re-label for mosaic display
levels(TV.df$Time) <- c("8", "9", "10")
# fit mode of joint independence, showing association of Network with Day*Time
mosaic(~ Day + Network + Time,
  data = TV.df,
  expected = ~ Day:Time + Network,
  legend = FALSE)


# with doubledecker arrangement
mosaic(~ Day + Network + Time,
  data = TV.df,
  expected = ~ Day:Time + Network,
  split = c(TRUE, TRUE, FALSE),
  spacing = spacing_highlighting,
  legend = FALSE)

}
\references{
Friendly, M. and Meyer, D. (2016).  \emph{Discrete Data Analysis
with R: Visualization and Modeling Techniques for Categorical and Count
Data}.  Boca Raton, FL: Chapman & Hall/CRC. \url{http://ddar.datavis.ca}.

Emerson, John W. Mosaic Displays in S-PLUS: A General Implementation and a
Case Study. \emph{Statistical Graphics and Computing Newsletter}, 1998,
9(1), 17--23, \url{http://www.stat.yale.edu/~jay/R/mosaic/v91.pdf}

Hartigan, J. A. & Kleiner, B. A Mosaic of Television Ratings. \emph{The
American Statistician}, 1984, 38, 32-35.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Asbestos}
\alias{Asbestos}
\title{Effect of Exposure to Asbestos}
\format{
The format is:
\preformatted{
 num [1:5, 1:4] 310 212 21 25 7 36 158 35 102 35 ...
 - attr(*, "dimnames")=List of 2
  ..$ exposure: chr [1:5] "0-9" "10-19" "20-29" "30-39" ...
  ..$ grade   : chr [1:4] "None" "Grade 1" "Grade 2" "Grade 3"#'
  }
}
\source{
Beh, E. J. & Lombardo, R. (2022). Features of the Polynomial Biplot
for Ordered Contingency Tables, \emph{Journal of Computational and Graphical
Statistics}, 31:2, 403-412, DOI: 10.1080/10618600.2021.1990773, Table 1.
}
\description{
A two-way contingency table formed from the cross-classification of the
number of years of occupational exposure to asbestos and the diagnosed
severity of asbestosis of 1117 New York workers. Asbestosis is a chronic
lung disease that results in the lung tissue being scared due to contact
with the fibers which can lead to severe breathing difficulties.
}
\details{
\code{exposure} and \code{grade} should be regarded as ordered factors. Beh
and Lombardo (2022) use this data to illustrate a polynomial biplot for
ordered categories.

The data summarized here was studied by Beh and Smith (2011) and comes from
the original data collected and published by Selikoff (1981) who examined
the link between asbestos exposure and asbestosis severity in 1963.
}
\examples{

data(Asbestos)
# mosaic plot
vcd::mosaic(Asbestos, shade=TRUE, legend=FALSE)

# do the correspondence analysis
library(ca)
Asbestos.ca <- ca(Asbestos)

plot(Asbestos.ca, lines=TRUE)


}
\references{
Beh, E. J., and D. R. Smith (2011b), Real World Occupational
Epidemiology, Part 2: A Visual Interpretation of Statistical Significance,
\emph{Archives of Environmental & Occupational Health}, \strong{66}, 245-248.

Selikoff, I. J. (1981), Household Risks With Inorganic Fibers,
\emph{Bulletin of the New York Academy of Medicine}, \strong{57}, 947-961.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Draft1970table}
\alias{Draft1970table}
\title{USA 1970 Draft Lottery Table}
\format{
The format is:

\preformatted{
'table' int [1:12, 1:3] 9 7 5 8 9 11 12 13 10 9 ...
- attr(*, "dimnames")=List of 2
..$ Month: chr [1:12] "Jan" "Feb" "Mar" "Apr" ...
..$ Risk : chr [1:3] "High" "Med" "Low"
}
}
\source{
This data is available in several forms, but the table version was
obtained from

\url{https://sas.uwaterloo.ca/~rwoldfor/software/eikosograms/data/draft-70}
}
\description{
This data set gives the results of the 1970 US draft lottery, in the form of
a frequency table. The rows are months of the year, Jan--Dec and columns
give the number of days in that month which fall into each of three draft
risk categories High, Medium, and Low, corresponding to the chances of being
called to serve in the US army.
}
\details{
The lottery numbers are divided into three categories of risk of being
called for the draft -- High, Medium, and Low -- each representing roughly
one third of the days in a year.  Those birthdays having the highest risk
have lottery numbers 1-122, medium risk have numbers 123-244, and the lowest
risk category contains lottery numbers 245-366.
}
\examples{

data(Draft1970table)
chisq.test(Draft1970table)

# plot.table -> graphics:::mosaicplot
plot(Draft1970table, shade=TRUE)
mosaic(Draft1970table, gp=shading_Friendly)

# correspondence analysis
if(require(ca)) {
  ca(Draft1970table)
  plot(ca(Draft1970table))
}

# convert to a frequency data frame with ordered factors
Draft1970df <- as.data.frame(Draft1970table)

Draft1970df <- within(Draft1970df, {
  Month <- ordered(Month)
  Risk <- ordered(Risk, levels=rev(levels(Risk)))
  })
str(Draft1970df)

# similar model, as a Poisson GLM
indep <- glm(Freq ~ Month + Risk, family = poisson, data = Draft1970df)

mosaic(indep, residuals_type="rstandard", gp=shading_Friendly)

# numeric scores for tests of ordinal factors
Cscore <- as.numeric(Draft1970df$Risk)
Rscore <- as.numeric(Draft1970df$Month)

# linear x linear association between Month and Risk
linlin <- glm(Freq ~ Month + Risk + Rscore:Cscore, family = poisson, data = Draft1970df)

# compare models
anova(indep, linlin, test="Chisq")
mosaic(linlin, residuals_type="rstandard", gp=shading_Friendly)



}
\references{
Fienberg, S. E. (1971), "Randomization and Social Affairs: The
1970 Draft Lottery," \emph{Science}, 171, 255-261.

Starr, N. (1997). Nonrandom Risk: The 1970 Draft Lottery, \emph{Journal of
Statistics Education}, v.5, n.2
\doi{10.1080/10691898.1997.11910534}
}
\seealso{
\code{\link{Draft1970}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logist_plot.R
\name{logist_plot}
\alias{logist_plot}
\alias{logist_plot.default}
\alias{logist_plot.data.frame}
\alias{logist_plot.formula}
\alias{logist_hist}
\alias{logist_point}
\alias{logist_density}
\title{Plot a fitted logistic regression with marginal distributions of the predictor}
\usage{
logist_plot(x, ...)

\method{logist_plot}{default}(
  x,
  y,
  marginal = c("hist", "points", "density"),
  bins = 30,
  adjust = 1,
  xlab = NULL,
  ylab = NULL,
  fit.color = "steelblue",
  marginal.color = "orange",
  fit.args = list(),
  marginal.args = list(),
  group = NULL,
  group.colors = NULL,
  marginal.height = NULL,
  ...
)

\method{logist_plot}{data.frame}(
  x,
  xvar = 1L,
  yvar = 2L,
  marginal = c("hist", "points", "density"),
  bins = 30,
  adjust = 1,
  xlab = NULL,
  ylab = NULL,
  fit.color = "steelblue",
  marginal.color = "orange",
  fit.args = list(),
  marginal.args = list(),
  group = NULL,
  group.colors = NULL,
  marginal.height = NULL,
  ...
)

\method{logist_plot}{formula}(
  formula,
  data,
  marginal = c("hist", "points", "density"),
  bins = 30,
  adjust = 1,
  xlab = NULL,
  ylab = NULL,
  fit.color = "steelblue",
  marginal.color = "orange",
  fit.args = list(),
  marginal.args = list(),
  group = NULL,
  group.colors = NULL,
  marginal.height = NULL,
  ...
)

logist_hist(...)

logist_point(...)

logist_density(...)
}
\arguments{
\item{x}{a numeric predictor vector or a data frame; see \code{formula} below for the
model-formula interface}

\item{...}{arguments passed to methods, or on to \code{logist_plot()} from the convenience
wrappers. Arguments not consumed by the selected method are an error rather than being
silently ignored. Use \code{fit.args} and \code{marginal.args} for layer customization.}

\item{y}{a binary (0/1, or 2-level factor/character/logical) response vector}

\item{marginal}{character string, how to represent the marginal distribution of \code{x} within
each \code{y} group: \code{"hist"}, mirrored histograms (default); \code{"points"}, jittered points; or
\code{"density"}, mirrored filled density estimates}

\item{bins}{number of histogram bins, for \code{marginal = "hist"}; default: 30}

\item{adjust}{positive numeric bandwidth adjustment passed to \code{\link[stats:density]{stats::density()}} for
\code{marginal = "density"}; default: 1}

\item{xlab, ylab}{axis labels; default to the deparsed \code{x}/\code{y} expressions}

\item{fit.color}{color of the fitted logistic curve and its confidence band; default:
"steelblue". This scalar is inactive when \code{group} is supplied; use \code{group.colors}
instead.}

\item{marginal.color}{color of the marginal representation of \code{x} within each \code{y} group
(histogram/density fill, or point color for \code{marginal = "points"}); default: "orange"
This scalar is inactive when \code{group} is supplied; use \code{group.colors} instead.}

\item{fit.args}{named list of graphical arguments for the fitted curve and confidence band.
Values override the defaults established by \code{fit.color}. The fit remains a binomial GLM,
so \code{data}, \code{mapping}, \code{stat}, \code{position}, \code{inherit.aes}, \code{method}, \code{formula}, and
\code{method.args} cannot be replaced. In grouped mode, \code{colour} and \code{fill} must instead be
controlled through \code{group.colors}.}

\item{marginal.args}{named list of graphical arguments for the active marginal layer.
Values override the defaults established by \code{marginal.color}. Valid arguments depend on
\code{marginal}: point aesthetics and \code{position} for \code{"points"}, rectangle aesthetics for
\code{"hist"}, or ribbon aesthetics for \code{"density"}. Histogram computation remains controlled
by \code{bins}; density bandwidth remains controlled by \code{adjust}; and marginal geometry height
remains controlled by \code{marginal.height}. In grouped mode, \code{colour} and \code{fill} must instead
be controlled through \code{group.colors}.}

\item{group}{optional grouping input. For the default method, a vector the same length as
\code{x} and \code{y}; for data-frame and formula methods, a single column name or position.
Grouping is supported for \code{marginal = "points"} and \code{"density"}, but not \code{"hist"}.}

\item{group.colors}{optional character vector of colours for grouped plots. An unnamed
vector is applied in group-level order; a named vector must contain every observed group
label. The same palette is used for fits, marginals, and the legend. The default \code{NULL}
uses ggplot2's discrete scales.}

\item{marginal.height}{optional finite positive number controlling marginal height. For
histograms and ungrouped densities, this is the maximum proportion of the fixed 0--1
probability panel occupied on each side; defaults are 0.25 and 0.15, respectively, and
values cannot exceed 0.5. For grouped densities, it is the full height of each outward
lane in the original probability-coordinate units; the default is 0.05 and values cannot
exceed 1. \code{NULL} selects the applicable default. For \code{marginal = "points"}, a non-\code{NULL}
value is ignored with a warning; control vertical jitter through \code{marginal.args} instead.}

\item{xvar, yvar}{which columns of \code{x} to use as predictor/response -- column name or
position; default to the first two columns (matches the original 2-column-data-frame
calling convention)}

\item{formula}{a model formula, \code{y ~ x} -- exactly one response and one predictor;
\code{formula} method only. The first argument may be passed positionally or as
\code{formula = y ~ x} (matching base R's \code{boxplot()}/\code{lm()} convention) -- unlike the other
methods, it is not named \code{x}}

\item{data}{a data frame -- \code{formula} method only}
}
\value{
A native \code{ggplot} object that can be extended with ordinary ggplot2 additions.
}
\description{
Plots predicted probabilities from a \code{glm(y ~ x, family = binomial)} fit for a single
quantitative predictor \code{x} and binary response \code{y}, and also with the smoothed
logistic fit and its confidence band.
What this plot method adds is a representation of
the marginal distribution of \code{x} within each \code{y} group -- mirrored histograms or filled
density estimates above and below the curve, or jittered points -- as suggested by Smart et
al. (2004). These help you
see where the data supporting the fit exist; e.g., where the data are "thin", so the confidence band is wide.
}
\details{
\code{logist_plot()} is generic, with methods for a pair of vectors, a data frame, or a
model formula. \code{logist_hist()}, \code{logist_point()}, and \code{logist_density()} are convenience
wrappers with \verb{marginal=} fixed to \code{"hist"}/\code{"points"}/\code{"density"}, but otherwise accept
the same \code{x}/\code{...} as \code{logist_plot()} -- i.e., they also work with a data frame or a formula.
An optional \code{group} produces separate fits and colour identities in point and density
modes. Grouped histograms are deliberately unsupported because stacked or overlapping
mirrored bars obscure both the distributions and the fitted curves.
Grouped density lanes begin at 0 and 1 and stack outward in fixed, narrow bands. Each
group's two response-specific densities are normalized together within that band, so the
shapes show conditional distributions but do not encode group sample sizes.

\code{bins}, \code{adjust}, and \code{marginal.height} control computation of the marginal geometry;
they are not graphical layer arguments and therefore belong at the top level rather than
inside \code{marginal.args}.
When a small histogram height would crowd the secondary count axis, its integer ticks are
thinned symmetrically while retaining the correct count-to-height mapping.

All marginal modes return a native ggplot object. Standard additions such as
\code{\link[ggplot2:labs]{ggplot2::labs()}} and \code{\link[ggplot2:theme]{ggplot2::theme()}} can therefore be applied after construction.
Adding another \verb{scale_y_*()} replaces the internally configured probability scale; in
histogram mode this can remove or invalidate the secondary count-axis mapping.
}
\examples{
data(Donner, package = "vcdExtra")

# three interfaces to the same underlying plot
logist_plot(Donner$age, Donner$survived, marginal = "points")
logist_plot(Donner[, c("age", "survived")], marginal = "hist")
logist_plot(survived ~ age, data = Donner, marginal = "density")

# post-hoc labels and themes work in histogram mode
logist_plot(survived ~ age, data = Donner, marginal = "hist") +
  ggplot2::labs(title = "Survival of the Donner Party") +
  ggplot2::theme(plot.title = ggplot2::element_text(face = "bold"))

# layer-specific customization; scoped lists override graphical defaults
logist_plot(
  survived ~ age, data = Donner, marginal = "hist",
  fit.args = list(linewidth = 2, fill = "lightblue"),
  marginal.args = list(colour = "black", linewidth = 0.2, alpha = 0.8)
)

# convenience wrappers -- marginal= fixed, still get all calling conventions
logist_point(survived ~ age, data = Donner)
logist_hist(survived ~ age, data = Donner)
logist_density(survived ~ age, data = Donner, adjust = 1.25)

# grouped fits and marginals; grouped density lanes extend outward from 0 and 1
logist_point(survived ~ age, data = Donner, group = "sex")
logist_density(
  survived ~ age, data = Donner, group = "sex",
  group.colors = c(Female = "#D55E00", Male = "#0072B2"),
  marginal.args = list(alpha = 0.35, linewidth = 0.6)
)

}
\references{
Smart, J. M. R., Sutherland, W. J., Watkinson, A. R., and Gill, J. A. (2004). A New Means of
Presenting the Results of Logistic Regression, \emph{Bulletin of the Ecological Society of
America}, 85(3), 100--102. \doi{10.1890/0012-9623(2004)85[100:ANMOPT]2.0.CO;2}
\url{https://esapubs.org/bulletin/backissues/085-3/bulletinjuly2004_2column.htm#tools1}
}
\seealso{
\code{\link[vcd:binreg_plot]{vcd::binreg_plot()}}, a similar plot using \code{grid} graphics directly.
}
\author{
Gavin Klorfine, Michael Friendly
}
\concept{logistic regression plots}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_model.R
\name{get_model}
\alias{get_model}
\alias{get_models}
\title{Extract Model Formulas from Model Objects or Model Lists}
\usage{
get_model(x, type = c("brackets", "formula"), abbrev = FALSE, ...)

get_models(x, type = c("brackets", "formula"), abbrev = FALSE, ...)
}
\arguments{
\item{x}{For \code{get_model()}: a \code{loglm} or \code{glm} object.
For \code{get_models()}: a \code{glmlist} or \code{loglmlist} object.}

\item{type}{Type of output: \code{"brackets"} for loglinear bracket notation
(e.g., \code{"[AB] [C]"}), or \code{"formula"} for R formula notation.
For \code{glm} and \code{glmlist} objects, only \code{"formula"} is meaningful.}

\item{abbrev}{Logical or integer. If \code{TRUE} or a positive integer,
abbreviate factor names to that many characters (default 1 when \code{TRUE}).
Only applies to bracket notation.}

\item{\dots}{Additional arguments passed to \code{loglin2string}
such as \code{sep} and \code{collapse}.}
}
\value{
For \code{get_model()}: a character string with the model formula or bracket notation.

\if{html}{\out{<div class="sourceCode">}}\preformatted{    For `get_models()`: a named character vector with the model formulas or bracket notations.
}\if{html}{\out{</div>}}
}
\description{
\code{get_model()} extracts the model formula or bracket notation from a single
\code{loglm} or \code{glm} object.
}
\details{
\code{get_models()} does the same for each model in a
\code{loglmlist} or \code{glmlist} object.
These are useful for labeling models in summaries and plots.

For \code{loglm} objects created by \code{\link{seq_loglm}}, the bracket
notation is stored in the \code{model.string} component. For other \code{loglm}
objects, it is constructed from the \code{margin} component using
\code{\link{loglin2string}}.

For \code{glm} objects, the formula is extracted using \code{formula()}.

\strong{Model notation}

For \code{loglmlist} objects created by \code{\link{seq_loglm}}, the bracket
notation distinguishes between models fit to marginal sub-tables and
models fit to the full table. Parentheses are used for marginal
sub-tables, e.g., \code{"(Class) (Sex)"}, while square brackets are used
for the full table, e.g., \code{"[Class,Sex,Age] [Survived]"}.

The \code{abbrev} argument can be used to abbreviate factor names
for more compact display, e.g., \code{"[C,S,A] [S]"}.
}
\examples{
data(Titanic)
tit.joint <- seq_loglm(Titanic, type = "joint")

# Single model
get_model(tit.joint[[2]])
get_model(tit.joint[[4]])
get_model(tit.joint[[4]], abbrev = TRUE)
get_model(tit.joint[[4]], type = "formula")

# Model list
get_models(tit.joint)
get_models(tit.joint, type = "formula")

# With abbreviated factor names
get_models(tit.joint, abbrev = TRUE)
get_models(tit.joint, abbrev = 2)

}
\seealso{
\code{\link{glmlist}}, \code{\link{loglmlist}},
\code{\link{loglin2string}}, \code{\link{LRstats}},
\code{\link{seq_mosaic}}

Other glmlist functions:
\code{\link[=Kway]{Kway()}},
\code{\link[=LRstats]{LRstats()}},
\code{\link[=glmlist]{glmlist()}},
\code{\link[=mosaic.glmlist]{mosaic.glmlist()}}

Other loglinear models:
\code{\link[=assoc_graph]{assoc_graph()}},
\code{\link[=glmlist]{glmlist()}},
\code{\link[=joint]{joint()}},
\code{\link[=plot.assoc_graph]{plot.assoc_graph()}},
\code{\link[=seq_loglm]{seq_loglm()}}
}
\concept{glmlist functions}
\concept{loglinear models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{CyclingDeaths}
\alias{CyclingDeaths}
\title{London Cycling Deaths}
\format{
A data frame with 208 observations on the following 2 variables.
\describe{
\item{\code{date}}{a Date}
\item{\code{deaths}}{number of deaths, a numeric vector}
}
}
\source{
\url{https://www.data.gov.uk/dataset/cb7ae6f0-4be6-4935-9277-47e5ce24a11f/road-accidents-safety-data},
STATS 19 data, 2005-2012, using the files \code{Casualty0512.csv} and
\code{Accidents0512.csv}
}
\description{
A data frame containing the number of deaths of cyclists in London from 2005
through 2012 in each fortnightly period.  Aberdein & Spiegelhalter (2013)
discuss these data in relation to the observation that six cyclists died in
London between Nov. 5 and Nov. 13, 2013.
}
\examples{

data(CyclingDeaths)

plot(deaths ~ date, data=CyclingDeaths,
  type="h",
	lwd=3,
	ylab="Number of deaths",
	axes=FALSE)
axis(1, at=seq(as.Date('2005-01-01'),
               by='years',
               length.out=9),
     labels=2005:2013)
axis(2, at=0:3)

# make a one-way frequency table
CyclingDeaths.tab <- table(CyclingDeaths$deaths)

gf <- goodfit(CyclingDeaths.tab)
gf
summary(gf)

rootogram(gf, xlab="Number of Deaths")
distplot(CyclingDeaths.tab)

# prob of 6 or more deaths in one fortnight
lambda <- gf$par$lambda
ppois(5, lambda, lower.tail=FALSE)

}
\references{
Aberdein, Jody and Spiegelhalter, David (2013). Have London's
roads become more dangerous for cyclists? \emph{Significance}, 10(6),
46--48.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/breslow_day_test.R
\name{breslow_day_test}
\alias{breslow_day_test}
\alias{print.breslow_day_test}
\title{Breslow-Day Test for Homogeneity of Odds Ratios}
\usage{
breslow_day_test(x, OR = NA, correct = FALSE, decompose = FALSE)

\method{print}{breslow_day_test}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{"breslow_day_test"}}

\item{OR}{The common odds ratio to test against. If \code{NA} (the default), the
Mantel-Haenszel estimate is used.}

\item{correct}{Logical. If \code{TRUE}, the Tarone (1985) correction is applied.
Defaults to \code{FALSE}.}

\item{decompose}{Logical. If \code{TRUE} and \code{x} is 4-dimensional
(a \eqn{2 \times 2 \times R \times C}{2 x 2 x R x C} table), the test is
decomposed into row effects, column effects, and residual. Defaults to \code{FALSE}.
Ignored for non-4-dimensional tables.}

\item{digits}{Number of significant digits for the common OR. Default 4.}

\item{...}{Additional arguments (currently unused).}
}
\value{
A list of class \code{"breslow_day_test"} (also inheriting from \code{"htest"})
containing:
\item{statistic}{the chi-squared test statistic.}
\item{parameter}{degrees of freedom.}
\item{p.value}{\eqn{p}-value.}
\item{method}{character string describing the test.}
\item{data.name}{character string giving the name of the data.}
\item{or_vars}{names of the first two dimensions (the 2x2 table variables).}
\item{strata_vars}{names of the stratifying variables (dimensions 3 and beyond).}
\item{OR}{the common odds ratio used (MH estimate if \code{OR = NA} was supplied).}
\item{correct}{logical indicating whether Tarone correction was applied.}
\item{observed}{observed \eqn{a_j} counts (cell \eqn{[1,1]} of each stratum).}
\item{expected}{expected \eqn{\tilde{a}_j} counts under the common OR.}
\item{decomposed}{logical indicating if decomposition was performed.}

When \code{decompose = TRUE} (only for 4-dimensional tables), additional components:
\item{rows}{list with \code{statistic}, \code{df}, \code{p.value} for row effects.}
\item{cols}{list with \code{statistic}, \code{df}, \code{p.value} for column effects.}
\item{residual}{list with \code{statistic}, \code{df}, \code{p.value} for residual (interaction).}
}
\description{
Tests for homogeneity of odds ratios across strata in \eqn{2 \times 2 \times k}{2 x 2 x k}
tables (i.e., whether a common odds ratio fits all strata). Generalized to handle tables
of any dimensionality beyond 3. For 4-dimensional tables, optionally provides a two-way
decomposition of the homogeneity test into row effects, column effects, and residual,
analogous to \code{\link[=woolf_test]{woolf_test()}} with \code{decompose = TRUE}.
}
\details{
The Breslow-Day test (Breslow & Day, 1980) tests the hypothesis that a common odds ratio
\eqn{\psi} fits all \eqn{k} strata. Given a common OR (by default the Mantel-Haenszel
estimate), the expected cell count \eqn{\tilde{a}_j} in cell \eqn{[1,1]} of stratum
\eqn{j} is found by solving the quadratic:

\deqn{(\psi - 1)\tilde{a}_j^2 - [n_{2j} - m_{1j} + \psi(n_{1j} + m_{1j})]\tilde{a}_j
      + \psi \, m_{1j} n_{1j} = 0}

where \eqn{m_{1j}} and \eqn{m_{2j}} are the row margins and \eqn{n_{1j}} and
\eqn{n_{2j}} are the column margins of stratum \eqn{j}. The test statistic is:

\deqn{\chi^2_{BD} = \sum_{j=1}^{k} \frac{(a_j - \tilde{a}_j)^2}{\widehat{\text{Var}}(a_j)}}

where \eqn{\widehat{\text{Var}}(a_j) = (1/\tilde{a}_j + 1/\tilde{b}_j + 1/\tilde{c}_j +
1/\tilde{d}_j)^{-1}} and \eqn{\tilde{b}_j, \tilde{c}_j, \tilde{d}_j} are the remaining
expected cell counts. Under the null hypothesis, \eqn{\chi^2_{BD}} follows a chi-squared
distribution with \eqn{k - 1} degrees of freedom.

The Tarone (1985) correction subtracts a term to account for estimation of the common OR:

\deqn{\chi^2_{BD,T} = \chi^2_{BD} -
      \frac{(\sum_j a_j - \sum_j \tilde{a}_j)^2}{\sum_j \widehat{\text{Var}}(a_j)}}

\strong{Comparison with the Woolf test:}
The Woolf test uses log odds ratios and tests deviation from their weighted mean,
whereas the Breslow-Day test works on the cell-count scale against a specified common OR.
For large samples they agree closely; Breslow-Day is generally preferred when the
Mantel-Haenszel common OR is the quantity of interest.

\strong{Decomposition for 4-way tables:}
For a \eqn{2 \times 2 \times R \times C} table, when \code{decompose = TRUE}, the overall
test is decomposed as:

\deqn{\chi^2_{\text{Total}} = \chi^2_{\text{Rows}} + \chi^2_{\text{Cols}} +
      \chi^2_{\text{Residual}}}

where the row and column components use the row- and column-marginal (pooled) tables,
all tested against the same common OR as the overall test. The residual is defined by
subtraction and has \eqn{(R-1)(C-1)} degrees of freedom.
}
\examples{
# 3-way table
data(CoalMiners, package = "vcd")
breslow_day_test(CoalMiners)
breslow_day_test(CoalMiners, correct = TRUE)   # Tarone correction

# Compare with Woolf test
woolf_test(CoalMiners)

data(Heart, package = "vcdExtra")
breslow_day_test(Heart)

# 4-way table without decomposition
data(Fungicide, package = "vcdExtra")
breslow_day_test(Fungicide)

# 4-way table with decomposition
breslow_day_test(Fungicide, decompose = TRUE)

}
\references{
Breslow, N. E. & Day, N. E. (1980). \emph{Statistical Methods in Cancer Research.
Vol. 1: The Analysis of Case-Control Studies}. IARC Scientific Publications No. 32.
Lyon: International Agency for Research on Cancer.

Tarone, R. E. (1985). On heterogeneity tests based on efficient scores.
\emph{Biometrika}, \bold{72}, 91-95.

Lachin, J. M. (2000). \emph{Biostatistical Methods: The Assessment of Relative Risks}.
Wiley, p. 124-125.
}
\seealso{
\code{\link[stats:mantelhaen.test]{stats::mantelhaen.test()}}, \code{\link[=woolf_test]{woolf_test()}}, \code{\link[DescTools:BreslowDayTest]{DescTools::BreslowDayTest()}}

Other association tests:
\code{\link[=CMHtest]{CMHtest()}},
\code{\link[=GKgamma]{GKgamma()}},
\code{\link[=HLtest]{HLtest()}},
\code{\link[=woolf_test]{woolf_test()}},
\code{\link[=zero.test]{zero.test()}}
}
\author{
Andri Signorell, Michael Friendly (enhancements)
}
\concept{association tests}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Geissler}
\alias{Geissler}
\title{Geissler's Data on the Human Sex Ratio}
\format{
A data frame with 90 observations on the following 4 variables.  The
rows represent the non-NA entries in Edwards' table.
\describe{
\item{\code{boys}}{number of boys in the family, \code{0:12}}
\item{\code{girls}}{number of girls in the family, \code{0:12}}
\item{\code{size}}{family size: \code{boys+girls}}
\item{\code{Freq}}{number of families with this sex composition}
}
}
\source{
Edwards, A. W. F. (1958). An Analysis Of Geissler's Data On The
Human Sex Ratio. \emph{Annals of Human Genetics}, 23, 6-15.
}
\description{
Geissler (1889) published data on the distributions of boys and girls in
families in Saxony, collected for the period 1876-1885. The \code{Geissler}
data tabulates the family composition of 991,958 families by the number of
boys and girls listed in the table supplied by Edwards (1958, Table 1).
}
\details{
The data on family composition was available because, on the birth of a
child, the parents had to state the sex of all their children on the birth
certificate. These family records are not necessarily independent, because a
given family may have had several children during this 10 year period,
included as multiple records.
}
\examples{

data(Geissler)
str(Geissler)

# reproduce Saxony data, families of size 12
Saxony12 <- subset(Geissler, size==12, select=c(boys, Freq))
rownames(Saxony12)<-NULL

# make a 1-way table
xtabs(Freq~boys, Saxony12)

# extract data for other family sizes
Saxony11 <- subset(Geissler, size==11, select=c(boys, Freq))
rownames(Saxony11)<-NULL

Saxony10 <- subset(Geissler, size==10, select=c(boys, Freq))
rownames(Saxony10)<-NULL


}
\references{
Friendly, M. and Meyer, D. (2016).  \emph{Discrete Data Analysis
with R: Visualization and Modeling Techniques for Categorical and Count
Data}.  Boca Raton, FL: Chapman & Hall/CRC. \url{http://ddar.datavis.ca}.

Geissler, A. (1889). \emph{Beitrage zur Frage des Geschlechts verhaltnisses
der Geborenen} Z. K. Sachsischen Statistischen Bureaus, 35, n.p.

Lindsey, J. K. & Altham, P. M. E. (1998).  Analysis of the human sex ratio
by using overdispersion models. \emph{Journal of the Royal Statistical
Society: Series C (Applied Statistics)}, 47, 149-157.
}
\seealso{
\code{\link[vcd]{Saxony}}, containing the data for families of size
12.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Gilby}
\alias{Gilby}
\title{Clothing and Intelligence Rating of Children}
\format{
A 2-dimensional array resulting from cross-tabulating 2 variables
for 1725 observations. The variable names and their levels are:

\tabular{rll}{
No \tab Name \tab Levels \cr
1\tab \code{Dullness}\tab \verb{"Ment. defective", "Slow", "Slow Intell", "Fairly Intell", "Capable", "V.Able"}\cr
2\tab \code{Clothing}\tab \verb{"V.Well clad", "Well clad", "Passable", "Insufficient"}\cr
}
}
\source{
Anscombe, F. J. (1981). \emph{Computing in Statistical Science Through APL}. New York: Springer-Verlag, p. 302
}
\description{
Schoolboys were classified according to their clothing and to their teachers
rating of "dullness" (lack of intelligence), in a 5 x 7 table originally
from Gilby (1911). Anscombe (1981) presents a slightly collapsed 4 x 6
table, used here, where the last two categories of clothing were pooled as
were the first two categories of dullness due to small counts.
}
\details{
Both \code{Dullness} and \code{Clothing} are ordered categories, so models
and methods that examine their association in terms of ordinal categories
are profitable.
}
\examples{

data(Gilby)

# CMH tests treating row/column variables as ordinal
CMHtest(Gilby)

mosaic(Gilby, shade=TRUE)

# correspondence analysis to see relations among categories
if(require(ca)){
	ca(Gilby)
	plot(ca(Gilby), lines=TRUE)

}



}
\references{
Gilby, W. H. (1911).
On the significance of the teacher's appreciation of general
intelligence.  \emph{Biometrika}, 8, 93-108 (esp. p. 94).  (Quoted by Kendall (1943,..., 1953) Table 13.1, p 320.)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Depends}
\alias{Depends}
\title{Dependencies of R Packages}
\format{
The format is a one-way frequency table of counts of packages with
0, 1, 2, ... dependencies.

\preformatted{
 table' int [1:15(1d)] 986 1347 993 685 375 298 155 65 32 19 ...
 - attr(*, "dimnames")=List of 1
 ..$ Depends: chr [1:15] "0" "1" "2" "3" ...
}
}
\source{
Using code from
\url{https://www.r-bloggers.com/2013/12/a-look-at-the-distribution-of-r-package-dependencies/}
}
\description{
This one-way table gives the type-token distribution of the number of
dependencies declared in 4983 packages listed on CRAN on January 17, 2014.
}
\examples{

data(Depends)
plot(Depends,
     xlab="Number of Dependencies",
     ylab="Number of R Packages",
     lwd=8)

# what type of distribution?
# Ord_plot can't classify this!
Ord_plot(Depends)

\dontrun{
# The code below, from Joseph Rickert, downloads and tabulates the data
p <- as.data.frame(available.packages(),stringsAsFactors=FALSE)
names(p)

pkgs <- data.frame(p[,c(1,4)])                  # Pick out Package names and Depends
row.names(pkgs) <- NULL                         # Get rid of row names
pkgs <- pkgs[complete.cases(pkgs[,2]),]         # Remove NAs

pkgs$Depends2 <-strsplit(pkgs$Depends,",")      # split list of Depends
pkgs$numDepends <- as.numeric(lapply(pkgs$Depends2,length)) # Count number of dependencies in list
zeros <- c(rep(0,dim(p)[1] - dim(pkgs)[1]))     # Account for packages with no dependencies
Deps <- as.vector(c(zeros,pkgs$numDepends))     # Set up to tablate
Depends <- table(Deps)

}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_array.R
\name{as_array}
\alias{as_array}
\title{Convert frequency, case, or table form data into an array}
\usage{
as_array(obj, freq = NULL, dims = NULL, prop = NULL)
}
\arguments{
\item{obj}{Object to be converted to an array.}

\item{freq}{If \code{obj} is in frequency form, this is the name of the frequency column.
Leave as \code{NULL} if \code{obj} is in any other form.}

\item{dims}{A character vector of dimensions. If not specified, all variables apart from
\code{freq} will be used as dimensions.}

\item{prop}{If set to \code{TRUE}, returns an array of proportions (that sum to 1). May also
be set to a character or numeric vector of dimensions to be used as margins
from which proportions will be computed.}
}
\value{
object in array form
}
\description{
Converts object (\code{obj}) in frequency, case or table form into an array. The
column containing the frequencies (\code{freq}) must be supplied if \code{obj} is in
frequency form.
}
\details{
Unclasses the \code{\link{as_table}} function to return an object in array form.
}
\examples{
library(vcdExtra)

data("HairEyeColor")

freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
caseForm <- expand.dft(freqForm) # Generate case form data

# Frequency form -> array form
as_array(freqForm, freq = "Freq") |> str()

# Warned if forgot to specify freq
as_array(freqForm) |> str()

# Case form -> array form
as_array(caseForm) |> str()

# Frequency (tibble) form -> array form
as_array(tidy_freqForm, freq = "n") |> str()

# For specific dimensions
as_array(tidy_freqForm, freq = "n", dims = c("Hair", "Eye")) |> str()

#-----For proportions-----#

as_array(freqForm, freq = "Freq", prop = TRUE) |> # proportions relative to grand total
  head(c(4,4,1)) 

# Marginalize proportions along "Sex" (i.e., male proportions sum to 1, female proportions sum to 1)
as_array(freqForm, freq = "Freq", prop = "Sex") |> head(c(4,4,1))

as_array(freqForm, freq = "Freq", prop = 3) |> head(c(4,4,1)) # Same as above

# Marginalize proportions along multiple variables
as_array(freqForm, freq = "Freq", prop = c("Hair", "Sex")) |> head(c(4,4,1))

as_array(freqForm, freq = "Freq", prop = c(1, 3)) |> head(c(4,4,1)) # Same as above

# Using dims and prop arguments in tandem
as_array(freqForm, freq = "Freq", dims = c("Hair", "Eye"), prop = TRUE)


}
\seealso{
\code{\link{as_table}}, \code{\link{as_freqform}}, \code{\link{as_caseform}},
\code{\link{as_matrix}}
}
\author{
Gavin M. Klorfine
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{WorkerSat}
\alias{WorkerSat}
\title{Worker Satisfaction Data}
\format{
A frequency data frame with 8 observations on the following 4
variables, representing the 2 x 2 x 2 classification of 715 cases.
\describe{
\item{\code{Manage}}{Quality of management, an ordered factor with levels \code{bad} < \code{good}}
\item{\code{Super}}{Supervisor satisfaction, an ordered factor with levels \code{low} < \code{high}}
\item{\code{Worker}}{Worker job satisfaction, an ordered factor with levels \code{low} < \code{high}}
\item{\code{Freq}}{a numeric vector}
}
}
\source{
Originally from \url{https://online.stat.psu.edu/stat504/Lesson10}
}
\description{
Blue collar workers job satisfaction from large scale investigation in
Denmark in 1968 (Andersen, 1991).
}
\examples{

data(WorkerSat)

worker.tab <- xtabs(Freq ~ Worker + Super + Manage, data=WorkerSat)
fourfold(worker.tab)
mosaic(worker.tab, shade=TRUE)


}
\references{
Andersen, E. B. (1991) Statistical Analysis of Categorical Data,
2nd Ed., Springer-Verlag.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Hoyt}
\alias{Hoyt}
\title{Minnesota High School Graduates}
\format{
A 4-dimensional array resulting from cross-tabulating 4 variables
for 13968 observations. The variable names and their levels are:

\tabular{rll}{
No \tab Name \tab Levels \cr
1\tab \code{Status}\tab \verb{"College", "School", "Job", "Other"}\cr
2\tab \code{Rank}\tab \verb{"Low", "Middle", "High"}\cr
3\tab \code{Occupation}\tab \verb{"1", "2", "3", "4", "5", "6", "7"}\cr
4\tab \code{Sex}\tab \verb{"Male", "Female"}\cr
}
}
\source{
Fienberg, S. E. (1980). \emph{The Analysis of Cross-Classified Categorical
Data}. Cambridge, MA: MIT Press, p. 91-92.

R. L. Plackett, (1974). \emph{The Analysis of Categorical Data}. London: Griffin.
}
\description{
Minnesota high school graduates of June 1930 were classified with respect to
(a) \code{Rank} by thirds in their graduating class, (b) post-high school
\code{Status} in April 1939 (4 levels), (c) \code{Sex}, (d) father's
\code{Occupation}al status (7 levels, from 1=High to 7=Low).
}
\details{
The data were first presented by Hoyt et al. (1959) and have been analyzed
by Fienberg(1980), Plackett(1974) and others.

Post high-school \code{Status} is natural to consider as the response.
\code{Rank} and father's \code{Occupation} are ordinal variables.
}
\examples{

data(Hoyt)

# display the table
structable(Status + Sex ~ Rank + Occupation, data=Hoyt)

# mosaic for independence model
plot(Hoyt, shade=TRUE)

# examine all pairwise mosaics
pairs(Hoyt, shade=TRUE)

# collapse Status to College vs. Non-College
Hoyt1 <- collapse.table(Hoyt, Status=c("College", rep("Non-College",3)))
plot(Hoyt1, shade=TRUE)

#################################################
# fitting models with loglm, plotting with mosaic
#################################################

# fit baseline log-linear model for Status as response
require(MASS)
hoyt.mod0 <- loglm(~ Status + (Sex*Rank*Occupation),
  data=Hoyt1)
hoyt.mod0

mosaic(hoyt.mod0,
  gp=shading_Friendly,
  main="Baseline model: Status + (Sex*Rank*Occ)")

# add one-way association of Status with factors
hoyt.mod1 <- loglm(~ Status * (Sex + Rank + Occupation) + (Sex*Rank*Occupation),
  data=Hoyt1)
hoyt.mod1

mosaic(hoyt.mod1,
  gp=shading_Friendly,
  main="Status * (Sex + Rank + Occ)")

# can we drop any terms?
drop1(hoyt.mod1, test="Chisq")

# assess model fit
anova(hoyt.mod0, hoyt.mod1)

# what terms to add?
add1(hoyt.mod1, ~.^2, test="Chisq")

# add interaction of Sex:Occupation on Status
hoyt.mod2 <- update(hoyt.mod1, ~ . + Status:Sex:Occupation)

mosaic(hoyt.mod2,
  gp=shading_Friendly,
  main="Adding Status:Sex:Occupation")

# compare model fits
anova(hoyt.mod0, hoyt.mod1, hoyt.mod2)

# Alternatively, try stepwise analysis, heading toward the saturated model
steps <- step(hoyt.mod0,
  direction="forward",
  scope=~Status*Sex*Rank*Occupation)

# display anova
steps$anova


}
\references{
Hoyt, C. J., Krishnaiah, P. R. and Torrance, E. P. (1959)
Analysis of complex contingency tables, \emph{Journal of Experimental
Education} 27, 187-194.
}
\seealso{
\code{\link[MASS]{minn38}} provides the same data as a data frame.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ICU}
\alias{ICU}
\title{ICU data set}
\format{
A data frame with 200 observations on the following 22 variables.
\describe{
\item{\code{died}}{Died before discharge?, a factor with levels \code{No} \code{Yes}}
\item{\code{age}}{Patient age, a numeric vector}
\item{\code{sex}}{Patient sex, a factor with levels \code{Female} \code{Male}}
\item{\code{race}}{Patient race, a factor with levels \code{Black} \code{Other} \code{White}.  Also represented here as \code{white}.}
\item{\code{service}}{Service at ICU Admission, a factor with levels \code{Medical} \code{Surgical}}
\item{\code{cancer}}{Cancer part of present problem?, a factor with levels \code{No} \code{Yes}}
\item{\code{renal}}{History of chronic renal failure?, a factor with levels \code{No} \code{Yes}}
\item{\code{infect}}{Infection probable at ICU admission?, a factor with levels \code{No} \code{Yes}}
\item{\code{cpr}}{Patient received CPR prior to ICU admission?, a factor with levels \code{No} \code{Yes}}
\item{\code{systolic}}{Systolic blood pressure at admission (mm Hg), a numeric vector}
\item{\code{hrtrate}}{Heart rate at ICU Admission (beats/min), a numeric vector}
\item{\code{previcu}}{Previous admission to an ICU within 6 Months?, a factor with levels \code{No} \code{Yes}}
\item{\code{admit}}{Type of admission, a factor with levels \code{Elective} \code{Emergency}}
\item{\code{fracture}}{Admission with a long bone, multiple, neck, single area, or hip fracture?
a factor with levels \code{No} \code{Yes}}
\item{\code{po2}}{PO2 from initial blood gases, a factor with levels \verb{>60} \verb{<=60}}
\item{\code{ph}}{pH from initial blood gases, a factor with levels \verb{>=7.25} \verb{<7.25}}
\item{\code{pco}}{PCO2 from initial blood gases, a factor with levels \verb{<=45} \verb{>45}}
\item{\code{bic}}{Bicarbonate (HCO3) level from initial blood gases, a factor with levels \verb{>=18} \verb{<18}}
\item{\code{creatin}}{Creatinine, from initial blood gases, a factor with levels \verb{<=2} \verb{>2}}
\item{\code{coma}}{Level of unconsciousness at admission to ICU,	a factor with levels \code{None} \code{Stupor} \code{Coma}}
\item{\code{white}}{a recoding of \code{race},  a factor with levels \code{White} \code{Non-white}}
\item{\code{uncons}}{a recoding of \code{coma} a factor with levels \code{No} \code{Yes}}
}
}
\source{
M. Friendly (2000), \emph{Visualizing Categorical Data}, Appendix
B.4. SAS Institute, Cary, NC.

Hosmer, D. W. Jr., Lemeshow, S. and Sturdivant, R. X. (2013) \emph{Applied
Logistic Regression}, NY: Wiley, Third Edition.
}
\description{
The ICU data set consists of a sample of 200 subjects who were part of a
much larger study on survival of patients following admission to an adult
intensive care unit (ICU), derived from Hosmer, Lemeshow and Sturdivant
(2013) and Friendly (2000).
}
\details{
The major goal of this study was to develop a logistic regression model to
predict the probability of survival to hospital discharge of these patients
and to study the risk factors associated with ICU mortality. The clinical
details of the study are described in Lemeshow, Teres, Avrunin, and Pastides
(1988).

This data set is often used to illustrate model selection methods for
logistic regression.

Patient ID numbers are the rownames of the data frame.

Note that the last two variables \code{white} and \code{uncons} are a
recoding of respectively \code{race} and \code{coma} to binary variables.
}
\examples{

data(ICU)
# remove redundant variables (race, coma)
ICU1 <- ICU[,-c(4,20)]

# fit full model
icu.full <- glm(died ~ ., data=ICU1, family=binomial)
summary(icu.full)

# simpler model (found from a "best" subsets procedure)
icu.mod1 <- glm(died ~ age + sex + cancer + systolic + admit + uncons,
  data=ICU1,
  family=binomial)
summary(icu.mod1)

# even simpler model
icu.mod2 <- glm(died ~ age + cancer  + admit + uncons,
  data=ICU1,
  family=binomial)
summary(icu.mod2)

anova(icu.mod2, icu.mod1, icu.full, test="Chisq")

## Reproduce Fig 6.12 from VCD

icu.fit <- data.frame(ICU, prob=predict(icu.mod2, type="response"))

# combine categorical risk factors to a single string
risks <- ICU[, c("cancer", "admit", "uncons")]
risks[,1] <- ifelse(risks[,1]=="Yes", "Cancer", "")
risks[,2] <- ifelse(risks[,2]=="Emergency", "Emerg", "")
risks[,3] <- ifelse(risks[,3]=="Yes", "Uncons", "")
risks <- apply(risks, 1, paste, collapse="")
risks[risks==""] <- "(none)"
icu.fit$risks <- risks

if (require("ggplot2")) {
print(ggplot(icu.fit, aes(x=age, y=prob, color=risks)) +
	geom_point(size=2) +
	geom_line(size=1.25, alpha=0.5) +
	theme_bw() + ylab("Probability of death"))
}


}
\references{
Lemeshow, S., Teres, D., Avrunin, J. S., Pastides, H. (1988). Predicting the
Outcome of Intensive Care Unit Patients. \emph{Journal of the American
Statistical Association}, 83, 348-356.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{HairEyePlace}
\alias{HairEyePlace}
\title{Hair Color and Eye Color in Caithness and Aberdeen}
\format{
The format is:
\preformatted{
  num [1:4, 1:5, 1:2] 326 688 343 98 38 116 84 48 241 584 ...
- attr(*, "dimnames")=List of 3
..$ Eye  : chr [1:4] "Blue" "Light" "Medium" "Dark"
..$ Hair : chr [1:5] "Fair" "Red" "Medium" "Dark" ...
..$ Place: chr [1:2] "Caithness" "Aberdeen"
}
}
\source{
This data was taken from the \code{colors} data in \pkg{logmult}.
}
\description{
A three-way frequency table crossing eye color and hair color in two places,
Caithness and Aberdeen, Scotland. These data were of interest to Fisher
(1940) and others because there are mixtures of people of Nordic, Celtic and
Anglo-Saxon origin.
}
\details{
One or both tables have been widely analyzed in conjunction with RC and
canonical correlation models for categorical data, e.g., Becker and Clogg
(1989).

The hair and eye colors are ordered as in the original source, suggesting
that they form ordered categories.
}
\examples{

data(HairEyePlace)

# separate mosaics
mosaic(HairEyePlace[,,1], shade=TRUE, main="Caithness")
mosaic(HairEyePlace[,,2], shade=TRUE, main="Aberdeen")

# condition on Place
mosaic(~Hair + Eye |Place, data=HairEyePlace, shade=TRUE, legend=FALSE)

cotabplot(~Hair+Eye|Place, data=HairEyePlace, shade=TRUE, legend=FALSE)

}
\references{
Becker, M. P., and Clogg, C. C. (1989).  Analysis of Sets of
Two-Way Contingency Tables Using Association Models.  \emph{Journal of the
American Statistical Association}, 84(405), 142-151.

Fisher, R.A. (1940) The precision of discriminant functions.  \emph{Annals
of Eugenics}, 10, 422-429.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/labeling_points.R
\name{labeling_points}
\alias{labeling_points}
\title{Labeling Function for Mosaic Displays Using Points}
\usage{
labeling_points(
  labels = TRUE,
  varnames = labels,
  value_type = c("observed", "expected"),
  scale = 1,
  pch = 19,
  size = unit(0.5, "char"),
  gp_points = gpar(col = "black", alpha = 0.7),
  margin = unit(0.05, "npc"),
  seed = NULL,
  jitter = 1,
  clip = FALSE,
  ...
)
}
\arguments{
\item{labels}{Logical vector or scalar indicating whether labels should be
drawn for the table dimensions via \code{\link[vcd]{labeling_border}}.
Defaults to \code{TRUE}.}

\item{varnames}{Logical vector or scalar indicating whether variable names
should be drawn. Defaults to \code{labels}.}

\item{value_type}{Character string specifying whether to display
\code{"observed"} or \code{"expected"} frequencies as points.}

\item{scale}{Numeric scaling factor. The number of points drawn equals
\code{round(frequency / scale)}. Use larger values for tables with
large counts.}

\item{pch}{Point character (plotting symbol). Default is 19 (filled circle).}

\item{size}{Point size as a \code{\link[grid]{unit}} object.
Default is \code{unit(0.5, "char")}.}

\item{gp_points}{A \code{\link[grid]{gpar}} object controlling point appearance
(color, alpha, etc.), for example: \code{gp_points = gpar(col = "red")}.}

\item{margin}{Margin inside cells as a \code{\link[grid]{unit}} object.
Points are drawn within this inset area. Default is \code{unit(0.05, "npc")}.}

\item{seed}{Optional integer seed for reproducible point placement across several similar plots.
If \code{NULL} (default), no seed is set.}

\item{jitter}{Numeric jitter amount (0-1) for point placement.
Default is 1 (full random). Values < 1 create more regular patterns.}

\item{clip}{Logical indicating whether to clip points at cell boundaries.
Default is \code{FALSE}.}

\item{...}{Additional arguments passed to \code{\link[vcd]{labeling_border}}
for axis labels.}
}
\value{
A function of class \code{"grapcon_generator"} suitable for use
as the \code{labeling} argument in \code{\link[vcd]{strucplot}} and
related functions like \code{\link[vcd]{mosaic}}.
}
\description{
This labeling function for use with \code{\link[vcd]{strucplot}} displays,
such as \code{\link[vcd]{mosaic}}, draws
random points within each cell of a mosaic plot, where the number of points
represents observed or expected frequencies. This creates a "dot-density"
visualization that provides a direct visual representation of cell frequencies.
}
\details{
This function follows the "grapcon_generator" pattern used by vcd labeling
functions. It returns a function that can be passed to the \code{labeling}
argument of \code{\link[vcd]{strucplot}}, \code{\link[vcd]{mosaic}}, or
related functions as the argument \code{labeling = labeling_poionts(...)}.

The visualization is inspired by the conceptual model described in
Friendly (1995), where cell frequencies are represented as physical
counts of objects. Under independence, point density would be uniform
across cells (adjusted for marginals), so departures from independence
become visible as density variations.

This approach is related to sieve diagrams (\code{\link[vcd]{sieve}}),
which also use density to represent association.
}
\examples{
library(vcd)

# 2-way table of Hair and Eye color
HairEye <- margin.table(HairEyeColor, 2:1)

# Basic usage - observed frequencies as points
mosaic(HairEye,
       labeling = labeling_points(scale = 1))

# Show expected frequencies instead of observed
mosaic(HairEye,
       labeling = labeling_points(
                     value_type = "expected",
                     scale = 2,
                     seed = 42)
      )

# Combine with residual shading
mosaic(HairEye,
       shade = TRUE, legend = FALSE,
       labeling = labeling_points(scale = 2, seed = 42))

# Make tiles show expected frequencies, show points for observed frequencies
# Reproduces: Fig 6 in Friendly (1995)
mosaic(HairEye,
       type = "expected",
       shade = TRUE, legend = FALSE,
       labeling = labeling_points(scale = 2, seed = 42))

}
\references{
Friendly, M. (1995). Conceptual and Visual Models for Categorical Data.
\emph{The American Statistician}, \strong{49}, 153-160.
\doi{10.1080/00031305.1995.10476131}

Friendly, M. (1997). Conceptual Models for Visualizing Contingency Table Data.
In M. Greenacre & J. Blasius (Eds.), \emph{Visualization of Categorical Data} (pp. 17–35).
Academic Press. \url{https://www.datavis.ca/papers/koln/kolnpapr.pdf}

Meyer, D., Zeileis, A., and Hornik, K. (2006).
The Strucplot Framework: Visualizing Multi-way Contingency Tables with vcd.
\emph{Journal of Statistical Software}, \strong{17}(3), 1-48.
\doi{10.18637/jss.v017.i03}
}
\seealso{
\code{\link[vcd]{labeling_cells}}, \code{\link[vcd]{labeling_border}},
\code{\link[vcd]{mosaic}}, \code{\link[vcd]{sieve}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logseries.R
\name{dlogseries}
\alias{dlogseries}
\alias{logseries}
\alias{plogseries}
\alias{qlogseries}
\alias{rlogseries}
\title{The Logarithmic Series Distribution}
\usage{
dlogseries(x, prob = 0.5, log = FALSE)

plogseries(q, prob = 0.5, lower.tail = TRUE, log.p = FALSE)

qlogseries(p, prob = 0.5, lower.tail = TRUE, log.p = FALSE, max.value = 10000)

rlogseries(n, prob = 0.5)
}
\arguments{
\item{x, q}{vector of quantiles representing the number of events.}

\item{prob}{parameter for the distribution, \verb{0 < prob < 1}}

\item{log, log.p}{logical; if TRUE, probabilities \code{p} are given as \code{log(p)}}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}{P[X <= x]}, otherwise, \eqn{P[X > x]}{P[X > x]}.}

\item{p}{vector of probabilities}

\item{max.value}{maximum value returned by \code{qlogseries}}

\item{n}{number of observations for \code{rlogseries}}
}
\value{
\code{dlogseries} gives the density,
\code{plogseries} gives the cumulative distribution function,
\code{qlogseries} gives the quantile function, and
\code{rlogseries} generates random deviates.
}
\description{
The logarithmic series distribution is a long-tailed distribution introduced
by Fisher etal. (1943) in connection with data on the abundance of
individuals classified by species.
}
\details{
These functions provide the density, distribution function, quantile
function and random generation for the logarithmic series distribution with
parameter \code{prob}.

The logarithmic series distribution with \code{prob} = \eqn{p} has density
\deqn{ p ( x ) = \alpha p^x / x } for \eqn{x = 1, 2, \dots},
where
\eqn{\alpha= -1 / \log(1 - p)} and \eqn{0 < p < 1}.
\% Note that counts \code{x==2} cannot occur.
}
\examples{

XL <-expand.grid(x=1:5, p=c(0.33, 0.66, 0.99))
lgs.df <- data.frame(XL, prob=dlogseries(XL[,"x"], XL[,"p"]))
lgs.df$p = factor(lgs.df$p)
str(lgs.df)

require(lattice)
mycol <- palette()[2:4]
xyplot( prob ~ x, data=lgs.df, groups=p,
	xlab=list('Number of events (k)', cex=1.25),
	ylab=list('Probability',  cex=1.25),
	type='b', pch=15:17, lwd=2, cex=1.25, col=mycol,
	key = list(
					title = 'p',
					points = list(pch=15:17, col=mycol, cex=1.25),
					lines = list(lwd=2, col=mycol),
					text = list(levels(lgs.df$p)),
					x=0.9, y=0.98, corner=c(x=1, y=1)
					)
	)


# random numbers
hist(rlogseries(200, prob=.4), xlab='x')
hist(rlogseries(200, prob=.8), xlab='x')



}
\references{
\url{https://en.wikipedia.org/wiki/Logarithmic_distribution}

Fisher, R. A. and Corbet, A. S. and Williams, C. B. (1943). The relation
between the number of species and the number of individuals \emph{Journal of
Animal Ecology}, 12, 42-58.
}
\seealso{
\code{\link[stats]{Distributions}}
}
\author{
Michael Friendly, using original code modified from the
\code{gmlss.dist} package by Mikis Stasinopoulos.
}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{DaytonSurvey}
\alias{DaytonSurvey}
\title{Dayton Student Survey on Substance Use}
\format{
A frequency data frame with 32 observations on the following 6
variables.
\describe{
\item{\code{cigarette}}{a factor with levels \code{Yes} \code{No}}
\item{\code{alcohol}}{a factor with levels \code{Yes} \code{No}}
\item{\code{marijuana}}{a factor with levels \code{Yes} \code{No}}
\item{\code{sex}}{a factor with levels \code{female} \code{male}}
\item{\code{race}}{a factor with levels \code{white} \code{other}}
\item{\code{Freq}}{a numeric vector}
}
}
\source{
Agresti, A. (2002). \emph{Categorical Data Analysis}, 2nd Ed., New
York: Wiley-Interscience, Table 9.1, p. 362.
}
\description{
This data, from Agresti (2002), Table 9.1, gives the result of a 1992 survey
in Dayton Ohio of 2276 high school seniors on whether they had ever used
alcohol, cigarettes and marijuana.
}
\details{
Agresti uses the letters G (\code{sex}), R (\code{race}), A
(\code{alcohol}), C (\code{cigarette}), M (\code{marijuana}) to refer to the
table variables, and this usage is followed in the examples below.

Background variables include \code{sex} and \code{race} of the respondent
(GR), typically treated as explanatory, so that any model for the full table
should include the term \code{sex:race}. Models for the reduced table,
collapsed over \code{sex} and \code{race} are not entirely unreasonable, but
don't permit the estimation of the effects of these variables on the
responses.

The full 5-way table contains a number of cells with counts of 0 or 1, as
well as many cells with large counts, and even the ACM table collapsed over
GR has some small cell counts.  Consequently, residuals for these models in
mosaic displays are best represented as standardized (adjusted) residuals.
}
\examples{

data(DaytonSurvey)

# mutual independence
mod.0  <- glm(Freq ~ ., data=DaytonSurvey, family=poisson)

# mutual independence + GR
mod.GR <- glm(Freq ~ . + sex*race, data=DaytonSurvey, family=poisson)
anova(mod.GR, test = "Chisq")

# all two-way terms
mod.all2way <- glm(Freq ~ .^2, data=DaytonSurvey, family=poisson)
anova(mod.all2way, test = "Chisq")

# compare models
LRstats(mod.0, mod.GR, mod.all2way)

# collapse over sex and race
Dayton.ACM <- aggregate(Freq ~ cigarette+alcohol+marijuana,
                        data=DaytonSurvey,
                        FUN=sum)
Dayton.ACM

}
\references{
Thompson, L. (2009). \emph{R (and S-PLUS) Manual to Accompany
Agresti's Categorical Data},
http://www.stat.ufl.edu/~aa/cda/Thompson_manual.pdf
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutfac.R
\name{cutfac}
\alias{cutfac}
\title{Cut a Numeric Variable to a Factor}
\usage{
cutfac(x, breaks = NULL, q = 10)
}
\arguments{
\item{x}{a numeric vector which is to be converted to a factor by cutting}

\item{breaks}{either a numeric vector of two or more unique cut points or a
single number (greater than or equal to 2) giving the number of intervals
into which \code{x} is to be cut.}

\item{q}{the number of quantile groups used to define \code{breaks}, if that
has not been specified.}
}
\value{
A \code{\link[base]{factor}} corresponding to \code{x} is returned
}
\description{
\code{cutfac} acts like \code{\link[base]{cut}}, dividing the range of
\code{x} into intervals and coding the values in \code{x} according in which
interval they fall. However, it gives nicer labels for the factor levels and
by default chooses convenient breaks among the values based on deciles.

It is particularly useful for plots in which one wants to make a numeric
variable discrete for the purpose of getting boxplots, spinograms or mosaic
plots.
}
\details{
By default, \code{\link[base]{cut}} chooses breaks by equal lengths of the
range of \code{x}, whereas \code{cutfac} uses \code{\link[stats]{quantile}}
to choose breaks of roughly equal count.
}
\examples{

if (require(AER)) {
data("NMES1988", package="AER")
nmes <- NMES1988[, c(1, 6:8, 13, 15, 18)]

plot(log(visits+1) ~ cutfac(chronic),
  data = nmes,
  ylab = "Physician office visits (log scale)",
  xlab = "Number of chronic conditions", main = "chronic")

plot(log(visits+1) ~ cutfac(hospital, c(0:2, 8)),
  data = nmes,
  ylab = "Physician office visits (log scale)",
  xlab = "Number of hospital stays", main = "hospital")
}

data("CrabSatellites", package = "vcdExtra")

# jittered scatterplot
plot(jitter(satellites) ~ width, data=CrabSatellites,
  ylab="Number of satellites (jittered)",
  xlab="Carapace width",
  cex.lab=1.25)
with(CrabSatellites,
     lines(lowess(width, satellites), col="red", lwd=2))

# boxplot, using deciles
plot(satellites ~ cutfac(width), data=CrabSatellites,
     ylab="Number of satellites",
     xlab="Carapace width (deciles)")



}
\references{
Friendly, M. and Meyer, D. (2016).  \emph{Discrete Data Analysis
with R: Visualization and Modeling Techniques for Categorical and Count
Data}.  Boca Raton, FL: Chapman & Hall/CRC. \url{http://ddar.datavis.ca}.
}
\seealso{
\code{\link[base]{cut}}, \code{\link[stats]{quantile}}
}
\author{
Achim Zeileis
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{AirCrash}
\alias{AirCrash}
\title{Air Crash Data}
\format{
A data frame with 439 observations on the following 5 variables.
\describe{
\item{\code{Phase}}{phase of the flight, a factor with levels \verb{en route} \code{landing}
\code{standing} \code{take-off}
\code{unknown}}
\item{\code{Cause}}{a factor with levels \code{criminal} \verb{human error} \code{mechanical}
\code{unknown} \code{weather}}
\item{\code{date}}{date of crash, a Date}
\item{\code{Fatalities}}{number of fatalities, a numeric vector}
\item{\code{Year}}{year, a numeric vector}
}
}
\source{
Originally from David McCandless,
\url{https://web.archive.org/web/2024/https://informationisbeautiful.net/visualizations/plane-truth-every-single-commercial-plane-crash-visualized/},
with the data at
\url{https://docs.google.com/spreadsheets/d/1OvDq4_BtbR6nSnnHnjD5hVC3HQ-ulZPGbo0RDGbzM3Q/edit?usp=drive_web},
downloaded April 14, 2015.
}
\description{
Data on all fatal commercial airplane crashes from 1993--2015. Excludes
small planes (less than 6 passengers) and non-commercial (cargo, military,
private) aircraft.
}
\details{
\code{Phase} of the flight was cleaned by combining related variants,
spelling, etc.
}
\examples{

data(AirCrash)
aircrash.tab <- xtabs(~Phase + Cause, data=AirCrash)
mosaic(aircrash.tab, shade=TRUE)

# fix label overlap
mosaic(aircrash.tab, shade=TRUE,
       labeling_args=list(rot_labels=c(30, 30, 30, 30)))

# reorder by Phase
phase.ord <- rev(c(3,4,1,2,5))
mosaic(aircrash.tab[phase.ord,], shade=TRUE,
       labeling_args=list(rot_labels=c(30, 30, 30, 30)),
       offset_varnames=0.5)

# reorder by frequency
phase.ord <- order(rowSums(aircrash.tab), decreasing=TRUE)
cause.ord <- order(colSums(aircrash.tab), decreasing=TRUE)
mosaic(aircrash.tab[phase.ord,cause.ord], shade=TRUE,
       labeling_args=list(rot_labels=c(30, 30, 30, 30)))


library(ca)
aircrash.ca <- ca(aircrash.tab)
plot(aircrash.ca)

}
\references{
Rick Wicklin,
\url{http://blogs.sas.com/content/iml/2015/03/30/visualizing-airline-crashes.html}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mosaic.glm.R
\name{mosaic.glm}
\alias{mosaic.glm}
\alias{sieve.glm}
\alias{assoc.glm}
\title{Mosaic plots for fitted generalized linear and generalized nonlinear models}
\usage{
\method{mosaic}{glm}(
  x,
  formula = NULL,
  panel = mosaic,
  type = c("observed", "expected"),
  residuals = NULL,
  residuals_type = c("pearson", "deviance", "rstandard"),
  gp = shading_hcl,
  gp_args = list(),
  ...
)

\method{sieve}{glm}(x, ...)

\method{assoc}{glm}(x, ...)
}
\arguments{
\item{x}{A \code{glm} or \code{gnm} object. The response variable, typically
a cell frequency, should be non-negative.}

\item{formula}{A one-sided formula with the indexing factors of the plot
separated by '+', determining the order in which the variables are used in
the mosaic.  A formula must be provided unless \code{x$data} inherits from
class \code{"table"} -- in which case the indexing factors of this table are
used, or the factors in \code{x$data} (or model.frame(x) if \code{x$data} is
an environment) exactly cross-classify the data -- in which case this set of
cross-classifying factors are used.}

\item{panel}{Panel function used to draw the plot for visualizing the
observed values, residuals and expected values. Currently, one of
\code{"mosaic"}, \code{"assoc"}, or \code{"sieve"} in \code{vcd}.}

\item{type}{A character string indicating whether the \code{"observed"} or
the \code{"expected"} values of the table should be visualized by the area
of the tiles or bars.}

\item{residuals}{An optional array or vector of residuals corresponding to
the cells in the data, for example, as calculated by
\code{residuals.glm(x)}, \code{residuals.gnm(x)}.}

\item{residuals_type}{If the \code{residuals} argument is \code{NULL},
residuals are calculated internally and used in the display.  In this case,
\code{residual_type} can be \code{"pearson"}, \code{"deviance"} or
\code{"rstandard"}.  Otherwise (when \code{residuals} is supplied),
\code{residuals_type} is used as a label for the legend in the plot.}

\item{gp}{Object of class \code{"gpar"}, shading function or a corresponding
generating function (see \code{\link[vcd]{strucplot}} Details and
\code{\link[vcd]{shadings}}).  Ignored if shade = FALSE.}

\item{gp_args}{A list of arguments for the shading-generating function, if
specified.}

\item{\dots}{Other arguments passed to the \code{panel} function e.g.,
\code{\link[vcd]{mosaic}}}
}
\value{
The \code{structable} visualized by \code{\link[vcd]{strucplot}} is
returned invisibly.
}
\description{
Produces mosaic plots (and other plots in the \code{\link[vcd]{strucplot}}
framework) for a log-linear model fitted with \code{\link[stats]{glm}} or
for a generalized nonlinear model fitted with \code{\link[gnm]{gnm}}.
}
\details{
These methods extend the range of strucplot visualizations well beyond the
models that can be fit with \code{\link[MASS]{loglm}}. They are intended for
models for counts using the Poisson family (or quasi-poisson), but should be
sensible as long as (a) the response variable is non-negative and (b) the
predictors visualized in the \code{strucplot} are discrete factors.

For both poisson family generalized linear models and loglinear models,
standardized residuals provided by \code{rstandard} (sometimes called
adjusted residuals) are often preferred because they have constant unit
asymptotic variance.

The \code{sieve} and \code{assoc} methods are simple convenience interfaces
to this plot method, setting the panel argument accordingly.
}
\examples{
library(vcdExtra)

GSStab <- xtabs(count ~ sex + party, data=GSS)
# using the data in table form
mod.glm1 <- glm(Freq ~ sex + party, family = poisson, data = GSStab)
res <- residuals(mod.glm1)
std <- rstandard(mod.glm1)

# For mosaic.default(), need to re-shape residuals to conform to data
stdtab <- array(std,
                dim=dim(GSStab),
                dimnames=dimnames(GSStab))

mosaic(GSStab,
       gp=shading_Friendly,
       residuals=stdtab,
       residuals_type="Std\nresiduals",
       labeling = labeling_residuals)


# Using externally calculated residuals with the glm() object
mosaic(mod.glm1,
       residuals=std,
       labeling = labeling_residuals,
       shade=TRUE)

# Using residuals_type
mosaic(mod.glm1,
       residuals_type="rstandard",
       labeling = labeling_residuals, shade=TRUE)

## Ordinal factors and structured associations
data(Mental)
xtabs(Freq ~ mental+ses, data=Mental)
long.labels <- list(set_varnames = c(mental="Mental Health Status",
                                     ses="Parent SES"))

# fit independence model
# Residual deviance: 47.418 on 15 degrees of freedom
indep <- glm(Freq ~ mental+ses,
             family = poisson, data = Mental)

long.labels <- list(set_varnames = c(mental="Mental Health Status",
                                     ses="Parent SES"))
mosaic(indep,
       formula = ~ses + mental,
       residuals_type="rstandard",
       labeling_args = long.labels,
       labeling=labeling_residuals)

# or, show as a sieve diagram
mosaic(indep,
       formula = ~ses + mental,
       labeling_args = long.labels,
       panel=sieve,
       gp=shading_Friendly)

# fit linear x linear (uniform) association.  Use integer scores for rows/cols
Cscore <- as.numeric(Mental$ses)
Rscore <- as.numeric(Mental$mental)

linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
                family = poisson, data = Mental)

mosaic(linlin,
       formula = ~ses + mental,
       residuals_type="rstandard",
       labeling_args = long.labels,
       labeling=labeling_residuals,
       suppress=1,
       gp=shading_Friendly,
       main="Lin x Lin model")

##  Goodman Row-Column association model fits even better (deviance 3.57, df 8)
if (require(gnm)) {
Mental$mental <- C(Mental$mental, treatment)
Mental$ses <- C(Mental$ses, treatment)
RC1model <- gnm(Freq ~ ses + mental + Mult(ses, mental),
                family = poisson, data = Mental)

mosaic(RC1model,
       formula = ~ses + mental,
       residuals_type="rstandard",
       labeling_args = long.labels,
       labeling=labeling_residuals,
       suppress=1,
       gp=shading_Friendly,
       main="RC1 model")
 }

 ############# UCB Admissions data, fit using glm()

structable(Dept ~ Admit+Gender,UCBAdmissions)

berkeley <- as.data.frame(UCBAdmissions)
berk.glm1 <- glm(Freq ~ Dept * (Gender+Admit), data=berkeley, family="poisson")
summary(berk.glm1)

mosaic(berk.glm1,
       gp=shading_Friendly,
       labeling=labeling_residuals,
       formula=~Admit+Dept+Gender)

# the same, displaying studentized residuals;
# note use of formula to reorder factors in the mosaic
mosaic(berk.glm1,
       residuals_type="rstandard",
       labeling=labeling_residuals,
       shade=TRUE,
	     formula=~Admit+Dept+Gender,
	     main="Model: [DeptGender][DeptAdmit]")

## all two-way model
berk.glm2 <- glm(Freq ~ (Dept + Gender + Admit)^2, data=berkeley, family="poisson")
summary(berk.glm2)

mosaic(berk.glm2,
       residuals_type="rstandard",
       labeling = labeling_residuals,
       shade=TRUE,
	     formula=~Admit+Dept+Gender,
	     main="Model: [DeptGender][DeptAdmit][AdmitGender]")

anova(berk.glm1, berk.glm2, test="Chisq")

# Add 1 df term for association of [GenderAdmit] only in Dept A
berkeley <- within(berkeley,
                   dept1AG <- (Dept=='A')*(Gender=='Female')*(Admit=='Admitted'))
berkeley[1:6,]

berk.glm3 <- glm(Freq ~ Dept * (Gender+Admit) + dept1AG, data=berkeley, family="poisson")
summary(berk.glm3)

mosaic(berk.glm3,
       residuals_type = "rstandard",
       labeling = labeling_residuals,
       shade=TRUE,
	      formula = ~Admit+Dept+Gender,
	      main = "Model: [DeptGender][DeptAdmit] + DeptA*[GA]")

# compare models
anova(berk.glm1, berk.glm3, test="Chisq")


}
\seealso{
\code{\link[stats]{glm}}, \code{\link[gnm]{gnm}},
\code{\link[vcd]{plot.loglm}}, \code{\link[vcd]{mosaic}}

Other mosaic plots:
\code{\link[=mosaic.glmlist]{mosaic.glmlist()}},
\code{\link[=mosaic3d]{mosaic3d()}}
}
\author{
Heather Turner, Michael Friendly, with help from Achim Zeileis
}
\concept{mosaic plots}
\keyword{hplot}
\keyword{models}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vcdExtra-deprecated.R
\name{vcdExtra-deprecated}
\alias{vcdExtra-deprecated}
\title{Deprecated functions in vcdExtra}
\description{
These functions are provided for compatibility with older versions of
vcdExtra and will be removed in a future release.
}
\details{
\itemize{
\item \code{\link[=Summarise]{Summarise()}} is deprecated in favor of \code{\link[=LRstats]{LRstats()}}, which provides
the same brief model-fit comparison under a name that doesn't
near-collide with \code{dplyr::summarise()}.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_table.R
\name{color_table}
\alias{color_table}
\alias{color_table.table}
\alias{color_table.ftable}
\alias{color_table.structable}
\alias{color_table.data.frame}
\alias{color_table.matrix}
\alias{color_table.default}
\title{Smart Display of Frequency Tables with Colored Cell Backgrounds}
\usage{
color_table(x, ...)

\method{color_table}{table}(
  x,
  formula = NULL,
  values = c("freq", "residuals"),
  shade = c("residuals", "freq", "pearson", "deviance"),
  model = NULL,
  expected = NULL,
  palette = NULL,
  legend = FALSE,
  margins = TRUE,
  digits = 0,
  title = NULL,
  filename = NULL,
  ...
)

\method{color_table}{ftable}(
  x,
  values = c("freq", "residuals"),
  shade = c("residuals", "freq", "pearson", "deviance"),
  model = NULL,
  expected = NULL,
  palette = NULL,
  legend = FALSE,
  margins = TRUE,
  digits = 0,
  title = NULL,
  filename = NULL,
  ...
)

\method{color_table}{structable}(
  x,
  values = c("freq", "residuals"),
  shade = c("residuals", "freq", "pearson", "deviance"),
  model = NULL,
  expected = NULL,
  palette = NULL,
  legend = FALSE,
  margins = TRUE,
  digits = 0,
  title = NULL,
  filename = NULL,
  ...
)

\method{color_table}{data.frame}(
  x,
  formula = NULL,
  freq_col = NULL,
  values = c("freq", "residuals"),
  shade = c("residuals", "freq", "pearson", "deviance"),
  model = NULL,
  expected = NULL,
  palette = NULL,
  legend = FALSE,
  margins = TRUE,
  digits = 0,
  title = NULL,
  filename = NULL,
  ...
)

\method{color_table}{matrix}(
  x,
  values = c("freq", "residuals"),
  shade = c("residuals", "freq", "pearson", "deviance"),
  model = NULL,
  expected = NULL,
  palette = NULL,
  legend = FALSE,
  margins = TRUE,
  digits = 0,
  title = NULL,
  filename = NULL,
  ...
)

\method{color_table}{default}(x, ...)
}
\arguments{
\item{x}{A \code{"table"}, \code{"xtabs"}, \code{"matrix"}, \code{"ftable"}, \code{"structable"}, or \code{"data.frame"} object}

\item{...}{Additional arguments passed to methods}

\item{formula}{Formula specifying a \code{col_vars ~ row_vars} layout (for multi-way tables) to
make them "flat" as defined for \code{vcd::structable()} and \code{stats::ftable()}.}

\item{values}{What values to display in cells: \code{"freq"} for observed frequencies (default),
or \code{"residuals"} to display the residual values. When \code{values = "residuals"}, margins
are suppressed since residuals don't have meaningful totals.}

\item{shade}{What values determine cell shading: \code{"residuals"} (default), \code{"freq"},
\code{"pearson"}, or \code{"deviance"}}

\item{model}{A fitted model (loglm or glm) to compute residuals from.
If NULL and shade involves residuals, uses an independence model for all factors.}

\item{expected}{Expected frequencies (alternative to \code{model}), a data structure of the same shape
as \code{x}}

\item{palette}{Color palette function or vector for background colors. Default depends on shade
type. When \code{shade = "freq"} the default is \code{palette = c("white", "firebrick")}; otherwise
\code{c("#B2182B", "white", , "#2166AC")} ranging from red to blue for negative and positive
residuals. The background colors are computed by interpolation using
\code{scales::col_numeric()}.}

\item{legend}{Controls display of shading interpretation note:
\code{TRUE} or \code{"note"} (default) adds a source note explaining the shading;
\code{FALSE} (default) suppresses the note, but a message is printed in the console.}

\item{margins}{Logical, include row/column totals?}

\item{digits}{Number of decimal places for displayed values}

\item{title}{Optional table title}

\item{filename}{Optional filename to save the table as an image. If provided,
the table is saved using \code{\link[gt]{gtsave}}. Supported formats include
\code{.png}, \code{.pdf}, \code{.html}, \code{.rtf}, and \code{.docx}.
The file format is determined by the file extension. Other arguments can be passed
to \code{\link[gt]{gtsave}} via \code{...}.}

\item{freq_col}{Name of the frequency column. If NULL, looks for "Freq" or "count".}
}
\value{
A gt table object that can be further customized
}
\description{
\code{color_table()} creates a formatted, semi-graphic "heatmap" table display of frequency data with cell backgrounds
colored according to observed frequencies or their residuals from a loglinear model. The goal is to provide a
"smart" tabular display of cross-classified frequency data, with some abilities to highlight possibly interesting
patterns or unusual cells.
This is an S3 generic function, providing methods for different input types: \code{"table"}, \code{"xtabs"}, \code{"matrix"}, \code{"ftable"}, \code{"structable"} or \code{"data.frame"}.
}
\details{
This function provides a heatmap-style representation of a frequency table,
where background coloring is used to visualize patterns and anomalies in the data.
When shading by \emph{residuals} (the default), cells with large positive residuals
(more observations than expected) are shaded red, while cells with large negative
residuals (fewer than expected) are shaded blue. This makes it easy to identify
cells that deviate substantially from what would be expected under a given model
(by default, the independence model).

For multi-way tables (3 or more dimensions), residuals are computed from the
model of complete independence among all factors using \code{\link[MASS]{loglm}}.
But you can specify a model using the \code{model} or \code{expected} arguments, in a way similar to that provided
by \code{mosaic.glm()}.
A message is printed showing the chi-squared statistic, degrees of freedom,
and p-value for this test.

\strong{Multi-variable row and column labels}

\code{formula} follows the \code{col_vars ~ row_vars} convention used by
\code{\link[vcd:structable]{vcd::structable()}} and \code{\link[stats:ftable]{stats::ftable()}} (confirmed to agree between the
two across many row/col splits of \code{HairEyeColor} and \code{Titanic} -- see
\code{dev/color_table/formula-row-col-comparison.R}). When \code{formula} specifies
more than one variable on either side, \code{color_table()} avoids concatenated
labels like \code{"Black_Male"}. Multiple column variables (on the left of \code{~},
e.g. \code{formula = Class + Sex + Age ~ Survived}) are shown using
\code{\link[gt:tab_spanner]{gt::tab_spanner()}} headings that group columns under their outer
variable(s). Multiple row variables (on the right of \code{~}, e.g.
\code{formula = Survived ~ Class + Sex}) are shown as separate stub columns, one
per row variable, using \code{gt}'s multi-column stub support (\code{rowname_col}
accepting a vector of column names), with repeated values left blank. The
two can be combined in the same table.

\strong{Contrast shading}

For cells with dark background colors, black text can be difficult to read.
This function automatically selects white or black text for each cell based
on which provides better contrast against the background color. If the
\pkg{colorspace} package is available, \code{\link[colorspace]{contrast_ratio}}
is used to determine the optimal text color according to WCAG 2.1 guidelines.
Otherwise, a fallback based on relative luminance (ITU-R BT.709) is used.

\strong{Use in documents}

In R Markdown (\code{.Rmd}) or Quarto (\code{.qmd}) documents, \pkg{gt} tables
render natively in \strong{HTML output} — simply return the \code{gt} object from
a chunk and knitr renders it automatically via \pkg{gt}'s built-in
\code{knit_print} method. No \code{filename} argument is needed.

For \strong{PDF or Word output}, \pkg{gt} does not render natively. Use the
\code{filename} argument to save the table as a \code{.png} image, then include
it with \code{\link[knitr]{include_graphics}}:

\preformatted{
    color_table(my_table, filename = "my_table.png")
    knitr::include_graphics("my_table.png")
}

The \code{vwidth} and \code{vheight} arguments (passed via \code{...}) control
the image viewport size in pixels.  Supported save formats are
\code{.png}, \code{.pdf}, \code{.html}, \code{.rtf}, and \code{.docx}.

For documents that target \strong{multiple output formats}, a small helper that
branches on \code{\link[knitr]{is_html_output}} avoids duplicating code:

\preformatted{
    gt_obj <- color_table(my_table)
    if (knitr::is_html_output()) {
      gt_obj
    } else {
      gt::gtsave(gt_obj, "my_table.png")
      knitr::include_graphics("my_table.png")
    }
}

If you need a caption or cross-reference label, use \code{gt::tab_caption()}
on the returned object:
\preformatted{
    color_table(my_table) |>
      gt::tab_caption("Table 1: Pattern of association in MyTable")
}
}
\section{Methods (by class)}{
\itemize{
\item \code{color_table(table)}: Method for table objects (including result of xtabs)

\item \code{color_table(ftable)}: Method for ftable objects

\item \code{color_table(structable)}: Method for structable objects (vcd package)

\item \code{color_table(data.frame)}: Method for data.frame in frequency form

\item \code{color_table(matrix)}: Method for matrix objects

\item \code{color_table(default)}: Default method

}}
\examples{
# Basic usage with 2-way table - shade by residuals from independence
data(HairEyeColor)
HEC <- margin.table(HairEyeColor, 1:2)  # 2-way: Hair x Eye
color_table(HEC)

# Shade by frequencies instead (no message printed)
color_table(HEC, shade = "freq")

# 3-way table - using a formula to specify layout (col_vars ~ row_vars):
# Eye is the single column variable, Hair and Sex are row variables shown
# as separate stub columns
color_table(HairEyeColor, formula = Eye ~ Hair + Sex)

# Display residual values in cells instead of frequencies
color_table(HEC, values = "residuals")

# Multiple column variables - shown using nested spanner headings instead
# of concatenated column labels
data(Titanic)
color_table(Titanic, formula = Class + Sex ~ Survived)

# Multiple row AND column variables together: column spanners (Class, Sex)
# combined with row-variable stub columns (Age, Survived)
color_table(Titanic, formula = Class + Sex ~ Age + Survived, legend = TRUE)

\dontrun{
# From a data.frame in frequency form (2-way)
hec_df <- as.data.frame(HEC)
color_table(hec_df)

# Save table as an image file
color_table(HEC, filename = "hair_eye_table.png")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Burt}
\alias{Burt}
\title{Burt (1950) Data on Hair, Eyes, Head and Stature}
\format{
A frequency data frame (representing a 3 x 3 x 2 x 2 frequency
table) with 36 cells on the following 5 variables.
\describe{
\item{\code{Hair}}{hair color, a factor with levels \code{Fair} \code{Red} \code{Dark}}
\item{\code{Eyes}}{eye color, a factor with levels \code{Light} \code{Mixed} \code{Dark}}
\item{\code{Head}}{head shape, a factor with levels \code{Narrow} \code{Wide}}
\item{\code{Stature}}{height, a factor with levels \code{Tall} \code{Short}}
\item{\code{Freq}}{a numeric vector}
}
}
\source{
Burt, C. (1950). The factorial analysis of qualitative data,
\emph{British Journal of Statistical Psychology}, \strong{3}(3), 166-185.
Table IX.
}
\description{
Cyril Burt (1950) gave these data, on a sample of 100 people from Liverpool,
to illustrate the application of a method of factor analysis (later called
multiple correspondence analysis) applied to categorical data.
}
\details{
He presented these data initially in the form that has come to be called a
"Burt table", giving the univariate and bivariate frequencies for an n-way
frequency table.

Burt says: "In all, 217 individuals were examined, about two-thirds of them
males. But, partly to simplify the calculations and partly because the later
observations were rather more trustworthy, I shall here restrict my analysis
to the data obtained from the last hundred males in the series."

\code{Head} and \code{Stature} reflect a binary coding where people are
classified according to whether they are below or above the average for the
population.
}
\examples{

data(Burt)
mosaic(Freq ~ Hair + Eyes + Head + Stature, data=Burt, shade=TRUE)

#or
burt.tab <- xtabs(Freq ~ Hair + Eyes + Head + Stature, data=Burt)
mosaic(burt.tab, shade=TRUE)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Heckman}
\alias{Heckman}
\title{Labour Force Participation of Married Women 1967-1971}
\format{
A 5-dimensional \eqn{2^5} array resulting from cross-tabulating 5
binary variables for 1583 observations. The variable names and their levels
are:

\tabular{rll}{
No \tab Name \tab Levels \cr
1\tab \code{e1971}\tab \verb{"71Yes", "No"}\cr
2\tab \code{e1970}\tab \verb{"70Yes", "No"}\cr
3\tab \code{e1969}\tab \verb{"69Yes", "No"}\cr
4\tab \code{e1968}\tab \verb{"68Yes", "No"}\cr
5\tab \code{e1967}\tab \verb{"67Yes", "No"}\cr
}
}
\source{
Lindsey, J. K. (1993).  \emph{Models for
Repeated Measurements} Oxford, UK: Oxford University Press, p. 185.
}
\description{
1583 married women were surveyed over the years 1967-1971, recording whether
or not they were employed in the labor force.
}
\details{
The data, originally from Heckman & Willis (1977) provide an example of
modeling longitudinal categorical data, e.g., with markov chain models for
dependence over time.

Lindsey (1993) fits an initial set of logistic regression models examining
the dependence of employment in 1971 (\code{e1971}) on successive subsets of
the previous years, \code{e1970}, \code{e1969}, \dots{} \code{e1967}.

Alternatively, one can examine markov chain models of first-order
(dependence on previous year), second-order (dependence on previous two
years), etc.
}
\examples{

data(Heckman)

# independence model
mosaic(Heckman, shade=TRUE)
# same, as a loglm()
require(MASS)
(heckman.mod0 <- loglm(~ e1971+e1970+e1969+e1968+e1967, data=Heckman))
mosaic(heckman.mod0, main="Independence model")

# first-order markov chain: bad fit
(heckman.mod1 <- loglm(~ e1971*e1970 + e1970*e1969 +e1969*e1968 + e1968*e1967, data=Heckman))
mosaic(heckman.mod1, main="1st order markov chain model")

# second-order markov chain: bad fit
(heckman.mod2 <- loglm(~ e1971*e1970*e1969 + e1970*e1969*e1968 +e1969*e1968*e1967, data=Heckman))
mosaic(heckman.mod2, main="2nd order markov chain model")

# third-order markov chain: fits OK
(heckman.mod3 <- loglm(~ e1971*e1970*e1969*e1968 + e1970*e1969*e1968*e1967, data=Heckman))
mosaic(heckman.mod2, main="3rd order markov chain model")


}
\references{
\% \cite{HeckmanWillis:77}
Heckman, J.J. & Willis, R.J. (1977).
"A beta-logistic model for the analysis of sequential labor force
participation by married women."  \emph{Journal of Political Economy}, 85:
27-58
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Dyke}
\alias{Dyke}
\title{Sources of Knowledge of Cancer}
\format{
A 5-dimensional array resulting from cross-tabulating 5 variables
for 1729 observations. The variable names and their levels are:

\tabular{rll}{
dim \tab Name \tab Levels \cr
1\tab \code{Knowledge}\tab \verb{"Good", "Poor"}\cr
2\tab \code{Reading}\tab \verb{"No", "Yes"}\cr
3\tab \code{Radio}\tab \verb{"No", "Yes"}\cr
4\tab \code{Lectures}\tab \verb{"No", "Yes"}\cr
5\tab \code{Newspaper}\tab \verb{"No", "Yes"}\cr
}
}
\source{
Fienberg, S. E. (1980). \emph{The Analysis of Cross-Classified
Categorical Data} Cambridge, MA: MIT Press, p. 85, Table 5-6.
}
\description{
Observational data on a sample of 1729 individuals, cross-classified in a
2^5 table according to their sources of information (read newspapers, listen
to the radio, do 'solid' reading, attend lectures) and whether they have
good or poor knowledge regarding cancer.  Knowledge of cancer is often
treated as the response.
}
\examples{

data(Dyke)

# independence model
mosaic(Dyke, shade=TRUE)

# null model, Knowledge as response, independent of others
require(MASS)
dyke.mod0 <- loglm(~ Knowledge + (Reading * Radio * Lectures * Newspaper), data=Dyke)
dyke.mod0
mosaic(dyke.mod0)

# view as doubledecker plot
Dyke <- Dyke[2:1,,,,]    # make Good the highlighted value of Knowledge
doubledecker(Knowledge ~ ., data=Dyke)

# better version, with some options
doubledecker(Knowledge ~ Lectures + Reading + Newspaper + Radio,
  data=Dyke,
	margins = c(1,6, length(dim(Dyke)) + 1, 1),
	fill_boxes=list(rep(c("white", gray(.90)),4))
	)

# separate (conditional) plots for those who attend lectures and those who do not
doubledecker(Knowledge ~ Reading + Newspaper + Radio,
  data=Dyke[,,,1,],
	main="Do not attend lectures",
	margins = c(1,6, length(dim(Dyke)) + 1, 1),
	fill_boxes=list(rep(c("white", gray(.90)),3))
	)
doubledecker(Knowledge ~ Reading + Newspaper + Radio,
  data=Dyke[,,,2,],
	main="Attend lectures",
	margins = c(1,6, length(dim(Dyke)) + 1, 1),
	fill_boxes=list(rep(c("white", gray(.90)),3))
	)


drop1(dyke.mod0, test="Chisq")


}
\references{
Dyke, G. V. and Patterson, H. D. (1952). Analysis of factorial
arrangements when the data are proportions. \emph{Biometrics}, 8, 1-12.

Lindsey, J. K. (1993).  \emph{Models for Repeated Measurements} Oxford, UK:
Oxford University Press, p. 57.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{GSS}
\alias{GSS}
\title{General Social Survey-- Sex and Party affiliation}
\format{
A data frame in frequency form with 6 observations on the following
3 variables.

\describe{
\item{\code{sex}}{a factor with levels \code{female} \code{male}}
\item{\code{party}}{a factor with levels \code{dem} \code{indep} \code{rep}}
\item{\code{count}}{a numeric vector}
}
}
\source{
Agresti, A. \emph{Categorical Data Analysis}, 2nd E., John Wiley &
Sons, 2002, Table 3.11, p. 106.
}
\description{
Data from the General Social Survey, 1991, on the relation between sex and
party affiliation.
}
\examples{

data(GSS)
str(GSS)

# use xtabs to show the table in a compact form
(GSStab <- xtabs(count ~ sex + party, data=GSS))

# fit the independence model
(mod.glm <- glm(count ~ sex + party, family = poisson, data = GSS))

# display all the residuals in a mosaic plot
mosaic(mod.glm,
  formula = ~ sex + party,
  labeling = labeling_residuals,
  suppress=0)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Cormorants}
\alias{Cormorants}
\title{Advertising Behavior by Males Cormorants}
\format{
A data frame with 343 observations on the following 8 variables.

\describe{
\item{\code{category}}{Time of season, divided into 3 categories based on breeding chronology, an ordered factor with levels \code{Pre} < \code{Incubation} < \verb{Chicks Present}}
\item{\code{week}}{Week of the season}
\item{\code{station}}{Station of observations on two different peninsulas in a park, a factor with levels \code{B1} \code{B2} \code{C1} \code{C2} \code{C3} \code{C4}}
\item{\code{nest}}{Type of nest, an ordered factor with levels \code{no} < \code{partial} < \code{full}}
\item{\code{height}}{Relative height of bird in the tree, an ordered factor with levels \code{low} < \code{mid} < \code{high}}
\item{\code{density}}{Number of other nests in the tree, an ordered factor with levels \code{zero} < \code{few} < \code{moderate} < \code{high}}
\item{\code{tree_health}}{Health of the tree the bird is advertising in, a factor with levels \code{dead} \code{healthy}}
\item{\code{count}}{Number of birds advertising, a numeric vector}
}
}
\source{
McRae, M. (2015). Spatial, Habitat and Frequency Changes in
Double-crested Cormorant Advertising Display in a Tree-nesting Colony.
Unpublished MA project, Environmental Studies, York University.
}
\description{
Male double-crested cormorants use advertising behavior to attract females
for breeding. In this study by Meagan McRae (2015), cormorants were observed
two or three times a week at six stations in a tree-nesting colony for an
entire season, April 10, 2014-July 10, 2014. The number of advertising birds
was counted and these observations were classified by characteristics of the
trees and nests.
}
\details{
The goal is to determine how this behavior varies temporally over the season
and spatially, as well as with characteristics of nesting sites.

Observations were made on only 2 days in weeks 3 and 4, but 3 days in all
other weeks. One should use log(days) as an offset, so that the response
measures rate.

\verb{Cormorants$days <- ifelse(Cormorants$week \\\%in\\\% 3:4, 2, 3)}
}
\examples{

data(Cormorants)
str(Cormorants)

if (require("ggplot2")) {
  print(ggplot(Cormorants, aes(count)) +
    geom_histogram(binwidth=0.5) +
	  labs(x="Number of birds advertising"))

# Quick look at the data, on the log scale, for plots of `count ~ week`,
#   stratified by something else.

  print(ggplot(Cormorants, aes(week, count, color=height)) +
    geom_jitter() +
	  stat_smooth(method="loess", size=2) +
	  scale_y_log10(breaks=c(1,2,5,10)) +
	  geom_vline(xintercept=c(4.5, 9.5)))
}

# ### models using week
fit1 <-glm(count ~ week + station + nest + height + density + tree_health,
           data=Cormorants,
           family =  poisson)

if (requireNamespace("car"))
  car::Anova(fit1)

# plot fitted effects
if (requireNamespace("effects"))
  plot(effects::allEffects(fit1))


}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Mice}
\alias{Mice}
\title{Mice Depletion Data}
\format{
A frequency data frame with 30 observations on the following 4
variables, representing a 5 x 2 x 3 contingency table.
\describe{
\item{\code{litter}}{litter size, a numeric vector}
\item{\code{treatment}}{treatment, a factor with levels \code{A} \code{B}}
\item{\code{deaths}}{number of depletions, a factor with levels \code{0} \code{1} \verb{2+}}
\item{\code{Freq}}{cell frequency, a numeric vector}
}
}
\source{
Goodman, L. A. (1983) The analysis of dependence in
cross-classifications having ordered categories, using log-linear models for
frequencies and log-linear models for odds. \emph{Biometrics}, 39, 149-160.
}
\description{
Data from Kastenbaum and Lamphiear (1959). The table gives the number of
depletions (deaths) in 657 litters of mice, classified by litter size and
treatment.  This data set has become a classic in the analysis of
contingency tables, yet unfortunately little information on the details of
the experiment has been published.
}
\examples{

data(Mice)
# make a table
ftable(mice.tab <- xtabs(Freq ~ litter + treatment + deaths, data=Mice))

#library(vcd)
vcd::mosaic(mice.tab, shade=TRUE)


}
\references{
Kastenbaum, M. A. & Lamphiear, D. E. (1959) Calculation of
chi-square to calculate the no three-factor interaction hypothesis.
\emph{Biometrics}, 15, 107-115.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Vietnam}
\alias{Vietnam}
\title{Student Opinion about the Vietnam War}
\format{
A frequency data frame with 40 observations representing a 2 x 5 x 4
contingency table on the following 4 variables.
\describe{
\item{\code{sex}}{a factor with levels \code{Female} \code{Male}}
\item{\code{year}}{year of study, an ordered factor with levels
\code{Freshmen}, \code{Sophomore}, \code{Junior}, \code{Senior}, \verb{Grad student}}
\item{\code{response}}{a factor with levels \code{A} \code{B} \code{C} \code{D}}
\item{\code{Freq}}{cell frequency, a numeric vector}
}
}
\source{
Aitken, M. etal, 1989, \emph{Statistical Modelling in GLIM}
}
\description{
A survey of student opinion on the Vietnam War was taken at the University
of North Carolina at Chapel Hill in May 1967 and published in the student
newspaper. Students were asked to fill in ballot papers stating which policy
out of A,B,C or D they supported. Responses were cross-classified by
gender/year.
}
\details{
The response categories were:
\describe{
\item{\code{A}}{Defeat North Vietnam by widespread bombing and land invasion}
\item{\code{B}}{Maintain the present policy}
\item{\code{C}}{De-escalate military activity, stop bombing and begin negotiations}
\item{\code{D}}{Withdraw military forces Immediately}
}

For some analyses, it is useful to treat \code{year} as numeric, and
possibly assign grad students a value \code{year=7}.
}
\examples{

data(Vietnam)
## maybe str(Vietnam) ; plot(Vietnam) ...

}
\references{
Friendly, M.  (2000), \emph{Visualizing Categorical Data}, SAS
Institute, Cary, NC, Example 7.9.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drop1.loglm.R
\name{drop1.loglm}
\alias{drop1.loglm}
\alias{LRanova}
\alias{assoc_strength}
\title{Term-level tests and effect sizes for loglm models}
\usage{
\method{drop1}{loglm}(object, scope, test = c("Chisq", "none"), abbrev = FALSE, ...)

LRanova(
  object,
  baseline = NULL,
  test = c("Chisq", "none"),
  abbrev = FALSE,
  ...
)

assoc_strength(
  object,
  scope,
  method = c("Cramer", "Cohen"),
  abbrev = FALSE,
  ...
)
}
\arguments{
\item{object}{a fitted \code{MASS::loglm} model}

\item{scope}{character vector of term labels (colon-separated, as in
\code{"A:B"}) to test; must be a subset of the model's generating class.
Defaults to the full generating class. Used by \code{drop1.loglm()} and
\code{assoc_strength()}.}

\item{test}{\code{"Chisq"} (default) to include an LR p-value column, or
\code{"none"} to omit it.}

\item{abbrev}{passed to \code{\link[=get_model]{get_model()}} for the \verb{Model:}/\verb{Baseline:}
heading line(s); \code{FALSE} (default) for full factor names, or an integer
(e.g. \code{4}) to abbreviate each factor name to that many characters --
useful when the generating class has many terms and the heading line
gets long (see the DaytonSurvey example below).}

\item{...}{currently unused in \code{drop1.loglm()}; passed on to it from
\code{LRanova()}/\code{assoc_strength()}.}

\item{baseline}{\code{NULL} (default) to use the model of mutual independence
over the variables in \code{object}'s generating class, or a fitted \code{loglm}
model nested within \code{object} to use as the reference for partial R^2.
Used by \code{LRanova()} only.}

\item{method}{\code{"Cramer"} (default) for partial Cramer's V, generalized to
terms of any order by using the \emph{smallest} factor level count among the
term's variables in place of the usual two-way \code{min(r,c)}; this reduces
exactly to the ordinary two-way Cramer's V (as used by
\code{\link[vcd:assocstats]{vcd::assocstats()}}) when the term has two factors. Bounded to \verb{[0,1]}.
\code{"Cohen"} for Cohen's w = \code{sqrt(Delta-X^2 / N)}: unbounded, but requires
no reference to table shape, so it stays well-defined uniformly for
terms of any order. Used by \code{assoc_strength()} only.}
}
\value{
An object of class \code{c("anova", "data.frame")} with columns \code{Df},
\verb{LR Chisq}, \verb{Pearson Chisq}, and (if \code{test = "Chisq"}) \verb{Pr(>Chi)}, one
row per tested term plus a \verb{<none>} reference row. The fitted models
underlying each row -- \code{object} itself as \code{"<none>"}, plus the refit
dropping each term in \code{scope} -- are attached as \code{attr(., "models")}, a
\code{loglmlist} (see \code{\link[=loglmlist]{loglmlist()}}) with names matching the table's row
names. This is free (the models are already fit to compute the
Delta-statistics) and makes the whole \verb{*.loglmlist} toolchain available
without refitting, e.g. \code{mosaic(attr(result, "models"), ask = FALSE)}
for a grid of the full model and every drop-one model, or
\code{LRstats(attr(result, "models"))} for an AIC/BIC comparison.

\code{LRanova()} adds a \verb{Partial R2} column. The baseline model is attached
separately as \code{attr(., "baseline")}, since it's a single reference
model rather than one of the \verb{<none>}/drop-one rows in
\code{attr(., "models")}.

\code{assoc_strength()} adds a \code{"Cramer's V"} or \code{"Cohen's w"} column. Note
this is a \emph{partial} association -- conditional on the other terms in
\code{object} -- not the marginal association from \code{vcd::assocstats()} on
the term's own two-way margin; the two can differ sharply (see the
UCBAdmissions \code{Admit:Gender} example below, a classic Simpson's-paradox
case).
}
\description{
\code{drop1.loglm()} performs single-term deletion tests for the generating
class of a fitted \code{\link[MASS:loglm]{MASS::loglm()}} model: for each term in that
class, it drops that term, refits via \code{update()}, and reports the change
in both the LR statistic (G^2) and the Pearson statistic (X^2).
}
\details{
\code{LRanova()} wraps \code{drop1.loglm()} and adds a partial R^2 effect-size
column: the fraction of a baseline model's G^2 attributable to each
dropped term.

\code{assoc_strength()} wraps \code{drop1.loglm()} and adds a partial Cramer's V or
Cohen's w effect-size column, converting each term's partial Pearson X^2
into a bounded or semi-bounded measure of association strength.

All three functions test terms in the model's \emph{generating class} --
the set of highest-order terms in a hierarchical loglinear model, from
which every lower-order relative (main effects, lower interactions) is
implied and so doesn't need to be listed or tested separately.

For example, with four factors \code{A}, \code{B}, \code{C}, \code{D} and the model
\code{~ (A + B + C + D)^2} (all terms up to two-way), the fitted model
contains ten terms in all -- four main effects and six two-way
interactions -- but its generating class is just the six two-way terms:
\code{A:B}, \code{A:C}, \code{A:D}, \code{B:C}, \code{B:D}, \code{C:D} (bracket notation \verb{[A,B] [A,C] [A,D] [B,C] [B,D] [C,D]}).

None of the four main effects is maximal on
its own -- \code{A}, say, is already implied by \code{A:B} (or any of \code{A:C},
\code{A:D}) -- so they're excluded from the generating class even though
they're very much still part of the fitted model. This is exactly what
\code{MASS::loglm()} stores in \code{object$margin}, and exactly the set of terms
\code{scope} defaults to and \code{drop1.loglm()} tests: dropping \code{A:B} removes
only that interaction, leaving \code{A} and \code{B}'s main effects in place
(still implied by \code{A:C}/\code{A:D} and \code{B:C}/\code{B:D} respectively), so the
reduced model stays hierarchical.
\subsection{Comparison with \code{drop1.glm()}}{

A hierarchical \code{loglm()} model has an equivalent fit via
\code{glm(family = poisson)}, and \code{stats::drop1.glm()}'s default \code{scope}
already restricts to the same generating-class terms as \code{object$margin},
so \code{Df} and the LR statistic agree exactly between the two. \code{drop1.loglm()}
works directly on table/array or \code{Freq ~ ...} data, adds the Pearson X^2
column, and returns the fitted drop-one models for reuse; its \code{scope}
validation also guards against testing a non-generating-class term, which
\code{drop1.glm()} will silently allow with a misleading result. See
\code{dev/anova-like/drop1-compare.R} for a worked comparison.
}
}
\examples{
library(MASS)
ucb <- loglm(~ (Admit + Gender + Dept)^2, data = UCBAdmissions)

drop1.loglm(ucb)
drop1.loglm(ucb, scope = "Admit:Gender")   # test only a subset
drop1.loglm(ucb, abbrev = 4)               # abbreviate factor names
names(attr(drop1.loglm(ucb), "models"))    # the fitted models, for free

# data.frame + `Freq ~ ...` form works too, not just array/table data
data(DaytonSurvey, package = "vcdExtra")
DS <- loglm(Freq ~ (cigarette + alcohol + marijuana + sex + race)^2,
            data = DaytonSurvey)
drop1.loglm(DS)

LRanova(ucb)  # default baseline: mutual independence over Admit, Gender, Dept

# supply a specific (nested) baseline other than the default -- here, one
# that already includes Admit:Gender, so partial R^2 is now relative to a
# smaller remaining G^2 than the mutual-independence baseline above
partial_baseline <- loglm(~ Admit + Gender + Dept + Admit:Gender, data = UCBAdmissions)
LRanova(ucb, baseline = partial_baseline)

assoc_strength(ucb)                     # Cramer's V (default)
assoc_strength(ucb, method = "Cohen")   # Cohen's w

# partial (conditional on Dept) vs. marginal association -- the small
# partial Admit:Gender V here versus the much larger marginal V below is
# the classic Simpson's-paradox story for this dataset
vcd::assocstats(margin.table(UCBAdmissions, c(1, 2)))

# Cramer's V and Cohen's w coincide when every factor in a term is binary,
# and diverge once a term involves a factor with more levels
hec <- loglm(~ (Hair + Eye + Sex)^2, data = HairEyeColor)
assoc_strength(hec)  # Hair, Eye each have 4 levels
}
\seealso{
\code{\link[=mosaic.loglmlist]{mosaic.loglmlist()}}, \code{\link[=LRstats]{LRstats()}}, \code{\link[=get_models]{get_models()}}
}
\author{
Michael Friendly
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Mental}
\alias{Mental}
\title{Mental Impairment and Parents SES}
\format{
A data frame frequency table with 24 observations on the following 3
variables.
\describe{
\item{\code{ses}}{an ordered factor with levels \code{1} < \code{2} < \code{3} < \code{4} < \code{5} < \code{6}}
\item{\code{mental}}{an ordered factor with levels \code{Well} < \code{Mild} < \code{Moderate} < \code{Impaired}}
\item{\code{Freq}}{cell frequency: a numeric vector}
}
}
\source{
Haberman, S. J.  \emph{The Analysis of Qualitative Data: New
Developments}, Academic Press, 1979, Vol. II, p. 375.

Srole, L.; Langner, T. S.; Michael, S. T.; Kirkpatrick, P.; Opler, M. K. &
Rennie, T. A. C.  \emph{Mental Health in the Metropolis: The Midtown
Manhattan Study}, NYU Press, 1978, p. 289
}
\description{
A 6 x 4 contingency table representing the cross-classification of mental
health status (\code{mental}) of 1660 young New York residents by their
parents' socioeconomic status (\code{ses}).
}
\details{
Both \code{ses} and \code{mental} can be treated as ordered factors or
integer scores.  For \code{ses}, 1="High" and 6="Low".
}
\examples{

data(Mental)
str(Mental)
(Mental.tab <- xtabs(Freq ~ ses + mental, data=Mental))

# mosaic and sieve plots
mosaic(Mental.tab, gp=shading_Friendly)
sieve(Mental.tab, gp=shading_Friendly)

if(require(ca)){
  plot(ca(Mental.tab), main="Mental impairment & SES", lines=TRUE)
}

# fit linear x linear (uniform) association model, using integer scores
# for rows/cols
indep <- glm(Freq ~ mental + ses, family = poisson, data = Mental)
Cscore <- as.numeric(Mental$ses)
Rscore <- as.numeric(Mental$mental)

linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
              family = poisson, data = Mental)
anova(linlin, test = "Chisq")

# use update.glm method to fit other models
linlin <- update(indep, . ~ . + Rscore:Cscore)
roweff <- update(indep, . ~ . + mental:Cscore)
coleff <- update(indep, . ~ . + Rscore:ses)
rowcol <- update(indep, . ~ . + Rscore:ses + mental:Cscore)

# compare models
LRstats(indep, linlin, roweff, coleff, rowcol)

# tests of nested models
anova(indep, linlin, roweff, test = "Chisq")

}
\references{
Friendly, M. \emph{Visualizing Categorical Data}, Cary, NC: SAS
Institute, 2000, Appendix B.7.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglin-utilities.R
\name{joint}
\alias{joint}
\alias{loglin-utilities}
\alias{conditional}
\alias{loglin2formula}
\alias{loglin2string}
\alias{markov}
\alias{mutual}
\alias{saturated}
\title{Loglinear Model Utilities}
\source{
Code from Henrique Dallazuanna, \href{mailto:wwwhsd@gmail.com}{wwwhsd@gmail.com}, R-help 7-4-2013
}
\usage{
joint(nf, table = NULL, factors = 1:nf, with = nf)

conditional(nf, table = NULL, factors = 1:nf, with = nf)

mutual(nf, table = NULL, factors = 1:nf)

saturated(nf, table = NULL, factors = 1:nf)

markov(nf, factors = 1:nf, order = 1)

loglin2formula(x, env = parent.frame())

loglin2string(x, brackets = c("[", "]"), sep = ",", collapse = " ", abbrev)
}
\arguments{
\item{nf}{number of factors for which to generate model}

\item{table}{a contingency table used for factor names, typically the output from \code{\link[base]{table}}}

\item{factors}{names of factors used in the model when \code{table} is not specified}

\item{with}{indices of the factors against which others are considered conditionally independent}

\item{order}{order of the markov chain}

\item{x}{a list of terms in a loglinear model, such as returned by \code{joint}, \code{conditional}, \dots}

\item{env}{environment in which to evaluate the formula}

\item{brackets}{characters to use to surround model terms.  Either a single character string containing two characters
or a character vector of length two.}

\item{sep}{characters used to separate factor names within a term}

\item{collapse}{characters used to separate terms}

\item{abbrev}{Unused as yet}
}
\description{
These functions generate lists of terms to specify a loglinear model in a
form compatible with \code{\link[stats]{loglin}} and also provide for
conversion to an equivalent \code{\link[MASS]{loglm}} specification or a
shorthand character string representation.

They allow for a more conceptual way to specify such models by a function
for their type, as opposed to just an uninterpreted list of model terms and
also allow easy specification of marginal models for a given contingency
table.
They are intended to be used as tools in higher-level modeling and graphics
functions, but can also be used directly.
}
\details{
The main model specification functions, \code{conditional}, \code{joint},
\code{markov}, \dots{}, \code{saturated}, return a list of vectors
indicating the marginal totals to be fit, via the \code{margin} argument to
\code{\link[stats]{loglin}}.
Each element of this list corresponds to a
high-order term in a hierarchical loglinear model, where, e.g., a term like
\code{c("A", "B")} is equivalent to the \code{\link[MASS]{loglm}} term
\code{"A:B"} and hence automatically includes all low-order terms.

Note that these can be used to supply the \code{expected} argument for the
default \code{\link[vcd]{mosaic}} function, when the data is supplied as a
contingency table.

The table below shows some typical results in terms of the standard
shorthand notation for loglinear models, with factors A, B, C, \dots{},
where brackets are used to delimit the high-order terms in the loglinear
model.
\tabular{llll}{
\strong{function} \tab \strong{3-way} \tab \strong{4-way} \tab \strong{5-way} \cr
\code{mutual} \tab  [A]  [B]  [C]  \tab  [A]  [B]  [C]  [D]  \tab [A]  [B]  [C]  [D]  [E] \cr
\code{joint}  \tab  [AB]  [C]  \tab  [ABC]  [D]  \tab  [ABCE]  [E]  \cr
\code{joint (with=1)} \tab   [A]  [BC]  \tab   [A]  [BCD]  \tab  [A]  [BCDE]  \cr
\code{conditional}  \tab  [AC]  [BC]  \tab  [AD]  [BD]  [CD]  \tab   [AE]  [BE]  [CE]  [DE] \cr
\code{condit (with=1)}  \tab  [AB]  [AC]  \tab   [AB]  [AC]  [AD]  \tab  [AB]  [AC]  [AD]  [AE] \cr
\code{markov (order=1)}  \tab   [AB]  [BC]  \tab   [AB]  [BC]  [CD]  \tab  [AB]  [BC]  [CD]  [DE] \cr
\code{markov (order=2)}  \tab   [A]  [B]  [C]  \tab  [ABC]  [BCD]  \tab   [ABC]  [BCD]  [CDE]  \cr
\code{saturated}  \tab [ABC] \tab [ABCD] \tab [ABCDE] \cr
}

\code{loglin2formula} converts the output of one of these to a model formula
suitable as the \code{formula} for of \code{\link[MASS]{loglm}}.

\code{loglin2string} converts the output of one of these to a string
describing the loglinear model in the shorthand bracket notation,
e.g., \code{"[A,B] [A,C]"}.
}
\seealso{
Other loglinear models:
\code{\link[=assoc_graph]{assoc_graph()}},
\code{\link[=get_model]{get_model()}},
\code{\link[=glmlist]{glmlist()}},
\code{\link[=plot.assoc_graph]{plot.assoc_graph()}},
\code{\link[=seq_loglm]{seq_loglm()}}
}
\concept{loglinear models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{HospVisits}
\alias{HospVisits}
\title{Hospital Visits Data}
\format{
A 3 by 3 frequency table, with format:
\preformatted{
table [1:3, 1:3] 43 6 9 16 11 18 3 10 16
- attr(*, "dimnames")=List of 2
..$ visit: chr [1:3] "Regular" "Infrequent" "Never"
..$ stay : chr [1:3] "2-9" "10-19" "20+"
}
}
\source{
Goodman, L. A. (1983) The analysis of dependence in
cross-classifications having ordered categories, using log-linear models for
frequencies and log-linear models for odds.  \emph{Biometrics}, 39, 149-160.
}
\description{
Length of stay in hospital for 132 schizophrenic patients, classified by
visiting patterns, originally from Wing (1962).
}
\details{
Both table variables can be considered ordinal. The variable \code{visit}
refers to visiting patterns recorded hospital.  The category labels are
abbreviations of those given by Goodman (1983); e.g., \code{"Regular"} is
short for \dQuote{received visitors regularly or patient went home}. The
variable \code{stay} refers to length of stay in hospital, in year groups.
}
\examples{

data(HospVisits)
mosaic(HospVisits, gp=shading_Friendly)

if(require(ca)){
  ca(HospVisits)
  # surprisingly 1D !
  plot(ca(HospVisits))
  }

}
\references{
Wing, J. K. (1962). Institutionalism in Mental Hospitals,
\emph{British Journal of Social and Clinical Psychology}, 1 (1), 38-51.
}
\seealso{
\code{\link[ca]{ca}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Draft1970}
\alias{Draft1970}
\title{USA 1970 Draft Lottery Data}
\format{
A data frame with 366 observations on the following 3 variables.
\describe{
\item{\code{Day}}{day of the year, 1:366}
\item{\code{Rank}}{draft priority rank of people born on that day}
\item{\code{Month}}{an ordered factor with levels \code{Jan} < \code{Feb} \dots < \code{Dec}}
}
}
\source{
Starr, N. (1997). Nonrandom Risk: The 1970 Draft Lottery,
\emph{Journal of Statistics Education}, v.5, n.2
\doi{10.1080/10691898.1997.11910534}
}
\description{
This data set gives the results of the 1970 US draft lottery, in the form of
a data frame.
}
\details{
The draft lottery was used to determine the order in which eligible men
would be called to the Selective Service draft. The days of the year
(including February 29) were represented by the numbers 1 through 366
written on slips of paper. The slips were placed in separate plastic
capsules that were mixed in a shoebox and then dumped into a deep glass jar.
Capsules were drawn from the jar one at a time.

The first number drawn was 258 (September 14), so all registrants with that
birthday were assigned lottery number \code{Rank} 1. The second number drawn
corresponded to April 24, and so forth.  All men of draft age (born 1944 to
1950) who shared a birthdate would be called to serve at once. The first 195
birthdates drawn were later called to serve in the order they were drawn;
the last of these was September 24.
}
\examples{

data(Draft1970)

# scatterplot
plot(Rank ~ Day, data=Draft1970)
with(Draft1970, lines(lowess(Day, Rank), col="red", lwd=2))
abline(lm(Rank ~ Day, data=Draft1970), col="blue")

# boxplots
plot(Rank ~ Month, data=Draft1970, col="bisque")

lm(Rank ~ Month, data=Draft1970)
anova(lm(Rank ~ Month, data=Draft1970))

# make the table version
Draft1970$Risk <- cut(Draft1970$Rank, breaks=3, labels=c("High", "Med", "Low"))
with(Draft1970, table(Month, Risk))

}
\references{
Fienberg, S. E. (1971), "Randomization and Social Affairs: The
1970 Draft Lottery," \emph{Science}, 171, 255-261.

\url{https://en.wikipedia.org/wiki/Draft_lottery_(1969)}

Hanley, J. A., "US Selective Service Lotteries 1917-1975: Datasets,"
\url{https://jhanley.biostat.mcgill.ca/DraftLotteries/}
}
\seealso{
\code{\link{Draft1970table}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Mammograms}
\alias{Mammograms}
\title{Mammogram Ratings}
\format{
A frequency table in matrix form.  The format is:
\preformatted{
  num [1:4, 1:4] 34 6 2 0 10 8 5 1 2 8 ...
- attr(*, "dimnames")=List of 2
..$ Reader2: chr [1:4] "Absent" "Minimal" "Moderate" "Severe"
..$ Reader1: chr [1:4] "Absent" "Minimal" "Moderate" "Severe"
}
}
\source{
Kundel, H. L. & Polansky, M. (2003), "Measurement of Observer
Agreement", \emph{Radiology}, \strong{228}, 303-308, Table A1
}
\description{
Kundel & Polansky (2003) give (possibly contrived) data on a set of 110
mammograms rated by two readers.
}
\examples{

data(Mammograms)
B <- agreementplot(Mammograms, main="Mammogram ratings")
# agreement measures
B
Kappa(Mammograms)

## other displays
mosaic(Mammograms, shade=TRUE)

sieve(Mammograms, pop = FALSE, shade = TRUE)
labeling_cells(text = Mammograms,
  gp_text = gpar(fontface = 2, cex=1.75))(as.table(Mammograms))

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.assoc_graph.R
\name{plot.assoc_graph}
\alias{plot.assoc_graph}
\title{Plot an Association Graph}
\usage{
\method{plot}{assoc_graph}(
  x,
  layout = NULL,
  groups = NULL,
  colors = c("lightblue", "lightyellow", "lightgreen", "lightsalmon", "plum"),
  vertex.size = 30,
  vertex.label.cex = 1.2,
  edge.width = 2,
  edge.label = NULL,
  ...
)
}
\arguments{
\item{x}{An \code{assoc_graph} object, as returned by \code{\link{assoc_graph}}.}

\item{layout}{Layout function or coordinate matrix for node positions.
Defaults to \code{\link[igraph]{layout_in_circle}} for up to 6 nodes,
\code{\link[igraph]{layout_with_fr}} otherwise.}

\item{groups}{Optional named list assigning variables to groups for coloring,
e.g., \code{list(response = "Survived", predictors = c("Class", "Sex", "Age"))}.}

\item{colors}{Character vector of colors for groups. Recycled as needed.}

\item{vertex.size}{Vertex size (default 30).}

\item{vertex.label.cex}{Label size for vertex names (default 1.2).}

\item{edge.width}{Edge width (default 2). If edge weights are present, widths are
scaled from the weights automatically.}

\item{edge.label}{Optional edge labels. If \code{TRUE} and edge weights are present,
the weight values are used as labels.}

\item{\dots}{Additional arguments passed to \code{\link[igraph]{plot.igraph}},
such as \code{main} for a title.}
}
\value{
The \code{assoc_graph} object \code{x}, returned invisibly.
}
\description{
Plot method for \code{\link{assoc_graph}} objects, displaying the association
structure of a loglinear model as a network diagram.
}
\examples{
# Basic structural plot
g <- conditional(3, factors = c("A", "B", "C")) |> assoc_graph()
plot(g, main = "Conditional independence: [AC] [BC]")

# With grouped node colors
g <- saturated(4, factors = c("A", "B", "C", "D")) |> assoc_graph()
plot(g, groups = list(c("A", "B"), c("C", "D")),
     main = "Saturated model")

}
\seealso{
\code{\link{assoc_graph}}, \code{\link[igraph]{plot.igraph}}

Other loglinear models:
\code{\link[=assoc_graph]{assoc_graph()}},
\code{\link[=get_model]{get_model()}},
\code{\link[=glmlist]{glmlist()}},
\code{\link[=joint]{joint()}},
\code{\link[=seq_loglm]{seq_loglm()}}
}
\concept{loglinear models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Abortion}
\alias{Abortion}
\title{Abortion Opinion Data}
\format{
A 3-dimensional array resulting from cross-tabulating 3 variables
for 1100 observations. The variable names and their levels are:

\tabular{rll}{
No \tab Name                    \tab Levels \cr
1  \tab \code{Sex}              \tab \verb{"Female", "Male"}\cr
2  \tab \code{Status}           \tab \verb{"Lo", "Hi"}\cr
3  \tab \code{Support_Abortion} \tab \verb{"Yes", "No"}\cr
}
}
\source{
Christensen, R. (1990).  \emph{Log-Linear Models}, New York, NY: Springer-Verlag, p. 92, Example 3.5.2.

Christensen, R. (1997).  \emph{Log-Linear Models and Logistic Regression},
New York, NY: Springer, p. 100, Example 3.5.2.
}
\description{
Opinions about abortion classified by gender and SES
}
\details{
\code{Support_Abortion} is a natural response variable.

The combinations of \code{Sex} and \code{Status} represent four independent
samples, having fixed \code{Sex}-\code{Status} marginal totals.  There were
500 females and 600 males. Within the female group, 250 of low status and
250 of high status were sampled. Similarly for the males, with 300 in each
of the low and hgh status sub-groups.

This is an example of a product-multinomial sampling scheme. the
\code{Sex:Status} association must be included in any loglinear model where
the goal is to determine how attitude toward abortion depends on the others.

Alternatively, a logit model for abortion support may provide a simpler
analysis.
}
\examples{

data(Abortion)


ftable(Abortion)
mosaic(Abortion, shade=TRUE)

# stratified by Sex
fourfold(aperm(Abortion, 3:1))
# stratified by Status
fourfold(aperm(Abortion, c(3,1,2)))


}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Detergent}
\alias{Detergent}
\title{Detergent Preference Data}
\format{
A 4-dimensional array resulting from cross-tabulating 4 variables
for 1008 observations. The variable names and their levels are:

\tabular{rll}{
dim \tab Name \tab Levels \cr
1\tab \code{Temperature}\tab \verb{"High", "Low"}\cr
2\tab \code{M_User}\tab \verb{"Yes", "No"}\cr
3\tab \code{Preference}\tab \verb{"Brand X", "Brand M"}\cr
4\tab \code{Water_softness}\tab \verb{"Soft", "Medium", "Hard"}\cr
}
}
\source{
Fienberg, S. E. (1980). \emph{The Analysis of
Cross-Classified Categorical Data} Cambridge, MA: MIT Press, p. 71.
}
\description{
Cross-classification of a sample of 1008 consumers according to (a) the
softness of the laundry water used, (b) previous use of detergent Brand M,
(c) the temperature of laundry water used and (d) expressed preference for
Brand X or Brand M in a blind trial.
}
\examples{

data(Detergent)

# basic mosaic plot
mosaic(Detergent, shade=TRUE)

require(MASS)
(det.mod0 <- loglm(~ Preference + Temperature + M_User + Water_softness,
                   data=Detergent))
# examine addition of two-way terms
add1(det.mod0, ~ .^2, test="Chisq")

# model for Preference as a response
(det.mod1 <- loglm(~ Preference + (Temperature * M_User * Water_softness),
                   data=Detergent))
mosaic(det.mod0)



}
\references{
Ries, P. N. & Smith, H. (1963). The use of
chi-square for preference testing in multidimensional problems.
\emph{Chemical Engineering Progress}, 59, 39-43.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zero.test.R
\name{zero.test}
\alias{zero.test}
\title{Score test for zero inflation in Poisson data}
\usage{
zero.test(x)
}
\arguments{
\item{x}{A vector of non-negative counts, or a one-way frequency table of
such counts.}
}
\value{
Returns invisibly a list of three elements:
\item{\code{statistic}}{Value of the test statistic}
\item{\code{df}}{Degrees of freedom}
\item{\code{pvalue}}{Upper tail p-value}
}
\description{
Carries out a simple score test (van den Broek, 1995) for excess zeros in an
otherwise Poisson distribution of counts.  It gives a \eqn{\chi^2_1}
statistic on one degree of freedom.
}
\details{
The test first calculates the rate estimate from the mean,
\eqn{\hat{\lambda} = \bar{x}}. The number of observed zeros, \eqn{n_0} is
then compared with the expected number, \eqn{n \hat{p_0}}, where
\eqn{\hat{p}_0=\exp[-\hat{\lambda}]}. Then the test statistic is calculated
by the formula:
\deqn{\frac{(n_0 - n\hat{p}_0)^2}{n\hat{p}_0(1-\hat{p}_0) - n\bar{x}\hat{p}_0^2}} .
This test statistic has a \eqn{\chi^2_1} distribution.
}
\examples{

# synthetic tests
zero.test(rpois(100, 1))
zero.test(rpois(100, 5))
# add some extra zeros
zero.test(c(rep(0, 20), rpois(100, 5)))

# Articles by Phd candidates
data(PhdPubs, package="vcdExtra")
zero.test(PhdPubs$articles)

phd.tab <- table(PhdPubs$articles)
zero.test(phd.tab)


}
\references{
The original R code came from a Stackexchange question,
\url{https://stats.stackexchange.com/questions/118322/how-to-test-for-zero-inflation-in-a-dataset}

Van den Broek, J. (1995).  A Score Test for Zero Inflation in a Poisson
Distribution.  \emph{Biometrics}, \strong{51}(2), 738-743.
https://www.jstor.org/stable/2532959

Yang, Zhao, James W. Hardin, and Cheryl L. Addy (2010).  Score Tests for
Zero-Inflation in Overdispersed Count Data.  \emph{Communications in
Statistics - Theory and Methods} \strong{39} (11) 2008-2030.
DOI: 10.1080/03610920902948228
}
\seealso{
Other association tests:
\code{\link[=CMHtest]{CMHtest()}},
\code{\link[=GKgamma]{GKgamma()}},
\code{\link[=HLtest]{HLtest()}},
\code{\link[=breslow_day_test]{breslow_day_test()}},
\code{\link[=woolf_test]{woolf_test()}}
}
\author{
Michael Friendly
}
\concept{association tests}
\keyword{htest}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{JobSat}
\alias{JobSat}
\title{Cross-classification of job satisfaction by income}
\format{
A 4 x 4 contingency table of \code{income} by \code{satisfaction},
with the following structure:
\preformatted{
  table [1:4, 1:4] 1 2 1 0 3 3 6 1 10 10 ...
  - attr(*, "dimnames")=List of 2
  ..$ income      : chr [1:4] "< 15k" "15-25k" "25-40k" "> 40k"
  ..$ satisfaction: chr [1:4] "VeryD" "LittleD" "ModerateS" "VeryS"
}
}
\source{
Agresti, A. Categorical Data Analysis John Wiley & Sons, 2002, Table
2.8, p. 57.
}
\description{
This data set is a contingency table of job satisfaction by income for a
small sample of black males from the 1996 General Social Survey, as used by
Agresti (2002) for an example.
}
\details{
Both \code{income} and \code{satisfaction} are ordinal variables, and are so
ordered in the table.  Measures of association, visualizations, and models
should take ordinality into account.
}
\examples{

data(JobSat)
assocstats(JobSat)
GKgamma(JobSat)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairs_diagonal_mosaic.R
\name{pairs_diagonal_mosaic}
\alias{pairs_diagonal_mosaic}
\alias{pairs.table}
\title{Diagonal Panel Function for Pairs Plots of Contingency Tables}
\usage{
pairs_diagonal_mosaic(
  split_vertical = TRUE,
  margins = unit(0, "lines"),
  offset_labels = -0.4,
  offset_varnames = 0,
  gp = NULL,
  fill = "grey",
  labeling = labeling_border,
  alternate_labels = TRUE,
  ...
)

\method{pairs}{table}(x, diag_panel = pairs_diagonal_mosaic, ...)
}
\arguments{
\item{split_vertical}{Logical; passed to \code{\link[vcd]{mosaic}}.
Default is \code{TRUE}.}

\item{margins}{A \code{\link[grid]{unit}} object giving the margins around
the mosaic. Default is \code{unit(0, "lines")}.}

\item{offset_labels}{Numeric; offset for the cell labels. Default is
\code{-0.4}.}

\item{offset_varnames}{Numeric; offset for the variable name labels.
Default is \code{0}.}

\item{gp}{A \code{\link[grid]{gpar}} object for the mosaic tiles, or
\code{NULL} (default) to use \code{fill}.}

\item{fill}{Either a color name or a function mapping an integer to a
vector of colors (e.g. a shading function). Default is \code{"grey"}.}

\item{labeling}{A labeling function such as
\code{\link[vcd]{labeling_border}} or \code{\link[vcd]{labeling_values}}.
Default is \code{\link[vcd]{labeling_border}} (no cell counts shown).}

\item{alternate_labels}{Logical; whether to alternate label positions on
the axes. Default is \code{TRUE}.}

\item{...}{Additional arguments passed to the \pkg{vcd} \code{pairs.table} method.}

\item{x}{A contingency table (object of class \code{"table"}).}

\item{diag_panel}{A \code{"grapcon_generator"} function or a panel function
for the diagonal cells. Defaults to the vcdExtra version of
\code{\link{pairs_diagonal_mosaic}}.}
}
\value{
A function of class \code{"grapcon_generator"} suitable for use as
the \code{diag_panel} argument in \code{\link[vcd]{pairs.table}}.
}
\description{
An enhanced replacement for \code{\link[vcd]{pairs_diagonal_mosaic}} from the
\pkg{vcd} package. This version fixes two bugs in the original (the
\code{labeling} and \code{alternate_labels} arguments were hardcoded and
ignored) and changes the default labeling to
\code{\link[vcd]{labeling_border}}.
}
\details{
A companion \code{pairs.table} method is also provided that uses this
improved function as the default diagonal panel.

The function follows the \code{"grapcon_generator"} pattern used throughout
\pkg{vcd}: calling \code{pairs_diagonal_mosaic(...)} returns a
\emph{function} suitable for passing as the \code{diag_panel} argument of
\code{\link[vcd]{pairs.table}}.

The original \code{vcd::pairs_diagonal_mosaic} has two bugs: (1) the
\code{labeling} argument is accepted but then hardcoded to
\code{labeling_values} inside the returned function, and (2)
\code{alternate_labels} is similarly hardcoded to \code{TRUE}.  This version
fixes both, making those arguments work as documented.

The default labeling scheme was changed from \code{\link[vcd]{labeling_values}} to
\code{\link[vcd]{labeling_border}}, so as to disable cell counts by default. To
enable cell counts, use \code{\link[vcd]{labeling_values}}.

\code{pairs.table} is an S3 method for \code{\link[graphics]{pairs}} that
overrides the version in \pkg{vcd} solely to change the default
\code{diag_panel} to the improved \code{pairs_diagonal_mosaic} defined in
this package. All actual rendering is delegated to the \pkg{vcd} implementation,
retrieved via \code{utils::getFromNamespace()} to avoid a \code{:::} check NOTE.
}
\examples{
data(Hoyt, package = "vcdExtra")
pred_tab <- margin.table(Hoyt, c("Rank", "Occupation", "Sex"))

# Default: no cell counts in diagonal panels
pairs(pred_tab)

# Show cell counts in diagonal panels
pairs(pred_tab, diag_panel_args = list(labeling = labeling_values))

# Use residual shading for off-diagonal panels
pairs(pred_tab, gp = shading_Friendly)

}
\seealso{
\code{\link[vcd]{pairs.table}}, \code{\link[vcd]{pairs_diagonal_mosaic}},
\code{\link[vcd]{labeling_border}}, \code{\link[vcd]{labeling_values}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Mobility}
\alias{Mobility}
\title{Social Mobility data}
\format{
A 2-dimensional array resulting from cross-tabulating 2 variables
for 19912 observations. The variable names and their levels are:

\tabular{rll}{
No \tab Name \tab Levels \cr
1\tab \verb{Son's_Occupation}\tab \verb{"UpNonMan", "LoNonMan", "UpManual", "LoManual", "Farm"}\cr
2\tab \verb{Father's_Occupation}\tab \verb{"UpNonMan", "LoNonMan", "UpManual", "LoManual", "Farm"}\cr
}
}
\source{
Falguerolles, A. de and Mathieu, J. R. (1988).  \emph{Proceedings of
COMPSTAT 88}, Copenhagen, Denmark, Springer-Verlag.

\% \cite{FeathermanHauser:78}

Featherman, D. L. and Hauser, R. M. Occupations and social mobility in the
United States.  \emph{Sociological Microjournal}, 12, Fiche 62. Copenhagen:
Sociological Institute.
}
\description{
Data on social mobility, recording the occupational category of fathers and
their sons.
}
\examples{

data(Mobility)
Mobility

# independence model
MASS::loglm(~Father_Occupation + Son_Occupation, data = Mobility)

vcd::mosaic(Mobility, shade=TRUE, legend = FALSE)



}
\seealso{
\code{\link{Glass}}, \code{\link{Hauser79}},
\code{\link{Yamaguchi87}} for other examples of mobility data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Heart}
\alias{Heart}
\title{Sex, Occupation and Heart Disease}
\format{
A 3-dimensional array resulting from cross-tabulating 3 variables
for 21522 observations. The variable names and their levels are:

\tabular{rll}{
No \tab Name \tab Levels \cr
1\tab \code{Disease}\tab \verb{"Disease", "None"}\cr
2\tab \code{Gender}\tab \verb{"Male", "Female"}\cr
3\tab \code{Occup}\tab \verb{"Unempl", "WhiteCol", "BlueCol"}\cr
}
}
\source{
\% \cite{Karger, 1980}
Karger, (1980).
}
\description{
Classification of individuals by gender, occupational category and
occurrence of heart disease
}
\examples{

data(Heart)
str(Heart)

# Display the frequencies for occupational categories.
# Each row is a 2 x 2 table
vcd::structable(Disease + Gender ~ Occup, data=Heart)

# display as fourfold plots
vcd::cotabplot(~ Disease + Gender | Occup, data=Heart, panel = cotab_fourfold)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vcdExtra-package.R
\name{vcdExtra-package}
\alias{vcdExtra-package}
\alias{vcdExtra}
\alias{_PACKAGE}
\title{Extensions and additions to vcd: Visualizing Categorical Data}
\description{
\if{html}{\figure{man/figures/logo.png}{options: alt='logo' width='100'}}
}
\details{
This package provides additional data sets, documentation, and a few
functions designed to extend the \code{vcd} package for Visualizing
Categorical Data and the \code{gnm} package for Generalized Nonlinear
Models. In particular, vcdExtra extends mosaic, assoc and sieve plots from
vcd to handle glm() and gnm() models and adds a 3D version in
\code{\link{mosaic3d}}.

This package is also a support package for the book, \emph{Discrete Data
Analysis with R} by Michael Friendly and David Meyer, Chapman & Hall/CRC,
2016,
\url{https://www.routledge.com/Discrete-Data-Analysis-with-R-Visualization-and-Modeling-Techniques-for-Categorical-and-Count-Data/Friendly-Meyer/p/book/9781498725835}
with a number of additional data sets, and functions. The web site for the
book is \url{http://ddar.datavis.ca}.

In addition, I teach a course, \emph{Psy 6136: Categorical Data Analysis},
\url{https://friendly.github.io/psy6136/} using this package.

The main purpose of this package is to serve as a sandbox for introducing
extensions of mosaic plots and related graphical methods that apply to
loglinear models fitted using \code{glm()} and related, generalized
nonlinear models fitted with \code{gnm()} in the
\code{\link[gnm]{gnm-package}} package. A related purpose is to fill in some
holes in the analysis of categorical data in R, not provided in base R, the
\pkg{vcd}, or other commonly used packages.

The method \code{\link{mosaic.glm}} extends the
\code{\link[vcd]{mosaic.loglm}} method in the \pkg{vcd} package to this
wider class of models.  This method also works for the generalized nonlinear
models fit with the \code{\link[gnm]{gnm-package}} package, including models
for square tables and models with multiplicative associations.

\code{\link{mosaic3d}} introduces a 3D generalization of mosaic displays
using the \pkg{rgl} package.

In addition, there are several new data sets, a tutorial vignette,
\describe{ \item{vcd-tutorial}{Working with categorical data with R and the
vcd package, \code{vignette("vcd-tutorial", package = "vcdExtra") }} } and a
few functions for manipulating categorical data sets and working with models
for categorical data.

A new class, \code{\link{glmlist}}, is introduced for working with
collections of \code{glm} objects, e.g., \code{\link{Kway}} for fitting all
K-way models from a basic marginal model, and \code{\link{LRstats}} for
brief statistical summaries of goodness-of-fit for a collection of models.

For square tables with ordered factors, \code{\link{Crossings}} supplements
the specification of terms in model formulas using \code{\link[gnm]{Symm}},
\code{\link[gnm]{Diag}}, \code{\link[gnm]{Topo}}, etc. in the
\code{\link[gnm]{gnm-package}}.

Some of these extensions may be migrated into \pkg{vcd} or \pkg{gnm}.

A collection of demos is included to illustrate fitting and visualizing a
wide variety of models: \describe{ \item{mental-glm}{Mental health data:
mosaics for glm() and gnm() models} \item{occStatus}{Occupational status
data: Compare mosaic using expected= to mosaic.glm}
\item{ucb-glm}{UCBAdmissions data: Conditional independence via loglm() and
glm()} \item{vision-quasi}{VisualAcuity data: Quasi- and Symmetry models}
\item{yaish-unidiff}{Yaish data: Unidiff model for 3-way table}
\item{Wong2-3}{Political views and support for women to work (U, R, C, R+C
and RC(1) models)} \item{Wong3-1}{Political views, support for women to work
and national welfare spending (3-way, marginal, and conditional independence
models)} \item{housing}{Visualize glm(), multinom() and polr() models from
\code{example(housing, package="MASS")}} } Use \code{ demo(package="vcdExtra")} for a complete current list.

The \pkg{vcdExtra} package now contains a large number of data sets
illustrating various forms of categorical data analysis and related
visualizations, from simple to advanced. Use \code{data(package="vcdExtra")}
for a complete list, or \code{datasets(package="vcdExtra")} for an annotated
one showing the \code{class} and \code{dim} for each data set.
}
\examples{

example(mosaic.glm)

demo("mental-glm")

}
\references{
Friendly, M. \emph{Visualizing Categorical Data}, Cary NC: SAS
Institute, 2000. Web materials: \url{http://www.datavis.ca/books/vcd/}.

Friendly, M. and Meyer, D. (2016). \emph{Discrete Data Analysis with R:
Visualization and Modeling Techniques for Categorical and Count Data}. Boca
Raton, FL: Chapman & Hall/CRC. \url{http://ddar.datavis.ca}.

Meyer, D.; Zeileis, A. & Hornik, K. The Strucplot Framework: Visualizing
Multi-way Contingency Tables with vcd \emph{Journal of Statistical
Software}, 2006, \strong{17}, 1-48. Available in R via
\code{vignette("strucplot", package = "vcd")}

Turner, H. and Firth, D. \emph{Generalized nonlinear models in R: An
overview of the gnm package}, 2007, \url{http://eprints.ncrm.ac.uk/472/}.
Available in R via \code{vignette("gnmOverview", package = "gnm")}.
}
\seealso{
\code{\link[gnm]{gnm-package}}, for an extended range of models for
contingency tables

\code{\link[vcd]{mosaic}} for details on mosaic displays within the
strucplot framework.
}
\author{
Michael Friendly

Maintainer: Michael Friendly
(\href{https://orcid.org/0000-0002-3237-0941}{ORCID})
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mosaic.glmlist.R
\name{mosaic.glmlist}
\alias{mosaic.glmlist}
\alias{mosaic.loglmlist}
\title{Mosaic Displays for glmlist and loglmlist Objects}
\usage{
\method{mosaic}{glmlist}(
  x,
  selection,
  panel = mosaic,
  type = c("observed", "expected"),
  legend = ask | !missing(selection),
  main = NULL,
  ask = TRUE,
  graphics = TRUE,
  rows,
  cols,
  newpage = TRUE,
  ...
)

\method{mosaic}{loglmlist}(
  x,
  selection,
  panel = mosaic,
  type = c("observed", "expected"),
  legend = ask | !missing(selection),
  main = NULL,
  ask = TRUE,
  graphics = TRUE,
  rows,
  cols,
  newpage = TRUE,
  ...
)
}
\arguments{
\item{x}{a \code{glmlist} or \code{loglmlist} object}

\item{selection}{the index or name of one \code{glm} or \code{loglm} object
in \code{x}.  If no selection is specified, a menu of models is presented or
all models are plotted.}

\item{panel}{a \code{\link[vcd]{strucplot}} panel function, typically
\code{\link[vcd]{mosaic}} or \code{\link[vcd]{sieve}}}

\item{type}{a character string indicating whether the \code{"observed"} or
the \code{"expected"} values of the table should be visualized}

\item{legend}{logical: show a legend for residuals in the mosaic display(s)?
The default behavior is to include a legend when only a single plot is
shown, i.e., if \code{ask} is \code{TRUE} or a \code{selection} has been
specified.}

\item{main}{either a logical, or a vector of character strings used for
plotting the main title.  If main is a logical and \code{TRUE}, the name of
the selected glm object is used.}

\item{ask}{logical: should the function display a menu of models, when one
is not specified in \code{selection}? If \code{selection} is not supplied
and \code{ask} is \code{TRUE} (the default), a menu of model names is
presented; if \code{ask} is \code{FALSE}, mosaics for all models are plotted
in an array.}

\item{graphics}{logical: use a graphic dialog box when \code{ask=TRUE}?}

\item{rows, cols}{when \code{ask=FALSE}, the number of rows and columns in
which to plot the mosaics.}

\item{newpage}{start a new page? (only applies to \code{ask=FALSE})}

\item{\dots}{other arguments passed to \code{\link{mosaic.glm}} and
ultimately to \code{\link[vcd]{mosaic}}.}
}
\value{
Returns the result of \code{\link{mosaic.glm}}.
}
\description{
This function provides a convenient interface for viewing mosaic displays
associated with a collection of glm models for frequency tables that have
been stored in a \code{glmlist} or \code{loglmlist} object.  You can plot
either selected models individually, or mosaics for all models in an array
of viewports.
}
\details{
Most details of the plots produced can be controlled via \dots{} arguments
as shown in some of the examples below.  In particular, with
\code{panel=sieve} you need to also pass \code{gp=shading_Friendly} to get a
color version.
}
\examples{

data(JobSatisfaction, package="vcd")

# view all pairwise mosaics
pairs(xtabs(Freq~management+supervisor+own, data=JobSatisfaction),
    shade=TRUE, diag_panel=pairs_diagonal_mosaic)

modSat <- Kway(Freq ~ management+supervisor+own, data=JobSatisfaction,
               family=poisson, prefix="JobSat")
names(modSat)

\dontrun{
mosaic(modSat)              # uses menu, if interactive()
}
mosaic(modSat, "JobSat.1")  # model label
mosaic(modSat, 2)           # model index

# supply a formula to determine the order of variables in the mosaic
mosaic(modSat, 2, formula=~own+supervisor+management)

mosaic(modSat, ask=FALSE)   # uses viewports

# use a different panel function, label the observed valued in the cells
mosaic(modSat, 1, main=TRUE, panel=sieve, gp=shading_Friendly, labeling=labeling_values)

data(Mental)
indep <- glm(Freq ~ mental+ses,
                family = poisson, data = Mental)
Cscore <- as.numeric(Mental$ses)
Rscore <- as.numeric(Mental$mental)

coleff <- glm(Freq ~ mental + ses + Rscore:ses,
                family = poisson, data = Mental)
roweff <- glm(Freq ~ mental + ses + mental:Cscore,
                family = poisson, data = Mental)
linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
                family = poisson, data = Mental)

# assign names for the plot labels
modMental <- glmlist(Indep=indep, ColEff=coleff, RowEff=roweff, `Lin x Lin`=linlin)
mosaic(modMental, ask=FALSE, margins=c(3,1,1,2), labeling_args=list(abbreviate_labs=5))



}
\references{
David Meyer, Achim Zeileis, and Kurt Hornik (2006). The
Strucplot Framework: Visualizing Multi-Way Contingency Tables with vcd.
\emph{Journal of Statistical Software}, 17(3), 1-48.
\url{https://www.jstatsoft.org/v17/i03/},
available as \code{vignette("strucplot", package="vcd")}.
}
\seealso{
\code{\link{glmlist}}, \code{\link{loglmlist}}, \code{\link{Kway}}

\code{\link{mosaic.glm}}, \code{\link[vcd]{mosaic}},
\code{\link[vcd]{strucplot}}, for the many parameters that control the
details of mosaic plots.

Other mosaic plots:
\code{\link[=mosaic.glm]{mosaic.glm()}},
\code{\link[=mosaic3d]{mosaic3d()}}

Other glmlist functions:
\code{\link[=Kway]{Kway()}},
\code{\link[=LRstats]{LRstats()}},
\code{\link[=get_model]{get_model()}},
\code{\link[=glmlist]{glmlist()}}
}
\author{
Michael Friendly
}
\concept{glmlist functions}
\concept{mosaic plots}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Cancer}
\alias{Cancer}
\title{Survival of Breast Cancer Patients}
\format{
A 3-dimensional array resulting from cross-tabulating 3 variables
for 474 observations. The variable names and their levels are:

\tabular{rll}{
dim \tab Name \tab Levels \cr
1\tab \code{Survival}\tab  \verb{"Died", "Surv"}\cr
2\tab \code{Grade}\tab \verb{"Malignant", "Benign"}\cr
3\tab \code{Center}\tab \verb{"Boston", "Glamorgan"}\cr
}
}
\source{
Lindsey, J. K. (1995).
Analysis of Frequency and Count Data Oxford, UK: Oxford University Press. p.
38, Table 2.5.

Whittaker, J. (1990) Graphical Models in Applied Multivariate Statistics New
York: John Wiley and Sons, p. 220.
}
\description{
Three year survival of 474 breast cancer patients according to nuclear grade
and diagnostic center.
}
\examples{

data(Cancer)

MASS::loglm(~Survival + Grade + Center, data = Cancer)

vcd::mosaic(Cancer, shade=TRUE)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_caseform.R
\name{as_caseform}
\alias{as_caseform}
\title{Convert frequency or table form into case form.}
\usage{
as_caseform(obj, freq = "Freq", dims = NULL, tidy = TRUE)
}
\arguments{
\item{obj}{Object to be converted to case form.}

\item{freq}{If \code{obj} is in frequency form, this is the name of the frequency column. If
\code{obj} is in any other form, do not supply an argument (see "Details").}

\item{dims}{A character vector of dimensions. If not specified, all variables apart from
\code{freq} will be used as dimensions.}

\item{tidy}{Returns a tibble if set to \code{TRUE}.}
}
\value{
object in case form.
}
\description{
Converts object (\code{obj}) in frequency or table form into case form. The
column containing the frequencies (\code{freq}) must be supplied if \code{obj} is in
frequency form. Returns a tibble if \code{tidy} is set to \code{TRUE}.
}
\details{
A wrapper for \code{\link{expand.dft}} that is able to handle arrays.

If a frequency column is not supplied, this function defaults to "Freq"
just like \code{\link{expand.dft}}. Converts \code{obj} to a table using
\code{\link{as_table}} before converting to case form.
}
\examples{
library(vcdExtra)

data("HairEyeColor")

freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
tableForm <- as_table(HairEyeColor) # Generate table form data
arrayDat <- as_array(HairEyeColor) # Generate an array

# Frequency form -> case form
as_caseform(freqForm) |> str()

# Frequency form (tibble) -> case form
as_caseform(tidy_freqForm, freq = "n") |> str()

# Array -> case form
as_caseform(arrayDat) |> str()

# Optionally specify dims
as_caseform(tableForm, dims = c("Hair", "Eye")) |> str()


}
\seealso{
\code{\link{as_table}}, \code{\link{as_freqform}}, \code{\link{as_array}},
\code{\link{as_matrix}}, \code{\link{expand.dft}}
}
\author{
Gavin M. Klorfine
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcaplot.R
\name{mcaplot}
\alias{mcaplot}
\title{Simple and enhanced plot of MCA solutions}
\usage{
mcaplot(
  obj,
  map = "symmetric",
  dim = 1:2,
  col = c("blue", "red", "brown", "black", "green3", "purple"),
  pch = 15:20,
  cex = 1.2,
  pos = 3,
  lines = TRUE,
  lwd = 2,
  legend = FALSE,
  legend.pos = "topright",
  xlab = "_auto_",
  ylab = "_auto_",
  rev.axes = c(FALSE, FALSE),
  ...
)
}
\arguments{
\item{obj}{An \code{"mjca"} object}

\item{map}{Character string specifying the map type, i.e., the scaling
applied to coordinates for different types of MCA representations. Allowed
options include: \code{"symmetric"} (default), \code{"rowprincipal"},
\code{"colprincipal"},
\code{"symbiplot"},
\code{"rowgab"},
\code{"colgab"},
\code{"rowgreen"},
\code{"colgreen"}.
See \code{\link[ca]{mjca}} for details.}

\item{dim}{Dimensions to plot, an integer vector of length 2}

\item{col}{Vector of colors, one for each factor in the MCA}

\item{pch}{Vector of point symbols for the category levels, one for each factor}

\item{cex}{Character size for points and level labels}

\item{pos}{Position of level labels relative to the category points; either
a single number or a vector of length equal to the number of category points.}

\item{lines}{A logical or an integer vector indicating which factors are to
be joined with lines using \code{\link[ca]{multilines}}}

\item{lwd}{Line width(s) for the lines}

\item{legend}{Logical; draw a legend for the factor names?}

\item{legend.pos}{Position of the legend in the plot, as in \code{\link[graphics]{legend}}}

\item{xlab, ylab}{Labels for horizontal and vertical axes.  The default,
\code{"_auto_"} means that the function auto-generates a label of the form
\code{"Dimension X (xx.x \\\%)"}}

\item{rev.axes}{A logical vector of length 2, where TRUE reverses the
direction of the corresponding axis}

\item{\dots}{Arguments passed down to \code{plot}}
}
\value{
Returns the coordinates of the category points invisibly
}
\description{
This function is intended as an alternative to \code{\link[ca]{plot.mjca}}
for plotting multiple correspondence analysis solutions. It provides more
flexibility for labeling factor levels and connecting them with lines. It
does not support some features of \code{plot.mjca} (centroids, supplementary
points, arrows, etc.)
}
\examples{

require(ca)
data(Titanic)
titanic.mca <- suppressWarnings(mjca(Titanic))     # ca pkg uses old version of expand.dft()
mcaplot(titanic.mca, legend=TRUE, legend.pos="topleft")

data(HairEyeColor)
haireye.mca <- suppressWarnings(mjca(HairEyeColor))
mcaplot(haireye.mca, legend=TRUE, cex.lab=1.3)



}
\seealso{
\code{\link[ca]{mjca}}, \code{\link[ca]{plot.mjca}}
\code{\link[ca]{cacoord}} returns CA and MCA coordinates,
\code{\link[ca]{multilines}} draw multiple lines according to a factor,
}
\author{
Michael Friendly
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLik.loglm.R
\name{logLik.loglm}
\alias{logLik.loglm}
\title{Log-Likelihood of a loglm Object}
\usage{
\method{logLik}{loglm}(object, ..., zero = 1e-10)
}
\arguments{
\item{object}{A \code{loglm} object}

\item{\dots}{For compatibility with the S3 generic; not used here}

\item{zero}{value used to replace zero frequencies in calculating the
log-likelihood}
}
\value{
Returns an object of class \code{logLik}.  This is a number with one
attribute, \code{"df"} (degrees of freedom), giving the number of
(estimated) parameters in the model.
}
\description{
Calculates the log-likelihood value of the \code{loglm} model represented by
\code{object} evaluated at the estimated coefficients.
}
\details{
It allows the use of \code{\link[stats]{AIC}} and \code{\link[stats]{BIC}},
which require that a \code{logLik} method exists to extract the
corresponding log-likelihood for the model.

If cell frequencies have not been stored with the \code{loglm} object (via
the argument \code{keep.frequencies = TRUE}), they are obtained using
\code{update}.

This function calculates the log-likelihood in a way that allows for
non-integer frequencies, such as the case where 0.5 has been added to all
cell frequencies to allow for sampling zeros.  If the frequencies still
contain zero values, those are replaced by the value of \code{start}.

For integer frequencies, it gives the same result as the corresponding model
fit using \code{\link[stats]{glm}}, whereas \code{\link[stats]{glm}} returns
\code{-Inf} if there are any non-integer frequencies.
}
\examples{

data(Titanic, package="datasets")

require(MASS)
titanic.mod1 <- loglm(~ (Class * Age * Sex) + Survived, data=Titanic)
titanic.mod2 <- loglm(~ (Class * Age * Sex) + Survived*(Class + Age + Sex), data=Titanic)
titanic.mod3 <- loglm(~ (Class * Age * Sex) + Survived*(Class + Age * Sex), data=Titanic)

logLik(titanic.mod1)
AIC(titanic.mod1, titanic.mod2, titanic.mod3)
BIC(titanic.mod1, titanic.mod2, titanic.mod3)

# compare with models fit using glm()
titanic <- as.data.frame(Titanic)
titanic.glm1 <- glm(Freq ~ (Class * Age * Sex) + Survived,
                    data=titanic, family=poisson)
titanic.glm2 <- glm(Freq ~ (Class * Age * Sex) + Survived*(Class + Age + Sex),
                    data=titanic, family=poisson)
titanic.glm3 <- glm(Freq ~ (Class * Age * Sex) + Survived*(Class + Age * Sex),
                    data=titanic, family=poisson)

logLik(titanic.glm1)
AIC(titanic.glm1, titanic.glm2, titanic.glm3)
BIC(titanic.glm1, titanic.glm2, titanic.glm3)


}
\seealso{
\code{\link[MASS]{loglm}}, \code{\link[stats]{AIC}},
\code{\link[stats]{BIC}},
}
\author{
Achim Zeileis
}
\keyword{htest}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Titanicp}
\alias{Titanicp}
\title{Passengers on the Titanic}
\format{
A data frame with 1309 observations on the following 6 variables.
\describe{
\item{\code{pclass}}{a factor with levels \verb{1st} \verb{2nd} \verb{3rd}}
\item{\code{survived}}{a factor with levels \code{died} \code{survived}}
\item{\code{sex}}{a factor with levels \code{female} \code{male}}
\item{\code{age}}{passenger age in years (or fractions of a year, for children), a numeric vector; age is missing for 263 of the passengers}
\item{\code{sibsp}}{number of siblings or spouses aboard, integer: \code{0:8}}
\item{\code{parch}}{number of parents or children aboard, integer: \code{0:6}}
}
}
\source{
The original R source for this dataset was compiled by Frank Harrell and
Robert Dawson:
\url{https://hbiostat.org/data/repo/titanic.txt},
described in more detail in
\url{https://hbiostat.org/data/repo/titanic}

For this version of the Titanic data, passenger details were deleted,
survived was cast as a factor, and the name changed to \code{Titanicp} to
minimize confusion with other versions.
}
\description{
Data on passengers on the RMS Titanic, excluding the Crew and some
individual identifier variables.
}
\details{
There are a number of related versions of the Titanic data, in various
formats. This version was derived from \code{ptitanic} in the
\pkg{rpart.plot} package, modifying it to remove the \verb{Class 'labelled'}
attributes for some variables (inherited from Frank Harrell's
\code{titanic3} version) which caused problems with some applications,
notably \code{ggplot2}.

Other versions:

\code{\link[datasets]{Titanic}} is the 4-way frequency table of all 2201
people aboard the Titanic, including passengers and crew.
}
\examples{

data(Titanicp)
## maybe str(Titanicp) ; plot(Titanicp) ...

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapse.table.R
\name{collapse.table}
\alias{collapse.table}
\title{Collapse Levels of a Table}
\usage{
collapse.table(table, ...)
}
\arguments{
\item{table}{A \code{\link[base]{table}}, \code{\link[base]{array}} or
\code{\link[stats]{ftable}} object}

\item{\dots}{A collection of one or more assignments of factors of the table
to a list of levels}
}
\value{
A \code{xtabs} and \code{table} object, representing the original
table with one or more of its factors collapsed or rearranged into other
levels.
}
\description{
Collapse (or re-label) variables in a a contingency table, array or
\code{ftable} object by re-assigning levels of the table variables.
}
\details{
Each of the \code{\dots{}} arguments must be of the form \code{variable = levels}, where \code{variable} is the name of one of the table dimensions,
and \code{levels} is a character or numeric vector of length equal to the
corresponding dimension of the table.
}
\examples{

# create some sample data in table form
sex <- c("Male", "Female")
age <- letters[1:6]
education <- c("low", 'med', 'high')
data <- expand.grid(sex=sex, age=age, education=education)
counts <- rpois(36, 100) 
data <- cbind(data, counts)
t1 <- xtabs(counts ~ sex + age + education, data=data)
structable(t1)

##                  age   a   b   c   d   e   f
## sex    education                            
## Male   low           119 101 109  85  99  93
##        med            94  98 103 108  84  84
##        high           81  88  96 110 100  92
## Female low           107 104  95  86 103  96
##        med           104  98  94  95 110 106
##        high           93  85  90 109  99  86


# collapse age to 3 levels
t2 <- collapse.table(t1, age=c("A", "A", "B", "B", "C", "C"))
structable(t2)

##                  age   A   B   C
## sex    education                
## Male   low           220 194 192
##        med           192 211 168
##        high          169 206 192
## Female low           211 181 199
##        med           202 189 216
##        high          178 199 185


# collapse age to 3 levels and pool education: "low" and "med" to "low"
t3 <- collapse.table(t1, age=c("A", "A", "B", "B", "C", "C"), 
    education=c("low", "low", "high"))
structable(t3)

##                  age   A   B   C
## sex    education                
## Male   low           412 405 360
##        high          169 206 192
## Female low           413 370 415
##        high          178 199 185



# change labels for levels of education to 1:3
t4 <- collapse.table(t1,  education=1:3)
structable(t4)

structable(t4)
##                  age   a   b   c   d   e   f
## sex    education                            
## Male   1             119 101 109  85  99  93
##        2              94  98 103 108  84  84
##        3              81  88  96 110 100  92
## Female 1             107 104  95  86 103  96
##        2             104  98  94  95 110 106
##        3              93  85  90 109  99  86




}
\seealso{
\code{\link{expand.dft}} and \code{\link{as_caseform}}: expands a frequency
data frame to case form.

\code{\link[base]{margin.table}} "collapses" a table in a different way, by
summing over table dimensions.

\code{\link{collapse_levels}} collapses in the same manner as
\code{collapse.table} but also works for frequency and case form data.
}
\author{
Michael Friendly
}
\keyword{attribute}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_matrix.R
\name{as_matrix}
\alias{as_matrix}
\title{Convert frequency, case, or table form data into a matrix.}
\usage{
as_matrix(obj, freq = NULL, dims = NULL, prop = NULL)
}
\arguments{
\item{obj}{Object to be converted into a matrix.}

\item{freq}{If \code{obj} is in frequency form, this is the name of the frequency column.
Leave as \code{NULL} if \code{obj} is in any other form.}

\item{dims}{A character vector of dimensions. If not specified, all variables apart from
\code{freq} will be used as dimensions.}

\item{prop}{If set to \code{TRUE}, returns a matrix of proportions (that sum to 1). May also
be set to a character or numeric vector of dimensions to be used as margins
from which proportions will be computed.}
}
\value{
Object in matrix form.
}
\description{
Converts object (\code{obj}) in frequency, case or table form into a matrix of
specified dimensions (\code{dims}). The column containing the frequencies (\code{freq})
must be supplied if \code{obj} is in frequency form.
}
\details{
First converts \code{obj} into an array using \code{\link{as_array}}. Then a
check is made to ensure the user inputted a 2D \code{obj}. If \code{obj} is not 2D, an
error is returned. If \code{obj} is 2D, \code{\link[base]{as.matrix}} is applied.
}
\examples{
library(vcdExtra)

data("HairEyeColor")

freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
caseForm <- expand.dft(freqForm) # Generate case form data
arrayDat <- as_array(HairEyeColor) # Generate an array

# Table form -> matrix
as_matrix(HairEyeColor, dims = c("Hair", "Sex")) |> str()

# Frequency form -> matrix
as_matrix(freqForm, freq = "Freq", dims = c("Hair", "Sex")) |> str()

# Case form -> matrix form
as_matrix(caseForm, dims = c("Hair", "Sex")) |> str()

# Frequency (tibble) form -> matrix form
as_matrix(tidy_freqForm, freq = "n", dims = c("Hair", "Sex")) |> str()

#-----For proportions-----#

# Proportions relative to grand total
as_matrix(HairEyeColor, dims = c("Hair", "Sex"), prop = TRUE)

# Marginalize proportions along "Sex" (i.e., male proportions sum to 1, 
# female proportions sum to 1)
as_matrix(HairEyeColor, dims = c("Hair", "Sex"), prop = "Sex")

as_matrix(HairEyeColor, dims = c("Hair", "Sex"), prop = 2) # Same as above


}
\seealso{
\code{\link{as_array}}, \code{\link{as_table}}, \code{\link{as_freqform}},
\code{\link{as_caseform}}
}
\author{
Gavin M. Klorfine
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Bartlett}
\alias{Bartlett}
\title{Bartlett Data on Plum Root Cuttings}
\format{
A 3-dimensional array resulting from cross-tabulating 3 variables
for 960 observations. The variable names and their levels are:

\tabular{rll}{
dim \tab Name          \tab Levels                \cr
1  \tab \code{Alive}  \tab \verb{"Alive", "Dead"}\cr
2  \tab \code{Time}   \tab \verb{"Now", "Spring"}\cr
3  \tab \code{Length} \tab \verb{"Long", "Short"}\cr
}
}
\source{
Hand, D. and Daly, F. and Lunn, A. D.and McConway, K. J. and Ostrowski, E. (1994). \emph{A Handbook of Small Data Sets}.
London: Chapman & Hall, p. 15, # 19.
}
\description{
In an experiment to investigate the effect of cutting length (two levels)
and planting time (two levels) on the survival of plum root cuttings, 240
cuttings were planted for each of the 2 x 2 combinations of these factors,
and their survival was later recorded.
}
\details{
Bartlett (1935) used these data to illustrate a method for testing for no
three-way interaction in a contingency table.
}
\examples{

data(Bartlett)

# measures of association
assocstats(Bartlett)
oddsratio(Bartlett)

# Test models

## Independence
MASS::loglm(formula = ~Alive + Time + Length, data = Bartlett)

## No three-way association
MASS::loglm(formula = ~(Alive + Time + Length)^2, data = Bartlett)

# Use woolf_test() for a formal test of homogeneity of odds ratios
vcd::woolf_test(Bartlett)


# Plots
fourfold(Bartlett, mfrow=c(1,2))

mosaic(Bartlett, shade=TRUE)
pairs(Bartlett, gp=shading_Friendly)

}
\references{
Bartlett, M. S. (1935).  Contingency Table Interactions \emph{Journal of the Royal Statistical Society}, Supplement,
1935, 2, 248-252.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update.xtabs.R
\name{update.xtabs}
\alias{update.xtabs}
\title{Update method for a \code{xtabs} object}
\usage{
\method{update}{xtabs}(object, formula., ..., evaluate = TRUE)
}
\arguments{
\item{object}{An existing \code{"xtabs"} object}

\item{formula.}{Changes to the formula ? see
\code{\link[stats]{update.formula}} for details}

\item{\dots}{Additional arguments to the call, or arguments with changed
values.}

\item{evaluate}{If \code{TRUE}, evaluate the new call else return the call}
}
\value{
If \code{evaluate == TRUE}, the new \code{"xtabs"} object, otherwise
the updated call
}
\description{
Provides an \code{update} method for \code{"xtabs"} objects, typically by
removing terms from the formula to collapse over them.
}
\examples{

vietnam.tab <- xtabs(Freq ~ sex + year + response, data=Vietnam)

update(vietnam.tab, formula = ~ . -year)



}
\seealso{
\code{\link[stats]{update.formula}} for details on updates to model
formulae

\code{\link[base]{margin.table}} does something similar,
\code{\link{collapse.table}} collapses category levels
}
\author{
Michael Friendly
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Accident}
\alias{Accident}
\title{Traffic Accident Victims in France in 1958}
\format{
A data frame in frequency form (comprising a 5 x 2 x 4 x 2 table)
with 80 observations on the following 5 variables.
\describe{
\item{\code{age}}{an ordered factor with levels \code{0-9} < \code{10-19} <\code{20-29}
< \code{30-49} < \verb{50+}}
\item{\code{result}}{a factor with levels \code{Died} \code{Injured}}
\item{\code{mode}}{mode of transportation, a factor with levels \code{4-Wheeled} \code{Bicycle}
\code{Motorcycle} \code{Pedestrian}}
\item{\code{gender}}{a factor with levels \code{Female} \code{Male}}
\item{\code{Freq}}{a numeric vector}
}
}
\source{
Bertin (1983), p. 30; original data from the Ministere des Travaux
Publics
}
\description{
Bertin (1983) used these data to illustrate the cross-classification of data
by numerous variables, each of which could have various types and could be
assigned to various visual attributes.
}
\details{
For modeling and visualization purposes, the data can be treated as a 4-way
table using loglinear models and mosaic displays, or as a frequency-weighted
data frame using a binomial response for \code{result} (\code{"Died"} vs.
\code{"Injured"}) and plots of predicted probabilities.

\code{age} is an ordered factor, but arguably, \code{mode} should be treated
as ordered, with levels \code{Pedestrian} < \code{Bicycle} <
\code{Motorcycle} < \code{4-Wheeled} as Bertin does.  This affects the
parameterization in models, so we don't do this directly in the data frame.
}
\examples{

# examples
data(Accident)
head(Accident)

# for graphs, reorder mode
Accident$mode <- ordered(Accident$mode,
   levels=levels(Accident$mode)[c(4,2,3,1)])

# Bertin's table
accident_tab <- xtabs(Freq ~ gender + mode + age + result, data=Accident)
structable(mode + gender ~ age + result, data=accident_tab)

## Loglinear models
## ----------------

# mutual independence
acc.mod0 <- glm(Freq ~ age + result + mode + gender,
                data=Accident,
                family=poisson)
LRstats(acc.mod0)

mosaic(acc.mod0, ~mode + age + gender + result)

# result as a response
acc.mod1 <- glm(Freq ~ age*mode*gender + result,
                data=Accident,
                family=poisson)
LRstats(acc.mod1)

mosaic(acc.mod1, ~mode + age + gender + result,
    labeling_args = list(abbreviate = c(gender=1, result=4)))

# allow two-way association of result with each explanatory variable
acc.mod2 <- glm(Freq ~ age*mode*gender + result*(age+mode+gender),
                data=Accident,
                family=poisson)
LRstats(acc.mod2)
mosaic(acc.mod2, ~mode + age + gender + result,
    labeling_args = list(abbreviate = c(gender=1, result=4)))

acc.mods <- glmlist(acc.mod0, acc.mod1, acc.mod2)
LRstats(acc.mods)

## Binomial (logistic regression) models for result
## ------------------------------------------------
library(car)  # for Anova()
acc.bin1 <- glm(result=='Died' ~ age + mode + gender,
    weights=Freq, data=Accident, family=binomial)
Anova(acc.bin1)

acc.bin2 <- glm(result=='Died' ~ (age + mode + gender)^2,
    weights=Freq, data=Accident, family=binomial)
Anova(acc.bin2)

acc.bin3 <- glm(result=='Died' ~ (age + mode + gender)^3,
    weights=Freq, data=Accident, family=binomial)
Anova(acc.bin3)

# compare models
anova(acc.bin1, acc.bin2, acc.bin3, test="Chisq")

# visualize probability of death with effect plots
\dontrun{
library(effects)
plot(allEffects(acc.bin1), ylab='Pr (Died)')

plot(allEffects(acc.bin2), ylab='Pr (Died)')
}


#
}
\references{
Bertin, J. (1983), \emph{Semiology of Graphics}, University of
Wisconsin Press.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Reinis}
\alias{Reinis}
\title{Risk Factors for Coronary Heart Disease}
\format{
A 6-dimensional array resulting from cross-tabulating 6 binary variables
for 1841 observations. The variable names and their levels are:

\tabular{rll}{
No \tab Name        \tab Levels (meaning) \cr
1  \tab \code{smoke}   \tab \verb{"y", "n"} (smoking)\cr
2  \tab \code{mental}  \tab \verb{"y", "n"} (strenuous mental work)\cr
3  \tab \code{phys}    \tab \verb{"y", "n"} (strenuous physical work)\cr
4  \tab \code{systol}  \tab \verb{"y", "n"} (systolic blood pressure > 140)\cr
5  \tab \code{protein} \tab \verb{"y", "n"} (ratio of beta to alpha lipoproteins > 3)\cr
6  \tab \code{family}  \tab \verb{"y", "n"} (family history of CHD)\cr
}
}
\source{
Originally from the \pkg{gRbase} package as \code{data(reinis, package = "gRbase")}
}
\description{
Data from the beginning of a 15-year follow-up study of probable risk factors
for coronary thrombosis. The data are from 1841 men employed in a Czechoslovakian
car factory and represent all possible combinations of the six risk factors.
}
\details{
The study was conducted to examine risk factors for coronary heart disease (CHD) and
collected information on smoking habits, mental and physical work strain, systolic
blood pressure, ratio of lipoproteins, and family history of CHD.

The six variables form a \eqn{2^6 = 64} contingency table. The data have been used
extensively to illustrate model search procedures for high-dimensional contingency tables.
}
\examples{
data(Reinis)
str(Reinis)
ftable(Reinis, row.vars = 1:3)

# Fit all 0-way through 6-way models using Kway()
Reinis.gmodels <- Kway(Freq ~ smoke + mental + phys + systol + protein + family,
                       data = Reinis, family = poisson)

# Examine fit statistics
LRstats(Reinis.gmodels)

# Sequential tests for k vs k+1 way effects
anova(Reinis.gmodels, test = "Chisq")

# Fit sequential models of joint independence
Reinis.seqjoint <- seq_loglm(Reinis, type = "joint", prefix="joint")
LRstats(Reinis.seqjoint)

# Fit sequential models of conditional independence
Reinis.seqcond <- seq_loglm(Reinis, type = "conditional", prefix = "cond")
LRstats(Reinis.seqcond)

}
\references{
Edwards, D. and Havranek, T. (1985).
A fast procedure for model search in multidimensional contingency tables.
\emph{Biometrika}, 72, 339-351.

Gauraha, N. and Parui, S. K. (2020).
Mutual conditional independence and its applications to model selection in Markov networks.
\emph{Annals of Mathematics and Artificial Intelligence}, 88, 951-972.
\doi{10.1007/s10472-020-09690-7}

Reinis, Z., Pokorny, J., Basika, V., Tiserova, J., Gorican, K., Horakova, D.,
Stuchlikova, E., Havranek, T., and Hrabovsky, F. (1981).
Prognostic significance of the risk profile in the prevention of coronary heart disease.
\emph{Bratis. lek. Listy}, 76, 137-150.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CMHtest.R
\name{CMHtest}
\alias{CMHtest}
\alias{CMHtest.formula}
\alias{CMHtest.default}
\alias{Cochran}
\alias{Mantel}
\alias{Haenszel}
\alias{test}
\alias{print.CMHtest}
\title{Generalized Cochran-Mantel-Haenszel Tests}
\usage{
CMHtest(x, ...)

\method{CMHtest}{formula}(formula, data = NULL, subset = NULL, na.action = NULL, ...)

\method{CMHtest}{default}(
  x,
  strata = NULL,
  rscores = 1:R,
  cscores = 1:C,
  types = c("cor", "rmeans", "cmeans", "general"),
  overall = FALSE,
  details = overall,
  ...
)

\method{print}{CMHtest}(
  x,
  digits = max(getOption("digits") - 2, 3),
  layout = c("table", "2x2"),
  stars = FALSE,
  scale = FALSE,
  ...
)
}
\arguments{
\item{x}{A 2+ way contingency table in array form, or a class \code{"table"}
object with optional category labels specified in the dimnames(x) attribute.}

\item{\dots}{Other arguments passed to default method.}

\item{formula}{a formula specifying the variables used to create a
contingency table from \code{data}.  This should be a one-sided formula when
\code{data} is in array form, and a two-sided formula with a response
\code{Freq} if \code{data} is a data frame with a cell frequency variable.
For convenience, conditioning formulas can be specified indicating strata.}

\item{data}{either a data frame, or an object of class \code{"table"} or \code{"ftable"}.}

\item{subset}{an optional vector specifying a subset of observations to be used.}

\item{na.action}{a function which indicates what should happen when the data contain \code{NA}s.
Ignored if \code{data} is a contingency table.}

\item{strata}{For a 3- or higher-way table, the names or numbers of the
factors to be treated as strata.  By default, the first 2 factors are
treated as the main table variables, and all others considered stratifying factors.}

\item{rscores}{Row scores.  Either a set of numbers (typically integers,
\code{1:R}) or the string \code{"midrank"} for standardized midrank scores,
or \code{NULL} to exclude tests that depend on row scores.}

\item{cscores}{Column scores. Same as for row scores.}

\item{types}{Types of CMH tests to compute: Any one or more of \code{c("cor", "cmeans", "rmeans", "general")}, or
\code{"ALL"} for all of these.}

\item{overall}{logical. Whether to calculate overall tests, controlling for the stratifying factors.}

\item{details}{logical.  Whether to include computational details in the result}

\item{digits}{Digits to print.}

\item{layout}{For \code{print.CMHtest()}, one of \code{"table"} (default) for the
traditional flat 4-row printout, or \code{"2x2"} to reorganize the four CMH
statistics into a 2x2 table crossing how the row and column variables are
each treated (general/nominal vs. ordered/scored). \code{layout = "2x2"}
requires all four of \code{cor}, \code{rmeans}, \code{cmeans}, \code{general} to be present in
\code{x}; if \code{types} or \code{rscores}/\code{cscores = NULL} excluded any of them, this
falls back to \code{layout = "table"} with a warning. See \strong{Details}.}

\item{stars}{For \code{print.CMHtest()} with \code{layout = "2x2"}, logical: annotate
each cell with significance stars (\code{'***'}/\code{'**'}/\code{'*'} for p < .001/.01/.05)
based on its own (Chisq, Df). Default \code{FALSE}, matching \code{layout = "table"}'s
plain display.}

\item{scale}{For \code{print.CMHtest()} with \code{layout = "2x2"}, logical: show
\code{Chisq/Df} instead of \code{Chisq (Df)} in each cell -- normalizes for the very
different degrees of freedom across cells. Default \code{FALSE}.}
}
\value{
An object of class \code{"CMHtest"} , a list with the following 4 components:

\item{table}{A matrix containing the test statistics, with columns \code{Chisq}, \code{Df} and \code{Prob} }
\item{names}{The names of the table row and column variables}
\item{rscore}{Row scores}
\item{cscore}{Column scores}

If \code{details==TRUE}, additional components are included.

If there are strata, the result is a list of \code{"CMHtest"} objects. If
\code{overall=TRUE} another component, labeled \code{ALL} is appended to the
list.
}
\description{
Provides generalized Cochran-Mantel-Haenszel tests of association of two
possibly ordered factors, optionally stratified other factor(s).  With
strata, \code{CMHtest} calculates these tests for each level of the
stratifying variables and also provides overall tests controlling for the
strata. See \strong{Details} for descriptions of what these entail and enhancements to this function.
}
\details{
For ordinal factors, more powerful tests than the test for general
association (independence) are obtained by assigning scores to the row and
column categories.

The standard \eqn{\chi^2} tests for association in a two-way table treat
both table factors as nominal (unordered) categories. When one or both
factors of a two-way table are quantitative or ordinal, more powerful tests
of association may be obtained by taking ordinality into account using row
and or column scores to test for linear trends or differences in row or
column means.

The CMH analysis for a two-way table produces \strong{generalized}
Cochran-Mantel-Haenszel statistics (Landis etal., 1978):
\itemize{
\item The CMH \strong{correlation} statistic (\code{"cor"}), treating both factors as
ordered. For a given statum, with equally spaced row and column scores,
this CMH statistic reduces to \eqn{(n-1) r^2}, where \eqn{r} is the
Pearson correlation between X and Y. With \code{"midrank"} scores, this CMH
statistic is analogous to \eqn{(n-1) r_S^2}, using the Spearman rank
correlation.
\item The \strong{ANOVA} (row mean scores and column mean scores) statistics,
treat the columns and rows respectively as ordinal, and are sensitive to
mean shifts over columns or rows. These are transforms of the \eqn{F}
statistics from one-way ANOVAs with equally spaced scores and to
Kruskal-Wallis tests with \code{"midrank"} scores.
\item The CMH \strong{general} association statistic treat both factors as
unordered, and give a test closely related to the Pearson \eqn{\chi^2}
test. When there is more than one stratum, the overall general CMH
statistic gives a stratum-adjusted Pearson \eqn{\chi^2}, equivalent to
what is calculated by \code{\link[stats]{mantelhaen.test}}.
}
\subsection{Strata}{

For a 3+ way table, one table of CMH tests is produced for each combination
of the factors identified as \code{strata}. If \code{overall=TRUE}, an
additional table is calculated for the same two primary variables,
controlling for (pooling over) the \code{strata} variables.

These overall tests implicitly assume no interactions between the primary
variables and the strata and they will have low power in the presence of
interactions.

Note that strata combinations with insufficient data (less than 2
observations) are automatically omitted from the analysis.
}

\subsection{2x2 layout}{

\code{print.CMHtest(layout = "2x2")} is an attempt to provide an alternative view of these
four statistics that may be more useful/understandable than the traditional flat 4-row
table: it reorganizes them into a 2x2 table crossing how the row and column variables
are each treated (general/nominal vs. ordered/scored).

The corner cell of this display (\code{general - rmeans - cmeans + cor}, a "diff of diffs")
is algebraically consistent but its distribution as chi-square(df) is
\strong{experimental/unverified} -- these are four different quadratic-form statistics, not
a nested sequence of LR tests, so the naive inclusion-exclusion combination is not
guaranteed to be non-negative in general (see \verb{dev/CMH-2x2.md} for a worked
counter-example and a loglinear \code{LRstats()}-based alternative).
}
}
\examples{

data(JobSat, package="vcdExtra")
CMHtest(JobSat)
CMHtest(JobSat, rscores="midrank", cscores="midrank")

# formula interface
CMHtest(~ ., data=JobSat)

# A 3-way table (both factors ordinal)
data(MSPatients, package="vcd")
CMHtest(MSPatients)


# also calculate overall tests, controlling for Patient
CMHtest(MSPatients, overall = TRUE)
# compare with mantelhaen.test
mantelhaen.test(MSPatients)

# formula interface
CMHtest(~ ., data = MSPatients, overall = TRUE)

# using a frequency data.frame
CMHtest(xtabs(Freq~ses + mental, data = Mental))
# or, more simply
CMHtest(Freq~ses + mental, data = Mental)

# conditioning formulae
CMHtest(Freq~right + left | gender, data = VisualAcuity)

CMHtest(Freq ~ attitude + memory | education + age, data = Punishment)

# 2x2 layout, reorganizing the four CMH statistics by how the row/column
# variables are treated (general/nominal vs. ordered/scored)
cmh_mental <- CMHtest(Freq ~ ses + mental, data = Mental)
print(cmh_mental, layout = "2x2")
print(cmh_mental, layout = "2x2", stars = TRUE)
print(cmh_mental, layout = "2x2", scale = TRUE)


# Stokes etal, Table 5.1, p 92: two unordered factors
parties <- matrix(
	c(221, 160, 360, 140,
	  200, 291, 160, 311,
	  208, 106, 316, 97),
	nrow=3, ncol=4,
	byrow=TRUE)
dimnames(parties) <- list(party=c("Dem", "Indep", "Rep"),
             neighborhood=c("Bayside", "Highland", "Longview", "Sheffield"))
CMHtest(parties, rscores=NULL, cscores=NULL)

# compare with Pearson chisquare
chisq.test(parties)

}
\references{
Stokes, M. E. & Davis, C. S.  & Koch, G., (2000).
\emph{Categorical Data Analysis using the SAS System}, 2nd Ed., Cary, NC:
SAS Institute, pp 74-75, 92-101, 124-129.  Details of the computation are
given at: \url{http://support.sas.com/documentation/cdl/en/statug/63033/HTML/default/viewer.htm#statug_freq_a0000000648.htm}

Cochran, W. G. (1954), Some Methods for Strengthening the Common
\eqn{\chi^2} Tests, \emph{Biometrics}, 10, 417-451.

Landis, R. J., Heyman, E. R., and Koch, G. G. (1978).  Average Partial
Association in Three-way Contingency Tables: A Review and Discussion of
Alternative Tests, \emph{International Statistical Review}, \strong{46},
237-254.

Mantel, N. (1963), Chi-square Tests with One Degree of Freedom: Extensions
of the Mantel-Haenszel Procedure," \emph{Journal of the American Statistical
Association}, 58, 690-700.
}
\seealso{
\code{\link[coin]{cmh_test}} provides the CMH test of general
association; \code{\link[coin]{lbl_test}} provides the CMH correlation test
of linear by linear association.

\code{\link[stats]{mantelhaen.test}} provides the overall general
Cochran-Mantel-Haenszel chi-squared test of the null that two nominal
variables are conditionally independent in each stratum, assuming that there
is no three-way interaction

Other association tests:
\code{\link[=GKgamma]{GKgamma()}},
\code{\link[=HLtest]{HLtest()}},
\code{\link[=breslow_day_test]{breslow_day_test()}},
\code{\link[=woolf_test]{woolf_test()}},
\code{\link[=zero.test]{zero.test()}}
}
\author{
Michael Friendly
}
\concept{association tests}
\keyword{htest}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/woolf_test.R
\name{woolf_test}
\alias{woolf_test}
\alias{print.woolf_test}
\title{Woolf Test for Homogeneity of Odds Ratios}
\usage{
woolf_test(x, decompose = FALSE)

\method{print}{woolf_test}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"woolf_test"}}

\item{decompose}{Logical. If \code{TRUE} and \code{x} is 4-dimensional
(a \eqn{2 \times 2 \times R \times C}{2 x 2 x R x C} table), the test
is decomposed into row effects, column effects, and residual (interaction).
Defaults to \code{FALSE}. Ignored for non-4-dimensional tables.}

\item{...}{Additional arguments (currently unused)}
}
\value{
A list of class \code{"woolf_test"} (also inheriting from \code{"htest"})
containing the following components:
\item{statistic}{the chi-squared test statistic.}
\item{parameter}{degrees of freedom of the approximate chi-squared
distribution of the test statistic.}
\item{p.value}{\eqn{p}-value for the test.}
\item{method}{a character string indicating the type of test performed.}
\item{data.name}{a character string giving the name of the data.}
\item{or_vars}{names of the first two dimensions (the 2x2 table variables).}
\item{strata_vars}{names of the stratifying variables (dimensions 3 and beyond).}
\item{observed}{the observed log odds ratios.}
\item{expected}{the expected log odds ratio under the null hypothesis (weighted mean).}
\item{decomposed}{logical indicating if decomposition was performed.}

When \code{decompose = TRUE} (only for 4-dimensional tables), additional components:
\item{rows}{list with statistic, df, p.value, observed, expected for row effects.}
\item{cols}{list with statistic, df, p.value, observed, expected for column effects.}
\item{residual}{list with statistic, df, p.value for residual (interaction).}
}
\description{
Test for homogeneity on \eqn{2 \times 2 \times k}{2 x 2 x k} tables
over strata (i.e., whether the log odds ratios are the same in all
strata). Generalized to handle tables of any dimensionality beyond 3.
For 4-dimensional tables, optionally provides a two-way decomposition
of the homogeneity test into row effects, column effects, and residual.
}
\details{
The Woolf test (Woolf, 1955) tests the hypothesis that the odds ratios
\eqn{\theta_i} are equal across all \eqn{k} strata. The test statistic
is computed as the weighted sum of squared deviations of the log odds ratios
from their weighted mean:

\deqn{\chi^2_W = \sum_{i=1}^{k} w_i [\log(\theta_i) - \log(\bar{\theta}_w)]^2
      = \sum_{i=1}^{k} w_i \log^2(\theta_i / \bar{\theta}_w)}

where \eqn{\theta_i = (n_{11i} n_{22i}) / (n_{12i} n_{21i})} is the odds ratio
in stratum \eqn{i}, and \eqn{\bar{\theta}_w} is the weighted average odds ratio
(computed as the exponential of the weighted mean of the log odds ratios).

The weights \eqn{w_i} are the inverse variances of the log odds ratios:
\deqn{w_i = 1 / \text{Var}(\log \theta_i) = 1 / (1/n_{11i} + 1/n_{12i} + 1/n_{21i} + 1/n_{22i})}

Under the null hypothesis of homogeneous odds ratios, \eqn{\chi^2_W} follows
a chi-squared distribution with \eqn{k - 1} degrees of freedom.

\strong{Decomposition for 4-way tables:}
For a \eqn{2 \times 2 \times R \times C} table, the strata
form an \eqn{R \times C} two-way layout with odds ratios
\eqn{\theta_{ij}} for row \eqn{i} and column \eqn{j}. This suggests
that overall Woolf test
of homogeneity can be decomposed into three components, conceptually
analogous to a two-way ANOVA with one observation per cell:

\deqn{\chi^2_{\text{W:Total}} = \chi^2_{\text{W:Rows}} + \chi^2_{\text{W:Cols}} + \chi^2_{\text{W:Residual}}}

where:
\itemize{
\item \eqn{\chi^2_{\text{W:Rows}} } tests whether the odds ratios differ among
row levels (pooling over columns), with \eqn{R - 1} df
\item \eqn{\chi^2_{\text{W:Cols}}} tests whether the odds ratios differ among
column levels (pooling over rows), with \eqn{C - 1} df
\item \eqn{\chi^2_{\text{W:Residual}}} tests the row \eqn{\times}{x} column interaction
(deviation from additivity on the log odds scale), with \eqn{(R-1)(C-1)} df
}

The row effect test compares the marginal log odds ratios
\eqn{\log \bar{\theta}_{i+}} (pooled over columns) to the overall weighted mean.
Similarly, the column effect test compares \eqn{\log \bar{\theta}_{+j}}
(pooled over rows). The residual tests whether the cell log odds ratios
\eqn{\log \theta_{ij}} are additive in the row and column effects.

\strong{Note:} The two-way ANOVA-like decomposition for 4-dimensional tables
appears to be a novel extension introduced in this package. The existing
literature on the Woolf test (and related Breslow-Day test) treats strata
as unstructured, testing only whether all \eqn{k} odds ratios are equal.
This decomposition exploits the factorial structure of \eqn{R \times C}{R x C}
strata to provide more detailed insight into the sources of heterogeneity.
}
\examples{
# 3-way tables
data(CoalMiners, package = "vcd")
woolf_test(CoalMiners)

data(Heart, package = "vcdExtra")
woolf_test(Heart)

# 4-way table without decomposition
data(Fungicide, package = "vcdExtra")
woolf_test(Fungicide)

# 4-way table with decomposition
woolf_test(Fungicide, decompose = TRUE)

# 4-way table, but need to rearrange dimensions
# How does association between `Preference` and `M_User` vary over strata?
data(Detergent, package = "vcdExtra")
dimnames(Detergent) |> names()
Detergent <- aperm(Detergent, c(3, 2, 1, 4))
woolf_test(Detergent)


}
\references{
Woolf, B. (1955). On estimating the relation between blood group and disease.
\emph{Annals of Human Genetics}, \bold{19}, 251-253.
}
\seealso{
\code{\link[stats]{mantelhaen.test}}, \code{\link[DescTools]{BreslowDayTest}}

Other association tests:
\code{\link[=CMHtest]{CMHtest()}},
\code{\link[=GKgamma]{GKgamma()}},
\code{\link[=HLtest]{HLtest()}},
\code{\link[=breslow_day_test]{breslow_day_test()}},
\code{\link[=zero.test]{zero.test()}}
}
\concept{association tests}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blogits.R
\name{blogits}
\alias{blogits}
\title{Bivariate Logits and Log Odds Ratio}
\usage{
blogits(Y, add, colnames, row.vars, rev = FALSE)
}
\arguments{
\item{Y}{A four-column matrix or data frame whose columns correspond to the
2 x 2 combinations of two binary responses.}

\item{add}{Constant added to all cells to allow for zero frequencies.  The
default is 0.5 if \code{any(Y)==0} and 0 otherwise.}

\item{colnames}{Names for the columns of the results. The default is
\code{c("logit1", "logit2", "logOR")}.  If less than three names are
supplied, the remaining ones are filled in from the default.}

\item{row.vars}{A data frame or matrix giving the factor levels of one or
more factors corresponding to the rows of \code{Y}}

\item{rev}{A logical, indicating whether the order of the columns in
\code{Y} should be reversed.}
}
\value{
A data frame with \code{nrow(Y)} rows and \code{3 + ncol(row.vars)}
columns
}
\description{
This function calculates the log odds and log odds ratio for two binary
responses classified by one or more stratifying variables.
}
\details{
It is useful for plotting the results of bivariate logistic regression
models, such as those fit using \code{\link[VGAM]{vglm}} in the \pkg{VGAM}.

For two binary variables with levels 0,1 the logits are calculated assuming
the columns in \code{Y} are given in the order 11, 10, 01, 00, so the logits
give the log odds of the 1 response compared to 0.  If this is not the case,
either use \code{rev=TRUE} or supply \code{Y[,4:1]} as the first argument.
}
\examples{

data(Toxaemia)
tox.tab <- xtabs(Freq~class + smoke + hyper + urea, Toxaemia)

# reshape to 4-column matrix
toxaemia <- t(matrix(aperm(tox.tab), 4, 15))
colnames(toxaemia) <- c("hu", "hU", "Hu", "HU")
rowlabs <- expand.grid(smoke=c("0", "1-19", "20+"), class=factor(1:5))
toxaemia <- cbind(toxaemia, rowlabs)

# logits for H and U
logitsTox <- blogits(toxaemia[,4:1], 
                     add=0.5, 
                     colnames=c("logitH", "logitW"), 
                     row.vars=rowlabs)
logitsTox


}
\references{
Friendly, M. and Meyer, D. (2016).  \emph{Discrete Data Analysis
with R: Visualization and Modeling Techniques for Categorical and Count
Data}.  Boca Raton, FL: Chapman & Hall/CRC. \url{http://ddar.datavis.ca}.
}
\seealso{
\code{\link[VGAM]{vglm}}
}
\author{
Michael Friendly
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Hauser79}
\alias{Hauser79}
\title{Hauser (1979) Data on Social Mobility}
\format{
A frequency data frame with 25 observations on the following 3
variables, representing the cross-classification of 19912 individuals by
father's occupation and son's first occupation.
\describe{
\item{\code{Son}}{a factor with levels \code{UpNM} \code{LoNM} \code{UpM} \code{LoM} \code{Farm}}
\item{\code{Father}}{a factor with levels \code{UpNM} \code{LoNM} \code{UpM} \code{LoM} \code{Farm}}
\item{\code{Freq}}{a numeric vector}
}
}
\source{
R.M. Hauser (1979), Some exploratory methods for modeling mobility
tables and other cross-classified data.  In: K.F. Schuessler (Ed.),
\emph{Sociological Methodology}, 1980, Jossey-Bass, San Francisco, pp.
413-458. Table 1.
}
\description{
Hauser (1979) presented this two-way frequency table, cross-classifying
occupational categories of sons and fathers in the United States.
}
\details{
It is a good example for exploring a variety of models for square tables:
quasi-independence, quasi-symmetry, row/column effects, uniform association,
etc., using the facilities of the \pkg{gnm}.

Hauser's data was first presented in 1979, and then published in 1980. The
name of the dataset reflects the earliest use.

It reflects the "frequencies in a classification of son's first full-time
civilian occupation by father's (or other family head's) occupation at son's
sixteenth birthday among American men who were aged 20 to 64 in 1973 and
were not currently enrolled in school".

As noted in Hauser's Table 1, "Counts are based on observations weighted to
estimate population counts and compensate for departures of the sampling
design from simple random sampling. Broad occupation groups are upper
nonmanual: professional and kindred workers, managers and officials, and
non-retail sales workers; lower nonmanual: proprietors, clerical and kindred
workers, and retail sales workers; upper manual: craftsmen, foremen, and
kindred workers; lower manual: service workers, operatives and kindred
workers, and laborers (except farm); farm: farmers and farm managers, farm
laborers, and foremen. density of mobility or immobility in the cells to
which they refer."

The table levels for \code{Son} and \code{Father} have been arranged in
order of decreasing status as is common for mobility tables.
}
\examples{

data(Hauser79)
str(Hauser79)

# display table
structable(~Father+Son, data=Hauser79)

#Examples from Powers & Xie, Table 4.15
# independence model
mosaic(Freq ~ Father + Son, data=Hauser79, shade=TRUE)

hauser.indep <- gnm(Freq ~ Father + Son,
  data=Hauser79,
  family=poisson)

mosaic(hauser.indep, ~Father+Son,
       main="Independence model",
       gp=shading_Friendly)

# Quasi-independence
hauser.quasi <-  update(hauser.indep,
                        ~ . + Diag(Father,Son))
mosaic(hauser.quasi, ~Father+Son,
       main="Quasi-independence model",
       gp=shading_Friendly)

# Quasi-symmetry
hauser.qsymm <-  update(hauser.indep,
                        ~ . + Diag(Father,Son) + Symm(Father,Son))

mosaic(hauser.qsymm, ~Father+Son,
       main="Quasi-symmetry model",
       gp=shading_Friendly)


# numeric scores for row/column effects
Sscore <- as.numeric(Hauser79$Son)
Fscore <- as.numeric(Hauser79$Father)

# row effects model
hauser.roweff <- update(hauser.indep, ~ . + Father*Sscore)
LRstats(hauser.roweff)

# uniform association
hauser.UA <- update(hauser.indep, ~ . + Fscore*Sscore)
LRstats(hauser.UA)

# uniform association, omitting diagonals
hauser.UAdiag <- update(hauser.indep, ~ . + Fscore*Sscore + Diag(Father,Son))
LRstats(hauser.UAdiag)

# Levels for Hauser 5-level model
levels <- matrix(c(
  2,  4,  5,  5,  5,
  3,  4,  5,  5,  5,
  5,  5,  5,  5,  5,
  5,  5,  5,  4,  4,
  5,  5,  5,  4,  1
  ), 5, 5, byrow=TRUE)

hauser.topo <- update(hauser.indep,
                      ~ . + Topo(Father, Son, spec=levels))

mosaic(hauser.topo, ~Father+Son,
       main="Topological model", gp=shading_Friendly)

# RC model
hauser.RC <- update(hauser.indep, ~ . + Mult(Father, Son), verbose=FALSE)
mosaic(hauser.RC, ~Father+Son, main="RC model", gp=shading_Friendly)
LRstats(hauser.RC)

# crossings models
hauser.CR <- update(hauser.indep, ~ . + Crossings(Father,Son))
mosaic(hauser.topo, ~Father+Son, main="Crossings model", gp=shading_Friendly)
LRstats(hauser.CR)

hauser.CRdiag <- update(hauser.indep, ~ . + Crossings(Father,Son) + Diag(Father,Son))
LRstats(hauser.CRdiag)


# compare model fit statistics
modlist <- glmlist(hauser.indep, hauser.roweff, hauser.UA, hauser.UAdiag,
                   hauser.quasi, hauser.qsymm,  hauser.topo,
                   hauser.RC, hauser.CR, hauser.CRdiag)
sumry <- LRstats(modlist)
sumry[order(sumry$AIC, decreasing=TRUE),]
# or, more simply
LRstats(modlist, sortby="AIC")

mods <- substring(rownames(sumry),8)
with(sumry,
	{plot(Df, AIC, cex=1.3, pch=19, xlab='Degrees of freedom', ylab='AIC')
	text(Df, AIC, mods, adj=c(0.5,-.5), col='red', xpd=TRUE)
	})



}
\references{
Powers, D.A. and Xie, Y. (2008). \emph{Statistical Methods for
Categorical Data Analysis}, Bingley, UK: Emerald.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Fungicide}
\alias{Fungicide}
\title{Carcinogenic Effects of a Fungicide}
\format{
The data comprise a set of four 2 x 2 tables classifying 403 mice,
either Control or Treated and whether or not a tumor was later observed.
The four groups represent the combinations of sex and strain of mice.

The format is:
\preformatted{
num [1:2, 1:2, 1:2, 1:2] 5 4 74 12 3 2 84 14 10 4 ...
- attr(*, "dimnames")=List of 4
..$ group  : chr [1:2] "Control" "Treated"
..$ outcome: chr [1:2] "Tumor" "NoTumor"
..$ sex    : chr [1:2] "M" "F"
..$ strain : chr [1:2] "1" "2"
}
}
\source{
Gart, J. J. (1971). The comparison of proportions: a review of
significance tests, confidence intervals and adjustments for stratification.
\emph{International Statistical Review}, 39, 148-169.
}
\description{
Data from Gart (1971) on the carcinogenic effects of a certain fungicide in
two strains of mice. Of interest is how the association between \code{group}
(Control, Treated) and \code{outcome} (Tumor, No Tumor) varies with
\code{sex} and \code{strain} of the mice.
}
\details{
Breslow (1976) used this data to illustrate the application of linear models
to log odds ratios.

All tables have some small cells, so a continuity correction is recommended.
}
\examples{

data(Fungicide)
# loddsratio was moved to vcd; requires vcd_1.3-3+
fung.lor <- loddsratio(Fungicide, correct=TRUE)
fung.lor
confint(fung.lor)

# visualize odds ratios in fourfold plots
cotabplot(Fungicide, panel=cotab_fourfold)
#  -- fourfold() requires vcd >= 1.2-10
fourfold(Fungicide, p_adjust_method="none")



}
\references{
Breslow, N. (1976), Regression analysis of the log odds ratio: A
method for retrospective studies, \emph{Biometrics}, 32(3), 409-416.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_table.R
\name{as_table}
\alias{as_table}
\title{Convert frequency or case form data into table form}
\usage{
as_table(obj, freq = NULL, dims = NULL, prop = NULL)
}
\arguments{
\item{obj}{Object to be converted to table form.}

\item{freq}{If \code{obj} is in frequency form, this is the name of the frequency column.
Leave as \code{NULL} if \code{obj} is in any other form.}

\item{dims}{A character vector of dimensions. If not specified, all variables apart from
\code{freq} will be used as dimensions.}

\item{prop}{If set to \code{TRUE}, returns a table of proportions (that sum to 1). May also
be set to a character or numeric vector of dimensions to be used as margins
from which proportions will be computed.}
}
\value{
Object in table form.
}
\description{
Converts object (\code{obj}) in frequency or case form into table form. The
column containing the frequencies (\code{freq}) must be supplied if \code{obj} is in
frequency form. Optionally returns a table of proportions with (optionally) specified margins.
}
\details{
If \code{obj} was in table form to begin with, it is returned to the user as-is
unless dimensions were specified (in which case it returns a table with
entries summed over excluded dimensions). When \code{prop} is set to \code{TRUE}, the
returned table will have proportions that sum to one, whereas if a character
or numerical vector of table dimensions is supplied to \code{prop}, proportions
will be marginalized across the specified dimensions.
}
\examples{
library(vcdExtra)

data("HairEyeColor")

freqForm <- as.data.frame(HairEyeColor) # Generate frequency form data
tidy_freqForm <- dplyr::as_tibble(HairEyeColor) # Generate tidy frequency form data
caseForm <- expand.dft(freqForm) # Generate case form data

# Frequency form -> table form
as_table(freqForm, freq = "Freq") |> str()

# Warned if forgot to specify freq
as_table(freqForm) |> str()

# Frequency form (tibble) -> table form
as_table(tidy_freqForm, freq = "n") |> str()

# Case form -> table form
as_table(caseForm) |> str()

# For specific dimensions
as_table(tidy_freqForm, freq = "n", dims = c("Hair", "Eye")) |> str()

#-----For proportions-----#

as_table(freqForm, freq = "Freq", prop = TRUE) |> head(c(4,4,1)) # print only Sex == Male rows

# Marginalize proportions along "Sex" (i.e., male proportions sum to 1, female proportions sum to 1)
as_table(freqForm, freq = "Freq", prop = "Sex") |> head(c(4,4,1))

as_table(freqForm, freq = "Freq", prop = 3) |> head(c(4,4,1)) # Same as above

# Marginalize proportions along multiple variables
as_table(freqForm, freq = "Freq", prop = c("Hair", "Sex")) |> head(c(4,4,1))

as_table(freqForm, freq = "Freq", prop = c(1, 3)) |> head(c(4,4,1)) # Same as above

# Using dims and prop arguments in tandem
as_table(freqForm, freq = "Freq", dims = c("Hair", "Eye"), prop = TRUE)


}
\seealso{
\code{\link{as_freqform}}, \code{\link{as_caseform}}, \code{\link{as_array}},
\code{\link{as_matrix}}
}
\author{
Gavin M. Klorfine
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LRstats.R
\name{LRstats}
\alias{LRstats}
\alias{LRstats.glmlist}
\alias{LRstats.loglmlist}
\alias{LRstats.default}
\title{Brief Summary of Model Fit for glm and loglm Models}
\usage{
LRstats(object, ...)

\method{LRstats}{glmlist}(
  object,
  ...,
  saturated = NULL,
  sortby = NULL,
  label = c("name", "formula"),
  label.args = list()
)

\method{LRstats}{loglmlist}(
  object,
  ...,
  saturated = NULL,
  sortby = NULL,
  label = c("name", "formula"),
  label.args = list()
)

\method{LRstats}{default}(
  object,
  ...,
  saturated = NULL,
  sortby = NULL,
  label = c("name", "formula"),
  label.args = list()
)
}
\arguments{
\item{object}{a fitted model object for which there exists a logLik method
to extract the corresponding log-likelihood}

\item{\dots}{optionally more fitted model objects}

\item{saturated}{saturated model log likelihood reference value (use 0 if
deviance is not available)}

\item{sortby}{either a numeric or character string specifying the column in
the result by which the rows are sorted (in decreasing order)}

\item{label}{character string specifying how to label the rows: \code{"name"}
(default) uses the model object names; \code{"formula"} uses model formulas or
bracket notation obtained from \code{\link{get_models}} (for \code{glmlist}
and \code{loglmlist} objects) or \code{\link{get_model}} (for individual
model objects passed to the default method).}

\item{label.args}{a list of additional arguments passed to \code{\link{get_models}}
when \code{label = "formula"}. Useful arguments include \code{abbrev} (logical or integer)
to abbreviate factor names and \code{sep} to change the separator in bracket notation.}
}
\value{
A data frame (also of class \code{anova}) with columns
\code{       c("AIC", "BIC", "LR Chisq", "Df", "Pr(>Chisq)")}. Row names are taken
from the names of the model object(s) or their model formulas.
}
\description{
For \code{glm} objects, the \code{print} and \code{summary} methods give too
much information if all one wants to see is a brief summary of model
goodness of fit, and there is no easy way to display a compact comparison of
model goodness of fit for a collection of models fit to the same data. All
\code{loglm} models have equivalent glm forms, but the \code{print} and
\code{summary} methods give quite different results.
}
\details{
\code{LRstats} provides a brief summary for one or more models fit to the
same dataset for which \code{logLik} and \code{nobs} methods exist (e.g.,
\code{glm} and \code{loglm} models).

The function relies on residual degrees of freedom for the LR chisq test
being available in the model object.  This is true for objects inheriting
from \code{lm}, \code{glm}, \code{loglm}, \code{polr} and \code{negbin}.
}
\examples{

data(Mental)
indep <- glm(Freq ~ mental+ses,
                family = poisson, data = Mental)
LRstats(indep)
Cscore <- as.numeric(Mental$ses)
Rscore <- as.numeric(Mental$mental)

coleff <- glm(Freq ~ mental + ses + Rscore:ses,
                family = poisson, data = Mental)
roweff <- glm(Freq ~ mental + ses + mental:Cscore,
                family = poisson, data = Mental)
linlin <- glm(Freq ~ mental + ses + Rscore:Cscore,
                family = poisson, data = Mental)

# compare models using object names (default)
LRstats(indep, coleff, roweff, linlin)

# compare models in a glmlist, using formula labels
mods <- glmlist(indep, coleff, roweff, linlin)
LRstats(mods, label = "formula")

# loglmlist example with bracket notation labels
data(Titanic)
tit.joint <- seq_loglm(Titanic, type = "joint")
LRstats(tit.joint)
LRstats(tit.joint, label = "formula")
LRstats(tit.joint, label = "formula", label.args = list(abbrev = TRUE))

}
\seealso{
\code{\link[stats]{logLik}}, \code{\link[stats]{glm}},
\code{\link[MASS]{loglm}},

\code{\link{logLik.loglm}}, \code{\link{modFit}},
\code{\link{get_models}}, \code{\link{get_model}}

Other glmlist functions:
\code{\link[=Kway]{Kway()}},
\code{\link[=get_model]{get_model()}},
\code{\link[=glmlist]{glmlist()}},
\code{\link[=mosaic.glmlist]{mosaic.glmlist()}}
}
\author{
Achim Zeileis, Michael Friendly
}
\concept{glmlist functions}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{HouseTasks}
\alias{HouseTasks}
\title{Household Tasks Performed by Husbands and Wives}
\format{
The format is:
\preformatted{
  'table' int [1:13, 1:4] 36 11 24 51 13 1 1 14 20 46 ...
- attr(*, "dimnames")=List of 2
..$ Task: chr [1:13] "Breakfast" "Dinner" "Dishes" "Driving" ...
..$ Who : chr [1:4] "Alternating" "Husband" "Jointly" "Wife"
}
}
\source{
This data set was taken from \code{\link[factoextra]{housetasks}}, a
13 x 4 data.frame. In this table version, the rows and columns were sorted
alphabetically (and a typo was corrected).
}
\description{
A 13 x 4 table of frequencies of household tasks performed by couples,
either by the \code{Husband}, \code{Wife}, \code{Alternating} or \code{Jointly}.
}
\examples{

data(HouseTasks)
str(HouseTasks)

chisq.test(HouseTasks)

# mosaic plot, illustrating some tweaks to handle overlapping labels
require(vcd)
mosaic(HouseTasks, shade = TRUE,
       labeling = labeling_border(rot_labels = c(45,0, 0, 0),
                                  offset_label =c(.5,5,0, 0),
                                  varnames = c(FALSE, TRUE),
                                  just_labels=c("center","right"),
                                  tl_varnames = FALSE),
       legend = FALSE)

# use seriation package to permute rows & cols using correspondence analysis
if(require(seriation)) {
order <- seriate(HouseTasks, method = "CA")
# the permuted row and column labels
rownames(HouseTasks)[order[[1]]]
colnames(HouseTasks)[order[[2]]]

# do the permutation
HT_perm <- permute(HouseTasks, order, margin=1)

mosaic(HT_perm, shade = TRUE,
       labeling = labeling_border(rot_labels = c(45,0, 0, 0),
                                  offset_label =c(.5,5,0, 0),
                                  varnames = c(FALSE, TRUE),
                                  just_labels=c("center","right"),
                                  tl_varnames = FALSE),
       legend = FALSE)
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Kway.R
\name{Kway}
\alias{Kway}
\title{Fit All K-way Models in a GLM}
\usage{
Kway(formula, family = poisson, data, ..., order = nt, prefix = "kway")
}
\arguments{
\item{formula}{a two-sided formula for the 1-way effects in the model. The LHS should be the response, and the RHS
should be the first-order terms connected by \code{+} signs.}

\item{family}{a description of the error distribution and link function to be used in the model.  This can be a character
string naming a family function, a family function or the result of a call to a family function.
(See \code{\link[stats]{family}} for details of family functions.)}

\item{data}{an optional data frame, list or environment (or object coercible
by \code{\link[base]{as.data.frame}} to a data frame) containing the
variables in the model. If not found in data, the variables are taken from
\code{environment(formula)}, typically the environment from which \code{glm}
is called.}

\item{\dots}{Other arguments passed to \code{glm}}

\item{order}{Highest order interaction of the models generated. Defaults to
the number of terms in the model formula.}

\item{prefix}{Prefix used to label the models fit in the \code{glmlist}
object.}
}
\value{
An object of class \code{glmlist}, of length \code{order+1}
containing the 0-way, 1-way, ...  models up to degree \code{order}.
}
\description{
Generate and fit all 0-way, 1-way, 2-way, ... k-way terms in a glm.
}
\details{
This function is designed mainly for hierarchical loglinear models (or
\code{glm}s in the poisson family), where it is desired to find the
highest-order terms necessary to achieve a satisfactory fit.

Using \code{\link[stats]{anova}} on the resulting \code{\link{glmlist}}
object will then give sequential tests of the pooled contributions of all
terms of degree \eqn{k+1} over and above those of degree \eqn{k}.

This function is also intended as an example of a generating function for
\code{\link{glmlist}} objects, to facilitate model comparison, extraction,
summary and plotting of model components, etc., perhaps using \code{lapply}
or similar.

With \code{y} as the response in the \code{formula}, the 0-way (null) model
is \code{y ~ 1}.  The 1-way ("main effects") model is that specified in the
\code{formula} argument.  The k-way model is generated using the formula
\code{. ~ .^k}. With the default \code{order = nt}, the final model is the
saturated model.

As presently written, the function requires a two-sided formula with an
explicit response on the LHS. For frequency data in table form (e.g.,
produced by \code{xtabs}) you the \code{data} argument is coerced to a
data.frame, so you should supply the \code{formula} in the form \verb{Freq ~ } \dots{}.
}
\examples{

## artificial data
factors <- expand.grid(A=factor(1:3),
                       B=factor(1:2),
                       C=factor(1:3),
                       D=factor(1:2))
Freq <- rpois(nrow(factors), lambda=40)
df <- cbind(factors, Freq)

mods3 <- Kway(Freq ~ A + B + C, data=df, family=poisson)
LRstats(mods3)
mods4 <- Kway(Freq ~ A + B + C + D, data=df, family=poisson)
LRstats(mods4)

# JobSatisfaction data
data(JobSatisfaction, package="vcd")
modSat <- Kway(Freq ~ management+supervisor+own,
               data=JobSatisfaction,
               family=poisson, prefix="JobSat")
LRstats(modSat)
anova(modSat, test="Chisq")

# Rochdale data: very sparse, in table form
data(Rochdale, package="vcd")
\dontrun{
modRoch <- Kway(Freq~EconActive + Age + HusbandEmployed + Child +
                     Education + HusbandEducation + Asian + HouseholdWorking,
                data=Rochdale, family=poisson)
LRstats(modRoch)
}

}
\seealso{
\code{\link{glmlist}}, \code{\link{Summarise}} (soon to be
deprecated), \code{\link{LRstats}}

Other glmlist functions:
\code{\link[=LRstats]{LRstats()}},
\code{\link[=get_model]{get_model()}},
\code{\link[=glmlist]{glmlist()}},
\code{\link[=mosaic.glmlist]{mosaic.glmlist()}}
}
\author{
Michael Friendly and Heather Turner
}
\concept{glmlist functions}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knit_include.R
\name{knit_include}
\alias{knit_include}
\title{Include an HTML-renderable object in any knitr output format}
\usage{
knit_include(
  x,
  file = tempfile("ki_", tmpdir = "."),
  width = 700,
  height = 400,
  ...
)
}
\arguments{
\item{x}{Any R object. Specialised handling for \code{gt_tbl} and
\code{htmlwidget}; all other classes are passed through
unchanged.}

\item{file}{Base filename (no extension) for temporary files written when
output is not HTML.  Defaults to a session-unique temp file in
the current directory so multiple calls don't collide.}

\item{width}{Viewport / screenshot width in pixels (non-HTML only).}

\item{height}{Viewport / screenshot height in pixels (non-HTML only).}

\item{...}{Additional arguments forwarded to \code{\link[gt]{gtsave}} or
\code{\link[webshot2]{webshot}}.}
}
\value{
For \code{gt_tbl} / \code{htmlwidget} in non-HTML output: the result
of \code{\link[knitr]{include_graphics}}. Otherwise: \code{x}
unchanged.
}
\description{
A pipe-friendly helper in Rmarkdown, Quarto, and other documents for objects that render natively in HTML output
(RStudio Viewer, HTML documents) but need a convenient image fallback for PDF, Word,
and other non-HTML formats. It was designed to solve a problem in rendering the result of \code{color_table()},
and then generalized to provide for other similar cases.
}
\details{
Supported classes (handled internally) solve this problem by saving the image to a file and then inserting it in the document
using \code{knitr::include_graphics()}:
\itemize{
\item \code{"gt_tbl"}     (gt package)       -- saved via \code{\link[gt]{gtsave}}
\item \code{"htmlwidget"} (htmlwidgets family: plotly, DT, leaflet, dygraphs, ...)
-- saved via
\code{\link[htmlwidgets]{saveWidget}}
+ \code{\link[webshot2]{webshot}}
}

Packages that handle cross-format output natively (\pkg{flextable}, \pkg{kableExtra},
\pkg{huxtable}) do NOT need this helper.

For any other class, \code{x} is returned as-is in all output formats and
knitr's normal printing applies.  This means the function is safe to use in
a pipe on any object: it only intervenes when it knows how to help!
}
\examples{
\dontrun{
data(HairEyeColor)
HEC <- margin.table(HairEyeColor, 1:2)

## gt_tbl from color_table()
color_table(HEC, title = "Hair x Eye Color") |> knit_include()

## DT htmlwidget
DT::datatable(mtcars) |> knit_include(width = 900, height = 500)

## plotly htmlwidget
plotly::plot_ly(mtcars, x = ~wt, y = ~mpg) |> knit_include()

## Any other object passes through unchanged
lm(mpg ~ wt, data = mtcars) |> knit_include()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mosaic3d.R
\name{mosaic3d}
\alias{mosaic3d}
\alias{mosaic3d.default}
\alias{mosaic3d.loglm}
\title{3D Mosaic Plots}
\usage{
mosaic3d(x, ...)

\method{mosaic3d}{loglm}(
  x,
  type = c("observed", "expected"),
  residuals_type = c("pearson", "deviance"),
  ...
)

\method{mosaic3d}{default}(
  x,
  expected = NULL,
  residuals = NULL,
  type = c("observed", "expected"),
  residuals_type = NULL,
  shape = rgl::cube3d(alpha = alpha),
  alpha = 0.5,
  spacing = 0.1,
  split_dir = 1:3,
  shading = shading_basic,
  interpolate = c(2, 4),
  zero_size = 0.05,
  label_edge,
  labeling_args = list(),
  newpage = TRUE,
  box = FALSE,
  ...
)
}
\arguments{
\item{x}{A \code{link[MASS]{loglm}} model object. Alternatively, a multidimensional \code{array} or \code{table}
or\code{\link[vcd]{structable}} of frequencies in a contingency table.  In the present implementation, the dimensions
are taken in sequential order. Use \code{link[base]{aperm}} or \code{\link[vcd]{structable}} to change this.}

\item{\dots}{Other arguments passed down to \code{mosaic.default} or 3D functions.}

\item{type}{a character string indicating whether the \code{"observed"} or the \code{"expected"} frequencies in the table should be
visualized by the volume of the 3D tiles.}

\item{residuals_type}{a character string indicating the type of residuals to be computed when none are supplied.  If residuals is \code{NULL},
\code{residuals_type} must be one of \code{"pearson"} (default; giving components of Pearson's chi-squared), \code{"deviance"}
(giving components of the likelihood ratio chi-squared), or \code{"FT"} for the Freeman-Tukey residuals.  The value of this
argument can be abbreviated.}

\item{expected}{optionally, for contingency tables, an array of expected frequencies of the same dimension as \code{x}, or alternatively the
corresponding loglinear model specification as used by\code{link[stats]{loglin}} or \code{link[MASS]{loglm}} (see
\code{\link[vcd]{structable}} for details).}

\item{residuals}{optionally, an array of residuals of the same dimension as \code{x} (see details).}

\item{shape}{The initial 3D shape on which the mosaic is based.  Typically this is a call to an rgl function, and must produce a \code{shape3d} object.
The default is a "unit cube" on (-1, +1), with transparency specified by \code{alpha}.}

\item{alpha}{Specifies the transparency of the 3D tiles used to compose the 3D mosaic.}

\item{spacing}{A number or vector giving the total amount of space used to separate the 3D tiles along each of the
dimensions of the table. The values specified are re-cycled to the number of table dimensions.}

\item{split_dir}{A numeric vector composed of the integers \code{1:3} or a character vector composed of \code{c("x", "y", "z")}, where
\code{split_dir[i]} specifies the axis along which the tiles should be split
for dimension \code{i} of the table. The values specified are re-cycled to the number of table dimensions.}

\item{shading}{A function, taking an array or vector of residuals for the
given model, returning a vector of colors.  At present, only the default \code{shading=shading_basic} is provided.
This is roughly equivalent to the
use of the \code{shade} argument in \code{\link[graphics]{mosaicplot}} or to
the use of \code{gp=shading_Friendly} in \code{\link[vcd]{mosaic}}.}

\item{interpolate}{a vector of interpolation values for the \code{shading} function.}

\item{zero_size}{The radius of a small sphere used to mark zero cells in the display.}

\item{label_edge}{A character vector composed of \code{c("-", "+")} indicating whether the labels for a given table dimension
are to be written at the minima (\code{"-"}) or maxima (\code{"+"}) of the \emph{other}
dimensions in the plot. The default is \code{rep( c('-', '+'), each=3, length=ndim)}, meaning that the first three
table variables are labeled at the minima, and successive ones at the maxima.}

\item{labeling_args}{This argument is intended to be used to specify details of the rendering of labels for the table dimensions, but
at present has no effect.}

\item{newpage}{logical indicating whether a new page should be created for the plot or not.}

\item{box}{logical indicating whether a bounding box should be drawn around the plot.}
}
\value{
Invisibly, the list of \code{shape3d} objects used to draw the 3D
mosaic, with names corresponding to the concatenation of the level labels,
separated by ":".
}
\description{
Produces a 3D mosaic plot for a contingency table (or a
\code{link[MASS]{loglm}} model) using the \code{\link[rgl]{rgl-package}}.
}
\details{
Generalizing the 2D mosaic plot, this begins with a given 3D shape (a unit
cube), and successively sub-divides it along the X, Y, Z dimensions
according to the table margins, generating a nested set of 3D tiles. The
volume of the resulting tiles is therefore proportional to the frequency
represented in the table cells. Residuals from a given loglinear model are
then used to color or shade each of the tiles.

This is a developing implementation.  The arguments and details are subject
to change.

Friendly (1995), Friendly (2000, Sect. 4.5) and Theus and Lauer (1999) have
all used the idea of 3D mosaic displays to explain various aspects of
loglinear models (the iterative proportional fitting algorithm, the
structure of various models for 3-way and n-way tables, etc.), but no
implementation of 3D mosaics was previously available.

For the default method, residuals, used to color and shade the 3D tiles, can
be passed explicitly, or, more typically, are computed as needed from
observed and expected frequencies. In this case, the expected frequencies
are optionally computed for a specified loglinear model given by the
\code{expected} argument. For the loglm method, residuals and observed
frequencies are calculated from the model object.
}
\examples{

# 2 x 2 x 2
if (!rgl::rgl.useNULL() && interactive()){
mosaic3d(Bartlett, box=TRUE)
# compare with expected frequencies under model of mutual independence
mosaic3d(Bartlett, type="expected", box=TRUE)

# 2 x 2 x 3
mosaic3d(Heart, box=TRUE)
}

# Make a dynamic display
\dontrun{
  mosaic3d(Heart, box=TRUE, alpha = 0.3, interpolate = c(1, 2, 4))
  play3d(spin3d(axis = c(0, 1, 0), rpm = 10), duration = 5)
}

\dontrun{
# 2 x 2 x 2 x 3
# illustrates a 4D table
mosaic3d(Detergent)

# compare 2D and 3D mosaics
#demo("mosaic-hec")
}


}
\references{
Friendly, M. (1995). Conceptual and Visual Models for
Categorical Data, \emph{The American Statistician}, \strong{49}, 153-160.

Friendly, M. \emph{Visualizing Categorical Data}, Cary NC: SAS Institute,
2000. Web materials: \url{http://www.datavis.ca/books/vcd/}.

Theus, M. & Lauer, S. R. W. (1999) Visualizing Loglinear Models.
\emph{Journal of Computational and Graphical Statistics}, \strong{8}, 396-412.
}
\seealso{
\code{\link[vcd]{strucplot}}, \code{\link[vcd]{mosaic}},
\code{\link[graphics]{mosaicplot}} for aspects of mosaic plots.

\code{\link[stats]{loglin}}, \code{\link[MASS]{loglm}} for details on
fitting loglinear models

\code{\link[rgl]{play3d}}, \code{\link[rgl]{movie3d}} for what you can do with a 3D mosaic plot

Other mosaic plots:
\code{\link[=mosaic.glm]{mosaic.glm()}},
\code{\link[=mosaic.glmlist]{mosaic.glmlist()}}
}
\author{
Michael Friendly, with the help of Duncan Murdoch and Achim Zeileis
}
\concept{3D}
\concept{mosaic plots}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GKgamma.R
\name{GKgamma}
\alias{GKgamma}
\alias{print.GKgamma}
\title{Calculate Goodman-Kruskal Gamma for ordered tables}
\usage{
GKgamma(x, level = 0.95)

\method{print}{GKgamma}(x, digits = 3, ...)
}
\arguments{
\item{x}{A two-way frequency table, in matrix or table form.  The rows and columns are considered to be ordinal factors}

\item{level}{Confidence level for a significance test of \eqn{\gamma \ne =}{gamma !=0}}

\item{digits}{number to digits to use in the print method}

\item{...}{other arguments (unused), for conformity with the print generic}
}
\value{
Returns an object of class \code{"GKgamma"} with 6 components, as follows
\describe{
\item{gamma}{The gamma statistic}
\item{C}{Total number of concordant pairs in the table}
\item{D}{Total number of discordant pairs in the table}
\item{sigma}{Standard error of gamma }
\item{CIlevel}{Confidence level}
\item{CI}{Confidence interval}
}
}
\description{
The Goodman-Kruskal \eqn{\gamma}{gamma} statistic is a measure of
association for ordinal factors in a two-way table proposed by Goodman and
Kruskal (1954).
}
\examples{

data(JobSat)
GKgamma(JobSat)

}
\references{
Agresti, A. \emph{Categorical Data Analysis}. John Wiley & Sons,
2002, pp. 57--59.

Goodman, L. A., & Kruskal, W. H. (1954). Measures of association for cross
classifications. \emph{Journal of the American Statistical Association}, 49,
732-764.

Goodman, L. A., & Kruskal, W. H. (1963). Measures of association for cross
classifications III: Approximate sampling theory. \emph{Journal of the
American Statistical Association}, 58, 310-364.
}
\seealso{
\code{\link[vcd]{assocstats}}, \link[vcd]{Kappa}

Other association tests:
\code{\link[=CMHtest]{CMHtest()}},
\code{\link[=HLtest]{HLtest()}},
\code{\link[=breslow_day_test]{breslow_day_test()}},
\code{\link[=woolf_test]{woolf_test()}},
\code{\link[=zero.test]{zero.test()}}
}
\author{
Michael Friendly; original version by Laura Thompson
}
\concept{association tests}
\keyword{category}
\keyword{htest}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split3d.R
\name{split3d}
\alias{split3d}
\alias{split3d.shape3d}
\alias{split3d.list}
\alias{range3d}
\alias{center3d}
\title{Subdivide a 3D Object}
\usage{
split3d(obj, ...)

\method{split3d}{shape3d}(obj, p, dim, space = 0.1, ...)

\method{split3d}{list}(obj, p, dim, space = 0.1, ...)

range3d(obj)

center3d(obj)
}
\arguments{
\item{obj}{A \code{shape3d} object, or a list composed of them}

\item{\dots}{Other arguments for split3d methods}

\item{p}{For a single \code{shade3d} object, a vector of proportions (or a
vector of non-negative numbers which will be normed to proportions)
indicating the number of subdivisions and their scaling along dimension
\code{dim}. For a list of \code{shade3d} objects, a matrix whose columns
indicate the subdivisions of each object.}

\item{dim}{The dimension along which the object is to be subdivided. Either
an integer: 1, 2, or 3, or a character: "x", "y", or "z".}

\item{space}{The total space used to separate the copies of the object along
dimension \code{dim}. The unit inter-object space is therefore
\code{space/(length(p)-1)}.}
}
\value{
\code{split3d} returns a list of \code{shape3d} objects.

\code{range3d} returns a 2 x 3 matrix, whose first row contains the minima
on dimensions x, y, z, and whose second row contains the maxima.

\code{center3d} returns a numeric vector containing the means of the minima
and maxima on dimensions x, y, z.
}
\description{
Subdivides a \code{shape3d} object or a list of \code{shape3d} objects into
objects of the same shape along a given dimension according to the
proportions or frequencies specified in vector(s).
}
\details{
\code{split3d} is the basic workhorse used in \code{\link{mosaic3d}}, but
may be useful in other contexts.

\code{range3d} and \code{center3d} are utility functions, also useful in
other contexts.

The resulting list of \code{shape3d} objects is actually composed of
\emph{copies} of the input object(s), scaled according to the proportions in
\code{p} and then translated to make their range along the splitting
dimension equal to that of the input object(s).
}
\examples{

if (!rgl::rgl.useNULL() && interactive()){
  open3d()
  cube <- cube3d(alpha=0.4)
  sl1 <- split3d(cube, c(.2, .3, .5), 1)
  col <- c("#FF000080", "#E5E5E580", "#0000FF80")
  shapelist3d(sl1, col=col)

  open3d()
  p <- matrix(c(.6, .4, .5, .5, .2, .8), nrow=2)
  sl2 <- split3d(sl1, p, 2)
  shapelist3d(sl2, col=col)
  }

}
\seealso{
\code{\link{mosaic3d}}

\code{\link[rgl]{shapelist3d}} for the plotting of lists of \code{shape3d}
objects.
}
\author{
Duncan Murdoch, with refinements by Michael Friendly
}
\concept{3D}
\keyword{dplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Vote1980}
\alias{Vote1980}
\title{Race and Politics in the 1980 Presidential Vote}
\format{
A frequency data frame representing a 2 x 7 x 2 table, with 28
observations on the following 4 variables.
\describe{
\item{\code{race}}{a factor with levels \code{NonWhite} \code{White}}
\item{\code{conservatism}}{a factor with levels \code{1} \code{2} \code{3} \code{4} \code{5} \code{6} \code{7},
\code{1}=most liberal, \code{7}=most conservative}
\item{\code{votefor}}{a factor with levels \code{Carter} \code{Reagan}; \code{Carter}represents Jimmy Carter or other.}
\item{\code{Freq}}{a numeric vector}
}
}
\source{
Clogg, C. & Shockey, J. W. (1988). In Nesselroade, J. R. & Cattell,
R. B. (ed.)  Multivariate Analysis of Discrete Data, \emph{Handbook of
Multivariate Experimental Psychology}, New York: Plenum Press.
}
\description{
Data from the 1982 General Social Survey on votes in the 1980 U.S.
presidential election in relation to race and political conservatism.
}
\details{
The data contains a number of sampling zeros in the frequencies of NonWhites
voting for Ronald Reagan.
}
\examples{

data(Vote1980)
fourfold(xtabs(Freq ~ race + votefor + conservatism,
  data=Vote1980),
  mfrow=c(2,4))


}
\references{
Agresti, A. (1990) \emph{Categorical Data Analysis}, Table 4.12
New York: Wiley-Interscience.

Friendly, M. (2000) \emph{Visualizing Categorical Data}, Example 7.5 Cary,
NC: SAS Institute.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CrabSatellites.R
\docType{data}
\name{CrabSatellites}
\alias{CrabSatellites}
\title{Horseshoe Crab Mating}
\format{
A data frame containing 173 observations on 5 variables.
\describe{
\item{\code{color}}{Ordered factor indicating color (light medium, medium, dark medium, dark).}
\item{\code{spine}}{Ordered factor indicating spine condition (both good, one worn or broken, both worn or broken).}
\item{\code{width}}{Carapace width (cm).}
\item{\code{weight}}{Weight (kg).}
\item{\code{satellites}}{Number of satellites.}
}
}
\source{
Table 4.3 in Agresti (2002). This dataset was taken from the \pkg{countreg} package, which is not on CRAN
}
\description{
Determinants of mating for male satellites to nesting horseshoe crabs.
The number of \code{satellites} is a natural outcome variable. This dataset is useful for exploring various count data models.
}
\details{
Brockmann (1996) investigates horseshoe crab mating. The crabs arrive on the beach in pairs to spawn. Furthermore, unattached males
also come to the beach, crowd around the nesting couples and compete with attached males for fertilizations. These so-called
satellite males form large groups around some couples while ignoring others. Brockmann (1996) shows that the groupings are
not driven by environmental factors but by properties of the nesting female crabs. Larger females that are in better condition
attract more satellites.

Agresti (2002, 2013) reanalyzes the number of satellites using count models. Explanatory variables are the female crab's
color, spine condition, weight, and carapace width. Color and spine condition are ordered factors but are treated as numeric in some analyses.
}
\examples{
## load data, use ordered factors as numeric, and grouped factor version of width
data("CrabSatellites", package = "vcdExtra")
CrabSatellites <- transform(CrabSatellites,
  color = as.numeric(color),
  spine = as.numeric(spine),
  cwidth = cut(width, c(-Inf, seq(23.25, 29.25), Inf))
)

## Agresti, Table 4.4
aggregate(CrabSatellites$satellites,
          list(CrabSatellites$cwidth), function(x)
  round(c(Number = length(x), Sum = sum(x), Mean = mean(x), Var = var(x)), digits = 2))

## Agresti, Figure 4.4
plot(tapply(satellites, cwidth, mean) ~ tapply(width, cwidth, mean),
  data = CrabSatellites,
  ylim = c(0, 6), pch = 19, cex = 1.5,
  xlab = "Mean carapace width (cm)",
  ylab = "Mean number of satellites")

## More examples: ?countreg::CrabSatellites` has examples of other plots and count data models
}
\references{
Agresti A (2002). Categorical Data Analysis, 2nd ed., John Wiley & Sons, Hoboken.

Agresti A (2013). Categorical Data Analysis, 3rd ed., John Wiley & Sons, Hoboken.
Brockmann HJ (1996). “Satellite Male Groups in Horseshoe Crabs, Limulus polyphemus”, Ethology, 102(1), 1–21.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.Kappa.R
\name{print.Kappa}
\alias{print.Kappa}
\title{Print Kappa}
\usage{
\method{print}{Kappa}(
  x,
  digits = max(getOption("digits") - 3, 3),
  CI = FALSE,
  level = 0.95,
  ...
)
}
\arguments{
\item{x}{A Kappa object}

\item{digits}{number of digits to print}

\item{CI}{Include confidence intervals in the display?}

\item{level}{confidence level}

\item{\dots}{Other arguments}
}
\value{
Returns the Kappa object, invisibly.
}
\description{
This is a replacement for the \code{print.Kappa} method in \code{vcd},
adding display of \code{z} values to the \code{vcd} version and optional
confidence intervals.
}
\examples{

data("SexualFun")
Kappa(SexualFun)
print(Kappa(SexualFun), CI=TRUE)

# stratified 3-way table
apply(MSPatients, 3, Kappa)

}
\seealso{
\code{\link[vcd]{confint.Kappa}}
}
\author{
Michael Friendly
}
\keyword{category}
\keyword{htest}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Donner}
\alias{Donner}
\title{Survival in the Donner Party}
\format{
A data frame with 90 observations on the following 5 variables.

\describe{
\item{\code{family}}{family name, a factor with 10 levels }
\item{\code{age}}{age of person, a numeric vector}
\item{\code{sex}}{a factor with levels \code{Female} \code{Male}}
\item{\code{survived}}{a numeric vector, 0 or 1}
\item{\code{death}}{date of death for those who died before rescue, a POSIXct}
}
}
\source{
D. K. Grayson, 1990, "Donner party deaths: A demographic
assessment", \emph{J. Anthropological Research}, \strong{46}, 223-242.

Johnson, K. (1996). \emph{Unfortunate Emigrants: Narratives of the Donner
Party}.  Logan, UT: Utah State University Press.  Additions, and dates of
death from \url{http://user.xmission.com/~octa/DonnerParty/Roster.htm}.
}
\description{
This data frame contains information on the members of the Donner Party, a
group of people who attempted to migrate to California in 1846. They were
trapped by an early blizzard on the eastern side of the Sierra Nevada
mountains, and before they could be rescued, nearly half of the party had
died.
}
\details{
What factors affected who lived and who died?

This data frame uses the person's name as row labels. \code{family} reflects
a recoding of the last names of individuals to reduce the number of factor
levels. The main families in the Donner party were: Donner, Graves, Breen
and Reed. The families of Murphy, Foster and Pike are grouped as
\code{'MurFosPik'}, those of Fosdick and Wolfinger are coded as
\code{'FosdWolf'}, and all others as \code{'Other'}.

\code{survived} is the response variable. What kind of models should be used
here?
}
\examples{

# conditional density plots
op <- par(mfrow=c(1,2), cex.lab=1.5)
cdplot(factor(survived) ~ age,
       subset=sex=='Male',
       data=Donner,
       main="Donner party: Males",
       ylevels=2:1,
       ylab="Survived",
       yaxlabels=c("yes", "no"))
with(Donner, rug(jitter(age[sex=="Male"]),
                 col="white", quiet=TRUE))

cdplot(factor(survived) ~ age,
       subset=sex=='Female',
       data=Donner,
       main="Donner party: Females",
       ylevels=2:1,
       ylab="Survived",
       yaxlabels=c("yes", "no"))
with(Donner, rug(jitter(age[sex=="Female"]),
                 col="white", quiet=TRUE))
par(op)


# fit some models
(mod1 <- glm(survived ~ age + sex, data=Donner, family=binomial))
(mod2 <- glm(survived ~ age * sex, data=Donner, family=binomial))
anova(mod2, test="Chisq")

(mod3 <- glm(survived ~ poly(age,2) * sex, data=Donner, family=binomial))
anova(mod3, test="Chisq")
LRstats(glmlist(mod1, mod2, mod3))

# plot fitted probabilities from mod2 and mod3
# idea from: http://www.ling.upenn.edu/~joseff/rstudy/summer2010_ggplot2_intro.html
if (require("ggplot2")) {

# separate linear fits on age for M/F
print(ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm",
              method.args = list(family = binomial),
              formula = y ~ x,
              alpha = 0.2,
              size=2,
              aes(fill = sex)))

# separate quadratics
print(ggplot(Donner, aes(age, survived, color = sex)) +
  geom_point(position = position_jitter(height = 0.02, width = 0)) +
  stat_smooth(method = "glm",
              method.args = list(family = binomial),
              formula = y ~ poly(x,2),
              alpha = 0.2,
              size=2,
              aes(fill = sex)))

}

}
\references{
Ramsey, F.L. and Schafer, D.W. (2002).  \emph{The Statistical
Sleuth: A Course in Methods of Data Analysis}, (2nd ed), Duxbury.

Friendly, M. and Meyer, D. (2016).  \emph{Discrete Data Analysis with R:
Visualization and Modeling Techniques for Categorical and Count Data}.  Boca
Raton, FL: Chapman & Hall/CRC. \url{http://ddar.datavis.ca}.
}
\seealso{
\code{donner} in \pkg{alr3}, \code{\link[Sleuth2]{case2001}} in
\pkg{Sleuth2}(adults only) provide similar data sets.
}
\keyword{datasets}
